/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|* A list of regular keyword attributes generated from the attribute definiti *|
|* ons                                                                        *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/

KEYWORD_ATTRIBUTE(__arm_agnostic, true, )
KEYWORD_ATTRIBUTE(__arm_in, true, )
KEYWORD_ATTRIBUTE(__arm_inout, true, )
KEYWORD_ATTRIBUTE(__arm_locally_streaming, false, )
KEYWORD_ATTRIBUTE(__arm_new, true, )
KEYWORD_ATTRIBUTE(__arm_out, true, )
KEYWORD_ATTRIBUTE(__arm_preserves, true, )
KEYWORD_ATTRIBUTE(__arm_streaming, false, )
KEYWORD_ATTRIBUTE(__arm_streaming_compatible, false, )
#undef KEYWORD_ATTRIBUTE
