/* SPDX-License-Identifier: GPL-2.0 WITH Linux-syscall-note */
#ifndef _LINUX_KERNEL_H
#define _LINUX_KERNEL_H

#ifdef __GLIBC__
#include <linux/sysinfo.h>
#endif
#include <linux/const.h>

#endif /* _LINUX_KERNEL_H */
