#define CLANG_VERSION 21.1.8+libcxx
#define CLANG_VERSION_STRING "21.1.8+libcxx"
#define CLANG_VERSION_MAJOR 21
#define CLANG_VERSION_MAJOR_STRING "21"
#define CLANG_VERSION_MINOR 1
#define CLANG_VERSION_PATCHLEVEL 8
#define MAX_CLANG_ABI_COMPAT_VERSION 21
