/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|* Intrinsic Function Source Fragment                                         *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/

#ifndef LLVM_IR_INTRINSIC_BPF_ENUMS_H
#define LLVM_IR_INTRINSIC_BPF_ENUMS_H
namespace llvm::Intrinsic {
enum BPFIntrinsics : unsigned {
// Enum values for intrinsics.
    bpf_btf_type_id = 3954,                           // llvm.bpf.btf.type.id
    bpf_compare,                               // llvm.bpf.compare
    bpf_getelementptr_and_load,                // llvm.bpf.getelementptr.and.load
    bpf_getelementptr_and_store,               // llvm.bpf.getelementptr.and.store
    bpf_load_byte,                             // llvm.bpf.load.byte
    bpf_load_half,                             // llvm.bpf.load.half
    bpf_load_word,                             // llvm.bpf.load.word
    bpf_passthrough,                           // llvm.bpf.passthrough
    bpf_preserve_enum_value,                   // llvm.bpf.preserve.enum.value
    bpf_preserve_field_info,                   // llvm.bpf.preserve.field.info
    bpf_preserve_type_info,                    // llvm.bpf.preserve.type.info
    bpf_pseudo,                                // llvm.bpf.pseudo
}; // enum
} // namespace llvm::Intrinsic
#endif

