#ifdef LLVM_GET_ARM_INTRIN_IMMCHECKTYPES
  ImmCheck0_0 = 17,
  ImmCheck0_15 = 18,
  ImmCheck1_64 = 25,
  ImmCheck0_63 = 26,
  ImmCheck1_1 = 21,
  ImmCheck0_2 = 15,
  ImmCheck1_7 = 23,
  ImmCheck0_7 = 6,
  ImmCheck0_255 = 19,
  ImmCheckShiftRight = 3,
  ImmCheckLaneIndexCompRotate = 9,
  ImmCheckComplexRot90_270 = 11,
  ImmCheckShiftRightNarrow = 4,
  ImmCheck2_4_Mul2 = 20,
  ImmCheck1_3 = 22,
  ImmCheck0_3 = 16,
  ImmCheckExtract = 2,
  ImmCheckLaneIndex = 7,
  ImmCheck0_1 = 14,
  ImmCheck0_13 = 13,
  ImmCheckCvt = 8,
  ImmCheck1_16 = 1,
  ImmCheckComplexRotAll90 = 12,
  ImmCheck1_32 = 24,
  ImmCheckLaneIndexDot = 10,
  ImmCheck0_31 = 0,
  ImmCheckShiftLeft = 5,
#endif

