#ifdef GET_SVE_LLVM_INTRINSIC_MAP
SVEMAP1(svaba_n_s16, aarch64_sve_saba, 4398046514178),
SVEMAP1(svaba_n_s32, aarch64_sve_saba, 4398046514179),
SVEMAP1(svaba_n_s64, aarch64_sve_saba, 4398046514180),
SVEMAP1(svaba_n_s8, aarch64_sve_saba, 4398046514177),
SVEMAP1(svaba_n_u16, aarch64_sve_uaba, 4398046514178),
SVEMAP1(svaba_n_u32, aarch64_sve_uaba, 4398046514179),
SVEMAP1(svaba_n_u64, aarch64_sve_uaba, 4398046514180),
SVEMAP1(svaba_n_u8, aarch64_sve_uaba, 4398046514177),
SVEMAP1(svaba_s16, aarch64_sve_saba, 4398046511106),
SVEMAP1(svaba_s32, aarch64_sve_saba, 4398046511107),
SVEMAP1(svaba_s64, aarch64_sve_saba, 4398046511108),
SVEMAP1(svaba_s8, aarch64_sve_saba, 4398046511105),
SVEMAP1(svaba_u16, aarch64_sve_uaba, 4398046511106),
SVEMAP1(svaba_u32, aarch64_sve_uaba, 4398046511107),
SVEMAP1(svaba_u64, aarch64_sve_uaba, 4398046511108),
SVEMAP1(svaba_u8, aarch64_sve_uaba, 4398046511105),
SVEMAP1(svabalb_n_s16, aarch64_sve_sabalb, 4398046514178),
SVEMAP1(svabalb_n_s32, aarch64_sve_sabalb, 4398046514179),
SVEMAP1(svabalb_n_s64, aarch64_sve_sabalb, 4398046514180),
SVEMAP1(svabalb_n_u16, aarch64_sve_uabalb, 4398046514178),
SVEMAP1(svabalb_n_u32, aarch64_sve_uabalb, 4398046514179),
SVEMAP1(svabalb_n_u64, aarch64_sve_uabalb, 4398046514180),
SVEMAP1(svabalb_s16, aarch64_sve_sabalb, 4398046511106),
SVEMAP1(svabalb_s32, aarch64_sve_sabalb, 4398046511107),
SVEMAP1(svabalb_s64, aarch64_sve_sabalb, 4398046511108),
SVEMAP1(svabalb_u16, aarch64_sve_uabalb, 4398046511106),
SVEMAP1(svabalb_u32, aarch64_sve_uabalb, 4398046511107),
SVEMAP1(svabalb_u64, aarch64_sve_uabalb, 4398046511108),
SVEMAP1(svabalt_n_s16, aarch64_sve_sabalt, 4398046514178),
SVEMAP1(svabalt_n_s32, aarch64_sve_sabalt, 4398046514179),
SVEMAP1(svabalt_n_s64, aarch64_sve_sabalt, 4398046514180),
SVEMAP1(svabalt_n_u16, aarch64_sve_uabalt, 4398046514178),
SVEMAP1(svabalt_n_u32, aarch64_sve_uabalt, 4398046514179),
SVEMAP1(svabalt_n_u64, aarch64_sve_uabalt, 4398046514180),
SVEMAP1(svabalt_s16, aarch64_sve_sabalt, 4398046511106),
SVEMAP1(svabalt_s32, aarch64_sve_sabalt, 4398046511107),
SVEMAP1(svabalt_s64, aarch64_sve_sabalt, 4398046511108),
SVEMAP1(svabalt_u16, aarch64_sve_uabalt, 4398046511106),
SVEMAP1(svabalt_u32, aarch64_sve_uabalt, 4398046511107),
SVEMAP1(svabalt_u64, aarch64_sve_uabalt, 4398046511108),
SVEMAP1(svabd_f16_m, aarch64_sve_fabd, 4398046511366),
SVEMAP1(svabd_f16_x, aarch64_sve_fabd_u, 4398046511238),
SVEMAP1(svabd_f16_z, aarch64_sve_fabd, 4398046511494),
SVEMAP1(svabd_f32_m, aarch64_sve_fabd, 4398046511367),
SVEMAP1(svabd_f32_x, aarch64_sve_fabd_u, 4398046511239),
SVEMAP1(svabd_f32_z, aarch64_sve_fabd, 4398046511495),
SVEMAP1(svabd_f64_m, aarch64_sve_fabd, 4398046511368),
SVEMAP1(svabd_f64_x, aarch64_sve_fabd_u, 4398046511240),
SVEMAP1(svabd_f64_z, aarch64_sve_fabd, 4398046511496),
SVEMAP1(svabd_n_f16_m, aarch64_sve_fabd, 4398046514438),
SVEMAP1(svabd_n_f16_x, aarch64_sve_fabd_u, 4398046514310),
SVEMAP1(svabd_n_f16_z, aarch64_sve_fabd, 4398046514566),
SVEMAP1(svabd_n_f32_m, aarch64_sve_fabd, 4398046514439),
SVEMAP1(svabd_n_f32_x, aarch64_sve_fabd_u, 4398046514311),
SVEMAP1(svabd_n_f32_z, aarch64_sve_fabd, 4398046514567),
SVEMAP1(svabd_n_f64_m, aarch64_sve_fabd, 4398046514440),
SVEMAP1(svabd_n_f64_x, aarch64_sve_fabd_u, 4398046514312),
SVEMAP1(svabd_n_f64_z, aarch64_sve_fabd, 4398046514568),
SVEMAP1(svabd_n_s16_m, aarch64_sve_sabd, 4398046514434),
SVEMAP1(svabd_n_s16_x, aarch64_sve_sabd_u, 4398046514306),
SVEMAP1(svabd_n_s16_z, aarch64_sve_sabd, 4398046514562),
SVEMAP1(svabd_n_s32_m, aarch64_sve_sabd, 4398046514435),
SVEMAP1(svabd_n_s32_x, aarch64_sve_sabd_u, 4398046514307),
SVEMAP1(svabd_n_s32_z, aarch64_sve_sabd, 4398046514563),
SVEMAP1(svabd_n_s64_m, aarch64_sve_sabd, 4398046514436),
SVEMAP1(svabd_n_s64_x, aarch64_sve_sabd_u, 4398046514308),
SVEMAP1(svabd_n_s64_z, aarch64_sve_sabd, 4398046514564),
SVEMAP1(svabd_n_s8_m, aarch64_sve_sabd, 4398046514433),
SVEMAP1(svabd_n_s8_x, aarch64_sve_sabd_u, 4398046514305),
SVEMAP1(svabd_n_s8_z, aarch64_sve_sabd, 4398046514561),
SVEMAP1(svabd_n_u16_m, aarch64_sve_uabd, 4398046514434),
SVEMAP1(svabd_n_u16_x, aarch64_sve_uabd_u, 4398046514306),
SVEMAP1(svabd_n_u16_z, aarch64_sve_uabd, 4398046514562),
SVEMAP1(svabd_n_u32_m, aarch64_sve_uabd, 4398046514435),
SVEMAP1(svabd_n_u32_x, aarch64_sve_uabd_u, 4398046514307),
SVEMAP1(svabd_n_u32_z, aarch64_sve_uabd, 4398046514563),
SVEMAP1(svabd_n_u64_m, aarch64_sve_uabd, 4398046514436),
SVEMAP1(svabd_n_u64_x, aarch64_sve_uabd_u, 4398046514308),
SVEMAP1(svabd_n_u64_z, aarch64_sve_uabd, 4398046514564),
SVEMAP1(svabd_n_u8_m, aarch64_sve_uabd, 4398046514433),
SVEMAP1(svabd_n_u8_x, aarch64_sve_uabd_u, 4398046514305),
SVEMAP1(svabd_n_u8_z, aarch64_sve_uabd, 4398046514561),
SVEMAP1(svabd_s16_m, aarch64_sve_sabd, 4398046511362),
SVEMAP1(svabd_s16_x, aarch64_sve_sabd_u, 4398046511234),
SVEMAP1(svabd_s16_z, aarch64_sve_sabd, 4398046511490),
SVEMAP1(svabd_s32_m, aarch64_sve_sabd, 4398046511363),
SVEMAP1(svabd_s32_x, aarch64_sve_sabd_u, 4398046511235),
SVEMAP1(svabd_s32_z, aarch64_sve_sabd, 4398046511491),
SVEMAP1(svabd_s64_m, aarch64_sve_sabd, 4398046511364),
SVEMAP1(svabd_s64_x, aarch64_sve_sabd_u, 4398046511236),
SVEMAP1(svabd_s64_z, aarch64_sve_sabd, 4398046511492),
SVEMAP1(svabd_s8_m, aarch64_sve_sabd, 4398046511361),
SVEMAP1(svabd_s8_x, aarch64_sve_sabd_u, 4398046511233),
SVEMAP1(svabd_s8_z, aarch64_sve_sabd, 4398046511489),
SVEMAP1(svabd_u16_m, aarch64_sve_uabd, 4398046511362),
SVEMAP1(svabd_u16_x, aarch64_sve_uabd_u, 4398046511234),
SVEMAP1(svabd_u16_z, aarch64_sve_uabd, 4398046511490),
SVEMAP1(svabd_u32_m, aarch64_sve_uabd, 4398046511363),
SVEMAP1(svabd_u32_x, aarch64_sve_uabd_u, 4398046511235),
SVEMAP1(svabd_u32_z, aarch64_sve_uabd, 4398046511491),
SVEMAP1(svabd_u64_m, aarch64_sve_uabd, 4398046511364),
SVEMAP1(svabd_u64_x, aarch64_sve_uabd_u, 4398046511236),
SVEMAP1(svabd_u64_z, aarch64_sve_uabd, 4398046511492),
SVEMAP1(svabd_u8_m, aarch64_sve_uabd, 4398046511361),
SVEMAP1(svabd_u8_x, aarch64_sve_uabd_u, 4398046511233),
SVEMAP1(svabd_u8_z, aarch64_sve_uabd, 4398046511489),
SVEMAP1(svabdlb_n_s16, aarch64_sve_sabdlb, 4398046513154),
SVEMAP1(svabdlb_n_s32, aarch64_sve_sabdlb, 4398046513155),
SVEMAP1(svabdlb_n_s64, aarch64_sve_sabdlb, 4398046513156),
SVEMAP1(svabdlb_n_u16, aarch64_sve_uabdlb, 4398046513154),
SVEMAP1(svabdlb_n_u32, aarch64_sve_uabdlb, 4398046513155),
SVEMAP1(svabdlb_n_u64, aarch64_sve_uabdlb, 4398046513156),
SVEMAP1(svabdlb_s16, aarch64_sve_sabdlb, 4398046511106),
SVEMAP1(svabdlb_s32, aarch64_sve_sabdlb, 4398046511107),
SVEMAP1(svabdlb_s64, aarch64_sve_sabdlb, 4398046511108),
SVEMAP1(svabdlb_u16, aarch64_sve_uabdlb, 4398046511106),
SVEMAP1(svabdlb_u32, aarch64_sve_uabdlb, 4398046511107),
SVEMAP1(svabdlb_u64, aarch64_sve_uabdlb, 4398046511108),
SVEMAP1(svabdlt_n_s16, aarch64_sve_sabdlt, 4398046513154),
SVEMAP1(svabdlt_n_s32, aarch64_sve_sabdlt, 4398046513155),
SVEMAP1(svabdlt_n_s64, aarch64_sve_sabdlt, 4398046513156),
SVEMAP1(svabdlt_n_u16, aarch64_sve_uabdlt, 4398046513154),
SVEMAP1(svabdlt_n_u32, aarch64_sve_uabdlt, 4398046513155),
SVEMAP1(svabdlt_n_u64, aarch64_sve_uabdlt, 4398046513156),
SVEMAP1(svabdlt_s16, aarch64_sve_sabdlt, 4398046511106),
SVEMAP1(svabdlt_s32, aarch64_sve_sabdlt, 4398046511107),
SVEMAP1(svabdlt_s64, aarch64_sve_sabdlt, 4398046511108),
SVEMAP1(svabdlt_u16, aarch64_sve_uabdlt, 4398046511106),
SVEMAP1(svabdlt_u32, aarch64_sve_uabdlt, 4398046511107),
SVEMAP1(svabdlt_u64, aarch64_sve_uabdlt, 4398046511108),
SVEMAP1(svabs_f16_m, aarch64_sve_fabs, 4398046511366),
SVEMAP1(svabs_f16_x, aarch64_sve_fabs, 4398046511622),
SVEMAP1(svabs_f16_z, aarch64_sve_fabs, 4398046511750),
SVEMAP1(svabs_f32_m, aarch64_sve_fabs, 4398046511367),
SVEMAP1(svabs_f32_x, aarch64_sve_fabs, 4398046511623),
SVEMAP1(svabs_f32_z, aarch64_sve_fabs, 4398046511751),
SVEMAP1(svabs_f64_m, aarch64_sve_fabs, 4398046511368),
SVEMAP1(svabs_f64_x, aarch64_sve_fabs, 4398046511624),
SVEMAP1(svabs_f64_z, aarch64_sve_fabs, 4398046511752),
SVEMAP1(svabs_s16_m, aarch64_sve_abs, 4398046511362),
SVEMAP1(svabs_s16_x, aarch64_sve_abs, 4398046511618),
SVEMAP1(svabs_s16_z, aarch64_sve_abs, 4398046511746),
SVEMAP1(svabs_s32_m, aarch64_sve_abs, 4398046511363),
SVEMAP1(svabs_s32_x, aarch64_sve_abs, 4398046511619),
SVEMAP1(svabs_s32_z, aarch64_sve_abs, 4398046511747),
SVEMAP1(svabs_s64_m, aarch64_sve_abs, 4398046511364),
SVEMAP1(svabs_s64_x, aarch64_sve_abs, 4398046511620),
SVEMAP1(svabs_s64_z, aarch64_sve_abs, 4398046511748),
SVEMAP1(svabs_s8_m, aarch64_sve_abs, 4398046511361),
SVEMAP1(svabs_s8_x, aarch64_sve_abs, 4398046511617),
SVEMAP1(svabs_s8_z, aarch64_sve_abs, 4398046511745),
SVEMAP1(svacge_f16, aarch64_sve_facge, 4398046511110),
SVEMAP1(svacge_f32, aarch64_sve_facge, 4398046511111),
SVEMAP1(svacge_f64, aarch64_sve_facge, 4398046511112),
SVEMAP1(svacge_n_f16, aarch64_sve_facge, 4398046514182),
SVEMAP1(svacge_n_f32, aarch64_sve_facge, 4398046514183),
SVEMAP1(svacge_n_f64, aarch64_sve_facge, 4398046514184),
SVEMAP1(svacgt_f16, aarch64_sve_facgt, 4398046511110),
SVEMAP1(svacgt_f32, aarch64_sve_facgt, 4398046511111),
SVEMAP1(svacgt_f64, aarch64_sve_facgt, 4398046511112),
SVEMAP1(svacgt_n_f16, aarch64_sve_facgt, 4398046514182),
SVEMAP1(svacgt_n_f32, aarch64_sve_facgt, 4398046514183),
SVEMAP1(svacgt_n_f64, aarch64_sve_facgt, 4398046514184),
SVEMAP1(svacle_f16, aarch64_sve_facge, 4398583382022),
SVEMAP1(svacle_f32, aarch64_sve_facge, 4398583382023),
SVEMAP1(svacle_f64, aarch64_sve_facge, 4398583382024),
SVEMAP1(svacle_n_f16, aarch64_sve_facge, 4398583385094),
SVEMAP1(svacle_n_f32, aarch64_sve_facge, 4398583385095),
SVEMAP1(svacle_n_f64, aarch64_sve_facge, 4398583385096),
SVEMAP1(svaclt_f16, aarch64_sve_facgt, 4398583382022),
SVEMAP1(svaclt_f32, aarch64_sve_facgt, 4398583382023),
SVEMAP1(svaclt_f64, aarch64_sve_facgt, 4398583382024),
SVEMAP1(svaclt_n_f16, aarch64_sve_facgt, 4398583385094),
SVEMAP1(svaclt_n_f32, aarch64_sve_facgt, 4398583385095),
SVEMAP1(svaclt_n_f64, aarch64_sve_facgt, 4398583385096),
SVEMAP1(svadalp_s16_m, aarch64_sve_sadalp, 4398046511362),
SVEMAP1(svadalp_s16_x, aarch64_sve_sadalp, 4398046511234),
SVEMAP1(svadalp_s16_z, aarch64_sve_sadalp, 4398046511490),
SVEMAP1(svadalp_s32_m, aarch64_sve_sadalp, 4398046511363),
SVEMAP1(svadalp_s32_x, aarch64_sve_sadalp, 4398046511235),
SVEMAP1(svadalp_s32_z, aarch64_sve_sadalp, 4398046511491),
SVEMAP1(svadalp_s64_m, aarch64_sve_sadalp, 4398046511364),
SVEMAP1(svadalp_s64_x, aarch64_sve_sadalp, 4398046511236),
SVEMAP1(svadalp_s64_z, aarch64_sve_sadalp, 4398046511492),
SVEMAP1(svadalp_u16_m, aarch64_sve_uadalp, 4398046511362),
SVEMAP1(svadalp_u16_x, aarch64_sve_uadalp, 4398046511234),
SVEMAP1(svadalp_u16_z, aarch64_sve_uadalp, 4398046511490),
SVEMAP1(svadalp_u32_m, aarch64_sve_uadalp, 4398046511363),
SVEMAP1(svadalp_u32_x, aarch64_sve_uadalp, 4398046511235),
SVEMAP1(svadalp_u32_z, aarch64_sve_uadalp, 4398046511491),
SVEMAP1(svadalp_u64_m, aarch64_sve_uadalp, 4398046511364),
SVEMAP1(svadalp_u64_x, aarch64_sve_uadalp, 4398046511236),
SVEMAP1(svadalp_u64_z, aarch64_sve_uadalp, 4398046511492),
SVEMAP1(svadclb_n_u32, aarch64_sve_adclb, 4398046514179),
SVEMAP1(svadclb_n_u64, aarch64_sve_adclb, 4398046514180),
SVEMAP1(svadclb_u32, aarch64_sve_adclb, 4398046511107),
SVEMAP1(svadclb_u64, aarch64_sve_adclb, 4398046511108),
SVEMAP1(svadclt_n_u32, aarch64_sve_adclt, 4398046514179),
SVEMAP1(svadclt_n_u64, aarch64_sve_adclt, 4398046514180),
SVEMAP1(svadclt_u32, aarch64_sve_adclt, 4398046511107),
SVEMAP1(svadclt_u64, aarch64_sve_adclt, 4398046511108),
SVEMAP1(svadd_bf16_m, aarch64_sve_fadd, 4398046511373),
SVEMAP1(svadd_bf16_x, aarch64_sve_fadd_u, 4398046511245),
SVEMAP1(svadd_bf16_z, aarch64_sve_fadd, 4398046511501),
SVEMAP1(svadd_f16_m, aarch64_sve_fadd, 4398046511366),
SVEMAP1(svadd_f16_x, aarch64_sve_fadd_u, 4398046511238),
SVEMAP1(svadd_f16_z, aarch64_sve_fadd, 4398046511494),
SVEMAP1(svadd_f32_m, aarch64_sve_fadd, 4398046511367),
SVEMAP1(svadd_f32_x, aarch64_sve_fadd_u, 4398046511239),
SVEMAP1(svadd_f32_z, aarch64_sve_fadd, 4398046511495),
SVEMAP1(svadd_f64_m, aarch64_sve_fadd, 4398046511368),
SVEMAP1(svadd_f64_x, aarch64_sve_fadd_u, 4398046511240),
SVEMAP1(svadd_f64_z, aarch64_sve_fadd, 4398046511496),
SVEMAP1(svadd_n_bf16_m, aarch64_sve_fadd, 4398046514445),
SVEMAP1(svadd_n_bf16_x, aarch64_sve_fadd_u, 4398046514317),
SVEMAP1(svadd_n_bf16_z, aarch64_sve_fadd, 4398046514573),
SVEMAP1(svadd_n_f16_m, aarch64_sve_fadd, 4398046514438),
SVEMAP1(svadd_n_f16_x, aarch64_sve_fadd_u, 4398046514310),
SVEMAP1(svadd_n_f16_z, aarch64_sve_fadd, 4398046514566),
SVEMAP1(svadd_n_f32_m, aarch64_sve_fadd, 4398046514439),
SVEMAP1(svadd_n_f32_x, aarch64_sve_fadd_u, 4398046514311),
SVEMAP1(svadd_n_f32_z, aarch64_sve_fadd, 4398046514567),
SVEMAP1(svadd_n_f64_m, aarch64_sve_fadd, 4398046514440),
SVEMAP1(svadd_n_f64_x, aarch64_sve_fadd_u, 4398046514312),
SVEMAP1(svadd_n_f64_z, aarch64_sve_fadd, 4398046514568),
SVEMAP1(svadd_n_s16_m, aarch64_sve_add, 4398046514434),
SVEMAP1(svadd_n_s16_x, aarch64_sve_add_u, 4398046514306),
SVEMAP1(svadd_n_s16_z, aarch64_sve_add, 4398046514562),
SVEMAP1(svadd_n_s32_m, aarch64_sve_add, 4398046514435),
SVEMAP1(svadd_n_s32_x, aarch64_sve_add_u, 4398046514307),
SVEMAP1(svadd_n_s32_z, aarch64_sve_add, 4398046514563),
SVEMAP1(svadd_n_s64_m, aarch64_sve_add, 4398046514436),
SVEMAP1(svadd_n_s64_x, aarch64_sve_add_u, 4398046514308),
SVEMAP1(svadd_n_s64_z, aarch64_sve_add, 4398046514564),
SVEMAP1(svadd_n_s8_m, aarch64_sve_add, 4398046514433),
SVEMAP1(svadd_n_s8_x, aarch64_sve_add_u, 4398046514305),
SVEMAP1(svadd_n_s8_z, aarch64_sve_add, 4398046514561),
SVEMAP1(svadd_n_u16_m, aarch64_sve_add, 4398046514434),
SVEMAP1(svadd_n_u16_x, aarch64_sve_add_u, 4398046514306),
SVEMAP1(svadd_n_u16_z, aarch64_sve_add, 4398046514562),
SVEMAP1(svadd_n_u32_m, aarch64_sve_add, 4398046514435),
SVEMAP1(svadd_n_u32_x, aarch64_sve_add_u, 4398046514307),
SVEMAP1(svadd_n_u32_z, aarch64_sve_add, 4398046514563),
SVEMAP1(svadd_n_u64_m, aarch64_sve_add, 4398046514436),
SVEMAP1(svadd_n_u64_x, aarch64_sve_add_u, 4398046514308),
SVEMAP1(svadd_n_u64_z, aarch64_sve_add, 4398046514564),
SVEMAP1(svadd_n_u8_m, aarch64_sve_add, 4398046514433),
SVEMAP1(svadd_n_u8_x, aarch64_sve_add_u, 4398046514305),
SVEMAP1(svadd_n_u8_z, aarch64_sve_add, 4398046514561),
SVEMAP1(svadd_s16_m, aarch64_sve_add, 4398046511362),
SVEMAP1(svadd_s16_x, aarch64_sve_add_u, 4398046511234),
SVEMAP1(svadd_s16_z, aarch64_sve_add, 4398046511490),
SVEMAP1(svadd_s32_m, aarch64_sve_add, 4398046511363),
SVEMAP1(svadd_s32_x, aarch64_sve_add_u, 4398046511235),
SVEMAP1(svadd_s32_z, aarch64_sve_add, 4398046511491),
SVEMAP1(svadd_s64_m, aarch64_sve_add, 4398046511364),
SVEMAP1(svadd_s64_x, aarch64_sve_add_u, 4398046511236),
SVEMAP1(svadd_s64_z, aarch64_sve_add, 4398046511492),
SVEMAP1(svadd_s8_m, aarch64_sve_add, 4398046511361),
SVEMAP1(svadd_s8_x, aarch64_sve_add_u, 4398046511233),
SVEMAP1(svadd_s8_z, aarch64_sve_add, 4398046511489),
SVEMAP1(svadd_single_s16_x2, aarch64_sve_add_single_x2, 137438953474),
SVEMAP1(svadd_single_s16_x4, aarch64_sve_add_single_x4, 137438953474),
SVEMAP1(svadd_single_s32_x2, aarch64_sve_add_single_x2, 137438953475),
SVEMAP1(svadd_single_s32_x4, aarch64_sve_add_single_x4, 137438953475),
SVEMAP1(svadd_single_s64_x2, aarch64_sve_add_single_x2, 137438953476),
SVEMAP1(svadd_single_s64_x4, aarch64_sve_add_single_x4, 137438953476),
SVEMAP1(svadd_single_s8_x2, aarch64_sve_add_single_x2, 137438953473),
SVEMAP1(svadd_single_s8_x4, aarch64_sve_add_single_x4, 137438953473),
SVEMAP1(svadd_single_u16_x2, aarch64_sve_add_single_x2, 137438953474),
SVEMAP1(svadd_single_u16_x4, aarch64_sve_add_single_x4, 137438953474),
SVEMAP1(svadd_single_u32_x2, aarch64_sve_add_single_x2, 137438953475),
SVEMAP1(svadd_single_u32_x4, aarch64_sve_add_single_x4, 137438953475),
SVEMAP1(svadd_single_u64_x2, aarch64_sve_add_single_x2, 137438953476),
SVEMAP1(svadd_single_u64_x4, aarch64_sve_add_single_x4, 137438953476),
SVEMAP1(svadd_single_u8_x2, aarch64_sve_add_single_x2, 137438953473),
SVEMAP1(svadd_single_u8_x4, aarch64_sve_add_single_x4, 137438953473),
SVEMAP1(svadd_u16_m, aarch64_sve_add, 4398046511362),
SVEMAP1(svadd_u16_x, aarch64_sve_add_u, 4398046511234),
SVEMAP1(svadd_u16_z, aarch64_sve_add, 4398046511490),
SVEMAP1(svadd_u32_m, aarch64_sve_add, 4398046511363),
SVEMAP1(svadd_u32_x, aarch64_sve_add_u, 4398046511235),
SVEMAP1(svadd_u32_z, aarch64_sve_add, 4398046511491),
SVEMAP1(svadd_u64_m, aarch64_sve_add, 4398046511364),
SVEMAP1(svadd_u64_x, aarch64_sve_add_u, 4398046511236),
SVEMAP1(svadd_u64_z, aarch64_sve_add, 4398046511492),
SVEMAP1(svadd_u8_m, aarch64_sve_add, 4398046511361),
SVEMAP1(svadd_u8_x, aarch64_sve_add_u, 4398046511233),
SVEMAP1(svadd_u8_z, aarch64_sve_add, 4398046511489),
SVEMAP1(svadda_f16, aarch64_sve_fadda, 6),
SVEMAP1(svadda_f32, aarch64_sve_fadda, 7),
SVEMAP1(svadda_f64, aarch64_sve_fadda, 8),
SVEMAP1(svaddhnb_n_s16, aarch64_sve_addhnb, 4398046513154),
SVEMAP1(svaddhnb_n_s32, aarch64_sve_addhnb, 4398046513155),
SVEMAP1(svaddhnb_n_s64, aarch64_sve_addhnb, 4398046513156),
SVEMAP1(svaddhnb_n_u16, aarch64_sve_addhnb, 4398046513154),
SVEMAP1(svaddhnb_n_u32, aarch64_sve_addhnb, 4398046513155),
SVEMAP1(svaddhnb_n_u64, aarch64_sve_addhnb, 4398046513156),
SVEMAP1(svaddhnb_s16, aarch64_sve_addhnb, 4398046511106),
SVEMAP1(svaddhnb_s32, aarch64_sve_addhnb, 4398046511107),
SVEMAP1(svaddhnb_s64, aarch64_sve_addhnb, 4398046511108),
SVEMAP1(svaddhnb_u16, aarch64_sve_addhnb, 4398046511106),
SVEMAP1(svaddhnb_u32, aarch64_sve_addhnb, 4398046511107),
SVEMAP1(svaddhnb_u64, aarch64_sve_addhnb, 4398046511108),
SVEMAP1(svaddhnt_n_s16, aarch64_sve_addhnt, 4398046514178),
SVEMAP1(svaddhnt_n_s32, aarch64_sve_addhnt, 4398046514179),
SVEMAP1(svaddhnt_n_s64, aarch64_sve_addhnt, 4398046514180),
SVEMAP1(svaddhnt_n_u16, aarch64_sve_addhnt, 4398046514178),
SVEMAP1(svaddhnt_n_u32, aarch64_sve_addhnt, 4398046514179),
SVEMAP1(svaddhnt_n_u64, aarch64_sve_addhnt, 4398046514180),
SVEMAP1(svaddhnt_s16, aarch64_sve_addhnt, 4398046511106),
SVEMAP1(svaddhnt_s32, aarch64_sve_addhnt, 4398046511107),
SVEMAP1(svaddhnt_s64, aarch64_sve_addhnt, 4398046511108),
SVEMAP1(svaddhnt_u16, aarch64_sve_addhnt, 4398046511106),
SVEMAP1(svaddhnt_u32, aarch64_sve_addhnt, 4398046511107),
SVEMAP1(svaddhnt_u64, aarch64_sve_addhnt, 4398046511108),
SVEMAP1(svaddlb_n_s16, aarch64_sve_saddlb, 4398046513154),
SVEMAP1(svaddlb_n_s32, aarch64_sve_saddlb, 4398046513155),
SVEMAP1(svaddlb_n_s64, aarch64_sve_saddlb, 4398046513156),
SVEMAP1(svaddlb_n_u16, aarch64_sve_uaddlb, 4398046513154),
SVEMAP1(svaddlb_n_u32, aarch64_sve_uaddlb, 4398046513155),
SVEMAP1(svaddlb_n_u64, aarch64_sve_uaddlb, 4398046513156),
SVEMAP1(svaddlb_s16, aarch64_sve_saddlb, 4398046511106),
SVEMAP1(svaddlb_s32, aarch64_sve_saddlb, 4398046511107),
SVEMAP1(svaddlb_s64, aarch64_sve_saddlb, 4398046511108),
SVEMAP1(svaddlb_u16, aarch64_sve_uaddlb, 4398046511106),
SVEMAP1(svaddlb_u32, aarch64_sve_uaddlb, 4398046511107),
SVEMAP1(svaddlb_u64, aarch64_sve_uaddlb, 4398046511108),
SVEMAP1(svaddlbt_n_s16, aarch64_sve_saddlbt, 4398046513154),
SVEMAP1(svaddlbt_n_s32, aarch64_sve_saddlbt, 4398046513155),
SVEMAP1(svaddlbt_n_s64, aarch64_sve_saddlbt, 4398046513156),
SVEMAP1(svaddlbt_s16, aarch64_sve_saddlbt, 4398046511106),
SVEMAP1(svaddlbt_s32, aarch64_sve_saddlbt, 4398046511107),
SVEMAP1(svaddlbt_s64, aarch64_sve_saddlbt, 4398046511108),
SVEMAP1(svaddlt_n_s16, aarch64_sve_saddlt, 4398046513154),
SVEMAP1(svaddlt_n_s32, aarch64_sve_saddlt, 4398046513155),
SVEMAP1(svaddlt_n_s64, aarch64_sve_saddlt, 4398046513156),
SVEMAP1(svaddlt_n_u16, aarch64_sve_uaddlt, 4398046513154),
SVEMAP1(svaddlt_n_u32, aarch64_sve_uaddlt, 4398046513155),
SVEMAP1(svaddlt_n_u64, aarch64_sve_uaddlt, 4398046513156),
SVEMAP1(svaddlt_s16, aarch64_sve_saddlt, 4398046511106),
SVEMAP1(svaddlt_s32, aarch64_sve_saddlt, 4398046511107),
SVEMAP1(svaddlt_s64, aarch64_sve_saddlt, 4398046511108),
SVEMAP1(svaddlt_u16, aarch64_sve_uaddlt, 4398046511106),
SVEMAP1(svaddlt_u32, aarch64_sve_uaddlt, 4398046511107),
SVEMAP1(svaddlt_u64, aarch64_sve_uaddlt, 4398046511108),
SVEMAP1(svaddp_f16_m, aarch64_sve_faddp, 4398046511366),
SVEMAP1(svaddp_f16_x, aarch64_sve_faddp, 4398046511238),
SVEMAP1(svaddp_f32_m, aarch64_sve_faddp, 4398046511367),
SVEMAP1(svaddp_f32_x, aarch64_sve_faddp, 4398046511239),
SVEMAP1(svaddp_f64_m, aarch64_sve_faddp, 4398046511368),
SVEMAP1(svaddp_f64_x, aarch64_sve_faddp, 4398046511240),
SVEMAP1(svaddp_s16_m, aarch64_sve_addp, 4398046511362),
SVEMAP1(svaddp_s16_x, aarch64_sve_addp, 4398046511234),
SVEMAP1(svaddp_s32_m, aarch64_sve_addp, 4398046511363),
SVEMAP1(svaddp_s32_x, aarch64_sve_addp, 4398046511235),
SVEMAP1(svaddp_s64_m, aarch64_sve_addp, 4398046511364),
SVEMAP1(svaddp_s64_x, aarch64_sve_addp, 4398046511236),
SVEMAP1(svaddp_s8_m, aarch64_sve_addp, 4398046511361),
SVEMAP1(svaddp_s8_x, aarch64_sve_addp, 4398046511233),
SVEMAP1(svaddp_u16_m, aarch64_sve_addp, 4398046511362),
SVEMAP1(svaddp_u16_x, aarch64_sve_addp, 4398046511234),
SVEMAP1(svaddp_u32_m, aarch64_sve_addp, 4398046511363),
SVEMAP1(svaddp_u32_x, aarch64_sve_addp, 4398046511235),
SVEMAP1(svaddp_u64_m, aarch64_sve_addp, 4398046511364),
SVEMAP1(svaddp_u64_x, aarch64_sve_addp, 4398046511236),
SVEMAP1(svaddp_u8_m, aarch64_sve_addp, 4398046511361),
SVEMAP1(svaddp_u8_x, aarch64_sve_addp, 4398046511233),
SVEMAP1(svaddqv_f16, aarch64_sve_faddqv, 6597069766662),
SVEMAP1(svaddqv_f32, aarch64_sve_faddqv, 6597069766663),
SVEMAP1(svaddqv_f64, aarch64_sve_faddqv, 6597069766664),
SVEMAP1(svaddqv_s16, aarch64_sve_addqv, 6597069766658),
SVEMAP1(svaddqv_s32, aarch64_sve_addqv, 6597069766659),
SVEMAP1(svaddqv_s64, aarch64_sve_addqv, 6597069766660),
SVEMAP1(svaddqv_s8, aarch64_sve_addqv, 6597069766657),
SVEMAP1(svaddqv_u16, aarch64_sve_addqv, 6597069766658),
SVEMAP1(svaddqv_u32, aarch64_sve_addqv, 6597069766659),
SVEMAP1(svaddqv_u64, aarch64_sve_addqv, 6597069766660),
SVEMAP1(svaddqv_u8, aarch64_sve_addqv, 6597069766657),
SVEMAP1(svaddv_f16, aarch64_sve_faddv, 4398046511110),
SVEMAP1(svaddv_f32, aarch64_sve_faddv, 4398046511111),
SVEMAP1(svaddv_f64, aarch64_sve_faddv, 4398046511112),
SVEMAP1(svaddv_s16, aarch64_sve_saddv, 4398046511106),
SVEMAP1(svaddv_s32, aarch64_sve_saddv, 4398046511107),
SVEMAP1(svaddv_s64, aarch64_sve_saddv, 4398046511108),
SVEMAP1(svaddv_s8, aarch64_sve_saddv, 4398046511105),
SVEMAP1(svaddv_u16, aarch64_sve_uaddv, 4398046511106),
SVEMAP1(svaddv_u32, aarch64_sve_uaddv, 4398046511107),
SVEMAP1(svaddv_u64, aarch64_sve_uaddv, 4398046511108),
SVEMAP1(svaddv_u8, aarch64_sve_uaddv, 4398046511105),
SVEMAP1(svaddwb_n_s16, aarch64_sve_saddwb, 4398046513154),
SVEMAP1(svaddwb_n_s32, aarch64_sve_saddwb, 4398046513155),
SVEMAP1(svaddwb_n_s64, aarch64_sve_saddwb, 4398046513156),
SVEMAP1(svaddwb_n_u16, aarch64_sve_uaddwb, 4398046513154),
SVEMAP1(svaddwb_n_u32, aarch64_sve_uaddwb, 4398046513155),
SVEMAP1(svaddwb_n_u64, aarch64_sve_uaddwb, 4398046513156),
SVEMAP1(svaddwb_s16, aarch64_sve_saddwb, 4398046511106),
SVEMAP1(svaddwb_s32, aarch64_sve_saddwb, 4398046511107),
SVEMAP1(svaddwb_s64, aarch64_sve_saddwb, 4398046511108),
SVEMAP1(svaddwb_u16, aarch64_sve_uaddwb, 4398046511106),
SVEMAP1(svaddwb_u32, aarch64_sve_uaddwb, 4398046511107),
SVEMAP1(svaddwb_u64, aarch64_sve_uaddwb, 4398046511108),
SVEMAP1(svaddwt_n_s16, aarch64_sve_saddwt, 4398046513154),
SVEMAP1(svaddwt_n_s32, aarch64_sve_saddwt, 4398046513155),
SVEMAP1(svaddwt_n_s64, aarch64_sve_saddwt, 4398046513156),
SVEMAP1(svaddwt_n_u16, aarch64_sve_uaddwt, 4398046513154),
SVEMAP1(svaddwt_n_u32, aarch64_sve_uaddwt, 4398046513155),
SVEMAP1(svaddwt_n_u64, aarch64_sve_uaddwt, 4398046513156),
SVEMAP1(svaddwt_s16, aarch64_sve_saddwt, 4398046511106),
SVEMAP1(svaddwt_s32, aarch64_sve_saddwt, 4398046511107),
SVEMAP1(svaddwt_s64, aarch64_sve_saddwt, 4398046511108),
SVEMAP1(svaddwt_u16, aarch64_sve_uaddwt, 4398046511106),
SVEMAP1(svaddwt_u32, aarch64_sve_uaddwt, 4398046511107),
SVEMAP1(svaddwt_u64, aarch64_sve_uaddwt, 4398046511108),
SVEMAP1(svadrb_u32base_s32offset, aarch64_sve_adrb, 3),
SVEMAP1(svadrb_u32base_u32offset, aarch64_sve_adrb, 3),
SVEMAP1(svadrb_u64base_s64offset, aarch64_sve_adrb, 4),
SVEMAP1(svadrb_u64base_u64offset, aarch64_sve_adrb, 4),
SVEMAP1(svadrd_u32base_s32index, aarch64_sve_adrd, 3),
SVEMAP1(svadrd_u32base_u32index, aarch64_sve_adrd, 3),
SVEMAP1(svadrd_u64base_s64index, aarch64_sve_adrd, 4),
SVEMAP1(svadrd_u64base_u64index, aarch64_sve_adrd, 4),
SVEMAP1(svadrh_u32base_s32index, aarch64_sve_adrh, 3),
SVEMAP1(svadrh_u32base_u32index, aarch64_sve_adrh, 3),
SVEMAP1(svadrh_u64base_s64index, aarch64_sve_adrh, 4),
SVEMAP1(svadrh_u64base_u64index, aarch64_sve_adrh, 4),
SVEMAP1(svadrw_u32base_s32index, aarch64_sve_adrw, 3),
SVEMAP1(svadrw_u32base_u32index, aarch64_sve_adrw, 3),
SVEMAP1(svadrw_u64base_s64index, aarch64_sve_adrw, 4),
SVEMAP1(svadrw_u64base_u64index, aarch64_sve_adrw, 4),
SVEMAP1(svaesd_u8, aarch64_sve_aesd, 1048577),
SVEMAP1(svaese_u8, aarch64_sve_aese, 1048577),
SVEMAP1(svaesimc_u8, aarch64_sve_aesimc, 1048577),
SVEMAP1(svaesmc_u8, aarch64_sve_aesmc, 1048577),
SVEMAP1(svamax_f16_m, aarch64_sve_famax, 4398046511366),
SVEMAP1(svamax_f16_x, aarch64_sve_famax_u, 4398046511238),
SVEMAP1(svamax_f16_x2, aarch64_sme_famax_x2, 137438953478),
SVEMAP1(svamax_f16_x4, aarch64_sme_famax_x4, 137438953478),
SVEMAP1(svamax_f16_z, aarch64_sve_famax, 4398046511494),
SVEMAP1(svamax_f32_m, aarch64_sve_famax, 4398046511367),
SVEMAP1(svamax_f32_x, aarch64_sve_famax_u, 4398046511239),
SVEMAP1(svamax_f32_x2, aarch64_sme_famax_x2, 137438953479),
SVEMAP1(svamax_f32_x4, aarch64_sme_famax_x4, 137438953479),
SVEMAP1(svamax_f32_z, aarch64_sve_famax, 4398046511495),
SVEMAP1(svamax_f64_m, aarch64_sve_famax, 4398046511368),
SVEMAP1(svamax_f64_x, aarch64_sve_famax_u, 4398046511240),
SVEMAP1(svamax_f64_x2, aarch64_sme_famax_x2, 137438953480),
SVEMAP1(svamax_f64_x4, aarch64_sme_famax_x4, 137438953480),
SVEMAP1(svamax_f64_z, aarch64_sve_famax, 4398046511496),
SVEMAP1(svamax_n_f16_m, aarch64_sve_famax, 4398046514438),
SVEMAP1(svamax_n_f16_x, aarch64_sve_famax_u, 4398046514310),
SVEMAP1(svamax_n_f16_z, aarch64_sve_famax, 4398046514566),
SVEMAP1(svamax_n_f32_m, aarch64_sve_famax, 4398046514439),
SVEMAP1(svamax_n_f32_x, aarch64_sve_famax_u, 4398046514311),
SVEMAP1(svamax_n_f32_z, aarch64_sve_famax, 4398046514567),
SVEMAP1(svamax_n_f64_m, aarch64_sve_famax, 4398046514440),
SVEMAP1(svamax_n_f64_x, aarch64_sve_famax_u, 4398046514312),
SVEMAP1(svamax_n_f64_z, aarch64_sve_famax, 4398046514568),
SVEMAP1(svamin_f16_m, aarch64_sve_famin, 4398046511366),
SVEMAP1(svamin_f16_x, aarch64_sve_famin_u, 4398046511238),
SVEMAP1(svamin_f16_x2, aarch64_sme_famin_x2, 137438953478),
SVEMAP1(svamin_f16_x4, aarch64_sme_famin_x4, 137438953478),
SVEMAP1(svamin_f16_z, aarch64_sve_famin, 4398046511494),
SVEMAP1(svamin_f32_m, aarch64_sve_famin, 4398046511367),
SVEMAP1(svamin_f32_x, aarch64_sve_famin_u, 4398046511239),
SVEMAP1(svamin_f32_x2, aarch64_sme_famin_x2, 137438953479),
SVEMAP1(svamin_f32_x4, aarch64_sme_famin_x4, 137438953479),
SVEMAP1(svamin_f32_z, aarch64_sve_famin, 4398046511495),
SVEMAP1(svamin_f64_m, aarch64_sve_famin, 4398046511368),
SVEMAP1(svamin_f64_x, aarch64_sve_famin_u, 4398046511240),
SVEMAP1(svamin_f64_x2, aarch64_sme_famin_x2, 137438953480),
SVEMAP1(svamin_f64_x4, aarch64_sme_famin_x4, 137438953480),
SVEMAP1(svamin_f64_z, aarch64_sve_famin, 4398046511496),
SVEMAP1(svamin_n_f16_m, aarch64_sve_famin, 4398046514438),
SVEMAP1(svamin_n_f16_x, aarch64_sve_famin_u, 4398046514310),
SVEMAP1(svamin_n_f16_z, aarch64_sve_famin, 4398046514566),
SVEMAP1(svamin_n_f32_m, aarch64_sve_famin, 4398046514439),
SVEMAP1(svamin_n_f32_x, aarch64_sve_famin_u, 4398046514311),
SVEMAP1(svamin_n_f32_z, aarch64_sve_famin, 4398046514567),
SVEMAP1(svamin_n_f64_m, aarch64_sve_famin, 4398046514440),
SVEMAP1(svamin_n_f64_x, aarch64_sve_famin_u, 4398046514312),
SVEMAP1(svamin_n_f64_z, aarch64_sve_famin, 4398046514568),
SVEMAP1(svand_b_z, aarch64_sve_and_z, 4398046511113),
SVEMAP1(svand_n_s16_m, aarch64_sve_and, 4398046514434),
SVEMAP1(svand_n_s16_x, aarch64_sve_and_u, 4398046514306),
SVEMAP1(svand_n_s16_z, aarch64_sve_and, 4398046514562),
SVEMAP1(svand_n_s32_m, aarch64_sve_and, 4398046514435),
SVEMAP1(svand_n_s32_x, aarch64_sve_and_u, 4398046514307),
SVEMAP1(svand_n_s32_z, aarch64_sve_and, 4398046514563),
SVEMAP1(svand_n_s64_m, aarch64_sve_and, 4398046514436),
SVEMAP1(svand_n_s64_x, aarch64_sve_and_u, 4398046514308),
SVEMAP1(svand_n_s64_z, aarch64_sve_and, 4398046514564),
SVEMAP1(svand_n_s8_m, aarch64_sve_and, 4398046514433),
SVEMAP1(svand_n_s8_x, aarch64_sve_and_u, 4398046514305),
SVEMAP1(svand_n_s8_z, aarch64_sve_and, 4398046514561),
SVEMAP1(svand_n_u16_m, aarch64_sve_and, 4398046514434),
SVEMAP1(svand_n_u16_x, aarch64_sve_and_u, 4398046514306),
SVEMAP1(svand_n_u16_z, aarch64_sve_and, 4398046514562),
SVEMAP1(svand_n_u32_m, aarch64_sve_and, 4398046514435),
SVEMAP1(svand_n_u32_x, aarch64_sve_and_u, 4398046514307),
SVEMAP1(svand_n_u32_z, aarch64_sve_and, 4398046514563),
SVEMAP1(svand_n_u64_m, aarch64_sve_and, 4398046514436),
SVEMAP1(svand_n_u64_x, aarch64_sve_and_u, 4398046514308),
SVEMAP1(svand_n_u64_z, aarch64_sve_and, 4398046514564),
SVEMAP1(svand_n_u8_m, aarch64_sve_and, 4398046514433),
SVEMAP1(svand_n_u8_x, aarch64_sve_and_u, 4398046514305),
SVEMAP1(svand_n_u8_z, aarch64_sve_and, 4398046514561),
SVEMAP1(svand_s16_m, aarch64_sve_and, 4398046511362),
SVEMAP1(svand_s16_x, aarch64_sve_and_u, 4398046511234),
SVEMAP1(svand_s16_z, aarch64_sve_and, 4398046511490),
SVEMAP1(svand_s32_m, aarch64_sve_and, 4398046511363),
SVEMAP1(svand_s32_x, aarch64_sve_and_u, 4398046511235),
SVEMAP1(svand_s32_z, aarch64_sve_and, 4398046511491),
SVEMAP1(svand_s64_m, aarch64_sve_and, 4398046511364),
SVEMAP1(svand_s64_x, aarch64_sve_and_u, 4398046511236),
SVEMAP1(svand_s64_z, aarch64_sve_and, 4398046511492),
SVEMAP1(svand_s8_m, aarch64_sve_and, 4398046511361),
SVEMAP1(svand_s8_x, aarch64_sve_and_u, 4398046511233),
SVEMAP1(svand_s8_z, aarch64_sve_and, 4398046511489),
SVEMAP1(svand_u16_m, aarch64_sve_and, 4398046511362),
SVEMAP1(svand_u16_x, aarch64_sve_and_u, 4398046511234),
SVEMAP1(svand_u16_z, aarch64_sve_and, 4398046511490),
SVEMAP1(svand_u32_m, aarch64_sve_and, 4398046511363),
SVEMAP1(svand_u32_x, aarch64_sve_and_u, 4398046511235),
SVEMAP1(svand_u32_z, aarch64_sve_and, 4398046511491),
SVEMAP1(svand_u64_m, aarch64_sve_and, 4398046511364),
SVEMAP1(svand_u64_x, aarch64_sve_and_u, 4398046511236),
SVEMAP1(svand_u64_z, aarch64_sve_and, 4398046511492),
SVEMAP1(svand_u8_m, aarch64_sve_and, 4398046511361),
SVEMAP1(svand_u8_x, aarch64_sve_and_u, 4398046511233),
SVEMAP1(svand_u8_z, aarch64_sve_and, 4398046511489),
SVEMAP1(svandqv_s16, aarch64_sve_andqv, 6597069766658),
SVEMAP1(svandqv_s32, aarch64_sve_andqv, 6597069766659),
SVEMAP1(svandqv_s64, aarch64_sve_andqv, 6597069766660),
SVEMAP1(svandqv_s8, aarch64_sve_andqv, 6597069766657),
SVEMAP1(svandqv_u16, aarch64_sve_andqv, 6597069766658),
SVEMAP1(svandqv_u32, aarch64_sve_andqv, 6597069766659),
SVEMAP1(svandqv_u64, aarch64_sve_andqv, 6597069766660),
SVEMAP1(svandqv_u8, aarch64_sve_andqv, 6597069766657),
SVEMAP1(svandv_s16, aarch64_sve_andv, 4398046511106),
SVEMAP1(svandv_s32, aarch64_sve_andv, 4398046511107),
SVEMAP1(svandv_s64, aarch64_sve_andv, 4398046511108),
SVEMAP1(svandv_s8, aarch64_sve_andv, 4398046511105),
SVEMAP1(svandv_u16, aarch64_sve_andv, 4398046511106),
SVEMAP1(svandv_u32, aarch64_sve_andv, 4398046511107),
SVEMAP1(svandv_u64, aarch64_sve_andv, 4398046511108),
SVEMAP1(svandv_u8, aarch64_sve_andv, 4398046511105),
SVEMAP1(svasr_n_s16_m, aarch64_sve_asr, 4398046514434),
SVEMAP1(svasr_n_s16_x, aarch64_sve_asr_u, 4398046514306),
SVEMAP1(svasr_n_s16_z, aarch64_sve_asr, 4398046514562),
SVEMAP1(svasr_n_s32_m, aarch64_sve_asr, 4398046514435),
SVEMAP1(svasr_n_s32_x, aarch64_sve_asr_u, 4398046514307),
SVEMAP1(svasr_n_s32_z, aarch64_sve_asr, 4398046514563),
SVEMAP1(svasr_n_s64_m, aarch64_sve_asr, 4398046514436),
SVEMAP1(svasr_n_s64_x, aarch64_sve_asr_u, 4398046514308),
SVEMAP1(svasr_n_s64_z, aarch64_sve_asr, 4398046514564),
SVEMAP1(svasr_n_s8_m, aarch64_sve_asr, 4398046514433),
SVEMAP1(svasr_n_s8_x, aarch64_sve_asr_u, 4398046514305),
SVEMAP1(svasr_n_s8_z, aarch64_sve_asr, 4398046514561),
SVEMAP1(svasr_s16_m, aarch64_sve_asr, 4398046511362),
SVEMAP1(svasr_s16_x, aarch64_sve_asr_u, 4398046511234),
SVEMAP1(svasr_s16_z, aarch64_sve_asr, 4398046511490),
SVEMAP1(svasr_s32_m, aarch64_sve_asr, 4398046511363),
SVEMAP1(svasr_s32_x, aarch64_sve_asr_u, 4398046511235),
SVEMAP1(svasr_s32_z, aarch64_sve_asr, 4398046511491),
SVEMAP1(svasr_s64_m, aarch64_sve_asr, 4398046511364),
SVEMAP1(svasr_s64_x, aarch64_sve_asr_u, 4398046511236),
SVEMAP1(svasr_s64_z, aarch64_sve_asr, 4398046511492),
SVEMAP1(svasr_s8_m, aarch64_sve_asr, 4398046511361),
SVEMAP1(svasr_s8_x, aarch64_sve_asr_u, 4398046511233),
SVEMAP1(svasr_s8_z, aarch64_sve_asr, 4398046511489),
SVEMAP1(svasr_wide_n_s16_m, aarch64_sve_asr_wide, 4398046514434),
SVEMAP1(svasr_wide_n_s16_x, aarch64_sve_asr_wide, 4398046514306),
SVEMAP1(svasr_wide_n_s16_z, aarch64_sve_asr_wide, 4398046514562),
SVEMAP1(svasr_wide_n_s32_m, aarch64_sve_asr_wide, 4398046514435),
SVEMAP1(svasr_wide_n_s32_x, aarch64_sve_asr_wide, 4398046514307),
SVEMAP1(svasr_wide_n_s32_z, aarch64_sve_asr_wide, 4398046514563),
SVEMAP1(svasr_wide_n_s8_m, aarch64_sve_asr_wide, 4398046514433),
SVEMAP1(svasr_wide_n_s8_x, aarch64_sve_asr_wide, 4398046514305),
SVEMAP1(svasr_wide_n_s8_z, aarch64_sve_asr_wide, 4398046514561),
SVEMAP1(svasr_wide_s16_m, aarch64_sve_asr_wide, 4398046511362),
SVEMAP1(svasr_wide_s16_x, aarch64_sve_asr_wide, 4398046511234),
SVEMAP1(svasr_wide_s16_z, aarch64_sve_asr_wide, 4398046511490),
SVEMAP1(svasr_wide_s32_m, aarch64_sve_asr_wide, 4398046511363),
SVEMAP1(svasr_wide_s32_x, aarch64_sve_asr_wide, 4398046511235),
SVEMAP1(svasr_wide_s32_z, aarch64_sve_asr_wide, 4398046511491),
SVEMAP1(svasr_wide_s8_m, aarch64_sve_asr_wide, 4398046511361),
SVEMAP1(svasr_wide_s8_x, aarch64_sve_asr_wide, 4398046511233),
SVEMAP1(svasr_wide_s8_z, aarch64_sve_asr_wide, 4398046511489),
SVEMAP1(svasrd_n_s16_m, aarch64_sve_asrd, 4398046511362),
SVEMAP1(svasrd_n_s16_x, aarch64_sve_asrd, 4398046511234),
SVEMAP1(svasrd_n_s16_z, aarch64_sve_asrd, 4398046511490),
SVEMAP1(svasrd_n_s32_m, aarch64_sve_asrd, 4398046511363),
SVEMAP1(svasrd_n_s32_x, aarch64_sve_asrd, 4398046511235),
SVEMAP1(svasrd_n_s32_z, aarch64_sve_asrd, 4398046511491),
SVEMAP1(svasrd_n_s64_m, aarch64_sve_asrd, 4398046511364),
SVEMAP1(svasrd_n_s64_x, aarch64_sve_asrd, 4398046511236),
SVEMAP1(svasrd_n_s64_z, aarch64_sve_asrd, 4398046511492),
SVEMAP1(svasrd_n_s8_m, aarch64_sve_asrd, 4398046511361),
SVEMAP1(svasrd_n_s8_x, aarch64_sve_asrd, 4398046511233),
SVEMAP1(svasrd_n_s8_z, aarch64_sve_asrd, 4398046511489),
SVEMAP1(svbcax_n_s16, aarch64_sve_bcax, 4398046514178),
SVEMAP1(svbcax_n_s32, aarch64_sve_bcax, 4398046514179),
SVEMAP1(svbcax_n_s64, aarch64_sve_bcax, 4398046514180),
SVEMAP1(svbcax_n_s8, aarch64_sve_bcax, 4398046514177),
SVEMAP1(svbcax_n_u16, aarch64_sve_bcax, 4398046514178),
SVEMAP1(svbcax_n_u32, aarch64_sve_bcax, 4398046514179),
SVEMAP1(svbcax_n_u64, aarch64_sve_bcax, 4398046514180),
SVEMAP1(svbcax_n_u8, aarch64_sve_bcax, 4398046514177),
SVEMAP1(svbcax_s16, aarch64_sve_bcax, 4398046511106),
SVEMAP1(svbcax_s32, aarch64_sve_bcax, 4398046511107),
SVEMAP1(svbcax_s64, aarch64_sve_bcax, 4398046511108),
SVEMAP1(svbcax_s8, aarch64_sve_bcax, 4398046511105),
SVEMAP1(svbcax_u16, aarch64_sve_bcax, 4398046511106),
SVEMAP1(svbcax_u32, aarch64_sve_bcax, 4398046511107),
SVEMAP1(svbcax_u64, aarch64_sve_bcax, 4398046511108),
SVEMAP1(svbcax_u8, aarch64_sve_bcax, 4398046511105),
SVEMAP1(svbdep_n_u16, aarch64_sve_bdep_x, 4398046513154),
SVEMAP1(svbdep_n_u32, aarch64_sve_bdep_x, 4398046513155),
SVEMAP1(svbdep_n_u64, aarch64_sve_bdep_x, 4398046513156),
SVEMAP1(svbdep_n_u8, aarch64_sve_bdep_x, 4398046513153),
SVEMAP1(svbdep_u16, aarch64_sve_bdep_x, 4398046511106),
SVEMAP1(svbdep_u32, aarch64_sve_bdep_x, 4398046511107),
SVEMAP1(svbdep_u64, aarch64_sve_bdep_x, 4398046511108),
SVEMAP1(svbdep_u8, aarch64_sve_bdep_x, 4398046511105),
SVEMAP1(svbext_n_u16, aarch64_sve_bext_x, 4398046513154),
SVEMAP1(svbext_n_u32, aarch64_sve_bext_x, 4398046513155),
SVEMAP1(svbext_n_u64, aarch64_sve_bext_x, 4398046513156),
SVEMAP1(svbext_n_u8, aarch64_sve_bext_x, 4398046513153),
SVEMAP1(svbext_u16, aarch64_sve_bext_x, 4398046511106),
SVEMAP1(svbext_u32, aarch64_sve_bext_x, 4398046511107),
SVEMAP1(svbext_u64, aarch64_sve_bext_x, 4398046511108),
SVEMAP1(svbext_u8, aarch64_sve_bext_x, 4398046511105),
SVEMAP1(svbfdot_f32, aarch64_sve_bfdot, 4398047559693),
SVEMAP1(svbfdot_lane_f32, aarch64_sve_bfdot_lane_v2, 4398047559693),
SVEMAP1(svbfdot_n_f32, aarch64_sve_bfdot, 4398047562765),
SVEMAP1(svbfmlalb_f32, aarch64_sve_bfmlalb, 4398047559693),
SVEMAP1(svbfmlalb_lane_f32, aarch64_sve_bfmlalb_lane_v2, 4398047559693),
SVEMAP1(svbfmlalb_n_f32, aarch64_sve_bfmlalb, 4398047562765),
SVEMAP1(svbfmlalt_f32, aarch64_sve_bfmlalt, 4398047559693),
SVEMAP1(svbfmlalt_lane_f32, aarch64_sve_bfmlalt_lane_v2, 4398047559693),
SVEMAP1(svbfmlalt_n_f32, aarch64_sve_bfmlalt, 4398047562765),
SVEMAP1(svbfmlslb_f32, aarch64_sve_bfmlslb, 4398047559687),
SVEMAP1(svbfmlslb_lane_f32, aarch64_sve_bfmlslb_lane, 4398047559687),
SVEMAP1(svbfmlslt_f32, aarch64_sve_bfmlslt, 4398047559687),
SVEMAP1(svbfmlslt_lane_f32, aarch64_sve_bfmlslt_lane, 4398047559687),
SVEMAP1(svbfmmla_f32, aarch64_sve_bfmmla, 1048589),
SVEMAP1(svbgrp_n_u16, aarch64_sve_bgrp_x, 4398046513154),
SVEMAP1(svbgrp_n_u32, aarch64_sve_bgrp_x, 4398046513155),
SVEMAP1(svbgrp_n_u64, aarch64_sve_bgrp_x, 4398046513156),
SVEMAP1(svbgrp_n_u8, aarch64_sve_bgrp_x, 4398046513153),
SVEMAP1(svbgrp_u16, aarch64_sve_bgrp_x, 4398046511106),
SVEMAP1(svbgrp_u32, aarch64_sve_bgrp_x, 4398046511107),
SVEMAP1(svbgrp_u64, aarch64_sve_bgrp_x, 4398046511108),
SVEMAP1(svbgrp_u8, aarch64_sve_bgrp_x, 4398046511105),
SVEMAP1(svbic_b_z, aarch64_sve_bic_z, 4398046511113),
SVEMAP1(svbic_n_s16_m, aarch64_sve_bic, 4398046514434),
SVEMAP1(svbic_n_s16_x, aarch64_sve_bic_u, 4398046514306),
SVEMAP1(svbic_n_s16_z, aarch64_sve_bic, 4398046514562),
SVEMAP1(svbic_n_s32_m, aarch64_sve_bic, 4398046514435),
SVEMAP1(svbic_n_s32_x, aarch64_sve_bic_u, 4398046514307),
SVEMAP1(svbic_n_s32_z, aarch64_sve_bic, 4398046514563),
SVEMAP1(svbic_n_s64_m, aarch64_sve_bic, 4398046514436),
SVEMAP1(svbic_n_s64_x, aarch64_sve_bic_u, 4398046514308),
SVEMAP1(svbic_n_s64_z, aarch64_sve_bic, 4398046514564),
SVEMAP1(svbic_n_s8_m, aarch64_sve_bic, 4398046514433),
SVEMAP1(svbic_n_s8_x, aarch64_sve_bic_u, 4398046514305),
SVEMAP1(svbic_n_s8_z, aarch64_sve_bic, 4398046514561),
SVEMAP1(svbic_n_u16_m, aarch64_sve_bic, 4398046514434),
SVEMAP1(svbic_n_u16_x, aarch64_sve_bic_u, 4398046514306),
SVEMAP1(svbic_n_u16_z, aarch64_sve_bic, 4398046514562),
SVEMAP1(svbic_n_u32_m, aarch64_sve_bic, 4398046514435),
SVEMAP1(svbic_n_u32_x, aarch64_sve_bic_u, 4398046514307),
SVEMAP1(svbic_n_u32_z, aarch64_sve_bic, 4398046514563),
SVEMAP1(svbic_n_u64_m, aarch64_sve_bic, 4398046514436),
SVEMAP1(svbic_n_u64_x, aarch64_sve_bic_u, 4398046514308),
SVEMAP1(svbic_n_u64_z, aarch64_sve_bic, 4398046514564),
SVEMAP1(svbic_n_u8_m, aarch64_sve_bic, 4398046514433),
SVEMAP1(svbic_n_u8_x, aarch64_sve_bic_u, 4398046514305),
SVEMAP1(svbic_n_u8_z, aarch64_sve_bic, 4398046514561),
SVEMAP1(svbic_s16_m, aarch64_sve_bic, 4398046511362),
SVEMAP1(svbic_s16_x, aarch64_sve_bic_u, 4398046511234),
SVEMAP1(svbic_s16_z, aarch64_sve_bic, 4398046511490),
SVEMAP1(svbic_s32_m, aarch64_sve_bic, 4398046511363),
SVEMAP1(svbic_s32_x, aarch64_sve_bic_u, 4398046511235),
SVEMAP1(svbic_s32_z, aarch64_sve_bic, 4398046511491),
SVEMAP1(svbic_s64_m, aarch64_sve_bic, 4398046511364),
SVEMAP1(svbic_s64_x, aarch64_sve_bic_u, 4398046511236),
SVEMAP1(svbic_s64_z, aarch64_sve_bic, 4398046511492),
SVEMAP1(svbic_s8_m, aarch64_sve_bic, 4398046511361),
SVEMAP1(svbic_s8_x, aarch64_sve_bic_u, 4398046511233),
SVEMAP1(svbic_s8_z, aarch64_sve_bic, 4398046511489),
SVEMAP1(svbic_u16_m, aarch64_sve_bic, 4398046511362),
SVEMAP1(svbic_u16_x, aarch64_sve_bic_u, 4398046511234),
SVEMAP1(svbic_u16_z, aarch64_sve_bic, 4398046511490),
SVEMAP1(svbic_u32_m, aarch64_sve_bic, 4398046511363),
SVEMAP1(svbic_u32_x, aarch64_sve_bic_u, 4398046511235),
SVEMAP1(svbic_u32_z, aarch64_sve_bic, 4398046511491),
SVEMAP1(svbic_u64_m, aarch64_sve_bic, 4398046511364),
SVEMAP1(svbic_u64_x, aarch64_sve_bic_u, 4398046511236),
SVEMAP1(svbic_u64_z, aarch64_sve_bic, 4398046511492),
SVEMAP1(svbic_u8_m, aarch64_sve_bic, 4398046511361),
SVEMAP1(svbic_u8_x, aarch64_sve_bic_u, 4398046511233),
SVEMAP1(svbic_u8_z, aarch64_sve_bic, 4398046511489),
SVEMAP1(svbrka_b_m, aarch64_sve_brka, 4398046511113),
SVEMAP1(svbrka_b_z, aarch64_sve_brka_z, 4398046511113),
SVEMAP1(svbrkb_b_m, aarch64_sve_brkb, 4398046511113),
SVEMAP1(svbrkb_b_z, aarch64_sve_brkb_z, 4398046511113),
SVEMAP1(svbrkn_b_z, aarch64_sve_brkn_z, 4398046511113),
SVEMAP1(svbrkpa_b_z, aarch64_sve_brkpa_z, 4398046511113),
SVEMAP1(svbrkpb_b_z, aarch64_sve_brkpb_z, 4398046511113),
SVEMAP1(svbsl1n_n_s16, aarch64_sve_bsl1n, 4398046514178),
SVEMAP1(svbsl1n_n_s32, aarch64_sve_bsl1n, 4398046514179),
SVEMAP1(svbsl1n_n_s64, aarch64_sve_bsl1n, 4398046514180),
SVEMAP1(svbsl1n_n_s8, aarch64_sve_bsl1n, 4398046514177),
SVEMAP1(svbsl1n_n_u16, aarch64_sve_bsl1n, 4398046514178),
SVEMAP1(svbsl1n_n_u32, aarch64_sve_bsl1n, 4398046514179),
SVEMAP1(svbsl1n_n_u64, aarch64_sve_bsl1n, 4398046514180),
SVEMAP1(svbsl1n_n_u8, aarch64_sve_bsl1n, 4398046514177),
SVEMAP1(svbsl1n_s16, aarch64_sve_bsl1n, 4398046511106),
SVEMAP1(svbsl1n_s32, aarch64_sve_bsl1n, 4398046511107),
SVEMAP1(svbsl1n_s64, aarch64_sve_bsl1n, 4398046511108),
SVEMAP1(svbsl1n_s8, aarch64_sve_bsl1n, 4398046511105),
SVEMAP1(svbsl1n_u16, aarch64_sve_bsl1n, 4398046511106),
SVEMAP1(svbsl1n_u32, aarch64_sve_bsl1n, 4398046511107),
SVEMAP1(svbsl1n_u64, aarch64_sve_bsl1n, 4398046511108),
SVEMAP1(svbsl1n_u8, aarch64_sve_bsl1n, 4398046511105),
SVEMAP1(svbsl2n_n_s16, aarch64_sve_bsl2n, 4398046514178),
SVEMAP1(svbsl2n_n_s32, aarch64_sve_bsl2n, 4398046514179),
SVEMAP1(svbsl2n_n_s64, aarch64_sve_bsl2n, 4398046514180),
SVEMAP1(svbsl2n_n_s8, aarch64_sve_bsl2n, 4398046514177),
SVEMAP1(svbsl2n_n_u16, aarch64_sve_bsl2n, 4398046514178),
SVEMAP1(svbsl2n_n_u32, aarch64_sve_bsl2n, 4398046514179),
SVEMAP1(svbsl2n_n_u64, aarch64_sve_bsl2n, 4398046514180),
SVEMAP1(svbsl2n_n_u8, aarch64_sve_bsl2n, 4398046514177),
SVEMAP1(svbsl2n_s16, aarch64_sve_bsl2n, 4398046511106),
SVEMAP1(svbsl2n_s32, aarch64_sve_bsl2n, 4398046511107),
SVEMAP1(svbsl2n_s64, aarch64_sve_bsl2n, 4398046511108),
SVEMAP1(svbsl2n_s8, aarch64_sve_bsl2n, 4398046511105),
SVEMAP1(svbsl2n_u16, aarch64_sve_bsl2n, 4398046511106),
SVEMAP1(svbsl2n_u32, aarch64_sve_bsl2n, 4398046511107),
SVEMAP1(svbsl2n_u64, aarch64_sve_bsl2n, 4398046511108),
SVEMAP1(svbsl2n_u8, aarch64_sve_bsl2n, 4398046511105),
SVEMAP1(svbsl_n_s16, aarch64_sve_bsl, 4398046514178),
SVEMAP1(svbsl_n_s32, aarch64_sve_bsl, 4398046514179),
SVEMAP1(svbsl_n_s64, aarch64_sve_bsl, 4398046514180),
SVEMAP1(svbsl_n_s8, aarch64_sve_bsl, 4398046514177),
SVEMAP1(svbsl_n_u16, aarch64_sve_bsl, 4398046514178),
SVEMAP1(svbsl_n_u32, aarch64_sve_bsl, 4398046514179),
SVEMAP1(svbsl_n_u64, aarch64_sve_bsl, 4398046514180),
SVEMAP1(svbsl_n_u8, aarch64_sve_bsl, 4398046514177),
SVEMAP1(svbsl_s16, aarch64_sve_bsl, 4398046511106),
SVEMAP1(svbsl_s32, aarch64_sve_bsl, 4398046511107),
SVEMAP1(svbsl_s64, aarch64_sve_bsl, 4398046511108),
SVEMAP1(svbsl_s8, aarch64_sve_bsl, 4398046511105),
SVEMAP1(svbsl_u16, aarch64_sve_bsl, 4398046511106),
SVEMAP1(svbsl_u32, aarch64_sve_bsl, 4398046511107),
SVEMAP1(svbsl_u64, aarch64_sve_bsl, 4398046511108),
SVEMAP1(svbsl_u8, aarch64_sve_bsl, 4398046511105),
SVEMAP1(svcadd_f16_m, aarch64_sve_fcadd, 4398046511366),
SVEMAP1(svcadd_f16_x, aarch64_sve_fcadd, 4398046511238),
SVEMAP1(svcadd_f16_z, aarch64_sve_fcadd, 4398046511494),
SVEMAP1(svcadd_f32_m, aarch64_sve_fcadd, 4398046511367),
SVEMAP1(svcadd_f32_x, aarch64_sve_fcadd, 4398046511239),
SVEMAP1(svcadd_f32_z, aarch64_sve_fcadd, 4398046511495),
SVEMAP1(svcadd_f64_m, aarch64_sve_fcadd, 4398046511368),
SVEMAP1(svcadd_f64_x, aarch64_sve_fcadd, 4398046511240),
SVEMAP1(svcadd_f64_z, aarch64_sve_fcadd, 4398046511496),
SVEMAP1(svcadd_s16, aarch64_sve_cadd_x, 4398046511106),
SVEMAP1(svcadd_s32, aarch64_sve_cadd_x, 4398046511107),
SVEMAP1(svcadd_s64, aarch64_sve_cadd_x, 4398046511108),
SVEMAP1(svcadd_s8, aarch64_sve_cadd_x, 4398046511105),
SVEMAP1(svcadd_u16, aarch64_sve_cadd_x, 4398046511106),
SVEMAP1(svcadd_u32, aarch64_sve_cadd_x, 4398046511107),
SVEMAP1(svcadd_u64, aarch64_sve_cadd_x, 4398046511108),
SVEMAP1(svcadd_u8, aarch64_sve_cadd_x, 4398046511105),
SVEMAP1(svcdot_lane_s32, aarch64_sve_cdot_lane, 4398046511107),
SVEMAP1(svcdot_lane_s64, aarch64_sve_cdot_lane, 4398046511108),
SVEMAP1(svcdot_s32, aarch64_sve_cdot, 4398046511107),
SVEMAP1(svcdot_s64, aarch64_sve_cdot, 4398046511108),
SVEMAP1(svclamp_bf16, aarch64_sve_fclamp, 4398046511117),
SVEMAP1(svclamp_f16, aarch64_sve_fclamp, 4398046511110),
SVEMAP1(svclamp_f32, aarch64_sve_fclamp, 4398046511111),
SVEMAP1(svclamp_f64, aarch64_sve_fclamp, 4398046511112),
SVEMAP1(svclamp_s16, aarch64_sve_sclamp, 4398046511106),
SVEMAP1(svclamp_s32, aarch64_sve_sclamp, 4398046511107),
SVEMAP1(svclamp_s64, aarch64_sve_sclamp, 4398046511108),
SVEMAP1(svclamp_s8, aarch64_sve_sclamp, 4398046511105),
SVEMAP1(svclamp_single_bf16_x2, aarch64_sve_bfclamp_single_x2, 137438953485),
SVEMAP1(svclamp_single_bf16_x4, aarch64_sve_bfclamp_single_x4, 137438953485),
SVEMAP1(svclamp_single_f16_x2, aarch64_sve_fclamp_single_x2, 137438953478),
SVEMAP1(svclamp_single_f16_x4, aarch64_sve_fclamp_single_x4, 137438953478),
SVEMAP1(svclamp_single_f32_x2, aarch64_sve_fclamp_single_x2, 137438953479),
SVEMAP1(svclamp_single_f32_x4, aarch64_sve_fclamp_single_x4, 137438953479),
SVEMAP1(svclamp_single_f64_x2, aarch64_sve_fclamp_single_x2, 137438953480),
SVEMAP1(svclamp_single_f64_x4, aarch64_sve_fclamp_single_x4, 137438953480),
SVEMAP1(svclamp_single_s16_x2, aarch64_sve_sclamp_single_x2, 137438953474),
SVEMAP1(svclamp_single_s16_x4, aarch64_sve_sclamp_single_x4, 137438953474),
SVEMAP1(svclamp_single_s32_x2, aarch64_sve_sclamp_single_x2, 137438953475),
SVEMAP1(svclamp_single_s32_x4, aarch64_sve_sclamp_single_x4, 137438953475),
SVEMAP1(svclamp_single_s64_x2, aarch64_sve_sclamp_single_x2, 137438953476),
SVEMAP1(svclamp_single_s64_x4, aarch64_sve_sclamp_single_x4, 137438953476),
SVEMAP1(svclamp_single_s8_x2, aarch64_sve_sclamp_single_x2, 137438953473),
SVEMAP1(svclamp_single_s8_x4, aarch64_sve_sclamp_single_x4, 137438953473),
SVEMAP1(svclamp_single_u16_x2, aarch64_sve_uclamp_single_x2, 137438953474),
SVEMAP1(svclamp_single_u16_x4, aarch64_sve_uclamp_single_x4, 137438953474),
SVEMAP1(svclamp_single_u32_x2, aarch64_sve_uclamp_single_x2, 137438953475),
SVEMAP1(svclamp_single_u32_x4, aarch64_sve_uclamp_single_x4, 137438953475),
SVEMAP1(svclamp_single_u64_x2, aarch64_sve_uclamp_single_x2, 137438953476),
SVEMAP1(svclamp_single_u64_x4, aarch64_sve_uclamp_single_x4, 137438953476),
SVEMAP1(svclamp_single_u8_x2, aarch64_sve_uclamp_single_x2, 137438953473),
SVEMAP1(svclamp_single_u8_x4, aarch64_sve_uclamp_single_x4, 137438953473),
SVEMAP1(svclamp_u16, aarch64_sve_uclamp, 4398046511106),
SVEMAP1(svclamp_u32, aarch64_sve_uclamp, 4398046511107),
SVEMAP1(svclamp_u64, aarch64_sve_uclamp, 4398046511108),
SVEMAP1(svclamp_u8, aarch64_sve_uclamp, 4398046511105),
SVEMAP1(svclasta_bf16, aarch64_sve_clasta, 4398046511117),
SVEMAP1(svclasta_f16, aarch64_sve_clasta, 4398046511110),
SVEMAP1(svclasta_f32, aarch64_sve_clasta, 4398046511111),
SVEMAP1(svclasta_f64, aarch64_sve_clasta, 4398046511112),
SVEMAP1(svclasta_n_bf16, aarch64_sve_clasta_n, 4398046511117),
SVEMAP1(svclasta_n_f16, aarch64_sve_clasta_n, 4398046511110),
SVEMAP1(svclasta_n_f32, aarch64_sve_clasta_n, 4398046511111),
SVEMAP1(svclasta_n_f64, aarch64_sve_clasta_n, 4398046511112),
SVEMAP1(svclasta_n_s16, aarch64_sve_clasta_n, 4398046511106),
SVEMAP1(svclasta_n_s32, aarch64_sve_clasta_n, 4398046511107),
SVEMAP1(svclasta_n_s64, aarch64_sve_clasta_n, 4398046511108),
SVEMAP1(svclasta_n_s8, aarch64_sve_clasta_n, 4398046511105),
SVEMAP1(svclasta_n_u16, aarch64_sve_clasta_n, 4398046511106),
SVEMAP1(svclasta_n_u32, aarch64_sve_clasta_n, 4398046511107),
SVEMAP1(svclasta_n_u64, aarch64_sve_clasta_n, 4398046511108),
SVEMAP1(svclasta_n_u8, aarch64_sve_clasta_n, 4398046511105),
SVEMAP1(svclasta_s16, aarch64_sve_clasta, 4398046511106),
SVEMAP1(svclasta_s32, aarch64_sve_clasta, 4398046511107),
SVEMAP1(svclasta_s64, aarch64_sve_clasta, 4398046511108),
SVEMAP1(svclasta_s8, aarch64_sve_clasta, 4398046511105),
SVEMAP1(svclasta_u16, aarch64_sve_clasta, 4398046511106),
SVEMAP1(svclasta_u32, aarch64_sve_clasta, 4398046511107),
SVEMAP1(svclasta_u64, aarch64_sve_clasta, 4398046511108),
SVEMAP1(svclasta_u8, aarch64_sve_clasta, 4398046511105),
SVEMAP1(svclastb_bf16, aarch64_sve_clastb, 4398046511117),
SVEMAP1(svclastb_f16, aarch64_sve_clastb, 4398046511110),
SVEMAP1(svclastb_f32, aarch64_sve_clastb, 4398046511111),
SVEMAP1(svclastb_f64, aarch64_sve_clastb, 4398046511112),
SVEMAP1(svclastb_n_bf16, aarch64_sve_clastb_n, 4398046511117),
SVEMAP1(svclastb_n_f16, aarch64_sve_clastb_n, 4398046511110),
SVEMAP1(svclastb_n_f32, aarch64_sve_clastb_n, 4398046511111),
SVEMAP1(svclastb_n_f64, aarch64_sve_clastb_n, 4398046511112),
SVEMAP1(svclastb_n_s16, aarch64_sve_clastb_n, 4398046511106),
SVEMAP1(svclastb_n_s32, aarch64_sve_clastb_n, 4398046511107),
SVEMAP1(svclastb_n_s64, aarch64_sve_clastb_n, 4398046511108),
SVEMAP1(svclastb_n_s8, aarch64_sve_clastb_n, 4398046511105),
SVEMAP1(svclastb_n_u16, aarch64_sve_clastb_n, 4398046511106),
SVEMAP1(svclastb_n_u32, aarch64_sve_clastb_n, 4398046511107),
SVEMAP1(svclastb_n_u64, aarch64_sve_clastb_n, 4398046511108),
SVEMAP1(svclastb_n_u8, aarch64_sve_clastb_n, 4398046511105),
SVEMAP1(svclastb_s16, aarch64_sve_clastb, 4398046511106),
SVEMAP1(svclastb_s32, aarch64_sve_clastb, 4398046511107),
SVEMAP1(svclastb_s64, aarch64_sve_clastb, 4398046511108),
SVEMAP1(svclastb_s8, aarch64_sve_clastb, 4398046511105),
SVEMAP1(svclastb_u16, aarch64_sve_clastb, 4398046511106),
SVEMAP1(svclastb_u32, aarch64_sve_clastb, 4398046511107),
SVEMAP1(svclastb_u64, aarch64_sve_clastb, 4398046511108),
SVEMAP1(svclastb_u8, aarch64_sve_clastb, 4398046511105),
SVEMAP1(svcls_s16_m, aarch64_sve_cls, 4398046511362),
SVEMAP1(svcls_s16_x, aarch64_sve_cls, 4398046511618),
SVEMAP1(svcls_s16_z, aarch64_sve_cls, 4398046511746),
SVEMAP1(svcls_s32_m, aarch64_sve_cls, 4398046511363),
SVEMAP1(svcls_s32_x, aarch64_sve_cls, 4398046511619),
SVEMAP1(svcls_s32_z, aarch64_sve_cls, 4398046511747),
SVEMAP1(svcls_s64_m, aarch64_sve_cls, 4398046511364),
SVEMAP1(svcls_s64_x, aarch64_sve_cls, 4398046511620),
SVEMAP1(svcls_s64_z, aarch64_sve_cls, 4398046511748),
SVEMAP1(svcls_s8_m, aarch64_sve_cls, 4398046511361),
SVEMAP1(svcls_s8_x, aarch64_sve_cls, 4398046511617),
SVEMAP1(svcls_s8_z, aarch64_sve_cls, 4398046511745),
SVEMAP1(svclz_s16_m, aarch64_sve_clz, 4398046511362),
SVEMAP1(svclz_s16_x, aarch64_sve_clz, 4398046511618),
SVEMAP1(svclz_s16_z, aarch64_sve_clz, 4398046511746),
SVEMAP1(svclz_s32_m, aarch64_sve_clz, 4398046511363),
SVEMAP1(svclz_s32_x, aarch64_sve_clz, 4398046511619),
SVEMAP1(svclz_s32_z, aarch64_sve_clz, 4398046511747),
SVEMAP1(svclz_s64_m, aarch64_sve_clz, 4398046511364),
SVEMAP1(svclz_s64_x, aarch64_sve_clz, 4398046511620),
SVEMAP1(svclz_s64_z, aarch64_sve_clz, 4398046511748),
SVEMAP1(svclz_s8_m, aarch64_sve_clz, 4398046511361),
SVEMAP1(svclz_s8_x, aarch64_sve_clz, 4398046511617),
SVEMAP1(svclz_s8_z, aarch64_sve_clz, 4398046511745),
SVEMAP1(svclz_u16_m, aarch64_sve_clz, 4398046511362),
SVEMAP1(svclz_u16_x, aarch64_sve_clz, 4398046511618),
SVEMAP1(svclz_u16_z, aarch64_sve_clz, 4398046511746),
SVEMAP1(svclz_u32_m, aarch64_sve_clz, 4398046511363),
SVEMAP1(svclz_u32_x, aarch64_sve_clz, 4398046511619),
SVEMAP1(svclz_u32_z, aarch64_sve_clz, 4398046511747),
SVEMAP1(svclz_u64_m, aarch64_sve_clz, 4398046511364),
SVEMAP1(svclz_u64_x, aarch64_sve_clz, 4398046511620),
SVEMAP1(svclz_u64_z, aarch64_sve_clz, 4398046511748),
SVEMAP1(svclz_u8_m, aarch64_sve_clz, 4398046511361),
SVEMAP1(svclz_u8_x, aarch64_sve_clz, 4398046511617),
SVEMAP1(svclz_u8_z, aarch64_sve_clz, 4398046511745),
SVEMAP1(svcmla_f16_m, aarch64_sve_fcmla, 4398046511366),
SVEMAP1(svcmla_f16_x, aarch64_sve_fcmla, 4398046511238),
SVEMAP1(svcmla_f16_z, aarch64_sve_fcmla, 4398046511494),
SVEMAP1(svcmla_f32_m, aarch64_sve_fcmla, 4398046511367),
SVEMAP1(svcmla_f32_x, aarch64_sve_fcmla, 4398046511239),
SVEMAP1(svcmla_f32_z, aarch64_sve_fcmla, 4398046511495),
SVEMAP1(svcmla_f64_m, aarch64_sve_fcmla, 4398046511368),
SVEMAP1(svcmla_f64_x, aarch64_sve_fcmla, 4398046511240),
SVEMAP1(svcmla_f64_z, aarch64_sve_fcmla, 4398046511496),
SVEMAP1(svcmla_lane_f16, aarch64_sve_fcmla_lane, 4398046511110),
SVEMAP1(svcmla_lane_f32, aarch64_sve_fcmla_lane, 4398046511111),
SVEMAP1(svcmla_lane_s16, aarch64_sve_cmla_lane_x, 4398046511106),
SVEMAP1(svcmla_lane_s32, aarch64_sve_cmla_lane_x, 4398046511107),
SVEMAP1(svcmla_lane_u16, aarch64_sve_cmla_lane_x, 4398046511106),
SVEMAP1(svcmla_lane_u32, aarch64_sve_cmla_lane_x, 4398046511107),
SVEMAP1(svcmla_s16, aarch64_sve_cmla_x, 4398046511106),
SVEMAP1(svcmla_s32, aarch64_sve_cmla_x, 4398046511107),
SVEMAP1(svcmla_s64, aarch64_sve_cmla_x, 4398046511108),
SVEMAP1(svcmla_s8, aarch64_sve_cmla_x, 4398046511105),
SVEMAP1(svcmla_u16, aarch64_sve_cmla_x, 4398046511106),
SVEMAP1(svcmla_u32, aarch64_sve_cmla_x, 4398046511107),
SVEMAP1(svcmla_u64, aarch64_sve_cmla_x, 4398046511108),
SVEMAP1(svcmla_u8, aarch64_sve_cmla_x, 4398046511105),
SVEMAP1(svcmpeq_f16, aarch64_sve_fcmpeq, 4398046511110),
SVEMAP1(svcmpeq_f32, aarch64_sve_fcmpeq, 4398046511111),
SVEMAP1(svcmpeq_f64, aarch64_sve_fcmpeq, 4398046511112),
SVEMAP1(svcmpeq_n_f16, aarch64_sve_fcmpeq, 4398046514182),
SVEMAP1(svcmpeq_n_f32, aarch64_sve_fcmpeq, 4398046514183),
SVEMAP1(svcmpeq_n_f64, aarch64_sve_fcmpeq, 4398046514184),
SVEMAP1(svcmpeq_n_s16, aarch64_sve_cmpeq, 4398046514178),
SVEMAP1(svcmpeq_n_s32, aarch64_sve_cmpeq, 4398046514179),
SVEMAP1(svcmpeq_n_s64, aarch64_sve_cmpeq, 4398046514180),
SVEMAP1(svcmpeq_n_s8, aarch64_sve_cmpeq, 4398046514177),
SVEMAP1(svcmpeq_n_u16, aarch64_sve_cmpeq, 4398046514178),
SVEMAP1(svcmpeq_n_u32, aarch64_sve_cmpeq, 4398046514179),
SVEMAP1(svcmpeq_n_u64, aarch64_sve_cmpeq, 4398046514180),
SVEMAP1(svcmpeq_n_u8, aarch64_sve_cmpeq, 4398046514177),
SVEMAP1(svcmpeq_s16, aarch64_sve_cmpeq, 4398046511106),
SVEMAP1(svcmpeq_s32, aarch64_sve_cmpeq, 4398046511107),
SVEMAP1(svcmpeq_s64, aarch64_sve_cmpeq, 4398046511108),
SVEMAP1(svcmpeq_s8, aarch64_sve_cmpeq, 4398046511105),
SVEMAP1(svcmpeq_u16, aarch64_sve_cmpeq, 4398046511106),
SVEMAP1(svcmpeq_u32, aarch64_sve_cmpeq, 4398046511107),
SVEMAP1(svcmpeq_u64, aarch64_sve_cmpeq, 4398046511108),
SVEMAP1(svcmpeq_u8, aarch64_sve_cmpeq, 4398046511105),
SVEMAP1(svcmpeq_wide_n_s16, aarch64_sve_cmpeq_wide, 4398046514178),
SVEMAP1(svcmpeq_wide_n_s32, aarch64_sve_cmpeq_wide, 4398046514179),
SVEMAP1(svcmpeq_wide_n_s8, aarch64_sve_cmpeq_wide, 4398046514177),
SVEMAP1(svcmpeq_wide_s16, aarch64_sve_cmpeq_wide, 4398046511106),
SVEMAP1(svcmpeq_wide_s32, aarch64_sve_cmpeq_wide, 4398046511107),
SVEMAP1(svcmpeq_wide_s8, aarch64_sve_cmpeq_wide, 4398046511105),
SVEMAP1(svcmpge_f16, aarch64_sve_fcmpge, 4398046511110),
SVEMAP1(svcmpge_f32, aarch64_sve_fcmpge, 4398046511111),
SVEMAP1(svcmpge_f64, aarch64_sve_fcmpge, 4398046511112),
SVEMAP1(svcmpge_n_f16, aarch64_sve_fcmpge, 4398046514182),
SVEMAP1(svcmpge_n_f32, aarch64_sve_fcmpge, 4398046514183),
SVEMAP1(svcmpge_n_f64, aarch64_sve_fcmpge, 4398046514184),
SVEMAP1(svcmpge_n_s16, aarch64_sve_cmpge, 4398046514178),
SVEMAP1(svcmpge_n_s32, aarch64_sve_cmpge, 4398046514179),
SVEMAP1(svcmpge_n_s64, aarch64_sve_cmpge, 4398046514180),
SVEMAP1(svcmpge_n_s8, aarch64_sve_cmpge, 4398046514177),
SVEMAP1(svcmpge_n_u16, aarch64_sve_cmphs, 4398046514178),
SVEMAP1(svcmpge_n_u32, aarch64_sve_cmphs, 4398046514179),
SVEMAP1(svcmpge_n_u64, aarch64_sve_cmphs, 4398046514180),
SVEMAP1(svcmpge_n_u8, aarch64_sve_cmphs, 4398046514177),
SVEMAP1(svcmpge_s16, aarch64_sve_cmpge, 4398046511106),
SVEMAP1(svcmpge_s32, aarch64_sve_cmpge, 4398046511107),
SVEMAP1(svcmpge_s64, aarch64_sve_cmpge, 4398046511108),
SVEMAP1(svcmpge_s8, aarch64_sve_cmpge, 4398046511105),
SVEMAP1(svcmpge_u16, aarch64_sve_cmphs, 4398046511106),
SVEMAP1(svcmpge_u32, aarch64_sve_cmphs, 4398046511107),
SVEMAP1(svcmpge_u64, aarch64_sve_cmphs, 4398046511108),
SVEMAP1(svcmpge_u8, aarch64_sve_cmphs, 4398046511105),
SVEMAP1(svcmpge_wide_n_s16, aarch64_sve_cmpge_wide, 4398046514178),
SVEMAP1(svcmpge_wide_n_s32, aarch64_sve_cmpge_wide, 4398046514179),
SVEMAP1(svcmpge_wide_n_s8, aarch64_sve_cmpge_wide, 4398046514177),
SVEMAP1(svcmpge_wide_n_u16, aarch64_sve_cmphs_wide, 4398046514178),
SVEMAP1(svcmpge_wide_n_u32, aarch64_sve_cmphs_wide, 4398046514179),
SVEMAP1(svcmpge_wide_n_u8, aarch64_sve_cmphs_wide, 4398046514177),
SVEMAP1(svcmpge_wide_s16, aarch64_sve_cmpge_wide, 4398046511106),
SVEMAP1(svcmpge_wide_s32, aarch64_sve_cmpge_wide, 4398046511107),
SVEMAP1(svcmpge_wide_s8, aarch64_sve_cmpge_wide, 4398046511105),
SVEMAP1(svcmpge_wide_u16, aarch64_sve_cmphs_wide, 4398046511106),
SVEMAP1(svcmpge_wide_u32, aarch64_sve_cmphs_wide, 4398046511107),
SVEMAP1(svcmpge_wide_u8, aarch64_sve_cmphs_wide, 4398046511105),
SVEMAP1(svcmpgt_f16, aarch64_sve_fcmpgt, 4398046511110),
SVEMAP1(svcmpgt_f32, aarch64_sve_fcmpgt, 4398046511111),
SVEMAP1(svcmpgt_f64, aarch64_sve_fcmpgt, 4398046511112),
SVEMAP1(svcmpgt_n_f16, aarch64_sve_fcmpgt, 4398046514182),
SVEMAP1(svcmpgt_n_f32, aarch64_sve_fcmpgt, 4398046514183),
SVEMAP1(svcmpgt_n_f64, aarch64_sve_fcmpgt, 4398046514184),
SVEMAP1(svcmpgt_n_s16, aarch64_sve_cmpgt, 4398046514178),
SVEMAP1(svcmpgt_n_s32, aarch64_sve_cmpgt, 4398046514179),
SVEMAP1(svcmpgt_n_s64, aarch64_sve_cmpgt, 4398046514180),
SVEMAP1(svcmpgt_n_s8, aarch64_sve_cmpgt, 4398046514177),
SVEMAP1(svcmpgt_n_u16, aarch64_sve_cmphi, 4398046514178),
SVEMAP1(svcmpgt_n_u32, aarch64_sve_cmphi, 4398046514179),
SVEMAP1(svcmpgt_n_u64, aarch64_sve_cmphi, 4398046514180),
SVEMAP1(svcmpgt_n_u8, aarch64_sve_cmphi, 4398046514177),
SVEMAP1(svcmpgt_s16, aarch64_sve_cmpgt, 4398046511106),
SVEMAP1(svcmpgt_s32, aarch64_sve_cmpgt, 4398046511107),
SVEMAP1(svcmpgt_s64, aarch64_sve_cmpgt, 4398046511108),
SVEMAP1(svcmpgt_s8, aarch64_sve_cmpgt, 4398046511105),
SVEMAP1(svcmpgt_u16, aarch64_sve_cmphi, 4398046511106),
SVEMAP1(svcmpgt_u32, aarch64_sve_cmphi, 4398046511107),
SVEMAP1(svcmpgt_u64, aarch64_sve_cmphi, 4398046511108),
SVEMAP1(svcmpgt_u8, aarch64_sve_cmphi, 4398046511105),
SVEMAP1(svcmpgt_wide_n_s16, aarch64_sve_cmpgt_wide, 4398046514178),
SVEMAP1(svcmpgt_wide_n_s32, aarch64_sve_cmpgt_wide, 4398046514179),
SVEMAP1(svcmpgt_wide_n_s8, aarch64_sve_cmpgt_wide, 4398046514177),
SVEMAP1(svcmpgt_wide_n_u16, aarch64_sve_cmphi_wide, 4398046514178),
SVEMAP1(svcmpgt_wide_n_u32, aarch64_sve_cmphi_wide, 4398046514179),
SVEMAP1(svcmpgt_wide_n_u8, aarch64_sve_cmphi_wide, 4398046514177),
SVEMAP1(svcmpgt_wide_s16, aarch64_sve_cmpgt_wide, 4398046511106),
SVEMAP1(svcmpgt_wide_s32, aarch64_sve_cmpgt_wide, 4398046511107),
SVEMAP1(svcmpgt_wide_s8, aarch64_sve_cmpgt_wide, 4398046511105),
SVEMAP1(svcmpgt_wide_u16, aarch64_sve_cmphi_wide, 4398046511106),
SVEMAP1(svcmpgt_wide_u32, aarch64_sve_cmphi_wide, 4398046511107),
SVEMAP1(svcmpgt_wide_u8, aarch64_sve_cmphi_wide, 4398046511105),
SVEMAP1(svcmple_f16, aarch64_sve_fcmpge, 4398583382022),
SVEMAP1(svcmple_f32, aarch64_sve_fcmpge, 4398583382023),
SVEMAP1(svcmple_f64, aarch64_sve_fcmpge, 4398583382024),
SVEMAP1(svcmple_n_f16, aarch64_sve_fcmpge, 4398583385094),
SVEMAP1(svcmple_n_f32, aarch64_sve_fcmpge, 4398583385095),
SVEMAP1(svcmple_n_f64, aarch64_sve_fcmpge, 4398583385096),
SVEMAP1(svcmple_n_s16, aarch64_sve_cmpge, 4398583385090),
SVEMAP1(svcmple_n_s32, aarch64_sve_cmpge, 4398583385091),
SVEMAP1(svcmple_n_s64, aarch64_sve_cmpge, 4398583385092),
SVEMAP1(svcmple_n_s8, aarch64_sve_cmpge, 4398583385089),
SVEMAP1(svcmple_n_u16, aarch64_sve_cmphs, 4398583385090),
SVEMAP1(svcmple_n_u32, aarch64_sve_cmphs, 4398583385091),
SVEMAP1(svcmple_n_u64, aarch64_sve_cmphs, 4398583385092),
SVEMAP1(svcmple_n_u8, aarch64_sve_cmphs, 4398583385089),
SVEMAP1(svcmple_s16, aarch64_sve_cmpge, 4398583382018),
SVEMAP1(svcmple_s32, aarch64_sve_cmpge, 4398583382019),
SVEMAP1(svcmple_s64, aarch64_sve_cmpge, 4398583382020),
SVEMAP1(svcmple_s8, aarch64_sve_cmpge, 4398583382017),
SVEMAP1(svcmple_u16, aarch64_sve_cmphs, 4398583382018),
SVEMAP1(svcmple_u32, aarch64_sve_cmphs, 4398583382019),
SVEMAP1(svcmple_u64, aarch64_sve_cmphs, 4398583382020),
SVEMAP1(svcmple_u8, aarch64_sve_cmphs, 4398583382017),
SVEMAP1(svcmple_wide_n_s16, aarch64_sve_cmple_wide, 4398046514178),
SVEMAP1(svcmple_wide_n_s32, aarch64_sve_cmple_wide, 4398046514179),
SVEMAP1(svcmple_wide_n_s8, aarch64_sve_cmple_wide, 4398046514177),
SVEMAP1(svcmple_wide_n_u16, aarch64_sve_cmpls_wide, 4398046514178),
SVEMAP1(svcmple_wide_n_u32, aarch64_sve_cmpls_wide, 4398046514179),
SVEMAP1(svcmple_wide_n_u8, aarch64_sve_cmpls_wide, 4398046514177),
SVEMAP1(svcmple_wide_s16, aarch64_sve_cmple_wide, 4398046511106),
SVEMAP1(svcmple_wide_s32, aarch64_sve_cmple_wide, 4398046511107),
SVEMAP1(svcmple_wide_s8, aarch64_sve_cmple_wide, 4398046511105),
SVEMAP1(svcmple_wide_u16, aarch64_sve_cmpls_wide, 4398046511106),
SVEMAP1(svcmple_wide_u32, aarch64_sve_cmpls_wide, 4398046511107),
SVEMAP1(svcmple_wide_u8, aarch64_sve_cmpls_wide, 4398046511105),
SVEMAP1(svcmplt_f16, aarch64_sve_fcmpgt, 4398583382022),
SVEMAP1(svcmplt_f32, aarch64_sve_fcmpgt, 4398583382023),
SVEMAP1(svcmplt_f64, aarch64_sve_fcmpgt, 4398583382024),
SVEMAP1(svcmplt_n_f16, aarch64_sve_fcmpgt, 4398583385094),
SVEMAP1(svcmplt_n_f32, aarch64_sve_fcmpgt, 4398583385095),
SVEMAP1(svcmplt_n_f64, aarch64_sve_fcmpgt, 4398583385096),
SVEMAP1(svcmplt_n_s16, aarch64_sve_cmpgt, 4398583385090),
SVEMAP1(svcmplt_n_s32, aarch64_sve_cmpgt, 4398583385091),
SVEMAP1(svcmplt_n_s64, aarch64_sve_cmpgt, 4398583385092),
SVEMAP1(svcmplt_n_s8, aarch64_sve_cmpgt, 4398583385089),
SVEMAP1(svcmplt_n_u16, aarch64_sve_cmphi, 4398583385090),
SVEMAP1(svcmplt_n_u32, aarch64_sve_cmphi, 4398583385091),
SVEMAP1(svcmplt_n_u64, aarch64_sve_cmphi, 4398583385092),
SVEMAP1(svcmplt_n_u8, aarch64_sve_cmphi, 4398583385089),
SVEMAP1(svcmplt_s16, aarch64_sve_cmpgt, 4398583382018),
SVEMAP1(svcmplt_s32, aarch64_sve_cmpgt, 4398583382019),
SVEMAP1(svcmplt_s64, aarch64_sve_cmpgt, 4398583382020),
SVEMAP1(svcmplt_s8, aarch64_sve_cmpgt, 4398583382017),
SVEMAP1(svcmplt_u16, aarch64_sve_cmphi, 4398583382018),
SVEMAP1(svcmplt_u32, aarch64_sve_cmphi, 4398583382019),
SVEMAP1(svcmplt_u64, aarch64_sve_cmphi, 4398583382020),
SVEMAP1(svcmplt_u8, aarch64_sve_cmphi, 4398583382017),
SVEMAP1(svcmplt_wide_n_s16, aarch64_sve_cmplt_wide, 4398046514178),
SVEMAP1(svcmplt_wide_n_s32, aarch64_sve_cmplt_wide, 4398046514179),
SVEMAP1(svcmplt_wide_n_s8, aarch64_sve_cmplt_wide, 4398046514177),
SVEMAP1(svcmplt_wide_n_u16, aarch64_sve_cmplo_wide, 4398046514178),
SVEMAP1(svcmplt_wide_n_u32, aarch64_sve_cmplo_wide, 4398046514179),
SVEMAP1(svcmplt_wide_n_u8, aarch64_sve_cmplo_wide, 4398046514177),
SVEMAP1(svcmplt_wide_s16, aarch64_sve_cmplt_wide, 4398046511106),
SVEMAP1(svcmplt_wide_s32, aarch64_sve_cmplt_wide, 4398046511107),
SVEMAP1(svcmplt_wide_s8, aarch64_sve_cmplt_wide, 4398046511105),
SVEMAP1(svcmplt_wide_u16, aarch64_sve_cmplo_wide, 4398046511106),
SVEMAP1(svcmplt_wide_u32, aarch64_sve_cmplo_wide, 4398046511107),
SVEMAP1(svcmplt_wide_u8, aarch64_sve_cmplo_wide, 4398046511105),
SVEMAP1(svcmpne_f16, aarch64_sve_fcmpne, 4398046511110),
SVEMAP1(svcmpne_f32, aarch64_sve_fcmpne, 4398046511111),
SVEMAP1(svcmpne_f64, aarch64_sve_fcmpne, 4398046511112),
SVEMAP1(svcmpne_n_f16, aarch64_sve_fcmpne, 4398046514182),
SVEMAP1(svcmpne_n_f32, aarch64_sve_fcmpne, 4398046514183),
SVEMAP1(svcmpne_n_f64, aarch64_sve_fcmpne, 4398046514184),
SVEMAP1(svcmpne_n_s16, aarch64_sve_cmpne, 4398046514178),
SVEMAP1(svcmpne_n_s32, aarch64_sve_cmpne, 4398046514179),
SVEMAP1(svcmpne_n_s64, aarch64_sve_cmpne, 4398046514180),
SVEMAP1(svcmpne_n_s8, aarch64_sve_cmpne, 4398046514177),
SVEMAP1(svcmpne_n_u16, aarch64_sve_cmpne, 4398046514178),
SVEMAP1(svcmpne_n_u32, aarch64_sve_cmpne, 4398046514179),
SVEMAP1(svcmpne_n_u64, aarch64_sve_cmpne, 4398046514180),
SVEMAP1(svcmpne_n_u8, aarch64_sve_cmpne, 4398046514177),
SVEMAP1(svcmpne_s16, aarch64_sve_cmpne, 4398046511106),
SVEMAP1(svcmpne_s32, aarch64_sve_cmpne, 4398046511107),
SVEMAP1(svcmpne_s64, aarch64_sve_cmpne, 4398046511108),
SVEMAP1(svcmpne_s8, aarch64_sve_cmpne, 4398046511105),
SVEMAP1(svcmpne_u16, aarch64_sve_cmpne, 4398046511106),
SVEMAP1(svcmpne_u32, aarch64_sve_cmpne, 4398046511107),
SVEMAP1(svcmpne_u64, aarch64_sve_cmpne, 4398046511108),
SVEMAP1(svcmpne_u8, aarch64_sve_cmpne, 4398046511105),
SVEMAP1(svcmpne_wide_n_s16, aarch64_sve_cmpne_wide, 4398046514178),
SVEMAP1(svcmpne_wide_n_s32, aarch64_sve_cmpne_wide, 4398046514179),
SVEMAP1(svcmpne_wide_n_s8, aarch64_sve_cmpne_wide, 4398046514177),
SVEMAP1(svcmpne_wide_s16, aarch64_sve_cmpne_wide, 4398046511106),
SVEMAP1(svcmpne_wide_s32, aarch64_sve_cmpne_wide, 4398046511107),
SVEMAP1(svcmpne_wide_s8, aarch64_sve_cmpne_wide, 4398046511105),
SVEMAP1(svcmpuo_f16, aarch64_sve_fcmpuo, 4398046511110),
SVEMAP1(svcmpuo_f32, aarch64_sve_fcmpuo, 4398046511111),
SVEMAP1(svcmpuo_f64, aarch64_sve_fcmpuo, 4398046511112),
SVEMAP1(svcmpuo_n_f16, aarch64_sve_fcmpuo, 4398046514182),
SVEMAP1(svcmpuo_n_f32, aarch64_sve_fcmpuo, 4398046514183),
SVEMAP1(svcmpuo_n_f64, aarch64_sve_fcmpuo, 4398046514184),
SVEMAP1(svcnot_s16_m, aarch64_sve_cnot, 4398046511362),
SVEMAP1(svcnot_s16_x, aarch64_sve_cnot, 4398046511618),
SVEMAP1(svcnot_s16_z, aarch64_sve_cnot, 4398046511746),
SVEMAP1(svcnot_s32_m, aarch64_sve_cnot, 4398046511363),
SVEMAP1(svcnot_s32_x, aarch64_sve_cnot, 4398046511619),
SVEMAP1(svcnot_s32_z, aarch64_sve_cnot, 4398046511747),
SVEMAP1(svcnot_s64_m, aarch64_sve_cnot, 4398046511364),
SVEMAP1(svcnot_s64_x, aarch64_sve_cnot, 4398046511620),
SVEMAP1(svcnot_s64_z, aarch64_sve_cnot, 4398046511748),
SVEMAP1(svcnot_s8_m, aarch64_sve_cnot, 4398046511361),
SVEMAP1(svcnot_s8_x, aarch64_sve_cnot, 4398046511617),
SVEMAP1(svcnot_s8_z, aarch64_sve_cnot, 4398046511745),
SVEMAP1(svcnot_u16_m, aarch64_sve_cnot, 4398046511362),
SVEMAP1(svcnot_u16_x, aarch64_sve_cnot, 4398046511618),
SVEMAP1(svcnot_u16_z, aarch64_sve_cnot, 4398046511746),
SVEMAP1(svcnot_u32_m, aarch64_sve_cnot, 4398046511363),
SVEMAP1(svcnot_u32_x, aarch64_sve_cnot, 4398046511619),
SVEMAP1(svcnot_u32_z, aarch64_sve_cnot, 4398046511747),
SVEMAP1(svcnot_u64_m, aarch64_sve_cnot, 4398046511364),
SVEMAP1(svcnot_u64_x, aarch64_sve_cnot, 4398046511620),
SVEMAP1(svcnot_u64_z, aarch64_sve_cnot, 4398046511748),
SVEMAP1(svcnot_u8_m, aarch64_sve_cnot, 4398046511361),
SVEMAP1(svcnot_u8_x, aarch64_sve_cnot, 4398046511617),
SVEMAP1(svcnot_u8_z, aarch64_sve_cnot, 4398046511745),
SVEMAP1(svcnt_bf16_m, aarch64_sve_cnt, 4398046511373),
SVEMAP1(svcnt_bf16_x, aarch64_sve_cnt, 4398046511629),
SVEMAP1(svcnt_bf16_z, aarch64_sve_cnt, 4398046511757),
SVEMAP1(svcnt_f16_m, aarch64_sve_cnt, 4398046511366),
SVEMAP1(svcnt_f16_x, aarch64_sve_cnt, 4398046511622),
SVEMAP1(svcnt_f16_z, aarch64_sve_cnt, 4398046511750),
SVEMAP1(svcnt_f32_m, aarch64_sve_cnt, 4398046511367),
SVEMAP1(svcnt_f32_x, aarch64_sve_cnt, 4398046511623),
SVEMAP1(svcnt_f32_z, aarch64_sve_cnt, 4398046511751),
SVEMAP1(svcnt_f64_m, aarch64_sve_cnt, 4398046511368),
SVEMAP1(svcnt_f64_x, aarch64_sve_cnt, 4398046511624),
SVEMAP1(svcnt_f64_z, aarch64_sve_cnt, 4398046511752),
SVEMAP1(svcnt_s16_m, aarch64_sve_cnt, 4398046511362),
SVEMAP1(svcnt_s16_x, aarch64_sve_cnt, 4398046511618),
SVEMAP1(svcnt_s16_z, aarch64_sve_cnt, 4398046511746),
SVEMAP1(svcnt_s32_m, aarch64_sve_cnt, 4398046511363),
SVEMAP1(svcnt_s32_x, aarch64_sve_cnt, 4398046511619),
SVEMAP1(svcnt_s32_z, aarch64_sve_cnt, 4398046511747),
SVEMAP1(svcnt_s64_m, aarch64_sve_cnt, 4398046511364),
SVEMAP1(svcnt_s64_x, aarch64_sve_cnt, 4398046511620),
SVEMAP1(svcnt_s64_z, aarch64_sve_cnt, 4398046511748),
SVEMAP1(svcnt_s8_m, aarch64_sve_cnt, 4398046511361),
SVEMAP1(svcnt_s8_x, aarch64_sve_cnt, 4398046511617),
SVEMAP1(svcnt_s8_z, aarch64_sve_cnt, 4398046511745),
SVEMAP1(svcnt_u16_m, aarch64_sve_cnt, 4398046511362),
SVEMAP1(svcnt_u16_x, aarch64_sve_cnt, 4398046511618),
SVEMAP1(svcnt_u16_z, aarch64_sve_cnt, 4398046511746),
SVEMAP1(svcnt_u32_m, aarch64_sve_cnt, 4398046511363),
SVEMAP1(svcnt_u32_x, aarch64_sve_cnt, 4398046511619),
SVEMAP1(svcnt_u32_z, aarch64_sve_cnt, 4398046511747),
SVEMAP1(svcnt_u64_m, aarch64_sve_cnt, 4398046511364),
SVEMAP1(svcnt_u64_x, aarch64_sve_cnt, 4398046511620),
SVEMAP1(svcnt_u64_z, aarch64_sve_cnt, 4398046511748),
SVEMAP1(svcnt_u8_m, aarch64_sve_cnt, 4398046511361),
SVEMAP1(svcnt_u8_x, aarch64_sve_cnt, 4398046511617),
SVEMAP1(svcnt_u8_z, aarch64_sve_cnt, 4398046511745),
SVEMAP1(svcntb, aarch64_sve_cntb, 4398081114115),
SVEMAP1(svcntb_pat, aarch64_sve_cntb, 4398047559683),
SVEMAP1(svcntd, aarch64_sve_cntd, 4398081114115),
SVEMAP1(svcntd_pat, aarch64_sve_cntd, 4398047559683),
SVEMAP1(svcnth, aarch64_sve_cnth, 4398081114115),
SVEMAP1(svcnth_pat, aarch64_sve_cnth, 4398047559683),
SVEMAP1(svcntp_b16, aarch64_sve_cntp, 4398046511114),
SVEMAP1(svcntp_b32, aarch64_sve_cntp, 4398046511115),
SVEMAP1(svcntp_b64, aarch64_sve_cntp, 4398046511116),
SVEMAP1(svcntp_b8, aarch64_sve_cntp, 4398046511113),
SVEMAP1(svcntp_c16, aarch64_sve_cntp_c16, 4398047559690),
SVEMAP1(svcntp_c32, aarch64_sve_cntp_c32, 4398047559691),
SVEMAP1(svcntp_c64, aarch64_sve_cntp_c64, 4398047559692),
SVEMAP1(svcntp_c8, aarch64_sve_cntp_c8, 4398047559689),
SVEMAP1(svcntw, aarch64_sve_cntw, 4398081114115),
SVEMAP1(svcntw_pat, aarch64_sve_cntw, 4398047559683),
SVEMAP1(svcompact_f32, aarch64_sve_compact, 7),
SVEMAP1(svcompact_f64, aarch64_sve_compact, 8),
SVEMAP1(svcompact_s32, aarch64_sve_compact, 3),
SVEMAP1(svcompact_s64, aarch64_sve_compact, 4),
SVEMAP1(svcompact_u32, aarch64_sve_compact, 3),
SVEMAP1(svcompact_u64, aarch64_sve_compact, 4),
SVEMAP2(svcreate2_b, 4402341478409),
SVEMAP2(svcreate2_bf16, 4402341478413),
SVEMAP2(svcreate2_f16, 4402341478406),
SVEMAP2(svcreate2_f32, 4402341478407),
SVEMAP2(svcreate2_f64, 4402341478408),
SVEMAP2(svcreate2_mf8, 4402341478414),
SVEMAP2(svcreate2_s16, 4402341478402),
SVEMAP2(svcreate2_s32, 4402341478403),
SVEMAP2(svcreate2_s64, 4402341478404),
SVEMAP2(svcreate2_s8, 4402341478401),
SVEMAP2(svcreate2_u16, 4402341478402),
SVEMAP2(svcreate2_u32, 4402341478403),
SVEMAP2(svcreate2_u64, 4402341478404),
SVEMAP2(svcreate2_u8, 4402341478401),
SVEMAP2(svcreate3_bf16, 4402341478413),
SVEMAP2(svcreate3_f16, 4402341478406),
SVEMAP2(svcreate3_f32, 4402341478407),
SVEMAP2(svcreate3_f64, 4402341478408),
SVEMAP2(svcreate3_mf8, 4402341478414),
SVEMAP2(svcreate3_s16, 4402341478402),
SVEMAP2(svcreate3_s32, 4402341478403),
SVEMAP2(svcreate3_s64, 4402341478404),
SVEMAP2(svcreate3_s8, 4402341478401),
SVEMAP2(svcreate3_u16, 4402341478402),
SVEMAP2(svcreate3_u32, 4402341478403),
SVEMAP2(svcreate3_u64, 4402341478404),
SVEMAP2(svcreate3_u8, 4402341478401),
SVEMAP2(svcreate4_b, 4402341478409),
SVEMAP2(svcreate4_bf16, 4402341478413),
SVEMAP2(svcreate4_f16, 4402341478406),
SVEMAP2(svcreate4_f32, 4402341478407),
SVEMAP2(svcreate4_f64, 4402341478408),
SVEMAP2(svcreate4_mf8, 4402341478414),
SVEMAP2(svcreate4_s16, 4402341478402),
SVEMAP2(svcreate4_s32, 4402341478403),
SVEMAP2(svcreate4_s64, 4402341478404),
SVEMAP2(svcreate4_s8, 4402341478401),
SVEMAP2(svcreate4_u16, 4402341478402),
SVEMAP2(svcreate4_u32, 4402341478403),
SVEMAP2(svcreate4_u64, 4402341478404),
SVEMAP2(svcreate4_u8, 4402341478401),
SVEMAP1(svcvt1_bf16_mf8_fpm, aarch64_sve_fp8_cvt1, 567347999932429),
SVEMAP1(svcvt1_bf16_mf8_x2_fpm, aarch64_sve_fp8_cvt1_x2, 563087392374797),
SVEMAP1(svcvt1_f16_mf8_fpm, aarch64_sve_fp8_cvt1, 567347999932422),
SVEMAP1(svcvt1_f16_mf8_x2_fpm, aarch64_sve_fp8_cvt1_x2, 563087392374790),
SVEMAP1(svcvt2_bf16_mf8_fpm, aarch64_sve_fp8_cvt2, 567347999932429),
SVEMAP1(svcvt2_bf16_mf8_x2_fpm, aarch64_sve_fp8_cvt2_x2, 563087392374797),
SVEMAP1(svcvt2_f16_mf8_fpm, aarch64_sve_fp8_cvt2, 567347999932422),
SVEMAP1(svcvt2_f16_mf8_x2_fpm, aarch64_sve_fp8_cvt2_x2, 563087392374790),
SVEMAP1(svcvt_bf16_f32_m, aarch64_sve_fcvt_bf16f32_v2, 4398047559943),
SVEMAP1(svcvt_bf16_f32_x, aarch64_sve_fcvt_bf16f32_v2, 4398047560199),
SVEMAP1(svcvt_bf16_f32_x2, aarch64_sve_bfcvt_x2, 137440002055),
SVEMAP1(svcvt_bf16_f32_z, aarch64_sve_fcvt_bf16f32_v2, 4398047560327),
SVEMAP1(svcvt_f16_f32_m, aarch64_sve_fcvt_f16f32, 4398047559943),
SVEMAP1(svcvt_f16_f32_x, aarch64_sve_fcvt_f16f32, 4398047560199),
SVEMAP1(svcvt_f16_f32_x2, aarch64_sve_fcvt_x2, 137438953479),
SVEMAP1(svcvt_f16_f32_z, aarch64_sve_fcvt_f16f32, 4398047560327),
SVEMAP1(svcvt_f16_f64_m, aarch64_sve_fcvt_f16f64, 4398047559944),
SVEMAP1(svcvt_f16_f64_x, aarch64_sve_fcvt_f16f64, 4398047560200),
SVEMAP1(svcvt_f16_f64_z, aarch64_sve_fcvt_f16f64, 4398047560328),
SVEMAP1(svcvt_f16_s16_m, aarch64_sve_scvtf, 4398054899970),
SVEMAP1(svcvt_f16_s16_x, aarch64_sve_scvtf, 4398054900226),
SVEMAP1(svcvt_f16_s16_z, aarch64_sve_scvtf, 4398054900354),
SVEMAP1(svcvt_f16_s32_m, aarch64_sve_scvtf_f16i32, 4398047559939),
SVEMAP1(svcvt_f16_s32_x, aarch64_sve_scvtf_f16i32, 4398047560195),
SVEMAP1(svcvt_f16_s32_z, aarch64_sve_scvtf_f16i32, 4398047560323),
SVEMAP1(svcvt_f16_s64_m, aarch64_sve_scvtf_f16i64, 4398047559940),
SVEMAP1(svcvt_f16_s64_x, aarch64_sve_scvtf_f16i64, 4398047560196),
SVEMAP1(svcvt_f16_s64_z, aarch64_sve_scvtf_f16i64, 4398047560324),
SVEMAP1(svcvt_f16_u16_m, aarch64_sve_ucvtf, 4398054899970),
SVEMAP1(svcvt_f16_u16_x, aarch64_sve_ucvtf, 4398054900226),
SVEMAP1(svcvt_f16_u16_z, aarch64_sve_ucvtf, 4398054900354),
SVEMAP1(svcvt_f16_u32_m, aarch64_sve_ucvtf_f16i32, 4398047559939),
SVEMAP1(svcvt_f16_u32_x, aarch64_sve_ucvtf_f16i32, 4398047560195),
SVEMAP1(svcvt_f16_u32_z, aarch64_sve_ucvtf_f16i32, 4398047560323),
SVEMAP1(svcvt_f16_u64_m, aarch64_sve_ucvtf_f16i64, 4398047559940),
SVEMAP1(svcvt_f16_u64_x, aarch64_sve_ucvtf_f16i64, 4398047560196),
SVEMAP1(svcvt_f16_u64_z, aarch64_sve_ucvtf_f16i64, 4398047560324),
SVEMAP1(svcvt_f32_f16_m, aarch64_sve_fcvt_f32f16, 4398047559943),
SVEMAP1(svcvt_f32_f16_x, aarch64_sve_fcvt_f32f16, 4398047560199),
SVEMAP1(svcvt_f32_f16_x2, aarch64_sve_fcvt_widen_x2, 137438953479),
SVEMAP1(svcvt_f32_f16_z, aarch64_sve_fcvt_f32f16, 4398047560327),
SVEMAP1(svcvt_f32_f64_m, aarch64_sve_fcvt_f32f64, 4398047559944),
SVEMAP1(svcvt_f32_f64_x, aarch64_sve_fcvt_f32f64, 4398047560200),
SVEMAP1(svcvt_f32_f64_z, aarch64_sve_fcvt_f32f64, 4398047560328),
SVEMAP1(svcvt_f32_s32_m, aarch64_sve_scvtf, 4398054899971),
SVEMAP1(svcvt_f32_s32_x, aarch64_sve_scvtf, 4398054900227),
SVEMAP1(svcvt_f32_s32_x2, aarch64_sve_scvtf_x2, 137441050631),
SVEMAP1(svcvt_f32_s32_x4, aarch64_sve_scvtf_x4, 137441050631),
SVEMAP1(svcvt_f32_s32_z, aarch64_sve_scvtf, 4398054900355),
SVEMAP1(svcvt_f32_s64_m, aarch64_sve_scvtf_f32i64, 4398047559940),
SVEMAP1(svcvt_f32_s64_x, aarch64_sve_scvtf_f32i64, 4398047560196),
SVEMAP1(svcvt_f32_s64_z, aarch64_sve_scvtf_f32i64, 4398047560324),
SVEMAP1(svcvt_f32_u32_m, aarch64_sve_ucvtf, 4398054899971),
SVEMAP1(svcvt_f32_u32_x, aarch64_sve_ucvtf, 4398054900227),
SVEMAP1(svcvt_f32_u32_x2, aarch64_sve_ucvtf_x2, 137441050631),
SVEMAP1(svcvt_f32_u32_x4, aarch64_sve_ucvtf_x4, 137441050631),
SVEMAP1(svcvt_f32_u32_z, aarch64_sve_ucvtf, 4398054900355),
SVEMAP1(svcvt_f32_u64_m, aarch64_sve_ucvtf_f32i64, 4398047559940),
SVEMAP1(svcvt_f32_u64_x, aarch64_sve_ucvtf_f32i64, 4398047560196),
SVEMAP1(svcvt_f32_u64_z, aarch64_sve_ucvtf_f32i64, 4398047560324),
SVEMAP1(svcvt_f64_f16_m, aarch64_sve_fcvt_f64f16, 4398047559944),
SVEMAP1(svcvt_f64_f16_x, aarch64_sve_fcvt_f64f16, 4398047560200),
SVEMAP1(svcvt_f64_f16_z, aarch64_sve_fcvt_f64f16, 4398047560328),
SVEMAP1(svcvt_f64_f32_m, aarch64_sve_fcvt_f64f32, 4398047559944),
SVEMAP1(svcvt_f64_f32_x, aarch64_sve_fcvt_f64f32, 4398047560200),
SVEMAP1(svcvt_f64_f32_z, aarch64_sve_fcvt_f64f32, 4398047560328),
SVEMAP1(svcvt_f64_s32_m, aarch64_sve_scvtf_f64i32, 4398047559944),
SVEMAP1(svcvt_f64_s32_x, aarch64_sve_scvtf_f64i32, 4398047560200),
SVEMAP1(svcvt_f64_s32_z, aarch64_sve_scvtf_f64i32, 4398047560328),
SVEMAP1(svcvt_f64_s64_m, aarch64_sve_scvtf, 4398054899972),
SVEMAP1(svcvt_f64_s64_x, aarch64_sve_scvtf, 4398054900228),
SVEMAP1(svcvt_f64_s64_z, aarch64_sve_scvtf, 4398054900356),
SVEMAP1(svcvt_f64_u32_m, aarch64_sve_ucvtf_f64i32, 4398047559944),
SVEMAP1(svcvt_f64_u32_x, aarch64_sve_ucvtf_f64i32, 4398047560200),
SVEMAP1(svcvt_f64_u32_z, aarch64_sve_ucvtf_f64i32, 4398047560328),
SVEMAP1(svcvt_f64_u64_m, aarch64_sve_ucvtf, 4398054899972),
SVEMAP1(svcvt_f64_u64_x, aarch64_sve_ucvtf, 4398054900228),
SVEMAP1(svcvt_f64_u64_z, aarch64_sve_ucvtf, 4398054900356),
SVEMAP1(svcvt_mf8_bf16_x2_fpm, aarch64_sve_fp8_cvt_x2, 563087392374797),
SVEMAP1(svcvt_mf8_f16_x2_fpm, aarch64_sve_fp8_cvt_x2, 563087392374790),
SVEMAP1(svcvt_mf8_f32_x4_fpm, aarch64_sve_fp8_cvt_x4, 563087393423367),
SVEMAP1(svcvt_s16_f16_m, aarch64_sve_fcvtzs, 4398054899970),
SVEMAP1(svcvt_s16_f16_x, aarch64_sve_fcvtzs, 4398054900226),
SVEMAP1(svcvt_s16_f16_z, aarch64_sve_fcvtzs, 4398054900354),
SVEMAP1(svcvt_s32_f16_m, aarch64_sve_fcvtzs_i32f16, 4398047559939),
SVEMAP1(svcvt_s32_f16_x, aarch64_sve_fcvtzs_i32f16, 4398047560195),
SVEMAP1(svcvt_s32_f16_z, aarch64_sve_fcvtzs_i32f16, 4398047560323),
SVEMAP1(svcvt_s32_f32_m, aarch64_sve_fcvtzs, 4398054899971),
SVEMAP1(svcvt_s32_f32_x, aarch64_sve_fcvtzs, 4398054900227),
SVEMAP1(svcvt_s32_f32_x2, aarch64_sve_fcvtzs_x2, 137441050627),
SVEMAP1(svcvt_s32_f32_x4, aarch64_sve_fcvtzs_x4, 137441050627),
SVEMAP1(svcvt_s32_f32_z, aarch64_sve_fcvtzs, 4398054900355),
SVEMAP1(svcvt_s32_f64_m, aarch64_sve_fcvtzs_i32f64, 4398047559944),
SVEMAP1(svcvt_s32_f64_x, aarch64_sve_fcvtzs_i32f64, 4398047560200),
SVEMAP1(svcvt_s32_f64_z, aarch64_sve_fcvtzs_i32f64, 4398047560328),
SVEMAP1(svcvt_s64_f16_m, aarch64_sve_fcvtzs_i64f16, 4398047559940),
SVEMAP1(svcvt_s64_f16_x, aarch64_sve_fcvtzs_i64f16, 4398047560196),
SVEMAP1(svcvt_s64_f16_z, aarch64_sve_fcvtzs_i64f16, 4398047560324),
SVEMAP1(svcvt_s64_f32_m, aarch64_sve_fcvtzs_i64f32, 4398047559940),
SVEMAP1(svcvt_s64_f32_x, aarch64_sve_fcvtzs_i64f32, 4398047560196),
SVEMAP1(svcvt_s64_f32_z, aarch64_sve_fcvtzs_i64f32, 4398047560324),
SVEMAP1(svcvt_s64_f64_m, aarch64_sve_fcvtzs, 4398054899972),
SVEMAP1(svcvt_s64_f64_x, aarch64_sve_fcvtzs, 4398054900228),
SVEMAP1(svcvt_s64_f64_z, aarch64_sve_fcvtzs, 4398054900356),
SVEMAP1(svcvt_u16_f16_m, aarch64_sve_fcvtzu, 4398054899970),
SVEMAP1(svcvt_u16_f16_x, aarch64_sve_fcvtzu, 4398054900226),
SVEMAP1(svcvt_u16_f16_z, aarch64_sve_fcvtzu, 4398054900354),
SVEMAP1(svcvt_u32_f16_m, aarch64_sve_fcvtzu_i32f16, 4398047559939),
SVEMAP1(svcvt_u32_f16_x, aarch64_sve_fcvtzu_i32f16, 4398047560195),
SVEMAP1(svcvt_u32_f16_z, aarch64_sve_fcvtzu_i32f16, 4398047560323),
SVEMAP1(svcvt_u32_f32_m, aarch64_sve_fcvtzu, 4398054899971),
SVEMAP1(svcvt_u32_f32_x, aarch64_sve_fcvtzu, 4398054900227),
SVEMAP1(svcvt_u32_f32_x2, aarch64_sve_fcvtzu_x2, 137441050627),
SVEMAP1(svcvt_u32_f32_x4, aarch64_sve_fcvtzu_x4, 137441050627),
SVEMAP1(svcvt_u32_f32_z, aarch64_sve_fcvtzu, 4398054900355),
SVEMAP1(svcvt_u32_f64_m, aarch64_sve_fcvtzu_i32f64, 4398047559944),
SVEMAP1(svcvt_u32_f64_x, aarch64_sve_fcvtzu_i32f64, 4398047560200),
SVEMAP1(svcvt_u32_f64_z, aarch64_sve_fcvtzu_i32f64, 4398047560328),
SVEMAP1(svcvt_u64_f16_m, aarch64_sve_fcvtzu_i64f16, 4398047559940),
SVEMAP1(svcvt_u64_f16_x, aarch64_sve_fcvtzu_i64f16, 4398047560196),
SVEMAP1(svcvt_u64_f16_z, aarch64_sve_fcvtzu_i64f16, 4398047560324),
SVEMAP1(svcvt_u64_f32_m, aarch64_sve_fcvtzu_i64f32, 4398047559940),
SVEMAP1(svcvt_u64_f32_x, aarch64_sve_fcvtzu_i64f32, 4398047560196),
SVEMAP1(svcvt_u64_f32_z, aarch64_sve_fcvtzu_i64f32, 4398047560324),
SVEMAP1(svcvt_u64_f64_m, aarch64_sve_fcvtzu, 4398054899972),
SVEMAP1(svcvt_u64_f64_x, aarch64_sve_fcvtzu, 4398054900228),
SVEMAP1(svcvt_u64_f64_z, aarch64_sve_fcvtzu, 4398054900356),
SVEMAP1(svcvtl1_bf16_mf8_x2_fpm, aarch64_sve_fp8_cvtl1_x2, 563087392374797),
SVEMAP1(svcvtl1_f16_mf8_x2_fpm, aarch64_sve_fp8_cvtl1_x2, 563087392374790),
SVEMAP1(svcvtl2_bf16_mf8_x2_fpm, aarch64_sve_fp8_cvtl2_x2, 563087392374797),
SVEMAP1(svcvtl2_f16_mf8_x2_fpm, aarch64_sve_fp8_cvtl2_x2, 563087392374790),
SVEMAP1(svcvtl_f32_f16_x2, aarch64_sve_fcvtl_widen_x2, 137438953479),
SVEMAP1(svcvtlt1_bf16_mf8_fpm, aarch64_sve_fp8_cvtlt1, 567347999932429),
SVEMAP1(svcvtlt1_f16_mf8_fpm, aarch64_sve_fp8_cvtlt1, 567347999932422),
SVEMAP1(svcvtlt2_bf16_mf8_fpm, aarch64_sve_fp8_cvtlt2, 567347999932429),
SVEMAP1(svcvtlt2_f16_mf8_fpm, aarch64_sve_fp8_cvtlt2, 567347999932422),
SVEMAP1(svcvtlt_f32_f16_m, aarch64_sve_fcvtlt_f32f16, 4398047559943),
SVEMAP1(svcvtlt_f32_f16_x, aarch64_sve_fcvtlt_f32f16, 4398047560199),
SVEMAP1(svcvtlt_f64_f32_m, aarch64_sve_fcvtlt_f64f32, 4398047559944),
SVEMAP1(svcvtlt_f64_f32_x, aarch64_sve_fcvtlt_f64f32, 4398047560200),
SVEMAP1(svcvtn_bf16_f32_x2, aarch64_sve_bfcvtn_x2, 137440002055),
SVEMAP1(svcvtn_f16_f32_x2, aarch64_sve_fcvtn_x2, 137438953479),
SVEMAP1(svcvtn_mf8_bf16_x2_fpm, aarch64_sve_fp8_cvtn, 567347999932429),
SVEMAP1(svcvtn_mf8_f16_x2_fpm, aarch64_sve_fp8_cvtn, 567347999932422),
SVEMAP1(svcvtn_mf8_f32_x4_fpm, aarch64_sve_fp8_cvtn_x4, 563087393423367),
SVEMAP1(svcvtnb_mf8_f32_x2_fpm, aarch64_sve_fp8_cvtnb, 567347999932423),
SVEMAP1(svcvtnt_bf16_f32_m, aarch64_sve_fcvtnt_bf16f32_v2, 4398047559943),
SVEMAP1(svcvtnt_f16_f32_m, aarch64_sve_fcvtnt_f16f32, 4398047559943),
SVEMAP1(svcvtnt_f32_f64_m, aarch64_sve_fcvtnt_f32f64, 4398047559944),
SVEMAP1(svcvtnt_mf8_f32_x2_fpm, aarch64_sve_fp8_cvtnt, 567347999932423),
SVEMAP1(svcvtx_f32_f64_m, aarch64_sve_fcvtx_f32f64, 4398047559944),
SVEMAP1(svcvtx_f32_f64_x, aarch64_sve_fcvtx_f32f64, 4398047560200),
SVEMAP1(svcvtx_f32_f64_z, aarch64_sve_fcvtx_f32f64, 4398047560328),
SVEMAP1(svcvtxnt_f32_f64_m, aarch64_sve_fcvtxnt_f32f64, 4398047559944),
SVEMAP1(svdiv_f16_m, aarch64_sve_fdiv, 4398046511366),
SVEMAP1(svdiv_f16_x, aarch64_sve_fdiv_u, 4398046511238),
SVEMAP1(svdiv_f16_z, aarch64_sve_fdiv, 4398046511494),
SVEMAP1(svdiv_f32_m, aarch64_sve_fdiv, 4398046511367),
SVEMAP1(svdiv_f32_x, aarch64_sve_fdiv_u, 4398046511239),
SVEMAP1(svdiv_f32_z, aarch64_sve_fdiv, 4398046511495),
SVEMAP1(svdiv_f64_m, aarch64_sve_fdiv, 4398046511368),
SVEMAP1(svdiv_f64_x, aarch64_sve_fdiv_u, 4398046511240),
SVEMAP1(svdiv_f64_z, aarch64_sve_fdiv, 4398046511496),
SVEMAP1(svdiv_n_f16_m, aarch64_sve_fdiv, 4398046514438),
SVEMAP1(svdiv_n_f16_x, aarch64_sve_fdiv_u, 4398046514310),
SVEMAP1(svdiv_n_f16_z, aarch64_sve_fdiv, 4398046514566),
SVEMAP1(svdiv_n_f32_m, aarch64_sve_fdiv, 4398046514439),
SVEMAP1(svdiv_n_f32_x, aarch64_sve_fdiv_u, 4398046514311),
SVEMAP1(svdiv_n_f32_z, aarch64_sve_fdiv, 4398046514567),
SVEMAP1(svdiv_n_f64_m, aarch64_sve_fdiv, 4398046514440),
SVEMAP1(svdiv_n_f64_x, aarch64_sve_fdiv_u, 4398046514312),
SVEMAP1(svdiv_n_f64_z, aarch64_sve_fdiv, 4398046514568),
SVEMAP1(svdiv_n_s32_m, aarch64_sve_sdiv, 4398046514435),
SVEMAP1(svdiv_n_s32_x, aarch64_sve_sdiv_u, 4398046514307),
SVEMAP1(svdiv_n_s32_z, aarch64_sve_sdiv, 4398046514563),
SVEMAP1(svdiv_n_s64_m, aarch64_sve_sdiv, 4398046514436),
SVEMAP1(svdiv_n_s64_x, aarch64_sve_sdiv_u, 4398046514308),
SVEMAP1(svdiv_n_s64_z, aarch64_sve_sdiv, 4398046514564),
SVEMAP1(svdiv_n_u32_m, aarch64_sve_udiv, 4398046514435),
SVEMAP1(svdiv_n_u32_x, aarch64_sve_udiv_u, 4398046514307),
SVEMAP1(svdiv_n_u32_z, aarch64_sve_udiv, 4398046514563),
SVEMAP1(svdiv_n_u64_m, aarch64_sve_udiv, 4398046514436),
SVEMAP1(svdiv_n_u64_x, aarch64_sve_udiv_u, 4398046514308),
SVEMAP1(svdiv_n_u64_z, aarch64_sve_udiv, 4398046514564),
SVEMAP1(svdiv_s32_m, aarch64_sve_sdiv, 4398046511363),
SVEMAP1(svdiv_s32_x, aarch64_sve_sdiv_u, 4398046511235),
SVEMAP1(svdiv_s32_z, aarch64_sve_sdiv, 4398046511491),
SVEMAP1(svdiv_s64_m, aarch64_sve_sdiv, 4398046511364),
SVEMAP1(svdiv_s64_x, aarch64_sve_sdiv_u, 4398046511236),
SVEMAP1(svdiv_s64_z, aarch64_sve_sdiv, 4398046511492),
SVEMAP1(svdiv_u32_m, aarch64_sve_udiv, 4398046511363),
SVEMAP1(svdiv_u32_x, aarch64_sve_udiv_u, 4398046511235),
SVEMAP1(svdiv_u32_z, aarch64_sve_udiv, 4398046511491),
SVEMAP1(svdiv_u64_m, aarch64_sve_udiv, 4398046511364),
SVEMAP1(svdiv_u64_x, aarch64_sve_udiv_u, 4398046511236),
SVEMAP1(svdiv_u64_z, aarch64_sve_udiv, 4398046511492),
SVEMAP1(svdivr_f16_m, aarch64_sve_fdivr, 4432406249734),
SVEMAP1(svdivr_f16_x, aarch64_sve_fdiv_u, 4432406249606),
SVEMAP1(svdivr_f16_z, aarch64_sve_fdivr, 4432406249862),
SVEMAP1(svdivr_f32_m, aarch64_sve_fdivr, 4432406249735),
SVEMAP1(svdivr_f32_x, aarch64_sve_fdiv_u, 4432406249607),
SVEMAP1(svdivr_f32_z, aarch64_sve_fdivr, 4432406249863),
SVEMAP1(svdivr_f64_m, aarch64_sve_fdivr, 4432406249736),
SVEMAP1(svdivr_f64_x, aarch64_sve_fdiv_u, 4432406249608),
SVEMAP1(svdivr_f64_z, aarch64_sve_fdivr, 4432406249864),
SVEMAP1(svdivr_n_f16_m, aarch64_sve_fdivr, 4432406252806),
SVEMAP1(svdivr_n_f16_x, aarch64_sve_fdiv_u, 4432406252678),
SVEMAP1(svdivr_n_f16_z, aarch64_sve_fdivr, 4432406252934),
SVEMAP1(svdivr_n_f32_m, aarch64_sve_fdivr, 4432406252807),
SVEMAP1(svdivr_n_f32_x, aarch64_sve_fdiv_u, 4432406252679),
SVEMAP1(svdivr_n_f32_z, aarch64_sve_fdivr, 4432406252935),
SVEMAP1(svdivr_n_f64_m, aarch64_sve_fdivr, 4432406252808),
SVEMAP1(svdivr_n_f64_x, aarch64_sve_fdiv_u, 4432406252680),
SVEMAP1(svdivr_n_f64_z, aarch64_sve_fdivr, 4432406252936),
SVEMAP1(svdivr_n_s32_m, aarch64_sve_sdivr, 4432406252803),
SVEMAP1(svdivr_n_s32_x, aarch64_sve_sdiv_u, 4432406252675),
SVEMAP1(svdivr_n_s32_z, aarch64_sve_sdivr, 4432406252931),
SVEMAP1(svdivr_n_s64_m, aarch64_sve_sdivr, 4432406252804),
SVEMAP1(svdivr_n_s64_x, aarch64_sve_sdiv_u, 4432406252676),
SVEMAP1(svdivr_n_s64_z, aarch64_sve_sdivr, 4432406252932),
SVEMAP1(svdivr_n_u32_m, aarch64_sve_udivr, 4432406252803),
SVEMAP1(svdivr_n_u32_x, aarch64_sve_udiv_u, 4432406252675),
SVEMAP1(svdivr_n_u32_z, aarch64_sve_udivr, 4432406252931),
SVEMAP1(svdivr_n_u64_m, aarch64_sve_udivr, 4432406252804),
SVEMAP1(svdivr_n_u64_x, aarch64_sve_udiv_u, 4432406252676),
SVEMAP1(svdivr_n_u64_z, aarch64_sve_udivr, 4432406252932),
SVEMAP1(svdivr_s32_m, aarch64_sve_sdivr, 4432406249731),
SVEMAP1(svdivr_s32_x, aarch64_sve_sdiv_u, 4432406249603),
SVEMAP1(svdivr_s32_z, aarch64_sve_sdivr, 4432406249859),
SVEMAP1(svdivr_s64_m, aarch64_sve_sdivr, 4432406249732),
SVEMAP1(svdivr_s64_x, aarch64_sve_sdiv_u, 4432406249604),
SVEMAP1(svdivr_s64_z, aarch64_sve_sdivr, 4432406249860),
SVEMAP1(svdivr_u32_m, aarch64_sve_udivr, 4432406249731),
SVEMAP1(svdivr_u32_x, aarch64_sve_udiv_u, 4432406249603),
SVEMAP1(svdivr_u32_z, aarch64_sve_udivr, 4432406249859),
SVEMAP1(svdivr_u64_m, aarch64_sve_udivr, 4432406249732),
SVEMAP1(svdivr_u64_x, aarch64_sve_udiv_u, 4432406249604),
SVEMAP1(svdivr_u64_z, aarch64_sve_udivr, 4432406249860),
SVEMAP1(svdot_f16_mf8_fpm, aarch64_sve_fp8_fdot, 567347999932422),
SVEMAP1(svdot_f32_f16, aarch64_sve_fdot_x2, 4398046511111),
SVEMAP1(svdot_f32_mf8_fpm, aarch64_sve_fp8_fdot, 567347999932423),
SVEMAP1(svdot_lane_f16_mf8_fpm, aarch64_sve_fp8_fdot_lane, 567347999932422),
SVEMAP1(svdot_lane_f32_f16, aarch64_sve_fdot_lane_x2, 4398046511111),
SVEMAP1(svdot_lane_f32_mf8_fpm, aarch64_sve_fp8_fdot_lane, 567347999932423),
SVEMAP1(svdot_lane_s32, aarch64_sve_sdot_lane, 4398046511107),
SVEMAP1(svdot_lane_s32_s16, aarch64_sve_sdot_lane_x2, 4398046511107),
SVEMAP1(svdot_lane_s64, aarch64_sve_sdot_lane, 4398046511108),
SVEMAP1(svdot_lane_u32, aarch64_sve_udot_lane, 4398046511107),
SVEMAP1(svdot_lane_u32_u16, aarch64_sve_udot_lane_x2, 4398046511107),
SVEMAP1(svdot_lane_u64, aarch64_sve_udot_lane, 4398046511108),
SVEMAP1(svdot_n_f16_mf8_fpm, aarch64_sve_fp8_fdot, 567347999935494),
SVEMAP1(svdot_n_f32_mf8_fpm, aarch64_sve_fp8_fdot, 567347999935495),
SVEMAP1(svdot_n_s32, aarch64_sve_sdot, 4398046514179),
SVEMAP1(svdot_n_s64, aarch64_sve_sdot, 4398046514180),
SVEMAP1(svdot_n_u32, aarch64_sve_udot, 4398046514179),
SVEMAP1(svdot_n_u64, aarch64_sve_udot, 4398046514180),
SVEMAP1(svdot_s32, aarch64_sve_sdot, 4398046511107),
SVEMAP1(svdot_s32_s16, aarch64_sve_sdot_x2, 4398046511107),
SVEMAP1(svdot_s64, aarch64_sve_sdot, 4398046511108),
SVEMAP1(svdot_u32, aarch64_sve_udot, 4398046511107),
SVEMAP1(svdot_u32_u16, aarch64_sve_udot_x2, 4398046511107),
SVEMAP1(svdot_u64, aarch64_sve_udot, 4398046511108),
SVEMAP1(svdup_lane_bf16, aarch64_sve_tbl, 4398046513165),
SVEMAP1(svdup_lane_f16, aarch64_sve_tbl, 4398046513158),
SVEMAP1(svdup_lane_f32, aarch64_sve_tbl, 4398046513159),
SVEMAP1(svdup_lane_f64, aarch64_sve_tbl, 4398046513160),
SVEMAP1(svdup_lane_s16, aarch64_sve_tbl, 4398046513154),
SVEMAP1(svdup_lane_s32, aarch64_sve_tbl, 4398046513155),
SVEMAP1(svdup_lane_s64, aarch64_sve_tbl, 4398046513156),
SVEMAP1(svdup_lane_s8, aarch64_sve_tbl, 4398046513153),
SVEMAP1(svdup_lane_u16, aarch64_sve_tbl, 4398046513154),
SVEMAP1(svdup_lane_u32, aarch64_sve_tbl, 4398046513155),
SVEMAP1(svdup_lane_u64, aarch64_sve_tbl, 4398046513156),
SVEMAP1(svdup_lane_u8, aarch64_sve_tbl, 4398046513153),
SVEMAP1(svdup_laneq_bf16, aarch64_sve_dup_laneq, 4398046511117),
SVEMAP1(svdup_laneq_f16, aarch64_sve_dup_laneq, 4398046511110),
SVEMAP1(svdup_laneq_f32, aarch64_sve_dup_laneq, 4398046511111),
SVEMAP1(svdup_laneq_f64, aarch64_sve_dup_laneq, 4398046511112),
SVEMAP1(svdup_laneq_mf8, aarch64_sve_dup_laneq, 4398046511118),
SVEMAP1(svdup_laneq_s16, aarch64_sve_dup_laneq, 4398046511106),
SVEMAP1(svdup_laneq_s32, aarch64_sve_dup_laneq, 4398046511107),
SVEMAP1(svdup_laneq_s64, aarch64_sve_dup_laneq, 4398046511108),
SVEMAP1(svdup_laneq_s8, aarch64_sve_dup_laneq, 4398046511105),
SVEMAP1(svdup_laneq_u16, aarch64_sve_dup_laneq, 4398046511106),
SVEMAP1(svdup_laneq_u32, aarch64_sve_dup_laneq, 4398046511107),
SVEMAP1(svdup_laneq_u64, aarch64_sve_dup_laneq, 4398046511108),
SVEMAP1(svdup_laneq_u8, aarch64_sve_dup_laneq, 4398046511105),
SVEMAP2(svdup_n_b16, 4398046511114),
SVEMAP2(svdup_n_b32, 4398046511115),
SVEMAP2(svdup_n_b64, 4398046511116),
SVEMAP2(svdup_n_b8, 4398046511113),
SVEMAP1(svdup_n_bf16, aarch64_sve_dup_x, 4398046511117),
SVEMAP1(svdup_n_bf16_m, aarch64_sve_dup, 4398046511373),
SVEMAP1(svdup_n_bf16_x, aarch64_sve_dup, 4398046511629),
SVEMAP1(svdup_n_bf16_z, aarch64_sve_dup, 4398046511757),
SVEMAP1(svdup_n_f16, aarch64_sve_dup_x, 4398046511110),
SVEMAP1(svdup_n_f16_m, aarch64_sve_dup, 4398046511366),
SVEMAP1(svdup_n_f16_x, aarch64_sve_dup, 4398046511622),
SVEMAP1(svdup_n_f16_z, aarch64_sve_dup, 4398046511750),
SVEMAP1(svdup_n_f32, aarch64_sve_dup_x, 4398046511111),
SVEMAP1(svdup_n_f32_m, aarch64_sve_dup, 4398046511367),
SVEMAP1(svdup_n_f32_x, aarch64_sve_dup, 4398046511623),
SVEMAP1(svdup_n_f32_z, aarch64_sve_dup, 4398046511751),
SVEMAP1(svdup_n_f64, aarch64_sve_dup_x, 4398046511112),
SVEMAP1(svdup_n_f64_m, aarch64_sve_dup, 4398046511368),
SVEMAP1(svdup_n_f64_x, aarch64_sve_dup, 4398046511624),
SVEMAP1(svdup_n_f64_z, aarch64_sve_dup, 4398046511752),
SVEMAP1(svdup_n_s16, aarch64_sve_dup_x, 4398046511106),
SVEMAP1(svdup_n_s16_m, aarch64_sve_dup, 4398046511362),
SVEMAP1(svdup_n_s16_x, aarch64_sve_dup, 4398046511618),
SVEMAP1(svdup_n_s16_z, aarch64_sve_dup, 4398046511746),
SVEMAP1(svdup_n_s32, aarch64_sve_dup_x, 4398046511107),
SVEMAP1(svdup_n_s32_m, aarch64_sve_dup, 4398046511363),
SVEMAP1(svdup_n_s32_x, aarch64_sve_dup, 4398046511619),
SVEMAP1(svdup_n_s32_z, aarch64_sve_dup, 4398046511747),
SVEMAP1(svdup_n_s64, aarch64_sve_dup_x, 4398046511108),
SVEMAP1(svdup_n_s64_m, aarch64_sve_dup, 4398046511364),
SVEMAP1(svdup_n_s64_x, aarch64_sve_dup, 4398046511620),
SVEMAP1(svdup_n_s64_z, aarch64_sve_dup, 4398046511748),
SVEMAP1(svdup_n_s8, aarch64_sve_dup_x, 4398046511105),
SVEMAP1(svdup_n_s8_m, aarch64_sve_dup, 4398046511361),
SVEMAP1(svdup_n_s8_x, aarch64_sve_dup, 4398046511617),
SVEMAP1(svdup_n_s8_z, aarch64_sve_dup, 4398046511745),
SVEMAP1(svdup_n_u16, aarch64_sve_dup_x, 4398046511106),
SVEMAP1(svdup_n_u16_m, aarch64_sve_dup, 4398046511362),
SVEMAP1(svdup_n_u16_x, aarch64_sve_dup, 4398046511618),
SVEMAP1(svdup_n_u16_z, aarch64_sve_dup, 4398046511746),
SVEMAP1(svdup_n_u32, aarch64_sve_dup_x, 4398046511107),
SVEMAP1(svdup_n_u32_m, aarch64_sve_dup, 4398046511363),
SVEMAP1(svdup_n_u32_x, aarch64_sve_dup, 4398046511619),
SVEMAP1(svdup_n_u32_z, aarch64_sve_dup, 4398046511747),
SVEMAP1(svdup_n_u64, aarch64_sve_dup_x, 4398046511108),
SVEMAP1(svdup_n_u64_m, aarch64_sve_dup, 4398046511364),
SVEMAP1(svdup_n_u64_x, aarch64_sve_dup, 4398046511620),
SVEMAP1(svdup_n_u64_z, aarch64_sve_dup, 4398046511748),
SVEMAP1(svdup_n_u8, aarch64_sve_dup_x, 4398046511105),
SVEMAP1(svdup_n_u8_m, aarch64_sve_dup, 4398046511361),
SVEMAP1(svdup_n_u8_x, aarch64_sve_dup, 4398046511617),
SVEMAP1(svdup_n_u8_z, aarch64_sve_dup, 4398046511745),
SVEMAP1(svdupq_lane_bf16, aarch64_sve_dupq_lane, 4398046511117),
SVEMAP1(svdupq_lane_f16, aarch64_sve_dupq_lane, 4398046511110),
SVEMAP1(svdupq_lane_f32, aarch64_sve_dupq_lane, 4398046511111),
SVEMAP1(svdupq_lane_f64, aarch64_sve_dupq_lane, 4398046511112),
SVEMAP1(svdupq_lane_s16, aarch64_sve_dupq_lane, 4398046511106),
SVEMAP1(svdupq_lane_s32, aarch64_sve_dupq_lane, 4398046511107),
SVEMAP1(svdupq_lane_s64, aarch64_sve_dupq_lane, 4398046511108),
SVEMAP1(svdupq_lane_s8, aarch64_sve_dupq_lane, 4398046511105),
SVEMAP1(svdupq_lane_u16, aarch64_sve_dupq_lane, 4398046511106),
SVEMAP1(svdupq_lane_u32, aarch64_sve_dupq_lane, 4398046511107),
SVEMAP1(svdupq_lane_u64, aarch64_sve_dupq_lane, 4398046511108),
SVEMAP1(svdupq_lane_u8, aarch64_sve_dupq_lane, 4398046511105),
SVEMAP2(svdupq_n_b16, 4398046511114),
SVEMAP2(svdupq_n_b32, 4398046511115),
SVEMAP2(svdupq_n_b64, 4398046511116),
SVEMAP2(svdupq_n_b8, 4398046511113),
SVEMAP2(svdupq_n_bf16, 4398046511117),
SVEMAP2(svdupq_n_f16, 4398046511110),
SVEMAP2(svdupq_n_f32, 4398046511111),
SVEMAP2(svdupq_n_f64, 4398046511112),
SVEMAP2(svdupq_n_s16, 4398046511106),
SVEMAP2(svdupq_n_s32, 4398046511107),
SVEMAP2(svdupq_n_s64, 4398046511108),
SVEMAP2(svdupq_n_s8, 4398046511105),
SVEMAP2(svdupq_n_u16, 4398046511106),
SVEMAP2(svdupq_n_u32, 4398046511107),
SVEMAP2(svdupq_n_u64, 4398046511108),
SVEMAP2(svdupq_n_u8, 4398046511105),
SVEMAP1(sveor3_n_s16, aarch64_sve_eor3, 4398046514178),
SVEMAP1(sveor3_n_s32, aarch64_sve_eor3, 4398046514179),
SVEMAP1(sveor3_n_s64, aarch64_sve_eor3, 4398046514180),
SVEMAP1(sveor3_n_s8, aarch64_sve_eor3, 4398046514177),
SVEMAP1(sveor3_n_u16, aarch64_sve_eor3, 4398046514178),
SVEMAP1(sveor3_n_u32, aarch64_sve_eor3, 4398046514179),
SVEMAP1(sveor3_n_u64, aarch64_sve_eor3, 4398046514180),
SVEMAP1(sveor3_n_u8, aarch64_sve_eor3, 4398046514177),
SVEMAP1(sveor3_s16, aarch64_sve_eor3, 4398046511106),
SVEMAP1(sveor3_s32, aarch64_sve_eor3, 4398046511107),
SVEMAP1(sveor3_s64, aarch64_sve_eor3, 4398046511108),
SVEMAP1(sveor3_s8, aarch64_sve_eor3, 4398046511105),
SVEMAP1(sveor3_u16, aarch64_sve_eor3, 4398046511106),
SVEMAP1(sveor3_u32, aarch64_sve_eor3, 4398046511107),
SVEMAP1(sveor3_u64, aarch64_sve_eor3, 4398046511108),
SVEMAP1(sveor3_u8, aarch64_sve_eor3, 4398046511105),
SVEMAP1(sveor_b_z, aarch64_sve_eor_z, 4398046511113),
SVEMAP1(sveor_n_s16_m, aarch64_sve_eor, 4398046514434),
SVEMAP1(sveor_n_s16_x, aarch64_sve_eor_u, 4398046514306),
SVEMAP1(sveor_n_s16_z, aarch64_sve_eor, 4398046514562),
SVEMAP1(sveor_n_s32_m, aarch64_sve_eor, 4398046514435),
SVEMAP1(sveor_n_s32_x, aarch64_sve_eor_u, 4398046514307),
SVEMAP1(sveor_n_s32_z, aarch64_sve_eor, 4398046514563),
SVEMAP1(sveor_n_s64_m, aarch64_sve_eor, 4398046514436),
SVEMAP1(sveor_n_s64_x, aarch64_sve_eor_u, 4398046514308),
SVEMAP1(sveor_n_s64_z, aarch64_sve_eor, 4398046514564),
SVEMAP1(sveor_n_s8_m, aarch64_sve_eor, 4398046514433),
SVEMAP1(sveor_n_s8_x, aarch64_sve_eor_u, 4398046514305),
SVEMAP1(sveor_n_s8_z, aarch64_sve_eor, 4398046514561),
SVEMAP1(sveor_n_u16_m, aarch64_sve_eor, 4398046514434),
SVEMAP1(sveor_n_u16_x, aarch64_sve_eor_u, 4398046514306),
SVEMAP1(sveor_n_u16_z, aarch64_sve_eor, 4398046514562),
SVEMAP1(sveor_n_u32_m, aarch64_sve_eor, 4398046514435),
SVEMAP1(sveor_n_u32_x, aarch64_sve_eor_u, 4398046514307),
SVEMAP1(sveor_n_u32_z, aarch64_sve_eor, 4398046514563),
SVEMAP1(sveor_n_u64_m, aarch64_sve_eor, 4398046514436),
SVEMAP1(sveor_n_u64_x, aarch64_sve_eor_u, 4398046514308),
SVEMAP1(sveor_n_u64_z, aarch64_sve_eor, 4398046514564),
SVEMAP1(sveor_n_u8_m, aarch64_sve_eor, 4398046514433),
SVEMAP1(sveor_n_u8_x, aarch64_sve_eor_u, 4398046514305),
SVEMAP1(sveor_n_u8_z, aarch64_sve_eor, 4398046514561),
SVEMAP1(sveor_s16_m, aarch64_sve_eor, 4398046511362),
SVEMAP1(sveor_s16_x, aarch64_sve_eor_u, 4398046511234),
SVEMAP1(sveor_s16_z, aarch64_sve_eor, 4398046511490),
SVEMAP1(sveor_s32_m, aarch64_sve_eor, 4398046511363),
SVEMAP1(sveor_s32_x, aarch64_sve_eor_u, 4398046511235),
SVEMAP1(sveor_s32_z, aarch64_sve_eor, 4398046511491),
SVEMAP1(sveor_s64_m, aarch64_sve_eor, 4398046511364),
SVEMAP1(sveor_s64_x, aarch64_sve_eor_u, 4398046511236),
SVEMAP1(sveor_s64_z, aarch64_sve_eor, 4398046511492),
SVEMAP1(sveor_s8_m, aarch64_sve_eor, 4398046511361),
SVEMAP1(sveor_s8_x, aarch64_sve_eor_u, 4398046511233),
SVEMAP1(sveor_s8_z, aarch64_sve_eor, 4398046511489),
SVEMAP1(sveor_u16_m, aarch64_sve_eor, 4398046511362),
SVEMAP1(sveor_u16_x, aarch64_sve_eor_u, 4398046511234),
SVEMAP1(sveor_u16_z, aarch64_sve_eor, 4398046511490),
SVEMAP1(sveor_u32_m, aarch64_sve_eor, 4398046511363),
SVEMAP1(sveor_u32_x, aarch64_sve_eor_u, 4398046511235),
SVEMAP1(sveor_u32_z, aarch64_sve_eor, 4398046511491),
SVEMAP1(sveor_u64_m, aarch64_sve_eor, 4398046511364),
SVEMAP1(sveor_u64_x, aarch64_sve_eor_u, 4398046511236),
SVEMAP1(sveor_u64_z, aarch64_sve_eor, 4398046511492),
SVEMAP1(sveor_u8_m, aarch64_sve_eor, 4398046511361),
SVEMAP1(sveor_u8_x, aarch64_sve_eor_u, 4398046511233),
SVEMAP1(sveor_u8_z, aarch64_sve_eor, 4398046511489),
SVEMAP1(sveorbt_n_s16, aarch64_sve_eorbt, 4398046514178),
SVEMAP1(sveorbt_n_s32, aarch64_sve_eorbt, 4398046514179),
SVEMAP1(sveorbt_n_s64, aarch64_sve_eorbt, 4398046514180),
SVEMAP1(sveorbt_n_s8, aarch64_sve_eorbt, 4398046514177),
SVEMAP1(sveorbt_n_u16, aarch64_sve_eorbt, 4398046514178),
SVEMAP1(sveorbt_n_u32, aarch64_sve_eorbt, 4398046514179),
SVEMAP1(sveorbt_n_u64, aarch64_sve_eorbt, 4398046514180),
SVEMAP1(sveorbt_n_u8, aarch64_sve_eorbt, 4398046514177),
SVEMAP1(sveorbt_s16, aarch64_sve_eorbt, 4398046511106),
SVEMAP1(sveorbt_s32, aarch64_sve_eorbt, 4398046511107),
SVEMAP1(sveorbt_s64, aarch64_sve_eorbt, 4398046511108),
SVEMAP1(sveorbt_s8, aarch64_sve_eorbt, 4398046511105),
SVEMAP1(sveorbt_u16, aarch64_sve_eorbt, 4398046511106),
SVEMAP1(sveorbt_u32, aarch64_sve_eorbt, 4398046511107),
SVEMAP1(sveorbt_u64, aarch64_sve_eorbt, 4398046511108),
SVEMAP1(sveorbt_u8, aarch64_sve_eorbt, 4398046511105),
SVEMAP1(sveorqv_s16, aarch64_sve_eorqv, 6597069766658),
SVEMAP1(sveorqv_s32, aarch64_sve_eorqv, 6597069766659),
SVEMAP1(sveorqv_s64, aarch64_sve_eorqv, 6597069766660),
SVEMAP1(sveorqv_s8, aarch64_sve_eorqv, 6597069766657),
SVEMAP1(sveorqv_u16, aarch64_sve_eorqv, 6597069766658),
SVEMAP1(sveorqv_u32, aarch64_sve_eorqv, 6597069766659),
SVEMAP1(sveorqv_u64, aarch64_sve_eorqv, 6597069766660),
SVEMAP1(sveorqv_u8, aarch64_sve_eorqv, 6597069766657),
SVEMAP1(sveortb_n_s16, aarch64_sve_eortb, 4398046514178),
SVEMAP1(sveortb_n_s32, aarch64_sve_eortb, 4398046514179),
SVEMAP1(sveortb_n_s64, aarch64_sve_eortb, 4398046514180),
SVEMAP1(sveortb_n_s8, aarch64_sve_eortb, 4398046514177),
SVEMAP1(sveortb_n_u16, aarch64_sve_eortb, 4398046514178),
SVEMAP1(sveortb_n_u32, aarch64_sve_eortb, 4398046514179),
SVEMAP1(sveortb_n_u64, aarch64_sve_eortb, 4398046514180),
SVEMAP1(sveortb_n_u8, aarch64_sve_eortb, 4398046514177),
SVEMAP1(sveortb_s16, aarch64_sve_eortb, 4398046511106),
SVEMAP1(sveortb_s32, aarch64_sve_eortb, 4398046511107),
SVEMAP1(sveortb_s64, aarch64_sve_eortb, 4398046511108),
SVEMAP1(sveortb_s8, aarch64_sve_eortb, 4398046511105),
SVEMAP1(sveortb_u16, aarch64_sve_eortb, 4398046511106),
SVEMAP1(sveortb_u32, aarch64_sve_eortb, 4398046511107),
SVEMAP1(sveortb_u64, aarch64_sve_eortb, 4398046511108),
SVEMAP1(sveortb_u8, aarch64_sve_eortb, 4398046511105),
SVEMAP1(sveorv_s16, aarch64_sve_eorv, 4398046511106),
SVEMAP1(sveorv_s32, aarch64_sve_eorv, 4398046511107),
SVEMAP1(sveorv_s64, aarch64_sve_eorv, 4398046511108),
SVEMAP1(sveorv_s8, aarch64_sve_eorv, 4398046511105),
SVEMAP1(sveorv_u16, aarch64_sve_eorv, 4398046511106),
SVEMAP1(sveorv_u32, aarch64_sve_eorv, 4398046511107),
SVEMAP1(sveorv_u64, aarch64_sve_eorv, 4398046511108),
SVEMAP1(sveorv_u8, aarch64_sve_eorv, 4398046511105),
SVEMAP1(svexpa_f16, aarch64_sve_fexpa_x, 4398046511110),
SVEMAP1(svexpa_f32, aarch64_sve_fexpa_x, 4398046511111),
SVEMAP1(svexpa_f64, aarch64_sve_fexpa_x, 4398046511112),
SVEMAP1(svext_bf16, aarch64_sve_ext, 4398046511117),
SVEMAP1(svext_f16, aarch64_sve_ext, 4398046511110),
SVEMAP1(svext_f32, aarch64_sve_ext, 4398046511111),
SVEMAP1(svext_f64, aarch64_sve_ext, 4398046511112),
SVEMAP1(svext_s16, aarch64_sve_ext, 4398046511106),
SVEMAP1(svext_s32, aarch64_sve_ext, 4398046511107),
SVEMAP1(svext_s64, aarch64_sve_ext, 4398046511108),
SVEMAP1(svext_s8, aarch64_sve_ext, 4398046511105),
SVEMAP1(svext_u16, aarch64_sve_ext, 4398046511106),
SVEMAP1(svext_u32, aarch64_sve_ext, 4398046511107),
SVEMAP1(svext_u64, aarch64_sve_ext, 4398046511108),
SVEMAP1(svext_u8, aarch64_sve_ext, 4398046511105),
SVEMAP1(svextb_s16_m, aarch64_sve_sxtb, 4398046511362),
SVEMAP1(svextb_s16_x, aarch64_sve_sxtb, 4398046511618),
SVEMAP1(svextb_s16_z, aarch64_sve_sxtb, 4398046511746),
SVEMAP1(svextb_s32_m, aarch64_sve_sxtb, 4398046511363),
SVEMAP1(svextb_s32_x, aarch64_sve_sxtb, 4398046511619),
SVEMAP1(svextb_s32_z, aarch64_sve_sxtb, 4398046511747),
SVEMAP1(svextb_s64_m, aarch64_sve_sxtb, 4398046511364),
SVEMAP1(svextb_s64_x, aarch64_sve_sxtb, 4398046511620),
SVEMAP1(svextb_s64_z, aarch64_sve_sxtb, 4398046511748),
SVEMAP1(svextb_u16_m, aarch64_sve_uxtb, 4398046511362),
SVEMAP1(svextb_u16_x, aarch64_sve_uxtb, 4398046511618),
SVEMAP1(svextb_u16_z, aarch64_sve_uxtb, 4398046511746),
SVEMAP1(svextb_u32_m, aarch64_sve_uxtb, 4398046511363),
SVEMAP1(svextb_u32_x, aarch64_sve_uxtb, 4398046511619),
SVEMAP1(svextb_u32_z, aarch64_sve_uxtb, 4398046511747),
SVEMAP1(svextb_u64_m, aarch64_sve_uxtb, 4398046511364),
SVEMAP1(svextb_u64_x, aarch64_sve_uxtb, 4398046511620),
SVEMAP1(svextb_u64_z, aarch64_sve_uxtb, 4398046511748),
SVEMAP1(svexth_s32_m, aarch64_sve_sxth, 4398046511363),
SVEMAP1(svexth_s32_x, aarch64_sve_sxth, 4398046511619),
SVEMAP1(svexth_s32_z, aarch64_sve_sxth, 4398046511747),
SVEMAP1(svexth_s64_m, aarch64_sve_sxth, 4398046511364),
SVEMAP1(svexth_s64_x, aarch64_sve_sxth, 4398046511620),
SVEMAP1(svexth_s64_z, aarch64_sve_sxth, 4398046511748),
SVEMAP1(svexth_u32_m, aarch64_sve_uxth, 4398046511363),
SVEMAP1(svexth_u32_x, aarch64_sve_uxth, 4398046511619),
SVEMAP1(svexth_u32_z, aarch64_sve_uxth, 4398046511747),
SVEMAP1(svexth_u64_m, aarch64_sve_uxth, 4398046511364),
SVEMAP1(svexth_u64_x, aarch64_sve_uxth, 4398046511620),
SVEMAP1(svexth_u64_z, aarch64_sve_uxth, 4398046511748),
SVEMAP1(svextq_bf16, aarch64_sve_extq, 4398046511117),
SVEMAP1(svextq_f16, aarch64_sve_extq, 4398046511110),
SVEMAP1(svextq_f32, aarch64_sve_extq, 4398046511111),
SVEMAP1(svextq_f64, aarch64_sve_extq, 4398046511112),
SVEMAP1(svextq_mf8, aarch64_sve_extq, 4398046511118),
SVEMAP1(svextq_s16, aarch64_sve_extq, 4398046511106),
SVEMAP1(svextq_s32, aarch64_sve_extq, 4398046511107),
SVEMAP1(svextq_s64, aarch64_sve_extq, 4398046511108),
SVEMAP1(svextq_s8, aarch64_sve_extq, 4398046511105),
SVEMAP1(svextq_u16, aarch64_sve_extq, 4398046511106),
SVEMAP1(svextq_u32, aarch64_sve_extq, 4398046511107),
SVEMAP1(svextq_u64, aarch64_sve_extq, 4398046511108),
SVEMAP1(svextq_u8, aarch64_sve_extq, 4398046511105),
SVEMAP1(svextw_s64_m, aarch64_sve_sxtw, 4398046511364),
SVEMAP1(svextw_s64_x, aarch64_sve_sxtw, 4398046511620),
SVEMAP1(svextw_s64_z, aarch64_sve_sxtw, 4398046511748),
SVEMAP1(svextw_u64_m, aarch64_sve_uxtw, 4398046511364),
SVEMAP1(svextw_u64_x, aarch64_sve_uxtw, 4398046511620),
SVEMAP1(svextw_u64_z, aarch64_sve_uxtw, 4398046511748),
SVEMAP2(svget2_b, 4406636445705),
SVEMAP2(svget2_bf16, 4406636445709),
SVEMAP2(svget2_f16, 4406636445702),
SVEMAP2(svget2_f32, 4406636445703),
SVEMAP2(svget2_f64, 4406636445704),
SVEMAP2(svget2_mf8, 4406636445710),
SVEMAP2(svget2_s16, 4406636445698),
SVEMAP2(svget2_s32, 4406636445699),
SVEMAP2(svget2_s64, 4406636445700),
SVEMAP2(svget2_s8, 4406636445697),
SVEMAP2(svget2_u16, 4406636445698),
SVEMAP2(svget2_u32, 4406636445699),
SVEMAP2(svget2_u64, 4406636445700),
SVEMAP2(svget2_u8, 4406636445697),
SVEMAP2(svget3_bf16, 4406636445709),
SVEMAP2(svget3_f16, 4406636445702),
SVEMAP2(svget3_f32, 4406636445703),
SVEMAP2(svget3_f64, 4406636445704),
SVEMAP2(svget3_mf8, 4406636445710),
SVEMAP2(svget3_s16, 4406636445698),
SVEMAP2(svget3_s32, 4406636445699),
SVEMAP2(svget3_s64, 4406636445700),
SVEMAP2(svget3_s8, 4406636445697),
SVEMAP2(svget3_u16, 4406636445698),
SVEMAP2(svget3_u32, 4406636445699),
SVEMAP2(svget3_u64, 4406636445700),
SVEMAP2(svget3_u8, 4406636445697),
SVEMAP2(svget4_b, 4406636445705),
SVEMAP2(svget4_bf16, 4406636445709),
SVEMAP2(svget4_f16, 4406636445702),
SVEMAP2(svget4_f32, 4406636445703),
SVEMAP2(svget4_f64, 4406636445704),
SVEMAP2(svget4_mf8, 4406636445710),
SVEMAP2(svget4_s16, 4406636445698),
SVEMAP2(svget4_s32, 4406636445699),
SVEMAP2(svget4_s64, 4406636445700),
SVEMAP2(svget4_s8, 4406636445697),
SVEMAP2(svget4_u16, 4406636445698),
SVEMAP2(svget4_u32, 4406636445699),
SVEMAP2(svget4_u64, 4406636445700),
SVEMAP2(svget4_u8, 4406636445697),
SVEMAP1(svhadd_n_s16_m, aarch64_sve_shadd, 4398046514434),
SVEMAP1(svhadd_n_s16_x, aarch64_sve_shadd, 4398046514306),
SVEMAP1(svhadd_n_s16_z, aarch64_sve_shadd, 4398046514562),
SVEMAP1(svhadd_n_s32_m, aarch64_sve_shadd, 4398046514435),
SVEMAP1(svhadd_n_s32_x, aarch64_sve_shadd, 4398046514307),
SVEMAP1(svhadd_n_s32_z, aarch64_sve_shadd, 4398046514563),
SVEMAP1(svhadd_n_s64_m, aarch64_sve_shadd, 4398046514436),
SVEMAP1(svhadd_n_s64_x, aarch64_sve_shadd, 4398046514308),
SVEMAP1(svhadd_n_s64_z, aarch64_sve_shadd, 4398046514564),
SVEMAP1(svhadd_n_s8_m, aarch64_sve_shadd, 4398046514433),
SVEMAP1(svhadd_n_s8_x, aarch64_sve_shadd, 4398046514305),
SVEMAP1(svhadd_n_s8_z, aarch64_sve_shadd, 4398046514561),
SVEMAP1(svhadd_n_u16_m, aarch64_sve_uhadd, 4398046514434),
SVEMAP1(svhadd_n_u16_x, aarch64_sve_uhadd, 4398046514306),
SVEMAP1(svhadd_n_u16_z, aarch64_sve_uhadd, 4398046514562),
SVEMAP1(svhadd_n_u32_m, aarch64_sve_uhadd, 4398046514435),
SVEMAP1(svhadd_n_u32_x, aarch64_sve_uhadd, 4398046514307),
SVEMAP1(svhadd_n_u32_z, aarch64_sve_uhadd, 4398046514563),
SVEMAP1(svhadd_n_u64_m, aarch64_sve_uhadd, 4398046514436),
SVEMAP1(svhadd_n_u64_x, aarch64_sve_uhadd, 4398046514308),
SVEMAP1(svhadd_n_u64_z, aarch64_sve_uhadd, 4398046514564),
SVEMAP1(svhadd_n_u8_m, aarch64_sve_uhadd, 4398046514433),
SVEMAP1(svhadd_n_u8_x, aarch64_sve_uhadd, 4398046514305),
SVEMAP1(svhadd_n_u8_z, aarch64_sve_uhadd, 4398046514561),
SVEMAP1(svhadd_s16_m, aarch64_sve_shadd, 4398046511362),
SVEMAP1(svhadd_s16_x, aarch64_sve_shadd, 4398046511234),
SVEMAP1(svhadd_s16_z, aarch64_sve_shadd, 4398046511490),
SVEMAP1(svhadd_s32_m, aarch64_sve_shadd, 4398046511363),
SVEMAP1(svhadd_s32_x, aarch64_sve_shadd, 4398046511235),
SVEMAP1(svhadd_s32_z, aarch64_sve_shadd, 4398046511491),
SVEMAP1(svhadd_s64_m, aarch64_sve_shadd, 4398046511364),
SVEMAP1(svhadd_s64_x, aarch64_sve_shadd, 4398046511236),
SVEMAP1(svhadd_s64_z, aarch64_sve_shadd, 4398046511492),
SVEMAP1(svhadd_s8_m, aarch64_sve_shadd, 4398046511361),
SVEMAP1(svhadd_s8_x, aarch64_sve_shadd, 4398046511233),
SVEMAP1(svhadd_s8_z, aarch64_sve_shadd, 4398046511489),
SVEMAP1(svhadd_u16_m, aarch64_sve_uhadd, 4398046511362),
SVEMAP1(svhadd_u16_x, aarch64_sve_uhadd, 4398046511234),
SVEMAP1(svhadd_u16_z, aarch64_sve_uhadd, 4398046511490),
SVEMAP1(svhadd_u32_m, aarch64_sve_uhadd, 4398046511363),
SVEMAP1(svhadd_u32_x, aarch64_sve_uhadd, 4398046511235),
SVEMAP1(svhadd_u32_z, aarch64_sve_uhadd, 4398046511491),
SVEMAP1(svhadd_u64_m, aarch64_sve_uhadd, 4398046511364),
SVEMAP1(svhadd_u64_x, aarch64_sve_uhadd, 4398046511236),
SVEMAP1(svhadd_u64_z, aarch64_sve_uhadd, 4398046511492),
SVEMAP1(svhadd_u8_m, aarch64_sve_uhadd, 4398046511361),
SVEMAP1(svhadd_u8_x, aarch64_sve_uhadd, 4398046511233),
SVEMAP1(svhadd_u8_z, aarch64_sve_uhadd, 4398046511489),
SVEMAP1(svhistcnt_s32_z, aarch64_sve_histcnt, 3),
SVEMAP1(svhistcnt_s64_z, aarch64_sve_histcnt, 4),
SVEMAP1(svhistcnt_u32_z, aarch64_sve_histcnt, 3),
SVEMAP1(svhistcnt_u64_z, aarch64_sve_histcnt, 4),
SVEMAP1(svhistseg_s8, aarch64_sve_histseg, 1),
SVEMAP1(svhistseg_u8, aarch64_sve_histseg, 1),
SVEMAP1(svhsub_n_s16_m, aarch64_sve_shsub, 4398046514434),
SVEMAP1(svhsub_n_s16_x, aarch64_sve_shsub, 4398046514306),
SVEMAP1(svhsub_n_s16_z, aarch64_sve_shsub, 4398046514562),
SVEMAP1(svhsub_n_s32_m, aarch64_sve_shsub, 4398046514435),
SVEMAP1(svhsub_n_s32_x, aarch64_sve_shsub, 4398046514307),
SVEMAP1(svhsub_n_s32_z, aarch64_sve_shsub, 4398046514563),
SVEMAP1(svhsub_n_s64_m, aarch64_sve_shsub, 4398046514436),
SVEMAP1(svhsub_n_s64_x, aarch64_sve_shsub, 4398046514308),
SVEMAP1(svhsub_n_s64_z, aarch64_sve_shsub, 4398046514564),
SVEMAP1(svhsub_n_s8_m, aarch64_sve_shsub, 4398046514433),
SVEMAP1(svhsub_n_s8_x, aarch64_sve_shsub, 4398046514305),
SVEMAP1(svhsub_n_s8_z, aarch64_sve_shsub, 4398046514561),
SVEMAP1(svhsub_n_u16_m, aarch64_sve_uhsub, 4398046514434),
SVEMAP1(svhsub_n_u16_x, aarch64_sve_uhsub, 4398046514306),
SVEMAP1(svhsub_n_u16_z, aarch64_sve_uhsub, 4398046514562),
SVEMAP1(svhsub_n_u32_m, aarch64_sve_uhsub, 4398046514435),
SVEMAP1(svhsub_n_u32_x, aarch64_sve_uhsub, 4398046514307),
SVEMAP1(svhsub_n_u32_z, aarch64_sve_uhsub, 4398046514563),
SVEMAP1(svhsub_n_u64_m, aarch64_sve_uhsub, 4398046514436),
SVEMAP1(svhsub_n_u64_x, aarch64_sve_uhsub, 4398046514308),
SVEMAP1(svhsub_n_u64_z, aarch64_sve_uhsub, 4398046514564),
SVEMAP1(svhsub_n_u8_m, aarch64_sve_uhsub, 4398046514433),
SVEMAP1(svhsub_n_u8_x, aarch64_sve_uhsub, 4398046514305),
SVEMAP1(svhsub_n_u8_z, aarch64_sve_uhsub, 4398046514561),
SVEMAP1(svhsub_s16_m, aarch64_sve_shsub, 4398046511362),
SVEMAP1(svhsub_s16_x, aarch64_sve_shsub, 4398046511234),
SVEMAP1(svhsub_s16_z, aarch64_sve_shsub, 4398046511490),
SVEMAP1(svhsub_s32_m, aarch64_sve_shsub, 4398046511363),
SVEMAP1(svhsub_s32_x, aarch64_sve_shsub, 4398046511235),
SVEMAP1(svhsub_s32_z, aarch64_sve_shsub, 4398046511491),
SVEMAP1(svhsub_s64_m, aarch64_sve_shsub, 4398046511364),
SVEMAP1(svhsub_s64_x, aarch64_sve_shsub, 4398046511236),
SVEMAP1(svhsub_s64_z, aarch64_sve_shsub, 4398046511492),
SVEMAP1(svhsub_s8_m, aarch64_sve_shsub, 4398046511361),
SVEMAP1(svhsub_s8_x, aarch64_sve_shsub, 4398046511233),
SVEMAP1(svhsub_s8_z, aarch64_sve_shsub, 4398046511489),
SVEMAP1(svhsub_u16_m, aarch64_sve_uhsub, 4398046511362),
SVEMAP1(svhsub_u16_x, aarch64_sve_uhsub, 4398046511234),
SVEMAP1(svhsub_u16_z, aarch64_sve_uhsub, 4398046511490),
SVEMAP1(svhsub_u32_m, aarch64_sve_uhsub, 4398046511363),
SVEMAP1(svhsub_u32_x, aarch64_sve_uhsub, 4398046511235),
SVEMAP1(svhsub_u32_z, aarch64_sve_uhsub, 4398046511491),
SVEMAP1(svhsub_u64_m, aarch64_sve_uhsub, 4398046511364),
SVEMAP1(svhsub_u64_x, aarch64_sve_uhsub, 4398046511236),
SVEMAP1(svhsub_u64_z, aarch64_sve_uhsub, 4398046511492),
SVEMAP1(svhsub_u8_m, aarch64_sve_uhsub, 4398046511361),
SVEMAP1(svhsub_u8_x, aarch64_sve_uhsub, 4398046511233),
SVEMAP1(svhsub_u8_z, aarch64_sve_uhsub, 4398046511489),
SVEMAP1(svhsubr_n_s16_m, aarch64_sve_shsubr, 4398046514434),
SVEMAP1(svhsubr_n_s16_x, aarch64_sve_shsubr, 4398046514306),
SVEMAP1(svhsubr_n_s16_z, aarch64_sve_shsubr, 4398046514562),
SVEMAP1(svhsubr_n_s32_m, aarch64_sve_shsubr, 4398046514435),
SVEMAP1(svhsubr_n_s32_x, aarch64_sve_shsubr, 4398046514307),
SVEMAP1(svhsubr_n_s32_z, aarch64_sve_shsubr, 4398046514563),
SVEMAP1(svhsubr_n_s64_m, aarch64_sve_shsubr, 4398046514436),
SVEMAP1(svhsubr_n_s64_x, aarch64_sve_shsubr, 4398046514308),
SVEMAP1(svhsubr_n_s64_z, aarch64_sve_shsubr, 4398046514564),
SVEMAP1(svhsubr_n_s8_m, aarch64_sve_shsubr, 4398046514433),
SVEMAP1(svhsubr_n_s8_x, aarch64_sve_shsubr, 4398046514305),
SVEMAP1(svhsubr_n_s8_z, aarch64_sve_shsubr, 4398046514561),
SVEMAP1(svhsubr_n_u16_m, aarch64_sve_uhsubr, 4398046514434),
SVEMAP1(svhsubr_n_u16_x, aarch64_sve_uhsubr, 4398046514306),
SVEMAP1(svhsubr_n_u16_z, aarch64_sve_uhsubr, 4398046514562),
SVEMAP1(svhsubr_n_u32_m, aarch64_sve_uhsubr, 4398046514435),
SVEMAP1(svhsubr_n_u32_x, aarch64_sve_uhsubr, 4398046514307),
SVEMAP1(svhsubr_n_u32_z, aarch64_sve_uhsubr, 4398046514563),
SVEMAP1(svhsubr_n_u64_m, aarch64_sve_uhsubr, 4398046514436),
SVEMAP1(svhsubr_n_u64_x, aarch64_sve_uhsubr, 4398046514308),
SVEMAP1(svhsubr_n_u64_z, aarch64_sve_uhsubr, 4398046514564),
SVEMAP1(svhsubr_n_u8_m, aarch64_sve_uhsubr, 4398046514433),
SVEMAP1(svhsubr_n_u8_x, aarch64_sve_uhsubr, 4398046514305),
SVEMAP1(svhsubr_n_u8_z, aarch64_sve_uhsubr, 4398046514561),
SVEMAP1(svhsubr_s16_m, aarch64_sve_shsubr, 4398046511362),
SVEMAP1(svhsubr_s16_x, aarch64_sve_shsubr, 4398046511234),
SVEMAP1(svhsubr_s16_z, aarch64_sve_shsubr, 4398046511490),
SVEMAP1(svhsubr_s32_m, aarch64_sve_shsubr, 4398046511363),
SVEMAP1(svhsubr_s32_x, aarch64_sve_shsubr, 4398046511235),
SVEMAP1(svhsubr_s32_z, aarch64_sve_shsubr, 4398046511491),
SVEMAP1(svhsubr_s64_m, aarch64_sve_shsubr, 4398046511364),
SVEMAP1(svhsubr_s64_x, aarch64_sve_shsubr, 4398046511236),
SVEMAP1(svhsubr_s64_z, aarch64_sve_shsubr, 4398046511492),
SVEMAP1(svhsubr_s8_m, aarch64_sve_shsubr, 4398046511361),
SVEMAP1(svhsubr_s8_x, aarch64_sve_shsubr, 4398046511233),
SVEMAP1(svhsubr_s8_z, aarch64_sve_shsubr, 4398046511489),
SVEMAP1(svhsubr_u16_m, aarch64_sve_uhsubr, 4398046511362),
SVEMAP1(svhsubr_u16_x, aarch64_sve_uhsubr, 4398046511234),
SVEMAP1(svhsubr_u16_z, aarch64_sve_uhsubr, 4398046511490),
SVEMAP1(svhsubr_u32_m, aarch64_sve_uhsubr, 4398046511363),
SVEMAP1(svhsubr_u32_x, aarch64_sve_uhsubr, 4398046511235),
SVEMAP1(svhsubr_u32_z, aarch64_sve_uhsubr, 4398046511491),
SVEMAP1(svhsubr_u64_m, aarch64_sve_uhsubr, 4398046511364),
SVEMAP1(svhsubr_u64_x, aarch64_sve_uhsubr, 4398046511236),
SVEMAP1(svhsubr_u64_z, aarch64_sve_uhsubr, 4398046511492),
SVEMAP1(svhsubr_u8_m, aarch64_sve_uhsubr, 4398046511361),
SVEMAP1(svhsubr_u8_x, aarch64_sve_uhsubr, 4398046511233),
SVEMAP1(svhsubr_u8_z, aarch64_sve_uhsubr, 4398046511489),
SVEMAP1(svindex_s16, aarch64_sve_index, 4398046511106),
SVEMAP1(svindex_s32, aarch64_sve_index, 4398046511107),
SVEMAP1(svindex_s64, aarch64_sve_index, 4398046511108),
SVEMAP1(svindex_s8, aarch64_sve_index, 4398046511105),
SVEMAP1(svindex_u16, aarch64_sve_index, 4398046511106),
SVEMAP1(svindex_u32, aarch64_sve_index, 4398046511107),
SVEMAP1(svindex_u64, aarch64_sve_index, 4398046511108),
SVEMAP1(svindex_u8, aarch64_sve_index, 4398046511105),
SVEMAP1(svinsr_n_bf16, aarch64_sve_insr, 4398046511117),
SVEMAP1(svinsr_n_f16, aarch64_sve_insr, 4398046511110),
SVEMAP1(svinsr_n_f32, aarch64_sve_insr, 4398046511111),
SVEMAP1(svinsr_n_f64, aarch64_sve_insr, 4398046511112),
SVEMAP1(svinsr_n_s16, aarch64_sve_insr, 4398046511106),
SVEMAP1(svinsr_n_s32, aarch64_sve_insr, 4398046511107),
SVEMAP1(svinsr_n_s64, aarch64_sve_insr, 4398046511108),
SVEMAP1(svinsr_n_s8, aarch64_sve_insr, 4398046511105),
SVEMAP1(svinsr_n_u16, aarch64_sve_insr, 4398046511106),
SVEMAP1(svinsr_n_u32, aarch64_sve_insr, 4398046511107),
SVEMAP1(svinsr_n_u64, aarch64_sve_insr, 4398046511108),
SVEMAP1(svinsr_n_u8, aarch64_sve_insr, 4398046511105),
SVEMAP1(svlasta_bf16, aarch64_sve_lasta, 4398046511117),
SVEMAP1(svlasta_f16, aarch64_sve_lasta, 4398046511110),
SVEMAP1(svlasta_f32, aarch64_sve_lasta, 4398046511111),
SVEMAP1(svlasta_f64, aarch64_sve_lasta, 4398046511112),
SVEMAP1(svlasta_s16, aarch64_sve_lasta, 4398046511106),
SVEMAP1(svlasta_s32, aarch64_sve_lasta, 4398046511107),
SVEMAP1(svlasta_s64, aarch64_sve_lasta, 4398046511108),
SVEMAP1(svlasta_s8, aarch64_sve_lasta, 4398046511105),
SVEMAP1(svlasta_u16, aarch64_sve_lasta, 4398046511106),
SVEMAP1(svlasta_u32, aarch64_sve_lasta, 4398046511107),
SVEMAP1(svlasta_u64, aarch64_sve_lasta, 4398046511108),
SVEMAP1(svlasta_u8, aarch64_sve_lasta, 4398046511105),
SVEMAP1(svlastb_bf16, aarch64_sve_lastb, 4398046511117),
SVEMAP1(svlastb_f16, aarch64_sve_lastb, 4398046511110),
SVEMAP1(svlastb_f32, aarch64_sve_lastb, 4398046511111),
SVEMAP1(svlastb_f64, aarch64_sve_lastb, 4398046511112),
SVEMAP1(svlastb_s16, aarch64_sve_lastb, 4398046511106),
SVEMAP1(svlastb_s32, aarch64_sve_lastb, 4398046511107),
SVEMAP1(svlastb_s64, aarch64_sve_lastb, 4398046511108),
SVEMAP1(svlastb_s8, aarch64_sve_lastb, 4398046511105),
SVEMAP1(svlastb_u16, aarch64_sve_lastb, 4398046511106),
SVEMAP1(svlastb_u32, aarch64_sve_lastb, 4398046511107),
SVEMAP1(svlastb_u64, aarch64_sve_lastb, 4398046511108),
SVEMAP1(svlastb_u8, aarch64_sve_lastb, 4398046511105),
SVEMAP1(svld1_bf16, aarch64_sve_ld1, 4398046519309),
SVEMAP1(svld1_bf16_x2, aarch64_sve_ld1_pn_x2, 4398046642189),
SVEMAP1(svld1_bf16_x4, aarch64_sve_ld1_pn_x4, 4398046642189),
SVEMAP1(svld1_f16, aarch64_sve_ld1, 4398046519302),
SVEMAP1(svld1_f16_x2, aarch64_sve_ld1_pn_x2, 4398046642182),
SVEMAP1(svld1_f16_x4, aarch64_sve_ld1_pn_x4, 4398046642182),
SVEMAP1(svld1_f32, aarch64_sve_ld1, 4398046519303),
SVEMAP1(svld1_f32_x2, aarch64_sve_ld1_pn_x2, 4398046642183),
SVEMAP1(svld1_f32_x4, aarch64_sve_ld1_pn_x4, 4398046642183),
SVEMAP1(svld1_f64, aarch64_sve_ld1, 4398046519304),
SVEMAP1(svld1_f64_x2, aarch64_sve_ld1_pn_x2, 4398046642184),
SVEMAP1(svld1_f64_x4, aarch64_sve_ld1_pn_x4, 4398046642184),
SVEMAP1(svld1_gather_s32index_f32, aarch64_sve_ld1_gather_sxtw_index, 32775),
SVEMAP1(svld1_gather_s32index_s32, aarch64_sve_ld1_gather_sxtw_index, 32771),
SVEMAP1(svld1_gather_s32index_u32, aarch64_sve_ld1_gather_sxtw_index, 32771),
SVEMAP1(svld1_gather_s32offset_f32, aarch64_sve_ld1_gather_sxtw, 16809991),
SVEMAP1(svld1_gather_s32offset_s32, aarch64_sve_ld1_gather_sxtw, 16809987),
SVEMAP1(svld1_gather_s32offset_u32, aarch64_sve_ld1_gather_sxtw, 16809987),
SVEMAP1(svld1_gather_s64index_f64, aarch64_sve_ld1_gather_index, 32776),
SVEMAP1(svld1_gather_s64index_s64, aarch64_sve_ld1_gather_index, 32772),
SVEMAP1(svld1_gather_s64index_u64, aarch64_sve_ld1_gather_index, 32772),
SVEMAP1(svld1_gather_s64offset_f64, aarch64_sve_ld1_gather, 16809992),
SVEMAP1(svld1_gather_s64offset_s64, aarch64_sve_ld1_gather, 16809988),
SVEMAP1(svld1_gather_s64offset_u64, aarch64_sve_ld1_gather, 16809988),
SVEMAP1(svld1_gather_u32base_f32, aarch64_sve_ld1_gather_scalar_offset, 32775),
SVEMAP1(svld1_gather_u32base_index_f32, aarch64_sve_ld1_gather_scalar_offset, 32775),
SVEMAP1(svld1_gather_u32base_index_s32, aarch64_sve_ld1_gather_scalar_offset, 32771),
SVEMAP1(svld1_gather_u32base_index_u32, aarch64_sve_ld1_gather_scalar_offset, 32771),
SVEMAP1(svld1_gather_u32base_offset_f32, aarch64_sve_ld1_gather_scalar_offset, 16809991),
SVEMAP1(svld1_gather_u32base_offset_s32, aarch64_sve_ld1_gather_scalar_offset, 16809987),
SVEMAP1(svld1_gather_u32base_offset_u32, aarch64_sve_ld1_gather_scalar_offset, 16809987),
SVEMAP1(svld1_gather_u32base_s32, aarch64_sve_ld1_gather_scalar_offset, 32771),
SVEMAP1(svld1_gather_u32base_u32, aarch64_sve_ld1_gather_scalar_offset, 32771),
SVEMAP1(svld1_gather_u32index_f32, aarch64_sve_ld1_gather_uxtw_index, 32775),
SVEMAP1(svld1_gather_u32index_s32, aarch64_sve_ld1_gather_uxtw_index, 32771),
SVEMAP1(svld1_gather_u32index_u32, aarch64_sve_ld1_gather_uxtw_index, 32771),
SVEMAP1(svld1_gather_u32offset_f32, aarch64_sve_ld1_gather_uxtw, 16809991),
SVEMAP1(svld1_gather_u32offset_s32, aarch64_sve_ld1_gather_uxtw, 16809987),
SVEMAP1(svld1_gather_u32offset_u32, aarch64_sve_ld1_gather_uxtw, 16809987),
SVEMAP1(svld1_gather_u64base_f64, aarch64_sve_ld1_gather_scalar_offset, 32776),
SVEMAP1(svld1_gather_u64base_index_f64, aarch64_sve_ld1_gather_scalar_offset, 32776),
SVEMAP1(svld1_gather_u64base_index_s64, aarch64_sve_ld1_gather_scalar_offset, 32772),
SVEMAP1(svld1_gather_u64base_index_u64, aarch64_sve_ld1_gather_scalar_offset, 32772),
SVEMAP1(svld1_gather_u64base_offset_f64, aarch64_sve_ld1_gather_scalar_offset, 16809992),
SVEMAP1(svld1_gather_u64base_offset_s64, aarch64_sve_ld1_gather_scalar_offset, 16809988),
SVEMAP1(svld1_gather_u64base_offset_u64, aarch64_sve_ld1_gather_scalar_offset, 16809988),
SVEMAP1(svld1_gather_u64base_s64, aarch64_sve_ld1_gather_scalar_offset, 32772),
SVEMAP1(svld1_gather_u64base_u64, aarch64_sve_ld1_gather_scalar_offset, 32772),
SVEMAP1(svld1_gather_u64index_f64, aarch64_sve_ld1_gather_index, 32776),
SVEMAP1(svld1_gather_u64index_s64, aarch64_sve_ld1_gather_index, 32772),
SVEMAP1(svld1_gather_u64index_u64, aarch64_sve_ld1_gather_index, 32772),
SVEMAP1(svld1_gather_u64offset_f64, aarch64_sve_ld1_gather, 16809992),
SVEMAP1(svld1_gather_u64offset_s64, aarch64_sve_ld1_gather, 16809988),
SVEMAP1(svld1_gather_u64offset_u64, aarch64_sve_ld1_gather, 16809988),
SVEMAP1(svld1_mf8, aarch64_sve_ld1, 4398046519310),
SVEMAP1(svld1_mf8_x2, aarch64_sve_ld1_pn_x2, 4398046642190),
SVEMAP1(svld1_mf8_x4, aarch64_sve_ld1_pn_x4, 4398046642190),
SVEMAP1(svld1_s16, aarch64_sve_ld1, 4398046519298),
SVEMAP1(svld1_s16_x2, aarch64_sve_ld1_pn_x2, 4398046642178),
SVEMAP1(svld1_s16_x4, aarch64_sve_ld1_pn_x4, 4398046642178),
SVEMAP1(svld1_s32, aarch64_sve_ld1, 4398046519299),
SVEMAP1(svld1_s32_x2, aarch64_sve_ld1_pn_x2, 4398046642179),
SVEMAP1(svld1_s32_x4, aarch64_sve_ld1_pn_x4, 4398046642179),
SVEMAP1(svld1_s64, aarch64_sve_ld1, 4398046519300),
SVEMAP1(svld1_s64_x2, aarch64_sve_ld1_pn_x2, 4398046642180),
SVEMAP1(svld1_s64_x4, aarch64_sve_ld1_pn_x4, 4398046642180),
SVEMAP1(svld1_s8, aarch64_sve_ld1, 4398046519297),
SVEMAP1(svld1_s8_x2, aarch64_sve_ld1_pn_x2, 4398046642177),
SVEMAP1(svld1_s8_x4, aarch64_sve_ld1_pn_x4, 4398046642177),
SVEMAP1(svld1_u16, aarch64_sve_ld1, 4398046519298),
SVEMAP1(svld1_u16_x2, aarch64_sve_ld1_pn_x2, 4398046642178),
SVEMAP1(svld1_u16_x4, aarch64_sve_ld1_pn_x4, 4398046642178),
SVEMAP1(svld1_u32, aarch64_sve_ld1, 4398046519299),
SVEMAP1(svld1_u32_x2, aarch64_sve_ld1_pn_x2, 4398046642179),
SVEMAP1(svld1_u32_x4, aarch64_sve_ld1_pn_x4, 4398046642179),
SVEMAP1(svld1_u64, aarch64_sve_ld1, 4398046519300),
SVEMAP1(svld1_u64_x2, aarch64_sve_ld1_pn_x2, 4398046642180),
SVEMAP1(svld1_u64_x4, aarch64_sve_ld1_pn_x4, 4398046642180),
SVEMAP1(svld1_u8, aarch64_sve_ld1, 4398046519297),
SVEMAP1(svld1_u8_x2, aarch64_sve_ld1_pn_x2, 4398046642177),
SVEMAP1(svld1_u8_x4, aarch64_sve_ld1_pn_x4, 4398046642177),
SVEMAP1(svld1_vnum_bf16, aarch64_sve_ld1, 4398046519309),
SVEMAP1(svld1_vnum_bf16_x2, aarch64_sve_ld1_pn_x2, 4398046642189),
SVEMAP1(svld1_vnum_bf16_x4, aarch64_sve_ld1_pn_x4, 4398046642189),
SVEMAP1(svld1_vnum_f16, aarch64_sve_ld1, 4398046519302),
SVEMAP1(svld1_vnum_f16_x2, aarch64_sve_ld1_pn_x2, 4398046642182),
SVEMAP1(svld1_vnum_f16_x4, aarch64_sve_ld1_pn_x4, 4398046642182),
SVEMAP1(svld1_vnum_f32, aarch64_sve_ld1, 4398046519303),
SVEMAP1(svld1_vnum_f32_x2, aarch64_sve_ld1_pn_x2, 4398046642183),
SVEMAP1(svld1_vnum_f32_x4, aarch64_sve_ld1_pn_x4, 4398046642183),
SVEMAP1(svld1_vnum_f64, aarch64_sve_ld1, 4398046519304),
SVEMAP1(svld1_vnum_f64_x2, aarch64_sve_ld1_pn_x2, 4398046642184),
SVEMAP1(svld1_vnum_f64_x4, aarch64_sve_ld1_pn_x4, 4398046642184),
SVEMAP1(svld1_vnum_mf8, aarch64_sve_ld1, 4398046519310),
SVEMAP1(svld1_vnum_mf8_x2, aarch64_sve_ld1_pn_x2, 4398046642190),
SVEMAP1(svld1_vnum_mf8_x4, aarch64_sve_ld1_pn_x4, 4398046642190),
SVEMAP1(svld1_vnum_s16, aarch64_sve_ld1, 4398046519298),
SVEMAP1(svld1_vnum_s16_x2, aarch64_sve_ld1_pn_x2, 4398046642178),
SVEMAP1(svld1_vnum_s16_x4, aarch64_sve_ld1_pn_x4, 4398046642178),
SVEMAP1(svld1_vnum_s32, aarch64_sve_ld1, 4398046519299),
SVEMAP1(svld1_vnum_s32_x2, aarch64_sve_ld1_pn_x2, 4398046642179),
SVEMAP1(svld1_vnum_s32_x4, aarch64_sve_ld1_pn_x4, 4398046642179),
SVEMAP1(svld1_vnum_s64, aarch64_sve_ld1, 4398046519300),
SVEMAP1(svld1_vnum_s64_x2, aarch64_sve_ld1_pn_x2, 4398046642180),
SVEMAP1(svld1_vnum_s64_x4, aarch64_sve_ld1_pn_x4, 4398046642180),
SVEMAP1(svld1_vnum_s8, aarch64_sve_ld1, 4398046519297),
SVEMAP1(svld1_vnum_s8_x2, aarch64_sve_ld1_pn_x2, 4398046642177),
SVEMAP1(svld1_vnum_s8_x4, aarch64_sve_ld1_pn_x4, 4398046642177),
SVEMAP1(svld1_vnum_u16, aarch64_sve_ld1, 4398046519298),
SVEMAP1(svld1_vnum_u16_x2, aarch64_sve_ld1_pn_x2, 4398046642178),
SVEMAP1(svld1_vnum_u16_x4, aarch64_sve_ld1_pn_x4, 4398046642178),
SVEMAP1(svld1_vnum_u32, aarch64_sve_ld1, 4398046519299),
SVEMAP1(svld1_vnum_u32_x2, aarch64_sve_ld1_pn_x2, 4398046642179),
SVEMAP1(svld1_vnum_u32_x4, aarch64_sve_ld1_pn_x4, 4398046642179),
SVEMAP1(svld1_vnum_u64, aarch64_sve_ld1, 4398046519300),
SVEMAP1(svld1_vnum_u64_x2, aarch64_sve_ld1_pn_x2, 4398046642180),
SVEMAP1(svld1_vnum_u64_x4, aarch64_sve_ld1_pn_x4, 4398046642180),
SVEMAP1(svld1_vnum_u8, aarch64_sve_ld1, 4398046519297),
SVEMAP1(svld1_vnum_u8_x2, aarch64_sve_ld1_pn_x2, 4398046642177),
SVEMAP1(svld1_vnum_u8_x4, aarch64_sve_ld1_pn_x4, 4398046642177),
SVEMAP1(svld1q_gather_u64base_bf16, aarch64_sve_ld1q_gather_scalar_offset, 16809997),
SVEMAP1(svld1q_gather_u64base_f16, aarch64_sve_ld1q_gather_scalar_offset, 16809990),
SVEMAP1(svld1q_gather_u64base_f32, aarch64_sve_ld1q_gather_scalar_offset, 16809991),
SVEMAP1(svld1q_gather_u64base_f64, aarch64_sve_ld1q_gather_scalar_offset, 16809992),
SVEMAP1(svld1q_gather_u64base_index_bf16, aarch64_sve_ld1q_gather_scalar_offset, 32781),
SVEMAP1(svld1q_gather_u64base_index_f16, aarch64_sve_ld1q_gather_scalar_offset, 32774),
SVEMAP1(svld1q_gather_u64base_index_f32, aarch64_sve_ld1q_gather_scalar_offset, 32775),
SVEMAP1(svld1q_gather_u64base_index_f64, aarch64_sve_ld1q_gather_scalar_offset, 32776),
SVEMAP1(svld1q_gather_u64base_index_s16, aarch64_sve_ld1q_gather_scalar_offset, 32770),
SVEMAP1(svld1q_gather_u64base_index_s32, aarch64_sve_ld1q_gather_scalar_offset, 32771),
SVEMAP1(svld1q_gather_u64base_index_s64, aarch64_sve_ld1q_gather_scalar_offset, 32772),
SVEMAP1(svld1q_gather_u64base_index_u16, aarch64_sve_ld1q_gather_scalar_offset, 32770),
SVEMAP1(svld1q_gather_u64base_index_u32, aarch64_sve_ld1q_gather_scalar_offset, 32771),
SVEMAP1(svld1q_gather_u64base_index_u64, aarch64_sve_ld1q_gather_scalar_offset, 32772),
SVEMAP1(svld1q_gather_u64base_mf8, aarch64_sve_ld1q_gather_scalar_offset, 16809998),
SVEMAP1(svld1q_gather_u64base_offset_bf16, aarch64_sve_ld1q_gather_scalar_offset, 16809997),
SVEMAP1(svld1q_gather_u64base_offset_f16, aarch64_sve_ld1q_gather_scalar_offset, 16809990),
SVEMAP1(svld1q_gather_u64base_offset_f32, aarch64_sve_ld1q_gather_scalar_offset, 16809991),
SVEMAP1(svld1q_gather_u64base_offset_f64, aarch64_sve_ld1q_gather_scalar_offset, 16809992),
SVEMAP1(svld1q_gather_u64base_offset_mf8, aarch64_sve_ld1q_gather_scalar_offset, 16809998),
SVEMAP1(svld1q_gather_u64base_offset_s16, aarch64_sve_ld1q_gather_scalar_offset, 16809986),
SVEMAP1(svld1q_gather_u64base_offset_s32, aarch64_sve_ld1q_gather_scalar_offset, 16809987),
SVEMAP1(svld1q_gather_u64base_offset_s64, aarch64_sve_ld1q_gather_scalar_offset, 16809988),
SVEMAP1(svld1q_gather_u64base_offset_s8, aarch64_sve_ld1q_gather_scalar_offset, 16809985),
SVEMAP1(svld1q_gather_u64base_offset_u16, aarch64_sve_ld1q_gather_scalar_offset, 16809986),
SVEMAP1(svld1q_gather_u64base_offset_u32, aarch64_sve_ld1q_gather_scalar_offset, 16809987),
SVEMAP1(svld1q_gather_u64base_offset_u64, aarch64_sve_ld1q_gather_scalar_offset, 16809988),
SVEMAP1(svld1q_gather_u64base_offset_u8, aarch64_sve_ld1q_gather_scalar_offset, 16809985),
SVEMAP1(svld1q_gather_u64base_s16, aarch64_sve_ld1q_gather_scalar_offset, 16809986),
SVEMAP1(svld1q_gather_u64base_s32, aarch64_sve_ld1q_gather_scalar_offset, 16809987),
SVEMAP1(svld1q_gather_u64base_s64, aarch64_sve_ld1q_gather_scalar_offset, 16809988),
SVEMAP1(svld1q_gather_u64base_s8, aarch64_sve_ld1q_gather_scalar_offset, 16809985),
SVEMAP1(svld1q_gather_u64base_u16, aarch64_sve_ld1q_gather_scalar_offset, 16809986),
SVEMAP1(svld1q_gather_u64base_u32, aarch64_sve_ld1q_gather_scalar_offset, 16809987),
SVEMAP1(svld1q_gather_u64base_u64, aarch64_sve_ld1q_gather_scalar_offset, 16809988),
SVEMAP1(svld1q_gather_u64base_u8, aarch64_sve_ld1q_gather_scalar_offset, 16809985),
SVEMAP1(svld1q_gather_u64index_bf16, aarch64_sve_ld1q_gather_index, 32781),
SVEMAP1(svld1q_gather_u64index_f16, aarch64_sve_ld1q_gather_index, 32774),
SVEMAP1(svld1q_gather_u64index_f32, aarch64_sve_ld1q_gather_index, 32775),
SVEMAP1(svld1q_gather_u64index_f64, aarch64_sve_ld1q_gather_index, 32776),
SVEMAP1(svld1q_gather_u64index_s16, aarch64_sve_ld1q_gather_index, 32770),
SVEMAP1(svld1q_gather_u64index_s32, aarch64_sve_ld1q_gather_index, 32771),
SVEMAP1(svld1q_gather_u64index_s64, aarch64_sve_ld1q_gather_index, 32772),
SVEMAP1(svld1q_gather_u64index_u16, aarch64_sve_ld1q_gather_index, 32770),
SVEMAP1(svld1q_gather_u64index_u32, aarch64_sve_ld1q_gather_index, 32771),
SVEMAP1(svld1q_gather_u64index_u64, aarch64_sve_ld1q_gather_index, 32772),
SVEMAP1(svld1q_gather_u64offset_bf16, aarch64_sve_ld1q_gather_vector_offset, 16809997),
SVEMAP1(svld1q_gather_u64offset_f16, aarch64_sve_ld1q_gather_vector_offset, 16809990),
SVEMAP1(svld1q_gather_u64offset_f32, aarch64_sve_ld1q_gather_vector_offset, 16809991),
SVEMAP1(svld1q_gather_u64offset_f64, aarch64_sve_ld1q_gather_vector_offset, 16809992),
SVEMAP1(svld1q_gather_u64offset_mf8, aarch64_sve_ld1q_gather_vector_offset, 16809998),
SVEMAP1(svld1q_gather_u64offset_s16, aarch64_sve_ld1q_gather_vector_offset, 16809986),
SVEMAP1(svld1q_gather_u64offset_s32, aarch64_sve_ld1q_gather_vector_offset, 16809987),
SVEMAP1(svld1q_gather_u64offset_s64, aarch64_sve_ld1q_gather_vector_offset, 16809988),
SVEMAP1(svld1q_gather_u64offset_s8, aarch64_sve_ld1q_gather_vector_offset, 16809985),
SVEMAP1(svld1q_gather_u64offset_u16, aarch64_sve_ld1q_gather_vector_offset, 16809986),
SVEMAP1(svld1q_gather_u64offset_u32, aarch64_sve_ld1q_gather_vector_offset, 16809987),
SVEMAP1(svld1q_gather_u64offset_u64, aarch64_sve_ld1q_gather_vector_offset, 16809988),
SVEMAP1(svld1q_gather_u64offset_u8, aarch64_sve_ld1q_gather_vector_offset, 16809985),
SVEMAP1(svld1ro_bf16, aarch64_sve_ld1ro, 13),
SVEMAP1(svld1ro_f16, aarch64_sve_ld1ro, 6),
SVEMAP1(svld1ro_f32, aarch64_sve_ld1ro, 7),
SVEMAP1(svld1ro_f64, aarch64_sve_ld1ro, 8),
SVEMAP1(svld1ro_mf8, aarch64_sve_ld1ro, 14),
SVEMAP1(svld1ro_s16, aarch64_sve_ld1ro, 2),
SVEMAP1(svld1ro_s32, aarch64_sve_ld1ro, 3),
SVEMAP1(svld1ro_s64, aarch64_sve_ld1ro, 4),
SVEMAP1(svld1ro_s8, aarch64_sve_ld1ro, 1),
SVEMAP1(svld1ro_u16, aarch64_sve_ld1ro, 2),
SVEMAP1(svld1ro_u32, aarch64_sve_ld1ro, 3),
SVEMAP1(svld1ro_u64, aarch64_sve_ld1ro, 4),
SVEMAP1(svld1ro_u8, aarch64_sve_ld1ro, 1),
SVEMAP1(svld1rq_bf16, aarch64_sve_ld1rq, 4398046511117),
SVEMAP1(svld1rq_f16, aarch64_sve_ld1rq, 4398046511110),
SVEMAP1(svld1rq_f32, aarch64_sve_ld1rq, 4398046511111),
SVEMAP1(svld1rq_f64, aarch64_sve_ld1rq, 4398046511112),
SVEMAP1(svld1rq_mf8, aarch64_sve_ld1rq, 4398046511118),
SVEMAP1(svld1rq_s16, aarch64_sve_ld1rq, 4398046511106),
SVEMAP1(svld1rq_s32, aarch64_sve_ld1rq, 4398046511107),
SVEMAP1(svld1rq_s64, aarch64_sve_ld1rq, 4398046511108),
SVEMAP1(svld1rq_s8, aarch64_sve_ld1rq, 4398046511105),
SVEMAP1(svld1rq_u16, aarch64_sve_ld1rq, 4398046511106),
SVEMAP1(svld1rq_u32, aarch64_sve_ld1rq, 4398046511107),
SVEMAP1(svld1rq_u64, aarch64_sve_ld1rq, 4398046511108),
SVEMAP1(svld1rq_u8, aarch64_sve_ld1rq, 4398046511105),
SVEMAP1(svld1sb_gather_s32offset_s32, aarch64_sve_ld1_gather_sxtw, 16810003),
SVEMAP1(svld1sb_gather_s32offset_u32, aarch64_sve_ld1_gather_sxtw, 16810003),
SVEMAP1(svld1sb_gather_s64offset_s64, aarch64_sve_ld1_gather, 16810004),
SVEMAP1(svld1sb_gather_s64offset_u64, aarch64_sve_ld1_gather, 16810004),
SVEMAP1(svld1sb_gather_u32base_offset_s32, aarch64_sve_ld1_gather_scalar_offset, 16810003),
SVEMAP1(svld1sb_gather_u32base_offset_u32, aarch64_sve_ld1_gather_scalar_offset, 16810003),
SVEMAP1(svld1sb_gather_u32base_s32, aarch64_sve_ld1_gather_scalar_offset, 32787),
SVEMAP1(svld1sb_gather_u32base_u32, aarch64_sve_ld1_gather_scalar_offset, 32787),
SVEMAP1(svld1sb_gather_u32offset_s32, aarch64_sve_ld1_gather_uxtw, 16810003),
SVEMAP1(svld1sb_gather_u32offset_u32, aarch64_sve_ld1_gather_uxtw, 16810003),
SVEMAP1(svld1sb_gather_u64base_offset_s64, aarch64_sve_ld1_gather_scalar_offset, 16810004),
SVEMAP1(svld1sb_gather_u64base_offset_u64, aarch64_sve_ld1_gather_scalar_offset, 16810004),
SVEMAP1(svld1sb_gather_u64base_s64, aarch64_sve_ld1_gather_scalar_offset, 32788),
SVEMAP1(svld1sb_gather_u64base_u64, aarch64_sve_ld1_gather_scalar_offset, 32788),
SVEMAP1(svld1sb_gather_u64offset_s64, aarch64_sve_ld1_gather, 16810004),
SVEMAP1(svld1sb_gather_u64offset_u64, aarch64_sve_ld1_gather, 16810004),
SVEMAP1(svld1sb_s16, aarch64_sve_ld1, 4398046519314),
SVEMAP1(svld1sb_s32, aarch64_sve_ld1, 4398046519315),
SVEMAP1(svld1sb_s64, aarch64_sve_ld1, 4398046519316),
SVEMAP1(svld1sb_u16, aarch64_sve_ld1, 4398046519314),
SVEMAP1(svld1sb_u32, aarch64_sve_ld1, 4398046519315),
SVEMAP1(svld1sb_u64, aarch64_sve_ld1, 4398046519316),
SVEMAP1(svld1sb_vnum_s16, aarch64_sve_ld1, 4398046519314),
SVEMAP1(svld1sb_vnum_s32, aarch64_sve_ld1, 4398046519315),
SVEMAP1(svld1sb_vnum_s64, aarch64_sve_ld1, 4398046519316),
SVEMAP1(svld1sb_vnum_u16, aarch64_sve_ld1, 4398046519314),
SVEMAP1(svld1sb_vnum_u32, aarch64_sve_ld1, 4398046519315),
SVEMAP1(svld1sb_vnum_u64, aarch64_sve_ld1, 4398046519316),
SVEMAP1(svld1sh_gather_s32index_s32, aarch64_sve_ld1_gather_sxtw_index, 32803),
SVEMAP1(svld1sh_gather_s32index_u32, aarch64_sve_ld1_gather_sxtw_index, 32803),
SVEMAP1(svld1sh_gather_s32offset_s32, aarch64_sve_ld1_gather_sxtw, 16810019),
SVEMAP1(svld1sh_gather_s32offset_u32, aarch64_sve_ld1_gather_sxtw, 16810019),
SVEMAP1(svld1sh_gather_s64index_s64, aarch64_sve_ld1_gather_index, 32804),
SVEMAP1(svld1sh_gather_s64index_u64, aarch64_sve_ld1_gather_index, 32804),
SVEMAP1(svld1sh_gather_s64offset_s64, aarch64_sve_ld1_gather, 16810020),
SVEMAP1(svld1sh_gather_s64offset_u64, aarch64_sve_ld1_gather, 16810020),
SVEMAP1(svld1sh_gather_u32base_index_s32, aarch64_sve_ld1_gather_scalar_offset, 32803),
SVEMAP1(svld1sh_gather_u32base_index_u32, aarch64_sve_ld1_gather_scalar_offset, 32803),
SVEMAP1(svld1sh_gather_u32base_offset_s32, aarch64_sve_ld1_gather_scalar_offset, 16810019),
SVEMAP1(svld1sh_gather_u32base_offset_u32, aarch64_sve_ld1_gather_scalar_offset, 16810019),
SVEMAP1(svld1sh_gather_u32base_s32, aarch64_sve_ld1_gather_scalar_offset, 32803),
SVEMAP1(svld1sh_gather_u32base_u32, aarch64_sve_ld1_gather_scalar_offset, 32803),
SVEMAP1(svld1sh_gather_u32index_s32, aarch64_sve_ld1_gather_uxtw_index, 32803),
SVEMAP1(svld1sh_gather_u32index_u32, aarch64_sve_ld1_gather_uxtw_index, 32803),
SVEMAP1(svld1sh_gather_u32offset_s32, aarch64_sve_ld1_gather_uxtw, 16810019),
SVEMAP1(svld1sh_gather_u32offset_u32, aarch64_sve_ld1_gather_uxtw, 16810019),
SVEMAP1(svld1sh_gather_u64base_index_s64, aarch64_sve_ld1_gather_scalar_offset, 32804),
SVEMAP1(svld1sh_gather_u64base_index_u64, aarch64_sve_ld1_gather_scalar_offset, 32804),
SVEMAP1(svld1sh_gather_u64base_offset_s64, aarch64_sve_ld1_gather_scalar_offset, 16810020),
SVEMAP1(svld1sh_gather_u64base_offset_u64, aarch64_sve_ld1_gather_scalar_offset, 16810020),
SVEMAP1(svld1sh_gather_u64base_s64, aarch64_sve_ld1_gather_scalar_offset, 32804),
SVEMAP1(svld1sh_gather_u64base_u64, aarch64_sve_ld1_gather_scalar_offset, 32804),
SVEMAP1(svld1sh_gather_u64index_s64, aarch64_sve_ld1_gather_index, 32804),
SVEMAP1(svld1sh_gather_u64index_u64, aarch64_sve_ld1_gather_index, 32804),
SVEMAP1(svld1sh_gather_u64offset_s64, aarch64_sve_ld1_gather, 16810020),
SVEMAP1(svld1sh_gather_u64offset_u64, aarch64_sve_ld1_gather, 16810020),
SVEMAP1(svld1sh_s32, aarch64_sve_ld1, 4398046519331),
SVEMAP1(svld1sh_s64, aarch64_sve_ld1, 4398046519332),
SVEMAP1(svld1sh_u32, aarch64_sve_ld1, 4398046519331),
SVEMAP1(svld1sh_u64, aarch64_sve_ld1, 4398046519332),
SVEMAP1(svld1sh_vnum_s32, aarch64_sve_ld1, 4398046519331),
SVEMAP1(svld1sh_vnum_s64, aarch64_sve_ld1, 4398046519332),
SVEMAP1(svld1sh_vnum_u32, aarch64_sve_ld1, 4398046519331),
SVEMAP1(svld1sh_vnum_u64, aarch64_sve_ld1, 4398046519332),
SVEMAP1(svld1sw_gather_s64index_s64, aarch64_sve_ld1_gather_index, 32820),
SVEMAP1(svld1sw_gather_s64index_u64, aarch64_sve_ld1_gather_index, 32820),
SVEMAP1(svld1sw_gather_s64offset_s64, aarch64_sve_ld1_gather, 16810036),
SVEMAP1(svld1sw_gather_s64offset_u64, aarch64_sve_ld1_gather, 16810036),
SVEMAP1(svld1sw_gather_u64base_index_s64, aarch64_sve_ld1_gather_scalar_offset, 32820),
SVEMAP1(svld1sw_gather_u64base_index_u64, aarch64_sve_ld1_gather_scalar_offset, 32820),
SVEMAP1(svld1sw_gather_u64base_offset_s64, aarch64_sve_ld1_gather_scalar_offset, 16810036),
SVEMAP1(svld1sw_gather_u64base_offset_u64, aarch64_sve_ld1_gather_scalar_offset, 16810036),
SVEMAP1(svld1sw_gather_u64base_s64, aarch64_sve_ld1_gather_scalar_offset, 32820),
SVEMAP1(svld1sw_gather_u64base_u64, aarch64_sve_ld1_gather_scalar_offset, 32820),
SVEMAP1(svld1sw_gather_u64index_s64, aarch64_sve_ld1_gather_index, 32820),
SVEMAP1(svld1sw_gather_u64index_u64, aarch64_sve_ld1_gather_index, 32820),
SVEMAP1(svld1sw_gather_u64offset_s64, aarch64_sve_ld1_gather, 16810036),
SVEMAP1(svld1sw_gather_u64offset_u64, aarch64_sve_ld1_gather, 16810036),
SVEMAP1(svld1sw_s64, aarch64_sve_ld1, 4398046519348),
SVEMAP1(svld1sw_u64, aarch64_sve_ld1, 4398046519348),
SVEMAP1(svld1sw_vnum_s64, aarch64_sve_ld1, 4398046519348),
SVEMAP1(svld1sw_vnum_u64, aarch64_sve_ld1, 4398046519348),
SVEMAP1(svld1ub_gather_s32offset_s32, aarch64_sve_ld1_gather_sxtw, 17334291),
SVEMAP1(svld1ub_gather_s32offset_u32, aarch64_sve_ld1_gather_sxtw, 17334291),
SVEMAP1(svld1ub_gather_s64offset_s64, aarch64_sve_ld1_gather, 17334292),
SVEMAP1(svld1ub_gather_s64offset_u64, aarch64_sve_ld1_gather, 17334292),
SVEMAP1(svld1ub_gather_u32base_offset_s32, aarch64_sve_ld1_gather_scalar_offset, 17334291),
SVEMAP1(svld1ub_gather_u32base_offset_u32, aarch64_sve_ld1_gather_scalar_offset, 17334291),
SVEMAP1(svld1ub_gather_u32base_s32, aarch64_sve_ld1_gather_scalar_offset, 557075),
SVEMAP1(svld1ub_gather_u32base_u32, aarch64_sve_ld1_gather_scalar_offset, 557075),
SVEMAP1(svld1ub_gather_u32offset_s32, aarch64_sve_ld1_gather_uxtw, 17334291),
SVEMAP1(svld1ub_gather_u32offset_u32, aarch64_sve_ld1_gather_uxtw, 17334291),
SVEMAP1(svld1ub_gather_u64base_offset_s64, aarch64_sve_ld1_gather_scalar_offset, 17334292),
SVEMAP1(svld1ub_gather_u64base_offset_u64, aarch64_sve_ld1_gather_scalar_offset, 17334292),
SVEMAP1(svld1ub_gather_u64base_s64, aarch64_sve_ld1_gather_scalar_offset, 557076),
SVEMAP1(svld1ub_gather_u64base_u64, aarch64_sve_ld1_gather_scalar_offset, 557076),
SVEMAP1(svld1ub_gather_u64offset_s64, aarch64_sve_ld1_gather, 17334292),
SVEMAP1(svld1ub_gather_u64offset_u64, aarch64_sve_ld1_gather, 17334292),
SVEMAP1(svld1ub_s16, aarch64_sve_ld1, 4398047043602),
SVEMAP1(svld1ub_s32, aarch64_sve_ld1, 4398047043603),
SVEMAP1(svld1ub_s64, aarch64_sve_ld1, 4398047043604),
SVEMAP1(svld1ub_u16, aarch64_sve_ld1, 4398047043602),
SVEMAP1(svld1ub_u32, aarch64_sve_ld1, 4398047043603),
SVEMAP1(svld1ub_u64, aarch64_sve_ld1, 4398047043604),
SVEMAP1(svld1ub_vnum_s16, aarch64_sve_ld1, 4398047043602),
SVEMAP1(svld1ub_vnum_s32, aarch64_sve_ld1, 4398047043603),
SVEMAP1(svld1ub_vnum_s64, aarch64_sve_ld1, 4398047043604),
SVEMAP1(svld1ub_vnum_u16, aarch64_sve_ld1, 4398047043602),
SVEMAP1(svld1ub_vnum_u32, aarch64_sve_ld1, 4398047043603),
SVEMAP1(svld1ub_vnum_u64, aarch64_sve_ld1, 4398047043604),
SVEMAP1(svld1udq_f64, aarch64_sve_ld1udq, 8264),
SVEMAP1(svld1udq_s64, aarch64_sve_ld1udq, 8260),
SVEMAP1(svld1udq_u64, aarch64_sve_ld1udq, 8260),
SVEMAP1(svld1udq_vnum_f64, aarch64_sve_ld1udq, 8264),
SVEMAP1(svld1udq_vnum_s64, aarch64_sve_ld1udq, 8260),
SVEMAP1(svld1udq_vnum_u64, aarch64_sve_ld1udq, 8260),
SVEMAP1(svld1uh_gather_s32index_s32, aarch64_sve_ld1_gather_sxtw_index, 557091),
SVEMAP1(svld1uh_gather_s32index_u32, aarch64_sve_ld1_gather_sxtw_index, 557091),
SVEMAP1(svld1uh_gather_s32offset_s32, aarch64_sve_ld1_gather_sxtw, 17334307),
SVEMAP1(svld1uh_gather_s32offset_u32, aarch64_sve_ld1_gather_sxtw, 17334307),
SVEMAP1(svld1uh_gather_s64index_s64, aarch64_sve_ld1_gather_index, 557092),
SVEMAP1(svld1uh_gather_s64index_u64, aarch64_sve_ld1_gather_index, 557092),
SVEMAP1(svld1uh_gather_s64offset_s64, aarch64_sve_ld1_gather, 17334308),
SVEMAP1(svld1uh_gather_s64offset_u64, aarch64_sve_ld1_gather, 17334308),
SVEMAP1(svld1uh_gather_u32base_index_s32, aarch64_sve_ld1_gather_scalar_offset, 557091),
SVEMAP1(svld1uh_gather_u32base_index_u32, aarch64_sve_ld1_gather_scalar_offset, 557091),
SVEMAP1(svld1uh_gather_u32base_offset_s32, aarch64_sve_ld1_gather_scalar_offset, 17334307),
SVEMAP1(svld1uh_gather_u32base_offset_u32, aarch64_sve_ld1_gather_scalar_offset, 17334307),
SVEMAP1(svld1uh_gather_u32base_s32, aarch64_sve_ld1_gather_scalar_offset, 557091),
SVEMAP1(svld1uh_gather_u32base_u32, aarch64_sve_ld1_gather_scalar_offset, 557091),
SVEMAP1(svld1uh_gather_u32index_s32, aarch64_sve_ld1_gather_uxtw_index, 557091),
SVEMAP1(svld1uh_gather_u32index_u32, aarch64_sve_ld1_gather_uxtw_index, 557091),
SVEMAP1(svld1uh_gather_u32offset_s32, aarch64_sve_ld1_gather_uxtw, 17334307),
SVEMAP1(svld1uh_gather_u32offset_u32, aarch64_sve_ld1_gather_uxtw, 17334307),
SVEMAP1(svld1uh_gather_u64base_index_s64, aarch64_sve_ld1_gather_scalar_offset, 557092),
SVEMAP1(svld1uh_gather_u64base_index_u64, aarch64_sve_ld1_gather_scalar_offset, 557092),
SVEMAP1(svld1uh_gather_u64base_offset_s64, aarch64_sve_ld1_gather_scalar_offset, 17334308),
SVEMAP1(svld1uh_gather_u64base_offset_u64, aarch64_sve_ld1_gather_scalar_offset, 17334308),
SVEMAP1(svld1uh_gather_u64base_s64, aarch64_sve_ld1_gather_scalar_offset, 557092),
SVEMAP1(svld1uh_gather_u64base_u64, aarch64_sve_ld1_gather_scalar_offset, 557092),
SVEMAP1(svld1uh_gather_u64index_s64, aarch64_sve_ld1_gather_index, 557092),
SVEMAP1(svld1uh_gather_u64index_u64, aarch64_sve_ld1_gather_index, 557092),
SVEMAP1(svld1uh_gather_u64offset_s64, aarch64_sve_ld1_gather, 17334308),
SVEMAP1(svld1uh_gather_u64offset_u64, aarch64_sve_ld1_gather, 17334308),
SVEMAP1(svld1uh_s32, aarch64_sve_ld1, 4398047043619),
SVEMAP1(svld1uh_s64, aarch64_sve_ld1, 4398047043620),
SVEMAP1(svld1uh_u32, aarch64_sve_ld1, 4398047043619),
SVEMAP1(svld1uh_u64, aarch64_sve_ld1, 4398047043620),
SVEMAP1(svld1uh_vnum_s32, aarch64_sve_ld1, 4398047043619),
SVEMAP1(svld1uh_vnum_s64, aarch64_sve_ld1, 4398047043620),
SVEMAP1(svld1uh_vnum_u32, aarch64_sve_ld1, 4398047043619),
SVEMAP1(svld1uh_vnum_u64, aarch64_sve_ld1, 4398047043620),
SVEMAP1(svld1uw_gather_s64index_s64, aarch64_sve_ld1_gather_index, 557108),
SVEMAP1(svld1uw_gather_s64index_u64, aarch64_sve_ld1_gather_index, 557108),
SVEMAP1(svld1uw_gather_s64offset_s64, aarch64_sve_ld1_gather, 17334324),
SVEMAP1(svld1uw_gather_s64offset_u64, aarch64_sve_ld1_gather, 17334324),
SVEMAP1(svld1uw_gather_u64base_index_s64, aarch64_sve_ld1_gather_scalar_offset, 557108),
SVEMAP1(svld1uw_gather_u64base_index_u64, aarch64_sve_ld1_gather_scalar_offset, 557108),
SVEMAP1(svld1uw_gather_u64base_offset_s64, aarch64_sve_ld1_gather_scalar_offset, 17334324),
SVEMAP1(svld1uw_gather_u64base_offset_u64, aarch64_sve_ld1_gather_scalar_offset, 17334324),
SVEMAP1(svld1uw_gather_u64base_s64, aarch64_sve_ld1_gather_scalar_offset, 557108),
SVEMAP1(svld1uw_gather_u64base_u64, aarch64_sve_ld1_gather_scalar_offset, 557108),
SVEMAP1(svld1uw_gather_u64index_s64, aarch64_sve_ld1_gather_index, 557108),
SVEMAP1(svld1uw_gather_u64index_u64, aarch64_sve_ld1_gather_index, 557108),
SVEMAP1(svld1uw_gather_u64offset_s64, aarch64_sve_ld1_gather, 17334324),
SVEMAP1(svld1uw_gather_u64offset_u64, aarch64_sve_ld1_gather, 17334324),
SVEMAP1(svld1uw_s64, aarch64_sve_ld1, 4398047043636),
SVEMAP1(svld1uw_u64, aarch64_sve_ld1, 4398047043636),
SVEMAP1(svld1uw_vnum_s64, aarch64_sve_ld1, 4398047043636),
SVEMAP1(svld1uw_vnum_u64, aarch64_sve_ld1, 4398047043636),
SVEMAP1(svld1uwq_f32, aarch64_sve_ld1uwq, 8247),
SVEMAP1(svld1uwq_s32, aarch64_sve_ld1uwq, 8243),
SVEMAP1(svld1uwq_u32, aarch64_sve_ld1uwq, 8243),
SVEMAP1(svld1uwq_vnum_f32, aarch64_sve_ld1uwq, 8247),
SVEMAP1(svld1uwq_vnum_s32, aarch64_sve_ld1uwq, 8243),
SVEMAP1(svld1uwq_vnum_u32, aarch64_sve_ld1uwq, 8243),
SVEMAP1(svld2_bf16, aarch64_sve_ld2_sret, 4398046642189),
SVEMAP1(svld2_f16, aarch64_sve_ld2_sret, 4398046642182),
SVEMAP1(svld2_f32, aarch64_sve_ld2_sret, 4398046642183),
SVEMAP1(svld2_f64, aarch64_sve_ld2_sret, 4398046642184),
SVEMAP1(svld2_mf8, aarch64_sve_ld2_sret, 4398046642190),
SVEMAP1(svld2_s16, aarch64_sve_ld2_sret, 4398046642178),
SVEMAP1(svld2_s32, aarch64_sve_ld2_sret, 4398046642179),
SVEMAP1(svld2_s64, aarch64_sve_ld2_sret, 4398046642180),
SVEMAP1(svld2_s8, aarch64_sve_ld2_sret, 4398046642177),
SVEMAP1(svld2_u16, aarch64_sve_ld2_sret, 4398046642178),
SVEMAP1(svld2_u32, aarch64_sve_ld2_sret, 4398046642179),
SVEMAP1(svld2_u64, aarch64_sve_ld2_sret, 4398046642180),
SVEMAP1(svld2_u8, aarch64_sve_ld2_sret, 4398046642177),
SVEMAP1(svld2_vnum_bf16, aarch64_sve_ld2_sret, 4398046642189),
SVEMAP1(svld2_vnum_f16, aarch64_sve_ld2_sret, 4398046642182),
SVEMAP1(svld2_vnum_f32, aarch64_sve_ld2_sret, 4398046642183),
SVEMAP1(svld2_vnum_f64, aarch64_sve_ld2_sret, 4398046642184),
SVEMAP1(svld2_vnum_mf8, aarch64_sve_ld2_sret, 4398046642190),
SVEMAP1(svld2_vnum_s16, aarch64_sve_ld2_sret, 4398046642178),
SVEMAP1(svld2_vnum_s32, aarch64_sve_ld2_sret, 4398046642179),
SVEMAP1(svld2_vnum_s64, aarch64_sve_ld2_sret, 4398046642180),
SVEMAP1(svld2_vnum_s8, aarch64_sve_ld2_sret, 4398046642177),
SVEMAP1(svld2_vnum_u16, aarch64_sve_ld2_sret, 4398046642178),
SVEMAP1(svld2_vnum_u32, aarch64_sve_ld2_sret, 4398046642179),
SVEMAP1(svld2_vnum_u64, aarch64_sve_ld2_sret, 4398046642180),
SVEMAP1(svld2_vnum_u8, aarch64_sve_ld2_sret, 4398046642177),
SVEMAP1(svld2q_bf16, aarch64_sve_ld2q_sret, 4398046642189),
SVEMAP1(svld2q_f16, aarch64_sve_ld2q_sret, 4398046642182),
SVEMAP1(svld2q_f32, aarch64_sve_ld2q_sret, 4398046642183),
SVEMAP1(svld2q_f64, aarch64_sve_ld2q_sret, 4398046642184),
SVEMAP1(svld2q_mf8, aarch64_sve_ld2q_sret, 4398046642190),
SVEMAP1(svld2q_s16, aarch64_sve_ld2q_sret, 4398046642178),
SVEMAP1(svld2q_s32, aarch64_sve_ld2q_sret, 4398046642179),
SVEMAP1(svld2q_s64, aarch64_sve_ld2q_sret, 4398046642180),
SVEMAP1(svld2q_s8, aarch64_sve_ld2q_sret, 4398046642177),
SVEMAP1(svld2q_u16, aarch64_sve_ld2q_sret, 4398046642178),
SVEMAP1(svld2q_u32, aarch64_sve_ld2q_sret, 4398046642179),
SVEMAP1(svld2q_u64, aarch64_sve_ld2q_sret, 4398046642180),
SVEMAP1(svld2q_u8, aarch64_sve_ld2q_sret, 4398046642177),
SVEMAP1(svld2q_vnum_bf16, aarch64_sve_ld2q_sret, 4398046642189),
SVEMAP1(svld2q_vnum_f16, aarch64_sve_ld2q_sret, 4398046642182),
SVEMAP1(svld2q_vnum_f32, aarch64_sve_ld2q_sret, 4398046642183),
SVEMAP1(svld2q_vnum_f64, aarch64_sve_ld2q_sret, 4398046642184),
SVEMAP1(svld2q_vnum_mf8, aarch64_sve_ld2q_sret, 4398046642190),
SVEMAP1(svld2q_vnum_s16, aarch64_sve_ld2q_sret, 4398046642178),
SVEMAP1(svld2q_vnum_s32, aarch64_sve_ld2q_sret, 4398046642179),
SVEMAP1(svld2q_vnum_s64, aarch64_sve_ld2q_sret, 4398046642180),
SVEMAP1(svld2q_vnum_s8, aarch64_sve_ld2q_sret, 4398046642177),
SVEMAP1(svld2q_vnum_u16, aarch64_sve_ld2q_sret, 4398046642178),
SVEMAP1(svld2q_vnum_u32, aarch64_sve_ld2q_sret, 4398046642179),
SVEMAP1(svld2q_vnum_u64, aarch64_sve_ld2q_sret, 4398046642180),
SVEMAP1(svld2q_vnum_u8, aarch64_sve_ld2q_sret, 4398046642177),
SVEMAP1(svld3_bf16, aarch64_sve_ld3_sret, 4398046642189),
SVEMAP1(svld3_f16, aarch64_sve_ld3_sret, 4398046642182),
SVEMAP1(svld3_f32, aarch64_sve_ld3_sret, 4398046642183),
SVEMAP1(svld3_f64, aarch64_sve_ld3_sret, 4398046642184),
SVEMAP1(svld3_mf8, aarch64_sve_ld3_sret, 4398046642190),
SVEMAP1(svld3_s16, aarch64_sve_ld3_sret, 4398046642178),
SVEMAP1(svld3_s32, aarch64_sve_ld3_sret, 4398046642179),
SVEMAP1(svld3_s64, aarch64_sve_ld3_sret, 4398046642180),
SVEMAP1(svld3_s8, aarch64_sve_ld3_sret, 4398046642177),
SVEMAP1(svld3_u16, aarch64_sve_ld3_sret, 4398046642178),
SVEMAP1(svld3_u32, aarch64_sve_ld3_sret, 4398046642179),
SVEMAP1(svld3_u64, aarch64_sve_ld3_sret, 4398046642180),
SVEMAP1(svld3_u8, aarch64_sve_ld3_sret, 4398046642177),
SVEMAP1(svld3_vnum_bf16, aarch64_sve_ld3_sret, 4398046642189),
SVEMAP1(svld3_vnum_f16, aarch64_sve_ld3_sret, 4398046642182),
SVEMAP1(svld3_vnum_f32, aarch64_sve_ld3_sret, 4398046642183),
SVEMAP1(svld3_vnum_f64, aarch64_sve_ld3_sret, 4398046642184),
SVEMAP1(svld3_vnum_mf8, aarch64_sve_ld3_sret, 4398046642190),
SVEMAP1(svld3_vnum_s16, aarch64_sve_ld3_sret, 4398046642178),
SVEMAP1(svld3_vnum_s32, aarch64_sve_ld3_sret, 4398046642179),
SVEMAP1(svld3_vnum_s64, aarch64_sve_ld3_sret, 4398046642180),
SVEMAP1(svld3_vnum_s8, aarch64_sve_ld3_sret, 4398046642177),
SVEMAP1(svld3_vnum_u16, aarch64_sve_ld3_sret, 4398046642178),
SVEMAP1(svld3_vnum_u32, aarch64_sve_ld3_sret, 4398046642179),
SVEMAP1(svld3_vnum_u64, aarch64_sve_ld3_sret, 4398046642180),
SVEMAP1(svld3_vnum_u8, aarch64_sve_ld3_sret, 4398046642177),
SVEMAP1(svld3q_bf16, aarch64_sve_ld3q_sret, 4398046642189),
SVEMAP1(svld3q_f16, aarch64_sve_ld3q_sret, 4398046642182),
SVEMAP1(svld3q_f32, aarch64_sve_ld3q_sret, 4398046642183),
SVEMAP1(svld3q_f64, aarch64_sve_ld3q_sret, 4398046642184),
SVEMAP1(svld3q_mf8, aarch64_sve_ld3q_sret, 4398046642190),
SVEMAP1(svld3q_s16, aarch64_sve_ld3q_sret, 4398046642178),
SVEMAP1(svld3q_s32, aarch64_sve_ld3q_sret, 4398046642179),
SVEMAP1(svld3q_s64, aarch64_sve_ld3q_sret, 4398046642180),
SVEMAP1(svld3q_s8, aarch64_sve_ld3q_sret, 4398046642177),
SVEMAP1(svld3q_u16, aarch64_sve_ld3q_sret, 4398046642178),
SVEMAP1(svld3q_u32, aarch64_sve_ld3q_sret, 4398046642179),
SVEMAP1(svld3q_u64, aarch64_sve_ld3q_sret, 4398046642180),
SVEMAP1(svld3q_u8, aarch64_sve_ld3q_sret, 4398046642177),
SVEMAP1(svld3q_vnum_bf16, aarch64_sve_ld3q_sret, 4398046642189),
SVEMAP1(svld3q_vnum_f16, aarch64_sve_ld3q_sret, 4398046642182),
SVEMAP1(svld3q_vnum_f32, aarch64_sve_ld3q_sret, 4398046642183),
SVEMAP1(svld3q_vnum_f64, aarch64_sve_ld3q_sret, 4398046642184),
SVEMAP1(svld3q_vnum_mf8, aarch64_sve_ld3q_sret, 4398046642190),
SVEMAP1(svld3q_vnum_s16, aarch64_sve_ld3q_sret, 4398046642178),
SVEMAP1(svld3q_vnum_s32, aarch64_sve_ld3q_sret, 4398046642179),
SVEMAP1(svld3q_vnum_s64, aarch64_sve_ld3q_sret, 4398046642180),
SVEMAP1(svld3q_vnum_s8, aarch64_sve_ld3q_sret, 4398046642177),
SVEMAP1(svld3q_vnum_u16, aarch64_sve_ld3q_sret, 4398046642178),
SVEMAP1(svld3q_vnum_u32, aarch64_sve_ld3q_sret, 4398046642179),
SVEMAP1(svld3q_vnum_u64, aarch64_sve_ld3q_sret, 4398046642180),
SVEMAP1(svld3q_vnum_u8, aarch64_sve_ld3q_sret, 4398046642177),
SVEMAP1(svld4_bf16, aarch64_sve_ld4_sret, 4398046642189),
SVEMAP1(svld4_f16, aarch64_sve_ld4_sret, 4398046642182),
SVEMAP1(svld4_f32, aarch64_sve_ld4_sret, 4398046642183),
SVEMAP1(svld4_f64, aarch64_sve_ld4_sret, 4398046642184),
SVEMAP1(svld4_mf8, aarch64_sve_ld4_sret, 4398046642190),
SVEMAP1(svld4_s16, aarch64_sve_ld4_sret, 4398046642178),
SVEMAP1(svld4_s32, aarch64_sve_ld4_sret, 4398046642179),
SVEMAP1(svld4_s64, aarch64_sve_ld4_sret, 4398046642180),
SVEMAP1(svld4_s8, aarch64_sve_ld4_sret, 4398046642177),
SVEMAP1(svld4_u16, aarch64_sve_ld4_sret, 4398046642178),
SVEMAP1(svld4_u32, aarch64_sve_ld4_sret, 4398046642179),
SVEMAP1(svld4_u64, aarch64_sve_ld4_sret, 4398046642180),
SVEMAP1(svld4_u8, aarch64_sve_ld4_sret, 4398046642177),
SVEMAP1(svld4_vnum_bf16, aarch64_sve_ld4_sret, 4398046642189),
SVEMAP1(svld4_vnum_f16, aarch64_sve_ld4_sret, 4398046642182),
SVEMAP1(svld4_vnum_f32, aarch64_sve_ld4_sret, 4398046642183),
SVEMAP1(svld4_vnum_f64, aarch64_sve_ld4_sret, 4398046642184),
SVEMAP1(svld4_vnum_mf8, aarch64_sve_ld4_sret, 4398046642190),
SVEMAP1(svld4_vnum_s16, aarch64_sve_ld4_sret, 4398046642178),
SVEMAP1(svld4_vnum_s32, aarch64_sve_ld4_sret, 4398046642179),
SVEMAP1(svld4_vnum_s64, aarch64_sve_ld4_sret, 4398046642180),
SVEMAP1(svld4_vnum_s8, aarch64_sve_ld4_sret, 4398046642177),
SVEMAP1(svld4_vnum_u16, aarch64_sve_ld4_sret, 4398046642178),
SVEMAP1(svld4_vnum_u32, aarch64_sve_ld4_sret, 4398046642179),
SVEMAP1(svld4_vnum_u64, aarch64_sve_ld4_sret, 4398046642180),
SVEMAP1(svld4_vnum_u8, aarch64_sve_ld4_sret, 4398046642177),
SVEMAP1(svld4q_bf16, aarch64_sve_ld4q_sret, 4398046642189),
SVEMAP1(svld4q_f16, aarch64_sve_ld4q_sret, 4398046642182),
SVEMAP1(svld4q_f32, aarch64_sve_ld4q_sret, 4398046642183),
SVEMAP1(svld4q_f64, aarch64_sve_ld4q_sret, 4398046642184),
SVEMAP1(svld4q_mf8, aarch64_sve_ld4q_sret, 4398046642190),
SVEMAP1(svld4q_s16, aarch64_sve_ld4q_sret, 4398046642178),
SVEMAP1(svld4q_s32, aarch64_sve_ld4q_sret, 4398046642179),
SVEMAP1(svld4q_s64, aarch64_sve_ld4q_sret, 4398046642180),
SVEMAP1(svld4q_s8, aarch64_sve_ld4q_sret, 4398046642177),
SVEMAP1(svld4q_u16, aarch64_sve_ld4q_sret, 4398046642178),
SVEMAP1(svld4q_u32, aarch64_sve_ld4q_sret, 4398046642179),
SVEMAP1(svld4q_u64, aarch64_sve_ld4q_sret, 4398046642180),
SVEMAP1(svld4q_u8, aarch64_sve_ld4q_sret, 4398046642177),
SVEMAP1(svld4q_vnum_bf16, aarch64_sve_ld4q_sret, 4398046642189),
SVEMAP1(svld4q_vnum_f16, aarch64_sve_ld4q_sret, 4398046642182),
SVEMAP1(svld4q_vnum_f32, aarch64_sve_ld4q_sret, 4398046642183),
SVEMAP1(svld4q_vnum_f64, aarch64_sve_ld4q_sret, 4398046642184),
SVEMAP1(svld4q_vnum_mf8, aarch64_sve_ld4q_sret, 4398046642190),
SVEMAP1(svld4q_vnum_s16, aarch64_sve_ld4q_sret, 4398046642178),
SVEMAP1(svld4q_vnum_s32, aarch64_sve_ld4q_sret, 4398046642179),
SVEMAP1(svld4q_vnum_s64, aarch64_sve_ld4q_sret, 4398046642180),
SVEMAP1(svld4q_vnum_s8, aarch64_sve_ld4q_sret, 4398046642177),
SVEMAP1(svld4q_vnum_u16, aarch64_sve_ld4q_sret, 4398046642178),
SVEMAP1(svld4q_vnum_u32, aarch64_sve_ld4q_sret, 4398046642179),
SVEMAP1(svld4q_vnum_u64, aarch64_sve_ld4q_sret, 4398046642180),
SVEMAP1(svld4q_vnum_u8, aarch64_sve_ld4q_sret, 4398046642177),
SVEMAP1(svldff1_bf16, aarch64_sve_ldff1, 8205),
SVEMAP1(svldff1_f16, aarch64_sve_ldff1, 8198),
SVEMAP1(svldff1_f32, aarch64_sve_ldff1, 8199),
SVEMAP1(svldff1_f64, aarch64_sve_ldff1, 8200),
SVEMAP1(svldff1_gather_s32index_f32, aarch64_sve_ldff1_gather_sxtw_index, 32775),
SVEMAP1(svldff1_gather_s32index_s32, aarch64_sve_ldff1_gather_sxtw_index, 32771),
SVEMAP1(svldff1_gather_s32index_u32, aarch64_sve_ldff1_gather_sxtw_index, 32771),
SVEMAP1(svldff1_gather_s32offset_f32, aarch64_sve_ldff1_gather_sxtw, 16809991),
SVEMAP1(svldff1_gather_s32offset_s32, aarch64_sve_ldff1_gather_sxtw, 16809987),
SVEMAP1(svldff1_gather_s32offset_u32, aarch64_sve_ldff1_gather_sxtw, 16809987),
SVEMAP1(svldff1_gather_s64index_f64, aarch64_sve_ldff1_gather_index, 32776),
SVEMAP1(svldff1_gather_s64index_s64, aarch64_sve_ldff1_gather_index, 32772),
SVEMAP1(svldff1_gather_s64index_u64, aarch64_sve_ldff1_gather_index, 32772),
SVEMAP1(svldff1_gather_s64offset_f64, aarch64_sve_ldff1_gather, 16809992),
SVEMAP1(svldff1_gather_s64offset_s64, aarch64_sve_ldff1_gather, 16809988),
SVEMAP1(svldff1_gather_s64offset_u64, aarch64_sve_ldff1_gather, 16809988),
SVEMAP1(svldff1_gather_u32base_f32, aarch64_sve_ldff1_gather_scalar_offset, 32775),
SVEMAP1(svldff1_gather_u32base_index_f32, aarch64_sve_ldff1_gather_scalar_offset, 32775),
SVEMAP1(svldff1_gather_u32base_index_s32, aarch64_sve_ldff1_gather_scalar_offset, 32771),
SVEMAP1(svldff1_gather_u32base_index_u32, aarch64_sve_ldff1_gather_scalar_offset, 32771),
SVEMAP1(svldff1_gather_u32base_offset_f32, aarch64_sve_ldff1_gather_scalar_offset, 16809991),
SVEMAP1(svldff1_gather_u32base_offset_s32, aarch64_sve_ldff1_gather_scalar_offset, 16809987),
SVEMAP1(svldff1_gather_u32base_offset_u32, aarch64_sve_ldff1_gather_scalar_offset, 16809987),
SVEMAP1(svldff1_gather_u32base_s32, aarch64_sve_ldff1_gather_scalar_offset, 32771),
SVEMAP1(svldff1_gather_u32base_u32, aarch64_sve_ldff1_gather_scalar_offset, 32771),
SVEMAP1(svldff1_gather_u32index_f32, aarch64_sve_ldff1_gather_uxtw_index, 32775),
SVEMAP1(svldff1_gather_u32index_s32, aarch64_sve_ldff1_gather_uxtw_index, 32771),
SVEMAP1(svldff1_gather_u32index_u32, aarch64_sve_ldff1_gather_uxtw_index, 32771),
SVEMAP1(svldff1_gather_u32offset_f32, aarch64_sve_ldff1_gather_uxtw, 16809991),
SVEMAP1(svldff1_gather_u32offset_s32, aarch64_sve_ldff1_gather_uxtw, 16809987),
SVEMAP1(svldff1_gather_u32offset_u32, aarch64_sve_ldff1_gather_uxtw, 16809987),
SVEMAP1(svldff1_gather_u64base_f64, aarch64_sve_ldff1_gather_scalar_offset, 32776),
SVEMAP1(svldff1_gather_u64base_index_f64, aarch64_sve_ldff1_gather_scalar_offset, 32776),
SVEMAP1(svldff1_gather_u64base_index_s64, aarch64_sve_ldff1_gather_scalar_offset, 32772),
SVEMAP1(svldff1_gather_u64base_index_u64, aarch64_sve_ldff1_gather_scalar_offset, 32772),
SVEMAP1(svldff1_gather_u64base_offset_f64, aarch64_sve_ldff1_gather_scalar_offset, 16809992),
SVEMAP1(svldff1_gather_u64base_offset_s64, aarch64_sve_ldff1_gather_scalar_offset, 16809988),
SVEMAP1(svldff1_gather_u64base_offset_u64, aarch64_sve_ldff1_gather_scalar_offset, 16809988),
SVEMAP1(svldff1_gather_u64base_s64, aarch64_sve_ldff1_gather_scalar_offset, 32772),
SVEMAP1(svldff1_gather_u64base_u64, aarch64_sve_ldff1_gather_scalar_offset, 32772),
SVEMAP1(svldff1_gather_u64index_f64, aarch64_sve_ldff1_gather_index, 32776),
SVEMAP1(svldff1_gather_u64index_s64, aarch64_sve_ldff1_gather_index, 32772),
SVEMAP1(svldff1_gather_u64index_u64, aarch64_sve_ldff1_gather_index, 32772),
SVEMAP1(svldff1_gather_u64offset_f64, aarch64_sve_ldff1_gather, 16809992),
SVEMAP1(svldff1_gather_u64offset_s64, aarch64_sve_ldff1_gather, 16809988),
SVEMAP1(svldff1_gather_u64offset_u64, aarch64_sve_ldff1_gather, 16809988),
SVEMAP1(svldff1_mf8, aarch64_sve_ldff1, 8206),
SVEMAP1(svldff1_s16, aarch64_sve_ldff1, 8194),
SVEMAP1(svldff1_s32, aarch64_sve_ldff1, 8195),
SVEMAP1(svldff1_s64, aarch64_sve_ldff1, 8196),
SVEMAP1(svldff1_s8, aarch64_sve_ldff1, 8193),
SVEMAP1(svldff1_u16, aarch64_sve_ldff1, 8194),
SVEMAP1(svldff1_u32, aarch64_sve_ldff1, 8195),
SVEMAP1(svldff1_u64, aarch64_sve_ldff1, 8196),
SVEMAP1(svldff1_u8, aarch64_sve_ldff1, 8193),
SVEMAP1(svldff1_vnum_bf16, aarch64_sve_ldff1, 8205),
SVEMAP1(svldff1_vnum_f16, aarch64_sve_ldff1, 8198),
SVEMAP1(svldff1_vnum_f32, aarch64_sve_ldff1, 8199),
SVEMAP1(svldff1_vnum_f64, aarch64_sve_ldff1, 8200),
SVEMAP1(svldff1_vnum_mf8, aarch64_sve_ldff1, 8206),
SVEMAP1(svldff1_vnum_s16, aarch64_sve_ldff1, 8194),
SVEMAP1(svldff1_vnum_s32, aarch64_sve_ldff1, 8195),
SVEMAP1(svldff1_vnum_s64, aarch64_sve_ldff1, 8196),
SVEMAP1(svldff1_vnum_s8, aarch64_sve_ldff1, 8193),
SVEMAP1(svldff1_vnum_u16, aarch64_sve_ldff1, 8194),
SVEMAP1(svldff1_vnum_u32, aarch64_sve_ldff1, 8195),
SVEMAP1(svldff1_vnum_u64, aarch64_sve_ldff1, 8196),
SVEMAP1(svldff1_vnum_u8, aarch64_sve_ldff1, 8193),
SVEMAP1(svldff1sb_gather_s32offset_s32, aarch64_sve_ldff1_gather_sxtw, 16810003),
SVEMAP1(svldff1sb_gather_s32offset_u32, aarch64_sve_ldff1_gather_sxtw, 16810003),
SVEMAP1(svldff1sb_gather_s64offset_s64, aarch64_sve_ldff1_gather, 16810004),
SVEMAP1(svldff1sb_gather_s64offset_u64, aarch64_sve_ldff1_gather, 16810004),
SVEMAP1(svldff1sb_gather_u32base_offset_s32, aarch64_sve_ldff1_gather_scalar_offset, 16810003),
SVEMAP1(svldff1sb_gather_u32base_offset_u32, aarch64_sve_ldff1_gather_scalar_offset, 16810003),
SVEMAP1(svldff1sb_gather_u32base_s32, aarch64_sve_ldff1_gather_scalar_offset, 32787),
SVEMAP1(svldff1sb_gather_u32base_u32, aarch64_sve_ldff1_gather_scalar_offset, 32787),
SVEMAP1(svldff1sb_gather_u32offset_s32, aarch64_sve_ldff1_gather_uxtw, 16810003),
SVEMAP1(svldff1sb_gather_u32offset_u32, aarch64_sve_ldff1_gather_uxtw, 16810003),
SVEMAP1(svldff1sb_gather_u64base_offset_s64, aarch64_sve_ldff1_gather_scalar_offset, 16810004),
SVEMAP1(svldff1sb_gather_u64base_offset_u64, aarch64_sve_ldff1_gather_scalar_offset, 16810004),
SVEMAP1(svldff1sb_gather_u64base_s64, aarch64_sve_ldff1_gather_scalar_offset, 32788),
SVEMAP1(svldff1sb_gather_u64base_u64, aarch64_sve_ldff1_gather_scalar_offset, 32788),
SVEMAP1(svldff1sb_gather_u64offset_s64, aarch64_sve_ldff1_gather, 16810004),
SVEMAP1(svldff1sb_gather_u64offset_u64, aarch64_sve_ldff1_gather, 16810004),
SVEMAP1(svldff1sb_s16, aarch64_sve_ldff1, 8210),
SVEMAP1(svldff1sb_s32, aarch64_sve_ldff1, 8211),
SVEMAP1(svldff1sb_s64, aarch64_sve_ldff1, 8212),
SVEMAP1(svldff1sb_u16, aarch64_sve_ldff1, 8210),
SVEMAP1(svldff1sb_u32, aarch64_sve_ldff1, 8211),
SVEMAP1(svldff1sb_u64, aarch64_sve_ldff1, 8212),
SVEMAP1(svldff1sb_vnum_s16, aarch64_sve_ldff1, 8210),
SVEMAP1(svldff1sb_vnum_s32, aarch64_sve_ldff1, 8211),
SVEMAP1(svldff1sb_vnum_s64, aarch64_sve_ldff1, 8212),
SVEMAP1(svldff1sb_vnum_u16, aarch64_sve_ldff1, 8210),
SVEMAP1(svldff1sb_vnum_u32, aarch64_sve_ldff1, 8211),
SVEMAP1(svldff1sb_vnum_u64, aarch64_sve_ldff1, 8212),
SVEMAP1(svldff1sh_gather_s32index_s32, aarch64_sve_ldff1_gather_sxtw_index, 32803),
SVEMAP1(svldff1sh_gather_s32index_u32, aarch64_sve_ldff1_gather_sxtw_index, 32803),
SVEMAP1(svldff1sh_gather_s32offset_s32, aarch64_sve_ldff1_gather_sxtw, 16810019),
SVEMAP1(svldff1sh_gather_s32offset_u32, aarch64_sve_ldff1_gather_sxtw, 16810019),
SVEMAP1(svldff1sh_gather_s64index_s64, aarch64_sve_ldff1_gather_index, 32804),
SVEMAP1(svldff1sh_gather_s64index_u64, aarch64_sve_ldff1_gather_index, 32804),
SVEMAP1(svldff1sh_gather_s64offset_s64, aarch64_sve_ldff1_gather, 16810020),
SVEMAP1(svldff1sh_gather_s64offset_u64, aarch64_sve_ldff1_gather, 16810020),
SVEMAP1(svldff1sh_gather_u32base_index_s32, aarch64_sve_ldff1_gather_scalar_offset, 32803),
SVEMAP1(svldff1sh_gather_u32base_index_u32, aarch64_sve_ldff1_gather_scalar_offset, 32803),
SVEMAP1(svldff1sh_gather_u32base_offset_s32, aarch64_sve_ldff1_gather_scalar_offset, 16810019),
SVEMAP1(svldff1sh_gather_u32base_offset_u32, aarch64_sve_ldff1_gather_scalar_offset, 16810019),
SVEMAP1(svldff1sh_gather_u32base_s32, aarch64_sve_ldff1_gather_scalar_offset, 32803),
SVEMAP1(svldff1sh_gather_u32base_u32, aarch64_sve_ldff1_gather_scalar_offset, 32803),
SVEMAP1(svldff1sh_gather_u32index_s32, aarch64_sve_ldff1_gather_uxtw_index, 32803),
SVEMAP1(svldff1sh_gather_u32index_u32, aarch64_sve_ldff1_gather_uxtw_index, 32803),
SVEMAP1(svldff1sh_gather_u32offset_s32, aarch64_sve_ldff1_gather_uxtw, 16810019),
SVEMAP1(svldff1sh_gather_u32offset_u32, aarch64_sve_ldff1_gather_uxtw, 16810019),
SVEMAP1(svldff1sh_gather_u64base_index_s64, aarch64_sve_ldff1_gather_scalar_offset, 32804),
SVEMAP1(svldff1sh_gather_u64base_index_u64, aarch64_sve_ldff1_gather_scalar_offset, 32804),
SVEMAP1(svldff1sh_gather_u64base_offset_s64, aarch64_sve_ldff1_gather_scalar_offset, 16810020),
SVEMAP1(svldff1sh_gather_u64base_offset_u64, aarch64_sve_ldff1_gather_scalar_offset, 16810020),
SVEMAP1(svldff1sh_gather_u64base_s64, aarch64_sve_ldff1_gather_scalar_offset, 32804),
SVEMAP1(svldff1sh_gather_u64base_u64, aarch64_sve_ldff1_gather_scalar_offset, 32804),
SVEMAP1(svldff1sh_gather_u64index_s64, aarch64_sve_ldff1_gather_index, 32804),
SVEMAP1(svldff1sh_gather_u64index_u64, aarch64_sve_ldff1_gather_index, 32804),
SVEMAP1(svldff1sh_gather_u64offset_s64, aarch64_sve_ldff1_gather, 16810020),
SVEMAP1(svldff1sh_gather_u64offset_u64, aarch64_sve_ldff1_gather, 16810020),
SVEMAP1(svldff1sh_s32, aarch64_sve_ldff1, 8227),
SVEMAP1(svldff1sh_s64, aarch64_sve_ldff1, 8228),
SVEMAP1(svldff1sh_u32, aarch64_sve_ldff1, 8227),
SVEMAP1(svldff1sh_u64, aarch64_sve_ldff1, 8228),
SVEMAP1(svldff1sh_vnum_s32, aarch64_sve_ldff1, 8227),
SVEMAP1(svldff1sh_vnum_s64, aarch64_sve_ldff1, 8228),
SVEMAP1(svldff1sh_vnum_u32, aarch64_sve_ldff1, 8227),
SVEMAP1(svldff1sh_vnum_u64, aarch64_sve_ldff1, 8228),
SVEMAP1(svldff1sw_gather_s64index_s64, aarch64_sve_ldff1_gather_index, 32820),
SVEMAP1(svldff1sw_gather_s64index_u64, aarch64_sve_ldff1_gather_index, 32820),
SVEMAP1(svldff1sw_gather_s64offset_s64, aarch64_sve_ldff1_gather, 16810036),
SVEMAP1(svldff1sw_gather_s64offset_u64, aarch64_sve_ldff1_gather, 16810036),
SVEMAP1(svldff1sw_gather_u64base_index_s64, aarch64_sve_ldff1_gather_scalar_offset, 32820),
SVEMAP1(svldff1sw_gather_u64base_index_u64, aarch64_sve_ldff1_gather_scalar_offset, 32820),
SVEMAP1(svldff1sw_gather_u64base_offset_s64, aarch64_sve_ldff1_gather_scalar_offset, 16810036),
SVEMAP1(svldff1sw_gather_u64base_offset_u64, aarch64_sve_ldff1_gather_scalar_offset, 16810036),
SVEMAP1(svldff1sw_gather_u64base_s64, aarch64_sve_ldff1_gather_scalar_offset, 32820),
SVEMAP1(svldff1sw_gather_u64base_u64, aarch64_sve_ldff1_gather_scalar_offset, 32820),
SVEMAP1(svldff1sw_gather_u64index_s64, aarch64_sve_ldff1_gather_index, 32820),
SVEMAP1(svldff1sw_gather_u64index_u64, aarch64_sve_ldff1_gather_index, 32820),
SVEMAP1(svldff1sw_gather_u64offset_s64, aarch64_sve_ldff1_gather, 16810036),
SVEMAP1(svldff1sw_gather_u64offset_u64, aarch64_sve_ldff1_gather, 16810036),
SVEMAP1(svldff1sw_s64, aarch64_sve_ldff1, 8244),
SVEMAP1(svldff1sw_u64, aarch64_sve_ldff1, 8244),
SVEMAP1(svldff1sw_vnum_s64, aarch64_sve_ldff1, 8244),
SVEMAP1(svldff1sw_vnum_u64, aarch64_sve_ldff1, 8244),
SVEMAP1(svldff1ub_gather_s32offset_s32, aarch64_sve_ldff1_gather_sxtw, 17334291),
SVEMAP1(svldff1ub_gather_s32offset_u32, aarch64_sve_ldff1_gather_sxtw, 17334291),
SVEMAP1(svldff1ub_gather_s64offset_s64, aarch64_sve_ldff1_gather, 17334292),
SVEMAP1(svldff1ub_gather_s64offset_u64, aarch64_sve_ldff1_gather, 17334292),
SVEMAP1(svldff1ub_gather_u32base_offset_s32, aarch64_sve_ldff1_gather_scalar_offset, 17334291),
SVEMAP1(svldff1ub_gather_u32base_offset_u32, aarch64_sve_ldff1_gather_scalar_offset, 17334291),
SVEMAP1(svldff1ub_gather_u32base_s32, aarch64_sve_ldff1_gather_scalar_offset, 557075),
SVEMAP1(svldff1ub_gather_u32base_u32, aarch64_sve_ldff1_gather_scalar_offset, 557075),
SVEMAP1(svldff1ub_gather_u32offset_s32, aarch64_sve_ldff1_gather_uxtw, 17334291),
SVEMAP1(svldff1ub_gather_u32offset_u32, aarch64_sve_ldff1_gather_uxtw, 17334291),
SVEMAP1(svldff1ub_gather_u64base_offset_s64, aarch64_sve_ldff1_gather_scalar_offset, 17334292),
SVEMAP1(svldff1ub_gather_u64base_offset_u64, aarch64_sve_ldff1_gather_scalar_offset, 17334292),
SVEMAP1(svldff1ub_gather_u64base_s64, aarch64_sve_ldff1_gather_scalar_offset, 557076),
SVEMAP1(svldff1ub_gather_u64base_u64, aarch64_sve_ldff1_gather_scalar_offset, 557076),
SVEMAP1(svldff1ub_gather_u64offset_s64, aarch64_sve_ldff1_gather, 17334292),
SVEMAP1(svldff1ub_gather_u64offset_u64, aarch64_sve_ldff1_gather, 17334292),
SVEMAP1(svldff1ub_s16, aarch64_sve_ldff1, 532498),
SVEMAP1(svldff1ub_s32, aarch64_sve_ldff1, 532499),
SVEMAP1(svldff1ub_s64, aarch64_sve_ldff1, 532500),
SVEMAP1(svldff1ub_u16, aarch64_sve_ldff1, 532498),
SVEMAP1(svldff1ub_u32, aarch64_sve_ldff1, 532499),
SVEMAP1(svldff1ub_u64, aarch64_sve_ldff1, 532500),
SVEMAP1(svldff1ub_vnum_s16, aarch64_sve_ldff1, 532498),
SVEMAP1(svldff1ub_vnum_s32, aarch64_sve_ldff1, 532499),
SVEMAP1(svldff1ub_vnum_s64, aarch64_sve_ldff1, 532500),
SVEMAP1(svldff1ub_vnum_u16, aarch64_sve_ldff1, 532498),
SVEMAP1(svldff1ub_vnum_u32, aarch64_sve_ldff1, 532499),
SVEMAP1(svldff1ub_vnum_u64, aarch64_sve_ldff1, 532500),
SVEMAP1(svldff1uh_gather_s32index_s32, aarch64_sve_ldff1_gather_sxtw_index, 557091),
SVEMAP1(svldff1uh_gather_s32index_u32, aarch64_sve_ldff1_gather_sxtw_index, 557091),
SVEMAP1(svldff1uh_gather_s32offset_s32, aarch64_sve_ldff1_gather_sxtw, 17334307),
SVEMAP1(svldff1uh_gather_s32offset_u32, aarch64_sve_ldff1_gather_sxtw, 17334307),
SVEMAP1(svldff1uh_gather_s64index_s64, aarch64_sve_ldff1_gather_index, 557092),
SVEMAP1(svldff1uh_gather_s64index_u64, aarch64_sve_ldff1_gather_index, 557092),
SVEMAP1(svldff1uh_gather_s64offset_s64, aarch64_sve_ldff1_gather, 17334308),
SVEMAP1(svldff1uh_gather_s64offset_u64, aarch64_sve_ldff1_gather, 17334308),
SVEMAP1(svldff1uh_gather_u32base_index_s32, aarch64_sve_ldff1_gather_scalar_offset, 557091),
SVEMAP1(svldff1uh_gather_u32base_index_u32, aarch64_sve_ldff1_gather_scalar_offset, 557091),
SVEMAP1(svldff1uh_gather_u32base_offset_s32, aarch64_sve_ldff1_gather_scalar_offset, 17334307),
SVEMAP1(svldff1uh_gather_u32base_offset_u32, aarch64_sve_ldff1_gather_scalar_offset, 17334307),
SVEMAP1(svldff1uh_gather_u32base_s32, aarch64_sve_ldff1_gather_scalar_offset, 557091),
SVEMAP1(svldff1uh_gather_u32base_u32, aarch64_sve_ldff1_gather_scalar_offset, 557091),
SVEMAP1(svldff1uh_gather_u32index_s32, aarch64_sve_ldff1_gather_uxtw_index, 557091),
SVEMAP1(svldff1uh_gather_u32index_u32, aarch64_sve_ldff1_gather_uxtw_index, 557091),
SVEMAP1(svldff1uh_gather_u32offset_s32, aarch64_sve_ldff1_gather_uxtw, 17334307),
SVEMAP1(svldff1uh_gather_u32offset_u32, aarch64_sve_ldff1_gather_uxtw, 17334307),
SVEMAP1(svldff1uh_gather_u64base_index_s64, aarch64_sve_ldff1_gather_scalar_offset, 557092),
SVEMAP1(svldff1uh_gather_u64base_index_u64, aarch64_sve_ldff1_gather_scalar_offset, 557092),
SVEMAP1(svldff1uh_gather_u64base_offset_s64, aarch64_sve_ldff1_gather_scalar_offset, 17334308),
SVEMAP1(svldff1uh_gather_u64base_offset_u64, aarch64_sve_ldff1_gather_scalar_offset, 17334308),
SVEMAP1(svldff1uh_gather_u64base_s64, aarch64_sve_ldff1_gather_scalar_offset, 557092),
SVEMAP1(svldff1uh_gather_u64base_u64, aarch64_sve_ldff1_gather_scalar_offset, 557092),
SVEMAP1(svldff1uh_gather_u64index_s64, aarch64_sve_ldff1_gather_index, 557092),
SVEMAP1(svldff1uh_gather_u64index_u64, aarch64_sve_ldff1_gather_index, 557092),
SVEMAP1(svldff1uh_gather_u64offset_s64, aarch64_sve_ldff1_gather, 17334308),
SVEMAP1(svldff1uh_gather_u64offset_u64, aarch64_sve_ldff1_gather, 17334308),
SVEMAP1(svldff1uh_s32, aarch64_sve_ldff1, 532515),
SVEMAP1(svldff1uh_s64, aarch64_sve_ldff1, 532516),
SVEMAP1(svldff1uh_u32, aarch64_sve_ldff1, 532515),
SVEMAP1(svldff1uh_u64, aarch64_sve_ldff1, 532516),
SVEMAP1(svldff1uh_vnum_s32, aarch64_sve_ldff1, 532515),
SVEMAP1(svldff1uh_vnum_s64, aarch64_sve_ldff1, 532516),
SVEMAP1(svldff1uh_vnum_u32, aarch64_sve_ldff1, 532515),
SVEMAP1(svldff1uh_vnum_u64, aarch64_sve_ldff1, 532516),
SVEMAP1(svldff1uw_gather_s64index_s64, aarch64_sve_ldff1_gather_index, 557108),
SVEMAP1(svldff1uw_gather_s64index_u64, aarch64_sve_ldff1_gather_index, 557108),
SVEMAP1(svldff1uw_gather_s64offset_s64, aarch64_sve_ldff1_gather, 17334324),
SVEMAP1(svldff1uw_gather_s64offset_u64, aarch64_sve_ldff1_gather, 17334324),
SVEMAP1(svldff1uw_gather_u64base_index_s64, aarch64_sve_ldff1_gather_scalar_offset, 557108),
SVEMAP1(svldff1uw_gather_u64base_index_u64, aarch64_sve_ldff1_gather_scalar_offset, 557108),
SVEMAP1(svldff1uw_gather_u64base_offset_s64, aarch64_sve_ldff1_gather_scalar_offset, 17334324),
SVEMAP1(svldff1uw_gather_u64base_offset_u64, aarch64_sve_ldff1_gather_scalar_offset, 17334324),
SVEMAP1(svldff1uw_gather_u64base_s64, aarch64_sve_ldff1_gather_scalar_offset, 557108),
SVEMAP1(svldff1uw_gather_u64base_u64, aarch64_sve_ldff1_gather_scalar_offset, 557108),
SVEMAP1(svldff1uw_gather_u64index_s64, aarch64_sve_ldff1_gather_index, 557108),
SVEMAP1(svldff1uw_gather_u64index_u64, aarch64_sve_ldff1_gather_index, 557108),
SVEMAP1(svldff1uw_gather_u64offset_s64, aarch64_sve_ldff1_gather, 17334324),
SVEMAP1(svldff1uw_gather_u64offset_u64, aarch64_sve_ldff1_gather, 17334324),
SVEMAP1(svldff1uw_s64, aarch64_sve_ldff1, 532532),
SVEMAP1(svldff1uw_u64, aarch64_sve_ldff1, 532532),
SVEMAP1(svldff1uw_vnum_s64, aarch64_sve_ldff1, 532532),
SVEMAP1(svldff1uw_vnum_u64, aarch64_sve_ldff1, 532532),
SVEMAP1(svldnf1_bf16, aarch64_sve_ldnf1, 8205),
SVEMAP1(svldnf1_f16, aarch64_sve_ldnf1, 8198),
SVEMAP1(svldnf1_f32, aarch64_sve_ldnf1, 8199),
SVEMAP1(svldnf1_f64, aarch64_sve_ldnf1, 8200),
SVEMAP1(svldnf1_mf8, aarch64_sve_ldnf1, 8206),
SVEMAP1(svldnf1_s16, aarch64_sve_ldnf1, 8194),
SVEMAP1(svldnf1_s32, aarch64_sve_ldnf1, 8195),
SVEMAP1(svldnf1_s64, aarch64_sve_ldnf1, 8196),
SVEMAP1(svldnf1_s8, aarch64_sve_ldnf1, 8193),
SVEMAP1(svldnf1_u16, aarch64_sve_ldnf1, 8194),
SVEMAP1(svldnf1_u32, aarch64_sve_ldnf1, 8195),
SVEMAP1(svldnf1_u64, aarch64_sve_ldnf1, 8196),
SVEMAP1(svldnf1_u8, aarch64_sve_ldnf1, 8193),
SVEMAP1(svldnf1_vnum_bf16, aarch64_sve_ldnf1, 8205),
SVEMAP1(svldnf1_vnum_f16, aarch64_sve_ldnf1, 8198),
SVEMAP1(svldnf1_vnum_f32, aarch64_sve_ldnf1, 8199),
SVEMAP1(svldnf1_vnum_f64, aarch64_sve_ldnf1, 8200),
SVEMAP1(svldnf1_vnum_mf8, aarch64_sve_ldnf1, 8206),
SVEMAP1(svldnf1_vnum_s16, aarch64_sve_ldnf1, 8194),
SVEMAP1(svldnf1_vnum_s32, aarch64_sve_ldnf1, 8195),
SVEMAP1(svldnf1_vnum_s64, aarch64_sve_ldnf1, 8196),
SVEMAP1(svldnf1_vnum_s8, aarch64_sve_ldnf1, 8193),
SVEMAP1(svldnf1_vnum_u16, aarch64_sve_ldnf1, 8194),
SVEMAP1(svldnf1_vnum_u32, aarch64_sve_ldnf1, 8195),
SVEMAP1(svldnf1_vnum_u64, aarch64_sve_ldnf1, 8196),
SVEMAP1(svldnf1_vnum_u8, aarch64_sve_ldnf1, 8193),
SVEMAP1(svldnf1sb_s16, aarch64_sve_ldnf1, 8210),
SVEMAP1(svldnf1sb_s32, aarch64_sve_ldnf1, 8211),
SVEMAP1(svldnf1sb_s64, aarch64_sve_ldnf1, 8212),
SVEMAP1(svldnf1sb_u16, aarch64_sve_ldnf1, 8210),
SVEMAP1(svldnf1sb_u32, aarch64_sve_ldnf1, 8211),
SVEMAP1(svldnf1sb_u64, aarch64_sve_ldnf1, 8212),
SVEMAP1(svldnf1sb_vnum_s16, aarch64_sve_ldnf1, 8210),
SVEMAP1(svldnf1sb_vnum_s32, aarch64_sve_ldnf1, 8211),
SVEMAP1(svldnf1sb_vnum_s64, aarch64_sve_ldnf1, 8212),
SVEMAP1(svldnf1sb_vnum_u16, aarch64_sve_ldnf1, 8210),
SVEMAP1(svldnf1sb_vnum_u32, aarch64_sve_ldnf1, 8211),
SVEMAP1(svldnf1sb_vnum_u64, aarch64_sve_ldnf1, 8212),
SVEMAP1(svldnf1sh_s32, aarch64_sve_ldnf1, 8227),
SVEMAP1(svldnf1sh_s64, aarch64_sve_ldnf1, 8228),
SVEMAP1(svldnf1sh_u32, aarch64_sve_ldnf1, 8227),
SVEMAP1(svldnf1sh_u64, aarch64_sve_ldnf1, 8228),
SVEMAP1(svldnf1sh_vnum_s32, aarch64_sve_ldnf1, 8227),
SVEMAP1(svldnf1sh_vnum_s64, aarch64_sve_ldnf1, 8228),
SVEMAP1(svldnf1sh_vnum_u32, aarch64_sve_ldnf1, 8227),
SVEMAP1(svldnf1sh_vnum_u64, aarch64_sve_ldnf1, 8228),
SVEMAP1(svldnf1sw_s64, aarch64_sve_ldnf1, 8244),
SVEMAP1(svldnf1sw_u64, aarch64_sve_ldnf1, 8244),
SVEMAP1(svldnf1sw_vnum_s64, aarch64_sve_ldnf1, 8244),
SVEMAP1(svldnf1sw_vnum_u64, aarch64_sve_ldnf1, 8244),
SVEMAP1(svldnf1ub_s16, aarch64_sve_ldnf1, 532498),
SVEMAP1(svldnf1ub_s32, aarch64_sve_ldnf1, 532499),
SVEMAP1(svldnf1ub_s64, aarch64_sve_ldnf1, 532500),
SVEMAP1(svldnf1ub_u16, aarch64_sve_ldnf1, 532498),
SVEMAP1(svldnf1ub_u32, aarch64_sve_ldnf1, 532499),
SVEMAP1(svldnf1ub_u64, aarch64_sve_ldnf1, 532500),
SVEMAP1(svldnf1ub_vnum_s16, aarch64_sve_ldnf1, 532498),
SVEMAP1(svldnf1ub_vnum_s32, aarch64_sve_ldnf1, 532499),
SVEMAP1(svldnf1ub_vnum_s64, aarch64_sve_ldnf1, 532500),
SVEMAP1(svldnf1ub_vnum_u16, aarch64_sve_ldnf1, 532498),
SVEMAP1(svldnf1ub_vnum_u32, aarch64_sve_ldnf1, 532499),
SVEMAP1(svldnf1ub_vnum_u64, aarch64_sve_ldnf1, 532500),
SVEMAP1(svldnf1uh_s32, aarch64_sve_ldnf1, 532515),
SVEMAP1(svldnf1uh_s64, aarch64_sve_ldnf1, 532516),
SVEMAP1(svldnf1uh_u32, aarch64_sve_ldnf1, 532515),
SVEMAP1(svldnf1uh_u64, aarch64_sve_ldnf1, 532516),
SVEMAP1(svldnf1uh_vnum_s32, aarch64_sve_ldnf1, 532515),
SVEMAP1(svldnf1uh_vnum_s64, aarch64_sve_ldnf1, 532516),
SVEMAP1(svldnf1uh_vnum_u32, aarch64_sve_ldnf1, 532515),
SVEMAP1(svldnf1uh_vnum_u64, aarch64_sve_ldnf1, 532516),
SVEMAP1(svldnf1uw_s64, aarch64_sve_ldnf1, 532532),
SVEMAP1(svldnf1uw_u64, aarch64_sve_ldnf1, 532532),
SVEMAP1(svldnf1uw_vnum_s64, aarch64_sve_ldnf1, 532532),
SVEMAP1(svldnf1uw_vnum_u64, aarch64_sve_ldnf1, 532532),
SVEMAP1(svldnt1_bf16, aarch64_sve_ldnt1, 4398046519309),
SVEMAP1(svldnt1_bf16_x2, aarch64_sve_ldnt1_pn_x2, 4398046642189),
SVEMAP1(svldnt1_bf16_x4, aarch64_sve_ldnt1_pn_x4, 4398046642189),
SVEMAP1(svldnt1_f16, aarch64_sve_ldnt1, 4398046519302),
SVEMAP1(svldnt1_f16_x2, aarch64_sve_ldnt1_pn_x2, 4398046642182),
SVEMAP1(svldnt1_f16_x4, aarch64_sve_ldnt1_pn_x4, 4398046642182),
SVEMAP1(svldnt1_f32, aarch64_sve_ldnt1, 4398046519303),
SVEMAP1(svldnt1_f32_x2, aarch64_sve_ldnt1_pn_x2, 4398046642183),
SVEMAP1(svldnt1_f32_x4, aarch64_sve_ldnt1_pn_x4, 4398046642183),
SVEMAP1(svldnt1_f64, aarch64_sve_ldnt1, 4398046519304),
SVEMAP1(svldnt1_f64_x2, aarch64_sve_ldnt1_pn_x2, 4398046642184),
SVEMAP1(svldnt1_f64_x4, aarch64_sve_ldnt1_pn_x4, 4398046642184),
SVEMAP1(svldnt1_gather_s64index_f64, aarch64_sve_ldnt1_gather_index, 32776),
SVEMAP1(svldnt1_gather_s64index_s64, aarch64_sve_ldnt1_gather_index, 32772),
SVEMAP1(svldnt1_gather_s64index_u64, aarch64_sve_ldnt1_gather_index, 32772),
SVEMAP1(svldnt1_gather_s64offset_f64, aarch64_sve_ldnt1_gather, 16809992),
SVEMAP1(svldnt1_gather_s64offset_s64, aarch64_sve_ldnt1_gather, 16809988),
SVEMAP1(svldnt1_gather_s64offset_u64, aarch64_sve_ldnt1_gather, 16809988),
SVEMAP1(svldnt1_gather_u32base_f32, aarch64_sve_ldnt1_gather_scalar_offset, 32775),
SVEMAP1(svldnt1_gather_u32base_index_f32, aarch64_sve_ldnt1_gather_scalar_offset, 32775),
SVEMAP1(svldnt1_gather_u32base_index_s32, aarch64_sve_ldnt1_gather_scalar_offset, 32771),
SVEMAP1(svldnt1_gather_u32base_index_u32, aarch64_sve_ldnt1_gather_scalar_offset, 32771),
SVEMAP1(svldnt1_gather_u32base_offset_f32, aarch64_sve_ldnt1_gather_scalar_offset, 16809991),
SVEMAP1(svldnt1_gather_u32base_offset_s32, aarch64_sve_ldnt1_gather_scalar_offset, 16809987),
SVEMAP1(svldnt1_gather_u32base_offset_u32, aarch64_sve_ldnt1_gather_scalar_offset, 16809987),
SVEMAP1(svldnt1_gather_u32base_s32, aarch64_sve_ldnt1_gather_scalar_offset, 32771),
SVEMAP1(svldnt1_gather_u32base_u32, aarch64_sve_ldnt1_gather_scalar_offset, 32771),
SVEMAP1(svldnt1_gather_u32offset_f32, aarch64_sve_ldnt1_gather_uxtw, 16809991),
SVEMAP1(svldnt1_gather_u32offset_s32, aarch64_sve_ldnt1_gather_uxtw, 16809987),
SVEMAP1(svldnt1_gather_u32offset_u32, aarch64_sve_ldnt1_gather_uxtw, 16809987),
SVEMAP1(svldnt1_gather_u64base_f64, aarch64_sve_ldnt1_gather_scalar_offset, 32776),
SVEMAP1(svldnt1_gather_u64base_index_f64, aarch64_sve_ldnt1_gather_scalar_offset, 32776),
SVEMAP1(svldnt1_gather_u64base_index_s64, aarch64_sve_ldnt1_gather_scalar_offset, 32772),
SVEMAP1(svldnt1_gather_u64base_index_u64, aarch64_sve_ldnt1_gather_scalar_offset, 32772),
SVEMAP1(svldnt1_gather_u64base_offset_f64, aarch64_sve_ldnt1_gather_scalar_offset, 16809992),
SVEMAP1(svldnt1_gather_u64base_offset_s64, aarch64_sve_ldnt1_gather_scalar_offset, 16809988),
SVEMAP1(svldnt1_gather_u64base_offset_u64, aarch64_sve_ldnt1_gather_scalar_offset, 16809988),
SVEMAP1(svldnt1_gather_u64base_s64, aarch64_sve_ldnt1_gather_scalar_offset, 32772),
SVEMAP1(svldnt1_gather_u64base_u64, aarch64_sve_ldnt1_gather_scalar_offset, 32772),
SVEMAP1(svldnt1_gather_u64index_f64, aarch64_sve_ldnt1_gather_index, 32776),
SVEMAP1(svldnt1_gather_u64index_s64, aarch64_sve_ldnt1_gather_index, 32772),
SVEMAP1(svldnt1_gather_u64index_u64, aarch64_sve_ldnt1_gather_index, 32772),
SVEMAP1(svldnt1_gather_u64offset_f64, aarch64_sve_ldnt1_gather, 16809992),
SVEMAP1(svldnt1_gather_u64offset_s64, aarch64_sve_ldnt1_gather, 16809988),
SVEMAP1(svldnt1_gather_u64offset_u64, aarch64_sve_ldnt1_gather, 16809988),
SVEMAP1(svldnt1_mf8, aarch64_sve_ldnt1, 4398046519310),
SVEMAP1(svldnt1_mf8_x2, aarch64_sve_ldnt1_pn_x2, 4398046642190),
SVEMAP1(svldnt1_mf8_x4, aarch64_sve_ldnt1_pn_x4, 4398046642190),
SVEMAP1(svldnt1_s16, aarch64_sve_ldnt1, 4398046519298),
SVEMAP1(svldnt1_s16_x2, aarch64_sve_ldnt1_pn_x2, 4398046642178),
SVEMAP1(svldnt1_s16_x4, aarch64_sve_ldnt1_pn_x4, 4398046642178),
SVEMAP1(svldnt1_s32, aarch64_sve_ldnt1, 4398046519299),
SVEMAP1(svldnt1_s32_x2, aarch64_sve_ldnt1_pn_x2, 4398046642179),
SVEMAP1(svldnt1_s32_x4, aarch64_sve_ldnt1_pn_x4, 4398046642179),
SVEMAP1(svldnt1_s64, aarch64_sve_ldnt1, 4398046519300),
SVEMAP1(svldnt1_s64_x2, aarch64_sve_ldnt1_pn_x2, 4398046642180),
SVEMAP1(svldnt1_s64_x4, aarch64_sve_ldnt1_pn_x4, 4398046642180),
SVEMAP1(svldnt1_s8, aarch64_sve_ldnt1, 4398046519297),
SVEMAP1(svldnt1_s8_x2, aarch64_sve_ldnt1_pn_x2, 4398046642177),
SVEMAP1(svldnt1_s8_x4, aarch64_sve_ldnt1_pn_x4, 4398046642177),
SVEMAP1(svldnt1_u16, aarch64_sve_ldnt1, 4398046519298),
SVEMAP1(svldnt1_u16_x2, aarch64_sve_ldnt1_pn_x2, 4398046642178),
SVEMAP1(svldnt1_u16_x4, aarch64_sve_ldnt1_pn_x4, 4398046642178),
SVEMAP1(svldnt1_u32, aarch64_sve_ldnt1, 4398046519299),
SVEMAP1(svldnt1_u32_x2, aarch64_sve_ldnt1_pn_x2, 4398046642179),
SVEMAP1(svldnt1_u32_x4, aarch64_sve_ldnt1_pn_x4, 4398046642179),
SVEMAP1(svldnt1_u64, aarch64_sve_ldnt1, 4398046519300),
SVEMAP1(svldnt1_u64_x2, aarch64_sve_ldnt1_pn_x2, 4398046642180),
SVEMAP1(svldnt1_u64_x4, aarch64_sve_ldnt1_pn_x4, 4398046642180),
SVEMAP1(svldnt1_u8, aarch64_sve_ldnt1, 4398046519297),
SVEMAP1(svldnt1_u8_x2, aarch64_sve_ldnt1_pn_x2, 4398046642177),
SVEMAP1(svldnt1_u8_x4, aarch64_sve_ldnt1_pn_x4, 4398046642177),
SVEMAP1(svldnt1_vnum_bf16, aarch64_sve_ldnt1, 4398046519309),
SVEMAP1(svldnt1_vnum_bf16_x2, aarch64_sve_ldnt1_pn_x2, 4398046642189),
SVEMAP1(svldnt1_vnum_bf16_x4, aarch64_sve_ldnt1_pn_x4, 4398046642189),
SVEMAP1(svldnt1_vnum_f16, aarch64_sve_ldnt1, 4398046519302),
SVEMAP1(svldnt1_vnum_f16_x2, aarch64_sve_ldnt1_pn_x2, 4398046642182),
SVEMAP1(svldnt1_vnum_f16_x4, aarch64_sve_ldnt1_pn_x4, 4398046642182),
SVEMAP1(svldnt1_vnum_f32, aarch64_sve_ldnt1, 4398046519303),
SVEMAP1(svldnt1_vnum_f32_x2, aarch64_sve_ldnt1_pn_x2, 4398046642183),
SVEMAP1(svldnt1_vnum_f32_x4, aarch64_sve_ldnt1_pn_x4, 4398046642183),
SVEMAP1(svldnt1_vnum_f64, aarch64_sve_ldnt1, 4398046519304),
SVEMAP1(svldnt1_vnum_f64_x2, aarch64_sve_ldnt1_pn_x2, 4398046642184),
SVEMAP1(svldnt1_vnum_f64_x4, aarch64_sve_ldnt1_pn_x4, 4398046642184),
SVEMAP1(svldnt1_vnum_mf8, aarch64_sve_ldnt1, 4398046519310),
SVEMAP1(svldnt1_vnum_mf8_x2, aarch64_sve_ldnt1_pn_x2, 4398046642190),
SVEMAP1(svldnt1_vnum_mf8_x4, aarch64_sve_ldnt1_pn_x4, 4398046642190),
SVEMAP1(svldnt1_vnum_s16, aarch64_sve_ldnt1, 4398046519298),
SVEMAP1(svldnt1_vnum_s16_x2, aarch64_sve_ldnt1_pn_x2, 4398046642178),
SVEMAP1(svldnt1_vnum_s16_x4, aarch64_sve_ldnt1_pn_x4, 4398046642178),
SVEMAP1(svldnt1_vnum_s32, aarch64_sve_ldnt1, 4398046519299),
SVEMAP1(svldnt1_vnum_s32_x2, aarch64_sve_ldnt1_pn_x2, 4398046642179),
SVEMAP1(svldnt1_vnum_s32_x4, aarch64_sve_ldnt1_pn_x4, 4398046642179),
SVEMAP1(svldnt1_vnum_s64, aarch64_sve_ldnt1, 4398046519300),
SVEMAP1(svldnt1_vnum_s64_x2, aarch64_sve_ldnt1_pn_x2, 4398046642180),
SVEMAP1(svldnt1_vnum_s64_x4, aarch64_sve_ldnt1_pn_x4, 4398046642180),
SVEMAP1(svldnt1_vnum_s8, aarch64_sve_ldnt1, 4398046519297),
SVEMAP1(svldnt1_vnum_s8_x2, aarch64_sve_ldnt1_pn_x2, 4398046642177),
SVEMAP1(svldnt1_vnum_s8_x4, aarch64_sve_ldnt1_pn_x4, 4398046642177),
SVEMAP1(svldnt1_vnum_u16, aarch64_sve_ldnt1, 4398046519298),
SVEMAP1(svldnt1_vnum_u16_x2, aarch64_sve_ldnt1_pn_x2, 4398046642178),
SVEMAP1(svldnt1_vnum_u16_x4, aarch64_sve_ldnt1_pn_x4, 4398046642178),
SVEMAP1(svldnt1_vnum_u32, aarch64_sve_ldnt1, 4398046519299),
SVEMAP1(svldnt1_vnum_u32_x2, aarch64_sve_ldnt1_pn_x2, 4398046642179),
SVEMAP1(svldnt1_vnum_u32_x4, aarch64_sve_ldnt1_pn_x4, 4398046642179),
SVEMAP1(svldnt1_vnum_u64, aarch64_sve_ldnt1, 4398046519300),
SVEMAP1(svldnt1_vnum_u64_x2, aarch64_sve_ldnt1_pn_x2, 4398046642180),
SVEMAP1(svldnt1_vnum_u64_x4, aarch64_sve_ldnt1_pn_x4, 4398046642180),
SVEMAP1(svldnt1_vnum_u8, aarch64_sve_ldnt1, 4398046519297),
SVEMAP1(svldnt1_vnum_u8_x2, aarch64_sve_ldnt1_pn_x2, 4398046642177),
SVEMAP1(svldnt1_vnum_u8_x4, aarch64_sve_ldnt1_pn_x4, 4398046642177),
SVEMAP1(svldnt1sb_gather_s64offset_s64, aarch64_sve_ldnt1_gather, 16810004),
SVEMAP1(svldnt1sb_gather_s64offset_u64, aarch64_sve_ldnt1_gather, 16810004),
SVEMAP1(svldnt1sb_gather_u32base_offset_s32, aarch64_sve_ldnt1_gather_scalar_offset, 16810003),
SVEMAP1(svldnt1sb_gather_u32base_offset_u32, aarch64_sve_ldnt1_gather_scalar_offset, 16810003),
SVEMAP1(svldnt1sb_gather_u32base_s32, aarch64_sve_ldnt1_gather_scalar_offset, 32787),
SVEMAP1(svldnt1sb_gather_u32base_u32, aarch64_sve_ldnt1_gather_scalar_offset, 32787),
SVEMAP1(svldnt1sb_gather_u32offset_s32, aarch64_sve_ldnt1_gather_uxtw, 16810003),
SVEMAP1(svldnt1sb_gather_u32offset_u32, aarch64_sve_ldnt1_gather_uxtw, 16810003),
SVEMAP1(svldnt1sb_gather_u64base_offset_s64, aarch64_sve_ldnt1_gather_scalar_offset, 16810004),
SVEMAP1(svldnt1sb_gather_u64base_offset_u64, aarch64_sve_ldnt1_gather_scalar_offset, 16810004),
SVEMAP1(svldnt1sb_gather_u64base_s64, aarch64_sve_ldnt1_gather_scalar_offset, 32788),
SVEMAP1(svldnt1sb_gather_u64base_u64, aarch64_sve_ldnt1_gather_scalar_offset, 32788),
SVEMAP1(svldnt1sb_gather_u64offset_s64, aarch64_sve_ldnt1_gather, 16810004),
SVEMAP1(svldnt1sb_gather_u64offset_u64, aarch64_sve_ldnt1_gather, 16810004),
SVEMAP1(svldnt1sh_gather_s64index_s64, aarch64_sve_ldnt1_gather_index, 32804),
SVEMAP1(svldnt1sh_gather_s64index_u64, aarch64_sve_ldnt1_gather_index, 32804),
SVEMAP1(svldnt1sh_gather_s64offset_s64, aarch64_sve_ldnt1_gather, 16810020),
SVEMAP1(svldnt1sh_gather_s64offset_u64, aarch64_sve_ldnt1_gather, 16810020),
SVEMAP1(svldnt1sh_gather_u32base_index_s32, aarch64_sve_ldnt1_gather_scalar_offset, 32803),
SVEMAP1(svldnt1sh_gather_u32base_index_u32, aarch64_sve_ldnt1_gather_scalar_offset, 32803),
SVEMAP1(svldnt1sh_gather_u32base_offset_s32, aarch64_sve_ldnt1_gather_scalar_offset, 16810019),
SVEMAP1(svldnt1sh_gather_u32base_offset_u32, aarch64_sve_ldnt1_gather_scalar_offset, 16810019),
SVEMAP1(svldnt1sh_gather_u32base_s32, aarch64_sve_ldnt1_gather_scalar_offset, 32803),
SVEMAP1(svldnt1sh_gather_u32base_u32, aarch64_sve_ldnt1_gather_scalar_offset, 32803),
SVEMAP1(svldnt1sh_gather_u32offset_s32, aarch64_sve_ldnt1_gather_uxtw, 16810019),
SVEMAP1(svldnt1sh_gather_u32offset_u32, aarch64_sve_ldnt1_gather_uxtw, 16810019),
SVEMAP1(svldnt1sh_gather_u64base_index_s64, aarch64_sve_ldnt1_gather_scalar_offset, 32804),
SVEMAP1(svldnt1sh_gather_u64base_index_u64, aarch64_sve_ldnt1_gather_scalar_offset, 32804),
SVEMAP1(svldnt1sh_gather_u64base_offset_s64, aarch64_sve_ldnt1_gather_scalar_offset, 16810020),
SVEMAP1(svldnt1sh_gather_u64base_offset_u64, aarch64_sve_ldnt1_gather_scalar_offset, 16810020),
SVEMAP1(svldnt1sh_gather_u64base_s64, aarch64_sve_ldnt1_gather_scalar_offset, 32804),
SVEMAP1(svldnt1sh_gather_u64base_u64, aarch64_sve_ldnt1_gather_scalar_offset, 32804),
SVEMAP1(svldnt1sh_gather_u64index_s64, aarch64_sve_ldnt1_gather_index, 32804),
SVEMAP1(svldnt1sh_gather_u64index_u64, aarch64_sve_ldnt1_gather_index, 32804),
SVEMAP1(svldnt1sh_gather_u64offset_s64, aarch64_sve_ldnt1_gather, 16810020),
SVEMAP1(svldnt1sh_gather_u64offset_u64, aarch64_sve_ldnt1_gather, 16810020),
SVEMAP1(svldnt1sw_gather_s64index_s64, aarch64_sve_ldnt1_gather_index, 32820),
SVEMAP1(svldnt1sw_gather_s64index_u64, aarch64_sve_ldnt1_gather_index, 32820),
SVEMAP1(svldnt1sw_gather_s64offset_s64, aarch64_sve_ldnt1_gather, 16810036),
SVEMAP1(svldnt1sw_gather_s64offset_u64, aarch64_sve_ldnt1_gather, 16810036),
SVEMAP1(svldnt1sw_gather_u64base_index_s64, aarch64_sve_ldnt1_gather_scalar_offset, 32820),
SVEMAP1(svldnt1sw_gather_u64base_index_u64, aarch64_sve_ldnt1_gather_scalar_offset, 32820),
SVEMAP1(svldnt1sw_gather_u64base_offset_s64, aarch64_sve_ldnt1_gather_scalar_offset, 16810036),
SVEMAP1(svldnt1sw_gather_u64base_offset_u64, aarch64_sve_ldnt1_gather_scalar_offset, 16810036),
SVEMAP1(svldnt1sw_gather_u64base_s64, aarch64_sve_ldnt1_gather_scalar_offset, 32820),
SVEMAP1(svldnt1sw_gather_u64base_u64, aarch64_sve_ldnt1_gather_scalar_offset, 32820),
SVEMAP1(svldnt1sw_gather_u64index_s64, aarch64_sve_ldnt1_gather_index, 32820),
SVEMAP1(svldnt1sw_gather_u64index_u64, aarch64_sve_ldnt1_gather_index, 32820),
SVEMAP1(svldnt1sw_gather_u64offset_s64, aarch64_sve_ldnt1_gather, 16810036),
SVEMAP1(svldnt1sw_gather_u64offset_u64, aarch64_sve_ldnt1_gather, 16810036),
SVEMAP1(svldnt1ub_gather_s64offset_s64, aarch64_sve_ldnt1_gather, 17334292),
SVEMAP1(svldnt1ub_gather_s64offset_u64, aarch64_sve_ldnt1_gather, 17334292),
SVEMAP1(svldnt1ub_gather_u32base_offset_s32, aarch64_sve_ldnt1_gather_scalar_offset, 17334291),
SVEMAP1(svldnt1ub_gather_u32base_offset_u32, aarch64_sve_ldnt1_gather_scalar_offset, 17334291),
SVEMAP1(svldnt1ub_gather_u32base_s32, aarch64_sve_ldnt1_gather_scalar_offset, 557075),
SVEMAP1(svldnt1ub_gather_u32base_u32, aarch64_sve_ldnt1_gather_scalar_offset, 557075),
SVEMAP1(svldnt1ub_gather_u32offset_s32, aarch64_sve_ldnt1_gather_uxtw, 17334291),
SVEMAP1(svldnt1ub_gather_u32offset_u32, aarch64_sve_ldnt1_gather_uxtw, 17334291),
SVEMAP1(svldnt1ub_gather_u64base_offset_s64, aarch64_sve_ldnt1_gather_scalar_offset, 17334292),
SVEMAP1(svldnt1ub_gather_u64base_offset_u64, aarch64_sve_ldnt1_gather_scalar_offset, 17334292),
SVEMAP1(svldnt1ub_gather_u64base_s64, aarch64_sve_ldnt1_gather_scalar_offset, 557076),
SVEMAP1(svldnt1ub_gather_u64base_u64, aarch64_sve_ldnt1_gather_scalar_offset, 557076),
SVEMAP1(svldnt1ub_gather_u64offset_s64, aarch64_sve_ldnt1_gather, 17334292),
SVEMAP1(svldnt1ub_gather_u64offset_u64, aarch64_sve_ldnt1_gather, 17334292),
SVEMAP1(svldnt1uh_gather_s64index_s64, aarch64_sve_ldnt1_gather_index, 557092),
SVEMAP1(svldnt1uh_gather_s64index_u64, aarch64_sve_ldnt1_gather_index, 557092),
SVEMAP1(svldnt1uh_gather_s64offset_s64, aarch64_sve_ldnt1_gather, 17334308),
SVEMAP1(svldnt1uh_gather_s64offset_u64, aarch64_sve_ldnt1_gather, 17334308),
SVEMAP1(svldnt1uh_gather_u32base_index_s32, aarch64_sve_ldnt1_gather_scalar_offset, 557091),
SVEMAP1(svldnt1uh_gather_u32base_index_u32, aarch64_sve_ldnt1_gather_scalar_offset, 557091),
SVEMAP1(svldnt1uh_gather_u32base_offset_s32, aarch64_sve_ldnt1_gather_scalar_offset, 17334307),
SVEMAP1(svldnt1uh_gather_u32base_offset_u32, aarch64_sve_ldnt1_gather_scalar_offset, 17334307),
SVEMAP1(svldnt1uh_gather_u32base_s32, aarch64_sve_ldnt1_gather_scalar_offset, 557091),
SVEMAP1(svldnt1uh_gather_u32base_u32, aarch64_sve_ldnt1_gather_scalar_offset, 557091),
SVEMAP1(svldnt1uh_gather_u32offset_s32, aarch64_sve_ldnt1_gather_uxtw, 17334307),
SVEMAP1(svldnt1uh_gather_u32offset_u32, aarch64_sve_ldnt1_gather_uxtw, 17334307),
SVEMAP1(svldnt1uh_gather_u64base_index_s64, aarch64_sve_ldnt1_gather_scalar_offset, 557092),
SVEMAP1(svldnt1uh_gather_u64base_index_u64, aarch64_sve_ldnt1_gather_scalar_offset, 557092),
SVEMAP1(svldnt1uh_gather_u64base_offset_s64, aarch64_sve_ldnt1_gather_scalar_offset, 17334308),
SVEMAP1(svldnt1uh_gather_u64base_offset_u64, aarch64_sve_ldnt1_gather_scalar_offset, 17334308),
SVEMAP1(svldnt1uh_gather_u64base_s64, aarch64_sve_ldnt1_gather_scalar_offset, 557092),
SVEMAP1(svldnt1uh_gather_u64base_u64, aarch64_sve_ldnt1_gather_scalar_offset, 557092),
SVEMAP1(svldnt1uh_gather_u64index_s64, aarch64_sve_ldnt1_gather_index, 557092),
SVEMAP1(svldnt1uh_gather_u64index_u64, aarch64_sve_ldnt1_gather_index, 557092),
SVEMAP1(svldnt1uh_gather_u64offset_s64, aarch64_sve_ldnt1_gather, 17334308),
SVEMAP1(svldnt1uh_gather_u64offset_u64, aarch64_sve_ldnt1_gather, 17334308),
SVEMAP1(svldnt1uw_gather_s64index_s64, aarch64_sve_ldnt1_gather_index, 557108),
SVEMAP1(svldnt1uw_gather_s64index_u64, aarch64_sve_ldnt1_gather_index, 557108),
SVEMAP1(svldnt1uw_gather_s64offset_s64, aarch64_sve_ldnt1_gather, 17334324),
SVEMAP1(svldnt1uw_gather_s64offset_u64, aarch64_sve_ldnt1_gather, 17334324),
SVEMAP1(svldnt1uw_gather_u64base_index_s64, aarch64_sve_ldnt1_gather_scalar_offset, 557108),
SVEMAP1(svldnt1uw_gather_u64base_index_u64, aarch64_sve_ldnt1_gather_scalar_offset, 557108),
SVEMAP1(svldnt1uw_gather_u64base_offset_s64, aarch64_sve_ldnt1_gather_scalar_offset, 17334324),
SVEMAP1(svldnt1uw_gather_u64base_offset_u64, aarch64_sve_ldnt1_gather_scalar_offset, 17334324),
SVEMAP1(svldnt1uw_gather_u64base_s64, aarch64_sve_ldnt1_gather_scalar_offset, 557108),
SVEMAP1(svldnt1uw_gather_u64base_u64, aarch64_sve_ldnt1_gather_scalar_offset, 557108),
SVEMAP1(svldnt1uw_gather_u64index_s64, aarch64_sve_ldnt1_gather_index, 557108),
SVEMAP1(svldnt1uw_gather_u64index_u64, aarch64_sve_ldnt1_gather_index, 557108),
SVEMAP1(svldnt1uw_gather_u64offset_s64, aarch64_sve_ldnt1_gather, 17334324),
SVEMAP1(svldnt1uw_gather_u64offset_u64, aarch64_sve_ldnt1_gather, 17334324),
SVEMAP2(svlen_bf16, 4398046511117),
SVEMAP2(svlen_f16, 4398046511110),
SVEMAP2(svlen_f32, 4398046511111),
SVEMAP2(svlen_f64, 4398046511112),
SVEMAP2(svlen_s16, 4398046511106),
SVEMAP2(svlen_s32, 4398046511107),
SVEMAP2(svlen_s64, 4398046511108),
SVEMAP2(svlen_s8, 4398046511105),
SVEMAP2(svlen_u16, 4398046511106),
SVEMAP2(svlen_u32, 4398046511107),
SVEMAP2(svlen_u64, 4398046511108),
SVEMAP2(svlen_u8, 4398046511105),
SVEMAP1(svlogb_f16_m, aarch64_sve_flogb, 4398046511366),
SVEMAP1(svlogb_f16_x, aarch64_sve_flogb, 4398046511622),
SVEMAP1(svlogb_f16_z, aarch64_sve_flogb, 4398046511750),
SVEMAP1(svlogb_f32_m, aarch64_sve_flogb, 4398046511367),
SVEMAP1(svlogb_f32_x, aarch64_sve_flogb, 4398046511623),
SVEMAP1(svlogb_f32_z, aarch64_sve_flogb, 4398046511751),
SVEMAP1(svlogb_f64_m, aarch64_sve_flogb, 4398046511368),
SVEMAP1(svlogb_f64_x, aarch64_sve_flogb, 4398046511624),
SVEMAP1(svlogb_f64_z, aarch64_sve_flogb, 4398046511752),
SVEMAP1(svlsl_n_s16_m, aarch64_sve_lsl, 4398046514434),
SVEMAP1(svlsl_n_s16_x, aarch64_sve_lsl_u, 4398046514306),
SVEMAP1(svlsl_n_s16_z, aarch64_sve_lsl, 4398046514562),
SVEMAP1(svlsl_n_s32_m, aarch64_sve_lsl, 4398046514435),
SVEMAP1(svlsl_n_s32_x, aarch64_sve_lsl_u, 4398046514307),
SVEMAP1(svlsl_n_s32_z, aarch64_sve_lsl, 4398046514563),
SVEMAP1(svlsl_n_s64_m, aarch64_sve_lsl, 4398046514436),
SVEMAP1(svlsl_n_s64_x, aarch64_sve_lsl_u, 4398046514308),
SVEMAP1(svlsl_n_s64_z, aarch64_sve_lsl, 4398046514564),
SVEMAP1(svlsl_n_s8_m, aarch64_sve_lsl, 4398046514433),
SVEMAP1(svlsl_n_s8_x, aarch64_sve_lsl_u, 4398046514305),
SVEMAP1(svlsl_n_s8_z, aarch64_sve_lsl, 4398046514561),
SVEMAP1(svlsl_n_u16_m, aarch64_sve_lsl, 4398046514434),
SVEMAP1(svlsl_n_u16_x, aarch64_sve_lsl_u, 4398046514306),
SVEMAP1(svlsl_n_u16_z, aarch64_sve_lsl, 4398046514562),
SVEMAP1(svlsl_n_u32_m, aarch64_sve_lsl, 4398046514435),
SVEMAP1(svlsl_n_u32_x, aarch64_sve_lsl_u, 4398046514307),
SVEMAP1(svlsl_n_u32_z, aarch64_sve_lsl, 4398046514563),
SVEMAP1(svlsl_n_u64_m, aarch64_sve_lsl, 4398046514436),
SVEMAP1(svlsl_n_u64_x, aarch64_sve_lsl_u, 4398046514308),
SVEMAP1(svlsl_n_u64_z, aarch64_sve_lsl, 4398046514564),
SVEMAP1(svlsl_n_u8_m, aarch64_sve_lsl, 4398046514433),
SVEMAP1(svlsl_n_u8_x, aarch64_sve_lsl_u, 4398046514305),
SVEMAP1(svlsl_n_u8_z, aarch64_sve_lsl, 4398046514561),
SVEMAP1(svlsl_s16_m, aarch64_sve_lsl, 4398046511362),
SVEMAP1(svlsl_s16_x, aarch64_sve_lsl_u, 4398046511234),
SVEMAP1(svlsl_s16_z, aarch64_sve_lsl, 4398046511490),
SVEMAP1(svlsl_s32_m, aarch64_sve_lsl, 4398046511363),
SVEMAP1(svlsl_s32_x, aarch64_sve_lsl_u, 4398046511235),
SVEMAP1(svlsl_s32_z, aarch64_sve_lsl, 4398046511491),
SVEMAP1(svlsl_s64_m, aarch64_sve_lsl, 4398046511364),
SVEMAP1(svlsl_s64_x, aarch64_sve_lsl_u, 4398046511236),
SVEMAP1(svlsl_s64_z, aarch64_sve_lsl, 4398046511492),
SVEMAP1(svlsl_s8_m, aarch64_sve_lsl, 4398046511361),
SVEMAP1(svlsl_s8_x, aarch64_sve_lsl_u, 4398046511233),
SVEMAP1(svlsl_s8_z, aarch64_sve_lsl, 4398046511489),
SVEMAP1(svlsl_u16_m, aarch64_sve_lsl, 4398046511362),
SVEMAP1(svlsl_u16_x, aarch64_sve_lsl_u, 4398046511234),
SVEMAP1(svlsl_u16_z, aarch64_sve_lsl, 4398046511490),
SVEMAP1(svlsl_u32_m, aarch64_sve_lsl, 4398046511363),
SVEMAP1(svlsl_u32_x, aarch64_sve_lsl_u, 4398046511235),
SVEMAP1(svlsl_u32_z, aarch64_sve_lsl, 4398046511491),
SVEMAP1(svlsl_u64_m, aarch64_sve_lsl, 4398046511364),
SVEMAP1(svlsl_u64_x, aarch64_sve_lsl_u, 4398046511236),
SVEMAP1(svlsl_u64_z, aarch64_sve_lsl, 4398046511492),
SVEMAP1(svlsl_u8_m, aarch64_sve_lsl, 4398046511361),
SVEMAP1(svlsl_u8_x, aarch64_sve_lsl_u, 4398046511233),
SVEMAP1(svlsl_u8_z, aarch64_sve_lsl, 4398046511489),
SVEMAP1(svlsl_wide_n_s16_m, aarch64_sve_lsl_wide, 4398046514434),
SVEMAP1(svlsl_wide_n_s16_x, aarch64_sve_lsl_wide, 4398046514306),
SVEMAP1(svlsl_wide_n_s16_z, aarch64_sve_lsl_wide, 4398046514562),
SVEMAP1(svlsl_wide_n_s32_m, aarch64_sve_lsl_wide, 4398046514435),
SVEMAP1(svlsl_wide_n_s32_x, aarch64_sve_lsl_wide, 4398046514307),
SVEMAP1(svlsl_wide_n_s32_z, aarch64_sve_lsl_wide, 4398046514563),
SVEMAP1(svlsl_wide_n_s8_m, aarch64_sve_lsl_wide, 4398046514433),
SVEMAP1(svlsl_wide_n_s8_x, aarch64_sve_lsl_wide, 4398046514305),
SVEMAP1(svlsl_wide_n_s8_z, aarch64_sve_lsl_wide, 4398046514561),
SVEMAP1(svlsl_wide_n_u16_m, aarch64_sve_lsl_wide, 4398046514434),
SVEMAP1(svlsl_wide_n_u16_x, aarch64_sve_lsl_wide, 4398046514306),
SVEMAP1(svlsl_wide_n_u16_z, aarch64_sve_lsl_wide, 4398046514562),
SVEMAP1(svlsl_wide_n_u32_m, aarch64_sve_lsl_wide, 4398046514435),
SVEMAP1(svlsl_wide_n_u32_x, aarch64_sve_lsl_wide, 4398046514307),
SVEMAP1(svlsl_wide_n_u32_z, aarch64_sve_lsl_wide, 4398046514563),
SVEMAP1(svlsl_wide_n_u8_m, aarch64_sve_lsl_wide, 4398046514433),
SVEMAP1(svlsl_wide_n_u8_x, aarch64_sve_lsl_wide, 4398046514305),
SVEMAP1(svlsl_wide_n_u8_z, aarch64_sve_lsl_wide, 4398046514561),
SVEMAP1(svlsl_wide_s16_m, aarch64_sve_lsl_wide, 4398046511362),
SVEMAP1(svlsl_wide_s16_x, aarch64_sve_lsl_wide, 4398046511234),
SVEMAP1(svlsl_wide_s16_z, aarch64_sve_lsl_wide, 4398046511490),
SVEMAP1(svlsl_wide_s32_m, aarch64_sve_lsl_wide, 4398046511363),
SVEMAP1(svlsl_wide_s32_x, aarch64_sve_lsl_wide, 4398046511235),
SVEMAP1(svlsl_wide_s32_z, aarch64_sve_lsl_wide, 4398046511491),
SVEMAP1(svlsl_wide_s8_m, aarch64_sve_lsl_wide, 4398046511361),
SVEMAP1(svlsl_wide_s8_x, aarch64_sve_lsl_wide, 4398046511233),
SVEMAP1(svlsl_wide_s8_z, aarch64_sve_lsl_wide, 4398046511489),
SVEMAP1(svlsl_wide_u16_m, aarch64_sve_lsl_wide, 4398046511362),
SVEMAP1(svlsl_wide_u16_x, aarch64_sve_lsl_wide, 4398046511234),
SVEMAP1(svlsl_wide_u16_z, aarch64_sve_lsl_wide, 4398046511490),
SVEMAP1(svlsl_wide_u32_m, aarch64_sve_lsl_wide, 4398046511363),
SVEMAP1(svlsl_wide_u32_x, aarch64_sve_lsl_wide, 4398046511235),
SVEMAP1(svlsl_wide_u32_z, aarch64_sve_lsl_wide, 4398046511491),
SVEMAP1(svlsl_wide_u8_m, aarch64_sve_lsl_wide, 4398046511361),
SVEMAP1(svlsl_wide_u8_x, aarch64_sve_lsl_wide, 4398046511233),
SVEMAP1(svlsl_wide_u8_z, aarch64_sve_lsl_wide, 4398046511489),
SVEMAP1(svlsr_n_u16_m, aarch64_sve_lsr, 4398046514434),
SVEMAP1(svlsr_n_u16_x, aarch64_sve_lsr_u, 4398046514306),
SVEMAP1(svlsr_n_u16_z, aarch64_sve_lsr, 4398046514562),
SVEMAP1(svlsr_n_u32_m, aarch64_sve_lsr, 4398046514435),
SVEMAP1(svlsr_n_u32_x, aarch64_sve_lsr_u, 4398046514307),
SVEMAP1(svlsr_n_u32_z, aarch64_sve_lsr, 4398046514563),
SVEMAP1(svlsr_n_u64_m, aarch64_sve_lsr, 4398046514436),
SVEMAP1(svlsr_n_u64_x, aarch64_sve_lsr_u, 4398046514308),
SVEMAP1(svlsr_n_u64_z, aarch64_sve_lsr, 4398046514564),
SVEMAP1(svlsr_n_u8_m, aarch64_sve_lsr, 4398046514433),
SVEMAP1(svlsr_n_u8_x, aarch64_sve_lsr_u, 4398046514305),
SVEMAP1(svlsr_n_u8_z, aarch64_sve_lsr, 4398046514561),
SVEMAP1(svlsr_u16_m, aarch64_sve_lsr, 4398046511362),
SVEMAP1(svlsr_u16_x, aarch64_sve_lsr_u, 4398046511234),
SVEMAP1(svlsr_u16_z, aarch64_sve_lsr, 4398046511490),
SVEMAP1(svlsr_u32_m, aarch64_sve_lsr, 4398046511363),
SVEMAP1(svlsr_u32_x, aarch64_sve_lsr_u, 4398046511235),
SVEMAP1(svlsr_u32_z, aarch64_sve_lsr, 4398046511491),
SVEMAP1(svlsr_u64_m, aarch64_sve_lsr, 4398046511364),
SVEMAP1(svlsr_u64_x, aarch64_sve_lsr_u, 4398046511236),
SVEMAP1(svlsr_u64_z, aarch64_sve_lsr, 4398046511492),
SVEMAP1(svlsr_u8_m, aarch64_sve_lsr, 4398046511361),
SVEMAP1(svlsr_u8_x, aarch64_sve_lsr_u, 4398046511233),
SVEMAP1(svlsr_u8_z, aarch64_sve_lsr, 4398046511489),
SVEMAP1(svlsr_wide_n_u16_m, aarch64_sve_lsr_wide, 4398046514434),
SVEMAP1(svlsr_wide_n_u16_x, aarch64_sve_lsr_wide, 4398046514306),
SVEMAP1(svlsr_wide_n_u16_z, aarch64_sve_lsr_wide, 4398046514562),
SVEMAP1(svlsr_wide_n_u32_m, aarch64_sve_lsr_wide, 4398046514435),
SVEMAP1(svlsr_wide_n_u32_x, aarch64_sve_lsr_wide, 4398046514307),
SVEMAP1(svlsr_wide_n_u32_z, aarch64_sve_lsr_wide, 4398046514563),
SVEMAP1(svlsr_wide_n_u8_m, aarch64_sve_lsr_wide, 4398046514433),
SVEMAP1(svlsr_wide_n_u8_x, aarch64_sve_lsr_wide, 4398046514305),
SVEMAP1(svlsr_wide_n_u8_z, aarch64_sve_lsr_wide, 4398046514561),
SVEMAP1(svlsr_wide_u16_m, aarch64_sve_lsr_wide, 4398046511362),
SVEMAP1(svlsr_wide_u16_x, aarch64_sve_lsr_wide, 4398046511234),
SVEMAP1(svlsr_wide_u16_z, aarch64_sve_lsr_wide, 4398046511490),
SVEMAP1(svlsr_wide_u32_m, aarch64_sve_lsr_wide, 4398046511363),
SVEMAP1(svlsr_wide_u32_x, aarch64_sve_lsr_wide, 4398046511235),
SVEMAP1(svlsr_wide_u32_z, aarch64_sve_lsr_wide, 4398046511491),
SVEMAP1(svlsr_wide_u8_m, aarch64_sve_lsr_wide, 4398046511361),
SVEMAP1(svlsr_wide_u8_x, aarch64_sve_lsr_wide, 4398046511233),
SVEMAP1(svlsr_wide_u8_z, aarch64_sve_lsr_wide, 4398046511489),
SVEMAP1(svluti2_lane_bf16, aarch64_sve_luti2_lane, 4398046511117),
SVEMAP1(svluti2_lane_f16, aarch64_sve_luti2_lane, 4398046511110),
SVEMAP1(svluti2_lane_s16, aarch64_sve_luti2_lane, 4398046511106),
SVEMAP1(svluti2_lane_s8, aarch64_sve_luti2_lane, 4398046511105),
SVEMAP1(svluti2_lane_u16, aarch64_sve_luti2_lane, 4398046511106),
SVEMAP1(svluti2_lane_u8, aarch64_sve_luti2_lane, 4398046511105),
SVEMAP1(svluti4_lane_bf16, aarch64_sve_luti4_lane, 4398046511117),
SVEMAP1(svluti4_lane_bf16_x2, aarch64_sve_luti4_lane_x2, 4398046511117),
SVEMAP1(svluti4_lane_f16, aarch64_sve_luti4_lane, 4398046511110),
SVEMAP1(svluti4_lane_f16_x2, aarch64_sve_luti4_lane_x2, 4398046511110),
SVEMAP1(svluti4_lane_s16, aarch64_sve_luti4_lane, 4398046511106),
SVEMAP1(svluti4_lane_s16_x2, aarch64_sve_luti4_lane_x2, 4398046511106),
SVEMAP1(svluti4_lane_s8, aarch64_sve_luti4_lane, 4398046511105),
SVEMAP1(svluti4_lane_u16, aarch64_sve_luti4_lane, 4398046511106),
SVEMAP1(svluti4_lane_u16_x2, aarch64_sve_luti4_lane_x2, 4398046511106),
SVEMAP1(svluti4_lane_u8, aarch64_sve_luti4_lane, 4398046511105),
SVEMAP1(svmad_f16_m, aarch64_sve_fmad, 4466765988102),
SVEMAP1(svmad_f16_x, aarch64_sve_fmla_u, 4466765987974),
SVEMAP1(svmad_f16_z, aarch64_sve_fmad, 4466765988230),
SVEMAP1(svmad_f32_m, aarch64_sve_fmad, 4466765988103),
SVEMAP1(svmad_f32_x, aarch64_sve_fmla_u, 4466765987975),
SVEMAP1(svmad_f32_z, aarch64_sve_fmad, 4466765988231),
SVEMAP1(svmad_f64_m, aarch64_sve_fmad, 4466765988104),
SVEMAP1(svmad_f64_x, aarch64_sve_fmla_u, 4466765987976),
SVEMAP1(svmad_f64_z, aarch64_sve_fmad, 4466765988232),
SVEMAP1(svmad_n_f16_m, aarch64_sve_fmad, 4466765992198),
SVEMAP1(svmad_n_f16_x, aarch64_sve_fmla_u, 4466765992070),
SVEMAP1(svmad_n_f16_z, aarch64_sve_fmad, 4466765992326),
SVEMAP1(svmad_n_f32_m, aarch64_sve_fmad, 4466765992199),
SVEMAP1(svmad_n_f32_x, aarch64_sve_fmla_u, 4466765992071),
SVEMAP1(svmad_n_f32_z, aarch64_sve_fmad, 4466765992327),
SVEMAP1(svmad_n_f64_m, aarch64_sve_fmad, 4466765992200),
SVEMAP1(svmad_n_f64_x, aarch64_sve_fmla_u, 4466765992072),
SVEMAP1(svmad_n_f64_z, aarch64_sve_fmad, 4466765992328),
SVEMAP1(svmad_n_s16_m, aarch64_sve_mad, 4466765992194),
SVEMAP1(svmad_n_s16_x, aarch64_sve_mla_u, 4466765992066),
SVEMAP1(svmad_n_s16_z, aarch64_sve_mad, 4466765992322),
SVEMAP1(svmad_n_s32_m, aarch64_sve_mad, 4466765992195),
SVEMAP1(svmad_n_s32_x, aarch64_sve_mla_u, 4466765992067),
SVEMAP1(svmad_n_s32_z, aarch64_sve_mad, 4466765992323),
SVEMAP1(svmad_n_s64_m, aarch64_sve_mad, 4466765992196),
SVEMAP1(svmad_n_s64_x, aarch64_sve_mla_u, 4466765992068),
SVEMAP1(svmad_n_s64_z, aarch64_sve_mad, 4466765992324),
SVEMAP1(svmad_n_s8_m, aarch64_sve_mad, 4466765992193),
SVEMAP1(svmad_n_s8_x, aarch64_sve_mla_u, 4466765992065),
SVEMAP1(svmad_n_s8_z, aarch64_sve_mad, 4466765992321),
SVEMAP1(svmad_n_u16_m, aarch64_sve_mad, 4466765992194),
SVEMAP1(svmad_n_u16_x, aarch64_sve_mla_u, 4466765992066),
SVEMAP1(svmad_n_u16_z, aarch64_sve_mad, 4466765992322),
SVEMAP1(svmad_n_u32_m, aarch64_sve_mad, 4466765992195),
SVEMAP1(svmad_n_u32_x, aarch64_sve_mla_u, 4466765992067),
SVEMAP1(svmad_n_u32_z, aarch64_sve_mad, 4466765992323),
SVEMAP1(svmad_n_u64_m, aarch64_sve_mad, 4466765992196),
SVEMAP1(svmad_n_u64_x, aarch64_sve_mla_u, 4466765992068),
SVEMAP1(svmad_n_u64_z, aarch64_sve_mad, 4466765992324),
SVEMAP1(svmad_n_u8_m, aarch64_sve_mad, 4466765992193),
SVEMAP1(svmad_n_u8_x, aarch64_sve_mla_u, 4466765992065),
SVEMAP1(svmad_n_u8_z, aarch64_sve_mad, 4466765992321),
SVEMAP1(svmad_s16_m, aarch64_sve_mad, 4466765988098),
SVEMAP1(svmad_s16_x, aarch64_sve_mla_u, 4466765987970),
SVEMAP1(svmad_s16_z, aarch64_sve_mad, 4466765988226),
SVEMAP1(svmad_s32_m, aarch64_sve_mad, 4466765988099),
SVEMAP1(svmad_s32_x, aarch64_sve_mla_u, 4466765987971),
SVEMAP1(svmad_s32_z, aarch64_sve_mad, 4466765988227),
SVEMAP1(svmad_s64_m, aarch64_sve_mad, 4466765988100),
SVEMAP1(svmad_s64_x, aarch64_sve_mla_u, 4466765987972),
SVEMAP1(svmad_s64_z, aarch64_sve_mad, 4466765988228),
SVEMAP1(svmad_s8_m, aarch64_sve_mad, 4466765988097),
SVEMAP1(svmad_s8_x, aarch64_sve_mla_u, 4466765987969),
SVEMAP1(svmad_s8_z, aarch64_sve_mad, 4466765988225),
SVEMAP1(svmad_u16_m, aarch64_sve_mad, 4466765988098),
SVEMAP1(svmad_u16_x, aarch64_sve_mla_u, 4466765987970),
SVEMAP1(svmad_u16_z, aarch64_sve_mad, 4466765988226),
SVEMAP1(svmad_u32_m, aarch64_sve_mad, 4466765988099),
SVEMAP1(svmad_u32_x, aarch64_sve_mla_u, 4466765987971),
SVEMAP1(svmad_u32_z, aarch64_sve_mad, 4466765988227),
SVEMAP1(svmad_u64_m, aarch64_sve_mad, 4466765988100),
SVEMAP1(svmad_u64_x, aarch64_sve_mla_u, 4466765987972),
SVEMAP1(svmad_u64_z, aarch64_sve_mad, 4466765988228),
SVEMAP1(svmad_u8_m, aarch64_sve_mad, 4466765988097),
SVEMAP1(svmad_u8_x, aarch64_sve_mla_u, 4466765987969),
SVEMAP1(svmad_u8_z, aarch64_sve_mad, 4466765988225),
SVEMAP1(svmatch_s16, aarch64_sve_match, 2),
SVEMAP1(svmatch_s8, aarch64_sve_match, 1),
SVEMAP1(svmatch_u16, aarch64_sve_match, 2),
SVEMAP1(svmatch_u8, aarch64_sve_match, 1),
SVEMAP1(svmax_bf16_m, aarch64_sve_fmax, 4398046511373),
SVEMAP1(svmax_bf16_x, aarch64_sve_fmax_u, 4398046511245),
SVEMAP1(svmax_bf16_x2, aarch64_sve_fmax_x2, 137438953485),
SVEMAP1(svmax_bf16_x4, aarch64_sve_fmax_x4, 137438953485),
SVEMAP1(svmax_bf16_z, aarch64_sve_fmax, 4398046511501),
SVEMAP1(svmax_f16_m, aarch64_sve_fmax, 4398046511366),
SVEMAP1(svmax_f16_x, aarch64_sve_fmax_u, 4398046511238),
SVEMAP1(svmax_f16_x2, aarch64_sve_fmax_x2, 137438953478),
SVEMAP1(svmax_f16_x4, aarch64_sve_fmax_x4, 137438953478),
SVEMAP1(svmax_f16_z, aarch64_sve_fmax, 4398046511494),
SVEMAP1(svmax_f32_m, aarch64_sve_fmax, 4398046511367),
SVEMAP1(svmax_f32_x, aarch64_sve_fmax_u, 4398046511239),
SVEMAP1(svmax_f32_x2, aarch64_sve_fmax_x2, 137438953479),
SVEMAP1(svmax_f32_x4, aarch64_sve_fmax_x4, 137438953479),
SVEMAP1(svmax_f32_z, aarch64_sve_fmax, 4398046511495),
SVEMAP1(svmax_f64_m, aarch64_sve_fmax, 4398046511368),
SVEMAP1(svmax_f64_x, aarch64_sve_fmax_u, 4398046511240),
SVEMAP1(svmax_f64_x2, aarch64_sve_fmax_x2, 137438953480),
SVEMAP1(svmax_f64_x4, aarch64_sve_fmax_x4, 137438953480),
SVEMAP1(svmax_f64_z, aarch64_sve_fmax, 4398046511496),
SVEMAP1(svmax_n_bf16_m, aarch64_sve_fmax, 4398046514445),
SVEMAP1(svmax_n_bf16_x, aarch64_sve_fmax_u, 4398046514317),
SVEMAP1(svmax_n_bf16_z, aarch64_sve_fmax, 4398046514573),
SVEMAP1(svmax_n_f16_m, aarch64_sve_fmax, 4398046514438),
SVEMAP1(svmax_n_f16_x, aarch64_sve_fmax_u, 4398046514310),
SVEMAP1(svmax_n_f16_z, aarch64_sve_fmax, 4398046514566),
SVEMAP1(svmax_n_f32_m, aarch64_sve_fmax, 4398046514439),
SVEMAP1(svmax_n_f32_x, aarch64_sve_fmax_u, 4398046514311),
SVEMAP1(svmax_n_f32_z, aarch64_sve_fmax, 4398046514567),
SVEMAP1(svmax_n_f64_m, aarch64_sve_fmax, 4398046514440),
SVEMAP1(svmax_n_f64_x, aarch64_sve_fmax_u, 4398046514312),
SVEMAP1(svmax_n_f64_z, aarch64_sve_fmax, 4398046514568),
SVEMAP1(svmax_n_s16_m, aarch64_sve_smax, 4398046514434),
SVEMAP1(svmax_n_s16_x, aarch64_sve_smax_u, 4398046514306),
SVEMAP1(svmax_n_s16_z, aarch64_sve_smax, 4398046514562),
SVEMAP1(svmax_n_s32_m, aarch64_sve_smax, 4398046514435),
SVEMAP1(svmax_n_s32_x, aarch64_sve_smax_u, 4398046514307),
SVEMAP1(svmax_n_s32_z, aarch64_sve_smax, 4398046514563),
SVEMAP1(svmax_n_s64_m, aarch64_sve_smax, 4398046514436),
SVEMAP1(svmax_n_s64_x, aarch64_sve_smax_u, 4398046514308),
SVEMAP1(svmax_n_s64_z, aarch64_sve_smax, 4398046514564),
SVEMAP1(svmax_n_s8_m, aarch64_sve_smax, 4398046514433),
SVEMAP1(svmax_n_s8_x, aarch64_sve_smax_u, 4398046514305),
SVEMAP1(svmax_n_s8_z, aarch64_sve_smax, 4398046514561),
SVEMAP1(svmax_n_u16_m, aarch64_sve_umax, 4398046514434),
SVEMAP1(svmax_n_u16_x, aarch64_sve_umax_u, 4398046514306),
SVEMAP1(svmax_n_u16_z, aarch64_sve_umax, 4398046514562),
SVEMAP1(svmax_n_u32_m, aarch64_sve_umax, 4398046514435),
SVEMAP1(svmax_n_u32_x, aarch64_sve_umax_u, 4398046514307),
SVEMAP1(svmax_n_u32_z, aarch64_sve_umax, 4398046514563),
SVEMAP1(svmax_n_u64_m, aarch64_sve_umax, 4398046514436),
SVEMAP1(svmax_n_u64_x, aarch64_sve_umax_u, 4398046514308),
SVEMAP1(svmax_n_u64_z, aarch64_sve_umax, 4398046514564),
SVEMAP1(svmax_n_u8_m, aarch64_sve_umax, 4398046514433),
SVEMAP1(svmax_n_u8_x, aarch64_sve_umax_u, 4398046514305),
SVEMAP1(svmax_n_u8_z, aarch64_sve_umax, 4398046514561),
SVEMAP1(svmax_s16_m, aarch64_sve_smax, 4398046511362),
SVEMAP1(svmax_s16_x, aarch64_sve_smax_u, 4398046511234),
SVEMAP1(svmax_s16_x2, aarch64_sve_smax_x2, 137438953474),
SVEMAP1(svmax_s16_x4, aarch64_sve_smax_x4, 137438953474),
SVEMAP1(svmax_s16_z, aarch64_sve_smax, 4398046511490),
SVEMAP1(svmax_s32_m, aarch64_sve_smax, 4398046511363),
SVEMAP1(svmax_s32_x, aarch64_sve_smax_u, 4398046511235),
SVEMAP1(svmax_s32_x2, aarch64_sve_smax_x2, 137438953475),
SVEMAP1(svmax_s32_x4, aarch64_sve_smax_x4, 137438953475),
SVEMAP1(svmax_s32_z, aarch64_sve_smax, 4398046511491),
SVEMAP1(svmax_s64_m, aarch64_sve_smax, 4398046511364),
SVEMAP1(svmax_s64_x, aarch64_sve_smax_u, 4398046511236),
SVEMAP1(svmax_s64_x2, aarch64_sve_smax_x2, 137438953476),
SVEMAP1(svmax_s64_x4, aarch64_sve_smax_x4, 137438953476),
SVEMAP1(svmax_s64_z, aarch64_sve_smax, 4398046511492),
SVEMAP1(svmax_s8_m, aarch64_sve_smax, 4398046511361),
SVEMAP1(svmax_s8_x, aarch64_sve_smax_u, 4398046511233),
SVEMAP1(svmax_s8_x2, aarch64_sve_smax_x2, 137438953473),
SVEMAP1(svmax_s8_x4, aarch64_sve_smax_x4, 137438953473),
SVEMAP1(svmax_s8_z, aarch64_sve_smax, 4398046511489),
SVEMAP1(svmax_single_bf16_x2, aarch64_sve_fmax_single_x2, 137438953485),
SVEMAP1(svmax_single_bf16_x4, aarch64_sve_fmax_single_x4, 137438953485),
SVEMAP1(svmax_single_f16_x2, aarch64_sve_fmax_single_x2, 137438953478),
SVEMAP1(svmax_single_f16_x4, aarch64_sve_fmax_single_x4, 137438953478),
SVEMAP1(svmax_single_f32_x2, aarch64_sve_fmax_single_x2, 137438953479),
SVEMAP1(svmax_single_f32_x4, aarch64_sve_fmax_single_x4, 137438953479),
SVEMAP1(svmax_single_f64_x2, aarch64_sve_fmax_single_x2, 137438953480),
SVEMAP1(svmax_single_f64_x4, aarch64_sve_fmax_single_x4, 137438953480),
SVEMAP1(svmax_single_s16_x2, aarch64_sve_smax_single_x2, 137438953474),
SVEMAP1(svmax_single_s16_x4, aarch64_sve_smax_single_x4, 137438953474),
SVEMAP1(svmax_single_s32_x2, aarch64_sve_smax_single_x2, 137438953475),
SVEMAP1(svmax_single_s32_x4, aarch64_sve_smax_single_x4, 137438953475),
SVEMAP1(svmax_single_s64_x2, aarch64_sve_smax_single_x2, 137438953476),
SVEMAP1(svmax_single_s64_x4, aarch64_sve_smax_single_x4, 137438953476),
SVEMAP1(svmax_single_s8_x2, aarch64_sve_smax_single_x2, 137438953473),
SVEMAP1(svmax_single_s8_x4, aarch64_sve_smax_single_x4, 137438953473),
SVEMAP1(svmax_single_u16_x2, aarch64_sve_umax_single_x2, 137438953474),
SVEMAP1(svmax_single_u16_x4, aarch64_sve_umax_single_x4, 137438953474),
SVEMAP1(svmax_single_u32_x2, aarch64_sve_umax_single_x2, 137438953475),
SVEMAP1(svmax_single_u32_x4, aarch64_sve_umax_single_x4, 137438953475),
SVEMAP1(svmax_single_u64_x2, aarch64_sve_umax_single_x2, 137438953476),
SVEMAP1(svmax_single_u64_x4, aarch64_sve_umax_single_x4, 137438953476),
SVEMAP1(svmax_single_u8_x2, aarch64_sve_umax_single_x2, 137438953473),
SVEMAP1(svmax_single_u8_x4, aarch64_sve_umax_single_x4, 137438953473),
SVEMAP1(svmax_u16_m, aarch64_sve_umax, 4398046511362),
SVEMAP1(svmax_u16_x, aarch64_sve_umax_u, 4398046511234),
SVEMAP1(svmax_u16_x2, aarch64_sve_umax_x2, 137438953474),
SVEMAP1(svmax_u16_x4, aarch64_sve_umax_x4, 137438953474),
SVEMAP1(svmax_u16_z, aarch64_sve_umax, 4398046511490),
SVEMAP1(svmax_u32_m, aarch64_sve_umax, 4398046511363),
SVEMAP1(svmax_u32_x, aarch64_sve_umax_u, 4398046511235),
SVEMAP1(svmax_u32_x2, aarch64_sve_umax_x2, 137438953475),
SVEMAP1(svmax_u32_x4, aarch64_sve_umax_x4, 137438953475),
SVEMAP1(svmax_u32_z, aarch64_sve_umax, 4398046511491),
SVEMAP1(svmax_u64_m, aarch64_sve_umax, 4398046511364),
SVEMAP1(svmax_u64_x, aarch64_sve_umax_u, 4398046511236),
SVEMAP1(svmax_u64_x2, aarch64_sve_umax_x2, 137438953476),
SVEMAP1(svmax_u64_x4, aarch64_sve_umax_x4, 137438953476),
SVEMAP1(svmax_u64_z, aarch64_sve_umax, 4398046511492),
SVEMAP1(svmax_u8_m, aarch64_sve_umax, 4398046511361),
SVEMAP1(svmax_u8_x, aarch64_sve_umax_u, 4398046511233),
SVEMAP1(svmax_u8_x2, aarch64_sve_umax_x2, 137438953473),
SVEMAP1(svmax_u8_x4, aarch64_sve_umax_x4, 137438953473),
SVEMAP1(svmax_u8_z, aarch64_sve_umax, 4398046511489),
SVEMAP1(svmaxnm_bf16_m, aarch64_sve_fmaxnm, 4398046511373),
SVEMAP1(svmaxnm_bf16_x, aarch64_sve_fmaxnm_u, 4398046511245),
SVEMAP1(svmaxnm_bf16_x2, aarch64_sve_fmaxnm_x2, 137438953485),
SVEMAP1(svmaxnm_bf16_x4, aarch64_sve_fmaxnm_x4, 137438953485),
SVEMAP1(svmaxnm_bf16_z, aarch64_sve_fmaxnm, 4398046511501),
SVEMAP1(svmaxnm_f16_m, aarch64_sve_fmaxnm, 4398046511366),
SVEMAP1(svmaxnm_f16_x, aarch64_sve_fmaxnm_u, 4398046511238),
SVEMAP1(svmaxnm_f16_x2, aarch64_sve_fmaxnm_x2, 137438953478),
SVEMAP1(svmaxnm_f16_x4, aarch64_sve_fmaxnm_x4, 137438953478),
SVEMAP1(svmaxnm_f16_z, aarch64_sve_fmaxnm, 4398046511494),
SVEMAP1(svmaxnm_f32_m, aarch64_sve_fmaxnm, 4398046511367),
SVEMAP1(svmaxnm_f32_x, aarch64_sve_fmaxnm_u, 4398046511239),
SVEMAP1(svmaxnm_f32_x2, aarch64_sve_fmaxnm_x2, 137438953479),
SVEMAP1(svmaxnm_f32_x4, aarch64_sve_fmaxnm_x4, 137438953479),
SVEMAP1(svmaxnm_f32_z, aarch64_sve_fmaxnm, 4398046511495),
SVEMAP1(svmaxnm_f64_m, aarch64_sve_fmaxnm, 4398046511368),
SVEMAP1(svmaxnm_f64_x, aarch64_sve_fmaxnm_u, 4398046511240),
SVEMAP1(svmaxnm_f64_x2, aarch64_sve_fmaxnm_x2, 137438953480),
SVEMAP1(svmaxnm_f64_x4, aarch64_sve_fmaxnm_x4, 137438953480),
SVEMAP1(svmaxnm_f64_z, aarch64_sve_fmaxnm, 4398046511496),
SVEMAP1(svmaxnm_n_bf16_m, aarch64_sve_fmaxnm, 4398046514445),
SVEMAP1(svmaxnm_n_bf16_x, aarch64_sve_fmaxnm_u, 4398046514317),
SVEMAP1(svmaxnm_n_bf16_z, aarch64_sve_fmaxnm, 4398046514573),
SVEMAP1(svmaxnm_n_f16_m, aarch64_sve_fmaxnm, 4398046514438),
SVEMAP1(svmaxnm_n_f16_x, aarch64_sve_fmaxnm_u, 4398046514310),
SVEMAP1(svmaxnm_n_f16_z, aarch64_sve_fmaxnm, 4398046514566),
SVEMAP1(svmaxnm_n_f32_m, aarch64_sve_fmaxnm, 4398046514439),
SVEMAP1(svmaxnm_n_f32_x, aarch64_sve_fmaxnm_u, 4398046514311),
SVEMAP1(svmaxnm_n_f32_z, aarch64_sve_fmaxnm, 4398046514567),
SVEMAP1(svmaxnm_n_f64_m, aarch64_sve_fmaxnm, 4398046514440),
SVEMAP1(svmaxnm_n_f64_x, aarch64_sve_fmaxnm_u, 4398046514312),
SVEMAP1(svmaxnm_n_f64_z, aarch64_sve_fmaxnm, 4398046514568),
SVEMAP1(svmaxnm_single_bf16_x2, aarch64_sve_fmaxnm_single_x2, 137438953485),
SVEMAP1(svmaxnm_single_bf16_x4, aarch64_sve_fmaxnm_single_x4, 137438953485),
SVEMAP1(svmaxnm_single_f16_x2, aarch64_sve_fmaxnm_single_x2, 137438953478),
SVEMAP1(svmaxnm_single_f16_x4, aarch64_sve_fmaxnm_single_x4, 137438953478),
SVEMAP1(svmaxnm_single_f32_x2, aarch64_sve_fmaxnm_single_x2, 137438953479),
SVEMAP1(svmaxnm_single_f32_x4, aarch64_sve_fmaxnm_single_x4, 137438953479),
SVEMAP1(svmaxnm_single_f64_x2, aarch64_sve_fmaxnm_single_x2, 137438953480),
SVEMAP1(svmaxnm_single_f64_x4, aarch64_sve_fmaxnm_single_x4, 137438953480),
SVEMAP1(svmaxnmp_f16_m, aarch64_sve_fmaxnmp, 4398046511366),
SVEMAP1(svmaxnmp_f16_x, aarch64_sve_fmaxnmp, 4398046511238),
SVEMAP1(svmaxnmp_f32_m, aarch64_sve_fmaxnmp, 4398046511367),
SVEMAP1(svmaxnmp_f32_x, aarch64_sve_fmaxnmp, 4398046511239),
SVEMAP1(svmaxnmp_f64_m, aarch64_sve_fmaxnmp, 4398046511368),
SVEMAP1(svmaxnmp_f64_x, aarch64_sve_fmaxnmp, 4398046511240),
SVEMAP1(svmaxnmqv_f16, aarch64_sve_fmaxnmqv, 6597069766662),
SVEMAP1(svmaxnmqv_f32, aarch64_sve_fmaxnmqv, 6597069766663),
SVEMAP1(svmaxnmqv_f64, aarch64_sve_fmaxnmqv, 6597069766664),
SVEMAP1(svmaxnmv_f16, aarch64_sve_fmaxnmv, 4398046511110),
SVEMAP1(svmaxnmv_f32, aarch64_sve_fmaxnmv, 4398046511111),
SVEMAP1(svmaxnmv_f64, aarch64_sve_fmaxnmv, 4398046511112),
SVEMAP1(svmaxp_f16_m, aarch64_sve_fmaxp, 4398046511366),
SVEMAP1(svmaxp_f16_x, aarch64_sve_fmaxp, 4398046511238),
SVEMAP1(svmaxp_f32_m, aarch64_sve_fmaxp, 4398046511367),
SVEMAP1(svmaxp_f32_x, aarch64_sve_fmaxp, 4398046511239),
SVEMAP1(svmaxp_f64_m, aarch64_sve_fmaxp, 4398046511368),
SVEMAP1(svmaxp_f64_x, aarch64_sve_fmaxp, 4398046511240),
SVEMAP1(svmaxp_s16_m, aarch64_sve_smaxp, 4398046511362),
SVEMAP1(svmaxp_s16_x, aarch64_sve_smaxp, 4398046511234),
SVEMAP1(svmaxp_s32_m, aarch64_sve_smaxp, 4398046511363),
SVEMAP1(svmaxp_s32_x, aarch64_sve_smaxp, 4398046511235),
SVEMAP1(svmaxp_s64_m, aarch64_sve_smaxp, 4398046511364),
SVEMAP1(svmaxp_s64_x, aarch64_sve_smaxp, 4398046511236),
SVEMAP1(svmaxp_s8_m, aarch64_sve_smaxp, 4398046511361),
SVEMAP1(svmaxp_s8_x, aarch64_sve_smaxp, 4398046511233),
SVEMAP1(svmaxp_u16_m, aarch64_sve_umaxp, 4398046511362),
SVEMAP1(svmaxp_u16_x, aarch64_sve_umaxp, 4398046511234),
SVEMAP1(svmaxp_u32_m, aarch64_sve_umaxp, 4398046511363),
SVEMAP1(svmaxp_u32_x, aarch64_sve_umaxp, 4398046511235),
SVEMAP1(svmaxp_u64_m, aarch64_sve_umaxp, 4398046511364),
SVEMAP1(svmaxp_u64_x, aarch64_sve_umaxp, 4398046511236),
SVEMAP1(svmaxp_u8_m, aarch64_sve_umaxp, 4398046511361),
SVEMAP1(svmaxp_u8_x, aarch64_sve_umaxp, 4398046511233),
SVEMAP1(svmaxqv_f16, aarch64_sve_fmaxqv, 6597069766662),
SVEMAP1(svmaxqv_f32, aarch64_sve_fmaxqv, 6597069766663),
SVEMAP1(svmaxqv_f64, aarch64_sve_fmaxqv, 6597069766664),
SVEMAP1(svmaxqv_s16, aarch64_sve_smaxqv, 6597069766658),
SVEMAP1(svmaxqv_s32, aarch64_sve_smaxqv, 6597069766659),
SVEMAP1(svmaxqv_s64, aarch64_sve_smaxqv, 6597069766660),
SVEMAP1(svmaxqv_s8, aarch64_sve_smaxqv, 6597069766657),
SVEMAP1(svmaxqv_u16, aarch64_sve_umaxqv, 6597069766658),
SVEMAP1(svmaxqv_u32, aarch64_sve_umaxqv, 6597069766659),
SVEMAP1(svmaxqv_u64, aarch64_sve_umaxqv, 6597069766660),
SVEMAP1(svmaxqv_u8, aarch64_sve_umaxqv, 6597069766657),
SVEMAP1(svmaxv_f16, aarch64_sve_fmaxv, 4398046511110),
SVEMAP1(svmaxv_f32, aarch64_sve_fmaxv, 4398046511111),
SVEMAP1(svmaxv_f64, aarch64_sve_fmaxv, 4398046511112),
SVEMAP1(svmaxv_s16, aarch64_sve_smaxv, 4398046511106),
SVEMAP1(svmaxv_s32, aarch64_sve_smaxv, 4398046511107),
SVEMAP1(svmaxv_s64, aarch64_sve_smaxv, 4398046511108),
SVEMAP1(svmaxv_s8, aarch64_sve_smaxv, 4398046511105),
SVEMAP1(svmaxv_u16, aarch64_sve_umaxv, 4398046511106),
SVEMAP1(svmaxv_u32, aarch64_sve_umaxv, 4398046511107),
SVEMAP1(svmaxv_u64, aarch64_sve_umaxv, 4398046511108),
SVEMAP1(svmaxv_u8, aarch64_sve_umaxv, 4398046511105),
SVEMAP1(svmin_bf16_m, aarch64_sve_fmin, 4398046511373),
SVEMAP1(svmin_bf16_x, aarch64_sve_fmin_u, 4398046511245),
SVEMAP1(svmin_bf16_x2, aarch64_sve_fmin_x2, 137438953485),
SVEMAP1(svmin_bf16_x4, aarch64_sve_fmin_x4, 137438953485),
SVEMAP1(svmin_bf16_z, aarch64_sve_fmin, 4398046511501),
SVEMAP1(svmin_f16_m, aarch64_sve_fmin, 4398046511366),
SVEMAP1(svmin_f16_x, aarch64_sve_fmin_u, 4398046511238),
SVEMAP1(svmin_f16_x2, aarch64_sve_fmin_x2, 137438953478),
SVEMAP1(svmin_f16_x4, aarch64_sve_fmin_x4, 137438953478),
SVEMAP1(svmin_f16_z, aarch64_sve_fmin, 4398046511494),
SVEMAP1(svmin_f32_m, aarch64_sve_fmin, 4398046511367),
SVEMAP1(svmin_f32_x, aarch64_sve_fmin_u, 4398046511239),
SVEMAP1(svmin_f32_x2, aarch64_sve_fmin_x2, 137438953479),
SVEMAP1(svmin_f32_x4, aarch64_sve_fmin_x4, 137438953479),
SVEMAP1(svmin_f32_z, aarch64_sve_fmin, 4398046511495),
SVEMAP1(svmin_f64_m, aarch64_sve_fmin, 4398046511368),
SVEMAP1(svmin_f64_x, aarch64_sve_fmin_u, 4398046511240),
SVEMAP1(svmin_f64_x2, aarch64_sve_fmin_x2, 137438953480),
SVEMAP1(svmin_f64_x4, aarch64_sve_fmin_x4, 137438953480),
SVEMAP1(svmin_f64_z, aarch64_sve_fmin, 4398046511496),
SVEMAP1(svmin_n_bf16_m, aarch64_sve_fmin, 4398046514445),
SVEMAP1(svmin_n_bf16_x, aarch64_sve_fmin_u, 4398046514317),
SVEMAP1(svmin_n_bf16_z, aarch64_sve_fmin, 4398046514573),
SVEMAP1(svmin_n_f16_m, aarch64_sve_fmin, 4398046514438),
SVEMAP1(svmin_n_f16_x, aarch64_sve_fmin_u, 4398046514310),
SVEMAP1(svmin_n_f16_z, aarch64_sve_fmin, 4398046514566),
SVEMAP1(svmin_n_f32_m, aarch64_sve_fmin, 4398046514439),
SVEMAP1(svmin_n_f32_x, aarch64_sve_fmin_u, 4398046514311),
SVEMAP1(svmin_n_f32_z, aarch64_sve_fmin, 4398046514567),
SVEMAP1(svmin_n_f64_m, aarch64_sve_fmin, 4398046514440),
SVEMAP1(svmin_n_f64_x, aarch64_sve_fmin_u, 4398046514312),
SVEMAP1(svmin_n_f64_z, aarch64_sve_fmin, 4398046514568),
SVEMAP1(svmin_n_s16_m, aarch64_sve_smin, 4398046514434),
SVEMAP1(svmin_n_s16_x, aarch64_sve_smin_u, 4398046514306),
SVEMAP1(svmin_n_s16_z, aarch64_sve_smin, 4398046514562),
SVEMAP1(svmin_n_s32_m, aarch64_sve_smin, 4398046514435),
SVEMAP1(svmin_n_s32_x, aarch64_sve_smin_u, 4398046514307),
SVEMAP1(svmin_n_s32_z, aarch64_sve_smin, 4398046514563),
SVEMAP1(svmin_n_s64_m, aarch64_sve_smin, 4398046514436),
SVEMAP1(svmin_n_s64_x, aarch64_sve_smin_u, 4398046514308),
SVEMAP1(svmin_n_s64_z, aarch64_sve_smin, 4398046514564),
SVEMAP1(svmin_n_s8_m, aarch64_sve_smin, 4398046514433),
SVEMAP1(svmin_n_s8_x, aarch64_sve_smin_u, 4398046514305),
SVEMAP1(svmin_n_s8_z, aarch64_sve_smin, 4398046514561),
SVEMAP1(svmin_n_u16_m, aarch64_sve_umin, 4398046514434),
SVEMAP1(svmin_n_u16_x, aarch64_sve_umin_u, 4398046514306),
SVEMAP1(svmin_n_u16_z, aarch64_sve_umin, 4398046514562),
SVEMAP1(svmin_n_u32_m, aarch64_sve_umin, 4398046514435),
SVEMAP1(svmin_n_u32_x, aarch64_sve_umin_u, 4398046514307),
SVEMAP1(svmin_n_u32_z, aarch64_sve_umin, 4398046514563),
SVEMAP1(svmin_n_u64_m, aarch64_sve_umin, 4398046514436),
SVEMAP1(svmin_n_u64_x, aarch64_sve_umin_u, 4398046514308),
SVEMAP1(svmin_n_u64_z, aarch64_sve_umin, 4398046514564),
SVEMAP1(svmin_n_u8_m, aarch64_sve_umin, 4398046514433),
SVEMAP1(svmin_n_u8_x, aarch64_sve_umin_u, 4398046514305),
SVEMAP1(svmin_n_u8_z, aarch64_sve_umin, 4398046514561),
SVEMAP1(svmin_s16_m, aarch64_sve_smin, 4398046511362),
SVEMAP1(svmin_s16_x, aarch64_sve_smin_u, 4398046511234),
SVEMAP1(svmin_s16_x2, aarch64_sve_smin_x2, 137438953474),
SVEMAP1(svmin_s16_x4, aarch64_sve_smin_x4, 137438953474),
SVEMAP1(svmin_s16_z, aarch64_sve_smin, 4398046511490),
SVEMAP1(svmin_s32_m, aarch64_sve_smin, 4398046511363),
SVEMAP1(svmin_s32_x, aarch64_sve_smin_u, 4398046511235),
SVEMAP1(svmin_s32_x2, aarch64_sve_smin_x2, 137438953475),
SVEMAP1(svmin_s32_x4, aarch64_sve_smin_x4, 137438953475),
SVEMAP1(svmin_s32_z, aarch64_sve_smin, 4398046511491),
SVEMAP1(svmin_s64_m, aarch64_sve_smin, 4398046511364),
SVEMAP1(svmin_s64_x, aarch64_sve_smin_u, 4398046511236),
SVEMAP1(svmin_s64_x2, aarch64_sve_smin_x2, 137438953476),
SVEMAP1(svmin_s64_x4, aarch64_sve_smin_x4, 137438953476),
SVEMAP1(svmin_s64_z, aarch64_sve_smin, 4398046511492),
SVEMAP1(svmin_s8_m, aarch64_sve_smin, 4398046511361),
SVEMAP1(svmin_s8_x, aarch64_sve_smin_u, 4398046511233),
SVEMAP1(svmin_s8_x2, aarch64_sve_smin_x2, 137438953473),
SVEMAP1(svmin_s8_x4, aarch64_sve_smin_x4, 137438953473),
SVEMAP1(svmin_s8_z, aarch64_sve_smin, 4398046511489),
SVEMAP1(svmin_single_bf16_x2, aarch64_sve_fmin_single_x2, 137438953485),
SVEMAP1(svmin_single_bf16_x4, aarch64_sve_fmin_single_x4, 137438953485),
SVEMAP1(svmin_single_f16_x2, aarch64_sve_fmin_single_x2, 137438953478),
SVEMAP1(svmin_single_f16_x4, aarch64_sve_fmin_single_x4, 137438953478),
SVEMAP1(svmin_single_f32_x2, aarch64_sve_fmin_single_x2, 137438953479),
SVEMAP1(svmin_single_f32_x4, aarch64_sve_fmin_single_x4, 137438953479),
SVEMAP1(svmin_single_f64_x2, aarch64_sve_fmin_single_x2, 137438953480),
SVEMAP1(svmin_single_f64_x4, aarch64_sve_fmin_single_x4, 137438953480),
SVEMAP1(svmin_single_s16_x2, aarch64_sve_smin_single_x2, 137438953474),
SVEMAP1(svmin_single_s16_x4, aarch64_sve_smin_single_x4, 137438953474),
SVEMAP1(svmin_single_s32_x2, aarch64_sve_smin_single_x2, 137438953475),
SVEMAP1(svmin_single_s32_x4, aarch64_sve_smin_single_x4, 137438953475),
SVEMAP1(svmin_single_s64_x2, aarch64_sve_smin_single_x2, 137438953476),
SVEMAP1(svmin_single_s64_x4, aarch64_sve_smin_single_x4, 137438953476),
SVEMAP1(svmin_single_s8_x2, aarch64_sve_smin_single_x2, 137438953473),
SVEMAP1(svmin_single_s8_x4, aarch64_sve_smin_single_x4, 137438953473),
SVEMAP1(svmin_single_u16_x2, aarch64_sve_umin_single_x2, 137438953474),
SVEMAP1(svmin_single_u16_x4, aarch64_sve_umin_single_x4, 137438953474),
SVEMAP1(svmin_single_u32_x2, aarch64_sve_umin_single_x2, 137438953475),
SVEMAP1(svmin_single_u32_x4, aarch64_sve_umin_single_x4, 137438953475),
SVEMAP1(svmin_single_u64_x2, aarch64_sve_umin_single_x2, 137438953476),
SVEMAP1(svmin_single_u64_x4, aarch64_sve_umin_single_x4, 137438953476),
SVEMAP1(svmin_single_u8_x2, aarch64_sve_umin_single_x2, 137438953473),
SVEMAP1(svmin_single_u8_x4, aarch64_sve_umin_single_x4, 137438953473),
SVEMAP1(svmin_u16_m, aarch64_sve_umin, 4398046511362),
SVEMAP1(svmin_u16_x, aarch64_sve_umin_u, 4398046511234),
SVEMAP1(svmin_u16_x2, aarch64_sve_umin_x2, 137438953474),
SVEMAP1(svmin_u16_x4, aarch64_sve_umin_x4, 137438953474),
SVEMAP1(svmin_u16_z, aarch64_sve_umin, 4398046511490),
SVEMAP1(svmin_u32_m, aarch64_sve_umin, 4398046511363),
SVEMAP1(svmin_u32_x, aarch64_sve_umin_u, 4398046511235),
SVEMAP1(svmin_u32_x2, aarch64_sve_umin_x2, 137438953475),
SVEMAP1(svmin_u32_x4, aarch64_sve_umin_x4, 137438953475),
SVEMAP1(svmin_u32_z, aarch64_sve_umin, 4398046511491),
SVEMAP1(svmin_u64_m, aarch64_sve_umin, 4398046511364),
SVEMAP1(svmin_u64_x, aarch64_sve_umin_u, 4398046511236),
SVEMAP1(svmin_u64_x2, aarch64_sve_umin_x2, 137438953476),
SVEMAP1(svmin_u64_x4, aarch64_sve_umin_x4, 137438953476),
SVEMAP1(svmin_u64_z, aarch64_sve_umin, 4398046511492),
SVEMAP1(svmin_u8_m, aarch64_sve_umin, 4398046511361),
SVEMAP1(svmin_u8_x, aarch64_sve_umin_u, 4398046511233),
SVEMAP1(svmin_u8_x2, aarch64_sve_umin_x2, 137438953473),
SVEMAP1(svmin_u8_x4, aarch64_sve_umin_x4, 137438953473),
SVEMAP1(svmin_u8_z, aarch64_sve_umin, 4398046511489),
SVEMAP1(svminnm_bf16_m, aarch64_sve_fminnm, 4398046511373),
SVEMAP1(svminnm_bf16_x, aarch64_sve_fminnm_u, 4398046511245),
SVEMAP1(svminnm_bf16_x2, aarch64_sve_fminnm_x2, 137438953485),
SVEMAP1(svminnm_bf16_x4, aarch64_sve_fminnm_x4, 137438953485),
SVEMAP1(svminnm_bf16_z, aarch64_sve_fminnm, 4398046511501),
SVEMAP1(svminnm_f16_m, aarch64_sve_fminnm, 4398046511366),
SVEMAP1(svminnm_f16_x, aarch64_sve_fminnm_u, 4398046511238),
SVEMAP1(svminnm_f16_x2, aarch64_sve_fminnm_x2, 137438953478),
SVEMAP1(svminnm_f16_x4, aarch64_sve_fminnm_x4, 137438953478),
SVEMAP1(svminnm_f16_z, aarch64_sve_fminnm, 4398046511494),
SVEMAP1(svminnm_f32_m, aarch64_sve_fminnm, 4398046511367),
SVEMAP1(svminnm_f32_x, aarch64_sve_fminnm_u, 4398046511239),
SVEMAP1(svminnm_f32_x2, aarch64_sve_fminnm_x2, 137438953479),
SVEMAP1(svminnm_f32_x4, aarch64_sve_fminnm_x4, 137438953479),
SVEMAP1(svminnm_f32_z, aarch64_sve_fminnm, 4398046511495),
SVEMAP1(svminnm_f64_m, aarch64_sve_fminnm, 4398046511368),
SVEMAP1(svminnm_f64_x, aarch64_sve_fminnm_u, 4398046511240),
SVEMAP1(svminnm_f64_x2, aarch64_sve_fminnm_x2, 137438953480),
SVEMAP1(svminnm_f64_x4, aarch64_sve_fminnm_x4, 137438953480),
SVEMAP1(svminnm_f64_z, aarch64_sve_fminnm, 4398046511496),
SVEMAP1(svminnm_n_bf16_m, aarch64_sve_fminnm, 4398046514445),
SVEMAP1(svminnm_n_bf16_x, aarch64_sve_fminnm_u, 4398046514317),
SVEMAP1(svminnm_n_bf16_z, aarch64_sve_fminnm, 4398046514573),
SVEMAP1(svminnm_n_f16_m, aarch64_sve_fminnm, 4398046514438),
SVEMAP1(svminnm_n_f16_x, aarch64_sve_fminnm_u, 4398046514310),
SVEMAP1(svminnm_n_f16_z, aarch64_sve_fminnm, 4398046514566),
SVEMAP1(svminnm_n_f32_m, aarch64_sve_fminnm, 4398046514439),
SVEMAP1(svminnm_n_f32_x, aarch64_sve_fminnm_u, 4398046514311),
SVEMAP1(svminnm_n_f32_z, aarch64_sve_fminnm, 4398046514567),
SVEMAP1(svminnm_n_f64_m, aarch64_sve_fminnm, 4398046514440),
SVEMAP1(svminnm_n_f64_x, aarch64_sve_fminnm_u, 4398046514312),
SVEMAP1(svminnm_n_f64_z, aarch64_sve_fminnm, 4398046514568),
SVEMAP1(svminnm_single_bf16_x2, aarch64_sve_fminnm_single_x2, 137438953485),
SVEMAP1(svminnm_single_bf16_x4, aarch64_sve_fminnm_single_x4, 137438953485),
SVEMAP1(svminnm_single_f16_x2, aarch64_sve_fminnm_single_x2, 137438953478),
SVEMAP1(svminnm_single_f16_x4, aarch64_sve_fminnm_single_x4, 137438953478),
SVEMAP1(svminnm_single_f32_x2, aarch64_sve_fminnm_single_x2, 137438953479),
SVEMAP1(svminnm_single_f32_x4, aarch64_sve_fminnm_single_x4, 137438953479),
SVEMAP1(svminnm_single_f64_x2, aarch64_sve_fminnm_single_x2, 137438953480),
SVEMAP1(svminnm_single_f64_x4, aarch64_sve_fminnm_single_x4, 137438953480),
SVEMAP1(svminnmp_f16_m, aarch64_sve_fminnmp, 4398046511366),
SVEMAP1(svminnmp_f16_x, aarch64_sve_fminnmp, 4398046511238),
SVEMAP1(svminnmp_f32_m, aarch64_sve_fminnmp, 4398046511367),
SVEMAP1(svminnmp_f32_x, aarch64_sve_fminnmp, 4398046511239),
SVEMAP1(svminnmp_f64_m, aarch64_sve_fminnmp, 4398046511368),
SVEMAP1(svminnmp_f64_x, aarch64_sve_fminnmp, 4398046511240),
SVEMAP1(svminnmqv_f16, aarch64_sve_fminnmqv, 6597069766662),
SVEMAP1(svminnmqv_f32, aarch64_sve_fminnmqv, 6597069766663),
SVEMAP1(svminnmqv_f64, aarch64_sve_fminnmqv, 6597069766664),
SVEMAP1(svminnmv_f16, aarch64_sve_fminnmv, 4398046511110),
SVEMAP1(svminnmv_f32, aarch64_sve_fminnmv, 4398046511111),
SVEMAP1(svminnmv_f64, aarch64_sve_fminnmv, 4398046511112),
SVEMAP1(svminp_f16_m, aarch64_sve_fminp, 4398046511366),
SVEMAP1(svminp_f16_x, aarch64_sve_fminp, 4398046511238),
SVEMAP1(svminp_f32_m, aarch64_sve_fminp, 4398046511367),
SVEMAP1(svminp_f32_x, aarch64_sve_fminp, 4398046511239),
SVEMAP1(svminp_f64_m, aarch64_sve_fminp, 4398046511368),
SVEMAP1(svminp_f64_x, aarch64_sve_fminp, 4398046511240),
SVEMAP1(svminp_s16_m, aarch64_sve_sminp, 4398046511362),
SVEMAP1(svminp_s16_x, aarch64_sve_sminp, 4398046511234),
SVEMAP1(svminp_s32_m, aarch64_sve_sminp, 4398046511363),
SVEMAP1(svminp_s32_x, aarch64_sve_sminp, 4398046511235),
SVEMAP1(svminp_s64_m, aarch64_sve_sminp, 4398046511364),
SVEMAP1(svminp_s64_x, aarch64_sve_sminp, 4398046511236),
SVEMAP1(svminp_s8_m, aarch64_sve_sminp, 4398046511361),
SVEMAP1(svminp_s8_x, aarch64_sve_sminp, 4398046511233),
SVEMAP1(svminp_u16_m, aarch64_sve_uminp, 4398046511362),
SVEMAP1(svminp_u16_x, aarch64_sve_uminp, 4398046511234),
SVEMAP1(svminp_u32_m, aarch64_sve_uminp, 4398046511363),
SVEMAP1(svminp_u32_x, aarch64_sve_uminp, 4398046511235),
SVEMAP1(svminp_u64_m, aarch64_sve_uminp, 4398046511364),
SVEMAP1(svminp_u64_x, aarch64_sve_uminp, 4398046511236),
SVEMAP1(svminp_u8_m, aarch64_sve_uminp, 4398046511361),
SVEMAP1(svminp_u8_x, aarch64_sve_uminp, 4398046511233),
SVEMAP1(svminqv_f16, aarch64_sve_fminqv, 6597069766662),
SVEMAP1(svminqv_f32, aarch64_sve_fminqv, 6597069766663),
SVEMAP1(svminqv_f64, aarch64_sve_fminqv, 6597069766664),
SVEMAP1(svminqv_s16, aarch64_sve_sminqv, 6597069766658),
SVEMAP1(svminqv_s32, aarch64_sve_sminqv, 6597069766659),
SVEMAP1(svminqv_s64, aarch64_sve_sminqv, 6597069766660),
SVEMAP1(svminqv_s8, aarch64_sve_sminqv, 6597069766657),
SVEMAP1(svminqv_u16, aarch64_sve_uminqv, 6597069766658),
SVEMAP1(svminqv_u32, aarch64_sve_uminqv, 6597069766659),
SVEMAP1(svminqv_u64, aarch64_sve_uminqv, 6597069766660),
SVEMAP1(svminqv_u8, aarch64_sve_uminqv, 6597069766657),
SVEMAP1(svminv_f16, aarch64_sve_fminv, 4398046511110),
SVEMAP1(svminv_f32, aarch64_sve_fminv, 4398046511111),
SVEMAP1(svminv_f64, aarch64_sve_fminv, 4398046511112),
SVEMAP1(svminv_s16, aarch64_sve_sminv, 4398046511106),
SVEMAP1(svminv_s32, aarch64_sve_sminv, 4398046511107),
SVEMAP1(svminv_s64, aarch64_sve_sminv, 4398046511108),
SVEMAP1(svminv_s8, aarch64_sve_sminv, 4398046511105),
SVEMAP1(svminv_u16, aarch64_sve_uminv, 4398046511106),
SVEMAP1(svminv_u32, aarch64_sve_uminv, 4398046511107),
SVEMAP1(svminv_u64, aarch64_sve_uminv, 4398046511108),
SVEMAP1(svminv_u8, aarch64_sve_uminv, 4398046511105),
SVEMAP1(svmla_bf16_m, aarch64_sve_fmla, 4398046511373),
SVEMAP1(svmla_bf16_x, aarch64_sve_fmla_u, 4398046511245),
SVEMAP1(svmla_bf16_z, aarch64_sve_fmla, 4398046511501),
SVEMAP1(svmla_f16_m, aarch64_sve_fmla, 4398046511366),
SVEMAP1(svmla_f16_x, aarch64_sve_fmla_u, 4398046511238),
SVEMAP1(svmla_f16_z, aarch64_sve_fmla, 4398046511494),
SVEMAP1(svmla_f32_m, aarch64_sve_fmla, 4398046511367),
SVEMAP1(svmla_f32_x, aarch64_sve_fmla_u, 4398046511239),
SVEMAP1(svmla_f32_z, aarch64_sve_fmla, 4398046511495),
SVEMAP1(svmla_f64_m, aarch64_sve_fmla, 4398046511368),
SVEMAP1(svmla_f64_x, aarch64_sve_fmla_u, 4398046511240),
SVEMAP1(svmla_f64_z, aarch64_sve_fmla, 4398046511496),
SVEMAP1(svmla_lane_bf16, aarch64_sve_fmla_lane, 4398046511117),
SVEMAP1(svmla_lane_f16, aarch64_sve_fmla_lane, 4398046511110),
SVEMAP1(svmla_lane_f32, aarch64_sve_fmla_lane, 4398046511111),
SVEMAP1(svmla_lane_f64, aarch64_sve_fmla_lane, 4398046511112),
SVEMAP1(svmla_lane_s16, aarch64_sve_mla_lane, 4398046511106),
SVEMAP1(svmla_lane_s32, aarch64_sve_mla_lane, 4398046511107),
SVEMAP1(svmla_lane_s64, aarch64_sve_mla_lane, 4398046511108),
SVEMAP1(svmla_lane_u16, aarch64_sve_mla_lane, 4398046511106),
SVEMAP1(svmla_lane_u32, aarch64_sve_mla_lane, 4398046511107),
SVEMAP1(svmla_lane_u64, aarch64_sve_mla_lane, 4398046511108),
SVEMAP1(svmla_n_bf16_m, aarch64_sve_fmla, 4398046515469),
SVEMAP1(svmla_n_bf16_x, aarch64_sve_fmla_u, 4398046515341),
SVEMAP1(svmla_n_bf16_z, aarch64_sve_fmla, 4398046515597),
SVEMAP1(svmla_n_f16_m, aarch64_sve_fmla, 4398046515462),
SVEMAP1(svmla_n_f16_x, aarch64_sve_fmla_u, 4398046515334),
SVEMAP1(svmla_n_f16_z, aarch64_sve_fmla, 4398046515590),
SVEMAP1(svmla_n_f32_m, aarch64_sve_fmla, 4398046515463),
SVEMAP1(svmla_n_f32_x, aarch64_sve_fmla_u, 4398046515335),
SVEMAP1(svmla_n_f32_z, aarch64_sve_fmla, 4398046515591),
SVEMAP1(svmla_n_f64_m, aarch64_sve_fmla, 4398046515464),
SVEMAP1(svmla_n_f64_x, aarch64_sve_fmla_u, 4398046515336),
SVEMAP1(svmla_n_f64_z, aarch64_sve_fmla, 4398046515592),
SVEMAP1(svmla_n_s16_m, aarch64_sve_mla, 4398046515458),
SVEMAP1(svmla_n_s16_x, aarch64_sve_mla_u, 4398046515330),
SVEMAP1(svmla_n_s16_z, aarch64_sve_mla, 4398046515586),
SVEMAP1(svmla_n_s32_m, aarch64_sve_mla, 4398046515459),
SVEMAP1(svmla_n_s32_x, aarch64_sve_mla_u, 4398046515331),
SVEMAP1(svmla_n_s32_z, aarch64_sve_mla, 4398046515587),
SVEMAP1(svmla_n_s64_m, aarch64_sve_mla, 4398046515460),
SVEMAP1(svmla_n_s64_x, aarch64_sve_mla_u, 4398046515332),
SVEMAP1(svmla_n_s64_z, aarch64_sve_mla, 4398046515588),
SVEMAP1(svmla_n_s8_m, aarch64_sve_mla, 4398046515457),
SVEMAP1(svmla_n_s8_x, aarch64_sve_mla_u, 4398046515329),
SVEMAP1(svmla_n_s8_z, aarch64_sve_mla, 4398046515585),
SVEMAP1(svmla_n_u16_m, aarch64_sve_mla, 4398046515458),
SVEMAP1(svmla_n_u16_x, aarch64_sve_mla_u, 4398046515330),
SVEMAP1(svmla_n_u16_z, aarch64_sve_mla, 4398046515586),
SVEMAP1(svmla_n_u32_m, aarch64_sve_mla, 4398046515459),
SVEMAP1(svmla_n_u32_x, aarch64_sve_mla_u, 4398046515331),
SVEMAP1(svmla_n_u32_z, aarch64_sve_mla, 4398046515587),
SVEMAP1(svmla_n_u64_m, aarch64_sve_mla, 4398046515460),
SVEMAP1(svmla_n_u64_x, aarch64_sve_mla_u, 4398046515332),
SVEMAP1(svmla_n_u64_z, aarch64_sve_mla, 4398046515588),
SVEMAP1(svmla_n_u8_m, aarch64_sve_mla, 4398046515457),
SVEMAP1(svmla_n_u8_x, aarch64_sve_mla_u, 4398046515329),
SVEMAP1(svmla_n_u8_z, aarch64_sve_mla, 4398046515585),
SVEMAP1(svmla_s16_m, aarch64_sve_mla, 4398046511362),
SVEMAP1(svmla_s16_x, aarch64_sve_mla_u, 4398046511234),
SVEMAP1(svmla_s16_z, aarch64_sve_mla, 4398046511490),
SVEMAP1(svmla_s32_m, aarch64_sve_mla, 4398046511363),
SVEMAP1(svmla_s32_x, aarch64_sve_mla_u, 4398046511235),
SVEMAP1(svmla_s32_z, aarch64_sve_mla, 4398046511491),
SVEMAP1(svmla_s64_m, aarch64_sve_mla, 4398046511364),
SVEMAP1(svmla_s64_x, aarch64_sve_mla_u, 4398046511236),
SVEMAP1(svmla_s64_z, aarch64_sve_mla, 4398046511492),
SVEMAP1(svmla_s8_m, aarch64_sve_mla, 4398046511361),
SVEMAP1(svmla_s8_x, aarch64_sve_mla_u, 4398046511233),
SVEMAP1(svmla_s8_z, aarch64_sve_mla, 4398046511489),
SVEMAP1(svmla_u16_m, aarch64_sve_mla, 4398046511362),
SVEMAP1(svmla_u16_x, aarch64_sve_mla_u, 4398046511234),
SVEMAP1(svmla_u16_z, aarch64_sve_mla, 4398046511490),
SVEMAP1(svmla_u32_m, aarch64_sve_mla, 4398046511363),
SVEMAP1(svmla_u32_x, aarch64_sve_mla_u, 4398046511235),
SVEMAP1(svmla_u32_z, aarch64_sve_mla, 4398046511491),
SVEMAP1(svmla_u64_m, aarch64_sve_mla, 4398046511364),
SVEMAP1(svmla_u64_x, aarch64_sve_mla_u, 4398046511236),
SVEMAP1(svmla_u64_z, aarch64_sve_mla, 4398046511492),
SVEMAP1(svmla_u8_m, aarch64_sve_mla, 4398046511361),
SVEMAP1(svmla_u8_x, aarch64_sve_mla_u, 4398046511233),
SVEMAP1(svmla_u8_z, aarch64_sve_mla, 4398046511489),
SVEMAP1(svmlalb_f16_mf8_fpm, aarch64_sve_fp8_fmlalb, 567347999932422),
SVEMAP1(svmlalb_f32, aarch64_sve_fmlalb, 4398046511111),
SVEMAP1(svmlalb_lane_f16_mf8_fpm, aarch64_sve_fp8_fmlalb_lane, 567347999932422),
SVEMAP1(svmlalb_lane_f32, aarch64_sve_fmlalb_lane, 4398046511111),
SVEMAP1(svmlalb_lane_s32, aarch64_sve_smlalb_lane, 4398046511107),
SVEMAP1(svmlalb_lane_s64, aarch64_sve_smlalb_lane, 4398046511108),
SVEMAP1(svmlalb_lane_u32, aarch64_sve_umlalb_lane, 4398046511107),
SVEMAP1(svmlalb_lane_u64, aarch64_sve_umlalb_lane, 4398046511108),
SVEMAP1(svmlalb_n_f16_mf8_fpm, aarch64_sve_fp8_fmlalb, 567347999935494),
SVEMAP1(svmlalb_n_f32, aarch64_sve_fmlalb, 4398046514183),
SVEMAP1(svmlalb_n_s16, aarch64_sve_smlalb, 4398046514178),
SVEMAP1(svmlalb_n_s32, aarch64_sve_smlalb, 4398046514179),
SVEMAP1(svmlalb_n_s64, aarch64_sve_smlalb, 4398046514180),
SVEMAP1(svmlalb_n_u16, aarch64_sve_umlalb, 4398046514178),
SVEMAP1(svmlalb_n_u32, aarch64_sve_umlalb, 4398046514179),
SVEMAP1(svmlalb_n_u64, aarch64_sve_umlalb, 4398046514180),
SVEMAP1(svmlalb_s16, aarch64_sve_smlalb, 4398046511106),
SVEMAP1(svmlalb_s32, aarch64_sve_smlalb, 4398046511107),
SVEMAP1(svmlalb_s64, aarch64_sve_smlalb, 4398046511108),
SVEMAP1(svmlalb_u16, aarch64_sve_umlalb, 4398046511106),
SVEMAP1(svmlalb_u32, aarch64_sve_umlalb, 4398046511107),
SVEMAP1(svmlalb_u64, aarch64_sve_umlalb, 4398046511108),
SVEMAP1(svmlallbb_f32_mf8_fpm, aarch64_sve_fp8_fmlallbb, 567347999932423),
SVEMAP1(svmlallbb_lane_f32_mf8_fpm, aarch64_sve_fp8_fmlallbb_lane, 567347999932423),
SVEMAP1(svmlallbb_n_f32_mf8_fpm, aarch64_sve_fp8_fmlallbb, 567347999935495),
SVEMAP1(svmlallbt_f32_mf8_fpm, aarch64_sve_fp8_fmlallbt, 567347999932423),
SVEMAP1(svmlallbt_lane_f32_mf8_fpm, aarch64_sve_fp8_fmlallbt_lane, 567347999932423),
SVEMAP1(svmlallbt_n_f32_mf8_fpm, aarch64_sve_fp8_fmlallbt, 567347999935495),
SVEMAP1(svmlalltb_f32_mf8_fpm, aarch64_sve_fp8_fmlalltb, 567347999932423),
SVEMAP1(svmlalltb_lane_f32_mf8_fpm, aarch64_sve_fp8_fmlalltb_lane, 567347999932423),
SVEMAP1(svmlalltb_n_f32_mf8_fpm, aarch64_sve_fp8_fmlalltb, 567347999935495),
SVEMAP1(svmlalltt_f32_mf8_fpm, aarch64_sve_fp8_fmlalltt, 567347999932423),
SVEMAP1(svmlalltt_lane_f32_mf8_fpm, aarch64_sve_fp8_fmlalltt_lane, 567347999932423),
SVEMAP1(svmlalltt_n_f32_mf8_fpm, aarch64_sve_fp8_fmlalltt, 567347999935495),
SVEMAP1(svmlalt_f16_mf8_fpm, aarch64_sve_fp8_fmlalt, 567347999932422),
SVEMAP1(svmlalt_f32, aarch64_sve_fmlalt, 4398046511111),
SVEMAP1(svmlalt_lane_f16_mf8_fpm, aarch64_sve_fp8_fmlalt_lane, 567347999932422),
SVEMAP1(svmlalt_lane_f32, aarch64_sve_fmlalt_lane, 4398046511111),
SVEMAP1(svmlalt_lane_s32, aarch64_sve_smlalt_lane, 4398046511107),
SVEMAP1(svmlalt_lane_s64, aarch64_sve_smlalt_lane, 4398046511108),
SVEMAP1(svmlalt_lane_u32, aarch64_sve_umlalt_lane, 4398046511107),
SVEMAP1(svmlalt_lane_u64, aarch64_sve_umlalt_lane, 4398046511108),
SVEMAP1(svmlalt_n_f16_mf8_fpm, aarch64_sve_fp8_fmlalt, 567347999935494),
SVEMAP1(svmlalt_n_f32, aarch64_sve_fmlalt, 4398046514183),
SVEMAP1(svmlalt_n_s16, aarch64_sve_smlalt, 4398046514178),
SVEMAP1(svmlalt_n_s32, aarch64_sve_smlalt, 4398046514179),
SVEMAP1(svmlalt_n_s64, aarch64_sve_smlalt, 4398046514180),
SVEMAP1(svmlalt_n_u16, aarch64_sve_umlalt, 4398046514178),
SVEMAP1(svmlalt_n_u32, aarch64_sve_umlalt, 4398046514179),
SVEMAP1(svmlalt_n_u64, aarch64_sve_umlalt, 4398046514180),
SVEMAP1(svmlalt_s16, aarch64_sve_smlalt, 4398046511106),
SVEMAP1(svmlalt_s32, aarch64_sve_smlalt, 4398046511107),
SVEMAP1(svmlalt_s64, aarch64_sve_smlalt, 4398046511108),
SVEMAP1(svmlalt_u16, aarch64_sve_umlalt, 4398046511106),
SVEMAP1(svmlalt_u32, aarch64_sve_umlalt, 4398046511107),
SVEMAP1(svmlalt_u64, aarch64_sve_umlalt, 4398046511108),
SVEMAP1(svmls_bf16_m, aarch64_sve_fmls, 4398046511373),
SVEMAP1(svmls_bf16_x, aarch64_sve_fmls_u, 4398046511245),
SVEMAP1(svmls_bf16_z, aarch64_sve_fmls, 4398046511501),
SVEMAP1(svmls_f16_m, aarch64_sve_fmls, 4398046511366),
SVEMAP1(svmls_f16_x, aarch64_sve_fmls_u, 4398046511238),
SVEMAP1(svmls_f16_z, aarch64_sve_fmls, 4398046511494),
SVEMAP1(svmls_f32_m, aarch64_sve_fmls, 4398046511367),
SVEMAP1(svmls_f32_x, aarch64_sve_fmls_u, 4398046511239),
SVEMAP1(svmls_f32_z, aarch64_sve_fmls, 4398046511495),
SVEMAP1(svmls_f64_m, aarch64_sve_fmls, 4398046511368),
SVEMAP1(svmls_f64_x, aarch64_sve_fmls_u, 4398046511240),
SVEMAP1(svmls_f64_z, aarch64_sve_fmls, 4398046511496),
SVEMAP1(svmls_lane_bf16, aarch64_sve_fmls_lane, 4398046511117),
SVEMAP1(svmls_lane_f16, aarch64_sve_fmls_lane, 4398046511110),
SVEMAP1(svmls_lane_f32, aarch64_sve_fmls_lane, 4398046511111),
SVEMAP1(svmls_lane_f64, aarch64_sve_fmls_lane, 4398046511112),
SVEMAP1(svmls_lane_s16, aarch64_sve_mls_lane, 4398046511106),
SVEMAP1(svmls_lane_s32, aarch64_sve_mls_lane, 4398046511107),
SVEMAP1(svmls_lane_s64, aarch64_sve_mls_lane, 4398046511108),
SVEMAP1(svmls_lane_u16, aarch64_sve_mls_lane, 4398046511106),
SVEMAP1(svmls_lane_u32, aarch64_sve_mls_lane, 4398046511107),
SVEMAP1(svmls_lane_u64, aarch64_sve_mls_lane, 4398046511108),
SVEMAP1(svmls_n_bf16_m, aarch64_sve_fmls, 4398046515469),
SVEMAP1(svmls_n_bf16_x, aarch64_sve_fmls_u, 4398046515341),
SVEMAP1(svmls_n_bf16_z, aarch64_sve_fmls, 4398046515597),
SVEMAP1(svmls_n_f16_m, aarch64_sve_fmls, 4398046515462),
SVEMAP1(svmls_n_f16_x, aarch64_sve_fmls_u, 4398046515334),
SVEMAP1(svmls_n_f16_z, aarch64_sve_fmls, 4398046515590),
SVEMAP1(svmls_n_f32_m, aarch64_sve_fmls, 4398046515463),
SVEMAP1(svmls_n_f32_x, aarch64_sve_fmls_u, 4398046515335),
SVEMAP1(svmls_n_f32_z, aarch64_sve_fmls, 4398046515591),
SVEMAP1(svmls_n_f64_m, aarch64_sve_fmls, 4398046515464),
SVEMAP1(svmls_n_f64_x, aarch64_sve_fmls_u, 4398046515336),
SVEMAP1(svmls_n_f64_z, aarch64_sve_fmls, 4398046515592),
SVEMAP1(svmls_n_s16_m, aarch64_sve_mls, 4398046515458),
SVEMAP1(svmls_n_s16_x, aarch64_sve_mls_u, 4398046515330),
SVEMAP1(svmls_n_s16_z, aarch64_sve_mls, 4398046515586),
SVEMAP1(svmls_n_s32_m, aarch64_sve_mls, 4398046515459),
SVEMAP1(svmls_n_s32_x, aarch64_sve_mls_u, 4398046515331),
SVEMAP1(svmls_n_s32_z, aarch64_sve_mls, 4398046515587),
SVEMAP1(svmls_n_s64_m, aarch64_sve_mls, 4398046515460),
SVEMAP1(svmls_n_s64_x, aarch64_sve_mls_u, 4398046515332),
SVEMAP1(svmls_n_s64_z, aarch64_sve_mls, 4398046515588),
SVEMAP1(svmls_n_s8_m, aarch64_sve_mls, 4398046515457),
SVEMAP1(svmls_n_s8_x, aarch64_sve_mls_u, 4398046515329),
SVEMAP1(svmls_n_s8_z, aarch64_sve_mls, 4398046515585),
SVEMAP1(svmls_n_u16_m, aarch64_sve_mls, 4398046515458),
SVEMAP1(svmls_n_u16_x, aarch64_sve_mls_u, 4398046515330),
SVEMAP1(svmls_n_u16_z, aarch64_sve_mls, 4398046515586),
SVEMAP1(svmls_n_u32_m, aarch64_sve_mls, 4398046515459),
SVEMAP1(svmls_n_u32_x, aarch64_sve_mls_u, 4398046515331),
SVEMAP1(svmls_n_u32_z, aarch64_sve_mls, 4398046515587),
SVEMAP1(svmls_n_u64_m, aarch64_sve_mls, 4398046515460),
SVEMAP1(svmls_n_u64_x, aarch64_sve_mls_u, 4398046515332),
SVEMAP1(svmls_n_u64_z, aarch64_sve_mls, 4398046515588),
SVEMAP1(svmls_n_u8_m, aarch64_sve_mls, 4398046515457),
SVEMAP1(svmls_n_u8_x, aarch64_sve_mls_u, 4398046515329),
SVEMAP1(svmls_n_u8_z, aarch64_sve_mls, 4398046515585),
SVEMAP1(svmls_s16_m, aarch64_sve_mls, 4398046511362),
SVEMAP1(svmls_s16_x, aarch64_sve_mls_u, 4398046511234),
SVEMAP1(svmls_s16_z, aarch64_sve_mls, 4398046511490),
SVEMAP1(svmls_s32_m, aarch64_sve_mls, 4398046511363),
SVEMAP1(svmls_s32_x, aarch64_sve_mls_u, 4398046511235),
SVEMAP1(svmls_s32_z, aarch64_sve_mls, 4398046511491),
SVEMAP1(svmls_s64_m, aarch64_sve_mls, 4398046511364),
SVEMAP1(svmls_s64_x, aarch64_sve_mls_u, 4398046511236),
SVEMAP1(svmls_s64_z, aarch64_sve_mls, 4398046511492),
SVEMAP1(svmls_s8_m, aarch64_sve_mls, 4398046511361),
SVEMAP1(svmls_s8_x, aarch64_sve_mls_u, 4398046511233),
SVEMAP1(svmls_s8_z, aarch64_sve_mls, 4398046511489),
SVEMAP1(svmls_u16_m, aarch64_sve_mls, 4398046511362),
SVEMAP1(svmls_u16_x, aarch64_sve_mls_u, 4398046511234),
SVEMAP1(svmls_u16_z, aarch64_sve_mls, 4398046511490),
SVEMAP1(svmls_u32_m, aarch64_sve_mls, 4398046511363),
SVEMAP1(svmls_u32_x, aarch64_sve_mls_u, 4398046511235),
SVEMAP1(svmls_u32_z, aarch64_sve_mls, 4398046511491),
SVEMAP1(svmls_u64_m, aarch64_sve_mls, 4398046511364),
SVEMAP1(svmls_u64_x, aarch64_sve_mls_u, 4398046511236),
SVEMAP1(svmls_u64_z, aarch64_sve_mls, 4398046511492),
SVEMAP1(svmls_u8_m, aarch64_sve_mls, 4398046511361),
SVEMAP1(svmls_u8_x, aarch64_sve_mls_u, 4398046511233),
SVEMAP1(svmls_u8_z, aarch64_sve_mls, 4398046511489),
SVEMAP1(svmlslb_f32, aarch64_sve_fmlslb, 4398046511111),
SVEMAP1(svmlslb_lane_f32, aarch64_sve_fmlslb_lane, 4398046511111),
SVEMAP1(svmlslb_lane_s32, aarch64_sve_smlslb_lane, 4398046511107),
SVEMAP1(svmlslb_lane_s64, aarch64_sve_smlslb_lane, 4398046511108),
SVEMAP1(svmlslb_lane_u32, aarch64_sve_umlslb_lane, 4398046511107),
SVEMAP1(svmlslb_lane_u64, aarch64_sve_umlslb_lane, 4398046511108),
SVEMAP1(svmlslb_n_f32, aarch64_sve_fmlslb, 4398046514183),
SVEMAP1(svmlslb_n_s16, aarch64_sve_smlslb, 4398046514178),
SVEMAP1(svmlslb_n_s32, aarch64_sve_smlslb, 4398046514179),
SVEMAP1(svmlslb_n_s64, aarch64_sve_smlslb, 4398046514180),
SVEMAP1(svmlslb_n_u16, aarch64_sve_umlslb, 4398046514178),
SVEMAP1(svmlslb_n_u32, aarch64_sve_umlslb, 4398046514179),
SVEMAP1(svmlslb_n_u64, aarch64_sve_umlslb, 4398046514180),
SVEMAP1(svmlslb_s16, aarch64_sve_smlslb, 4398046511106),
SVEMAP1(svmlslb_s32, aarch64_sve_smlslb, 4398046511107),
SVEMAP1(svmlslb_s64, aarch64_sve_smlslb, 4398046511108),
SVEMAP1(svmlslb_u16, aarch64_sve_umlslb, 4398046511106),
SVEMAP1(svmlslb_u32, aarch64_sve_umlslb, 4398046511107),
SVEMAP1(svmlslb_u64, aarch64_sve_umlslb, 4398046511108),
SVEMAP1(svmlslt_f32, aarch64_sve_fmlslt, 4398046511111),
SVEMAP1(svmlslt_lane_f32, aarch64_sve_fmlslt_lane, 4398046511111),
SVEMAP1(svmlslt_lane_s32, aarch64_sve_smlslt_lane, 4398046511107),
SVEMAP1(svmlslt_lane_s64, aarch64_sve_smlslt_lane, 4398046511108),
SVEMAP1(svmlslt_lane_u32, aarch64_sve_umlslt_lane, 4398046511107),
SVEMAP1(svmlslt_lane_u64, aarch64_sve_umlslt_lane, 4398046511108),
SVEMAP1(svmlslt_n_f32, aarch64_sve_fmlslt, 4398046514183),
SVEMAP1(svmlslt_n_s16, aarch64_sve_smlslt, 4398046514178),
SVEMAP1(svmlslt_n_s32, aarch64_sve_smlslt, 4398046514179),
SVEMAP1(svmlslt_n_s64, aarch64_sve_smlslt, 4398046514180),
SVEMAP1(svmlslt_n_u16, aarch64_sve_umlslt, 4398046514178),
SVEMAP1(svmlslt_n_u32, aarch64_sve_umlslt, 4398046514179),
SVEMAP1(svmlslt_n_u64, aarch64_sve_umlslt, 4398046514180),
SVEMAP1(svmlslt_s16, aarch64_sve_smlslt, 4398046511106),
SVEMAP1(svmlslt_s32, aarch64_sve_smlslt, 4398046511107),
SVEMAP1(svmlslt_s64, aarch64_sve_smlslt, 4398046511108),
SVEMAP1(svmlslt_u16, aarch64_sve_umlslt, 4398046511106),
SVEMAP1(svmlslt_u32, aarch64_sve_umlslt, 4398046511107),
SVEMAP1(svmlslt_u64, aarch64_sve_umlslt, 4398046511108),
SVEMAP1(svmmla_f32, aarch64_sve_fmmla, 7),
SVEMAP1(svmmla_f64, aarch64_sve_fmmla, 8),
SVEMAP1(svmmla_s32, aarch64_sve_smmla, 3),
SVEMAP1(svmmla_u32, aarch64_sve_ummla, 3),
SVEMAP2(svmov_b_z, 4398046511113),
SVEMAP2(svmovlb_s16, 4398046511106),
SVEMAP2(svmovlb_s32, 4398046511107),
SVEMAP2(svmovlb_s64, 4398046511108),
SVEMAP2(svmovlb_u16, 4398046511106),
SVEMAP2(svmovlb_u32, 4398046511107),
SVEMAP2(svmovlb_u64, 4398046511108),
SVEMAP2(svmovlt_s16, 4398046511106),
SVEMAP2(svmovlt_s32, 4398046511107),
SVEMAP2(svmovlt_s64, 4398046511108),
SVEMAP2(svmovlt_u16, 4398046511106),
SVEMAP2(svmovlt_u32, 4398046511107),
SVEMAP2(svmovlt_u64, 4398046511108),
SVEMAP1(svmsb_f16_m, aarch64_sve_fmsb, 4466765988102),
SVEMAP1(svmsb_f16_x, aarch64_sve_fmls_u, 4466765987974),
SVEMAP1(svmsb_f16_z, aarch64_sve_fmsb, 4466765988230),
SVEMAP1(svmsb_f32_m, aarch64_sve_fmsb, 4466765988103),
SVEMAP1(svmsb_f32_x, aarch64_sve_fmls_u, 4466765987975),
SVEMAP1(svmsb_f32_z, aarch64_sve_fmsb, 4466765988231),
SVEMAP1(svmsb_f64_m, aarch64_sve_fmsb, 4466765988104),
SVEMAP1(svmsb_f64_x, aarch64_sve_fmls_u, 4466765987976),
SVEMAP1(svmsb_f64_z, aarch64_sve_fmsb, 4466765988232),
SVEMAP1(svmsb_n_f16_m, aarch64_sve_fmsb, 4466765992198),
SVEMAP1(svmsb_n_f16_x, aarch64_sve_fmls_u, 4466765992070),
SVEMAP1(svmsb_n_f16_z, aarch64_sve_fmsb, 4466765992326),
SVEMAP1(svmsb_n_f32_m, aarch64_sve_fmsb, 4466765992199),
SVEMAP1(svmsb_n_f32_x, aarch64_sve_fmls_u, 4466765992071),
SVEMAP1(svmsb_n_f32_z, aarch64_sve_fmsb, 4466765992327),
SVEMAP1(svmsb_n_f64_m, aarch64_sve_fmsb, 4466765992200),
SVEMAP1(svmsb_n_f64_x, aarch64_sve_fmls_u, 4466765992072),
SVEMAP1(svmsb_n_f64_z, aarch64_sve_fmsb, 4466765992328),
SVEMAP1(svmsb_n_s16_m, aarch64_sve_msb, 4466765992194),
SVEMAP1(svmsb_n_s16_x, aarch64_sve_mls_u, 4466765992066),
SVEMAP1(svmsb_n_s16_z, aarch64_sve_msb, 4466765992322),
SVEMAP1(svmsb_n_s32_m, aarch64_sve_msb, 4466765992195),
SVEMAP1(svmsb_n_s32_x, aarch64_sve_mls_u, 4466765992067),
SVEMAP1(svmsb_n_s32_z, aarch64_sve_msb, 4466765992323),
SVEMAP1(svmsb_n_s64_m, aarch64_sve_msb, 4466765992196),
SVEMAP1(svmsb_n_s64_x, aarch64_sve_mls_u, 4466765992068),
SVEMAP1(svmsb_n_s64_z, aarch64_sve_msb, 4466765992324),
SVEMAP1(svmsb_n_s8_m, aarch64_sve_msb, 4466765992193),
SVEMAP1(svmsb_n_s8_x, aarch64_sve_mls_u, 4466765992065),
SVEMAP1(svmsb_n_s8_z, aarch64_sve_msb, 4466765992321),
SVEMAP1(svmsb_n_u16_m, aarch64_sve_msb, 4466765992194),
SVEMAP1(svmsb_n_u16_x, aarch64_sve_mls_u, 4466765992066),
SVEMAP1(svmsb_n_u16_z, aarch64_sve_msb, 4466765992322),
SVEMAP1(svmsb_n_u32_m, aarch64_sve_msb, 4466765992195),
SVEMAP1(svmsb_n_u32_x, aarch64_sve_mls_u, 4466765992067),
SVEMAP1(svmsb_n_u32_z, aarch64_sve_msb, 4466765992323),
SVEMAP1(svmsb_n_u64_m, aarch64_sve_msb, 4466765992196),
SVEMAP1(svmsb_n_u64_x, aarch64_sve_mls_u, 4466765992068),
SVEMAP1(svmsb_n_u64_z, aarch64_sve_msb, 4466765992324),
SVEMAP1(svmsb_n_u8_m, aarch64_sve_msb, 4466765992193),
SVEMAP1(svmsb_n_u8_x, aarch64_sve_mls_u, 4466765992065),
SVEMAP1(svmsb_n_u8_z, aarch64_sve_msb, 4466765992321),
SVEMAP1(svmsb_s16_m, aarch64_sve_msb, 4466765988098),
SVEMAP1(svmsb_s16_x, aarch64_sve_mls_u, 4466765987970),
SVEMAP1(svmsb_s16_z, aarch64_sve_msb, 4466765988226),
SVEMAP1(svmsb_s32_m, aarch64_sve_msb, 4466765988099),
SVEMAP1(svmsb_s32_x, aarch64_sve_mls_u, 4466765987971),
SVEMAP1(svmsb_s32_z, aarch64_sve_msb, 4466765988227),
SVEMAP1(svmsb_s64_m, aarch64_sve_msb, 4466765988100),
SVEMAP1(svmsb_s64_x, aarch64_sve_mls_u, 4466765987972),
SVEMAP1(svmsb_s64_z, aarch64_sve_msb, 4466765988228),
SVEMAP1(svmsb_s8_m, aarch64_sve_msb, 4466765988097),
SVEMAP1(svmsb_s8_x, aarch64_sve_mls_u, 4466765987969),
SVEMAP1(svmsb_s8_z, aarch64_sve_msb, 4466765988225),
SVEMAP1(svmsb_u16_m, aarch64_sve_msb, 4466765988098),
SVEMAP1(svmsb_u16_x, aarch64_sve_mls_u, 4466765987970),
SVEMAP1(svmsb_u16_z, aarch64_sve_msb, 4466765988226),
SVEMAP1(svmsb_u32_m, aarch64_sve_msb, 4466765988099),
SVEMAP1(svmsb_u32_x, aarch64_sve_mls_u, 4466765987971),
SVEMAP1(svmsb_u32_z, aarch64_sve_msb, 4466765988227),
SVEMAP1(svmsb_u64_m, aarch64_sve_msb, 4466765988100),
SVEMAP1(svmsb_u64_x, aarch64_sve_mls_u, 4466765987972),
SVEMAP1(svmsb_u64_z, aarch64_sve_msb, 4466765988228),
SVEMAP1(svmsb_u8_m, aarch64_sve_msb, 4466765988097),
SVEMAP1(svmsb_u8_x, aarch64_sve_mls_u, 4466765987969),
SVEMAP1(svmsb_u8_z, aarch64_sve_msb, 4466765988225),
SVEMAP1(svmul_bf16_m, aarch64_sve_fmul, 4398046511373),
SVEMAP1(svmul_bf16_x, aarch64_sve_fmul_u, 4398046511245),
SVEMAP1(svmul_bf16_z, aarch64_sve_fmul, 4398046511501),
SVEMAP1(svmul_f16_m, aarch64_sve_fmul, 4398046511366),
SVEMAP1(svmul_f16_x, aarch64_sve_fmul_u, 4398046511238),
SVEMAP1(svmul_f16_z, aarch64_sve_fmul, 4398046511494),
SVEMAP1(svmul_f32_m, aarch64_sve_fmul, 4398046511367),
SVEMAP1(svmul_f32_x, aarch64_sve_fmul_u, 4398046511239),
SVEMAP1(svmul_f32_z, aarch64_sve_fmul, 4398046511495),
SVEMAP1(svmul_f64_m, aarch64_sve_fmul, 4398046511368),
SVEMAP1(svmul_f64_x, aarch64_sve_fmul_u, 4398046511240),
SVEMAP1(svmul_f64_z, aarch64_sve_fmul, 4398046511496),
SVEMAP1(svmul_lane_bf16, aarch64_sve_fmul_lane, 4398046511117),
SVEMAP1(svmul_lane_f16, aarch64_sve_fmul_lane, 4398046511110),
SVEMAP1(svmul_lane_f32, aarch64_sve_fmul_lane, 4398046511111),
SVEMAP1(svmul_lane_f64, aarch64_sve_fmul_lane, 4398046511112),
SVEMAP1(svmul_lane_s16, aarch64_sve_mul_lane, 4398046511106),
SVEMAP1(svmul_lane_s32, aarch64_sve_mul_lane, 4398046511107),
SVEMAP1(svmul_lane_s64, aarch64_sve_mul_lane, 4398046511108),
SVEMAP1(svmul_lane_u16, aarch64_sve_mul_lane, 4398046511106),
SVEMAP1(svmul_lane_u32, aarch64_sve_mul_lane, 4398046511107),
SVEMAP1(svmul_lane_u64, aarch64_sve_mul_lane, 4398046511108),
SVEMAP1(svmul_n_bf16_m, aarch64_sve_fmul, 4398046514445),
SVEMAP1(svmul_n_bf16_x, aarch64_sve_fmul_u, 4398046514317),
SVEMAP1(svmul_n_bf16_z, aarch64_sve_fmul, 4398046514573),
SVEMAP1(svmul_n_f16_m, aarch64_sve_fmul, 4398046514438),
SVEMAP1(svmul_n_f16_x, aarch64_sve_fmul_u, 4398046514310),
SVEMAP1(svmul_n_f16_z, aarch64_sve_fmul, 4398046514566),
SVEMAP1(svmul_n_f32_m, aarch64_sve_fmul, 4398046514439),
SVEMAP1(svmul_n_f32_x, aarch64_sve_fmul_u, 4398046514311),
SVEMAP1(svmul_n_f32_z, aarch64_sve_fmul, 4398046514567),
SVEMAP1(svmul_n_f64_m, aarch64_sve_fmul, 4398046514440),
SVEMAP1(svmul_n_f64_x, aarch64_sve_fmul_u, 4398046514312),
SVEMAP1(svmul_n_f64_z, aarch64_sve_fmul, 4398046514568),
SVEMAP1(svmul_n_s16_m, aarch64_sve_mul, 4398046514434),
SVEMAP1(svmul_n_s16_x, aarch64_sve_mul_u, 4398046514306),
SVEMAP1(svmul_n_s16_z, aarch64_sve_mul, 4398046514562),
SVEMAP1(svmul_n_s32_m, aarch64_sve_mul, 4398046514435),
SVEMAP1(svmul_n_s32_x, aarch64_sve_mul_u, 4398046514307),
SVEMAP1(svmul_n_s32_z, aarch64_sve_mul, 4398046514563),
SVEMAP1(svmul_n_s64_m, aarch64_sve_mul, 4398046514436),
SVEMAP1(svmul_n_s64_x, aarch64_sve_mul_u, 4398046514308),
SVEMAP1(svmul_n_s64_z, aarch64_sve_mul, 4398046514564),
SVEMAP1(svmul_n_s8_m, aarch64_sve_mul, 4398046514433),
SVEMAP1(svmul_n_s8_x, aarch64_sve_mul_u, 4398046514305),
SVEMAP1(svmul_n_s8_z, aarch64_sve_mul, 4398046514561),
SVEMAP1(svmul_n_u16_m, aarch64_sve_mul, 4398046514434),
SVEMAP1(svmul_n_u16_x, aarch64_sve_mul_u, 4398046514306),
SVEMAP1(svmul_n_u16_z, aarch64_sve_mul, 4398046514562),
SVEMAP1(svmul_n_u32_m, aarch64_sve_mul, 4398046514435),
SVEMAP1(svmul_n_u32_x, aarch64_sve_mul_u, 4398046514307),
SVEMAP1(svmul_n_u32_z, aarch64_sve_mul, 4398046514563),
SVEMAP1(svmul_n_u64_m, aarch64_sve_mul, 4398046514436),
SVEMAP1(svmul_n_u64_x, aarch64_sve_mul_u, 4398046514308),
SVEMAP1(svmul_n_u64_z, aarch64_sve_mul, 4398046514564),
SVEMAP1(svmul_n_u8_m, aarch64_sve_mul, 4398046514433),
SVEMAP1(svmul_n_u8_x, aarch64_sve_mul_u, 4398046514305),
SVEMAP1(svmul_n_u8_z, aarch64_sve_mul, 4398046514561),
SVEMAP1(svmul_s16_m, aarch64_sve_mul, 4398046511362),
SVEMAP1(svmul_s16_x, aarch64_sve_mul_u, 4398046511234),
SVEMAP1(svmul_s16_z, aarch64_sve_mul, 4398046511490),
SVEMAP1(svmul_s32_m, aarch64_sve_mul, 4398046511363),
SVEMAP1(svmul_s32_x, aarch64_sve_mul_u, 4398046511235),
SVEMAP1(svmul_s32_z, aarch64_sve_mul, 4398046511491),
SVEMAP1(svmul_s64_m, aarch64_sve_mul, 4398046511364),
SVEMAP1(svmul_s64_x, aarch64_sve_mul_u, 4398046511236),
SVEMAP1(svmul_s64_z, aarch64_sve_mul, 4398046511492),
SVEMAP1(svmul_s8_m, aarch64_sve_mul, 4398046511361),
SVEMAP1(svmul_s8_x, aarch64_sve_mul_u, 4398046511233),
SVEMAP1(svmul_s8_z, aarch64_sve_mul, 4398046511489),
SVEMAP1(svmul_u16_m, aarch64_sve_mul, 4398046511362),
SVEMAP1(svmul_u16_x, aarch64_sve_mul_u, 4398046511234),
SVEMAP1(svmul_u16_z, aarch64_sve_mul, 4398046511490),
SVEMAP1(svmul_u32_m, aarch64_sve_mul, 4398046511363),
SVEMAP1(svmul_u32_x, aarch64_sve_mul_u, 4398046511235),
SVEMAP1(svmul_u32_z, aarch64_sve_mul, 4398046511491),
SVEMAP1(svmul_u64_m, aarch64_sve_mul, 4398046511364),
SVEMAP1(svmul_u64_x, aarch64_sve_mul_u, 4398046511236),
SVEMAP1(svmul_u64_z, aarch64_sve_mul, 4398046511492),
SVEMAP1(svmul_u8_m, aarch64_sve_mul, 4398046511361),
SVEMAP1(svmul_u8_x, aarch64_sve_mul_u, 4398046511233),
SVEMAP1(svmul_u8_z, aarch64_sve_mul, 4398046511489),
SVEMAP1(svmulh_n_s16_m, aarch64_sve_smulh, 4398046514434),
SVEMAP1(svmulh_n_s16_x, aarch64_sve_smulh_u, 4398046514306),
SVEMAP1(svmulh_n_s16_z, aarch64_sve_smulh, 4398046514562),
SVEMAP1(svmulh_n_s32_m, aarch64_sve_smulh, 4398046514435),
SVEMAP1(svmulh_n_s32_x, aarch64_sve_smulh_u, 4398046514307),
SVEMAP1(svmulh_n_s32_z, aarch64_sve_smulh, 4398046514563),
SVEMAP1(svmulh_n_s64_m, aarch64_sve_smulh, 4398046514436),
SVEMAP1(svmulh_n_s64_x, aarch64_sve_smulh_u, 4398046514308),
SVEMAP1(svmulh_n_s64_z, aarch64_sve_smulh, 4398046514564),
SVEMAP1(svmulh_n_s8_m, aarch64_sve_smulh, 4398046514433),
SVEMAP1(svmulh_n_s8_x, aarch64_sve_smulh_u, 4398046514305),
SVEMAP1(svmulh_n_s8_z, aarch64_sve_smulh, 4398046514561),
SVEMAP1(svmulh_n_u16_m, aarch64_sve_umulh, 4398046514434),
SVEMAP1(svmulh_n_u16_x, aarch64_sve_umulh_u, 4398046514306),
SVEMAP1(svmulh_n_u16_z, aarch64_sve_umulh, 4398046514562),
SVEMAP1(svmulh_n_u32_m, aarch64_sve_umulh, 4398046514435),
SVEMAP1(svmulh_n_u32_x, aarch64_sve_umulh_u, 4398046514307),
SVEMAP1(svmulh_n_u32_z, aarch64_sve_umulh, 4398046514563),
SVEMAP1(svmulh_n_u64_m, aarch64_sve_umulh, 4398046514436),
SVEMAP1(svmulh_n_u64_x, aarch64_sve_umulh_u, 4398046514308),
SVEMAP1(svmulh_n_u64_z, aarch64_sve_umulh, 4398046514564),
SVEMAP1(svmulh_n_u8_m, aarch64_sve_umulh, 4398046514433),
SVEMAP1(svmulh_n_u8_x, aarch64_sve_umulh_u, 4398046514305),
SVEMAP1(svmulh_n_u8_z, aarch64_sve_umulh, 4398046514561),
SVEMAP1(svmulh_s16_m, aarch64_sve_smulh, 4398046511362),
SVEMAP1(svmulh_s16_x, aarch64_sve_smulh_u, 4398046511234),
SVEMAP1(svmulh_s16_z, aarch64_sve_smulh, 4398046511490),
SVEMAP1(svmulh_s32_m, aarch64_sve_smulh, 4398046511363),
SVEMAP1(svmulh_s32_x, aarch64_sve_smulh_u, 4398046511235),
SVEMAP1(svmulh_s32_z, aarch64_sve_smulh, 4398046511491),
SVEMAP1(svmulh_s64_m, aarch64_sve_smulh, 4398046511364),
SVEMAP1(svmulh_s64_x, aarch64_sve_smulh_u, 4398046511236),
SVEMAP1(svmulh_s64_z, aarch64_sve_smulh, 4398046511492),
SVEMAP1(svmulh_s8_m, aarch64_sve_smulh, 4398046511361),
SVEMAP1(svmulh_s8_x, aarch64_sve_smulh_u, 4398046511233),
SVEMAP1(svmulh_s8_z, aarch64_sve_smulh, 4398046511489),
SVEMAP1(svmulh_u16_m, aarch64_sve_umulh, 4398046511362),
SVEMAP1(svmulh_u16_x, aarch64_sve_umulh_u, 4398046511234),
SVEMAP1(svmulh_u16_z, aarch64_sve_umulh, 4398046511490),
SVEMAP1(svmulh_u32_m, aarch64_sve_umulh, 4398046511363),
SVEMAP1(svmulh_u32_x, aarch64_sve_umulh_u, 4398046511235),
SVEMAP1(svmulh_u32_z, aarch64_sve_umulh, 4398046511491),
SVEMAP1(svmulh_u64_m, aarch64_sve_umulh, 4398046511364),
SVEMAP1(svmulh_u64_x, aarch64_sve_umulh_u, 4398046511236),
SVEMAP1(svmulh_u64_z, aarch64_sve_umulh, 4398046511492),
SVEMAP1(svmulh_u8_m, aarch64_sve_umulh, 4398046511361),
SVEMAP1(svmulh_u8_x, aarch64_sve_umulh_u, 4398046511233),
SVEMAP1(svmulh_u8_z, aarch64_sve_umulh, 4398046511489),
SVEMAP1(svmullb_lane_s32, aarch64_sve_smullb_lane, 4398046511107),
SVEMAP1(svmullb_lane_s64, aarch64_sve_smullb_lane, 4398046511108),
SVEMAP1(svmullb_lane_u32, aarch64_sve_umullb_lane, 4398046511107),
SVEMAP1(svmullb_lane_u64, aarch64_sve_umullb_lane, 4398046511108),
SVEMAP1(svmullb_n_s16, aarch64_sve_smullb, 4398046513154),
SVEMAP1(svmullb_n_s32, aarch64_sve_smullb, 4398046513155),
SVEMAP1(svmullb_n_s64, aarch64_sve_smullb, 4398046513156),
SVEMAP1(svmullb_n_u16, aarch64_sve_umullb, 4398046513154),
SVEMAP1(svmullb_n_u32, aarch64_sve_umullb, 4398046513155),
SVEMAP1(svmullb_n_u64, aarch64_sve_umullb, 4398046513156),
SVEMAP1(svmullb_s16, aarch64_sve_smullb, 4398046511106),
SVEMAP1(svmullb_s32, aarch64_sve_smullb, 4398046511107),
SVEMAP1(svmullb_s64, aarch64_sve_smullb, 4398046511108),
SVEMAP1(svmullb_u16, aarch64_sve_umullb, 4398046511106),
SVEMAP1(svmullb_u32, aarch64_sve_umullb, 4398046511107),
SVEMAP1(svmullb_u64, aarch64_sve_umullb, 4398046511108),
SVEMAP1(svmullt_lane_s32, aarch64_sve_smullt_lane, 4398046511107),
SVEMAP1(svmullt_lane_s64, aarch64_sve_smullt_lane, 4398046511108),
SVEMAP1(svmullt_lane_u32, aarch64_sve_umullt_lane, 4398046511107),
SVEMAP1(svmullt_lane_u64, aarch64_sve_umullt_lane, 4398046511108),
SVEMAP1(svmullt_n_s16, aarch64_sve_smullt, 4398046513154),
SVEMAP1(svmullt_n_s32, aarch64_sve_smullt, 4398046513155),
SVEMAP1(svmullt_n_s64, aarch64_sve_smullt, 4398046513156),
SVEMAP1(svmullt_n_u16, aarch64_sve_umullt, 4398046513154),
SVEMAP1(svmullt_n_u32, aarch64_sve_umullt, 4398046513155),
SVEMAP1(svmullt_n_u64, aarch64_sve_umullt, 4398046513156),
SVEMAP1(svmullt_s16, aarch64_sve_smullt, 4398046511106),
SVEMAP1(svmullt_s32, aarch64_sve_smullt, 4398046511107),
SVEMAP1(svmullt_s64, aarch64_sve_smullt, 4398046511108),
SVEMAP1(svmullt_u16, aarch64_sve_umullt, 4398046511106),
SVEMAP1(svmullt_u32, aarch64_sve_umullt, 4398046511107),
SVEMAP1(svmullt_u64, aarch64_sve_umullt, 4398046511108),
SVEMAP1(svmulx_f16_m, aarch64_sve_fmulx, 4398046511366),
SVEMAP1(svmulx_f16_x, aarch64_sve_fmulx_u, 4398046511238),
SVEMAP1(svmulx_f16_z, aarch64_sve_fmulx, 4398046511494),
SVEMAP1(svmulx_f32_m, aarch64_sve_fmulx, 4398046511367),
SVEMAP1(svmulx_f32_x, aarch64_sve_fmulx_u, 4398046511239),
SVEMAP1(svmulx_f32_z, aarch64_sve_fmulx, 4398046511495),
SVEMAP1(svmulx_f64_m, aarch64_sve_fmulx, 4398046511368),
SVEMAP1(svmulx_f64_x, aarch64_sve_fmulx_u, 4398046511240),
SVEMAP1(svmulx_f64_z, aarch64_sve_fmulx, 4398046511496),
SVEMAP1(svmulx_n_f16_m, aarch64_sve_fmulx, 4398046514438),
SVEMAP1(svmulx_n_f16_x, aarch64_sve_fmulx_u, 4398046514310),
SVEMAP1(svmulx_n_f16_z, aarch64_sve_fmulx, 4398046514566),
SVEMAP1(svmulx_n_f32_m, aarch64_sve_fmulx, 4398046514439),
SVEMAP1(svmulx_n_f32_x, aarch64_sve_fmulx_u, 4398046514311),
SVEMAP1(svmulx_n_f32_z, aarch64_sve_fmulx, 4398046514567),
SVEMAP1(svmulx_n_f64_m, aarch64_sve_fmulx, 4398046514440),
SVEMAP1(svmulx_n_f64_x, aarch64_sve_fmulx_u, 4398046514312),
SVEMAP1(svmulx_n_f64_z, aarch64_sve_fmulx, 4398046514568),
SVEMAP1(svnand_b_z, aarch64_sve_nand_z, 4398046511113),
SVEMAP1(svnbsl_n_s16, aarch64_sve_nbsl, 4398046514178),
SVEMAP1(svnbsl_n_s32, aarch64_sve_nbsl, 4398046514179),
SVEMAP1(svnbsl_n_s64, aarch64_sve_nbsl, 4398046514180),
SVEMAP1(svnbsl_n_s8, aarch64_sve_nbsl, 4398046514177),
SVEMAP1(svnbsl_n_u16, aarch64_sve_nbsl, 4398046514178),
SVEMAP1(svnbsl_n_u32, aarch64_sve_nbsl, 4398046514179),
SVEMAP1(svnbsl_n_u64, aarch64_sve_nbsl, 4398046514180),
SVEMAP1(svnbsl_n_u8, aarch64_sve_nbsl, 4398046514177),
SVEMAP1(svnbsl_s16, aarch64_sve_nbsl, 4398046511106),
SVEMAP1(svnbsl_s32, aarch64_sve_nbsl, 4398046511107),
SVEMAP1(svnbsl_s64, aarch64_sve_nbsl, 4398046511108),
SVEMAP1(svnbsl_s8, aarch64_sve_nbsl, 4398046511105),
SVEMAP1(svnbsl_u16, aarch64_sve_nbsl, 4398046511106),
SVEMAP1(svnbsl_u32, aarch64_sve_nbsl, 4398046511107),
SVEMAP1(svnbsl_u64, aarch64_sve_nbsl, 4398046511108),
SVEMAP1(svnbsl_u8, aarch64_sve_nbsl, 4398046511105),
SVEMAP1(svneg_f16_m, aarch64_sve_fneg, 4398046511366),
SVEMAP1(svneg_f16_x, aarch64_sve_fneg, 4398046511622),
SVEMAP1(svneg_f16_z, aarch64_sve_fneg, 4398046511750),
SVEMAP1(svneg_f32_m, aarch64_sve_fneg, 4398046511367),
SVEMAP1(svneg_f32_x, aarch64_sve_fneg, 4398046511623),
SVEMAP1(svneg_f32_z, aarch64_sve_fneg, 4398046511751),
SVEMAP1(svneg_f64_m, aarch64_sve_fneg, 4398046511368),
SVEMAP1(svneg_f64_x, aarch64_sve_fneg, 4398046511624),
SVEMAP1(svneg_f64_z, aarch64_sve_fneg, 4398046511752),
SVEMAP1(svneg_s16_m, aarch64_sve_neg, 4398046511362),
SVEMAP1(svneg_s16_x, aarch64_sve_neg, 4398046511618),
SVEMAP1(svneg_s16_z, aarch64_sve_neg, 4398046511746),
SVEMAP1(svneg_s32_m, aarch64_sve_neg, 4398046511363),
SVEMAP1(svneg_s32_x, aarch64_sve_neg, 4398046511619),
SVEMAP1(svneg_s32_z, aarch64_sve_neg, 4398046511747),
SVEMAP1(svneg_s64_m, aarch64_sve_neg, 4398046511364),
SVEMAP1(svneg_s64_x, aarch64_sve_neg, 4398046511620),
SVEMAP1(svneg_s64_z, aarch64_sve_neg, 4398046511748),
SVEMAP1(svneg_s8_m, aarch64_sve_neg, 4398046511361),
SVEMAP1(svneg_s8_x, aarch64_sve_neg, 4398046511617),
SVEMAP1(svneg_s8_z, aarch64_sve_neg, 4398046511745),
SVEMAP1(svnmad_f16_m, aarch64_sve_fnmad, 4466765988102),
SVEMAP1(svnmad_f16_x, aarch64_sve_fnmla_u, 4466765987974),
SVEMAP1(svnmad_f16_z, aarch64_sve_fnmad, 4466765988230),
SVEMAP1(svnmad_f32_m, aarch64_sve_fnmad, 4466765988103),
SVEMAP1(svnmad_f32_x, aarch64_sve_fnmla_u, 4466765987975),
SVEMAP1(svnmad_f32_z, aarch64_sve_fnmad, 4466765988231),
SVEMAP1(svnmad_f64_m, aarch64_sve_fnmad, 4466765988104),
SVEMAP1(svnmad_f64_x, aarch64_sve_fnmla_u, 4466765987976),
SVEMAP1(svnmad_f64_z, aarch64_sve_fnmad, 4466765988232),
SVEMAP1(svnmad_n_f16_m, aarch64_sve_fnmad, 4466765992198),
SVEMAP1(svnmad_n_f16_x, aarch64_sve_fnmla_u, 4466765992070),
SVEMAP1(svnmad_n_f16_z, aarch64_sve_fnmad, 4466765992326),
SVEMAP1(svnmad_n_f32_m, aarch64_sve_fnmad, 4466765992199),
SVEMAP1(svnmad_n_f32_x, aarch64_sve_fnmla_u, 4466765992071),
SVEMAP1(svnmad_n_f32_z, aarch64_sve_fnmad, 4466765992327),
SVEMAP1(svnmad_n_f64_m, aarch64_sve_fnmad, 4466765992200),
SVEMAP1(svnmad_n_f64_x, aarch64_sve_fnmla_u, 4466765992072),
SVEMAP1(svnmad_n_f64_z, aarch64_sve_fnmad, 4466765992328),
SVEMAP1(svnmatch_s16, aarch64_sve_nmatch, 2),
SVEMAP1(svnmatch_s8, aarch64_sve_nmatch, 1),
SVEMAP1(svnmatch_u16, aarch64_sve_nmatch, 2),
SVEMAP1(svnmatch_u8, aarch64_sve_nmatch, 1),
SVEMAP1(svnmla_f16_m, aarch64_sve_fnmla, 4398046511366),
SVEMAP1(svnmla_f16_x, aarch64_sve_fnmla_u, 4398046511238),
SVEMAP1(svnmla_f16_z, aarch64_sve_fnmla, 4398046511494),
SVEMAP1(svnmla_f32_m, aarch64_sve_fnmla, 4398046511367),
SVEMAP1(svnmla_f32_x, aarch64_sve_fnmla_u, 4398046511239),
SVEMAP1(svnmla_f32_z, aarch64_sve_fnmla, 4398046511495),
SVEMAP1(svnmla_f64_m, aarch64_sve_fnmla, 4398046511368),
SVEMAP1(svnmla_f64_x, aarch64_sve_fnmla_u, 4398046511240),
SVEMAP1(svnmla_f64_z, aarch64_sve_fnmla, 4398046511496),
SVEMAP1(svnmla_n_f16_m, aarch64_sve_fnmla, 4398046515462),
SVEMAP1(svnmla_n_f16_x, aarch64_sve_fnmla_u, 4398046515334),
SVEMAP1(svnmla_n_f16_z, aarch64_sve_fnmla, 4398046515590),
SVEMAP1(svnmla_n_f32_m, aarch64_sve_fnmla, 4398046515463),
SVEMAP1(svnmla_n_f32_x, aarch64_sve_fnmla_u, 4398046515335),
SVEMAP1(svnmla_n_f32_z, aarch64_sve_fnmla, 4398046515591),
SVEMAP1(svnmla_n_f64_m, aarch64_sve_fnmla, 4398046515464),
SVEMAP1(svnmla_n_f64_x, aarch64_sve_fnmla_u, 4398046515336),
SVEMAP1(svnmla_n_f64_z, aarch64_sve_fnmla, 4398046515592),
SVEMAP1(svnmls_f16_m, aarch64_sve_fnmls, 4398046511366),
SVEMAP1(svnmls_f16_x, aarch64_sve_fnmls_u, 4398046511238),
SVEMAP1(svnmls_f16_z, aarch64_sve_fnmls, 4398046511494),
SVEMAP1(svnmls_f32_m, aarch64_sve_fnmls, 4398046511367),
SVEMAP1(svnmls_f32_x, aarch64_sve_fnmls_u, 4398046511239),
SVEMAP1(svnmls_f32_z, aarch64_sve_fnmls, 4398046511495),
SVEMAP1(svnmls_f64_m, aarch64_sve_fnmls, 4398046511368),
SVEMAP1(svnmls_f64_x, aarch64_sve_fnmls_u, 4398046511240),
SVEMAP1(svnmls_f64_z, aarch64_sve_fnmls, 4398046511496),
SVEMAP1(svnmls_n_f16_m, aarch64_sve_fnmls, 4398046515462),
SVEMAP1(svnmls_n_f16_x, aarch64_sve_fnmls_u, 4398046515334),
SVEMAP1(svnmls_n_f16_z, aarch64_sve_fnmls, 4398046515590),
SVEMAP1(svnmls_n_f32_m, aarch64_sve_fnmls, 4398046515463),
SVEMAP1(svnmls_n_f32_x, aarch64_sve_fnmls_u, 4398046515335),
SVEMAP1(svnmls_n_f32_z, aarch64_sve_fnmls, 4398046515591),
SVEMAP1(svnmls_n_f64_m, aarch64_sve_fnmls, 4398046515464),
SVEMAP1(svnmls_n_f64_x, aarch64_sve_fnmls_u, 4398046515336),
SVEMAP1(svnmls_n_f64_z, aarch64_sve_fnmls, 4398046515592),
SVEMAP1(svnmsb_f16_m, aarch64_sve_fnmsb, 4466765988102),
SVEMAP1(svnmsb_f16_x, aarch64_sve_fnmls_u, 4466765987974),
SVEMAP1(svnmsb_f16_z, aarch64_sve_fnmsb, 4466765988230),
SVEMAP1(svnmsb_f32_m, aarch64_sve_fnmsb, 4466765988103),
SVEMAP1(svnmsb_f32_x, aarch64_sve_fnmls_u, 4466765987975),
SVEMAP1(svnmsb_f32_z, aarch64_sve_fnmsb, 4466765988231),
SVEMAP1(svnmsb_f64_m, aarch64_sve_fnmsb, 4466765988104),
SVEMAP1(svnmsb_f64_x, aarch64_sve_fnmls_u, 4466765987976),
SVEMAP1(svnmsb_f64_z, aarch64_sve_fnmsb, 4466765988232),
SVEMAP1(svnmsb_n_f16_m, aarch64_sve_fnmsb, 4466765992198),
SVEMAP1(svnmsb_n_f16_x, aarch64_sve_fnmls_u, 4466765992070),
SVEMAP1(svnmsb_n_f16_z, aarch64_sve_fnmsb, 4466765992326),
SVEMAP1(svnmsb_n_f32_m, aarch64_sve_fnmsb, 4466765992199),
SVEMAP1(svnmsb_n_f32_x, aarch64_sve_fnmls_u, 4466765992071),
SVEMAP1(svnmsb_n_f32_z, aarch64_sve_fnmsb, 4466765992327),
SVEMAP1(svnmsb_n_f64_m, aarch64_sve_fnmsb, 4466765992200),
SVEMAP1(svnmsb_n_f64_x, aarch64_sve_fnmls_u, 4466765992072),
SVEMAP1(svnmsb_n_f64_z, aarch64_sve_fnmsb, 4466765992328),
SVEMAP1(svnor_b_z, aarch64_sve_nor_z, 4398046511113),
SVEMAP2(svnot_b_z, 4398046511113),
SVEMAP1(svnot_s16_m, aarch64_sve_not, 4398046511362),
SVEMAP1(svnot_s16_x, aarch64_sve_not, 4398046511618),
SVEMAP1(svnot_s16_z, aarch64_sve_not, 4398046511746),
SVEMAP1(svnot_s32_m, aarch64_sve_not, 4398046511363),
SVEMAP1(svnot_s32_x, aarch64_sve_not, 4398046511619),
SVEMAP1(svnot_s32_z, aarch64_sve_not, 4398046511747),
SVEMAP1(svnot_s64_m, aarch64_sve_not, 4398046511364),
SVEMAP1(svnot_s64_x, aarch64_sve_not, 4398046511620),
SVEMAP1(svnot_s64_z, aarch64_sve_not, 4398046511748),
SVEMAP1(svnot_s8_m, aarch64_sve_not, 4398046511361),
SVEMAP1(svnot_s8_x, aarch64_sve_not, 4398046511617),
SVEMAP1(svnot_s8_z, aarch64_sve_not, 4398046511745),
SVEMAP1(svnot_u16_m, aarch64_sve_not, 4398046511362),
SVEMAP1(svnot_u16_x, aarch64_sve_not, 4398046511618),
SVEMAP1(svnot_u16_z, aarch64_sve_not, 4398046511746),
SVEMAP1(svnot_u32_m, aarch64_sve_not, 4398046511363),
SVEMAP1(svnot_u32_x, aarch64_sve_not, 4398046511619),
SVEMAP1(svnot_u32_z, aarch64_sve_not, 4398046511747),
SVEMAP1(svnot_u64_m, aarch64_sve_not, 4398046511364),
SVEMAP1(svnot_u64_x, aarch64_sve_not, 4398046511620),
SVEMAP1(svnot_u64_z, aarch64_sve_not, 4398046511748),
SVEMAP1(svnot_u8_m, aarch64_sve_not, 4398046511361),
SVEMAP1(svnot_u8_x, aarch64_sve_not, 4398046511617),
SVEMAP1(svnot_u8_z, aarch64_sve_not, 4398046511745),
SVEMAP1(svorn_b_z, aarch64_sve_orn_z, 4398046511113),
SVEMAP1(svorqv_s16, aarch64_sve_orqv, 6597069766658),
SVEMAP1(svorqv_s32, aarch64_sve_orqv, 6597069766659),
SVEMAP1(svorqv_s64, aarch64_sve_orqv, 6597069766660),
SVEMAP1(svorqv_s8, aarch64_sve_orqv, 6597069766657),
SVEMAP1(svorqv_u16, aarch64_sve_orqv, 6597069766658),
SVEMAP1(svorqv_u32, aarch64_sve_orqv, 6597069766659),
SVEMAP1(svorqv_u64, aarch64_sve_orqv, 6597069766660),
SVEMAP1(svorqv_u8, aarch64_sve_orqv, 6597069766657),
SVEMAP1(svorr_b_z, aarch64_sve_orr_z, 4398046511113),
SVEMAP1(svorr_n_s16_m, aarch64_sve_orr, 4398046514434),
SVEMAP1(svorr_n_s16_x, aarch64_sve_orr_u, 4398046514306),
SVEMAP1(svorr_n_s16_z, aarch64_sve_orr, 4398046514562),
SVEMAP1(svorr_n_s32_m, aarch64_sve_orr, 4398046514435),
SVEMAP1(svorr_n_s32_x, aarch64_sve_orr_u, 4398046514307),
SVEMAP1(svorr_n_s32_z, aarch64_sve_orr, 4398046514563),
SVEMAP1(svorr_n_s64_m, aarch64_sve_orr, 4398046514436),
SVEMAP1(svorr_n_s64_x, aarch64_sve_orr_u, 4398046514308),
SVEMAP1(svorr_n_s64_z, aarch64_sve_orr, 4398046514564),
SVEMAP1(svorr_n_s8_m, aarch64_sve_orr, 4398046514433),
SVEMAP1(svorr_n_s8_x, aarch64_sve_orr_u, 4398046514305),
SVEMAP1(svorr_n_s8_z, aarch64_sve_orr, 4398046514561),
SVEMAP1(svorr_n_u16_m, aarch64_sve_orr, 4398046514434),
SVEMAP1(svorr_n_u16_x, aarch64_sve_orr_u, 4398046514306),
SVEMAP1(svorr_n_u16_z, aarch64_sve_orr, 4398046514562),
SVEMAP1(svorr_n_u32_m, aarch64_sve_orr, 4398046514435),
SVEMAP1(svorr_n_u32_x, aarch64_sve_orr_u, 4398046514307),
SVEMAP1(svorr_n_u32_z, aarch64_sve_orr, 4398046514563),
SVEMAP1(svorr_n_u64_m, aarch64_sve_orr, 4398046514436),
SVEMAP1(svorr_n_u64_x, aarch64_sve_orr_u, 4398046514308),
SVEMAP1(svorr_n_u64_z, aarch64_sve_orr, 4398046514564),
SVEMAP1(svorr_n_u8_m, aarch64_sve_orr, 4398046514433),
SVEMAP1(svorr_n_u8_x, aarch64_sve_orr_u, 4398046514305),
SVEMAP1(svorr_n_u8_z, aarch64_sve_orr, 4398046514561),
SVEMAP1(svorr_s16_m, aarch64_sve_orr, 4398046511362),
SVEMAP1(svorr_s16_x, aarch64_sve_orr_u, 4398046511234),
SVEMAP1(svorr_s16_z, aarch64_sve_orr, 4398046511490),
SVEMAP1(svorr_s32_m, aarch64_sve_orr, 4398046511363),
SVEMAP1(svorr_s32_x, aarch64_sve_orr_u, 4398046511235),
SVEMAP1(svorr_s32_z, aarch64_sve_orr, 4398046511491),
SVEMAP1(svorr_s64_m, aarch64_sve_orr, 4398046511364),
SVEMAP1(svorr_s64_x, aarch64_sve_orr_u, 4398046511236),
SVEMAP1(svorr_s64_z, aarch64_sve_orr, 4398046511492),
SVEMAP1(svorr_s8_m, aarch64_sve_orr, 4398046511361),
SVEMAP1(svorr_s8_x, aarch64_sve_orr_u, 4398046511233),
SVEMAP1(svorr_s8_z, aarch64_sve_orr, 4398046511489),
SVEMAP1(svorr_u16_m, aarch64_sve_orr, 4398046511362),
SVEMAP1(svorr_u16_x, aarch64_sve_orr_u, 4398046511234),
SVEMAP1(svorr_u16_z, aarch64_sve_orr, 4398046511490),
SVEMAP1(svorr_u32_m, aarch64_sve_orr, 4398046511363),
SVEMAP1(svorr_u32_x, aarch64_sve_orr_u, 4398046511235),
SVEMAP1(svorr_u32_z, aarch64_sve_orr, 4398046511491),
SVEMAP1(svorr_u64_m, aarch64_sve_orr, 4398046511364),
SVEMAP1(svorr_u64_x, aarch64_sve_orr_u, 4398046511236),
SVEMAP1(svorr_u64_z, aarch64_sve_orr, 4398046511492),
SVEMAP1(svorr_u8_m, aarch64_sve_orr, 4398046511361),
SVEMAP1(svorr_u8_x, aarch64_sve_orr_u, 4398046511233),
SVEMAP1(svorr_u8_z, aarch64_sve_orr, 4398046511489),
SVEMAP1(svorv_s16, aarch64_sve_orv, 4398046511106),
SVEMAP1(svorv_s32, aarch64_sve_orv, 4398046511107),
SVEMAP1(svorv_s64, aarch64_sve_orv, 4398046511108),
SVEMAP1(svorv_s8, aarch64_sve_orv, 4398046511105),
SVEMAP1(svorv_u16, aarch64_sve_orv, 4398046511106),
SVEMAP1(svorv_u32, aarch64_sve_orv, 4398046511107),
SVEMAP1(svorv_u64, aarch64_sve_orv, 4398046511108),
SVEMAP1(svorv_u8, aarch64_sve_orv, 4398046511105),
SVEMAP1(svpext_lane_c16, aarch64_sve_pext, 4398046511114),
SVEMAP1(svpext_lane_c16_x2, aarch64_sve_pext_x2, 4398046511114),
SVEMAP1(svpext_lane_c32, aarch64_sve_pext, 4398046511115),
SVEMAP1(svpext_lane_c32_x2, aarch64_sve_pext_x2, 4398046511115),
SVEMAP1(svpext_lane_c64, aarch64_sve_pext, 4398046511116),
SVEMAP1(svpext_lane_c64_x2, aarch64_sve_pext_x2, 4398046511116),
SVEMAP1(svpext_lane_c8, aarch64_sve_pext, 4398046511113),
SVEMAP1(svpext_lane_c8_x2, aarch64_sve_pext_x2, 4398046511113),
SVEMAP2(svpfalse_b, 4398047559683),
SVEMAP2(svpfalse_c, 4398047559683),
SVEMAP1(svpfirst_b, aarch64_sve_pfirst, 4398046511113),
SVEMAP1(svpmov_lane_s16, aarch64_sve_pmov_to_pred_lane, 4398046511106),
SVEMAP1(svpmov_lane_s16_m, aarch64_sve_pmov_to_vector_lane_merging, 4398046511362),
SVEMAP1(svpmov_lane_s32, aarch64_sve_pmov_to_pred_lane, 4398046511107),
SVEMAP1(svpmov_lane_s32_m, aarch64_sve_pmov_to_vector_lane_merging, 4398046511363),
SVEMAP1(svpmov_lane_s64, aarch64_sve_pmov_to_pred_lane, 4398046511108),
SVEMAP1(svpmov_lane_s64_m, aarch64_sve_pmov_to_vector_lane_merging, 4398046511364),
SVEMAP1(svpmov_lane_s8, aarch64_sve_pmov_to_pred_lane, 4398046511105),
SVEMAP1(svpmov_lane_u16, aarch64_sve_pmov_to_pred_lane, 4398046511106),
SVEMAP1(svpmov_lane_u16_m, aarch64_sve_pmov_to_vector_lane_merging, 4398046511362),
SVEMAP1(svpmov_lane_u32, aarch64_sve_pmov_to_pred_lane, 4398046511107),
SVEMAP1(svpmov_lane_u32_m, aarch64_sve_pmov_to_vector_lane_merging, 4398046511363),
SVEMAP1(svpmov_lane_u64, aarch64_sve_pmov_to_pred_lane, 4398046511108),
SVEMAP1(svpmov_lane_u64_m, aarch64_sve_pmov_to_vector_lane_merging, 4398046511364),
SVEMAP1(svpmov_lane_u8, aarch64_sve_pmov_to_pred_lane, 4398046511105),
SVEMAP1(svpmov_s16, aarch64_sve_pmov_to_pred_lane_zero, 4398046511106),
SVEMAP1(svpmov_s16_z, aarch64_sve_pmov_to_vector_lane_zeroing, 4398046511106),
SVEMAP1(svpmov_s32, aarch64_sve_pmov_to_pred_lane_zero, 4398046511107),
SVEMAP1(svpmov_s32_z, aarch64_sve_pmov_to_vector_lane_zeroing, 4398046511107),
SVEMAP1(svpmov_s64, aarch64_sve_pmov_to_pred_lane_zero, 4398046511108),
SVEMAP1(svpmov_s64_z, aarch64_sve_pmov_to_vector_lane_zeroing, 4398046511108),
SVEMAP1(svpmov_s8, aarch64_sve_pmov_to_pred_lane_zero, 4398046511105),
SVEMAP1(svpmov_s8_z, aarch64_sve_pmov_to_vector_lane_zeroing, 4398046511105),
SVEMAP1(svpmov_u16, aarch64_sve_pmov_to_pred_lane_zero, 4398046511106),
SVEMAP1(svpmov_u16_z, aarch64_sve_pmov_to_vector_lane_zeroing, 4398046511106),
SVEMAP1(svpmov_u32, aarch64_sve_pmov_to_pred_lane_zero, 4398046511107),
SVEMAP1(svpmov_u32_z, aarch64_sve_pmov_to_vector_lane_zeroing, 4398046511107),
SVEMAP1(svpmov_u64, aarch64_sve_pmov_to_pred_lane_zero, 4398046511108),
SVEMAP1(svpmov_u64_z, aarch64_sve_pmov_to_vector_lane_zeroing, 4398046511108),
SVEMAP1(svpmov_u8, aarch64_sve_pmov_to_pred_lane_zero, 4398046511105),
SVEMAP1(svpmov_u8_z, aarch64_sve_pmov_to_vector_lane_zeroing, 4398046511105),
SVEMAP1(svpmul_n_u8, aarch64_sve_pmul, 4398046513153),
SVEMAP1(svpmul_u8, aarch64_sve_pmul, 4398046511105),
SVEMAP2(svpmullb_n_u16, 4398046513154),
SVEMAP2(svpmullb_n_u64, 4398046513156),
SVEMAP1(svpmullb_pair_n_u32, aarch64_sve_pmullb_pair, 4398046513155),
SVEMAP1(svpmullb_pair_n_u64, aarch64_sve_pmullb_pair, 2052),
SVEMAP1(svpmullb_pair_n_u8, aarch64_sve_pmullb_pair, 4398046513153),
SVEMAP1(svpmullb_pair_u32, aarch64_sve_pmullb_pair, 4398046511107),
SVEMAP1(svpmullb_pair_u64, aarch64_sve_pmullb_pair, 4),
SVEMAP1(svpmullb_pair_u8, aarch64_sve_pmullb_pair, 4398046511105),
SVEMAP2(svpmullb_u16, 4398046511106),
SVEMAP2(svpmullb_u64, 4398046511108),
SVEMAP2(svpmullt_n_u16, 4398046513154),
SVEMAP2(svpmullt_n_u64, 4398046513156),
SVEMAP1(svpmullt_pair_n_u32, aarch64_sve_pmullt_pair, 4398046513155),
SVEMAP1(svpmullt_pair_n_u64, aarch64_sve_pmullt_pair, 2052),
SVEMAP1(svpmullt_pair_n_u8, aarch64_sve_pmullt_pair, 4398046513153),
SVEMAP1(svpmullt_pair_u32, aarch64_sve_pmullt_pair, 4398046511107),
SVEMAP1(svpmullt_pair_u64, aarch64_sve_pmullt_pair, 4),
SVEMAP1(svpmullt_pair_u8, aarch64_sve_pmullt_pair, 4398046511105),
SVEMAP2(svpmullt_u16, 4398046511106),
SVEMAP2(svpmullt_u64, 4398046511108),
SVEMAP1(svpnext_b16, aarch64_sve_pnext, 4398046511114),
SVEMAP1(svpnext_b32, aarch64_sve_pnext, 4398046511115),
SVEMAP1(svpnext_b64, aarch64_sve_pnext, 4398046511116),
SVEMAP1(svpnext_b8, aarch64_sve_pnext, 4398046511113),
SVEMAP1(svprfb, aarch64_sve_prf, 4398180728849),
SVEMAP1(svprfb_gather_s32offset, aarch64_sve_prfb_gather_sxtw_index, 268435475),
SVEMAP1(svprfb_gather_s64offset, aarch64_sve_prfb_gather_index, 268435476),
SVEMAP1(svprfb_gather_u32base, aarch64_sve_prfb_gather_scalar_offset, 268435475),
SVEMAP1(svprfb_gather_u32base_offset, aarch64_sve_prfb_gather_scalar_offset, 268435475),
SVEMAP1(svprfb_gather_u32offset, aarch64_sve_prfb_gather_uxtw_index, 268435475),
SVEMAP1(svprfb_gather_u64base, aarch64_sve_prfb_gather_scalar_offset, 268435476),
SVEMAP1(svprfb_gather_u64base_offset, aarch64_sve_prfb_gather_scalar_offset, 268435476),
SVEMAP1(svprfb_gather_u64offset, aarch64_sve_prfb_gather_index, 268435476),
SVEMAP1(svprfb_vnum, aarch64_sve_prf, 4398180728849),
SVEMAP1(svprfd, aarch64_sve_prf, 4398180728900),
SVEMAP1(svprfd_gather_s32index, aarch64_sve_prfd_gather_sxtw_index, 268435523),
SVEMAP1(svprfd_gather_s64index, aarch64_sve_prfd_gather_index, 268435524),
SVEMAP1(svprfd_gather_u32base, aarch64_sve_prfd_gather_scalar_offset, 268435523),
SVEMAP1(svprfd_gather_u32base_index, aarch64_sve_prfd_gather_scalar_offset, 268435523),
SVEMAP1(svprfd_gather_u32index, aarch64_sve_prfd_gather_uxtw_index, 268435523),
SVEMAP1(svprfd_gather_u64base, aarch64_sve_prfd_gather_scalar_offset, 268435524),
SVEMAP1(svprfd_gather_u64base_index, aarch64_sve_prfd_gather_scalar_offset, 268435524),
SVEMAP1(svprfd_gather_u64index, aarch64_sve_prfd_gather_index, 268435524),
SVEMAP1(svprfd_vnum, aarch64_sve_prf, 4398180728900),
SVEMAP1(svprfh, aarch64_sve_prf, 4398180728866),
SVEMAP1(svprfh_gather_s32index, aarch64_sve_prfh_gather_sxtw_index, 268435491),
SVEMAP1(svprfh_gather_s64index, aarch64_sve_prfh_gather_index, 268435492),
SVEMAP1(svprfh_gather_u32base, aarch64_sve_prfh_gather_scalar_offset, 268435491),
SVEMAP1(svprfh_gather_u32base_index, aarch64_sve_prfh_gather_scalar_offset, 268435491),
SVEMAP1(svprfh_gather_u32index, aarch64_sve_prfh_gather_uxtw_index, 268435491),
SVEMAP1(svprfh_gather_u64base, aarch64_sve_prfh_gather_scalar_offset, 268435492),
SVEMAP1(svprfh_gather_u64base_index, aarch64_sve_prfh_gather_scalar_offset, 268435492),
SVEMAP1(svprfh_gather_u64index, aarch64_sve_prfh_gather_index, 268435492),
SVEMAP1(svprfh_vnum, aarch64_sve_prf, 4398180728866),
SVEMAP1(svprfw, aarch64_sve_prf, 4398180728883),
SVEMAP1(svprfw_gather_s32index, aarch64_sve_prfw_gather_sxtw_index, 268435507),
SVEMAP1(svprfw_gather_s64index, aarch64_sve_prfw_gather_index, 268435508),
SVEMAP1(svprfw_gather_u32base, aarch64_sve_prfw_gather_scalar_offset, 268435507),
SVEMAP1(svprfw_gather_u32base_index, aarch64_sve_prfw_gather_scalar_offset, 268435507),
SVEMAP1(svprfw_gather_u32index, aarch64_sve_prfw_gather_uxtw_index, 268435507),
SVEMAP1(svprfw_gather_u64base, aarch64_sve_prfw_gather_scalar_offset, 268435508),
SVEMAP1(svprfw_gather_u64base_index, aarch64_sve_prfw_gather_scalar_offset, 268435508),
SVEMAP1(svprfw_gather_u64index, aarch64_sve_prfw_gather_index, 268435508),
SVEMAP1(svprfw_vnum, aarch64_sve_prf, 4398180728883),
SVEMAP2(svpsel_lane_b16, 4398046511114),
SVEMAP2(svpsel_lane_b32, 4398046511115),
SVEMAP2(svpsel_lane_b64, 4398046511116),
SVEMAP2(svpsel_lane_b8, 4398046511113),
SVEMAP2(svpsel_lane_c16, 4398046511114),
SVEMAP2(svpsel_lane_c32, 4398046511115),
SVEMAP2(svpsel_lane_c64, 4398046511116),
SVEMAP2(svpsel_lane_c8, 4398046511113),
SVEMAP1(svptest_any, aarch64_sve_ptest_any, 4398046511113),
SVEMAP1(svptest_first, aarch64_sve_ptest_first, 4398046511113),
SVEMAP1(svptest_last, aarch64_sve_ptest_last, 4398046511113),
SVEMAP1(svptrue_b16, aarch64_sve_ptrue, 4398080065546),
SVEMAP1(svptrue_b32, aarch64_sve_ptrue, 4398080065547),
SVEMAP1(svptrue_b64, aarch64_sve_ptrue, 4398080065548),
SVEMAP1(svptrue_b8, aarch64_sve_ptrue, 4398080065545),
SVEMAP1(svptrue_c16, aarch64_sve_ptrue_c16, 4398047559690),
SVEMAP1(svptrue_c32, aarch64_sve_ptrue_c32, 4398047559691),
SVEMAP1(svptrue_c64, aarch64_sve_ptrue_c64, 4398047559692),
SVEMAP1(svptrue_c8, aarch64_sve_ptrue_c8, 4398047559689),
SVEMAP1(svptrue_pat_b16, aarch64_sve_ptrue, 4398046511114),
SVEMAP1(svptrue_pat_b32, aarch64_sve_ptrue, 4398046511115),
SVEMAP1(svptrue_pat_b64, aarch64_sve_ptrue, 4398046511116),
SVEMAP1(svptrue_pat_b8, aarch64_sve_ptrue, 4398046511113),
SVEMAP1(svqabs_s16_m, aarch64_sve_sqabs, 4398046511362),
SVEMAP1(svqabs_s16_x, aarch64_sve_sqabs, 4398046511618),
SVEMAP1(svqabs_s16_z, aarch64_sve_sqabs, 4398046511746),
SVEMAP1(svqabs_s32_m, aarch64_sve_sqabs, 4398046511363),
SVEMAP1(svqabs_s32_x, aarch64_sve_sqabs, 4398046511619),
SVEMAP1(svqabs_s32_z, aarch64_sve_sqabs, 4398046511747),
SVEMAP1(svqabs_s64_m, aarch64_sve_sqabs, 4398046511364),
SVEMAP1(svqabs_s64_x, aarch64_sve_sqabs, 4398046511620),
SVEMAP1(svqabs_s64_z, aarch64_sve_sqabs, 4398046511748),
SVEMAP1(svqabs_s8_m, aarch64_sve_sqabs, 4398046511361),
SVEMAP1(svqabs_s8_x, aarch64_sve_sqabs, 4398046511617),
SVEMAP1(svqabs_s8_z, aarch64_sve_sqabs, 4398046511745),
SVEMAP1(svqadd_n_s16, aarch64_sve_sqadd_x, 4398046513154),
SVEMAP1(svqadd_n_s16_m, aarch64_sve_sqadd, 4398046514434),
SVEMAP1(svqadd_n_s16_x, aarch64_sve_sqadd, 4398046514306),
SVEMAP1(svqadd_n_s16_z, aarch64_sve_sqadd, 4398046514562),
SVEMAP1(svqadd_n_s32, aarch64_sve_sqadd_x, 4398046513155),
SVEMAP1(svqadd_n_s32_m, aarch64_sve_sqadd, 4398046514435),
SVEMAP1(svqadd_n_s32_x, aarch64_sve_sqadd, 4398046514307),
SVEMAP1(svqadd_n_s32_z, aarch64_sve_sqadd, 4398046514563),
SVEMAP1(svqadd_n_s64, aarch64_sve_sqadd_x, 4398046513156),
SVEMAP1(svqadd_n_s64_m, aarch64_sve_sqadd, 4398046514436),
SVEMAP1(svqadd_n_s64_x, aarch64_sve_sqadd, 4398046514308),
SVEMAP1(svqadd_n_s64_z, aarch64_sve_sqadd, 4398046514564),
SVEMAP1(svqadd_n_s8, aarch64_sve_sqadd_x, 4398046513153),
SVEMAP1(svqadd_n_s8_m, aarch64_sve_sqadd, 4398046514433),
SVEMAP1(svqadd_n_s8_x, aarch64_sve_sqadd, 4398046514305),
SVEMAP1(svqadd_n_s8_z, aarch64_sve_sqadd, 4398046514561),
SVEMAP1(svqadd_n_u16, aarch64_sve_uqadd_x, 4398046513154),
SVEMAP1(svqadd_n_u16_m, aarch64_sve_uqadd, 4398046514434),
SVEMAP1(svqadd_n_u16_x, aarch64_sve_uqadd, 4398046514306),
SVEMAP1(svqadd_n_u16_z, aarch64_sve_uqadd, 4398046514562),
SVEMAP1(svqadd_n_u32, aarch64_sve_uqadd_x, 4398046513155),
SVEMAP1(svqadd_n_u32_m, aarch64_sve_uqadd, 4398046514435),
SVEMAP1(svqadd_n_u32_x, aarch64_sve_uqadd, 4398046514307),
SVEMAP1(svqadd_n_u32_z, aarch64_sve_uqadd, 4398046514563),
SVEMAP1(svqadd_n_u64, aarch64_sve_uqadd_x, 4398046513156),
SVEMAP1(svqadd_n_u64_m, aarch64_sve_uqadd, 4398046514436),
SVEMAP1(svqadd_n_u64_x, aarch64_sve_uqadd, 4398046514308),
SVEMAP1(svqadd_n_u64_z, aarch64_sve_uqadd, 4398046514564),
SVEMAP1(svqadd_n_u8, aarch64_sve_uqadd_x, 4398046513153),
SVEMAP1(svqadd_n_u8_m, aarch64_sve_uqadd, 4398046514433),
SVEMAP1(svqadd_n_u8_x, aarch64_sve_uqadd, 4398046514305),
SVEMAP1(svqadd_n_u8_z, aarch64_sve_uqadd, 4398046514561),
SVEMAP1(svqadd_s16, aarch64_sve_sqadd_x, 4398046511106),
SVEMAP1(svqadd_s16_m, aarch64_sve_sqadd, 4398046511362),
SVEMAP1(svqadd_s16_x, aarch64_sve_sqadd, 4398046511234),
SVEMAP1(svqadd_s16_z, aarch64_sve_sqadd, 4398046511490),
SVEMAP1(svqadd_s32, aarch64_sve_sqadd_x, 4398046511107),
SVEMAP1(svqadd_s32_m, aarch64_sve_sqadd, 4398046511363),
SVEMAP1(svqadd_s32_x, aarch64_sve_sqadd, 4398046511235),
SVEMAP1(svqadd_s32_z, aarch64_sve_sqadd, 4398046511491),
SVEMAP1(svqadd_s64, aarch64_sve_sqadd_x, 4398046511108),
SVEMAP1(svqadd_s64_m, aarch64_sve_sqadd, 4398046511364),
SVEMAP1(svqadd_s64_x, aarch64_sve_sqadd, 4398046511236),
SVEMAP1(svqadd_s64_z, aarch64_sve_sqadd, 4398046511492),
SVEMAP1(svqadd_s8, aarch64_sve_sqadd_x, 4398046511105),
SVEMAP1(svqadd_s8_m, aarch64_sve_sqadd, 4398046511361),
SVEMAP1(svqadd_s8_x, aarch64_sve_sqadd, 4398046511233),
SVEMAP1(svqadd_s8_z, aarch64_sve_sqadd, 4398046511489),
SVEMAP1(svqadd_u16, aarch64_sve_uqadd_x, 4398046511106),
SVEMAP1(svqadd_u16_m, aarch64_sve_uqadd, 4398046511362),
SVEMAP1(svqadd_u16_x, aarch64_sve_uqadd, 4398046511234),
SVEMAP1(svqadd_u16_z, aarch64_sve_uqadd, 4398046511490),
SVEMAP1(svqadd_u32, aarch64_sve_uqadd_x, 4398046511107),
SVEMAP1(svqadd_u32_m, aarch64_sve_uqadd, 4398046511363),
SVEMAP1(svqadd_u32_x, aarch64_sve_uqadd, 4398046511235),
SVEMAP1(svqadd_u32_z, aarch64_sve_uqadd, 4398046511491),
SVEMAP1(svqadd_u64, aarch64_sve_uqadd_x, 4398046511108),
SVEMAP1(svqadd_u64_m, aarch64_sve_uqadd, 4398046511364),
SVEMAP1(svqadd_u64_x, aarch64_sve_uqadd, 4398046511236),
SVEMAP1(svqadd_u64_z, aarch64_sve_uqadd, 4398046511492),
SVEMAP1(svqadd_u8, aarch64_sve_uqadd_x, 4398046511105),
SVEMAP1(svqadd_u8_m, aarch64_sve_uqadd, 4398046511361),
SVEMAP1(svqadd_u8_x, aarch64_sve_uqadd, 4398046511233),
SVEMAP1(svqadd_u8_z, aarch64_sve_uqadd, 4398046511489),
SVEMAP1(svqcadd_s16, aarch64_sve_sqcadd_x, 4398046511106),
SVEMAP1(svqcadd_s32, aarch64_sve_sqcadd_x, 4398046511107),
SVEMAP1(svqcadd_s64, aarch64_sve_sqcadd_x, 4398046511108),
SVEMAP1(svqcadd_s8, aarch64_sve_sqcadd_x, 4398046511105),
SVEMAP1(svqcvt_s16_s32_x2, aarch64_sve_sqcvt_x2, 137438953475),
SVEMAP1(svqcvt_s16_s64_x4, aarch64_sve_sqcvt_x4, 137438953476),
SVEMAP1(svqcvt_s8_s32_x4, aarch64_sve_sqcvt_x4, 137438953475),
SVEMAP1(svqcvt_u16_s32_x2, aarch64_sve_sqcvtu_x2, 137438953475),
SVEMAP1(svqcvt_u16_s64_x4, aarch64_sve_sqcvtu_x4, 137438953476),
SVEMAP1(svqcvt_u16_u32_x2, aarch64_sve_uqcvt_x2, 137438953475),
SVEMAP1(svqcvt_u16_u64_x4, aarch64_sve_uqcvt_x4, 137438953476),
SVEMAP1(svqcvt_u8_s32_x4, aarch64_sve_sqcvtu_x4, 137438953475),
SVEMAP1(svqcvt_u8_u32_x4, aarch64_sve_uqcvt_x4, 137438953475),
SVEMAP1(svqcvtn_s16_s32_x2, aarch64_sve_sqcvtn_x2, 4398046511107),
SVEMAP1(svqcvtn_s16_s64_x4, aarch64_sve_sqcvtn_x4, 137438953476),
SVEMAP1(svqcvtn_s8_s32_x4, aarch64_sve_sqcvtn_x4, 137438953475),
SVEMAP1(svqcvtn_u16_s32_x2, aarch64_sve_sqcvtun_x2, 4398046511107),
SVEMAP1(svqcvtn_u16_s64_x4, aarch64_sve_sqcvtun_x4, 137438953476),
SVEMAP1(svqcvtn_u16_u32_x2, aarch64_sve_uqcvtn_x2, 4398046511107),
SVEMAP1(svqcvtn_u16_u64_x4, aarch64_sve_uqcvtn_x4, 137438953476),
SVEMAP1(svqcvtn_u8_s32_x4, aarch64_sve_sqcvtun_x4, 137438953475),
SVEMAP1(svqcvtn_u8_u32_x4, aarch64_sve_uqcvtn_x4, 137438953475),
SVEMAP1(svqdecb_n_s32, aarch64_sve_sqdecb_n32, 4398114668547),
SVEMAP1(svqdecb_n_s64, aarch64_sve_sqdecb_n64, 4398114668548),
SVEMAP1(svqdecb_n_u32, aarch64_sve_uqdecb_n32, 4398114668547),
SVEMAP1(svqdecb_n_u64, aarch64_sve_uqdecb_n64, 4398114668548),
SVEMAP1(svqdecb_pat_n_s32, aarch64_sve_sqdecb_n32, 4398047559683),
SVEMAP1(svqdecb_pat_n_s64, aarch64_sve_sqdecb_n64, 4398047559684),
SVEMAP1(svqdecb_pat_n_u32, aarch64_sve_uqdecb_n32, 4398047559683),
SVEMAP1(svqdecb_pat_n_u64, aarch64_sve_uqdecb_n64, 4398047559684),
SVEMAP1(svqdecd_n_s32, aarch64_sve_sqdecd_n32, 4398114668547),
SVEMAP1(svqdecd_n_s64, aarch64_sve_sqdecd_n64, 4398114668548),
SVEMAP1(svqdecd_n_u32, aarch64_sve_uqdecd_n32, 4398114668547),
SVEMAP1(svqdecd_n_u64, aarch64_sve_uqdecd_n64, 4398114668548),
SVEMAP1(svqdecd_pat_n_s32, aarch64_sve_sqdecd_n32, 4398047559683),
SVEMAP1(svqdecd_pat_n_s64, aarch64_sve_sqdecd_n64, 4398047559684),
SVEMAP1(svqdecd_pat_n_u32, aarch64_sve_uqdecd_n32, 4398047559683),
SVEMAP1(svqdecd_pat_n_u64, aarch64_sve_uqdecd_n64, 4398047559684),
SVEMAP1(svqdecd_pat_s64, aarch64_sve_sqdecd, 4398046511108),
SVEMAP1(svqdecd_pat_u64, aarch64_sve_uqdecd, 4398046511108),
SVEMAP1(svqdecd_s64, aarch64_sve_sqdecd, 4398113619972),
SVEMAP1(svqdecd_u64, aarch64_sve_uqdecd, 4398113619972),
SVEMAP1(svqdech_n_s32, aarch64_sve_sqdech_n32, 4398114668547),
SVEMAP1(svqdech_n_s64, aarch64_sve_sqdech_n64, 4398114668548),
SVEMAP1(svqdech_n_u32, aarch64_sve_uqdech_n32, 4398114668547),
SVEMAP1(svqdech_n_u64, aarch64_sve_uqdech_n64, 4398114668548),
SVEMAP1(svqdech_pat_n_s32, aarch64_sve_sqdech_n32, 4398047559683),
SVEMAP1(svqdech_pat_n_s64, aarch64_sve_sqdech_n64, 4398047559684),
SVEMAP1(svqdech_pat_n_u32, aarch64_sve_uqdech_n32, 4398047559683),
SVEMAP1(svqdech_pat_n_u64, aarch64_sve_uqdech_n64, 4398047559684),
SVEMAP1(svqdech_pat_s16, aarch64_sve_sqdech, 4398046511106),
SVEMAP1(svqdech_pat_u16, aarch64_sve_uqdech, 4398046511106),
SVEMAP1(svqdech_s16, aarch64_sve_sqdech, 4398113619970),
SVEMAP1(svqdech_u16, aarch64_sve_uqdech, 4398113619970),
SVEMAP1(svqdecp_n_s32_b16, aarch64_sve_sqdecp_n32, 4398046511114),
SVEMAP1(svqdecp_n_s32_b32, aarch64_sve_sqdecp_n32, 4398046511115),
SVEMAP1(svqdecp_n_s32_b64, aarch64_sve_sqdecp_n32, 4398046511116),
SVEMAP1(svqdecp_n_s32_b8, aarch64_sve_sqdecp_n32, 4398046511113),
SVEMAP1(svqdecp_n_s64_b16, aarch64_sve_sqdecp_n64, 4398046511114),
SVEMAP1(svqdecp_n_s64_b32, aarch64_sve_sqdecp_n64, 4398046511115),
SVEMAP1(svqdecp_n_s64_b64, aarch64_sve_sqdecp_n64, 4398046511116),
SVEMAP1(svqdecp_n_s64_b8, aarch64_sve_sqdecp_n64, 4398046511113),
SVEMAP1(svqdecp_n_u32_b16, aarch64_sve_uqdecp_n32, 4398046511114),
SVEMAP1(svqdecp_n_u32_b32, aarch64_sve_uqdecp_n32, 4398046511115),
SVEMAP1(svqdecp_n_u32_b64, aarch64_sve_uqdecp_n32, 4398046511116),
SVEMAP1(svqdecp_n_u32_b8, aarch64_sve_uqdecp_n32, 4398046511113),
SVEMAP1(svqdecp_n_u64_b16, aarch64_sve_uqdecp_n64, 4398046511114),
SVEMAP1(svqdecp_n_u64_b32, aarch64_sve_uqdecp_n64, 4398046511115),
SVEMAP1(svqdecp_n_u64_b64, aarch64_sve_uqdecp_n64, 4398046511116),
SVEMAP1(svqdecp_n_u64_b8, aarch64_sve_uqdecp_n64, 4398046511113),
SVEMAP1(svqdecp_s16, aarch64_sve_sqdecp, 4398046511106),
SVEMAP1(svqdecp_s32, aarch64_sve_sqdecp, 4398046511107),
SVEMAP1(svqdecp_s64, aarch64_sve_sqdecp, 4398046511108),
SVEMAP1(svqdecp_u16, aarch64_sve_uqdecp, 4398046511106),
SVEMAP1(svqdecp_u32, aarch64_sve_uqdecp, 4398046511107),
SVEMAP1(svqdecp_u64, aarch64_sve_uqdecp, 4398046511108),
SVEMAP1(svqdecw_n_s32, aarch64_sve_sqdecw_n32, 4398114668547),
SVEMAP1(svqdecw_n_s64, aarch64_sve_sqdecw_n64, 4398114668548),
SVEMAP1(svqdecw_n_u32, aarch64_sve_uqdecw_n32, 4398114668547),
SVEMAP1(svqdecw_n_u64, aarch64_sve_uqdecw_n64, 4398114668548),
SVEMAP1(svqdecw_pat_n_s32, aarch64_sve_sqdecw_n32, 4398047559683),
SVEMAP1(svqdecw_pat_n_s64, aarch64_sve_sqdecw_n64, 4398047559684),
SVEMAP1(svqdecw_pat_n_u32, aarch64_sve_uqdecw_n32, 4398047559683),
SVEMAP1(svqdecw_pat_n_u64, aarch64_sve_uqdecw_n64, 4398047559684),
SVEMAP1(svqdecw_pat_s32, aarch64_sve_sqdecw, 4398046511107),
SVEMAP1(svqdecw_pat_u32, aarch64_sve_uqdecw, 4398046511107),
SVEMAP1(svqdecw_s32, aarch64_sve_sqdecw, 4398113619971),
SVEMAP1(svqdecw_u32, aarch64_sve_uqdecw, 4398113619971),
SVEMAP1(svqdmlalb_lane_s32, aarch64_sve_sqdmlalb_lane, 4398046511107),
SVEMAP1(svqdmlalb_lane_s64, aarch64_sve_sqdmlalb_lane, 4398046511108),
SVEMAP1(svqdmlalb_n_s16, aarch64_sve_sqdmlalb, 4398046514178),
SVEMAP1(svqdmlalb_n_s32, aarch64_sve_sqdmlalb, 4398046514179),
SVEMAP1(svqdmlalb_n_s64, aarch64_sve_sqdmlalb, 4398046514180),
SVEMAP1(svqdmlalb_s16, aarch64_sve_sqdmlalb, 4398046511106),
SVEMAP1(svqdmlalb_s32, aarch64_sve_sqdmlalb, 4398046511107),
SVEMAP1(svqdmlalb_s64, aarch64_sve_sqdmlalb, 4398046511108),
SVEMAP1(svqdmlalbt_n_s16, aarch64_sve_sqdmlalbt, 4398046514178),
SVEMAP1(svqdmlalbt_n_s32, aarch64_sve_sqdmlalbt, 4398046514179),
SVEMAP1(svqdmlalbt_n_s64, aarch64_sve_sqdmlalbt, 4398046514180),
SVEMAP1(svqdmlalbt_s16, aarch64_sve_sqdmlalbt, 4398046511106),
SVEMAP1(svqdmlalbt_s32, aarch64_sve_sqdmlalbt, 4398046511107),
SVEMAP1(svqdmlalbt_s64, aarch64_sve_sqdmlalbt, 4398046511108),
SVEMAP1(svqdmlalt_lane_s32, aarch64_sve_sqdmlalt_lane, 4398046511107),
SVEMAP1(svqdmlalt_lane_s64, aarch64_sve_sqdmlalt_lane, 4398046511108),
SVEMAP1(svqdmlalt_n_s16, aarch64_sve_sqdmlalt, 4398046514178),
SVEMAP1(svqdmlalt_n_s32, aarch64_sve_sqdmlalt, 4398046514179),
SVEMAP1(svqdmlalt_n_s64, aarch64_sve_sqdmlalt, 4398046514180),
SVEMAP1(svqdmlalt_s16, aarch64_sve_sqdmlalt, 4398046511106),
SVEMAP1(svqdmlalt_s32, aarch64_sve_sqdmlalt, 4398046511107),
SVEMAP1(svqdmlalt_s64, aarch64_sve_sqdmlalt, 4398046511108),
SVEMAP1(svqdmlslb_lane_s32, aarch64_sve_sqdmlslb_lane, 4398046511107),
SVEMAP1(svqdmlslb_lane_s64, aarch64_sve_sqdmlslb_lane, 4398046511108),
SVEMAP1(svqdmlslb_n_s16, aarch64_sve_sqdmlslb, 4398046514178),
SVEMAP1(svqdmlslb_n_s32, aarch64_sve_sqdmlslb, 4398046514179),
SVEMAP1(svqdmlslb_n_s64, aarch64_sve_sqdmlslb, 4398046514180),
SVEMAP1(svqdmlslb_s16, aarch64_sve_sqdmlslb, 4398046511106),
SVEMAP1(svqdmlslb_s32, aarch64_sve_sqdmlslb, 4398046511107),
SVEMAP1(svqdmlslb_s64, aarch64_sve_sqdmlslb, 4398046511108),
SVEMAP1(svqdmlslbt_n_s16, aarch64_sve_sqdmlslbt, 4398046514178),
SVEMAP1(svqdmlslbt_n_s32, aarch64_sve_sqdmlslbt, 4398046514179),
SVEMAP1(svqdmlslbt_n_s64, aarch64_sve_sqdmlslbt, 4398046514180),
SVEMAP1(svqdmlslbt_s16, aarch64_sve_sqdmlslbt, 4398046511106),
SVEMAP1(svqdmlslbt_s32, aarch64_sve_sqdmlslbt, 4398046511107),
SVEMAP1(svqdmlslbt_s64, aarch64_sve_sqdmlslbt, 4398046511108),
SVEMAP1(svqdmlslt_lane_s32, aarch64_sve_sqdmlslt_lane, 4398046511107),
SVEMAP1(svqdmlslt_lane_s64, aarch64_sve_sqdmlslt_lane, 4398046511108),
SVEMAP1(svqdmlslt_n_s16, aarch64_sve_sqdmlslt, 4398046514178),
SVEMAP1(svqdmlslt_n_s32, aarch64_sve_sqdmlslt, 4398046514179),
SVEMAP1(svqdmlslt_n_s64, aarch64_sve_sqdmlslt, 4398046514180),
SVEMAP1(svqdmlslt_s16, aarch64_sve_sqdmlslt, 4398046511106),
SVEMAP1(svqdmlslt_s32, aarch64_sve_sqdmlslt, 4398046511107),
SVEMAP1(svqdmlslt_s64, aarch64_sve_sqdmlslt, 4398046511108),
SVEMAP1(svqdmulh_lane_s16, aarch64_sve_sqdmulh_lane, 4398046511106),
SVEMAP1(svqdmulh_lane_s32, aarch64_sve_sqdmulh_lane, 4398046511107),
SVEMAP1(svqdmulh_lane_s64, aarch64_sve_sqdmulh_lane, 4398046511108),
SVEMAP1(svqdmulh_n_s16, aarch64_sve_sqdmulh, 4398046513154),
SVEMAP1(svqdmulh_n_s32, aarch64_sve_sqdmulh, 4398046513155),
SVEMAP1(svqdmulh_n_s64, aarch64_sve_sqdmulh, 4398046513156),
SVEMAP1(svqdmulh_n_s8, aarch64_sve_sqdmulh, 4398046513153),
SVEMAP1(svqdmulh_s16, aarch64_sve_sqdmulh, 4398046511106),
SVEMAP1(svqdmulh_s16_x2, aarch64_sve_sqdmulh_vgx2, 137438953474),
SVEMAP1(svqdmulh_s16_x4, aarch64_sve_sqdmulh_vgx4, 137438953474),
SVEMAP1(svqdmulh_s32, aarch64_sve_sqdmulh, 4398046511107),
SVEMAP1(svqdmulh_s32_x2, aarch64_sve_sqdmulh_vgx2, 137438953475),
SVEMAP1(svqdmulh_s32_x4, aarch64_sve_sqdmulh_vgx4, 137438953475),
SVEMAP1(svqdmulh_s64, aarch64_sve_sqdmulh, 4398046511108),
SVEMAP1(svqdmulh_s64_x2, aarch64_sve_sqdmulh_vgx2, 137438953476),
SVEMAP1(svqdmulh_s64_x4, aarch64_sve_sqdmulh_vgx4, 137438953476),
SVEMAP1(svqdmulh_s8, aarch64_sve_sqdmulh, 4398046511105),
SVEMAP1(svqdmulh_s8_x2, aarch64_sve_sqdmulh_vgx2, 137438953473),
SVEMAP1(svqdmulh_s8_x4, aarch64_sve_sqdmulh_vgx4, 137438953473),
SVEMAP1(svqdmulh_single_s16_x2, aarch64_sve_sqdmulh_single_vgx2, 137438953474),
SVEMAP1(svqdmulh_single_s16_x4, aarch64_sve_sqdmulh_single_vgx4, 137438953474),
SVEMAP1(svqdmulh_single_s32_x2, aarch64_sve_sqdmulh_single_vgx2, 137438953475),
SVEMAP1(svqdmulh_single_s32_x4, aarch64_sve_sqdmulh_single_vgx4, 137438953475),
SVEMAP1(svqdmulh_single_s64_x2, aarch64_sve_sqdmulh_single_vgx2, 137438953476),
SVEMAP1(svqdmulh_single_s64_x4, aarch64_sve_sqdmulh_single_vgx4, 137438953476),
SVEMAP1(svqdmulh_single_s8_x2, aarch64_sve_sqdmulh_single_vgx2, 137438953473),
SVEMAP1(svqdmulh_single_s8_x4, aarch64_sve_sqdmulh_single_vgx4, 137438953473),
SVEMAP1(svqdmullb_lane_s32, aarch64_sve_sqdmullb_lane, 4398046511107),
SVEMAP1(svqdmullb_lane_s64, aarch64_sve_sqdmullb_lane, 4398046511108),
SVEMAP1(svqdmullb_n_s16, aarch64_sve_sqdmullb, 4398046513154),
SVEMAP1(svqdmullb_n_s32, aarch64_sve_sqdmullb, 4398046513155),
SVEMAP1(svqdmullb_n_s64, aarch64_sve_sqdmullb, 4398046513156),
SVEMAP1(svqdmullb_s16, aarch64_sve_sqdmullb, 4398046511106),
SVEMAP1(svqdmullb_s32, aarch64_sve_sqdmullb, 4398046511107),
SVEMAP1(svqdmullb_s64, aarch64_sve_sqdmullb, 4398046511108),
SVEMAP1(svqdmullt_lane_s32, aarch64_sve_sqdmullt_lane, 4398046511107),
SVEMAP1(svqdmullt_lane_s64, aarch64_sve_sqdmullt_lane, 4398046511108),
SVEMAP1(svqdmullt_n_s16, aarch64_sve_sqdmullt, 4398046513154),
SVEMAP1(svqdmullt_n_s32, aarch64_sve_sqdmullt, 4398046513155),
SVEMAP1(svqdmullt_n_s64, aarch64_sve_sqdmullt, 4398046513156),
SVEMAP1(svqdmullt_s16, aarch64_sve_sqdmullt, 4398046511106),
SVEMAP1(svqdmullt_s32, aarch64_sve_sqdmullt, 4398046511107),
SVEMAP1(svqdmullt_s64, aarch64_sve_sqdmullt, 4398046511108),
SVEMAP1(svqincb_n_s32, aarch64_sve_sqincb_n32, 4398114668547),
SVEMAP1(svqincb_n_s64, aarch64_sve_sqincb_n64, 4398114668548),
SVEMAP1(svqincb_n_u32, aarch64_sve_uqincb_n32, 4398114668547),
SVEMAP1(svqincb_n_u64, aarch64_sve_uqincb_n64, 4398114668548),
SVEMAP1(svqincb_pat_n_s32, aarch64_sve_sqincb_n32, 4398047559683),
SVEMAP1(svqincb_pat_n_s64, aarch64_sve_sqincb_n64, 4398047559684),
SVEMAP1(svqincb_pat_n_u32, aarch64_sve_uqincb_n32, 4398047559683),
SVEMAP1(svqincb_pat_n_u64, aarch64_sve_uqincb_n64, 4398047559684),
SVEMAP1(svqincd_n_s32, aarch64_sve_sqincd_n32, 4398114668547),
SVEMAP1(svqincd_n_s64, aarch64_sve_sqincd_n64, 4398114668548),
SVEMAP1(svqincd_n_u32, aarch64_sve_uqincd_n32, 4398114668547),
SVEMAP1(svqincd_n_u64, aarch64_sve_uqincd_n64, 4398114668548),
SVEMAP1(svqincd_pat_n_s32, aarch64_sve_sqincd_n32, 4398047559683),
SVEMAP1(svqincd_pat_n_s64, aarch64_sve_sqincd_n64, 4398047559684),
SVEMAP1(svqincd_pat_n_u32, aarch64_sve_uqincd_n32, 4398047559683),
SVEMAP1(svqincd_pat_n_u64, aarch64_sve_uqincd_n64, 4398047559684),
SVEMAP1(svqincd_pat_s64, aarch64_sve_sqincd, 4398046511108),
SVEMAP1(svqincd_pat_u64, aarch64_sve_uqincd, 4398046511108),
SVEMAP1(svqincd_s64, aarch64_sve_sqincd, 4398113619972),
SVEMAP1(svqincd_u64, aarch64_sve_uqincd, 4398113619972),
SVEMAP1(svqinch_n_s32, aarch64_sve_sqinch_n32, 4398114668547),
SVEMAP1(svqinch_n_s64, aarch64_sve_sqinch_n64, 4398114668548),
SVEMAP1(svqinch_n_u32, aarch64_sve_uqinch_n32, 4398114668547),
SVEMAP1(svqinch_n_u64, aarch64_sve_uqinch_n64, 4398114668548),
SVEMAP1(svqinch_pat_n_s32, aarch64_sve_sqinch_n32, 4398047559683),
SVEMAP1(svqinch_pat_n_s64, aarch64_sve_sqinch_n64, 4398047559684),
SVEMAP1(svqinch_pat_n_u32, aarch64_sve_uqinch_n32, 4398047559683),
SVEMAP1(svqinch_pat_n_u64, aarch64_sve_uqinch_n64, 4398047559684),
SVEMAP1(svqinch_pat_s16, aarch64_sve_sqinch, 4398046511106),
SVEMAP1(svqinch_pat_u16, aarch64_sve_uqinch, 4398046511106),
SVEMAP1(svqinch_s16, aarch64_sve_sqinch, 4398113619970),
SVEMAP1(svqinch_u16, aarch64_sve_uqinch, 4398113619970),
SVEMAP1(svqincp_n_s32_b16, aarch64_sve_sqincp_n32, 4398046511114),
SVEMAP1(svqincp_n_s32_b32, aarch64_sve_sqincp_n32, 4398046511115),
SVEMAP1(svqincp_n_s32_b64, aarch64_sve_sqincp_n32, 4398046511116),
SVEMAP1(svqincp_n_s32_b8, aarch64_sve_sqincp_n32, 4398046511113),
SVEMAP1(svqincp_n_s64_b16, aarch64_sve_sqincp_n64, 4398046511114),
SVEMAP1(svqincp_n_s64_b32, aarch64_sve_sqincp_n64, 4398046511115),
SVEMAP1(svqincp_n_s64_b64, aarch64_sve_sqincp_n64, 4398046511116),
SVEMAP1(svqincp_n_s64_b8, aarch64_sve_sqincp_n64, 4398046511113),
SVEMAP1(svqincp_n_u32_b16, aarch64_sve_uqincp_n32, 4398046511114),
SVEMAP1(svqincp_n_u32_b32, aarch64_sve_uqincp_n32, 4398046511115),
SVEMAP1(svqincp_n_u32_b64, aarch64_sve_uqincp_n32, 4398046511116),
SVEMAP1(svqincp_n_u32_b8, aarch64_sve_uqincp_n32, 4398046511113),
SVEMAP1(svqincp_n_u64_b16, aarch64_sve_uqincp_n64, 4398046511114),
SVEMAP1(svqincp_n_u64_b32, aarch64_sve_uqincp_n64, 4398046511115),
SVEMAP1(svqincp_n_u64_b64, aarch64_sve_uqincp_n64, 4398046511116),
SVEMAP1(svqincp_n_u64_b8, aarch64_sve_uqincp_n64, 4398046511113),
SVEMAP1(svqincp_s16, aarch64_sve_sqincp, 4398046511106),
SVEMAP1(svqincp_s32, aarch64_sve_sqincp, 4398046511107),
SVEMAP1(svqincp_s64, aarch64_sve_sqincp, 4398046511108),
SVEMAP1(svqincp_u16, aarch64_sve_uqincp, 4398046511106),
SVEMAP1(svqincp_u32, aarch64_sve_uqincp, 4398046511107),
SVEMAP1(svqincp_u64, aarch64_sve_uqincp, 4398046511108),
SVEMAP1(svqincw_n_s32, aarch64_sve_sqincw_n32, 4398114668547),
SVEMAP1(svqincw_n_s64, aarch64_sve_sqincw_n64, 4398114668548),
SVEMAP1(svqincw_n_u32, aarch64_sve_uqincw_n32, 4398114668547),
SVEMAP1(svqincw_n_u64, aarch64_sve_uqincw_n64, 4398114668548),
SVEMAP1(svqincw_pat_n_s32, aarch64_sve_sqincw_n32, 4398047559683),
SVEMAP1(svqincw_pat_n_s64, aarch64_sve_sqincw_n64, 4398047559684),
SVEMAP1(svqincw_pat_n_u32, aarch64_sve_uqincw_n32, 4398047559683),
SVEMAP1(svqincw_pat_n_u64, aarch64_sve_uqincw_n64, 4398047559684),
SVEMAP1(svqincw_pat_s32, aarch64_sve_sqincw, 4398046511107),
SVEMAP1(svqincw_pat_u32, aarch64_sve_uqincw, 4398046511107),
SVEMAP1(svqincw_s32, aarch64_sve_sqincw, 4398113619971),
SVEMAP1(svqincw_u32, aarch64_sve_uqincw, 4398113619971),
SVEMAP1(svqneg_s16_m, aarch64_sve_sqneg, 4398046511362),
SVEMAP1(svqneg_s16_x, aarch64_sve_sqneg, 4398046511618),
SVEMAP1(svqneg_s16_z, aarch64_sve_sqneg, 4398046511746),
SVEMAP1(svqneg_s32_m, aarch64_sve_sqneg, 4398046511363),
SVEMAP1(svqneg_s32_x, aarch64_sve_sqneg, 4398046511619),
SVEMAP1(svqneg_s32_z, aarch64_sve_sqneg, 4398046511747),
SVEMAP1(svqneg_s64_m, aarch64_sve_sqneg, 4398046511364),
SVEMAP1(svqneg_s64_x, aarch64_sve_sqneg, 4398046511620),
SVEMAP1(svqneg_s64_z, aarch64_sve_sqneg, 4398046511748),
SVEMAP1(svqneg_s8_m, aarch64_sve_sqneg, 4398046511361),
SVEMAP1(svqneg_s8_x, aarch64_sve_sqneg, 4398046511617),
SVEMAP1(svqneg_s8_z, aarch64_sve_sqneg, 4398046511745),
SVEMAP1(svqrdcmlah_lane_s16, aarch64_sve_sqrdcmlah_lane_x, 4398046511106),
SVEMAP1(svqrdcmlah_lane_s32, aarch64_sve_sqrdcmlah_lane_x, 4398046511107),
SVEMAP1(svqrdcmlah_s16, aarch64_sve_sqrdcmlah_x, 4398046511106),
SVEMAP1(svqrdcmlah_s32, aarch64_sve_sqrdcmlah_x, 4398046511107),
SVEMAP1(svqrdcmlah_s64, aarch64_sve_sqrdcmlah_x, 4398046511108),
SVEMAP1(svqrdcmlah_s8, aarch64_sve_sqrdcmlah_x, 4398046511105),
SVEMAP1(svqrdmlah_lane_s16, aarch64_sve_sqrdmlah_lane, 4398046511106),
SVEMAP1(svqrdmlah_lane_s32, aarch64_sve_sqrdmlah_lane, 4398046511107),
SVEMAP1(svqrdmlah_lane_s64, aarch64_sve_sqrdmlah_lane, 4398046511108),
SVEMAP1(svqrdmlah_n_s16, aarch64_sve_sqrdmlah, 4398046514178),
SVEMAP1(svqrdmlah_n_s32, aarch64_sve_sqrdmlah, 4398046514179),
SVEMAP1(svqrdmlah_n_s64, aarch64_sve_sqrdmlah, 4398046514180),
SVEMAP1(svqrdmlah_n_s8, aarch64_sve_sqrdmlah, 4398046514177),
SVEMAP1(svqrdmlah_s16, aarch64_sve_sqrdmlah, 4398046511106),
SVEMAP1(svqrdmlah_s32, aarch64_sve_sqrdmlah, 4398046511107),
SVEMAP1(svqrdmlah_s64, aarch64_sve_sqrdmlah, 4398046511108),
SVEMAP1(svqrdmlah_s8, aarch64_sve_sqrdmlah, 4398046511105),
SVEMAP1(svqrdmlsh_lane_s16, aarch64_sve_sqrdmlsh_lane, 4398046511106),
SVEMAP1(svqrdmlsh_lane_s32, aarch64_sve_sqrdmlsh_lane, 4398046511107),
SVEMAP1(svqrdmlsh_lane_s64, aarch64_sve_sqrdmlsh_lane, 4398046511108),
SVEMAP1(svqrdmlsh_n_s16, aarch64_sve_sqrdmlsh, 4398046514178),
SVEMAP1(svqrdmlsh_n_s32, aarch64_sve_sqrdmlsh, 4398046514179),
SVEMAP1(svqrdmlsh_n_s64, aarch64_sve_sqrdmlsh, 4398046514180),
SVEMAP1(svqrdmlsh_n_s8, aarch64_sve_sqrdmlsh, 4398046514177),
SVEMAP1(svqrdmlsh_s16, aarch64_sve_sqrdmlsh, 4398046511106),
SVEMAP1(svqrdmlsh_s32, aarch64_sve_sqrdmlsh, 4398046511107),
SVEMAP1(svqrdmlsh_s64, aarch64_sve_sqrdmlsh, 4398046511108),
SVEMAP1(svqrdmlsh_s8, aarch64_sve_sqrdmlsh, 4398046511105),
SVEMAP1(svqrdmulh_lane_s16, aarch64_sve_sqrdmulh_lane, 4398046511106),
SVEMAP1(svqrdmulh_lane_s32, aarch64_sve_sqrdmulh_lane, 4398046511107),
SVEMAP1(svqrdmulh_lane_s64, aarch64_sve_sqrdmulh_lane, 4398046511108),
SVEMAP1(svqrdmulh_n_s16, aarch64_sve_sqrdmulh, 4398046513154),
SVEMAP1(svqrdmulh_n_s32, aarch64_sve_sqrdmulh, 4398046513155),
SVEMAP1(svqrdmulh_n_s64, aarch64_sve_sqrdmulh, 4398046513156),
SVEMAP1(svqrdmulh_n_s8, aarch64_sve_sqrdmulh, 4398046513153),
SVEMAP1(svqrdmulh_s16, aarch64_sve_sqrdmulh, 4398046511106),
SVEMAP1(svqrdmulh_s32, aarch64_sve_sqrdmulh, 4398046511107),
SVEMAP1(svqrdmulh_s64, aarch64_sve_sqrdmulh, 4398046511108),
SVEMAP1(svqrdmulh_s8, aarch64_sve_sqrdmulh, 4398046511105),
SVEMAP1(svqrshl_n_s16_m, aarch64_sve_sqrshl, 4398046514434),
SVEMAP1(svqrshl_n_s16_x, aarch64_sve_sqrshl, 4398046514306),
SVEMAP1(svqrshl_n_s16_z, aarch64_sve_sqrshl, 4398046514562),
SVEMAP1(svqrshl_n_s32_m, aarch64_sve_sqrshl, 4398046514435),
SVEMAP1(svqrshl_n_s32_x, aarch64_sve_sqrshl, 4398046514307),
SVEMAP1(svqrshl_n_s32_z, aarch64_sve_sqrshl, 4398046514563),
SVEMAP1(svqrshl_n_s64_m, aarch64_sve_sqrshl, 4398046514436),
SVEMAP1(svqrshl_n_s64_x, aarch64_sve_sqrshl, 4398046514308),
SVEMAP1(svqrshl_n_s64_z, aarch64_sve_sqrshl, 4398046514564),
SVEMAP1(svqrshl_n_s8_m, aarch64_sve_sqrshl, 4398046514433),
SVEMAP1(svqrshl_n_s8_x, aarch64_sve_sqrshl, 4398046514305),
SVEMAP1(svqrshl_n_s8_z, aarch64_sve_sqrshl, 4398046514561),
SVEMAP1(svqrshl_n_u16_m, aarch64_sve_uqrshl, 4398046514434),
SVEMAP1(svqrshl_n_u16_x, aarch64_sve_uqrshl, 4398046514306),
SVEMAP1(svqrshl_n_u16_z, aarch64_sve_uqrshl, 4398046514562),
SVEMAP1(svqrshl_n_u32_m, aarch64_sve_uqrshl, 4398046514435),
SVEMAP1(svqrshl_n_u32_x, aarch64_sve_uqrshl, 4398046514307),
SVEMAP1(svqrshl_n_u32_z, aarch64_sve_uqrshl, 4398046514563),
SVEMAP1(svqrshl_n_u64_m, aarch64_sve_uqrshl, 4398046514436),
SVEMAP1(svqrshl_n_u64_x, aarch64_sve_uqrshl, 4398046514308),
SVEMAP1(svqrshl_n_u64_z, aarch64_sve_uqrshl, 4398046514564),
SVEMAP1(svqrshl_n_u8_m, aarch64_sve_uqrshl, 4398046514433),
SVEMAP1(svqrshl_n_u8_x, aarch64_sve_uqrshl, 4398046514305),
SVEMAP1(svqrshl_n_u8_z, aarch64_sve_uqrshl, 4398046514561),
SVEMAP1(svqrshl_s16_m, aarch64_sve_sqrshl, 4398046511362),
SVEMAP1(svqrshl_s16_x, aarch64_sve_sqrshl, 4398046511234),
SVEMAP1(svqrshl_s16_z, aarch64_sve_sqrshl, 4398046511490),
SVEMAP1(svqrshl_s32_m, aarch64_sve_sqrshl, 4398046511363),
SVEMAP1(svqrshl_s32_x, aarch64_sve_sqrshl, 4398046511235),
SVEMAP1(svqrshl_s32_z, aarch64_sve_sqrshl, 4398046511491),
SVEMAP1(svqrshl_s64_m, aarch64_sve_sqrshl, 4398046511364),
SVEMAP1(svqrshl_s64_x, aarch64_sve_sqrshl, 4398046511236),
SVEMAP1(svqrshl_s64_z, aarch64_sve_sqrshl, 4398046511492),
SVEMAP1(svqrshl_s8_m, aarch64_sve_sqrshl, 4398046511361),
SVEMAP1(svqrshl_s8_x, aarch64_sve_sqrshl, 4398046511233),
SVEMAP1(svqrshl_s8_z, aarch64_sve_sqrshl, 4398046511489),
SVEMAP1(svqrshl_u16_m, aarch64_sve_uqrshl, 4398046511362),
SVEMAP1(svqrshl_u16_x, aarch64_sve_uqrshl, 4398046511234),
SVEMAP1(svqrshl_u16_z, aarch64_sve_uqrshl, 4398046511490),
SVEMAP1(svqrshl_u32_m, aarch64_sve_uqrshl, 4398046511363),
SVEMAP1(svqrshl_u32_x, aarch64_sve_uqrshl, 4398046511235),
SVEMAP1(svqrshl_u32_z, aarch64_sve_uqrshl, 4398046511491),
SVEMAP1(svqrshl_u64_m, aarch64_sve_uqrshl, 4398046511364),
SVEMAP1(svqrshl_u64_x, aarch64_sve_uqrshl, 4398046511236),
SVEMAP1(svqrshl_u64_z, aarch64_sve_uqrshl, 4398046511492),
SVEMAP1(svqrshl_u8_m, aarch64_sve_uqrshl, 4398046511361),
SVEMAP1(svqrshl_u8_x, aarch64_sve_uqrshl, 4398046511233),
SVEMAP1(svqrshl_u8_z, aarch64_sve_uqrshl, 4398046511489),
SVEMAP1(svqrshr_n_s16_s32_x2, aarch64_sve_sqrshr_x2, 137438953475),
SVEMAP1(svqrshr_n_s16_s64_x4, aarch64_sve_sqrshr_x4, 137438953476),
SVEMAP1(svqrshr_n_s8_s32_x4, aarch64_sve_sqrshr_x4, 137438953475),
SVEMAP1(svqrshr_n_u16_u32_x2, aarch64_sve_uqrshr_x2, 137438953475),
SVEMAP1(svqrshr_n_u16_u64_x4, aarch64_sve_uqrshr_x4, 137438953476),
SVEMAP1(svqrshr_n_u8_u32_x4, aarch64_sve_uqrshr_x4, 137438953475),
SVEMAP1(svqrshrn_n_s16_s32_x2, aarch64_sve_sqrshrn_x2, 4398046511107),
SVEMAP1(svqrshrn_n_s16_s64_x4, aarch64_sve_sqrshrn_x4, 137438953476),
SVEMAP1(svqrshrn_n_s8_s32_x4, aarch64_sve_sqrshrn_x4, 137438953475),
SVEMAP1(svqrshrn_n_u16_u32_x2, aarch64_sve_uqrshrn_x2, 4398046511107),
SVEMAP1(svqrshrn_n_u16_u64_x4, aarch64_sve_uqrshrn_x4, 137438953476),
SVEMAP1(svqrshrn_n_u8_u32_x4, aarch64_sve_uqrshrn_x4, 137438953475),
SVEMAP1(svqrshrnb_n_s16, aarch64_sve_sqrshrnb, 4398046511106),
SVEMAP1(svqrshrnb_n_s32, aarch64_sve_sqrshrnb, 4398046511107),
SVEMAP1(svqrshrnb_n_s64, aarch64_sve_sqrshrnb, 4398046511108),
SVEMAP1(svqrshrnb_n_u16, aarch64_sve_uqrshrnb, 4398046511106),
SVEMAP1(svqrshrnb_n_u32, aarch64_sve_uqrshrnb, 4398046511107),
SVEMAP1(svqrshrnb_n_u64, aarch64_sve_uqrshrnb, 4398046511108),
SVEMAP1(svqrshrnt_n_s16, aarch64_sve_sqrshrnt, 4398046511106),
SVEMAP1(svqrshrnt_n_s32, aarch64_sve_sqrshrnt, 4398046511107),
SVEMAP1(svqrshrnt_n_s64, aarch64_sve_sqrshrnt, 4398046511108),
SVEMAP1(svqrshrnt_n_u16, aarch64_sve_uqrshrnt, 4398046511106),
SVEMAP1(svqrshrnt_n_u32, aarch64_sve_uqrshrnt, 4398046511107),
SVEMAP1(svqrshrnt_n_u64, aarch64_sve_uqrshrnt, 4398046511108),
SVEMAP1(svqrshru_n_u16_s32_x2, aarch64_sve_sqrshru_x2, 137438953475),
SVEMAP1(svqrshru_n_u16_s64_x4, aarch64_sve_sqrshru_x4, 137438953476),
SVEMAP1(svqrshru_n_u8_s32_x4, aarch64_sve_sqrshru_x4, 137438953475),
SVEMAP1(svqrshrun_n_u16_s32_x2, aarch64_sve_sqrshrun_x2, 4398046511107),
SVEMAP1(svqrshrun_n_u16_s64_x4, aarch64_sve_sqrshrun_x4, 137438953476),
SVEMAP1(svqrshrun_n_u8_s32_x4, aarch64_sve_sqrshrun_x4, 137438953475),
SVEMAP1(svqrshrunb_n_s16, aarch64_sve_sqrshrunb, 4398046511106),
SVEMAP1(svqrshrunb_n_s32, aarch64_sve_sqrshrunb, 4398046511107),
SVEMAP1(svqrshrunb_n_s64, aarch64_sve_sqrshrunb, 4398046511108),
SVEMAP1(svqrshrunt_n_s16, aarch64_sve_sqrshrunt, 4398046511106),
SVEMAP1(svqrshrunt_n_s32, aarch64_sve_sqrshrunt, 4398046511107),
SVEMAP1(svqrshrunt_n_s64, aarch64_sve_sqrshrunt, 4398046511108),
SVEMAP1(svqshl_n_s16_m, aarch64_sve_sqshl, 4398046514434),
SVEMAP1(svqshl_n_s16_x, aarch64_sve_sqshl, 4398046514306),
SVEMAP1(svqshl_n_s16_z, aarch64_sve_sqshl, 4398046514562),
SVEMAP1(svqshl_n_s32_m, aarch64_sve_sqshl, 4398046514435),
SVEMAP1(svqshl_n_s32_x, aarch64_sve_sqshl, 4398046514307),
SVEMAP1(svqshl_n_s32_z, aarch64_sve_sqshl, 4398046514563),
SVEMAP1(svqshl_n_s64_m, aarch64_sve_sqshl, 4398046514436),
SVEMAP1(svqshl_n_s64_x, aarch64_sve_sqshl, 4398046514308),
SVEMAP1(svqshl_n_s64_z, aarch64_sve_sqshl, 4398046514564),
SVEMAP1(svqshl_n_s8_m, aarch64_sve_sqshl, 4398046514433),
SVEMAP1(svqshl_n_s8_x, aarch64_sve_sqshl, 4398046514305),
SVEMAP1(svqshl_n_s8_z, aarch64_sve_sqshl, 4398046514561),
SVEMAP1(svqshl_n_u16_m, aarch64_sve_uqshl, 4398046514434),
SVEMAP1(svqshl_n_u16_x, aarch64_sve_uqshl, 4398046514306),
SVEMAP1(svqshl_n_u16_z, aarch64_sve_uqshl, 4398046514562),
SVEMAP1(svqshl_n_u32_m, aarch64_sve_uqshl, 4398046514435),
SVEMAP1(svqshl_n_u32_x, aarch64_sve_uqshl, 4398046514307),
SVEMAP1(svqshl_n_u32_z, aarch64_sve_uqshl, 4398046514563),
SVEMAP1(svqshl_n_u64_m, aarch64_sve_uqshl, 4398046514436),
SVEMAP1(svqshl_n_u64_x, aarch64_sve_uqshl, 4398046514308),
SVEMAP1(svqshl_n_u64_z, aarch64_sve_uqshl, 4398046514564),
SVEMAP1(svqshl_n_u8_m, aarch64_sve_uqshl, 4398046514433),
SVEMAP1(svqshl_n_u8_x, aarch64_sve_uqshl, 4398046514305),
SVEMAP1(svqshl_n_u8_z, aarch64_sve_uqshl, 4398046514561),
SVEMAP1(svqshl_s16_m, aarch64_sve_sqshl, 4398046511362),
SVEMAP1(svqshl_s16_x, aarch64_sve_sqshl, 4398046511234),
SVEMAP1(svqshl_s16_z, aarch64_sve_sqshl, 4398046511490),
SVEMAP1(svqshl_s32_m, aarch64_sve_sqshl, 4398046511363),
SVEMAP1(svqshl_s32_x, aarch64_sve_sqshl, 4398046511235),
SVEMAP1(svqshl_s32_z, aarch64_sve_sqshl, 4398046511491),
SVEMAP1(svqshl_s64_m, aarch64_sve_sqshl, 4398046511364),
SVEMAP1(svqshl_s64_x, aarch64_sve_sqshl, 4398046511236),
SVEMAP1(svqshl_s64_z, aarch64_sve_sqshl, 4398046511492),
SVEMAP1(svqshl_s8_m, aarch64_sve_sqshl, 4398046511361),
SVEMAP1(svqshl_s8_x, aarch64_sve_sqshl, 4398046511233),
SVEMAP1(svqshl_s8_z, aarch64_sve_sqshl, 4398046511489),
SVEMAP1(svqshl_u16_m, aarch64_sve_uqshl, 4398046511362),
SVEMAP1(svqshl_u16_x, aarch64_sve_uqshl, 4398046511234),
SVEMAP1(svqshl_u16_z, aarch64_sve_uqshl, 4398046511490),
SVEMAP1(svqshl_u32_m, aarch64_sve_uqshl, 4398046511363),
SVEMAP1(svqshl_u32_x, aarch64_sve_uqshl, 4398046511235),
SVEMAP1(svqshl_u32_z, aarch64_sve_uqshl, 4398046511491),
SVEMAP1(svqshl_u64_m, aarch64_sve_uqshl, 4398046511364),
SVEMAP1(svqshl_u64_x, aarch64_sve_uqshl, 4398046511236),
SVEMAP1(svqshl_u64_z, aarch64_sve_uqshl, 4398046511492),
SVEMAP1(svqshl_u8_m, aarch64_sve_uqshl, 4398046511361),
SVEMAP1(svqshl_u8_x, aarch64_sve_uqshl, 4398046511233),
SVEMAP1(svqshl_u8_z, aarch64_sve_uqshl, 4398046511489),
SVEMAP1(svqshlu_n_s16_m, aarch64_sve_sqshlu, 4398046511362),
SVEMAP1(svqshlu_n_s16_x, aarch64_sve_sqshlu, 4398046511234),
SVEMAP1(svqshlu_n_s16_z, aarch64_sve_sqshlu, 4398046511490),
SVEMAP1(svqshlu_n_s32_m, aarch64_sve_sqshlu, 4398046511363),
SVEMAP1(svqshlu_n_s32_x, aarch64_sve_sqshlu, 4398046511235),
SVEMAP1(svqshlu_n_s32_z, aarch64_sve_sqshlu, 4398046511491),
SVEMAP1(svqshlu_n_s64_m, aarch64_sve_sqshlu, 4398046511364),
SVEMAP1(svqshlu_n_s64_x, aarch64_sve_sqshlu, 4398046511236),
SVEMAP1(svqshlu_n_s64_z, aarch64_sve_sqshlu, 4398046511492),
SVEMAP1(svqshlu_n_s8_m, aarch64_sve_sqshlu, 4398046511361),
SVEMAP1(svqshlu_n_s8_x, aarch64_sve_sqshlu, 4398046511233),
SVEMAP1(svqshlu_n_s8_z, aarch64_sve_sqshlu, 4398046511489),
SVEMAP1(svqshrnb_n_s16, aarch64_sve_sqshrnb, 4398046511106),
SVEMAP1(svqshrnb_n_s32, aarch64_sve_sqshrnb, 4398046511107),
SVEMAP1(svqshrnb_n_s64, aarch64_sve_sqshrnb, 4398046511108),
SVEMAP1(svqshrnb_n_u16, aarch64_sve_uqshrnb, 4398046511106),
SVEMAP1(svqshrnb_n_u32, aarch64_sve_uqshrnb, 4398046511107),
SVEMAP1(svqshrnb_n_u64, aarch64_sve_uqshrnb, 4398046511108),
SVEMAP1(svqshrnt_n_s16, aarch64_sve_sqshrnt, 4398046511106),
SVEMAP1(svqshrnt_n_s32, aarch64_sve_sqshrnt, 4398046511107),
SVEMAP1(svqshrnt_n_s64, aarch64_sve_sqshrnt, 4398046511108),
SVEMAP1(svqshrnt_n_u16, aarch64_sve_uqshrnt, 4398046511106),
SVEMAP1(svqshrnt_n_u32, aarch64_sve_uqshrnt, 4398046511107),
SVEMAP1(svqshrnt_n_u64, aarch64_sve_uqshrnt, 4398046511108),
SVEMAP1(svqshrunb_n_s16, aarch64_sve_sqshrunb, 4398046511106),
SVEMAP1(svqshrunb_n_s32, aarch64_sve_sqshrunb, 4398046511107),
SVEMAP1(svqshrunb_n_s64, aarch64_sve_sqshrunb, 4398046511108),
SVEMAP1(svqshrunt_n_s16, aarch64_sve_sqshrunt, 4398046511106),
SVEMAP1(svqshrunt_n_s32, aarch64_sve_sqshrunt, 4398046511107),
SVEMAP1(svqshrunt_n_s64, aarch64_sve_sqshrunt, 4398046511108),
SVEMAP1(svqsub_n_s16, aarch64_sve_sqsub_x, 4398046513154),
SVEMAP1(svqsub_n_s16_m, aarch64_sve_sqsub, 4398046514434),
SVEMAP1(svqsub_n_s16_x, aarch64_sve_sqsub_u, 4398046514306),
SVEMAP1(svqsub_n_s16_z, aarch64_sve_sqsub, 4398046514562),
SVEMAP1(svqsub_n_s32, aarch64_sve_sqsub_x, 4398046513155),
SVEMAP1(svqsub_n_s32_m, aarch64_sve_sqsub, 4398046514435),
SVEMAP1(svqsub_n_s32_x, aarch64_sve_sqsub_u, 4398046514307),
SVEMAP1(svqsub_n_s32_z, aarch64_sve_sqsub, 4398046514563),
SVEMAP1(svqsub_n_s64, aarch64_sve_sqsub_x, 4398046513156),
SVEMAP1(svqsub_n_s64_m, aarch64_sve_sqsub, 4398046514436),
SVEMAP1(svqsub_n_s64_x, aarch64_sve_sqsub_u, 4398046514308),
SVEMAP1(svqsub_n_s64_z, aarch64_sve_sqsub, 4398046514564),
SVEMAP1(svqsub_n_s8, aarch64_sve_sqsub_x, 4398046513153),
SVEMAP1(svqsub_n_s8_m, aarch64_sve_sqsub, 4398046514433),
SVEMAP1(svqsub_n_s8_x, aarch64_sve_sqsub_u, 4398046514305),
SVEMAP1(svqsub_n_s8_z, aarch64_sve_sqsub, 4398046514561),
SVEMAP1(svqsub_n_u16, aarch64_sve_uqsub_x, 4398046513154),
SVEMAP1(svqsub_n_u16_m, aarch64_sve_uqsub, 4398046514434),
SVEMAP1(svqsub_n_u16_x, aarch64_sve_uqsub_u, 4398046514306),
SVEMAP1(svqsub_n_u16_z, aarch64_sve_uqsub, 4398046514562),
SVEMAP1(svqsub_n_u32, aarch64_sve_uqsub_x, 4398046513155),
SVEMAP1(svqsub_n_u32_m, aarch64_sve_uqsub, 4398046514435),
SVEMAP1(svqsub_n_u32_x, aarch64_sve_uqsub_u, 4398046514307),
SVEMAP1(svqsub_n_u32_z, aarch64_sve_uqsub, 4398046514563),
SVEMAP1(svqsub_n_u64, aarch64_sve_uqsub_x, 4398046513156),
SVEMAP1(svqsub_n_u64_m, aarch64_sve_uqsub, 4398046514436),
SVEMAP1(svqsub_n_u64_x, aarch64_sve_uqsub_u, 4398046514308),
SVEMAP1(svqsub_n_u64_z, aarch64_sve_uqsub, 4398046514564),
SVEMAP1(svqsub_n_u8, aarch64_sve_uqsub_x, 4398046513153),
SVEMAP1(svqsub_n_u8_m, aarch64_sve_uqsub, 4398046514433),
SVEMAP1(svqsub_n_u8_x, aarch64_sve_uqsub_u, 4398046514305),
SVEMAP1(svqsub_n_u8_z, aarch64_sve_uqsub, 4398046514561),
SVEMAP1(svqsub_s16, aarch64_sve_sqsub_x, 4398046511106),
SVEMAP1(svqsub_s16_m, aarch64_sve_sqsub, 4398046511362),
SVEMAP1(svqsub_s16_x, aarch64_sve_sqsub_u, 4398046511234),
SVEMAP1(svqsub_s16_z, aarch64_sve_sqsub, 4398046511490),
SVEMAP1(svqsub_s32, aarch64_sve_sqsub_x, 4398046511107),
SVEMAP1(svqsub_s32_m, aarch64_sve_sqsub, 4398046511363),
SVEMAP1(svqsub_s32_x, aarch64_sve_sqsub_u, 4398046511235),
SVEMAP1(svqsub_s32_z, aarch64_sve_sqsub, 4398046511491),
SVEMAP1(svqsub_s64, aarch64_sve_sqsub_x, 4398046511108),
SVEMAP1(svqsub_s64_m, aarch64_sve_sqsub, 4398046511364),
SVEMAP1(svqsub_s64_x, aarch64_sve_sqsub_u, 4398046511236),
SVEMAP1(svqsub_s64_z, aarch64_sve_sqsub, 4398046511492),
SVEMAP1(svqsub_s8, aarch64_sve_sqsub_x, 4398046511105),
SVEMAP1(svqsub_s8_m, aarch64_sve_sqsub, 4398046511361),
SVEMAP1(svqsub_s8_x, aarch64_sve_sqsub_u, 4398046511233),
SVEMAP1(svqsub_s8_z, aarch64_sve_sqsub, 4398046511489),
SVEMAP1(svqsub_u16, aarch64_sve_uqsub_x, 4398046511106),
SVEMAP1(svqsub_u16_m, aarch64_sve_uqsub, 4398046511362),
SVEMAP1(svqsub_u16_x, aarch64_sve_uqsub_u, 4398046511234),
SVEMAP1(svqsub_u16_z, aarch64_sve_uqsub, 4398046511490),
SVEMAP1(svqsub_u32, aarch64_sve_uqsub_x, 4398046511107),
SVEMAP1(svqsub_u32_m, aarch64_sve_uqsub, 4398046511363),
SVEMAP1(svqsub_u32_x, aarch64_sve_uqsub_u, 4398046511235),
SVEMAP1(svqsub_u32_z, aarch64_sve_uqsub, 4398046511491),
SVEMAP1(svqsub_u64, aarch64_sve_uqsub_x, 4398046511108),
SVEMAP1(svqsub_u64_m, aarch64_sve_uqsub, 4398046511364),
SVEMAP1(svqsub_u64_x, aarch64_sve_uqsub_u, 4398046511236),
SVEMAP1(svqsub_u64_z, aarch64_sve_uqsub, 4398046511492),
SVEMAP1(svqsub_u8, aarch64_sve_uqsub_x, 4398046511105),
SVEMAP1(svqsub_u8_m, aarch64_sve_uqsub, 4398046511361),
SVEMAP1(svqsub_u8_x, aarch64_sve_uqsub_u, 4398046511233),
SVEMAP1(svqsub_u8_z, aarch64_sve_uqsub, 4398046511489),
SVEMAP1(svqsubr_n_s16_m, aarch64_sve_sqsubr, 4432406252802),
SVEMAP1(svqsubr_n_s16_x, aarch64_sve_sqsub_u, 4432406252674),
SVEMAP1(svqsubr_n_s16_z, aarch64_sve_sqsubr, 4432406252930),
SVEMAP1(svqsubr_n_s32_m, aarch64_sve_sqsubr, 4432406252803),
SVEMAP1(svqsubr_n_s32_x, aarch64_sve_sqsub_u, 4432406252675),
SVEMAP1(svqsubr_n_s32_z, aarch64_sve_sqsubr, 4432406252931),
SVEMAP1(svqsubr_n_s64_m, aarch64_sve_sqsubr, 4432406252804),
SVEMAP1(svqsubr_n_s64_x, aarch64_sve_sqsub_u, 4432406252676),
SVEMAP1(svqsubr_n_s64_z, aarch64_sve_sqsubr, 4432406252932),
SVEMAP1(svqsubr_n_s8_m, aarch64_sve_sqsubr, 4432406252801),
SVEMAP1(svqsubr_n_s8_x, aarch64_sve_sqsub_u, 4432406252673),
SVEMAP1(svqsubr_n_s8_z, aarch64_sve_sqsubr, 4432406252929),
SVEMAP1(svqsubr_n_u16_m, aarch64_sve_uqsubr, 4432406252802),
SVEMAP1(svqsubr_n_u16_x, aarch64_sve_uqsub_u, 4432406252674),
SVEMAP1(svqsubr_n_u16_z, aarch64_sve_uqsubr, 4432406252930),
SVEMAP1(svqsubr_n_u32_m, aarch64_sve_uqsubr, 4432406252803),
SVEMAP1(svqsubr_n_u32_x, aarch64_sve_uqsub_u, 4432406252675),
SVEMAP1(svqsubr_n_u32_z, aarch64_sve_uqsubr, 4432406252931),
SVEMAP1(svqsubr_n_u64_m, aarch64_sve_uqsubr, 4432406252804),
SVEMAP1(svqsubr_n_u64_x, aarch64_sve_uqsub_u, 4432406252676),
SVEMAP1(svqsubr_n_u64_z, aarch64_sve_uqsubr, 4432406252932),
SVEMAP1(svqsubr_n_u8_m, aarch64_sve_uqsubr, 4432406252801),
SVEMAP1(svqsubr_n_u8_x, aarch64_sve_uqsub_u, 4432406252673),
SVEMAP1(svqsubr_n_u8_z, aarch64_sve_uqsubr, 4432406252929),
SVEMAP1(svqsubr_s16_m, aarch64_sve_sqsubr, 4432406249730),
SVEMAP1(svqsubr_s16_x, aarch64_sve_sqsub_u, 4432406249602),
SVEMAP1(svqsubr_s16_z, aarch64_sve_sqsubr, 4432406249858),
SVEMAP1(svqsubr_s32_m, aarch64_sve_sqsubr, 4432406249731),
SVEMAP1(svqsubr_s32_x, aarch64_sve_sqsub_u, 4432406249603),
SVEMAP1(svqsubr_s32_z, aarch64_sve_sqsubr, 4432406249859),
SVEMAP1(svqsubr_s64_m, aarch64_sve_sqsubr, 4432406249732),
SVEMAP1(svqsubr_s64_x, aarch64_sve_sqsub_u, 4432406249604),
SVEMAP1(svqsubr_s64_z, aarch64_sve_sqsubr, 4432406249860),
SVEMAP1(svqsubr_s8_m, aarch64_sve_sqsubr, 4432406249729),
SVEMAP1(svqsubr_s8_x, aarch64_sve_sqsub_u, 4432406249601),
SVEMAP1(svqsubr_s8_z, aarch64_sve_sqsubr, 4432406249857),
SVEMAP1(svqsubr_u16_m, aarch64_sve_uqsubr, 4432406249730),
SVEMAP1(svqsubr_u16_x, aarch64_sve_uqsub_u, 4432406249602),
SVEMAP1(svqsubr_u16_z, aarch64_sve_uqsubr, 4432406249858),
SVEMAP1(svqsubr_u32_m, aarch64_sve_uqsubr, 4432406249731),
SVEMAP1(svqsubr_u32_x, aarch64_sve_uqsub_u, 4432406249603),
SVEMAP1(svqsubr_u32_z, aarch64_sve_uqsubr, 4432406249859),
SVEMAP1(svqsubr_u64_m, aarch64_sve_uqsubr, 4432406249732),
SVEMAP1(svqsubr_u64_x, aarch64_sve_uqsub_u, 4432406249604),
SVEMAP1(svqsubr_u64_z, aarch64_sve_uqsubr, 4432406249860),
SVEMAP1(svqsubr_u8_m, aarch64_sve_uqsubr, 4432406249729),
SVEMAP1(svqsubr_u8_x, aarch64_sve_uqsub_u, 4432406249601),
SVEMAP1(svqsubr_u8_z, aarch64_sve_uqsubr, 4432406249857),
SVEMAP1(svqxtnb_s16, aarch64_sve_sqxtnb, 4398046511106),
SVEMAP1(svqxtnb_s32, aarch64_sve_sqxtnb, 4398046511107),
SVEMAP1(svqxtnb_s64, aarch64_sve_sqxtnb, 4398046511108),
SVEMAP1(svqxtnb_u16, aarch64_sve_uqxtnb, 4398046511106),
SVEMAP1(svqxtnb_u32, aarch64_sve_uqxtnb, 4398046511107),
SVEMAP1(svqxtnb_u64, aarch64_sve_uqxtnb, 4398046511108),
SVEMAP1(svqxtnt_s16, aarch64_sve_sqxtnt, 4398046511106),
SVEMAP1(svqxtnt_s32, aarch64_sve_sqxtnt, 4398046511107),
SVEMAP1(svqxtnt_s64, aarch64_sve_sqxtnt, 4398046511108),
SVEMAP1(svqxtnt_u16, aarch64_sve_uqxtnt, 4398046511106),
SVEMAP1(svqxtnt_u32, aarch64_sve_uqxtnt, 4398046511107),
SVEMAP1(svqxtnt_u64, aarch64_sve_uqxtnt, 4398046511108),
SVEMAP1(svqxtunb_s16, aarch64_sve_sqxtunb, 4398046511106),
SVEMAP1(svqxtunb_s32, aarch64_sve_sqxtunb, 4398046511107),
SVEMAP1(svqxtunb_s64, aarch64_sve_sqxtunb, 4398046511108),
SVEMAP1(svqxtunt_s16, aarch64_sve_sqxtunt, 4398046511106),
SVEMAP1(svqxtunt_s32, aarch64_sve_sqxtunt, 4398046511107),
SVEMAP1(svqxtunt_s64, aarch64_sve_sqxtunt, 4398046511108),
SVEMAP1(svraddhnb_n_s16, aarch64_sve_raddhnb, 4398046513154),
SVEMAP1(svraddhnb_n_s32, aarch64_sve_raddhnb, 4398046513155),
SVEMAP1(svraddhnb_n_s64, aarch64_sve_raddhnb, 4398046513156),
SVEMAP1(svraddhnb_n_u16, aarch64_sve_raddhnb, 4398046513154),
SVEMAP1(svraddhnb_n_u32, aarch64_sve_raddhnb, 4398046513155),
SVEMAP1(svraddhnb_n_u64, aarch64_sve_raddhnb, 4398046513156),
SVEMAP1(svraddhnb_s16, aarch64_sve_raddhnb, 4398046511106),
SVEMAP1(svraddhnb_s32, aarch64_sve_raddhnb, 4398046511107),
SVEMAP1(svraddhnb_s64, aarch64_sve_raddhnb, 4398046511108),
SVEMAP1(svraddhnb_u16, aarch64_sve_raddhnb, 4398046511106),
SVEMAP1(svraddhnb_u32, aarch64_sve_raddhnb, 4398046511107),
SVEMAP1(svraddhnb_u64, aarch64_sve_raddhnb, 4398046511108),
SVEMAP1(svraddhnt_n_s16, aarch64_sve_raddhnt, 4398046514178),
SVEMAP1(svraddhnt_n_s32, aarch64_sve_raddhnt, 4398046514179),
SVEMAP1(svraddhnt_n_s64, aarch64_sve_raddhnt, 4398046514180),
SVEMAP1(svraddhnt_n_u16, aarch64_sve_raddhnt, 4398046514178),
SVEMAP1(svraddhnt_n_u32, aarch64_sve_raddhnt, 4398046514179),
SVEMAP1(svraddhnt_n_u64, aarch64_sve_raddhnt, 4398046514180),
SVEMAP1(svraddhnt_s16, aarch64_sve_raddhnt, 4398046511106),
SVEMAP1(svraddhnt_s32, aarch64_sve_raddhnt, 4398046511107),
SVEMAP1(svraddhnt_s64, aarch64_sve_raddhnt, 4398046511108),
SVEMAP1(svraddhnt_u16, aarch64_sve_raddhnt, 4398046511106),
SVEMAP1(svraddhnt_u32, aarch64_sve_raddhnt, 4398046511107),
SVEMAP1(svraddhnt_u64, aarch64_sve_raddhnt, 4398046511108),
SVEMAP1(svrax1_s64, aarch64_sve_rax1, 4398047559684),
SVEMAP1(svrax1_u64, aarch64_sve_rax1, 4398047559684),
SVEMAP1(svrbit_s16_m, aarch64_sve_rbit, 4398046511362),
SVEMAP1(svrbit_s16_x, aarch64_sve_rbit, 4398046511618),
SVEMAP1(svrbit_s16_z, aarch64_sve_rbit, 4398046511746),
SVEMAP1(svrbit_s32_m, aarch64_sve_rbit, 4398046511363),
SVEMAP1(svrbit_s32_x, aarch64_sve_rbit, 4398046511619),
SVEMAP1(svrbit_s32_z, aarch64_sve_rbit, 4398046511747),
SVEMAP1(svrbit_s64_m, aarch64_sve_rbit, 4398046511364),
SVEMAP1(svrbit_s64_x, aarch64_sve_rbit, 4398046511620),
SVEMAP1(svrbit_s64_z, aarch64_sve_rbit, 4398046511748),
SVEMAP1(svrbit_s8_m, aarch64_sve_rbit, 4398046511361),
SVEMAP1(svrbit_s8_x, aarch64_sve_rbit, 4398046511617),
SVEMAP1(svrbit_s8_z, aarch64_sve_rbit, 4398046511745),
SVEMAP1(svrbit_u16_m, aarch64_sve_rbit, 4398046511362),
SVEMAP1(svrbit_u16_x, aarch64_sve_rbit, 4398046511618),
SVEMAP1(svrbit_u16_z, aarch64_sve_rbit, 4398046511746),
SVEMAP1(svrbit_u32_m, aarch64_sve_rbit, 4398046511363),
SVEMAP1(svrbit_u32_x, aarch64_sve_rbit, 4398046511619),
SVEMAP1(svrbit_u32_z, aarch64_sve_rbit, 4398046511747),
SVEMAP1(svrbit_u64_m, aarch64_sve_rbit, 4398046511364),
SVEMAP1(svrbit_u64_x, aarch64_sve_rbit, 4398046511620),
SVEMAP1(svrbit_u64_z, aarch64_sve_rbit, 4398046511748),
SVEMAP1(svrbit_u8_m, aarch64_sve_rbit, 4398046511361),
SVEMAP1(svrbit_u8_x, aarch64_sve_rbit, 4398046511617),
SVEMAP1(svrbit_u8_z, aarch64_sve_rbit, 4398046511745),
SVEMAP2(svrdffr, 1048585),
SVEMAP2(svrdffr_z, 1048585),
SVEMAP1(svrecpe_f16, aarch64_sve_frecpe_x, 4398046511110),
SVEMAP1(svrecpe_f32, aarch64_sve_frecpe_x, 4398046511111),
SVEMAP1(svrecpe_f64, aarch64_sve_frecpe_x, 4398046511112),
SVEMAP1(svrecpe_u32_m, aarch64_sve_urecpe, 4398046511363),
SVEMAP1(svrecpe_u32_x, aarch64_sve_urecpe, 4398046511619),
SVEMAP1(svrecpe_u32_z, aarch64_sve_urecpe, 4398046511747),
SVEMAP1(svrecps_f16, aarch64_sve_frecps_x, 4398046511110),
SVEMAP1(svrecps_f32, aarch64_sve_frecps_x, 4398046511111),
SVEMAP1(svrecps_f64, aarch64_sve_frecps_x, 4398046511112),
SVEMAP1(svrecpx_f16_m, aarch64_sve_frecpx, 4398046511366),
SVEMAP1(svrecpx_f16_x, aarch64_sve_frecpx, 4398046511622),
SVEMAP1(svrecpx_f16_z, aarch64_sve_frecpx, 4398046511750),
SVEMAP1(svrecpx_f32_m, aarch64_sve_frecpx, 4398046511367),
SVEMAP1(svrecpx_f32_x, aarch64_sve_frecpx, 4398046511623),
SVEMAP1(svrecpx_f32_z, aarch64_sve_frecpx, 4398046511751),
SVEMAP1(svrecpx_f64_m, aarch64_sve_frecpx, 4398046511368),
SVEMAP1(svrecpx_f64_x, aarch64_sve_frecpx, 4398046511624),
SVEMAP1(svrecpx_f64_z, aarch64_sve_frecpx, 4398046511752),
SVEMAP2(svreinterpret_b, 4398046511113),
SVEMAP2(svreinterpret_c, 4398046511113),
SVEMAP1(svrev_b16, aarch64_sve_rev_b16, 4398047559689),
SVEMAP1(svrev_b32, aarch64_sve_rev_b32, 4398047559689),
SVEMAP1(svrev_b64, aarch64_sve_rev_b64, 4398047559689),
SVEMAP1(svrev_b8, aarch64_sve_rev, 4398046511113),
SVEMAP1(svrev_bf16, aarch64_sve_rev, 4398046511117),
SVEMAP1(svrev_f16, aarch64_sve_rev, 4398046511110),
SVEMAP1(svrev_f32, aarch64_sve_rev, 4398046511111),
SVEMAP1(svrev_f64, aarch64_sve_rev, 4398046511112),
SVEMAP1(svrev_s16, aarch64_sve_rev, 4398046511106),
SVEMAP1(svrev_s32, aarch64_sve_rev, 4398046511107),
SVEMAP1(svrev_s64, aarch64_sve_rev, 4398046511108),
SVEMAP1(svrev_s8, aarch64_sve_rev, 4398046511105),
SVEMAP1(svrev_u16, aarch64_sve_rev, 4398046511106),
SVEMAP1(svrev_u32, aarch64_sve_rev, 4398046511107),
SVEMAP1(svrev_u64, aarch64_sve_rev, 4398046511108),
SVEMAP1(svrev_u8, aarch64_sve_rev, 4398046511105),
SVEMAP1(svrevb_s16_m, aarch64_sve_revb, 4398046511362),
SVEMAP1(svrevb_s16_x, aarch64_sve_revb, 4398046511618),
SVEMAP1(svrevb_s16_z, aarch64_sve_revb, 4398046511746),
SVEMAP1(svrevb_s32_m, aarch64_sve_revb, 4398046511363),
SVEMAP1(svrevb_s32_x, aarch64_sve_revb, 4398046511619),
SVEMAP1(svrevb_s32_z, aarch64_sve_revb, 4398046511747),
SVEMAP1(svrevb_s64_m, aarch64_sve_revb, 4398046511364),
SVEMAP1(svrevb_s64_x, aarch64_sve_revb, 4398046511620),
SVEMAP1(svrevb_s64_z, aarch64_sve_revb, 4398046511748),
SVEMAP1(svrevb_u16_m, aarch64_sve_revb, 4398046511362),
SVEMAP1(svrevb_u16_x, aarch64_sve_revb, 4398046511618),
SVEMAP1(svrevb_u16_z, aarch64_sve_revb, 4398046511746),
SVEMAP1(svrevb_u32_m, aarch64_sve_revb, 4398046511363),
SVEMAP1(svrevb_u32_x, aarch64_sve_revb, 4398046511619),
SVEMAP1(svrevb_u32_z, aarch64_sve_revb, 4398046511747),
SVEMAP1(svrevb_u64_m, aarch64_sve_revb, 4398046511364),
SVEMAP1(svrevb_u64_x, aarch64_sve_revb, 4398046511620),
SVEMAP1(svrevb_u64_z, aarch64_sve_revb, 4398046511748),
SVEMAP1(svrevd_bf16_m, aarch64_sve_revd, 4398046511373),
SVEMAP1(svrevd_bf16_x, aarch64_sve_revd, 4398046511629),
SVEMAP1(svrevd_bf16_z, aarch64_sve_revd, 4398046511757),
SVEMAP1(svrevd_f16_m, aarch64_sve_revd, 4398046511366),
SVEMAP1(svrevd_f16_x, aarch64_sve_revd, 4398046511622),
SVEMAP1(svrevd_f16_z, aarch64_sve_revd, 4398046511750),
SVEMAP1(svrevd_f32_m, aarch64_sve_revd, 4398046511367),
SVEMAP1(svrevd_f32_x, aarch64_sve_revd, 4398046511623),
SVEMAP1(svrevd_f32_z, aarch64_sve_revd, 4398046511751),
SVEMAP1(svrevd_f64_m, aarch64_sve_revd, 4398046511368),
SVEMAP1(svrevd_f64_x, aarch64_sve_revd, 4398046511624),
SVEMAP1(svrevd_f64_z, aarch64_sve_revd, 4398046511752),
SVEMAP1(svrevd_mf8_m, aarch64_sve_revd, 4398046511374),
SVEMAP1(svrevd_mf8_x, aarch64_sve_revd, 4398046511630),
SVEMAP1(svrevd_mf8_z, aarch64_sve_revd, 4398046511758),
SVEMAP1(svrevd_s16_m, aarch64_sve_revd, 4398046511362),
SVEMAP1(svrevd_s16_x, aarch64_sve_revd, 4398046511618),
SVEMAP1(svrevd_s16_z, aarch64_sve_revd, 4398046511746),
SVEMAP1(svrevd_s32_m, aarch64_sve_revd, 4398046511363),
SVEMAP1(svrevd_s32_x, aarch64_sve_revd, 4398046511619),
SVEMAP1(svrevd_s32_z, aarch64_sve_revd, 4398046511747),
SVEMAP1(svrevd_s64_m, aarch64_sve_revd, 4398046511364),
SVEMAP1(svrevd_s64_x, aarch64_sve_revd, 4398046511620),
SVEMAP1(svrevd_s64_z, aarch64_sve_revd, 4398046511748),
SVEMAP1(svrevd_s8_m, aarch64_sve_revd, 4398046511361),
SVEMAP1(svrevd_s8_x, aarch64_sve_revd, 4398046511617),
SVEMAP1(svrevd_s8_z, aarch64_sve_revd, 4398046511745),
SVEMAP1(svrevd_u16_m, aarch64_sve_revd, 4398046511362),
SVEMAP1(svrevd_u16_x, aarch64_sve_revd, 4398046511618),
SVEMAP1(svrevd_u16_z, aarch64_sve_revd, 4398046511746),
SVEMAP1(svrevd_u32_m, aarch64_sve_revd, 4398046511363),
SVEMAP1(svrevd_u32_x, aarch64_sve_revd, 4398046511619),
SVEMAP1(svrevd_u32_z, aarch64_sve_revd, 4398046511747),
SVEMAP1(svrevd_u64_m, aarch64_sve_revd, 4398046511364),
SVEMAP1(svrevd_u64_x, aarch64_sve_revd, 4398046511620),
SVEMAP1(svrevd_u64_z, aarch64_sve_revd, 4398046511748),
SVEMAP1(svrevd_u8_m, aarch64_sve_revd, 4398046511361),
SVEMAP1(svrevd_u8_x, aarch64_sve_revd, 4398046511617),
SVEMAP1(svrevd_u8_z, aarch64_sve_revd, 4398046511745),
SVEMAP1(svrevh_s32_m, aarch64_sve_revh, 4398046511363),
SVEMAP1(svrevh_s32_x, aarch64_sve_revh, 4398046511619),
SVEMAP1(svrevh_s32_z, aarch64_sve_revh, 4398046511747),
SVEMAP1(svrevh_s64_m, aarch64_sve_revh, 4398046511364),
SVEMAP1(svrevh_s64_x, aarch64_sve_revh, 4398046511620),
SVEMAP1(svrevh_s64_z, aarch64_sve_revh, 4398046511748),
SVEMAP1(svrevh_u32_m, aarch64_sve_revh, 4398046511363),
SVEMAP1(svrevh_u32_x, aarch64_sve_revh, 4398046511619),
SVEMAP1(svrevh_u32_z, aarch64_sve_revh, 4398046511747),
SVEMAP1(svrevh_u64_m, aarch64_sve_revh, 4398046511364),
SVEMAP1(svrevh_u64_x, aarch64_sve_revh, 4398046511620),
SVEMAP1(svrevh_u64_z, aarch64_sve_revh, 4398046511748),
SVEMAP1(svrevw_s64_m, aarch64_sve_revw, 4398046511364),
SVEMAP1(svrevw_s64_x, aarch64_sve_revw, 4398046511620),
SVEMAP1(svrevw_s64_z, aarch64_sve_revw, 4398046511748),
SVEMAP1(svrevw_u64_m, aarch64_sve_revw, 4398046511364),
SVEMAP1(svrevw_u64_x, aarch64_sve_revw, 4398046511620),
SVEMAP1(svrevw_u64_z, aarch64_sve_revw, 4398046511748),
SVEMAP1(svrhadd_n_s16_m, aarch64_sve_srhadd, 4398046514434),
SVEMAP1(svrhadd_n_s16_x, aarch64_sve_srhadd, 4398046514306),
SVEMAP1(svrhadd_n_s16_z, aarch64_sve_srhadd, 4398046514562),
SVEMAP1(svrhadd_n_s32_m, aarch64_sve_srhadd, 4398046514435),
SVEMAP1(svrhadd_n_s32_x, aarch64_sve_srhadd, 4398046514307),
SVEMAP1(svrhadd_n_s32_z, aarch64_sve_srhadd, 4398046514563),
SVEMAP1(svrhadd_n_s64_m, aarch64_sve_srhadd, 4398046514436),
SVEMAP1(svrhadd_n_s64_x, aarch64_sve_srhadd, 4398046514308),
SVEMAP1(svrhadd_n_s64_z, aarch64_sve_srhadd, 4398046514564),
SVEMAP1(svrhadd_n_s8_m, aarch64_sve_srhadd, 4398046514433),
SVEMAP1(svrhadd_n_s8_x, aarch64_sve_srhadd, 4398046514305),
SVEMAP1(svrhadd_n_s8_z, aarch64_sve_srhadd, 4398046514561),
SVEMAP1(svrhadd_n_u16_m, aarch64_sve_urhadd, 4398046514434),
SVEMAP1(svrhadd_n_u16_x, aarch64_sve_urhadd, 4398046514306),
SVEMAP1(svrhadd_n_u16_z, aarch64_sve_urhadd, 4398046514562),
SVEMAP1(svrhadd_n_u32_m, aarch64_sve_urhadd, 4398046514435),
SVEMAP1(svrhadd_n_u32_x, aarch64_sve_urhadd, 4398046514307),
SVEMAP1(svrhadd_n_u32_z, aarch64_sve_urhadd, 4398046514563),
SVEMAP1(svrhadd_n_u64_m, aarch64_sve_urhadd, 4398046514436),
SVEMAP1(svrhadd_n_u64_x, aarch64_sve_urhadd, 4398046514308),
SVEMAP1(svrhadd_n_u64_z, aarch64_sve_urhadd, 4398046514564),
SVEMAP1(svrhadd_n_u8_m, aarch64_sve_urhadd, 4398046514433),
SVEMAP1(svrhadd_n_u8_x, aarch64_sve_urhadd, 4398046514305),
SVEMAP1(svrhadd_n_u8_z, aarch64_sve_urhadd, 4398046514561),
SVEMAP1(svrhadd_s16_m, aarch64_sve_srhadd, 4398046511362),
SVEMAP1(svrhadd_s16_x, aarch64_sve_srhadd, 4398046511234),
SVEMAP1(svrhadd_s16_z, aarch64_sve_srhadd, 4398046511490),
SVEMAP1(svrhadd_s32_m, aarch64_sve_srhadd, 4398046511363),
SVEMAP1(svrhadd_s32_x, aarch64_sve_srhadd, 4398046511235),
SVEMAP1(svrhadd_s32_z, aarch64_sve_srhadd, 4398046511491),
SVEMAP1(svrhadd_s64_m, aarch64_sve_srhadd, 4398046511364),
SVEMAP1(svrhadd_s64_x, aarch64_sve_srhadd, 4398046511236),
SVEMAP1(svrhadd_s64_z, aarch64_sve_srhadd, 4398046511492),
SVEMAP1(svrhadd_s8_m, aarch64_sve_srhadd, 4398046511361),
SVEMAP1(svrhadd_s8_x, aarch64_sve_srhadd, 4398046511233),
SVEMAP1(svrhadd_s8_z, aarch64_sve_srhadd, 4398046511489),
SVEMAP1(svrhadd_u16_m, aarch64_sve_urhadd, 4398046511362),
SVEMAP1(svrhadd_u16_x, aarch64_sve_urhadd, 4398046511234),
SVEMAP1(svrhadd_u16_z, aarch64_sve_urhadd, 4398046511490),
SVEMAP1(svrhadd_u32_m, aarch64_sve_urhadd, 4398046511363),
SVEMAP1(svrhadd_u32_x, aarch64_sve_urhadd, 4398046511235),
SVEMAP1(svrhadd_u32_z, aarch64_sve_urhadd, 4398046511491),
SVEMAP1(svrhadd_u64_m, aarch64_sve_urhadd, 4398046511364),
SVEMAP1(svrhadd_u64_x, aarch64_sve_urhadd, 4398046511236),
SVEMAP1(svrhadd_u64_z, aarch64_sve_urhadd, 4398046511492),
SVEMAP1(svrhadd_u8_m, aarch64_sve_urhadd, 4398046511361),
SVEMAP1(svrhadd_u8_x, aarch64_sve_urhadd, 4398046511233),
SVEMAP1(svrhadd_u8_z, aarch64_sve_urhadd, 4398046511489),
SVEMAP1(svrinta_f16_m, aarch64_sve_frinta, 4398046511366),
SVEMAP1(svrinta_f16_x, aarch64_sve_frinta, 4398046511622),
SVEMAP1(svrinta_f16_z, aarch64_sve_frinta, 4398046511750),
SVEMAP1(svrinta_f32_m, aarch64_sve_frinta, 4398046511367),
SVEMAP1(svrinta_f32_x, aarch64_sve_frinta, 4398046511623),
SVEMAP1(svrinta_f32_x2, aarch64_sve_frinta_x2, 137438953479),
SVEMAP1(svrinta_f32_x4, aarch64_sve_frinta_x4, 137438953479),
SVEMAP1(svrinta_f32_z, aarch64_sve_frinta, 4398046511751),
SVEMAP1(svrinta_f64_m, aarch64_sve_frinta, 4398046511368),
SVEMAP1(svrinta_f64_x, aarch64_sve_frinta, 4398046511624),
SVEMAP1(svrinta_f64_z, aarch64_sve_frinta, 4398046511752),
SVEMAP1(svrinti_f16_m, aarch64_sve_frinti, 4398046511366),
SVEMAP1(svrinti_f16_x, aarch64_sve_frinti, 4398046511622),
SVEMAP1(svrinti_f16_z, aarch64_sve_frinti, 4398046511750),
SVEMAP1(svrinti_f32_m, aarch64_sve_frinti, 4398046511367),
SVEMAP1(svrinti_f32_x, aarch64_sve_frinti, 4398046511623),
SVEMAP1(svrinti_f32_z, aarch64_sve_frinti, 4398046511751),
SVEMAP1(svrinti_f64_m, aarch64_sve_frinti, 4398046511368),
SVEMAP1(svrinti_f64_x, aarch64_sve_frinti, 4398046511624),
SVEMAP1(svrinti_f64_z, aarch64_sve_frinti, 4398046511752),
SVEMAP1(svrintm_f16_m, aarch64_sve_frintm, 4398046511366),
SVEMAP1(svrintm_f16_x, aarch64_sve_frintm, 4398046511622),
SVEMAP1(svrintm_f16_z, aarch64_sve_frintm, 4398046511750),
SVEMAP1(svrintm_f32_m, aarch64_sve_frintm, 4398046511367),
SVEMAP1(svrintm_f32_x, aarch64_sve_frintm, 4398046511623),
SVEMAP1(svrintm_f32_x2, aarch64_sve_frintm_x2, 137438953479),
SVEMAP1(svrintm_f32_x4, aarch64_sve_frintm_x4, 137438953479),
SVEMAP1(svrintm_f32_z, aarch64_sve_frintm, 4398046511751),
SVEMAP1(svrintm_f64_m, aarch64_sve_frintm, 4398046511368),
SVEMAP1(svrintm_f64_x, aarch64_sve_frintm, 4398046511624),
SVEMAP1(svrintm_f64_z, aarch64_sve_frintm, 4398046511752),
SVEMAP1(svrintn_f16_m, aarch64_sve_frintn, 4398046511366),
SVEMAP1(svrintn_f16_x, aarch64_sve_frintn, 4398046511622),
SVEMAP1(svrintn_f16_z, aarch64_sve_frintn, 4398046511750),
SVEMAP1(svrintn_f32_m, aarch64_sve_frintn, 4398046511367),
SVEMAP1(svrintn_f32_x, aarch64_sve_frintn, 4398046511623),
SVEMAP1(svrintn_f32_x2, aarch64_sve_frintn_x2, 137438953479),
SVEMAP1(svrintn_f32_x4, aarch64_sve_frintn_x4, 137438953479),
SVEMAP1(svrintn_f32_z, aarch64_sve_frintn, 4398046511751),
SVEMAP1(svrintn_f64_m, aarch64_sve_frintn, 4398046511368),
SVEMAP1(svrintn_f64_x, aarch64_sve_frintn, 4398046511624),
SVEMAP1(svrintn_f64_z, aarch64_sve_frintn, 4398046511752),
SVEMAP1(svrintp_f16_m, aarch64_sve_frintp, 4398046511366),
SVEMAP1(svrintp_f16_x, aarch64_sve_frintp, 4398046511622),
SVEMAP1(svrintp_f16_z, aarch64_sve_frintp, 4398046511750),
SVEMAP1(svrintp_f32_m, aarch64_sve_frintp, 4398046511367),
SVEMAP1(svrintp_f32_x, aarch64_sve_frintp, 4398046511623),
SVEMAP1(svrintp_f32_x2, aarch64_sve_frintp_x2, 137438953479),
SVEMAP1(svrintp_f32_x4, aarch64_sve_frintp_x4, 137438953479),
SVEMAP1(svrintp_f32_z, aarch64_sve_frintp, 4398046511751),
SVEMAP1(svrintp_f64_m, aarch64_sve_frintp, 4398046511368),
SVEMAP1(svrintp_f64_x, aarch64_sve_frintp, 4398046511624),
SVEMAP1(svrintp_f64_z, aarch64_sve_frintp, 4398046511752),
SVEMAP1(svrintx_f16_m, aarch64_sve_frintx, 4398046511366),
SVEMAP1(svrintx_f16_x, aarch64_sve_frintx, 4398046511622),
SVEMAP1(svrintx_f16_z, aarch64_sve_frintx, 4398046511750),
SVEMAP1(svrintx_f32_m, aarch64_sve_frintx, 4398046511367),
SVEMAP1(svrintx_f32_x, aarch64_sve_frintx, 4398046511623),
SVEMAP1(svrintx_f32_z, aarch64_sve_frintx, 4398046511751),
SVEMAP1(svrintx_f64_m, aarch64_sve_frintx, 4398046511368),
SVEMAP1(svrintx_f64_x, aarch64_sve_frintx, 4398046511624),
SVEMAP1(svrintx_f64_z, aarch64_sve_frintx, 4398046511752),
SVEMAP1(svrintz_f16_m, aarch64_sve_frintz, 4398046511366),
SVEMAP1(svrintz_f16_x, aarch64_sve_frintz, 4398046511622),
SVEMAP1(svrintz_f16_z, aarch64_sve_frintz, 4398046511750),
SVEMAP1(svrintz_f32_m, aarch64_sve_frintz, 4398046511367),
SVEMAP1(svrintz_f32_x, aarch64_sve_frintz, 4398046511623),
SVEMAP1(svrintz_f32_z, aarch64_sve_frintz, 4398046511751),
SVEMAP1(svrintz_f64_m, aarch64_sve_frintz, 4398046511368),
SVEMAP1(svrintz_f64_x, aarch64_sve_frintz, 4398046511624),
SVEMAP1(svrintz_f64_z, aarch64_sve_frintz, 4398046511752),
SVEMAP1(svrshl_n_s16_m, aarch64_sve_srshl, 4398046514434),
SVEMAP1(svrshl_n_s16_x, aarch64_sve_srshl, 4398046514306),
SVEMAP1(svrshl_n_s16_z, aarch64_sve_srshl, 4398046514562),
SVEMAP1(svrshl_n_s32_m, aarch64_sve_srshl, 4398046514435),
SVEMAP1(svrshl_n_s32_x, aarch64_sve_srshl, 4398046514307),
SVEMAP1(svrshl_n_s32_z, aarch64_sve_srshl, 4398046514563),
SVEMAP1(svrshl_n_s64_m, aarch64_sve_srshl, 4398046514436),
SVEMAP1(svrshl_n_s64_x, aarch64_sve_srshl, 4398046514308),
SVEMAP1(svrshl_n_s64_z, aarch64_sve_srshl, 4398046514564),
SVEMAP1(svrshl_n_s8_m, aarch64_sve_srshl, 4398046514433),
SVEMAP1(svrshl_n_s8_x, aarch64_sve_srshl, 4398046514305),
SVEMAP1(svrshl_n_s8_z, aarch64_sve_srshl, 4398046514561),
SVEMAP1(svrshl_n_u16_m, aarch64_sve_urshl, 4398046514434),
SVEMAP1(svrshl_n_u16_x, aarch64_sve_urshl, 4398046514306),
SVEMAP1(svrshl_n_u16_z, aarch64_sve_urshl, 4398046514562),
SVEMAP1(svrshl_n_u32_m, aarch64_sve_urshl, 4398046514435),
SVEMAP1(svrshl_n_u32_x, aarch64_sve_urshl, 4398046514307),
SVEMAP1(svrshl_n_u32_z, aarch64_sve_urshl, 4398046514563),
SVEMAP1(svrshl_n_u64_m, aarch64_sve_urshl, 4398046514436),
SVEMAP1(svrshl_n_u64_x, aarch64_sve_urshl, 4398046514308),
SVEMAP1(svrshl_n_u64_z, aarch64_sve_urshl, 4398046514564),
SVEMAP1(svrshl_n_u8_m, aarch64_sve_urshl, 4398046514433),
SVEMAP1(svrshl_n_u8_x, aarch64_sve_urshl, 4398046514305),
SVEMAP1(svrshl_n_u8_z, aarch64_sve_urshl, 4398046514561),
SVEMAP1(svrshl_s16_m, aarch64_sve_srshl, 4398046511362),
SVEMAP1(svrshl_s16_x, aarch64_sve_srshl, 4398046511234),
SVEMAP1(svrshl_s16_x2, aarch64_sve_srshl_x2, 137438953474),
SVEMAP1(svrshl_s16_x4, aarch64_sve_srshl_x4, 137438953474),
SVEMAP1(svrshl_s16_z, aarch64_sve_srshl, 4398046511490),
SVEMAP1(svrshl_s32_m, aarch64_sve_srshl, 4398046511363),
SVEMAP1(svrshl_s32_x, aarch64_sve_srshl, 4398046511235),
SVEMAP1(svrshl_s32_x2, aarch64_sve_srshl_x2, 137438953475),
SVEMAP1(svrshl_s32_x4, aarch64_sve_srshl_x4, 137438953475),
SVEMAP1(svrshl_s32_z, aarch64_sve_srshl, 4398046511491),
SVEMAP1(svrshl_s64_m, aarch64_sve_srshl, 4398046511364),
SVEMAP1(svrshl_s64_x, aarch64_sve_srshl, 4398046511236),
SVEMAP1(svrshl_s64_x2, aarch64_sve_srshl_x2, 137438953476),
SVEMAP1(svrshl_s64_x4, aarch64_sve_srshl_x4, 137438953476),
SVEMAP1(svrshl_s64_z, aarch64_sve_srshl, 4398046511492),
SVEMAP1(svrshl_s8_m, aarch64_sve_srshl, 4398046511361),
SVEMAP1(svrshl_s8_x, aarch64_sve_srshl, 4398046511233),
SVEMAP1(svrshl_s8_x2, aarch64_sve_srshl_x2, 137438953473),
SVEMAP1(svrshl_s8_x4, aarch64_sve_srshl_x4, 137438953473),
SVEMAP1(svrshl_s8_z, aarch64_sve_srshl, 4398046511489),
SVEMAP1(svrshl_single_s16_x2, aarch64_sve_srshl_single_x2, 137438953474),
SVEMAP1(svrshl_single_s16_x4, aarch64_sve_srshl_single_x4, 137438953474),
SVEMAP1(svrshl_single_s32_x2, aarch64_sve_srshl_single_x2, 137438953475),
SVEMAP1(svrshl_single_s32_x4, aarch64_sve_srshl_single_x4, 137438953475),
SVEMAP1(svrshl_single_s64_x2, aarch64_sve_srshl_single_x2, 137438953476),
SVEMAP1(svrshl_single_s64_x4, aarch64_sve_srshl_single_x4, 137438953476),
SVEMAP1(svrshl_single_s8_x2, aarch64_sve_srshl_single_x2, 137438953473),
SVEMAP1(svrshl_single_s8_x4, aarch64_sve_srshl_single_x4, 137438953473),
SVEMAP1(svrshl_single_u16_x2, aarch64_sve_urshl_single_x2, 137438953474),
SVEMAP1(svrshl_single_u16_x4, aarch64_sve_urshl_single_x4, 137438953474),
SVEMAP1(svrshl_single_u32_x2, aarch64_sve_urshl_single_x2, 137438953475),
SVEMAP1(svrshl_single_u32_x4, aarch64_sve_urshl_single_x4, 137438953475),
SVEMAP1(svrshl_single_u64_x2, aarch64_sve_urshl_single_x2, 137438953476),
SVEMAP1(svrshl_single_u64_x4, aarch64_sve_urshl_single_x4, 137438953476),
SVEMAP1(svrshl_single_u8_x2, aarch64_sve_urshl_single_x2, 137438953473),
SVEMAP1(svrshl_single_u8_x4, aarch64_sve_urshl_single_x4, 137438953473),
SVEMAP1(svrshl_u16_m, aarch64_sve_urshl, 4398046511362),
SVEMAP1(svrshl_u16_x, aarch64_sve_urshl, 4398046511234),
SVEMAP1(svrshl_u16_x2, aarch64_sve_urshl_x2, 137438953474),
SVEMAP1(svrshl_u16_x4, aarch64_sve_urshl_x4, 137438953474),
SVEMAP1(svrshl_u16_z, aarch64_sve_urshl, 4398046511490),
SVEMAP1(svrshl_u32_m, aarch64_sve_urshl, 4398046511363),
SVEMAP1(svrshl_u32_x, aarch64_sve_urshl, 4398046511235),
SVEMAP1(svrshl_u32_x2, aarch64_sve_urshl_x2, 137438953475),
SVEMAP1(svrshl_u32_x4, aarch64_sve_urshl_x4, 137438953475),
SVEMAP1(svrshl_u32_z, aarch64_sve_urshl, 4398046511491),
SVEMAP1(svrshl_u64_m, aarch64_sve_urshl, 4398046511364),
SVEMAP1(svrshl_u64_x, aarch64_sve_urshl, 4398046511236),
SVEMAP1(svrshl_u64_x2, aarch64_sve_urshl_x2, 137438953476),
SVEMAP1(svrshl_u64_x4, aarch64_sve_urshl_x4, 137438953476),
SVEMAP1(svrshl_u64_z, aarch64_sve_urshl, 4398046511492),
SVEMAP1(svrshl_u8_m, aarch64_sve_urshl, 4398046511361),
SVEMAP1(svrshl_u8_x, aarch64_sve_urshl, 4398046511233),
SVEMAP1(svrshl_u8_x2, aarch64_sve_urshl_x2, 137438953473),
SVEMAP1(svrshl_u8_x4, aarch64_sve_urshl_x4, 137438953473),
SVEMAP1(svrshl_u8_z, aarch64_sve_urshl, 4398046511489),
SVEMAP1(svrshr_n_s16_m, aarch64_sve_srshr, 4398046511362),
SVEMAP1(svrshr_n_s16_x, aarch64_sve_srshr, 4398046511234),
SVEMAP1(svrshr_n_s16_z, aarch64_sve_srshr, 4398046511490),
SVEMAP1(svrshr_n_s32_m, aarch64_sve_srshr, 4398046511363),
SVEMAP1(svrshr_n_s32_x, aarch64_sve_srshr, 4398046511235),
SVEMAP1(svrshr_n_s32_z, aarch64_sve_srshr, 4398046511491),
SVEMAP1(svrshr_n_s64_m, aarch64_sve_srshr, 4398046511364),
SVEMAP1(svrshr_n_s64_x, aarch64_sve_srshr, 4398046511236),
SVEMAP1(svrshr_n_s64_z, aarch64_sve_srshr, 4398046511492),
SVEMAP1(svrshr_n_s8_m, aarch64_sve_srshr, 4398046511361),
SVEMAP1(svrshr_n_s8_x, aarch64_sve_srshr, 4398046511233),
SVEMAP1(svrshr_n_s8_z, aarch64_sve_srshr, 4398046511489),
SVEMAP1(svrshr_n_u16_m, aarch64_sve_urshr, 4398046511362),
SVEMAP1(svrshr_n_u16_x, aarch64_sve_urshr, 4398046511234),
SVEMAP1(svrshr_n_u16_z, aarch64_sve_urshr, 4398046511490),
SVEMAP1(svrshr_n_u32_m, aarch64_sve_urshr, 4398046511363),
SVEMAP1(svrshr_n_u32_x, aarch64_sve_urshr, 4398046511235),
SVEMAP1(svrshr_n_u32_z, aarch64_sve_urshr, 4398046511491),
SVEMAP1(svrshr_n_u64_m, aarch64_sve_urshr, 4398046511364),
SVEMAP1(svrshr_n_u64_x, aarch64_sve_urshr, 4398046511236),
SVEMAP1(svrshr_n_u64_z, aarch64_sve_urshr, 4398046511492),
SVEMAP1(svrshr_n_u8_m, aarch64_sve_urshr, 4398046511361),
SVEMAP1(svrshr_n_u8_x, aarch64_sve_urshr, 4398046511233),
SVEMAP1(svrshr_n_u8_z, aarch64_sve_urshr, 4398046511489),
SVEMAP1(svrshrnb_n_s16, aarch64_sve_rshrnb, 4398046511106),
SVEMAP1(svrshrnb_n_s32, aarch64_sve_rshrnb, 4398046511107),
SVEMAP1(svrshrnb_n_s64, aarch64_sve_rshrnb, 4398046511108),
SVEMAP1(svrshrnb_n_u16, aarch64_sve_rshrnb, 4398046511106),
SVEMAP1(svrshrnb_n_u32, aarch64_sve_rshrnb, 4398046511107),
SVEMAP1(svrshrnb_n_u64, aarch64_sve_rshrnb, 4398046511108),
SVEMAP1(svrshrnt_n_s16, aarch64_sve_rshrnt, 4398046511106),
SVEMAP1(svrshrnt_n_s32, aarch64_sve_rshrnt, 4398046511107),
SVEMAP1(svrshrnt_n_s64, aarch64_sve_rshrnt, 4398046511108),
SVEMAP1(svrshrnt_n_u16, aarch64_sve_rshrnt, 4398046511106),
SVEMAP1(svrshrnt_n_u32, aarch64_sve_rshrnt, 4398046511107),
SVEMAP1(svrshrnt_n_u64, aarch64_sve_rshrnt, 4398046511108),
SVEMAP1(svrsqrte_f16, aarch64_sve_frsqrte_x, 4398046511110),
SVEMAP1(svrsqrte_f32, aarch64_sve_frsqrte_x, 4398046511111),
SVEMAP1(svrsqrte_f64, aarch64_sve_frsqrte_x, 4398046511112),
SVEMAP1(svrsqrte_u32_m, aarch64_sve_ursqrte, 4398046511363),
SVEMAP1(svrsqrte_u32_x, aarch64_sve_ursqrte, 4398046511619),
SVEMAP1(svrsqrte_u32_z, aarch64_sve_ursqrte, 4398046511747),
SVEMAP1(svrsqrts_f16, aarch64_sve_frsqrts_x, 4398046511110),
SVEMAP1(svrsqrts_f32, aarch64_sve_frsqrts_x, 4398046511111),
SVEMAP1(svrsqrts_f64, aarch64_sve_frsqrts_x, 4398046511112),
SVEMAP1(svrsra_n_s16, aarch64_sve_srsra, 4398046511106),
SVEMAP1(svrsra_n_s32, aarch64_sve_srsra, 4398046511107),
SVEMAP1(svrsra_n_s64, aarch64_sve_srsra, 4398046511108),
SVEMAP1(svrsra_n_s8, aarch64_sve_srsra, 4398046511105),
SVEMAP1(svrsra_n_u16, aarch64_sve_ursra, 4398046511106),
SVEMAP1(svrsra_n_u32, aarch64_sve_ursra, 4398046511107),
SVEMAP1(svrsra_n_u64, aarch64_sve_ursra, 4398046511108),
SVEMAP1(svrsra_n_u8, aarch64_sve_ursra, 4398046511105),
SVEMAP1(svrsubhnb_n_s16, aarch64_sve_rsubhnb, 4398046513154),
SVEMAP1(svrsubhnb_n_s32, aarch64_sve_rsubhnb, 4398046513155),
SVEMAP1(svrsubhnb_n_s64, aarch64_sve_rsubhnb, 4398046513156),
SVEMAP1(svrsubhnb_n_u16, aarch64_sve_rsubhnb, 4398046513154),
SVEMAP1(svrsubhnb_n_u32, aarch64_sve_rsubhnb, 4398046513155),
SVEMAP1(svrsubhnb_n_u64, aarch64_sve_rsubhnb, 4398046513156),
SVEMAP1(svrsubhnb_s16, aarch64_sve_rsubhnb, 4398046511106),
SVEMAP1(svrsubhnb_s32, aarch64_sve_rsubhnb, 4398046511107),
SVEMAP1(svrsubhnb_s64, aarch64_sve_rsubhnb, 4398046511108),
SVEMAP1(svrsubhnb_u16, aarch64_sve_rsubhnb, 4398046511106),
SVEMAP1(svrsubhnb_u32, aarch64_sve_rsubhnb, 4398046511107),
SVEMAP1(svrsubhnb_u64, aarch64_sve_rsubhnb, 4398046511108),
SVEMAP1(svrsubhnt_n_s16, aarch64_sve_rsubhnt, 4398046514178),
SVEMAP1(svrsubhnt_n_s32, aarch64_sve_rsubhnt, 4398046514179),
SVEMAP1(svrsubhnt_n_s64, aarch64_sve_rsubhnt, 4398046514180),
SVEMAP1(svrsubhnt_n_u16, aarch64_sve_rsubhnt, 4398046514178),
SVEMAP1(svrsubhnt_n_u32, aarch64_sve_rsubhnt, 4398046514179),
SVEMAP1(svrsubhnt_n_u64, aarch64_sve_rsubhnt, 4398046514180),
SVEMAP1(svrsubhnt_s16, aarch64_sve_rsubhnt, 4398046511106),
SVEMAP1(svrsubhnt_s32, aarch64_sve_rsubhnt, 4398046511107),
SVEMAP1(svrsubhnt_s64, aarch64_sve_rsubhnt, 4398046511108),
SVEMAP1(svrsubhnt_u16, aarch64_sve_rsubhnt, 4398046511106),
SVEMAP1(svrsubhnt_u32, aarch64_sve_rsubhnt, 4398046511107),
SVEMAP1(svrsubhnt_u64, aarch64_sve_rsubhnt, 4398046511108),
SVEMAP1(svsbclb_n_u32, aarch64_sve_sbclb, 4398046514179),
SVEMAP1(svsbclb_n_u64, aarch64_sve_sbclb, 4398046514180),
SVEMAP1(svsbclb_u32, aarch64_sve_sbclb, 4398046511107),
SVEMAP1(svsbclb_u64, aarch64_sve_sbclb, 4398046511108),
SVEMAP1(svsbclt_n_u32, aarch64_sve_sbclt, 4398046514179),
SVEMAP1(svsbclt_n_u64, aarch64_sve_sbclt, 4398046514180),
SVEMAP1(svsbclt_u32, aarch64_sve_sbclt, 4398046511107),
SVEMAP1(svsbclt_u64, aarch64_sve_sbclt, 4398046511108),
SVEMAP1(svscale_f16_m, aarch64_sve_fscale, 4398046511366),
SVEMAP1(svscale_f16_x, aarch64_sve_fscale, 4398046511238),
SVEMAP1(svscale_f16_x2, aarch64_sme_fp8_scale_x2, 137438953478),
SVEMAP1(svscale_f16_x4, aarch64_sme_fp8_scale_x4, 137438953478),
SVEMAP1(svscale_f16_z, aarch64_sve_fscale, 4398046511494),
SVEMAP1(svscale_f32_m, aarch64_sve_fscale, 4398046511367),
SVEMAP1(svscale_f32_x, aarch64_sve_fscale, 4398046511239),
SVEMAP1(svscale_f32_x2, aarch64_sme_fp8_scale_x2, 137438953479),
SVEMAP1(svscale_f32_x4, aarch64_sme_fp8_scale_x4, 137438953479),
SVEMAP1(svscale_f32_z, aarch64_sve_fscale, 4398046511495),
SVEMAP1(svscale_f64_m, aarch64_sve_fscale, 4398046511368),
SVEMAP1(svscale_f64_x, aarch64_sve_fscale, 4398046511240),
SVEMAP1(svscale_f64_x2, aarch64_sme_fp8_scale_x2, 137438953480),
SVEMAP1(svscale_f64_x4, aarch64_sme_fp8_scale_x4, 137438953480),
SVEMAP1(svscale_f64_z, aarch64_sve_fscale, 4398046511496),
SVEMAP1(svscale_n_f16_m, aarch64_sve_fscale, 4398046514438),
SVEMAP1(svscale_n_f16_x, aarch64_sve_fscale, 4398046514310),
SVEMAP1(svscale_n_f16_z, aarch64_sve_fscale, 4398046514566),
SVEMAP1(svscale_n_f32_m, aarch64_sve_fscale, 4398046514439),
SVEMAP1(svscale_n_f32_x, aarch64_sve_fscale, 4398046514311),
SVEMAP1(svscale_n_f32_z, aarch64_sve_fscale, 4398046514567),
SVEMAP1(svscale_n_f64_m, aarch64_sve_fscale, 4398046514440),
SVEMAP1(svscale_n_f64_x, aarch64_sve_fscale, 4398046514312),
SVEMAP1(svscale_n_f64_z, aarch64_sve_fscale, 4398046514568),
SVEMAP1(svscale_single_f16_x2, aarch64_sme_fp8_scale_single_x2, 137438953478),
SVEMAP1(svscale_single_f16_x4, aarch64_sme_fp8_scale_single_x4, 137438953478),
SVEMAP1(svscale_single_f32_x2, aarch64_sme_fp8_scale_single_x2, 137438953479),
SVEMAP1(svscale_single_f32_x4, aarch64_sme_fp8_scale_single_x4, 137438953479),
SVEMAP1(svscale_single_f64_x2, aarch64_sme_fp8_scale_single_x2, 137438953480),
SVEMAP1(svscale_single_f64_x4, aarch64_sme_fp8_scale_single_x4, 137438953480),
SVEMAP1(svsel_b, aarch64_sve_sel, 4398046511113),
SVEMAP1(svsel_bf16, aarch64_sve_sel, 4398046511117),
SVEMAP1(svsel_bf16_x2, aarch64_sve_sel_x2, 137438953485),
SVEMAP1(svsel_bf16_x4, aarch64_sve_sel_x4, 137438953485),
SVEMAP1(svsel_f16, aarch64_sve_sel, 4398046511110),
SVEMAP1(svsel_f16_x2, aarch64_sve_sel_x2, 137438953478),
SVEMAP1(svsel_f16_x4, aarch64_sve_sel_x4, 137438953478),
SVEMAP1(svsel_f32, aarch64_sve_sel, 4398046511111),
SVEMAP1(svsel_f32_x2, aarch64_sve_sel_x2, 137438953479),
SVEMAP1(svsel_f32_x4, aarch64_sve_sel_x4, 137438953479),
SVEMAP1(svsel_f64, aarch64_sve_sel, 4398046511112),
SVEMAP1(svsel_f64_x2, aarch64_sve_sel_x2, 137438953480),
SVEMAP1(svsel_f64_x4, aarch64_sve_sel_x4, 137438953480),
SVEMAP1(svsel_mf8_x2, aarch64_sve_sel_x2, 137438953486),
SVEMAP1(svsel_mf8_x4, aarch64_sve_sel_x4, 137438953486),
SVEMAP1(svsel_s16, aarch64_sve_sel, 4398046511106),
SVEMAP1(svsel_s16_x2, aarch64_sve_sel_x2, 137438953474),
SVEMAP1(svsel_s16_x4, aarch64_sve_sel_x4, 137438953474),
SVEMAP1(svsel_s32, aarch64_sve_sel, 4398046511107),
SVEMAP1(svsel_s32_x2, aarch64_sve_sel_x2, 137438953475),
SVEMAP1(svsel_s32_x4, aarch64_sve_sel_x4, 137438953475),
SVEMAP1(svsel_s64, aarch64_sve_sel, 4398046511108),
SVEMAP1(svsel_s64_x2, aarch64_sve_sel_x2, 137438953476),
SVEMAP1(svsel_s64_x4, aarch64_sve_sel_x4, 137438953476),
SVEMAP1(svsel_s8, aarch64_sve_sel, 4398046511105),
SVEMAP1(svsel_s8_x2, aarch64_sve_sel_x2, 137438953473),
SVEMAP1(svsel_s8_x4, aarch64_sve_sel_x4, 137438953473),
SVEMAP1(svsel_u16, aarch64_sve_sel, 4398046511106),
SVEMAP1(svsel_u16_x2, aarch64_sve_sel_x2, 137438953474),
SVEMAP1(svsel_u16_x4, aarch64_sve_sel_x4, 137438953474),
SVEMAP1(svsel_u32, aarch64_sve_sel, 4398046511107),
SVEMAP1(svsel_u32_x2, aarch64_sve_sel_x2, 137438953475),
SVEMAP1(svsel_u32_x4, aarch64_sve_sel_x4, 137438953475),
SVEMAP1(svsel_u64, aarch64_sve_sel, 4398046511108),
SVEMAP1(svsel_u64_x2, aarch64_sve_sel_x2, 137438953476),
SVEMAP1(svsel_u64_x4, aarch64_sve_sel_x4, 137438953476),
SVEMAP1(svsel_u8, aarch64_sve_sel, 4398046511105),
SVEMAP1(svsel_u8_x2, aarch64_sve_sel_x2, 137438953473),
SVEMAP1(svsel_u8_x4, aarch64_sve_sel_x4, 137438953473),
SVEMAP2(svset2_b, 4415226380297),
SVEMAP2(svset2_bf16, 4415226380301),
SVEMAP2(svset2_f16, 4415226380294),
SVEMAP2(svset2_f32, 4415226380295),
SVEMAP2(svset2_f64, 4415226380296),
SVEMAP2(svset2_mf8, 4415226380302),
SVEMAP2(svset2_s16, 4415226380290),
SVEMAP2(svset2_s32, 4415226380291),
SVEMAP2(svset2_s64, 4415226380292),
SVEMAP2(svset2_s8, 4415226380289),
SVEMAP2(svset2_u16, 4415226380290),
SVEMAP2(svset2_u32, 4415226380291),
SVEMAP2(svset2_u64, 4415226380292),
SVEMAP2(svset2_u8, 4415226380289),
SVEMAP2(svset3_bf16, 4415226380301),
SVEMAP2(svset3_f16, 4415226380294),
SVEMAP2(svset3_f32, 4415226380295),
SVEMAP2(svset3_f64, 4415226380296),
SVEMAP2(svset3_mf8, 4415226380302),
SVEMAP2(svset3_s16, 4415226380290),
SVEMAP2(svset3_s32, 4415226380291),
SVEMAP2(svset3_s64, 4415226380292),
SVEMAP2(svset3_s8, 4415226380289),
SVEMAP2(svset3_u16, 4415226380290),
SVEMAP2(svset3_u32, 4415226380291),
SVEMAP2(svset3_u64, 4415226380292),
SVEMAP2(svset3_u8, 4415226380289),
SVEMAP2(svset4_b, 4415226380297),
SVEMAP2(svset4_bf16, 4415226380301),
SVEMAP2(svset4_f16, 4415226380294),
SVEMAP2(svset4_f32, 4415226380295),
SVEMAP2(svset4_f64, 4415226380296),
SVEMAP2(svset4_mf8, 4415226380302),
SVEMAP2(svset4_s16, 4415226380290),
SVEMAP2(svset4_s32, 4415226380291),
SVEMAP2(svset4_s64, 4415226380292),
SVEMAP2(svset4_s8, 4415226380289),
SVEMAP2(svset4_u16, 4415226380290),
SVEMAP2(svset4_u32, 4415226380291),
SVEMAP2(svset4_u64, 4415226380292),
SVEMAP2(svset4_u8, 4415226380289),
SVEMAP2(svsetffr, 1048579),
SVEMAP1(svshllb_n_s16, aarch64_sve_sshllb, 4398046511106),
SVEMAP1(svshllb_n_s32, aarch64_sve_sshllb, 4398046511107),
SVEMAP1(svshllb_n_s64, aarch64_sve_sshllb, 4398046511108),
SVEMAP1(svshllb_n_u16, aarch64_sve_ushllb, 4398046511106),
SVEMAP1(svshllb_n_u32, aarch64_sve_ushllb, 4398046511107),
SVEMAP1(svshllb_n_u64, aarch64_sve_ushllb, 4398046511108),
SVEMAP1(svshllt_n_s16, aarch64_sve_sshllt, 4398046511106),
SVEMAP1(svshllt_n_s32, aarch64_sve_sshllt, 4398046511107),
SVEMAP1(svshllt_n_s64, aarch64_sve_sshllt, 4398046511108),
SVEMAP1(svshllt_n_u16, aarch64_sve_ushllt, 4398046511106),
SVEMAP1(svshllt_n_u32, aarch64_sve_ushllt, 4398046511107),
SVEMAP1(svshllt_n_u64, aarch64_sve_ushllt, 4398046511108),
SVEMAP1(svshrnb_n_s16, aarch64_sve_shrnb, 4398046511106),
SVEMAP1(svshrnb_n_s32, aarch64_sve_shrnb, 4398046511107),
SVEMAP1(svshrnb_n_s64, aarch64_sve_shrnb, 4398046511108),
SVEMAP1(svshrnb_n_u16, aarch64_sve_shrnb, 4398046511106),
SVEMAP1(svshrnb_n_u32, aarch64_sve_shrnb, 4398046511107),
SVEMAP1(svshrnb_n_u64, aarch64_sve_shrnb, 4398046511108),
SVEMAP1(svshrnt_n_s16, aarch64_sve_shrnt, 4398046511106),
SVEMAP1(svshrnt_n_s32, aarch64_sve_shrnt, 4398046511107),
SVEMAP1(svshrnt_n_s64, aarch64_sve_shrnt, 4398046511108),
SVEMAP1(svshrnt_n_u16, aarch64_sve_shrnt, 4398046511106),
SVEMAP1(svshrnt_n_u32, aarch64_sve_shrnt, 4398046511107),
SVEMAP1(svshrnt_n_u64, aarch64_sve_shrnt, 4398046511108),
SVEMAP1(svsli_n_s16, aarch64_sve_sli, 4398046511106),
SVEMAP1(svsli_n_s32, aarch64_sve_sli, 4398046511107),
SVEMAP1(svsli_n_s64, aarch64_sve_sli, 4398046511108),
SVEMAP1(svsli_n_s8, aarch64_sve_sli, 4398046511105),
SVEMAP1(svsli_n_u16, aarch64_sve_sli, 4398046511106),
SVEMAP1(svsli_n_u32, aarch64_sve_sli, 4398046511107),
SVEMAP1(svsli_n_u64, aarch64_sve_sli, 4398046511108),
SVEMAP1(svsli_n_u8, aarch64_sve_sli, 4398046511105),
SVEMAP1(svsm4e_u32, aarch64_sve_sm4e, 1048579),
SVEMAP1(svsm4ekey_u32, aarch64_sve_sm4ekey, 1048579),
SVEMAP1(svsplice_bf16, aarch64_sve_splice, 4398046511117),
SVEMAP1(svsplice_f16, aarch64_sve_splice, 4398046511110),
SVEMAP1(svsplice_f32, aarch64_sve_splice, 4398046511111),
SVEMAP1(svsplice_f64, aarch64_sve_splice, 4398046511112),
SVEMAP1(svsplice_s16, aarch64_sve_splice, 4398046511106),
SVEMAP1(svsplice_s32, aarch64_sve_splice, 4398046511107),
SVEMAP1(svsplice_s64, aarch64_sve_splice, 4398046511108),
SVEMAP1(svsplice_s8, aarch64_sve_splice, 4398046511105),
SVEMAP1(svsplice_u16, aarch64_sve_splice, 4398046511106),
SVEMAP1(svsplice_u32, aarch64_sve_splice, 4398046511107),
SVEMAP1(svsplice_u64, aarch64_sve_splice, 4398046511108),
SVEMAP1(svsplice_u8, aarch64_sve_splice, 4398046511105),
SVEMAP1(svsqadd_n_u16_m, aarch64_sve_usqadd, 4398046514434),
SVEMAP1(svsqadd_n_u16_x, aarch64_sve_usqadd, 4398046514306),
SVEMAP1(svsqadd_n_u16_z, aarch64_sve_usqadd, 4398046514562),
SVEMAP1(svsqadd_n_u32_m, aarch64_sve_usqadd, 4398046514435),
SVEMAP1(svsqadd_n_u32_x, aarch64_sve_usqadd, 4398046514307),
SVEMAP1(svsqadd_n_u32_z, aarch64_sve_usqadd, 4398046514563),
SVEMAP1(svsqadd_n_u64_m, aarch64_sve_usqadd, 4398046514436),
SVEMAP1(svsqadd_n_u64_x, aarch64_sve_usqadd, 4398046514308),
SVEMAP1(svsqadd_n_u64_z, aarch64_sve_usqadd, 4398046514564),
SVEMAP1(svsqadd_n_u8_m, aarch64_sve_usqadd, 4398046514433),
SVEMAP1(svsqadd_n_u8_x, aarch64_sve_usqadd, 4398046514305),
SVEMAP1(svsqadd_n_u8_z, aarch64_sve_usqadd, 4398046514561),
SVEMAP1(svsqadd_u16_m, aarch64_sve_usqadd, 4398046511362),
SVEMAP1(svsqadd_u16_x, aarch64_sve_usqadd, 4398046511234),
SVEMAP1(svsqadd_u16_z, aarch64_sve_usqadd, 4398046511490),
SVEMAP1(svsqadd_u32_m, aarch64_sve_usqadd, 4398046511363),
SVEMAP1(svsqadd_u32_x, aarch64_sve_usqadd, 4398046511235),
SVEMAP1(svsqadd_u32_z, aarch64_sve_usqadd, 4398046511491),
SVEMAP1(svsqadd_u64_m, aarch64_sve_usqadd, 4398046511364),
SVEMAP1(svsqadd_u64_x, aarch64_sve_usqadd, 4398046511236),
SVEMAP1(svsqadd_u64_z, aarch64_sve_usqadd, 4398046511492),
SVEMAP1(svsqadd_u8_m, aarch64_sve_usqadd, 4398046511361),
SVEMAP1(svsqadd_u8_x, aarch64_sve_usqadd, 4398046511233),
SVEMAP1(svsqadd_u8_z, aarch64_sve_usqadd, 4398046511489),
SVEMAP1(svsqrt_f16_m, aarch64_sve_fsqrt, 4398046511366),
SVEMAP1(svsqrt_f16_x, aarch64_sve_fsqrt, 4398046511622),
SVEMAP1(svsqrt_f16_z, aarch64_sve_fsqrt, 4398046511750),
SVEMAP1(svsqrt_f32_m, aarch64_sve_fsqrt, 4398046511367),
SVEMAP1(svsqrt_f32_x, aarch64_sve_fsqrt, 4398046511623),
SVEMAP1(svsqrt_f32_z, aarch64_sve_fsqrt, 4398046511751),
SVEMAP1(svsqrt_f64_m, aarch64_sve_fsqrt, 4398046511368),
SVEMAP1(svsqrt_f64_x, aarch64_sve_fsqrt, 4398046511624),
SVEMAP1(svsqrt_f64_z, aarch64_sve_fsqrt, 4398046511752),
SVEMAP1(svsra_n_s16, aarch64_sve_ssra, 4398046511106),
SVEMAP1(svsra_n_s32, aarch64_sve_ssra, 4398046511107),
SVEMAP1(svsra_n_s64, aarch64_sve_ssra, 4398046511108),
SVEMAP1(svsra_n_s8, aarch64_sve_ssra, 4398046511105),
SVEMAP1(svsra_n_u16, aarch64_sve_usra, 4398046511106),
SVEMAP1(svsra_n_u32, aarch64_sve_usra, 4398046511107),
SVEMAP1(svsra_n_u64, aarch64_sve_usra, 4398046511108),
SVEMAP1(svsra_n_u8, aarch64_sve_usra, 4398046511105),
SVEMAP1(svsri_n_s16, aarch64_sve_sri, 4398046511106),
SVEMAP1(svsri_n_s32, aarch64_sve_sri, 4398046511107),
SVEMAP1(svsri_n_s64, aarch64_sve_sri, 4398046511108),
SVEMAP1(svsri_n_s8, aarch64_sve_sri, 4398046511105),
SVEMAP1(svsri_n_u16, aarch64_sve_sri, 4398046511106),
SVEMAP1(svsri_n_u32, aarch64_sve_sri, 4398046511107),
SVEMAP1(svsri_n_u64, aarch64_sve_sri, 4398046511108),
SVEMAP1(svsri_n_u8, aarch64_sve_sri, 4398046511105),
SVEMAP1(svst1_bf16, aarch64_sve_st1, 4398046527501),
SVEMAP1(svst1_bf16_x2, aarch64_sve_st1_pn_x2, 4398046773261),
SVEMAP1(svst1_bf16_x4, aarch64_sve_st1_pn_x4, 4398046773261),
SVEMAP1(svst1_f16, aarch64_sve_st1, 4398046527494),
SVEMAP1(svst1_f16_x2, aarch64_sve_st1_pn_x2, 4398046773254),
SVEMAP1(svst1_f16_x4, aarch64_sve_st1_pn_x4, 4398046773254),
SVEMAP1(svst1_f32, aarch64_sve_st1, 4398046527495),
SVEMAP1(svst1_f32_x2, aarch64_sve_st1_pn_x2, 4398046773255),
SVEMAP1(svst1_f32_x4, aarch64_sve_st1_pn_x4, 4398046773255),
SVEMAP1(svst1_f64, aarch64_sve_st1, 4398046527496),
SVEMAP1(svst1_f64_x2, aarch64_sve_st1_pn_x2, 4398046773256),
SVEMAP1(svst1_f64_x4, aarch64_sve_st1_pn_x4, 4398046773256),
SVEMAP1(svst1_mf8, aarch64_sve_st1, 4398046527502),
SVEMAP1(svst1_mf8_x2, aarch64_sve_st1_pn_x2, 4398046773262),
SVEMAP1(svst1_mf8_x4, aarch64_sve_st1_pn_x4, 4398046773262),
SVEMAP1(svst1_s16, aarch64_sve_st1, 4398046527490),
SVEMAP1(svst1_s16_x2, aarch64_sve_st1_pn_x2, 4398046773250),
SVEMAP1(svst1_s16_x4, aarch64_sve_st1_pn_x4, 4398046773250),
SVEMAP1(svst1_s32, aarch64_sve_st1, 4398046527491),
SVEMAP1(svst1_s32_x2, aarch64_sve_st1_pn_x2, 4398046773251),
SVEMAP1(svst1_s32_x4, aarch64_sve_st1_pn_x4, 4398046773251),
SVEMAP1(svst1_s64, aarch64_sve_st1, 4398046527492),
SVEMAP1(svst1_s64_x2, aarch64_sve_st1_pn_x2, 4398046773252),
SVEMAP1(svst1_s64_x4, aarch64_sve_st1_pn_x4, 4398046773252),
SVEMAP1(svst1_s8, aarch64_sve_st1, 4398046527489),
SVEMAP1(svst1_s8_x2, aarch64_sve_st1_pn_x2, 4398046773249),
SVEMAP1(svst1_s8_x4, aarch64_sve_st1_pn_x4, 4398046773249),
SVEMAP1(svst1_scatter_s32index_f32, aarch64_sve_st1_scatter_sxtw_index, 65543),
SVEMAP1(svst1_scatter_s32index_s32, aarch64_sve_st1_scatter_sxtw_index, 65539),
SVEMAP1(svst1_scatter_s32index_u32, aarch64_sve_st1_scatter_sxtw_index, 65539),
SVEMAP1(svst1_scatter_s32offset_f32, aarch64_sve_st1_scatter_sxtw, 16842759),
SVEMAP1(svst1_scatter_s32offset_s32, aarch64_sve_st1_scatter_sxtw, 16842755),
SVEMAP1(svst1_scatter_s32offset_u32, aarch64_sve_st1_scatter_sxtw, 16842755),
SVEMAP1(svst1_scatter_s64index_f64, aarch64_sve_st1_scatter_index, 65544),
SVEMAP1(svst1_scatter_s64index_s64, aarch64_sve_st1_scatter_index, 65540),
SVEMAP1(svst1_scatter_s64index_u64, aarch64_sve_st1_scatter_index, 65540),
SVEMAP1(svst1_scatter_s64offset_f64, aarch64_sve_st1_scatter, 16842760),
SVEMAP1(svst1_scatter_s64offset_s64, aarch64_sve_st1_scatter, 16842756),
SVEMAP1(svst1_scatter_s64offset_u64, aarch64_sve_st1_scatter, 16842756),
SVEMAP1(svst1_scatter_u32base_f32, aarch64_sve_st1_scatter_scalar_offset, 65543),
SVEMAP1(svst1_scatter_u32base_index_f32, aarch64_sve_st1_scatter_scalar_offset, 65543),
SVEMAP1(svst1_scatter_u32base_index_s32, aarch64_sve_st1_scatter_scalar_offset, 65539),
SVEMAP1(svst1_scatter_u32base_index_u32, aarch64_sve_st1_scatter_scalar_offset, 65539),
SVEMAP1(svst1_scatter_u32base_offset_f32, aarch64_sve_st1_scatter_scalar_offset, 16842759),
SVEMAP1(svst1_scatter_u32base_offset_s32, aarch64_sve_st1_scatter_scalar_offset, 16842755),
SVEMAP1(svst1_scatter_u32base_offset_u32, aarch64_sve_st1_scatter_scalar_offset, 16842755),
SVEMAP1(svst1_scatter_u32base_s32, aarch64_sve_st1_scatter_scalar_offset, 65539),
SVEMAP1(svst1_scatter_u32base_u32, aarch64_sve_st1_scatter_scalar_offset, 65539),
SVEMAP1(svst1_scatter_u32index_f32, aarch64_sve_st1_scatter_uxtw_index, 65543),
SVEMAP1(svst1_scatter_u32index_s32, aarch64_sve_st1_scatter_uxtw_index, 65539),
SVEMAP1(svst1_scatter_u32index_u32, aarch64_sve_st1_scatter_uxtw_index, 65539),
SVEMAP1(svst1_scatter_u32offset_f32, aarch64_sve_st1_scatter_uxtw, 16842759),
SVEMAP1(svst1_scatter_u32offset_s32, aarch64_sve_st1_scatter_uxtw, 16842755),
SVEMAP1(svst1_scatter_u32offset_u32, aarch64_sve_st1_scatter_uxtw, 16842755),
SVEMAP1(svst1_scatter_u64base_f64, aarch64_sve_st1_scatter_scalar_offset, 65544),
SVEMAP1(svst1_scatter_u64base_index_f64, aarch64_sve_st1_scatter_scalar_offset, 65544),
SVEMAP1(svst1_scatter_u64base_index_s64, aarch64_sve_st1_scatter_scalar_offset, 65540),
SVEMAP1(svst1_scatter_u64base_index_u64, aarch64_sve_st1_scatter_scalar_offset, 65540),
SVEMAP1(svst1_scatter_u64base_offset_f64, aarch64_sve_st1_scatter_scalar_offset, 16842760),
SVEMAP1(svst1_scatter_u64base_offset_s64, aarch64_sve_st1_scatter_scalar_offset, 16842756),
SVEMAP1(svst1_scatter_u64base_offset_u64, aarch64_sve_st1_scatter_scalar_offset, 16842756),
SVEMAP1(svst1_scatter_u64base_s64, aarch64_sve_st1_scatter_scalar_offset, 65540),
SVEMAP1(svst1_scatter_u64base_u64, aarch64_sve_st1_scatter_scalar_offset, 65540),
SVEMAP1(svst1_scatter_u64index_f64, aarch64_sve_st1_scatter_index, 65544),
SVEMAP1(svst1_scatter_u64index_s64, aarch64_sve_st1_scatter_index, 65540),
SVEMAP1(svst1_scatter_u64index_u64, aarch64_sve_st1_scatter_index, 65540),
SVEMAP1(svst1_scatter_u64offset_f64, aarch64_sve_st1_scatter, 16842760),
SVEMAP1(svst1_scatter_u64offset_s64, aarch64_sve_st1_scatter, 16842756),
SVEMAP1(svst1_scatter_u64offset_u64, aarch64_sve_st1_scatter, 16842756),
SVEMAP1(svst1_u16, aarch64_sve_st1, 4398046527490),
SVEMAP1(svst1_u16_x2, aarch64_sve_st1_pn_x2, 4398046773250),
SVEMAP1(svst1_u16_x4, aarch64_sve_st1_pn_x4, 4398046773250),
SVEMAP1(svst1_u32, aarch64_sve_st1, 4398046527491),
SVEMAP1(svst1_u32_x2, aarch64_sve_st1_pn_x2, 4398046773251),
SVEMAP1(svst1_u32_x4, aarch64_sve_st1_pn_x4, 4398046773251),
SVEMAP1(svst1_u64, aarch64_sve_st1, 4398046527492),
SVEMAP1(svst1_u64_x2, aarch64_sve_st1_pn_x2, 4398046773252),
SVEMAP1(svst1_u64_x4, aarch64_sve_st1_pn_x4, 4398046773252),
SVEMAP1(svst1_u8, aarch64_sve_st1, 4398046527489),
SVEMAP1(svst1_u8_x2, aarch64_sve_st1_pn_x2, 4398046773249),
SVEMAP1(svst1_u8_x4, aarch64_sve_st1_pn_x4, 4398046773249),
SVEMAP1(svst1_vnum_bf16, aarch64_sve_st1, 4398046527501),
SVEMAP1(svst1_vnum_bf16_x2, aarch64_sve_st1_pn_x2, 4398046773261),
SVEMAP1(svst1_vnum_bf16_x4, aarch64_sve_st1_pn_x4, 4398046773261),
SVEMAP1(svst1_vnum_f16, aarch64_sve_st1, 4398046527494),
SVEMAP1(svst1_vnum_f16_x2, aarch64_sve_st1_pn_x2, 4398046773254),
SVEMAP1(svst1_vnum_f16_x4, aarch64_sve_st1_pn_x4, 4398046773254),
SVEMAP1(svst1_vnum_f32, aarch64_sve_st1, 4398046527495),
SVEMAP1(svst1_vnum_f32_x2, aarch64_sve_st1_pn_x2, 4398046773255),
SVEMAP1(svst1_vnum_f32_x4, aarch64_sve_st1_pn_x4, 4398046773255),
SVEMAP1(svst1_vnum_f64, aarch64_sve_st1, 4398046527496),
SVEMAP1(svst1_vnum_f64_x2, aarch64_sve_st1_pn_x2, 4398046773256),
SVEMAP1(svst1_vnum_f64_x4, aarch64_sve_st1_pn_x4, 4398046773256),
SVEMAP1(svst1_vnum_mf8, aarch64_sve_st1, 4398046527502),
SVEMAP1(svst1_vnum_mf8_x2, aarch64_sve_st1_pn_x2, 4398046773262),
SVEMAP1(svst1_vnum_mf8_x4, aarch64_sve_st1_pn_x4, 4398046773262),
SVEMAP1(svst1_vnum_s16, aarch64_sve_st1, 4398046527490),
SVEMAP1(svst1_vnum_s16_x2, aarch64_sve_st1_pn_x2, 4398046773250),
SVEMAP1(svst1_vnum_s16_x4, aarch64_sve_st1_pn_x4, 4398046773250),
SVEMAP1(svst1_vnum_s32, aarch64_sve_st1, 4398046527491),
SVEMAP1(svst1_vnum_s32_x2, aarch64_sve_st1_pn_x2, 4398046773251),
SVEMAP1(svst1_vnum_s32_x4, aarch64_sve_st1_pn_x4, 4398046773251),
SVEMAP1(svst1_vnum_s64, aarch64_sve_st1, 4398046527492),
SVEMAP1(svst1_vnum_s64_x2, aarch64_sve_st1_pn_x2, 4398046773252),
SVEMAP1(svst1_vnum_s64_x4, aarch64_sve_st1_pn_x4, 4398046773252),
SVEMAP1(svst1_vnum_s8, aarch64_sve_st1, 4398046527489),
SVEMAP1(svst1_vnum_s8_x2, aarch64_sve_st1_pn_x2, 4398046773249),
SVEMAP1(svst1_vnum_s8_x4, aarch64_sve_st1_pn_x4, 4398046773249),
SVEMAP1(svst1_vnum_u16, aarch64_sve_st1, 4398046527490),
SVEMAP1(svst1_vnum_u16_x2, aarch64_sve_st1_pn_x2, 4398046773250),
SVEMAP1(svst1_vnum_u16_x4, aarch64_sve_st1_pn_x4, 4398046773250),
SVEMAP1(svst1_vnum_u32, aarch64_sve_st1, 4398046527491),
SVEMAP1(svst1_vnum_u32_x2, aarch64_sve_st1_pn_x2, 4398046773251),
SVEMAP1(svst1_vnum_u32_x4, aarch64_sve_st1_pn_x4, 4398046773251),
SVEMAP1(svst1_vnum_u64, aarch64_sve_st1, 4398046527492),
SVEMAP1(svst1_vnum_u64_x2, aarch64_sve_st1_pn_x2, 4398046773252),
SVEMAP1(svst1_vnum_u64_x4, aarch64_sve_st1_pn_x4, 4398046773252),
SVEMAP1(svst1_vnum_u8, aarch64_sve_st1, 4398046527489),
SVEMAP1(svst1_vnum_u8_x2, aarch64_sve_st1_pn_x2, 4398046773249),
SVEMAP1(svst1_vnum_u8_x4, aarch64_sve_st1_pn_x4, 4398046773249),
SVEMAP1(svst1b_s16, aarch64_sve_st1, 4398046527506),
SVEMAP1(svst1b_s32, aarch64_sve_st1, 4398046527507),
SVEMAP1(svst1b_s64, aarch64_sve_st1, 4398046527508),
SVEMAP1(svst1b_scatter_s32offset_s32, aarch64_sve_st1_scatter_sxtw, 16842771),
SVEMAP1(svst1b_scatter_s32offset_u32, aarch64_sve_st1_scatter_sxtw, 16842771),
SVEMAP1(svst1b_scatter_s64offset_s64, aarch64_sve_st1_scatter, 16842772),
SVEMAP1(svst1b_scatter_s64offset_u64, aarch64_sve_st1_scatter, 16842772),
SVEMAP1(svst1b_scatter_u32base_offset_s32, aarch64_sve_st1_scatter_scalar_offset, 16842771),
SVEMAP1(svst1b_scatter_u32base_offset_u32, aarch64_sve_st1_scatter_scalar_offset, 16842771),
SVEMAP1(svst1b_scatter_u32base_s32, aarch64_sve_st1_scatter_scalar_offset, 65555),
SVEMAP1(svst1b_scatter_u32base_u32, aarch64_sve_st1_scatter_scalar_offset, 65555),
SVEMAP1(svst1b_scatter_u32offset_s32, aarch64_sve_st1_scatter_uxtw, 16842771),
SVEMAP1(svst1b_scatter_u32offset_u32, aarch64_sve_st1_scatter_uxtw, 16842771),
SVEMAP1(svst1b_scatter_u64base_offset_s64, aarch64_sve_st1_scatter_scalar_offset, 16842772),
SVEMAP1(svst1b_scatter_u64base_offset_u64, aarch64_sve_st1_scatter_scalar_offset, 16842772),
SVEMAP1(svst1b_scatter_u64base_s64, aarch64_sve_st1_scatter_scalar_offset, 65556),
SVEMAP1(svst1b_scatter_u64base_u64, aarch64_sve_st1_scatter_scalar_offset, 65556),
SVEMAP1(svst1b_scatter_u64offset_s64, aarch64_sve_st1_scatter, 16842772),
SVEMAP1(svst1b_scatter_u64offset_u64, aarch64_sve_st1_scatter, 16842772),
SVEMAP1(svst1b_u16, aarch64_sve_st1, 4398046527506),
SVEMAP1(svst1b_u32, aarch64_sve_st1, 4398046527507),
SVEMAP1(svst1b_u64, aarch64_sve_st1, 4398046527508),
SVEMAP1(svst1b_vnum_s16, aarch64_sve_st1, 4398046527506),
SVEMAP1(svst1b_vnum_s32, aarch64_sve_st1, 4398046527507),
SVEMAP1(svst1b_vnum_s64, aarch64_sve_st1, 4398046527508),
SVEMAP1(svst1b_vnum_u16, aarch64_sve_st1, 4398046527506),
SVEMAP1(svst1b_vnum_u32, aarch64_sve_st1, 4398046527507),
SVEMAP1(svst1b_vnum_u64, aarch64_sve_st1, 4398046527508),
SVEMAP1(svst1dq_f64, aarch64_sve_st1dq, 16456),
SVEMAP1(svst1dq_s64, aarch64_sve_st1dq, 16452),
SVEMAP1(svst1dq_u64, aarch64_sve_st1dq, 16452),
SVEMAP1(svst1dq_vnum_f64, aarch64_sve_st1dq, 16456),
SVEMAP1(svst1dq_vnum_s64, aarch64_sve_st1dq, 16452),
SVEMAP1(svst1dq_vnum_u64, aarch64_sve_st1dq, 16452),
SVEMAP1(svst1h_s32, aarch64_sve_st1, 4398046527523),
SVEMAP1(svst1h_s64, aarch64_sve_st1, 4398046527524),
SVEMAP1(svst1h_scatter_s32index_s32, aarch64_sve_st1_scatter_sxtw_index, 65571),
SVEMAP1(svst1h_scatter_s32index_u32, aarch64_sve_st1_scatter_sxtw_index, 65571),
SVEMAP1(svst1h_scatter_s32offset_s32, aarch64_sve_st1_scatter_sxtw, 16842787),
SVEMAP1(svst1h_scatter_s32offset_u32, aarch64_sve_st1_scatter_sxtw, 16842787),
SVEMAP1(svst1h_scatter_s64index_s64, aarch64_sve_st1_scatter_index, 65572),
SVEMAP1(svst1h_scatter_s64index_u64, aarch64_sve_st1_scatter_index, 65572),
SVEMAP1(svst1h_scatter_s64offset_s64, aarch64_sve_st1_scatter, 16842788),
SVEMAP1(svst1h_scatter_s64offset_u64, aarch64_sve_st1_scatter, 16842788),
SVEMAP1(svst1h_scatter_u32base_index_s32, aarch64_sve_st1_scatter_scalar_offset, 65571),
SVEMAP1(svst1h_scatter_u32base_index_u32, aarch64_sve_st1_scatter_scalar_offset, 65571),
SVEMAP1(svst1h_scatter_u32base_offset_s32, aarch64_sve_st1_scatter_scalar_offset, 16842787),
SVEMAP1(svst1h_scatter_u32base_offset_u32, aarch64_sve_st1_scatter_scalar_offset, 16842787),
SVEMAP1(svst1h_scatter_u32base_s32, aarch64_sve_st1_scatter_scalar_offset, 65571),
SVEMAP1(svst1h_scatter_u32base_u32, aarch64_sve_st1_scatter_scalar_offset, 65571),
SVEMAP1(svst1h_scatter_u32index_s32, aarch64_sve_st1_scatter_uxtw_index, 65571),
SVEMAP1(svst1h_scatter_u32index_u32, aarch64_sve_st1_scatter_uxtw_index, 65571),
SVEMAP1(svst1h_scatter_u32offset_s32, aarch64_sve_st1_scatter_uxtw, 16842787),
SVEMAP1(svst1h_scatter_u32offset_u32, aarch64_sve_st1_scatter_uxtw, 16842787),
SVEMAP1(svst1h_scatter_u64base_index_s64, aarch64_sve_st1_scatter_scalar_offset, 65572),
SVEMAP1(svst1h_scatter_u64base_index_u64, aarch64_sve_st1_scatter_scalar_offset, 65572),
SVEMAP1(svst1h_scatter_u64base_offset_s64, aarch64_sve_st1_scatter_scalar_offset, 16842788),
SVEMAP1(svst1h_scatter_u64base_offset_u64, aarch64_sve_st1_scatter_scalar_offset, 16842788),
SVEMAP1(svst1h_scatter_u64base_s64, aarch64_sve_st1_scatter_scalar_offset, 65572),
SVEMAP1(svst1h_scatter_u64base_u64, aarch64_sve_st1_scatter_scalar_offset, 65572),
SVEMAP1(svst1h_scatter_u64index_s64, aarch64_sve_st1_scatter_index, 65572),
SVEMAP1(svst1h_scatter_u64index_u64, aarch64_sve_st1_scatter_index, 65572),
SVEMAP1(svst1h_scatter_u64offset_s64, aarch64_sve_st1_scatter, 16842788),
SVEMAP1(svst1h_scatter_u64offset_u64, aarch64_sve_st1_scatter, 16842788),
SVEMAP1(svst1h_u32, aarch64_sve_st1, 4398046527523),
SVEMAP1(svst1h_u64, aarch64_sve_st1, 4398046527524),
SVEMAP1(svst1h_vnum_s32, aarch64_sve_st1, 4398046527523),
SVEMAP1(svst1h_vnum_s64, aarch64_sve_st1, 4398046527524),
SVEMAP1(svst1h_vnum_u32, aarch64_sve_st1, 4398046527523),
SVEMAP1(svst1h_vnum_u64, aarch64_sve_st1, 4398046527524),
SVEMAP1(svst1q_scatter_s64index_bf16, aarch64_sve_st1q_scatter_index, 65549),
SVEMAP1(svst1q_scatter_s64index_f16, aarch64_sve_st1q_scatter_index, 65542),
SVEMAP1(svst1q_scatter_s64index_f32, aarch64_sve_st1q_scatter_index, 65543),
SVEMAP1(svst1q_scatter_s64index_f64, aarch64_sve_st1q_scatter_index, 65544),
SVEMAP1(svst1q_scatter_s64index_s16, aarch64_sve_st1q_scatter_index, 65538),
SVEMAP1(svst1q_scatter_s64index_s32, aarch64_sve_st1q_scatter_index, 65539),
SVEMAP1(svst1q_scatter_s64index_s64, aarch64_sve_st1q_scatter_index, 65540),
SVEMAP1(svst1q_scatter_s64index_u16, aarch64_sve_st1q_scatter_index, 65538),
SVEMAP1(svst1q_scatter_s64index_u32, aarch64_sve_st1q_scatter_index, 65539),
SVEMAP1(svst1q_scatter_s64index_u64, aarch64_sve_st1q_scatter_index, 65540),
SVEMAP1(svst1q_scatter_s64offset_bf16, aarch64_sve_st1q_scatter_vector_offset, 16842765),
SVEMAP1(svst1q_scatter_s64offset_f16, aarch64_sve_st1q_scatter_vector_offset, 16842758),
SVEMAP1(svst1q_scatter_s64offset_f32, aarch64_sve_st1q_scatter_vector_offset, 16842759),
SVEMAP1(svst1q_scatter_s64offset_f64, aarch64_sve_st1q_scatter_vector_offset, 16842760),
SVEMAP1(svst1q_scatter_s64offset_mf8, aarch64_sve_st1q_scatter_vector_offset, 16842766),
SVEMAP1(svst1q_scatter_s64offset_s16, aarch64_sve_st1q_scatter_vector_offset, 16842754),
SVEMAP1(svst1q_scatter_s64offset_s32, aarch64_sve_st1q_scatter_vector_offset, 16842755),
SVEMAP1(svst1q_scatter_s64offset_s64, aarch64_sve_st1q_scatter_vector_offset, 16842756),
SVEMAP1(svst1q_scatter_s64offset_s8, aarch64_sve_st1q_scatter_vector_offset, 16842753),
SVEMAP1(svst1q_scatter_s64offset_u16, aarch64_sve_st1q_scatter_vector_offset, 16842754),
SVEMAP1(svst1q_scatter_s64offset_u32, aarch64_sve_st1q_scatter_vector_offset, 16842755),
SVEMAP1(svst1q_scatter_s64offset_u64, aarch64_sve_st1q_scatter_vector_offset, 16842756),
SVEMAP1(svst1q_scatter_s64offset_u8, aarch64_sve_st1q_scatter_vector_offset, 16842753),
SVEMAP1(svst1q_scatter_u64base_bf16, aarch64_sve_st1q_scatter_scalar_offset, 16842765),
SVEMAP1(svst1q_scatter_u64base_f16, aarch64_sve_st1q_scatter_scalar_offset, 16842758),
SVEMAP1(svst1q_scatter_u64base_f32, aarch64_sve_st1q_scatter_scalar_offset, 16842759),
SVEMAP1(svst1q_scatter_u64base_f64, aarch64_sve_st1q_scatter_scalar_offset, 16842760),
SVEMAP1(svst1q_scatter_u64base_index_bf16, aarch64_sve_st1q_scatter_scalar_offset, 65549),
SVEMAP1(svst1q_scatter_u64base_index_f16, aarch64_sve_st1q_scatter_scalar_offset, 65542),
SVEMAP1(svst1q_scatter_u64base_index_f32, aarch64_sve_st1q_scatter_scalar_offset, 65543),
SVEMAP1(svst1q_scatter_u64base_index_f64, aarch64_sve_st1q_scatter_scalar_offset, 65544),
SVEMAP1(svst1q_scatter_u64base_index_s16, aarch64_sve_st1q_scatter_scalar_offset, 65538),
SVEMAP1(svst1q_scatter_u64base_index_s32, aarch64_sve_st1q_scatter_scalar_offset, 65539),
SVEMAP1(svst1q_scatter_u64base_index_s64, aarch64_sve_st1q_scatter_scalar_offset, 65540),
SVEMAP1(svst1q_scatter_u64base_index_u16, aarch64_sve_st1q_scatter_scalar_offset, 65538),
SVEMAP1(svst1q_scatter_u64base_index_u32, aarch64_sve_st1q_scatter_scalar_offset, 65539),
SVEMAP1(svst1q_scatter_u64base_index_u64, aarch64_sve_st1q_scatter_scalar_offset, 65540),
SVEMAP1(svst1q_scatter_u64base_mf8, aarch64_sve_st1q_scatter_scalar_offset, 16842766),
SVEMAP1(svst1q_scatter_u64base_offset_bf16, aarch64_sve_st1q_scatter_scalar_offset, 16842765),
SVEMAP1(svst1q_scatter_u64base_offset_f16, aarch64_sve_st1q_scatter_scalar_offset, 16842758),
SVEMAP1(svst1q_scatter_u64base_offset_f32, aarch64_sve_st1q_scatter_scalar_offset, 16842759),
SVEMAP1(svst1q_scatter_u64base_offset_f64, aarch64_sve_st1q_scatter_scalar_offset, 16842760),
SVEMAP1(svst1q_scatter_u64base_offset_mf8, aarch64_sve_st1q_scatter_scalar_offset, 16842766),
SVEMAP1(svst1q_scatter_u64base_offset_s16, aarch64_sve_st1q_scatter_scalar_offset, 16842754),
SVEMAP1(svst1q_scatter_u64base_offset_s32, aarch64_sve_st1q_scatter_scalar_offset, 16842755),
SVEMAP1(svst1q_scatter_u64base_offset_s64, aarch64_sve_st1q_scatter_scalar_offset, 16842756),
SVEMAP1(svst1q_scatter_u64base_offset_s8, aarch64_sve_st1q_scatter_scalar_offset, 16842753),
SVEMAP1(svst1q_scatter_u64base_offset_u16, aarch64_sve_st1q_scatter_scalar_offset, 16842754),
SVEMAP1(svst1q_scatter_u64base_offset_u32, aarch64_sve_st1q_scatter_scalar_offset, 16842755),
SVEMAP1(svst1q_scatter_u64base_offset_u64, aarch64_sve_st1q_scatter_scalar_offset, 16842756),
SVEMAP1(svst1q_scatter_u64base_offset_u8, aarch64_sve_st1q_scatter_scalar_offset, 16842753),
SVEMAP1(svst1q_scatter_u64base_s16, aarch64_sve_st1q_scatter_scalar_offset, 16842754),
SVEMAP1(svst1q_scatter_u64base_s32, aarch64_sve_st1q_scatter_scalar_offset, 16842755),
SVEMAP1(svst1q_scatter_u64base_s64, aarch64_sve_st1q_scatter_scalar_offset, 16842756),
SVEMAP1(svst1q_scatter_u64base_s8, aarch64_sve_st1q_scatter_scalar_offset, 16842753),
SVEMAP1(svst1q_scatter_u64base_u16, aarch64_sve_st1q_scatter_scalar_offset, 16842754),
SVEMAP1(svst1q_scatter_u64base_u32, aarch64_sve_st1q_scatter_scalar_offset, 16842755),
SVEMAP1(svst1q_scatter_u64base_u64, aarch64_sve_st1q_scatter_scalar_offset, 16842756),
SVEMAP1(svst1q_scatter_u64base_u8, aarch64_sve_st1q_scatter_scalar_offset, 16842753),
SVEMAP1(svst1q_scatter_u64index_bf16, aarch64_sve_st1q_scatter_index, 65549),
SVEMAP1(svst1q_scatter_u64index_f16, aarch64_sve_st1q_scatter_index, 65542),
SVEMAP1(svst1q_scatter_u64index_f32, aarch64_sve_st1q_scatter_index, 65543),
SVEMAP1(svst1q_scatter_u64index_f64, aarch64_sve_st1q_scatter_index, 65544),
SVEMAP1(svst1q_scatter_u64index_s16, aarch64_sve_st1q_scatter_index, 65538),
SVEMAP1(svst1q_scatter_u64index_s32, aarch64_sve_st1q_scatter_index, 65539),
SVEMAP1(svst1q_scatter_u64index_s64, aarch64_sve_st1q_scatter_index, 65540),
SVEMAP1(svst1q_scatter_u64index_u16, aarch64_sve_st1q_scatter_index, 65538),
SVEMAP1(svst1q_scatter_u64index_u32, aarch64_sve_st1q_scatter_index, 65539),
SVEMAP1(svst1q_scatter_u64index_u64, aarch64_sve_st1q_scatter_index, 65540),
SVEMAP1(svst1q_scatter_u64offset_bf16, aarch64_sve_st1q_scatter_vector_offset, 16842765),
SVEMAP1(svst1q_scatter_u64offset_f16, aarch64_sve_st1q_scatter_vector_offset, 16842758),
SVEMAP1(svst1q_scatter_u64offset_f32, aarch64_sve_st1q_scatter_vector_offset, 16842759),
SVEMAP1(svst1q_scatter_u64offset_f64, aarch64_sve_st1q_scatter_vector_offset, 16842760),
SVEMAP1(svst1q_scatter_u64offset_mf8, aarch64_sve_st1q_scatter_vector_offset, 16842766),
SVEMAP1(svst1q_scatter_u64offset_s16, aarch64_sve_st1q_scatter_vector_offset, 16842754),
SVEMAP1(svst1q_scatter_u64offset_s32, aarch64_sve_st1q_scatter_vector_offset, 16842755),
SVEMAP1(svst1q_scatter_u64offset_s64, aarch64_sve_st1q_scatter_vector_offset, 16842756),
SVEMAP1(svst1q_scatter_u64offset_s8, aarch64_sve_st1q_scatter_vector_offset, 16842753),
SVEMAP1(svst1q_scatter_u64offset_u16, aarch64_sve_st1q_scatter_vector_offset, 16842754),
SVEMAP1(svst1q_scatter_u64offset_u32, aarch64_sve_st1q_scatter_vector_offset, 16842755),
SVEMAP1(svst1q_scatter_u64offset_u64, aarch64_sve_st1q_scatter_vector_offset, 16842756),
SVEMAP1(svst1q_scatter_u64offset_u8, aarch64_sve_st1q_scatter_vector_offset, 16842753),
SVEMAP1(svst1w_s64, aarch64_sve_st1, 4398046527540),
SVEMAP1(svst1w_scatter_s64index_s64, aarch64_sve_st1_scatter_index, 65588),
SVEMAP1(svst1w_scatter_s64index_u64, aarch64_sve_st1_scatter_index, 65588),
SVEMAP1(svst1w_scatter_s64offset_s64, aarch64_sve_st1_scatter, 16842804),
SVEMAP1(svst1w_scatter_s64offset_u64, aarch64_sve_st1_scatter, 16842804),
SVEMAP1(svst1w_scatter_u64base_index_s64, aarch64_sve_st1_scatter_scalar_offset, 65588),
SVEMAP1(svst1w_scatter_u64base_index_u64, aarch64_sve_st1_scatter_scalar_offset, 65588),
SVEMAP1(svst1w_scatter_u64base_offset_s64, aarch64_sve_st1_scatter_scalar_offset, 16842804),
SVEMAP1(svst1w_scatter_u64base_offset_u64, aarch64_sve_st1_scatter_scalar_offset, 16842804),
SVEMAP1(svst1w_scatter_u64base_s64, aarch64_sve_st1_scatter_scalar_offset, 65588),
SVEMAP1(svst1w_scatter_u64base_u64, aarch64_sve_st1_scatter_scalar_offset, 65588),
SVEMAP1(svst1w_scatter_u64index_s64, aarch64_sve_st1_scatter_index, 65588),
SVEMAP1(svst1w_scatter_u64index_u64, aarch64_sve_st1_scatter_index, 65588),
SVEMAP1(svst1w_scatter_u64offset_s64, aarch64_sve_st1_scatter, 16842804),
SVEMAP1(svst1w_scatter_u64offset_u64, aarch64_sve_st1_scatter, 16842804),
SVEMAP1(svst1w_u64, aarch64_sve_st1, 4398046527540),
SVEMAP1(svst1w_vnum_s64, aarch64_sve_st1, 4398046527540),
SVEMAP1(svst1w_vnum_u64, aarch64_sve_st1, 4398046527540),
SVEMAP1(svst1wq_f32, aarch64_sve_st1wq, 16439),
SVEMAP1(svst1wq_s32, aarch64_sve_st1wq, 16435),
SVEMAP1(svst1wq_u32, aarch64_sve_st1wq, 16435),
SVEMAP1(svst1wq_vnum_f32, aarch64_sve_st1wq, 16439),
SVEMAP1(svst1wq_vnum_s32, aarch64_sve_st1wq, 16435),
SVEMAP1(svst1wq_vnum_u32, aarch64_sve_st1wq, 16435),
SVEMAP1(svst2_bf16, aarch64_sve_st2, 4398046773261),
SVEMAP1(svst2_f16, aarch64_sve_st2, 4398046773254),
SVEMAP1(svst2_f32, aarch64_sve_st2, 4398046773255),
SVEMAP1(svst2_f64, aarch64_sve_st2, 4398046773256),
SVEMAP1(svst2_mf8, aarch64_sve_st2, 4398046773262),
SVEMAP1(svst2_s16, aarch64_sve_st2, 4398046773250),
SVEMAP1(svst2_s32, aarch64_sve_st2, 4398046773251),
SVEMAP1(svst2_s64, aarch64_sve_st2, 4398046773252),
SVEMAP1(svst2_s8, aarch64_sve_st2, 4398046773249),
SVEMAP1(svst2_u16, aarch64_sve_st2, 4398046773250),
SVEMAP1(svst2_u32, aarch64_sve_st2, 4398046773251),
SVEMAP1(svst2_u64, aarch64_sve_st2, 4398046773252),
SVEMAP1(svst2_u8, aarch64_sve_st2, 4398046773249),
SVEMAP1(svst2_vnum_bf16, aarch64_sve_st2, 4398046773261),
SVEMAP1(svst2_vnum_f16, aarch64_sve_st2, 4398046773254),
SVEMAP1(svst2_vnum_f32, aarch64_sve_st2, 4398046773255),
SVEMAP1(svst2_vnum_f64, aarch64_sve_st2, 4398046773256),
SVEMAP1(svst2_vnum_mf8, aarch64_sve_st2, 4398046773262),
SVEMAP1(svst2_vnum_s16, aarch64_sve_st2, 4398046773250),
SVEMAP1(svst2_vnum_s32, aarch64_sve_st2, 4398046773251),
SVEMAP1(svst2_vnum_s64, aarch64_sve_st2, 4398046773252),
SVEMAP1(svst2_vnum_s8, aarch64_sve_st2, 4398046773249),
SVEMAP1(svst2_vnum_u16, aarch64_sve_st2, 4398046773250),
SVEMAP1(svst2_vnum_u32, aarch64_sve_st2, 4398046773251),
SVEMAP1(svst2_vnum_u64, aarch64_sve_st2, 4398046773252),
SVEMAP1(svst2_vnum_u8, aarch64_sve_st2, 4398046773249),
SVEMAP1(svst2q_bf16, aarch64_sve_st2q, 4398046773261),
SVEMAP1(svst2q_f16, aarch64_sve_st2q, 4398046773254),
SVEMAP1(svst2q_f32, aarch64_sve_st2q, 4398046773255),
SVEMAP1(svst2q_f64, aarch64_sve_st2q, 4398046773256),
SVEMAP1(svst2q_mf8, aarch64_sve_st2q, 4398046773262),
SVEMAP1(svst2q_s16, aarch64_sve_st2q, 4398046773250),
SVEMAP1(svst2q_s32, aarch64_sve_st2q, 4398046773251),
SVEMAP1(svst2q_s64, aarch64_sve_st2q, 4398046773252),
SVEMAP1(svst2q_s8, aarch64_sve_st2q, 4398046773249),
SVEMAP1(svst2q_u16, aarch64_sve_st2q, 4398046773250),
SVEMAP1(svst2q_u32, aarch64_sve_st2q, 4398046773251),
SVEMAP1(svst2q_u64, aarch64_sve_st2q, 4398046773252),
SVEMAP1(svst2q_u8, aarch64_sve_st2q, 4398046773249),
SVEMAP1(svst2q_vnum_bf16, aarch64_sve_st2q, 4398046773261),
SVEMAP1(svst2q_vnum_f16, aarch64_sve_st2q, 4398046773254),
SVEMAP1(svst2q_vnum_f32, aarch64_sve_st2q, 4398046773255),
SVEMAP1(svst2q_vnum_f64, aarch64_sve_st2q, 4398046773256),
SVEMAP1(svst2q_vnum_mf8, aarch64_sve_st2q, 4398046773262),
SVEMAP1(svst2q_vnum_s16, aarch64_sve_st2q, 4398046773250),
SVEMAP1(svst2q_vnum_s32, aarch64_sve_st2q, 4398046773251),
SVEMAP1(svst2q_vnum_s64, aarch64_sve_st2q, 4398046773252),
SVEMAP1(svst2q_vnum_s8, aarch64_sve_st2q, 4398046773249),
SVEMAP1(svst2q_vnum_u16, aarch64_sve_st2q, 4398046773250),
SVEMAP1(svst2q_vnum_u32, aarch64_sve_st2q, 4398046773251),
SVEMAP1(svst2q_vnum_u64, aarch64_sve_st2q, 4398046773252),
SVEMAP1(svst2q_vnum_u8, aarch64_sve_st2q, 4398046773249),
SVEMAP1(svst3_bf16, aarch64_sve_st3, 4398046773261),
SVEMAP1(svst3_f16, aarch64_sve_st3, 4398046773254),
SVEMAP1(svst3_f32, aarch64_sve_st3, 4398046773255),
SVEMAP1(svst3_f64, aarch64_sve_st3, 4398046773256),
SVEMAP1(svst3_mf8, aarch64_sve_st3, 4398046773262),
SVEMAP1(svst3_s16, aarch64_sve_st3, 4398046773250),
SVEMAP1(svst3_s32, aarch64_sve_st3, 4398046773251),
SVEMAP1(svst3_s64, aarch64_sve_st3, 4398046773252),
SVEMAP1(svst3_s8, aarch64_sve_st3, 4398046773249),
SVEMAP1(svst3_u16, aarch64_sve_st3, 4398046773250),
SVEMAP1(svst3_u32, aarch64_sve_st3, 4398046773251),
SVEMAP1(svst3_u64, aarch64_sve_st3, 4398046773252),
SVEMAP1(svst3_u8, aarch64_sve_st3, 4398046773249),
SVEMAP1(svst3_vnum_bf16, aarch64_sve_st3, 4398046773261),
SVEMAP1(svst3_vnum_f16, aarch64_sve_st3, 4398046773254),
SVEMAP1(svst3_vnum_f32, aarch64_sve_st3, 4398046773255),
SVEMAP1(svst3_vnum_f64, aarch64_sve_st3, 4398046773256),
SVEMAP1(svst3_vnum_mf8, aarch64_sve_st3, 4398046773262),
SVEMAP1(svst3_vnum_s16, aarch64_sve_st3, 4398046773250),
SVEMAP1(svst3_vnum_s32, aarch64_sve_st3, 4398046773251),
SVEMAP1(svst3_vnum_s64, aarch64_sve_st3, 4398046773252),
SVEMAP1(svst3_vnum_s8, aarch64_sve_st3, 4398046773249),
SVEMAP1(svst3_vnum_u16, aarch64_sve_st3, 4398046773250),
SVEMAP1(svst3_vnum_u32, aarch64_sve_st3, 4398046773251),
SVEMAP1(svst3_vnum_u64, aarch64_sve_st3, 4398046773252),
SVEMAP1(svst3_vnum_u8, aarch64_sve_st3, 4398046773249),
SVEMAP1(svst3q_bf16, aarch64_sve_st3q, 4398046773261),
SVEMAP1(svst3q_f16, aarch64_sve_st3q, 4398046773254),
SVEMAP1(svst3q_f32, aarch64_sve_st3q, 4398046773255),
SVEMAP1(svst3q_f64, aarch64_sve_st3q, 4398046773256),
SVEMAP1(svst3q_mf8, aarch64_sve_st3q, 4398046773262),
SVEMAP1(svst3q_s16, aarch64_sve_st3q, 4398046773250),
SVEMAP1(svst3q_s32, aarch64_sve_st3q, 4398046773251),
SVEMAP1(svst3q_s64, aarch64_sve_st3q, 4398046773252),
SVEMAP1(svst3q_s8, aarch64_sve_st3q, 4398046773249),
SVEMAP1(svst3q_u16, aarch64_sve_st3q, 4398046773250),
SVEMAP1(svst3q_u32, aarch64_sve_st3q, 4398046773251),
SVEMAP1(svst3q_u64, aarch64_sve_st3q, 4398046773252),
SVEMAP1(svst3q_u8, aarch64_sve_st3q, 4398046773249),
SVEMAP1(svst3q_vnum_bf16, aarch64_sve_st3q, 4398046773261),
SVEMAP1(svst3q_vnum_f16, aarch64_sve_st3q, 4398046773254),
SVEMAP1(svst3q_vnum_f32, aarch64_sve_st3q, 4398046773255),
SVEMAP1(svst3q_vnum_f64, aarch64_sve_st3q, 4398046773256),
SVEMAP1(svst3q_vnum_mf8, aarch64_sve_st3q, 4398046773262),
SVEMAP1(svst3q_vnum_s16, aarch64_sve_st3q, 4398046773250),
SVEMAP1(svst3q_vnum_s32, aarch64_sve_st3q, 4398046773251),
SVEMAP1(svst3q_vnum_s64, aarch64_sve_st3q, 4398046773252),
SVEMAP1(svst3q_vnum_s8, aarch64_sve_st3q, 4398046773249),
SVEMAP1(svst3q_vnum_u16, aarch64_sve_st3q, 4398046773250),
SVEMAP1(svst3q_vnum_u32, aarch64_sve_st3q, 4398046773251),
SVEMAP1(svst3q_vnum_u64, aarch64_sve_st3q, 4398046773252),
SVEMAP1(svst3q_vnum_u8, aarch64_sve_st3q, 4398046773249),
SVEMAP1(svst4_bf16, aarch64_sve_st4, 4398046773261),
SVEMAP1(svst4_f16, aarch64_sve_st4, 4398046773254),
SVEMAP1(svst4_f32, aarch64_sve_st4, 4398046773255),
SVEMAP1(svst4_f64, aarch64_sve_st4, 4398046773256),
SVEMAP1(svst4_mf8, aarch64_sve_st4, 4398046773262),
SVEMAP1(svst4_s16, aarch64_sve_st4, 4398046773250),
SVEMAP1(svst4_s32, aarch64_sve_st4, 4398046773251),
SVEMAP1(svst4_s64, aarch64_sve_st4, 4398046773252),
SVEMAP1(svst4_s8, aarch64_sve_st4, 4398046773249),
SVEMAP1(svst4_u16, aarch64_sve_st4, 4398046773250),
SVEMAP1(svst4_u32, aarch64_sve_st4, 4398046773251),
SVEMAP1(svst4_u64, aarch64_sve_st4, 4398046773252),
SVEMAP1(svst4_u8, aarch64_sve_st4, 4398046773249),
SVEMAP1(svst4_vnum_bf16, aarch64_sve_st4, 4398046773261),
SVEMAP1(svst4_vnum_f16, aarch64_sve_st4, 4398046773254),
SVEMAP1(svst4_vnum_f32, aarch64_sve_st4, 4398046773255),
SVEMAP1(svst4_vnum_f64, aarch64_sve_st4, 4398046773256),
SVEMAP1(svst4_vnum_mf8, aarch64_sve_st4, 4398046773262),
SVEMAP1(svst4_vnum_s16, aarch64_sve_st4, 4398046773250),
SVEMAP1(svst4_vnum_s32, aarch64_sve_st4, 4398046773251),
SVEMAP1(svst4_vnum_s64, aarch64_sve_st4, 4398046773252),
SVEMAP1(svst4_vnum_s8, aarch64_sve_st4, 4398046773249),
SVEMAP1(svst4_vnum_u16, aarch64_sve_st4, 4398046773250),
SVEMAP1(svst4_vnum_u32, aarch64_sve_st4, 4398046773251),
SVEMAP1(svst4_vnum_u64, aarch64_sve_st4, 4398046773252),
SVEMAP1(svst4_vnum_u8, aarch64_sve_st4, 4398046773249),
SVEMAP1(svst4q_bf16, aarch64_sve_st4q, 4398046773261),
SVEMAP1(svst4q_f16, aarch64_sve_st4q, 4398046773254),
SVEMAP1(svst4q_f32, aarch64_sve_st4q, 4398046773255),
SVEMAP1(svst4q_f64, aarch64_sve_st4q, 4398046773256),
SVEMAP1(svst4q_mf8, aarch64_sve_st4q, 4398046773262),
SVEMAP1(svst4q_s16, aarch64_sve_st4q, 4398046773250),
SVEMAP1(svst4q_s32, aarch64_sve_st4q, 4398046773251),
SVEMAP1(svst4q_s64, aarch64_sve_st4q, 4398046773252),
SVEMAP1(svst4q_s8, aarch64_sve_st4q, 4398046773249),
SVEMAP1(svst4q_u16, aarch64_sve_st4q, 4398046773250),
SVEMAP1(svst4q_u32, aarch64_sve_st4q, 4398046773251),
SVEMAP1(svst4q_u64, aarch64_sve_st4q, 4398046773252),
SVEMAP1(svst4q_u8, aarch64_sve_st4q, 4398046773249),
SVEMAP1(svst4q_vnum_bf16, aarch64_sve_st4q, 4398046773261),
SVEMAP1(svst4q_vnum_f16, aarch64_sve_st4q, 4398046773254),
SVEMAP1(svst4q_vnum_f32, aarch64_sve_st4q, 4398046773255),
SVEMAP1(svst4q_vnum_f64, aarch64_sve_st4q, 4398046773256),
SVEMAP1(svst4q_vnum_mf8, aarch64_sve_st4q, 4398046773262),
SVEMAP1(svst4q_vnum_s16, aarch64_sve_st4q, 4398046773250),
SVEMAP1(svst4q_vnum_s32, aarch64_sve_st4q, 4398046773251),
SVEMAP1(svst4q_vnum_s64, aarch64_sve_st4q, 4398046773252),
SVEMAP1(svst4q_vnum_s8, aarch64_sve_st4q, 4398046773249),
SVEMAP1(svst4q_vnum_u16, aarch64_sve_st4q, 4398046773250),
SVEMAP1(svst4q_vnum_u32, aarch64_sve_st4q, 4398046773251),
SVEMAP1(svst4q_vnum_u64, aarch64_sve_st4q, 4398046773252),
SVEMAP1(svst4q_vnum_u8, aarch64_sve_st4q, 4398046773249),
SVEMAP1(svstnt1_bf16, aarch64_sve_stnt1, 4398046527501),
SVEMAP1(svstnt1_bf16_x2, aarch64_sve_stnt1_pn_x2, 4398046773261),
SVEMAP1(svstnt1_bf16_x4, aarch64_sve_stnt1_pn_x4, 4398046773261),
SVEMAP1(svstnt1_f16, aarch64_sve_stnt1, 4398046527494),
SVEMAP1(svstnt1_f16_x2, aarch64_sve_stnt1_pn_x2, 4398046773254),
SVEMAP1(svstnt1_f16_x4, aarch64_sve_stnt1_pn_x4, 4398046773254),
SVEMAP1(svstnt1_f32, aarch64_sve_stnt1, 4398046527495),
SVEMAP1(svstnt1_f32_x2, aarch64_sve_stnt1_pn_x2, 4398046773255),
SVEMAP1(svstnt1_f32_x4, aarch64_sve_stnt1_pn_x4, 4398046773255),
SVEMAP1(svstnt1_f64, aarch64_sve_stnt1, 4398046527496),
SVEMAP1(svstnt1_f64_x2, aarch64_sve_stnt1_pn_x2, 4398046773256),
SVEMAP1(svstnt1_f64_x4, aarch64_sve_stnt1_pn_x4, 4398046773256),
SVEMAP1(svstnt1_mf8, aarch64_sve_stnt1, 4398046527502),
SVEMAP1(svstnt1_mf8_x2, aarch64_sve_stnt1_pn_x2, 4398046773262),
SVEMAP1(svstnt1_mf8_x4, aarch64_sve_stnt1_pn_x4, 4398046773262),
SVEMAP1(svstnt1_s16, aarch64_sve_stnt1, 4398046527490),
SVEMAP1(svstnt1_s16_x2, aarch64_sve_stnt1_pn_x2, 4398046773250),
SVEMAP1(svstnt1_s16_x4, aarch64_sve_stnt1_pn_x4, 4398046773250),
SVEMAP1(svstnt1_s32, aarch64_sve_stnt1, 4398046527491),
SVEMAP1(svstnt1_s32_x2, aarch64_sve_stnt1_pn_x2, 4398046773251),
SVEMAP1(svstnt1_s32_x4, aarch64_sve_stnt1_pn_x4, 4398046773251),
SVEMAP1(svstnt1_s64, aarch64_sve_stnt1, 4398046527492),
SVEMAP1(svstnt1_s64_x2, aarch64_sve_stnt1_pn_x2, 4398046773252),
SVEMAP1(svstnt1_s64_x4, aarch64_sve_stnt1_pn_x4, 4398046773252),
SVEMAP1(svstnt1_s8, aarch64_sve_stnt1, 4398046527489),
SVEMAP1(svstnt1_s8_x2, aarch64_sve_stnt1_pn_x2, 4398046773249),
SVEMAP1(svstnt1_s8_x4, aarch64_sve_stnt1_pn_x4, 4398046773249),
SVEMAP1(svstnt1_scatter_s64index_f64, aarch64_sve_stnt1_scatter_index, 65544),
SVEMAP1(svstnt1_scatter_s64index_s64, aarch64_sve_stnt1_scatter_index, 65540),
SVEMAP1(svstnt1_scatter_s64index_u64, aarch64_sve_stnt1_scatter_index, 65540),
SVEMAP1(svstnt1_scatter_s64offset_f64, aarch64_sve_stnt1_scatter, 16842760),
SVEMAP1(svstnt1_scatter_s64offset_s64, aarch64_sve_stnt1_scatter, 16842756),
SVEMAP1(svstnt1_scatter_s64offset_u64, aarch64_sve_stnt1_scatter, 16842756),
SVEMAP1(svstnt1_scatter_u32base_f32, aarch64_sve_stnt1_scatter_scalar_offset, 65543),
SVEMAP1(svstnt1_scatter_u32base_index_f32, aarch64_sve_stnt1_scatter_scalar_offset, 65543),
SVEMAP1(svstnt1_scatter_u32base_index_s32, aarch64_sve_stnt1_scatter_scalar_offset, 65539),
SVEMAP1(svstnt1_scatter_u32base_index_u32, aarch64_sve_stnt1_scatter_scalar_offset, 65539),
SVEMAP1(svstnt1_scatter_u32base_offset_f32, aarch64_sve_stnt1_scatter_scalar_offset, 16842759),
SVEMAP1(svstnt1_scatter_u32base_offset_s32, aarch64_sve_stnt1_scatter_scalar_offset, 16842755),
SVEMAP1(svstnt1_scatter_u32base_offset_u32, aarch64_sve_stnt1_scatter_scalar_offset, 16842755),
SVEMAP1(svstnt1_scatter_u32base_s32, aarch64_sve_stnt1_scatter_scalar_offset, 65539),
SVEMAP1(svstnt1_scatter_u32base_u32, aarch64_sve_stnt1_scatter_scalar_offset, 65539),
SVEMAP1(svstnt1_scatter_u32offset_f32, aarch64_sve_stnt1_scatter_uxtw, 16842759),
SVEMAP1(svstnt1_scatter_u32offset_s32, aarch64_sve_stnt1_scatter_uxtw, 16842755),
SVEMAP1(svstnt1_scatter_u32offset_u32, aarch64_sve_stnt1_scatter_uxtw, 16842755),
SVEMAP1(svstnt1_scatter_u64base_f64, aarch64_sve_stnt1_scatter_scalar_offset, 65544),
SVEMAP1(svstnt1_scatter_u64base_index_f64, aarch64_sve_stnt1_scatter_scalar_offset, 65544),
SVEMAP1(svstnt1_scatter_u64base_index_s64, aarch64_sve_stnt1_scatter_scalar_offset, 65540),
SVEMAP1(svstnt1_scatter_u64base_index_u64, aarch64_sve_stnt1_scatter_scalar_offset, 65540),
SVEMAP1(svstnt1_scatter_u64base_offset_f64, aarch64_sve_stnt1_scatter_scalar_offset, 16842760),
SVEMAP1(svstnt1_scatter_u64base_offset_s64, aarch64_sve_stnt1_scatter_scalar_offset, 16842756),
SVEMAP1(svstnt1_scatter_u64base_offset_u64, aarch64_sve_stnt1_scatter_scalar_offset, 16842756),
SVEMAP1(svstnt1_scatter_u64base_s64, aarch64_sve_stnt1_scatter_scalar_offset, 65540),
SVEMAP1(svstnt1_scatter_u64base_u64, aarch64_sve_stnt1_scatter_scalar_offset, 65540),
SVEMAP1(svstnt1_scatter_u64index_f64, aarch64_sve_stnt1_scatter_index, 65544),
SVEMAP1(svstnt1_scatter_u64index_s64, aarch64_sve_stnt1_scatter_index, 65540),
SVEMAP1(svstnt1_scatter_u64index_u64, aarch64_sve_stnt1_scatter_index, 65540),
SVEMAP1(svstnt1_scatter_u64offset_f64, aarch64_sve_stnt1_scatter, 16842760),
SVEMAP1(svstnt1_scatter_u64offset_s64, aarch64_sve_stnt1_scatter, 16842756),
SVEMAP1(svstnt1_scatter_u64offset_u64, aarch64_sve_stnt1_scatter, 16842756),
SVEMAP1(svstnt1_u16, aarch64_sve_stnt1, 4398046527490),
SVEMAP1(svstnt1_u16_x2, aarch64_sve_stnt1_pn_x2, 4398046773250),
SVEMAP1(svstnt1_u16_x4, aarch64_sve_stnt1_pn_x4, 4398046773250),
SVEMAP1(svstnt1_u32, aarch64_sve_stnt1, 4398046527491),
SVEMAP1(svstnt1_u32_x2, aarch64_sve_stnt1_pn_x2, 4398046773251),
SVEMAP1(svstnt1_u32_x4, aarch64_sve_stnt1_pn_x4, 4398046773251),
SVEMAP1(svstnt1_u64, aarch64_sve_stnt1, 4398046527492),
SVEMAP1(svstnt1_u64_x2, aarch64_sve_stnt1_pn_x2, 4398046773252),
SVEMAP1(svstnt1_u64_x4, aarch64_sve_stnt1_pn_x4, 4398046773252),
SVEMAP1(svstnt1_u8, aarch64_sve_stnt1, 4398046527489),
SVEMAP1(svstnt1_u8_x2, aarch64_sve_stnt1_pn_x2, 4398046773249),
SVEMAP1(svstnt1_u8_x4, aarch64_sve_stnt1_pn_x4, 4398046773249),
SVEMAP1(svstnt1_vnum_bf16, aarch64_sve_stnt1, 4398046527501),
SVEMAP1(svstnt1_vnum_bf16_x2, aarch64_sve_stnt1_pn_x2, 4398046773261),
SVEMAP1(svstnt1_vnum_bf16_x4, aarch64_sve_stnt1_pn_x4, 4398046773261),
SVEMAP1(svstnt1_vnum_f16, aarch64_sve_stnt1, 4398046527494),
SVEMAP1(svstnt1_vnum_f16_x2, aarch64_sve_stnt1_pn_x2, 4398046773254),
SVEMAP1(svstnt1_vnum_f16_x4, aarch64_sve_stnt1_pn_x4, 4398046773254),
SVEMAP1(svstnt1_vnum_f32, aarch64_sve_stnt1, 4398046527495),
SVEMAP1(svstnt1_vnum_f32_x2, aarch64_sve_stnt1_pn_x2, 4398046773255),
SVEMAP1(svstnt1_vnum_f32_x4, aarch64_sve_stnt1_pn_x4, 4398046773255),
SVEMAP1(svstnt1_vnum_f64, aarch64_sve_stnt1, 4398046527496),
SVEMAP1(svstnt1_vnum_f64_x2, aarch64_sve_stnt1_pn_x2, 4398046773256),
SVEMAP1(svstnt1_vnum_f64_x4, aarch64_sve_stnt1_pn_x4, 4398046773256),
SVEMAP1(svstnt1_vnum_mf8, aarch64_sve_stnt1, 4398046527502),
SVEMAP1(svstnt1_vnum_mf8_x2, aarch64_sve_stnt1_pn_x2, 4398046773262),
SVEMAP1(svstnt1_vnum_mf8_x4, aarch64_sve_stnt1_pn_x4, 4398046773262),
SVEMAP1(svstnt1_vnum_s16, aarch64_sve_stnt1, 4398046527490),
SVEMAP1(svstnt1_vnum_s16_x2, aarch64_sve_stnt1_pn_x2, 4398046773250),
SVEMAP1(svstnt1_vnum_s16_x4, aarch64_sve_stnt1_pn_x4, 4398046773250),
SVEMAP1(svstnt1_vnum_s32, aarch64_sve_stnt1, 4398046527491),
SVEMAP1(svstnt1_vnum_s32_x2, aarch64_sve_stnt1_pn_x2, 4398046773251),
SVEMAP1(svstnt1_vnum_s32_x4, aarch64_sve_stnt1_pn_x4, 4398046773251),
SVEMAP1(svstnt1_vnum_s64, aarch64_sve_stnt1, 4398046527492),
SVEMAP1(svstnt1_vnum_s64_x2, aarch64_sve_stnt1_pn_x2, 4398046773252),
SVEMAP1(svstnt1_vnum_s64_x4, aarch64_sve_stnt1_pn_x4, 4398046773252),
SVEMAP1(svstnt1_vnum_s8, aarch64_sve_stnt1, 4398046527489),
SVEMAP1(svstnt1_vnum_s8_x2, aarch64_sve_stnt1_pn_x2, 4398046773249),
SVEMAP1(svstnt1_vnum_s8_x4, aarch64_sve_stnt1_pn_x4, 4398046773249),
SVEMAP1(svstnt1_vnum_u16, aarch64_sve_stnt1, 4398046527490),
SVEMAP1(svstnt1_vnum_u16_x2, aarch64_sve_stnt1_pn_x2, 4398046773250),
SVEMAP1(svstnt1_vnum_u16_x4, aarch64_sve_stnt1_pn_x4, 4398046773250),
SVEMAP1(svstnt1_vnum_u32, aarch64_sve_stnt1, 4398046527491),
SVEMAP1(svstnt1_vnum_u32_x2, aarch64_sve_stnt1_pn_x2, 4398046773251),
SVEMAP1(svstnt1_vnum_u32_x4, aarch64_sve_stnt1_pn_x4, 4398046773251),
SVEMAP1(svstnt1_vnum_u64, aarch64_sve_stnt1, 4398046527492),
SVEMAP1(svstnt1_vnum_u64_x2, aarch64_sve_stnt1_pn_x2, 4398046773252),
SVEMAP1(svstnt1_vnum_u64_x4, aarch64_sve_stnt1_pn_x4, 4398046773252),
SVEMAP1(svstnt1_vnum_u8, aarch64_sve_stnt1, 4398046527489),
SVEMAP1(svstnt1_vnum_u8_x2, aarch64_sve_stnt1_pn_x2, 4398046773249),
SVEMAP1(svstnt1_vnum_u8_x4, aarch64_sve_stnt1_pn_x4, 4398046773249),
SVEMAP1(svstnt1b_scatter_s64offset_s64, aarch64_sve_stnt1_scatter, 16842772),
SVEMAP1(svstnt1b_scatter_s64offset_u64, aarch64_sve_stnt1_scatter, 16842772),
SVEMAP1(svstnt1b_scatter_u32base_offset_s32, aarch64_sve_stnt1_scatter_scalar_offset, 16842771),
SVEMAP1(svstnt1b_scatter_u32base_offset_u32, aarch64_sve_stnt1_scatter_scalar_offset, 16842771),
SVEMAP1(svstnt1b_scatter_u32base_s32, aarch64_sve_stnt1_scatter_scalar_offset, 65555),
SVEMAP1(svstnt1b_scatter_u32base_u32, aarch64_sve_stnt1_scatter_scalar_offset, 65555),
SVEMAP1(svstnt1b_scatter_u32offset_s32, aarch64_sve_stnt1_scatter_uxtw, 16842771),
SVEMAP1(svstnt1b_scatter_u32offset_u32, aarch64_sve_stnt1_scatter_uxtw, 16842771),
SVEMAP1(svstnt1b_scatter_u64base_offset_s64, aarch64_sve_stnt1_scatter_scalar_offset, 16842772),
SVEMAP1(svstnt1b_scatter_u64base_offset_u64, aarch64_sve_stnt1_scatter_scalar_offset, 16842772),
SVEMAP1(svstnt1b_scatter_u64base_s64, aarch64_sve_stnt1_scatter_scalar_offset, 65556),
SVEMAP1(svstnt1b_scatter_u64base_u64, aarch64_sve_stnt1_scatter_scalar_offset, 65556),
SVEMAP1(svstnt1b_scatter_u64offset_s64, aarch64_sve_stnt1_scatter, 16842772),
SVEMAP1(svstnt1b_scatter_u64offset_u64, aarch64_sve_stnt1_scatter, 16842772),
SVEMAP1(svstnt1h_scatter_s64index_s64, aarch64_sve_stnt1_scatter_index, 65572),
SVEMAP1(svstnt1h_scatter_s64index_u64, aarch64_sve_stnt1_scatter_index, 65572),
SVEMAP1(svstnt1h_scatter_s64offset_s64, aarch64_sve_stnt1_scatter, 16842788),
SVEMAP1(svstnt1h_scatter_s64offset_u64, aarch64_sve_stnt1_scatter, 16842788),
SVEMAP1(svstnt1h_scatter_u32base_index_s32, aarch64_sve_stnt1_scatter_scalar_offset, 65571),
SVEMAP1(svstnt1h_scatter_u32base_index_u32, aarch64_sve_stnt1_scatter_scalar_offset, 65571),
SVEMAP1(svstnt1h_scatter_u32base_offset_s32, aarch64_sve_stnt1_scatter_scalar_offset, 16842787),
SVEMAP1(svstnt1h_scatter_u32base_offset_u32, aarch64_sve_stnt1_scatter_scalar_offset, 16842787),
SVEMAP1(svstnt1h_scatter_u32base_s32, aarch64_sve_stnt1_scatter_scalar_offset, 65571),
SVEMAP1(svstnt1h_scatter_u32base_u32, aarch64_sve_stnt1_scatter_scalar_offset, 65571),
SVEMAP1(svstnt1h_scatter_u32offset_s32, aarch64_sve_stnt1_scatter_uxtw, 16842787),
SVEMAP1(svstnt1h_scatter_u32offset_u32, aarch64_sve_stnt1_scatter_uxtw, 16842787),
SVEMAP1(svstnt1h_scatter_u64base_index_s64, aarch64_sve_stnt1_scatter_scalar_offset, 65572),
SVEMAP1(svstnt1h_scatter_u64base_index_u64, aarch64_sve_stnt1_scatter_scalar_offset, 65572),
SVEMAP1(svstnt1h_scatter_u64base_offset_s64, aarch64_sve_stnt1_scatter_scalar_offset, 16842788),
SVEMAP1(svstnt1h_scatter_u64base_offset_u64, aarch64_sve_stnt1_scatter_scalar_offset, 16842788),
SVEMAP1(svstnt1h_scatter_u64base_s64, aarch64_sve_stnt1_scatter_scalar_offset, 65572),
SVEMAP1(svstnt1h_scatter_u64base_u64, aarch64_sve_stnt1_scatter_scalar_offset, 65572),
SVEMAP1(svstnt1h_scatter_u64index_s64, aarch64_sve_stnt1_scatter_index, 65572),
SVEMAP1(svstnt1h_scatter_u64index_u64, aarch64_sve_stnt1_scatter_index, 65572),
SVEMAP1(svstnt1h_scatter_u64offset_s64, aarch64_sve_stnt1_scatter, 16842788),
SVEMAP1(svstnt1h_scatter_u64offset_u64, aarch64_sve_stnt1_scatter, 16842788),
SVEMAP1(svstnt1w_scatter_s64index_s64, aarch64_sve_stnt1_scatter_index, 65588),
SVEMAP1(svstnt1w_scatter_s64index_u64, aarch64_sve_stnt1_scatter_index, 65588),
SVEMAP1(svstnt1w_scatter_s64offset_s64, aarch64_sve_stnt1_scatter, 16842804),
SVEMAP1(svstnt1w_scatter_s64offset_u64, aarch64_sve_stnt1_scatter, 16842804),
SVEMAP1(svstnt1w_scatter_u64base_index_s64, aarch64_sve_stnt1_scatter_scalar_offset, 65588),
SVEMAP1(svstnt1w_scatter_u64base_index_u64, aarch64_sve_stnt1_scatter_scalar_offset, 65588),
SVEMAP1(svstnt1w_scatter_u64base_offset_s64, aarch64_sve_stnt1_scatter_scalar_offset, 16842804),
SVEMAP1(svstnt1w_scatter_u64base_offset_u64, aarch64_sve_stnt1_scatter_scalar_offset, 16842804),
SVEMAP1(svstnt1w_scatter_u64base_s64, aarch64_sve_stnt1_scatter_scalar_offset, 65588),
SVEMAP1(svstnt1w_scatter_u64base_u64, aarch64_sve_stnt1_scatter_scalar_offset, 65588),
SVEMAP1(svstnt1w_scatter_u64index_s64, aarch64_sve_stnt1_scatter_index, 65588),
SVEMAP1(svstnt1w_scatter_u64index_u64, aarch64_sve_stnt1_scatter_index, 65588),
SVEMAP1(svstnt1w_scatter_u64offset_s64, aarch64_sve_stnt1_scatter, 16842804),
SVEMAP1(svstnt1w_scatter_u64offset_u64, aarch64_sve_stnt1_scatter, 16842804),
SVEMAP1(svsub_bf16_m, aarch64_sve_fsub, 4398046511373),
SVEMAP1(svsub_bf16_x, aarch64_sve_fsub_u, 4398046511245),
SVEMAP1(svsub_bf16_z, aarch64_sve_fsub, 4398046511501),
SVEMAP1(svsub_f16_m, aarch64_sve_fsub, 4398046511366),
SVEMAP1(svsub_f16_x, aarch64_sve_fsub_u, 4398046511238),
SVEMAP1(svsub_f16_z, aarch64_sve_fsub, 4398046511494),
SVEMAP1(svsub_f32_m, aarch64_sve_fsub, 4398046511367),
SVEMAP1(svsub_f32_x, aarch64_sve_fsub_u, 4398046511239),
SVEMAP1(svsub_f32_z, aarch64_sve_fsub, 4398046511495),
SVEMAP1(svsub_f64_m, aarch64_sve_fsub, 4398046511368),
SVEMAP1(svsub_f64_x, aarch64_sve_fsub_u, 4398046511240),
SVEMAP1(svsub_f64_z, aarch64_sve_fsub, 4398046511496),
SVEMAP1(svsub_n_bf16_m, aarch64_sve_fsub, 4398046514445),
SVEMAP1(svsub_n_bf16_x, aarch64_sve_fsub_u, 4398046514317),
SVEMAP1(svsub_n_bf16_z, aarch64_sve_fsub, 4398046514573),
SVEMAP1(svsub_n_f16_m, aarch64_sve_fsub, 4398046514438),
SVEMAP1(svsub_n_f16_x, aarch64_sve_fsub_u, 4398046514310),
SVEMAP1(svsub_n_f16_z, aarch64_sve_fsub, 4398046514566),
SVEMAP1(svsub_n_f32_m, aarch64_sve_fsub, 4398046514439),
SVEMAP1(svsub_n_f32_x, aarch64_sve_fsub_u, 4398046514311),
SVEMAP1(svsub_n_f32_z, aarch64_sve_fsub, 4398046514567),
SVEMAP1(svsub_n_f64_m, aarch64_sve_fsub, 4398046514440),
SVEMAP1(svsub_n_f64_x, aarch64_sve_fsub_u, 4398046514312),
SVEMAP1(svsub_n_f64_z, aarch64_sve_fsub, 4398046514568),
SVEMAP1(svsub_n_s16_m, aarch64_sve_sub, 4398046514434),
SVEMAP1(svsub_n_s16_x, aarch64_sve_sub_u, 4398046514306),
SVEMAP1(svsub_n_s16_z, aarch64_sve_sub, 4398046514562),
SVEMAP1(svsub_n_s32_m, aarch64_sve_sub, 4398046514435),
SVEMAP1(svsub_n_s32_x, aarch64_sve_sub_u, 4398046514307),
SVEMAP1(svsub_n_s32_z, aarch64_sve_sub, 4398046514563),
SVEMAP1(svsub_n_s64_m, aarch64_sve_sub, 4398046514436),
SVEMAP1(svsub_n_s64_x, aarch64_sve_sub_u, 4398046514308),
SVEMAP1(svsub_n_s64_z, aarch64_sve_sub, 4398046514564),
SVEMAP1(svsub_n_s8_m, aarch64_sve_sub, 4398046514433),
SVEMAP1(svsub_n_s8_x, aarch64_sve_sub_u, 4398046514305),
SVEMAP1(svsub_n_s8_z, aarch64_sve_sub, 4398046514561),
SVEMAP1(svsub_n_u16_m, aarch64_sve_sub, 4398046514434),
SVEMAP1(svsub_n_u16_x, aarch64_sve_sub_u, 4398046514306),
SVEMAP1(svsub_n_u16_z, aarch64_sve_sub, 4398046514562),
SVEMAP1(svsub_n_u32_m, aarch64_sve_sub, 4398046514435),
SVEMAP1(svsub_n_u32_x, aarch64_sve_sub_u, 4398046514307),
SVEMAP1(svsub_n_u32_z, aarch64_sve_sub, 4398046514563),
SVEMAP1(svsub_n_u64_m, aarch64_sve_sub, 4398046514436),
SVEMAP1(svsub_n_u64_x, aarch64_sve_sub_u, 4398046514308),
SVEMAP1(svsub_n_u64_z, aarch64_sve_sub, 4398046514564),
SVEMAP1(svsub_n_u8_m, aarch64_sve_sub, 4398046514433),
SVEMAP1(svsub_n_u8_x, aarch64_sve_sub_u, 4398046514305),
SVEMAP1(svsub_n_u8_z, aarch64_sve_sub, 4398046514561),
SVEMAP1(svsub_s16_m, aarch64_sve_sub, 4398046511362),
SVEMAP1(svsub_s16_x, aarch64_sve_sub_u, 4398046511234),
SVEMAP1(svsub_s16_z, aarch64_sve_sub, 4398046511490),
SVEMAP1(svsub_s32_m, aarch64_sve_sub, 4398046511363),
SVEMAP1(svsub_s32_x, aarch64_sve_sub_u, 4398046511235),
SVEMAP1(svsub_s32_z, aarch64_sve_sub, 4398046511491),
SVEMAP1(svsub_s64_m, aarch64_sve_sub, 4398046511364),
SVEMAP1(svsub_s64_x, aarch64_sve_sub_u, 4398046511236),
SVEMAP1(svsub_s64_z, aarch64_sve_sub, 4398046511492),
SVEMAP1(svsub_s8_m, aarch64_sve_sub, 4398046511361),
SVEMAP1(svsub_s8_x, aarch64_sve_sub_u, 4398046511233),
SVEMAP1(svsub_s8_z, aarch64_sve_sub, 4398046511489),
SVEMAP1(svsub_u16_m, aarch64_sve_sub, 4398046511362),
SVEMAP1(svsub_u16_x, aarch64_sve_sub_u, 4398046511234),
SVEMAP1(svsub_u16_z, aarch64_sve_sub, 4398046511490),
SVEMAP1(svsub_u32_m, aarch64_sve_sub, 4398046511363),
SVEMAP1(svsub_u32_x, aarch64_sve_sub_u, 4398046511235),
SVEMAP1(svsub_u32_z, aarch64_sve_sub, 4398046511491),
SVEMAP1(svsub_u64_m, aarch64_sve_sub, 4398046511364),
SVEMAP1(svsub_u64_x, aarch64_sve_sub_u, 4398046511236),
SVEMAP1(svsub_u64_z, aarch64_sve_sub, 4398046511492),
SVEMAP1(svsub_u8_m, aarch64_sve_sub, 4398046511361),
SVEMAP1(svsub_u8_x, aarch64_sve_sub_u, 4398046511233),
SVEMAP1(svsub_u8_z, aarch64_sve_sub, 4398046511489),
SVEMAP1(svsubhnb_n_s16, aarch64_sve_subhnb, 4398046513154),
SVEMAP1(svsubhnb_n_s32, aarch64_sve_subhnb, 4398046513155),
SVEMAP1(svsubhnb_n_s64, aarch64_sve_subhnb, 4398046513156),
SVEMAP1(svsubhnb_n_u16, aarch64_sve_subhnb, 4398046513154),
SVEMAP1(svsubhnb_n_u32, aarch64_sve_subhnb, 4398046513155),
SVEMAP1(svsubhnb_n_u64, aarch64_sve_subhnb, 4398046513156),
SVEMAP1(svsubhnb_s16, aarch64_sve_subhnb, 4398046511106),
SVEMAP1(svsubhnb_s32, aarch64_sve_subhnb, 4398046511107),
SVEMAP1(svsubhnb_s64, aarch64_sve_subhnb, 4398046511108),
SVEMAP1(svsubhnb_u16, aarch64_sve_subhnb, 4398046511106),
SVEMAP1(svsubhnb_u32, aarch64_sve_subhnb, 4398046511107),
SVEMAP1(svsubhnb_u64, aarch64_sve_subhnb, 4398046511108),
SVEMAP1(svsubhnt_n_s16, aarch64_sve_subhnt, 4398046514178),
SVEMAP1(svsubhnt_n_s32, aarch64_sve_subhnt, 4398046514179),
SVEMAP1(svsubhnt_n_s64, aarch64_sve_subhnt, 4398046514180),
SVEMAP1(svsubhnt_n_u16, aarch64_sve_subhnt, 4398046514178),
SVEMAP1(svsubhnt_n_u32, aarch64_sve_subhnt, 4398046514179),
SVEMAP1(svsubhnt_n_u64, aarch64_sve_subhnt, 4398046514180),
SVEMAP1(svsubhnt_s16, aarch64_sve_subhnt, 4398046511106),
SVEMAP1(svsubhnt_s32, aarch64_sve_subhnt, 4398046511107),
SVEMAP1(svsubhnt_s64, aarch64_sve_subhnt, 4398046511108),
SVEMAP1(svsubhnt_u16, aarch64_sve_subhnt, 4398046511106),
SVEMAP1(svsubhnt_u32, aarch64_sve_subhnt, 4398046511107),
SVEMAP1(svsubhnt_u64, aarch64_sve_subhnt, 4398046511108),
SVEMAP1(svsublb_n_s16, aarch64_sve_ssublb, 4398046513154),
SVEMAP1(svsublb_n_s32, aarch64_sve_ssublb, 4398046513155),
SVEMAP1(svsublb_n_s64, aarch64_sve_ssublb, 4398046513156),
SVEMAP1(svsublb_n_u16, aarch64_sve_usublb, 4398046513154),
SVEMAP1(svsublb_n_u32, aarch64_sve_usublb, 4398046513155),
SVEMAP1(svsublb_n_u64, aarch64_sve_usublb, 4398046513156),
SVEMAP1(svsublb_s16, aarch64_sve_ssublb, 4398046511106),
SVEMAP1(svsublb_s32, aarch64_sve_ssublb, 4398046511107),
SVEMAP1(svsublb_s64, aarch64_sve_ssublb, 4398046511108),
SVEMAP1(svsublb_u16, aarch64_sve_usublb, 4398046511106),
SVEMAP1(svsublb_u32, aarch64_sve_usublb, 4398046511107),
SVEMAP1(svsublb_u64, aarch64_sve_usublb, 4398046511108),
SVEMAP1(svsublbt_n_s16, aarch64_sve_ssublbt, 4398046513154),
SVEMAP1(svsublbt_n_s32, aarch64_sve_ssublbt, 4398046513155),
SVEMAP1(svsublbt_n_s64, aarch64_sve_ssublbt, 4398046513156),
SVEMAP1(svsublbt_s16, aarch64_sve_ssublbt, 4398046511106),
SVEMAP1(svsublbt_s32, aarch64_sve_ssublbt, 4398046511107),
SVEMAP1(svsublbt_s64, aarch64_sve_ssublbt, 4398046511108),
SVEMAP1(svsublt_n_s16, aarch64_sve_ssublt, 4398046513154),
SVEMAP1(svsublt_n_s32, aarch64_sve_ssublt, 4398046513155),
SVEMAP1(svsublt_n_s64, aarch64_sve_ssublt, 4398046513156),
SVEMAP1(svsublt_n_u16, aarch64_sve_usublt, 4398046513154),
SVEMAP1(svsublt_n_u32, aarch64_sve_usublt, 4398046513155),
SVEMAP1(svsublt_n_u64, aarch64_sve_usublt, 4398046513156),
SVEMAP1(svsublt_s16, aarch64_sve_ssublt, 4398046511106),
SVEMAP1(svsublt_s32, aarch64_sve_ssublt, 4398046511107),
SVEMAP1(svsublt_s64, aarch64_sve_ssublt, 4398046511108),
SVEMAP1(svsublt_u16, aarch64_sve_usublt, 4398046511106),
SVEMAP1(svsublt_u32, aarch64_sve_usublt, 4398046511107),
SVEMAP1(svsublt_u64, aarch64_sve_usublt, 4398046511108),
SVEMAP1(svsubltb_n_s16, aarch64_sve_ssubltb, 4398046513154),
SVEMAP1(svsubltb_n_s32, aarch64_sve_ssubltb, 4398046513155),
SVEMAP1(svsubltb_n_s64, aarch64_sve_ssubltb, 4398046513156),
SVEMAP1(svsubltb_s16, aarch64_sve_ssubltb, 4398046511106),
SVEMAP1(svsubltb_s32, aarch64_sve_ssubltb, 4398046511107),
SVEMAP1(svsubltb_s64, aarch64_sve_ssubltb, 4398046511108),
SVEMAP1(svsubr_f16_m, aarch64_sve_fsubr, 4432406249734),
SVEMAP1(svsubr_f16_x, aarch64_sve_fsub_u, 4432406249606),
SVEMAP1(svsubr_f16_z, aarch64_sve_fsubr, 4432406249862),
SVEMAP1(svsubr_f32_m, aarch64_sve_fsubr, 4432406249735),
SVEMAP1(svsubr_f32_x, aarch64_sve_fsub_u, 4432406249607),
SVEMAP1(svsubr_f32_z, aarch64_sve_fsubr, 4432406249863),
SVEMAP1(svsubr_f64_m, aarch64_sve_fsubr, 4432406249736),
SVEMAP1(svsubr_f64_x, aarch64_sve_fsub_u, 4432406249608),
SVEMAP1(svsubr_f64_z, aarch64_sve_fsubr, 4432406249864),
SVEMAP1(svsubr_n_f16_m, aarch64_sve_fsubr, 4432406252806),
SVEMAP1(svsubr_n_f16_x, aarch64_sve_fsub_u, 4432406252678),
SVEMAP1(svsubr_n_f16_z, aarch64_sve_fsubr, 4432406252934),
SVEMAP1(svsubr_n_f32_m, aarch64_sve_fsubr, 4432406252807),
SVEMAP1(svsubr_n_f32_x, aarch64_sve_fsub_u, 4432406252679),
SVEMAP1(svsubr_n_f32_z, aarch64_sve_fsubr, 4432406252935),
SVEMAP1(svsubr_n_f64_m, aarch64_sve_fsubr, 4432406252808),
SVEMAP1(svsubr_n_f64_x, aarch64_sve_fsub_u, 4432406252680),
SVEMAP1(svsubr_n_f64_z, aarch64_sve_fsubr, 4432406252936),
SVEMAP1(svsubr_n_s16_m, aarch64_sve_subr, 4432406252802),
SVEMAP1(svsubr_n_s16_x, aarch64_sve_sub_u, 4432406252674),
SVEMAP1(svsubr_n_s16_z, aarch64_sve_subr, 4432406252930),
SVEMAP1(svsubr_n_s32_m, aarch64_sve_subr, 4432406252803),
SVEMAP1(svsubr_n_s32_x, aarch64_sve_sub_u, 4432406252675),
SVEMAP1(svsubr_n_s32_z, aarch64_sve_subr, 4432406252931),
SVEMAP1(svsubr_n_s64_m, aarch64_sve_subr, 4432406252804),
SVEMAP1(svsubr_n_s64_x, aarch64_sve_sub_u, 4432406252676),
SVEMAP1(svsubr_n_s64_z, aarch64_sve_subr, 4432406252932),
SVEMAP1(svsubr_n_s8_m, aarch64_sve_subr, 4432406252801),
SVEMAP1(svsubr_n_s8_x, aarch64_sve_sub_u, 4432406252673),
SVEMAP1(svsubr_n_s8_z, aarch64_sve_subr, 4432406252929),
SVEMAP1(svsubr_n_u16_m, aarch64_sve_subr, 4432406252802),
SVEMAP1(svsubr_n_u16_x, aarch64_sve_sub_u, 4432406252674),
SVEMAP1(svsubr_n_u16_z, aarch64_sve_subr, 4432406252930),
SVEMAP1(svsubr_n_u32_m, aarch64_sve_subr, 4432406252803),
SVEMAP1(svsubr_n_u32_x, aarch64_sve_sub_u, 4432406252675),
SVEMAP1(svsubr_n_u32_z, aarch64_sve_subr, 4432406252931),
SVEMAP1(svsubr_n_u64_m, aarch64_sve_subr, 4432406252804),
SVEMAP1(svsubr_n_u64_x, aarch64_sve_sub_u, 4432406252676),
SVEMAP1(svsubr_n_u64_z, aarch64_sve_subr, 4432406252932),
SVEMAP1(svsubr_n_u8_m, aarch64_sve_subr, 4432406252801),
SVEMAP1(svsubr_n_u8_x, aarch64_sve_sub_u, 4432406252673),
SVEMAP1(svsubr_n_u8_z, aarch64_sve_subr, 4432406252929),
SVEMAP1(svsubr_s16_m, aarch64_sve_subr, 4432406249730),
SVEMAP1(svsubr_s16_x, aarch64_sve_sub_u, 4432406249602),
SVEMAP1(svsubr_s16_z, aarch64_sve_subr, 4432406249858),
SVEMAP1(svsubr_s32_m, aarch64_sve_subr, 4432406249731),
SVEMAP1(svsubr_s32_x, aarch64_sve_sub_u, 4432406249603),
SVEMAP1(svsubr_s32_z, aarch64_sve_subr, 4432406249859),
SVEMAP1(svsubr_s64_m, aarch64_sve_subr, 4432406249732),
SVEMAP1(svsubr_s64_x, aarch64_sve_sub_u, 4432406249604),
SVEMAP1(svsubr_s64_z, aarch64_sve_subr, 4432406249860),
SVEMAP1(svsubr_s8_m, aarch64_sve_subr, 4432406249729),
SVEMAP1(svsubr_s8_x, aarch64_sve_sub_u, 4432406249601),
SVEMAP1(svsubr_s8_z, aarch64_sve_subr, 4432406249857),
SVEMAP1(svsubr_u16_m, aarch64_sve_subr, 4432406249730),
SVEMAP1(svsubr_u16_x, aarch64_sve_sub_u, 4432406249602),
SVEMAP1(svsubr_u16_z, aarch64_sve_subr, 4432406249858),
SVEMAP1(svsubr_u32_m, aarch64_sve_subr, 4432406249731),
SVEMAP1(svsubr_u32_x, aarch64_sve_sub_u, 4432406249603),
SVEMAP1(svsubr_u32_z, aarch64_sve_subr, 4432406249859),
SVEMAP1(svsubr_u64_m, aarch64_sve_subr, 4432406249732),
SVEMAP1(svsubr_u64_x, aarch64_sve_sub_u, 4432406249604),
SVEMAP1(svsubr_u64_z, aarch64_sve_subr, 4432406249860),
SVEMAP1(svsubr_u8_m, aarch64_sve_subr, 4432406249729),
SVEMAP1(svsubr_u8_x, aarch64_sve_sub_u, 4432406249601),
SVEMAP1(svsubr_u8_z, aarch64_sve_subr, 4432406249857),
SVEMAP1(svsubwb_n_s16, aarch64_sve_ssubwb, 4398046513154),
SVEMAP1(svsubwb_n_s32, aarch64_sve_ssubwb, 4398046513155),
SVEMAP1(svsubwb_n_s64, aarch64_sve_ssubwb, 4398046513156),
SVEMAP1(svsubwb_n_u16, aarch64_sve_usubwb, 4398046513154),
SVEMAP1(svsubwb_n_u32, aarch64_sve_usubwb, 4398046513155),
SVEMAP1(svsubwb_n_u64, aarch64_sve_usubwb, 4398046513156),
SVEMAP1(svsubwb_s16, aarch64_sve_ssubwb, 4398046511106),
SVEMAP1(svsubwb_s32, aarch64_sve_ssubwb, 4398046511107),
SVEMAP1(svsubwb_s64, aarch64_sve_ssubwb, 4398046511108),
SVEMAP1(svsubwb_u16, aarch64_sve_usubwb, 4398046511106),
SVEMAP1(svsubwb_u32, aarch64_sve_usubwb, 4398046511107),
SVEMAP1(svsubwb_u64, aarch64_sve_usubwb, 4398046511108),
SVEMAP1(svsubwt_n_s16, aarch64_sve_ssubwt, 4398046513154),
SVEMAP1(svsubwt_n_s32, aarch64_sve_ssubwt, 4398046513155),
SVEMAP1(svsubwt_n_s64, aarch64_sve_ssubwt, 4398046513156),
SVEMAP1(svsubwt_n_u16, aarch64_sve_usubwt, 4398046513154),
SVEMAP1(svsubwt_n_u32, aarch64_sve_usubwt, 4398046513155),
SVEMAP1(svsubwt_n_u64, aarch64_sve_usubwt, 4398046513156),
SVEMAP1(svsubwt_s16, aarch64_sve_ssubwt, 4398046511106),
SVEMAP1(svsubwt_s32, aarch64_sve_ssubwt, 4398046511107),
SVEMAP1(svsubwt_s64, aarch64_sve_ssubwt, 4398046511108),
SVEMAP1(svsubwt_u16, aarch64_sve_usubwt, 4398046511106),
SVEMAP1(svsubwt_u32, aarch64_sve_usubwt, 4398046511107),
SVEMAP1(svsubwt_u64, aarch64_sve_usubwt, 4398046511108),
SVEMAP1(svsudot_lane_s32, aarch64_sve_sudot_lane, 4398046511107),
SVEMAP1(svsudot_n_s32, aarch64_sve_usdot, 4399120256003),
SVEMAP1(svsudot_s32, aarch64_sve_usdot, 4399120252931),
SVEMAP2(svtbl2_bf16, 4398046511117),
SVEMAP2(svtbl2_f16, 4398046511110),
SVEMAP2(svtbl2_f32, 4398046511111),
SVEMAP2(svtbl2_f64, 4398046511112),
SVEMAP2(svtbl2_s16, 4398046511106),
SVEMAP2(svtbl2_s32, 4398046511107),
SVEMAP2(svtbl2_s64, 4398046511108),
SVEMAP2(svtbl2_s8, 4398046511105),
SVEMAP2(svtbl2_u16, 4398046511106),
SVEMAP2(svtbl2_u32, 4398046511107),
SVEMAP2(svtbl2_u64, 4398046511108),
SVEMAP2(svtbl2_u8, 4398046511105),
SVEMAP1(svtbl_bf16, aarch64_sve_tbl, 4398046511117),
SVEMAP1(svtbl_f16, aarch64_sve_tbl, 4398046511110),
SVEMAP1(svtbl_f32, aarch64_sve_tbl, 4398046511111),
SVEMAP1(svtbl_f64, aarch64_sve_tbl, 4398046511112),
SVEMAP1(svtbl_s16, aarch64_sve_tbl, 4398046511106),
SVEMAP1(svtbl_s32, aarch64_sve_tbl, 4398046511107),
SVEMAP1(svtbl_s64, aarch64_sve_tbl, 4398046511108),
SVEMAP1(svtbl_s8, aarch64_sve_tbl, 4398046511105),
SVEMAP1(svtbl_u16, aarch64_sve_tbl, 4398046511106),
SVEMAP1(svtbl_u32, aarch64_sve_tbl, 4398046511107),
SVEMAP1(svtbl_u64, aarch64_sve_tbl, 4398046511108),
SVEMAP1(svtbl_u8, aarch64_sve_tbl, 4398046511105),
SVEMAP1(svtblq_bf16, aarch64_sve_tblq, 4398046511117),
SVEMAP1(svtblq_f16, aarch64_sve_tblq, 4398046511110),
SVEMAP1(svtblq_f32, aarch64_sve_tblq, 4398046511111),
SVEMAP1(svtblq_f64, aarch64_sve_tblq, 4398046511112),
SVEMAP1(svtblq_mf8, aarch64_sve_tblq, 4398046511118),
SVEMAP1(svtblq_s16, aarch64_sve_tblq, 4398046511106),
SVEMAP1(svtblq_s32, aarch64_sve_tblq, 4398046511107),
SVEMAP1(svtblq_s64, aarch64_sve_tblq, 4398046511108),
SVEMAP1(svtblq_s8, aarch64_sve_tblq, 4398046511105),
SVEMAP1(svtblq_u16, aarch64_sve_tblq, 4398046511106),
SVEMAP1(svtblq_u32, aarch64_sve_tblq, 4398046511107),
SVEMAP1(svtblq_u64, aarch64_sve_tblq, 4398046511108),
SVEMAP1(svtblq_u8, aarch64_sve_tblq, 4398046511105),
SVEMAP1(svtbx_bf16, aarch64_sve_tbx, 4398046511117),
SVEMAP1(svtbx_f16, aarch64_sve_tbx, 4398046511110),
SVEMAP1(svtbx_f32, aarch64_sve_tbx, 4398046511111),
SVEMAP1(svtbx_f64, aarch64_sve_tbx, 4398046511112),
SVEMAP1(svtbx_s16, aarch64_sve_tbx, 4398046511106),
SVEMAP1(svtbx_s32, aarch64_sve_tbx, 4398046511107),
SVEMAP1(svtbx_s64, aarch64_sve_tbx, 4398046511108),
SVEMAP1(svtbx_s8, aarch64_sve_tbx, 4398046511105),
SVEMAP1(svtbx_u16, aarch64_sve_tbx, 4398046511106),
SVEMAP1(svtbx_u32, aarch64_sve_tbx, 4398046511107),
SVEMAP1(svtbx_u64, aarch64_sve_tbx, 4398046511108),
SVEMAP1(svtbx_u8, aarch64_sve_tbx, 4398046511105),
SVEMAP1(svtbxq_bf16, aarch64_sve_tbxq, 4398046511117),
SVEMAP1(svtbxq_f16, aarch64_sve_tbxq, 4398046511110),
SVEMAP1(svtbxq_f32, aarch64_sve_tbxq, 4398046511111),
SVEMAP1(svtbxq_f64, aarch64_sve_tbxq, 4398046511112),
SVEMAP1(svtbxq_mf8, aarch64_sve_tbxq, 4398046511118),
SVEMAP1(svtbxq_s16, aarch64_sve_tbxq, 4398046511106),
SVEMAP1(svtbxq_s32, aarch64_sve_tbxq, 4398046511107),
SVEMAP1(svtbxq_s64, aarch64_sve_tbxq, 4398046511108),
SVEMAP1(svtbxq_s8, aarch64_sve_tbxq, 4398046511105),
SVEMAP1(svtbxq_u16, aarch64_sve_tbxq, 4398046511106),
SVEMAP1(svtbxq_u32, aarch64_sve_tbxq, 4398046511107),
SVEMAP1(svtbxq_u64, aarch64_sve_tbxq, 4398046511108),
SVEMAP1(svtbxq_u8, aarch64_sve_tbxq, 4398046511105),
SVEMAP1(svtmad_f16, aarch64_sve_ftmad_x, 6),
SVEMAP1(svtmad_f32, aarch64_sve_ftmad_x, 7),
SVEMAP1(svtmad_f64, aarch64_sve_ftmad_x, 8),
SVEMAP1(svtrn1_b16, aarch64_sve_trn1_b16, 4398047559689),
SVEMAP1(svtrn1_b32, aarch64_sve_trn1_b32, 4398047559689),
SVEMAP1(svtrn1_b64, aarch64_sve_trn1_b64, 4398047559689),
SVEMAP1(svtrn1_b8, aarch64_sve_trn1, 4398046511113),
SVEMAP1(svtrn1_bf16, aarch64_sve_trn1, 4398046511117),
SVEMAP1(svtrn1_f16, aarch64_sve_trn1, 4398046511110),
SVEMAP1(svtrn1_f32, aarch64_sve_trn1, 4398046511111),
SVEMAP1(svtrn1_f64, aarch64_sve_trn1, 4398046511112),
SVEMAP1(svtrn1_s16, aarch64_sve_trn1, 4398046511106),
SVEMAP1(svtrn1_s32, aarch64_sve_trn1, 4398046511107),
SVEMAP1(svtrn1_s64, aarch64_sve_trn1, 4398046511108),
SVEMAP1(svtrn1_s8, aarch64_sve_trn1, 4398046511105),
SVEMAP1(svtrn1_u16, aarch64_sve_trn1, 4398046511106),
SVEMAP1(svtrn1_u32, aarch64_sve_trn1, 4398046511107),
SVEMAP1(svtrn1_u64, aarch64_sve_trn1, 4398046511108),
SVEMAP1(svtrn1_u8, aarch64_sve_trn1, 4398046511105),
SVEMAP1(svtrn1q_bf16, aarch64_sve_trn1q, 13),
SVEMAP1(svtrn1q_f16, aarch64_sve_trn1q, 6),
SVEMAP1(svtrn1q_f32, aarch64_sve_trn1q, 7),
SVEMAP1(svtrn1q_f64, aarch64_sve_trn1q, 8),
SVEMAP1(svtrn1q_s16, aarch64_sve_trn1q, 2),
SVEMAP1(svtrn1q_s32, aarch64_sve_trn1q, 3),
SVEMAP1(svtrn1q_s64, aarch64_sve_trn1q, 4),
SVEMAP1(svtrn1q_s8, aarch64_sve_trn1q, 1),
SVEMAP1(svtrn1q_u16, aarch64_sve_trn1q, 2),
SVEMAP1(svtrn1q_u32, aarch64_sve_trn1q, 3),
SVEMAP1(svtrn1q_u64, aarch64_sve_trn1q, 4),
SVEMAP1(svtrn1q_u8, aarch64_sve_trn1q, 1),
SVEMAP1(svtrn2_b16, aarch64_sve_trn2_b16, 4398047559689),
SVEMAP1(svtrn2_b32, aarch64_sve_trn2_b32, 4398047559689),
SVEMAP1(svtrn2_b64, aarch64_sve_trn2_b64, 4398047559689),
SVEMAP1(svtrn2_b8, aarch64_sve_trn2, 4398046511113),
SVEMAP1(svtrn2_bf16, aarch64_sve_trn2, 4398046511117),
SVEMAP1(svtrn2_f16, aarch64_sve_trn2, 4398046511110),
SVEMAP1(svtrn2_f32, aarch64_sve_trn2, 4398046511111),
SVEMAP1(svtrn2_f64, aarch64_sve_trn2, 4398046511112),
SVEMAP1(svtrn2_s16, aarch64_sve_trn2, 4398046511106),
SVEMAP1(svtrn2_s32, aarch64_sve_trn2, 4398046511107),
SVEMAP1(svtrn2_s64, aarch64_sve_trn2, 4398046511108),
SVEMAP1(svtrn2_s8, aarch64_sve_trn2, 4398046511105),
SVEMAP1(svtrn2_u16, aarch64_sve_trn2, 4398046511106),
SVEMAP1(svtrn2_u32, aarch64_sve_trn2, 4398046511107),
SVEMAP1(svtrn2_u64, aarch64_sve_trn2, 4398046511108),
SVEMAP1(svtrn2_u8, aarch64_sve_trn2, 4398046511105),
SVEMAP1(svtrn2q_bf16, aarch64_sve_trn2q, 13),
SVEMAP1(svtrn2q_f16, aarch64_sve_trn2q, 6),
SVEMAP1(svtrn2q_f32, aarch64_sve_trn2q, 7),
SVEMAP1(svtrn2q_f64, aarch64_sve_trn2q, 8),
SVEMAP1(svtrn2q_s16, aarch64_sve_trn2q, 2),
SVEMAP1(svtrn2q_s32, aarch64_sve_trn2q, 3),
SVEMAP1(svtrn2q_s64, aarch64_sve_trn2q, 4),
SVEMAP1(svtrn2q_s8, aarch64_sve_trn2q, 1),
SVEMAP1(svtrn2q_u16, aarch64_sve_trn2q, 2),
SVEMAP1(svtrn2q_u32, aarch64_sve_trn2q, 3),
SVEMAP1(svtrn2q_u64, aarch64_sve_trn2q, 4),
SVEMAP1(svtrn2q_u8, aarch64_sve_trn2q, 1),
SVEMAP1(svtsmul_f16, aarch64_sve_ftsmul_x, 6),
SVEMAP1(svtsmul_f32, aarch64_sve_ftsmul_x, 7),
SVEMAP1(svtsmul_f64, aarch64_sve_ftsmul_x, 8),
SVEMAP1(svtssel_f16, aarch64_sve_ftssel_x, 6),
SVEMAP1(svtssel_f32, aarch64_sve_ftssel_x, 7),
SVEMAP1(svtssel_f64, aarch64_sve_ftssel_x, 8),
SVEMAP2(svundef2_b, 4400193994761),
SVEMAP2(svundef2_bf16, 4400193994765),
SVEMAP2(svundef2_f16, 4400193994758),
SVEMAP2(svundef2_f32, 4400193994759),
SVEMAP2(svundef2_f64, 4400193994760),
SVEMAP2(svundef2_mf8, 4400193994766),
SVEMAP2(svundef2_s16, 4400193994754),
SVEMAP2(svundef2_s32, 4400193994755),
SVEMAP2(svundef2_s64, 4400193994756),
SVEMAP2(svundef2_s8, 4400193994753),
SVEMAP2(svundef2_u16, 4400193994754),
SVEMAP2(svundef2_u32, 4400193994755),
SVEMAP2(svundef2_u64, 4400193994756),
SVEMAP2(svundef2_u8, 4400193994753),
SVEMAP2(svundef3_bf16, 4400193994765),
SVEMAP2(svundef3_f16, 4400193994758),
SVEMAP2(svundef3_f32, 4400193994759),
SVEMAP2(svundef3_f64, 4400193994760),
SVEMAP2(svundef3_mf8, 4400193994766),
SVEMAP2(svundef3_s16, 4400193994754),
SVEMAP2(svundef3_s32, 4400193994755),
SVEMAP2(svundef3_s64, 4400193994756),
SVEMAP2(svundef3_s8, 4400193994753),
SVEMAP2(svundef3_u16, 4400193994754),
SVEMAP2(svundef3_u32, 4400193994755),
SVEMAP2(svundef3_u64, 4400193994756),
SVEMAP2(svundef3_u8, 4400193994753),
SVEMAP2(svundef4_b, 4400193994761),
SVEMAP2(svundef4_bf16, 4400193994765),
SVEMAP2(svundef4_f16, 4400193994758),
SVEMAP2(svundef4_f32, 4400193994759),
SVEMAP2(svundef4_f64, 4400193994760),
SVEMAP2(svundef4_mf8, 4400193994766),
SVEMAP2(svundef4_s16, 4400193994754),
SVEMAP2(svundef4_s32, 4400193994755),
SVEMAP2(svundef4_s64, 4400193994756),
SVEMAP2(svundef4_s8, 4400193994753),
SVEMAP2(svundef4_u16, 4400193994754),
SVEMAP2(svundef4_u32, 4400193994755),
SVEMAP2(svundef4_u64, 4400193994756),
SVEMAP2(svundef4_u8, 4400193994753),
SVEMAP2(svundef_bf16, 4400193994765),
SVEMAP2(svundef_f16, 4400193994758),
SVEMAP2(svundef_f32, 4400193994759),
SVEMAP2(svundef_f64, 4400193994760),
SVEMAP2(svundef_mf8, 4400193994766),
SVEMAP2(svundef_s16, 4400193994754),
SVEMAP2(svundef_s32, 4400193994755),
SVEMAP2(svundef_s64, 4400193994756),
SVEMAP2(svundef_s8, 4400193994753),
SVEMAP2(svundef_u16, 4400193994754),
SVEMAP2(svundef_u32, 4400193994755),
SVEMAP2(svundef_u64, 4400193994756),
SVEMAP2(svundef_u8, 4400193994753),
SVEMAP1(svunpk_s16_s8_x2, aarch64_sve_sunpk_x2, 137438953474),
SVEMAP1(svunpk_s16_s8_x4, aarch64_sve_sunpk_x4, 137438953474),
SVEMAP1(svunpk_s32_s16_x2, aarch64_sve_sunpk_x2, 137438953475),
SVEMAP1(svunpk_s32_s16_x4, aarch64_sve_sunpk_x4, 137438953475),
SVEMAP1(svunpk_s64_s32_x2, aarch64_sve_sunpk_x2, 137438953476),
SVEMAP1(svunpk_s64_s32_x4, aarch64_sve_sunpk_x4, 137438953476),
SVEMAP1(svunpk_u16_u8_x2, aarch64_sve_uunpk_x2, 137438953474),
SVEMAP1(svunpk_u16_u8_x4, aarch64_sve_uunpk_x4, 137438953474),
SVEMAP1(svunpk_u32_u16_x2, aarch64_sve_uunpk_x2, 137438953475),
SVEMAP1(svunpk_u32_u16_x4, aarch64_sve_uunpk_x4, 137438953475),
SVEMAP1(svunpk_u64_u32_x2, aarch64_sve_uunpk_x2, 137438953476),
SVEMAP1(svunpk_u64_u32_x4, aarch64_sve_uunpk_x4, 137438953476),
SVEMAP1(svunpkhi_b, aarch64_sve_punpkhi, 4398046511113),
SVEMAP1(svunpkhi_s16, aarch64_sve_sunpkhi, 4398046511106),
SVEMAP1(svunpkhi_s32, aarch64_sve_sunpkhi, 4398046511107),
SVEMAP1(svunpkhi_s64, aarch64_sve_sunpkhi, 4398046511108),
SVEMAP1(svunpkhi_u16, aarch64_sve_uunpkhi, 4398046511106),
SVEMAP1(svunpkhi_u32, aarch64_sve_uunpkhi, 4398046511107),
SVEMAP1(svunpkhi_u64, aarch64_sve_uunpkhi, 4398046511108),
SVEMAP1(svunpklo_b, aarch64_sve_punpklo, 4398046511113),
SVEMAP1(svunpklo_s16, aarch64_sve_sunpklo, 4398046511106),
SVEMAP1(svunpklo_s32, aarch64_sve_sunpklo, 4398046511107),
SVEMAP1(svunpklo_s64, aarch64_sve_sunpklo, 4398046511108),
SVEMAP1(svunpklo_u16, aarch64_sve_uunpklo, 4398046511106),
SVEMAP1(svunpklo_u32, aarch64_sve_uunpklo, 4398046511107),
SVEMAP1(svunpklo_u64, aarch64_sve_uunpklo, 4398046511108),
SVEMAP1(svuqadd_n_s16_m, aarch64_sve_suqadd, 4398046514434),
SVEMAP1(svuqadd_n_s16_x, aarch64_sve_suqadd, 4398046514306),
SVEMAP1(svuqadd_n_s16_z, aarch64_sve_suqadd, 4398046514562),
SVEMAP1(svuqadd_n_s32_m, aarch64_sve_suqadd, 4398046514435),
SVEMAP1(svuqadd_n_s32_x, aarch64_sve_suqadd, 4398046514307),
SVEMAP1(svuqadd_n_s32_z, aarch64_sve_suqadd, 4398046514563),
SVEMAP1(svuqadd_n_s64_m, aarch64_sve_suqadd, 4398046514436),
SVEMAP1(svuqadd_n_s64_x, aarch64_sve_suqadd, 4398046514308),
SVEMAP1(svuqadd_n_s64_z, aarch64_sve_suqadd, 4398046514564),
SVEMAP1(svuqadd_n_s8_m, aarch64_sve_suqadd, 4398046514433),
SVEMAP1(svuqadd_n_s8_x, aarch64_sve_suqadd, 4398046514305),
SVEMAP1(svuqadd_n_s8_z, aarch64_sve_suqadd, 4398046514561),
SVEMAP1(svuqadd_s16_m, aarch64_sve_suqadd, 4398046511362),
SVEMAP1(svuqadd_s16_x, aarch64_sve_suqadd, 4398046511234),
SVEMAP1(svuqadd_s16_z, aarch64_sve_suqadd, 4398046511490),
SVEMAP1(svuqadd_s32_m, aarch64_sve_suqadd, 4398046511363),
SVEMAP1(svuqadd_s32_x, aarch64_sve_suqadd, 4398046511235),
SVEMAP1(svuqadd_s32_z, aarch64_sve_suqadd, 4398046511491),
SVEMAP1(svuqadd_s64_m, aarch64_sve_suqadd, 4398046511364),
SVEMAP1(svuqadd_s64_x, aarch64_sve_suqadd, 4398046511236),
SVEMAP1(svuqadd_s64_z, aarch64_sve_suqadd, 4398046511492),
SVEMAP1(svuqadd_s8_m, aarch64_sve_suqadd, 4398046511361),
SVEMAP1(svuqadd_s8_x, aarch64_sve_suqadd, 4398046511233),
SVEMAP1(svuqadd_s8_z, aarch64_sve_suqadd, 4398046511489),
SVEMAP1(svusdot_lane_s32, aarch64_sve_usdot_lane, 4398046511107),
SVEMAP1(svusdot_n_s32, aarch64_sve_usdot, 4398046514179),
SVEMAP1(svusdot_s32, aarch64_sve_usdot, 4398046511107),
SVEMAP1(svusmmla_s32, aarch64_sve_usmmla, 3),
SVEMAP1(svuzp1_b16, aarch64_sve_uzp1_b16, 4398047559689),
SVEMAP1(svuzp1_b32, aarch64_sve_uzp1_b32, 4398047559689),
SVEMAP1(svuzp1_b64, aarch64_sve_uzp1_b64, 4398047559689),
SVEMAP1(svuzp1_b8, aarch64_sve_uzp1, 4398046511113),
SVEMAP1(svuzp1_bf16, aarch64_sve_uzp1, 4398046511117),
SVEMAP1(svuzp1_f16, aarch64_sve_uzp1, 4398046511110),
SVEMAP1(svuzp1_f32, aarch64_sve_uzp1, 4398046511111),
SVEMAP1(svuzp1_f64, aarch64_sve_uzp1, 4398046511112),
SVEMAP1(svuzp1_s16, aarch64_sve_uzp1, 4398046511106),
SVEMAP1(svuzp1_s32, aarch64_sve_uzp1, 4398046511107),
SVEMAP1(svuzp1_s64, aarch64_sve_uzp1, 4398046511108),
SVEMAP1(svuzp1_s8, aarch64_sve_uzp1, 4398046511105),
SVEMAP1(svuzp1_u16, aarch64_sve_uzp1, 4398046511106),
SVEMAP1(svuzp1_u32, aarch64_sve_uzp1, 4398046511107),
SVEMAP1(svuzp1_u64, aarch64_sve_uzp1, 4398046511108),
SVEMAP1(svuzp1_u8, aarch64_sve_uzp1, 4398046511105),
SVEMAP1(svuzp1q_bf16, aarch64_sve_uzp1q, 13),
SVEMAP1(svuzp1q_f16, aarch64_sve_uzp1q, 6),
SVEMAP1(svuzp1q_f32, aarch64_sve_uzp1q, 7),
SVEMAP1(svuzp1q_f64, aarch64_sve_uzp1q, 8),
SVEMAP1(svuzp1q_s16, aarch64_sve_uzp1q, 2),
SVEMAP1(svuzp1q_s32, aarch64_sve_uzp1q, 3),
SVEMAP1(svuzp1q_s64, aarch64_sve_uzp1q, 4),
SVEMAP1(svuzp1q_s8, aarch64_sve_uzp1q, 1),
SVEMAP1(svuzp1q_u16, aarch64_sve_uzp1q, 2),
SVEMAP1(svuzp1q_u32, aarch64_sve_uzp1q, 3),
SVEMAP1(svuzp1q_u64, aarch64_sve_uzp1q, 4),
SVEMAP1(svuzp1q_u8, aarch64_sve_uzp1q, 1),
SVEMAP1(svuzp2_b16, aarch64_sve_uzp2_b16, 4398047559689),
SVEMAP1(svuzp2_b32, aarch64_sve_uzp2_b32, 4398047559689),
SVEMAP1(svuzp2_b64, aarch64_sve_uzp2_b64, 4398047559689),
SVEMAP1(svuzp2_b8, aarch64_sve_uzp2, 4398046511113),
SVEMAP1(svuzp2_bf16, aarch64_sve_uzp2, 4398046511117),
SVEMAP1(svuzp2_f16, aarch64_sve_uzp2, 4398046511110),
SVEMAP1(svuzp2_f32, aarch64_sve_uzp2, 4398046511111),
SVEMAP1(svuzp2_f64, aarch64_sve_uzp2, 4398046511112),
SVEMAP1(svuzp2_s16, aarch64_sve_uzp2, 4398046511106),
SVEMAP1(svuzp2_s32, aarch64_sve_uzp2, 4398046511107),
SVEMAP1(svuzp2_s64, aarch64_sve_uzp2, 4398046511108),
SVEMAP1(svuzp2_s8, aarch64_sve_uzp2, 4398046511105),
SVEMAP1(svuzp2_u16, aarch64_sve_uzp2, 4398046511106),
SVEMAP1(svuzp2_u32, aarch64_sve_uzp2, 4398046511107),
SVEMAP1(svuzp2_u64, aarch64_sve_uzp2, 4398046511108),
SVEMAP1(svuzp2_u8, aarch64_sve_uzp2, 4398046511105),
SVEMAP1(svuzp2q_bf16, aarch64_sve_uzp2q, 13),
SVEMAP1(svuzp2q_f16, aarch64_sve_uzp2q, 6),
SVEMAP1(svuzp2q_f32, aarch64_sve_uzp2q, 7),
SVEMAP1(svuzp2q_f64, aarch64_sve_uzp2q, 8),
SVEMAP1(svuzp2q_s16, aarch64_sve_uzp2q, 2),
SVEMAP1(svuzp2q_s32, aarch64_sve_uzp2q, 3),
SVEMAP1(svuzp2q_s64, aarch64_sve_uzp2q, 4),
SVEMAP1(svuzp2q_s8, aarch64_sve_uzp2q, 1),
SVEMAP1(svuzp2q_u16, aarch64_sve_uzp2q, 2),
SVEMAP1(svuzp2q_u32, aarch64_sve_uzp2q, 3),
SVEMAP1(svuzp2q_u64, aarch64_sve_uzp2q, 4),
SVEMAP1(svuzp2q_u8, aarch64_sve_uzp2q, 1),
SVEMAP1(svuzp_bf16_x2, aarch64_sve_uzp_x2, 137438953485),
SVEMAP1(svuzp_bf16_x4, aarch64_sve_uzp_x4, 137438953485),
SVEMAP1(svuzp_f16_x2, aarch64_sve_uzp_x2, 137438953478),
SVEMAP1(svuzp_f16_x4, aarch64_sve_uzp_x4, 137438953478),
SVEMAP1(svuzp_f32_x2, aarch64_sve_uzp_x2, 137438953479),
SVEMAP1(svuzp_f32_x4, aarch64_sve_uzp_x4, 137438953479),
SVEMAP1(svuzp_f64_x2, aarch64_sve_uzp_x2, 137438953480),
SVEMAP1(svuzp_f64_x4, aarch64_sve_uzp_x4, 137438953480),
SVEMAP1(svuzp_mf8_x2, aarch64_sve_uzp_x2, 137438953486),
SVEMAP1(svuzp_mf8_x4, aarch64_sve_uzp_x4, 137438953486),
SVEMAP1(svuzp_s16_x2, aarch64_sve_uzp_x2, 137438953474),
SVEMAP1(svuzp_s16_x4, aarch64_sve_uzp_x4, 137438953474),
SVEMAP1(svuzp_s32_x2, aarch64_sve_uzp_x2, 137438953475),
SVEMAP1(svuzp_s32_x4, aarch64_sve_uzp_x4, 137438953475),
SVEMAP1(svuzp_s64_x2, aarch64_sve_uzp_x2, 137438953476),
SVEMAP1(svuzp_s64_x4, aarch64_sve_uzp_x4, 137438953476),
SVEMAP1(svuzp_s8_x2, aarch64_sve_uzp_x2, 137438953473),
SVEMAP1(svuzp_s8_x4, aarch64_sve_uzp_x4, 137438953473),
SVEMAP1(svuzp_u16_x2, aarch64_sve_uzp_x2, 137438953474),
SVEMAP1(svuzp_u16_x4, aarch64_sve_uzp_x4, 137438953474),
SVEMAP1(svuzp_u32_x2, aarch64_sve_uzp_x2, 137438953475),
SVEMAP1(svuzp_u32_x4, aarch64_sve_uzp_x4, 137438953475),
SVEMAP1(svuzp_u64_x2, aarch64_sve_uzp_x2, 137438953476),
SVEMAP1(svuzp_u64_x4, aarch64_sve_uzp_x4, 137438953476),
SVEMAP1(svuzp_u8_x2, aarch64_sve_uzp_x2, 137438953473),
SVEMAP1(svuzp_u8_x4, aarch64_sve_uzp_x4, 137438953473),
SVEMAP1(svuzpq1_bf16, aarch64_sve_uzpq1, 4398046511117),
SVEMAP1(svuzpq1_f16, aarch64_sve_uzpq1, 4398046511110),
SVEMAP1(svuzpq1_f32, aarch64_sve_uzpq1, 4398046511111),
SVEMAP1(svuzpq1_f64, aarch64_sve_uzpq1, 4398046511112),
SVEMAP1(svuzpq1_mf8, aarch64_sve_uzpq1, 4398046511118),
SVEMAP1(svuzpq1_s16, aarch64_sve_uzpq1, 4398046511106),
SVEMAP1(svuzpq1_s32, aarch64_sve_uzpq1, 4398046511107),
SVEMAP1(svuzpq1_s64, aarch64_sve_uzpq1, 4398046511108),
SVEMAP1(svuzpq1_s8, aarch64_sve_uzpq1, 4398046511105),
SVEMAP1(svuzpq1_u16, aarch64_sve_uzpq1, 4398046511106),
SVEMAP1(svuzpq1_u32, aarch64_sve_uzpq1, 4398046511107),
SVEMAP1(svuzpq1_u64, aarch64_sve_uzpq1, 4398046511108),
SVEMAP1(svuzpq1_u8, aarch64_sve_uzpq1, 4398046511105),
SVEMAP1(svuzpq2_bf16, aarch64_sve_uzpq2, 4398046511117),
SVEMAP1(svuzpq2_f16, aarch64_sve_uzpq2, 4398046511110),
SVEMAP1(svuzpq2_f32, aarch64_sve_uzpq2, 4398046511111),
SVEMAP1(svuzpq2_f64, aarch64_sve_uzpq2, 4398046511112),
SVEMAP1(svuzpq2_mf8, aarch64_sve_uzpq2, 4398046511118),
SVEMAP1(svuzpq2_s16, aarch64_sve_uzpq2, 4398046511106),
SVEMAP1(svuzpq2_s32, aarch64_sve_uzpq2, 4398046511107),
SVEMAP1(svuzpq2_s64, aarch64_sve_uzpq2, 4398046511108),
SVEMAP1(svuzpq2_s8, aarch64_sve_uzpq2, 4398046511105),
SVEMAP1(svuzpq2_u16, aarch64_sve_uzpq2, 4398046511106),
SVEMAP1(svuzpq2_u32, aarch64_sve_uzpq2, 4398046511107),
SVEMAP1(svuzpq2_u64, aarch64_sve_uzpq2, 4398046511108),
SVEMAP1(svuzpq2_u8, aarch64_sve_uzpq2, 4398046511105),
SVEMAP1(svuzpq_bf16_x2, aarch64_sve_uzpq_x2, 137438953485),
SVEMAP1(svuzpq_bf16_x4, aarch64_sve_uzpq_x4, 137438953485),
SVEMAP1(svuzpq_f16_x2, aarch64_sve_uzpq_x2, 137438953478),
SVEMAP1(svuzpq_f16_x4, aarch64_sve_uzpq_x4, 137438953478),
SVEMAP1(svuzpq_f32_x2, aarch64_sve_uzpq_x2, 137438953479),
SVEMAP1(svuzpq_f32_x4, aarch64_sve_uzpq_x4, 137438953479),
SVEMAP1(svuzpq_f64_x2, aarch64_sve_uzpq_x2, 137438953480),
SVEMAP1(svuzpq_f64_x4, aarch64_sve_uzpq_x4, 137438953480),
SVEMAP1(svuzpq_mf8_x2, aarch64_sve_uzpq_x2, 137438953486),
SVEMAP1(svuzpq_mf8_x4, aarch64_sve_uzpq_x4, 137438953486),
SVEMAP1(svuzpq_s16_x2, aarch64_sve_uzpq_x2, 137438953474),
SVEMAP1(svuzpq_s16_x4, aarch64_sve_uzpq_x4, 137438953474),
SVEMAP1(svuzpq_s32_x2, aarch64_sve_uzpq_x2, 137438953475),
SVEMAP1(svuzpq_s32_x4, aarch64_sve_uzpq_x4, 137438953475),
SVEMAP1(svuzpq_s64_x2, aarch64_sve_uzpq_x2, 137438953476),
SVEMAP1(svuzpq_s64_x4, aarch64_sve_uzpq_x4, 137438953476),
SVEMAP1(svuzpq_s8_x2, aarch64_sve_uzpq_x2, 137438953473),
SVEMAP1(svuzpq_s8_x4, aarch64_sve_uzpq_x4, 137438953473),
SVEMAP1(svuzpq_u16_x2, aarch64_sve_uzpq_x2, 137438953474),
SVEMAP1(svuzpq_u16_x4, aarch64_sve_uzpq_x4, 137438953474),
SVEMAP1(svuzpq_u32_x2, aarch64_sve_uzpq_x2, 137438953475),
SVEMAP1(svuzpq_u32_x4, aarch64_sve_uzpq_x4, 137438953475),
SVEMAP1(svuzpq_u64_x2, aarch64_sve_uzpq_x2, 137438953476),
SVEMAP1(svuzpq_u64_x4, aarch64_sve_uzpq_x4, 137438953476),
SVEMAP1(svuzpq_u8_x2, aarch64_sve_uzpq_x2, 137438953473),
SVEMAP1(svuzpq_u8_x4, aarch64_sve_uzpq_x4, 137438953473),
SVEMAP1(svwhilege_b16_s32, aarch64_sve_whilege, 4398048608266),
SVEMAP1(svwhilege_b16_s64, aarch64_sve_whilege, 4398048608266),
SVEMAP1(svwhilege_b16_s64_x2, aarch64_sve_whilege_x2, 4398046511114),
SVEMAP1(svwhilege_b16_u32, aarch64_sve_whilehs, 4398048608266),
SVEMAP1(svwhilege_b16_u64, aarch64_sve_whilehs, 4398048608266),
SVEMAP1(svwhilege_b16_u64_x2, aarch64_sve_whilehs_x2, 4398046511114),
SVEMAP1(svwhilege_b32_s32, aarch64_sve_whilege, 4398048608267),
SVEMAP1(svwhilege_b32_s64, aarch64_sve_whilege, 4398048608267),
SVEMAP1(svwhilege_b32_s64_x2, aarch64_sve_whilege_x2, 4398046511115),
SVEMAP1(svwhilege_b32_u32, aarch64_sve_whilehs, 4398048608267),
SVEMAP1(svwhilege_b32_u64, aarch64_sve_whilehs, 4398048608267),
SVEMAP1(svwhilege_b32_u64_x2, aarch64_sve_whilehs_x2, 4398046511115),
SVEMAP1(svwhilege_b64_s32, aarch64_sve_whilege, 4398048608268),
SVEMAP1(svwhilege_b64_s64, aarch64_sve_whilege, 4398048608268),
SVEMAP1(svwhilege_b64_s64_x2, aarch64_sve_whilege_x2, 4398046511116),
SVEMAP1(svwhilege_b64_u32, aarch64_sve_whilehs, 4398048608268),
SVEMAP1(svwhilege_b64_u64, aarch64_sve_whilehs, 4398048608268),
SVEMAP1(svwhilege_b64_u64_x2, aarch64_sve_whilehs_x2, 4398046511116),
SVEMAP1(svwhilege_b8_s32, aarch64_sve_whilege, 4398048608265),
SVEMAP1(svwhilege_b8_s64, aarch64_sve_whilege, 4398048608265),
SVEMAP1(svwhilege_b8_s64_x2, aarch64_sve_whilege_x2, 4398046511113),
SVEMAP1(svwhilege_b8_u32, aarch64_sve_whilehs, 4398048608265),
SVEMAP1(svwhilege_b8_u64, aarch64_sve_whilehs, 4398048608265),
SVEMAP1(svwhilege_b8_u64_x2, aarch64_sve_whilehs_x2, 4398046511113),
SVEMAP1(svwhilege_c16_s64, aarch64_sve_whilege_c16, 4398047559690),
SVEMAP1(svwhilege_c16_u64, aarch64_sve_whilehs_c16, 4398047559690),
SVEMAP1(svwhilege_c32_s64, aarch64_sve_whilege_c32, 4398047559691),
SVEMAP1(svwhilege_c32_u64, aarch64_sve_whilehs_c32, 4398047559691),
SVEMAP1(svwhilege_c64_s64, aarch64_sve_whilege_c64, 4398047559692),
SVEMAP1(svwhilege_c64_u64, aarch64_sve_whilehs_c64, 4398047559692),
SVEMAP1(svwhilege_c8_s64, aarch64_sve_whilege_c8, 4398047559689),
SVEMAP1(svwhilege_c8_u64, aarch64_sve_whilehs_c8, 4398047559689),
SVEMAP1(svwhilegt_b16_s32, aarch64_sve_whilegt, 4398048608266),
SVEMAP1(svwhilegt_b16_s64, aarch64_sve_whilegt, 4398048608266),
SVEMAP1(svwhilegt_b16_s64_x2, aarch64_sve_whilegt_x2, 4398046511114),
SVEMAP1(svwhilegt_b16_u32, aarch64_sve_whilehi, 4398048608266),
SVEMAP1(svwhilegt_b16_u64, aarch64_sve_whilehi, 4398048608266),
SVEMAP1(svwhilegt_b16_u64_x2, aarch64_sve_whilehi_x2, 4398046511114),
SVEMAP1(svwhilegt_b32_s32, aarch64_sve_whilegt, 4398048608267),
SVEMAP1(svwhilegt_b32_s64, aarch64_sve_whilegt, 4398048608267),
SVEMAP1(svwhilegt_b32_s64_x2, aarch64_sve_whilegt_x2, 4398046511115),
SVEMAP1(svwhilegt_b32_u32, aarch64_sve_whilehi, 4398048608267),
SVEMAP1(svwhilegt_b32_u64, aarch64_sve_whilehi, 4398048608267),
SVEMAP1(svwhilegt_b32_u64_x2, aarch64_sve_whilehi_x2, 4398046511115),
SVEMAP1(svwhilegt_b64_s32, aarch64_sve_whilegt, 4398048608268),
SVEMAP1(svwhilegt_b64_s64, aarch64_sve_whilegt, 4398048608268),
SVEMAP1(svwhilegt_b64_s64_x2, aarch64_sve_whilegt_x2, 4398046511116),
SVEMAP1(svwhilegt_b64_u32, aarch64_sve_whilehi, 4398048608268),
SVEMAP1(svwhilegt_b64_u64, aarch64_sve_whilehi, 4398048608268),
SVEMAP1(svwhilegt_b64_u64_x2, aarch64_sve_whilehi_x2, 4398046511116),
SVEMAP1(svwhilegt_b8_s32, aarch64_sve_whilegt, 4398048608265),
SVEMAP1(svwhilegt_b8_s64, aarch64_sve_whilegt, 4398048608265),
SVEMAP1(svwhilegt_b8_s64_x2, aarch64_sve_whilegt_x2, 4398046511113),
SVEMAP1(svwhilegt_b8_u32, aarch64_sve_whilehi, 4398048608265),
SVEMAP1(svwhilegt_b8_u64, aarch64_sve_whilehi, 4398048608265),
SVEMAP1(svwhilegt_b8_u64_x2, aarch64_sve_whilehi_x2, 4398046511113),
SVEMAP1(svwhilegt_c16_s64, aarch64_sve_whilegt_c16, 4398047559690),
SVEMAP1(svwhilegt_c16_u64, aarch64_sve_whilehi_c16, 4398047559690),
SVEMAP1(svwhilegt_c32_s64, aarch64_sve_whilegt_c32, 4398047559691),
SVEMAP1(svwhilegt_c32_u64, aarch64_sve_whilehi_c32, 4398047559691),
SVEMAP1(svwhilegt_c64_s64, aarch64_sve_whilegt_c64, 4398047559692),
SVEMAP1(svwhilegt_c64_u64, aarch64_sve_whilehi_c64, 4398047559692),
SVEMAP1(svwhilegt_c8_s64, aarch64_sve_whilegt_c8, 4398047559689),
SVEMAP1(svwhilegt_c8_u64, aarch64_sve_whilehi_c8, 4398047559689),
SVEMAP1(svwhilele_b16_s32, aarch64_sve_whilele, 4398048608266),
SVEMAP1(svwhilele_b16_s64, aarch64_sve_whilele, 4398048608266),
SVEMAP1(svwhilele_b16_s64_x2, aarch64_sve_whilele_x2, 4398046511114),
SVEMAP1(svwhilele_b16_u32, aarch64_sve_whilels, 4398048608266),
SVEMAP1(svwhilele_b16_u64, aarch64_sve_whilels, 4398048608266),
SVEMAP1(svwhilele_b16_u64_x2, aarch64_sve_whilels_x2, 4398046511114),
SVEMAP1(svwhilele_b32_s32, aarch64_sve_whilele, 4398048608267),
SVEMAP1(svwhilele_b32_s64, aarch64_sve_whilele, 4398048608267),
SVEMAP1(svwhilele_b32_s64_x2, aarch64_sve_whilele_x2, 4398046511115),
SVEMAP1(svwhilele_b32_u32, aarch64_sve_whilels, 4398048608267),
SVEMAP1(svwhilele_b32_u64, aarch64_sve_whilels, 4398048608267),
SVEMAP1(svwhilele_b32_u64_x2, aarch64_sve_whilels_x2, 4398046511115),
SVEMAP1(svwhilele_b64_s32, aarch64_sve_whilele, 4398048608268),
SVEMAP1(svwhilele_b64_s64, aarch64_sve_whilele, 4398048608268),
SVEMAP1(svwhilele_b64_s64_x2, aarch64_sve_whilele_x2, 4398046511116),
SVEMAP1(svwhilele_b64_u32, aarch64_sve_whilels, 4398048608268),
SVEMAP1(svwhilele_b64_u64, aarch64_sve_whilels, 4398048608268),
SVEMAP1(svwhilele_b64_u64_x2, aarch64_sve_whilels_x2, 4398046511116),
SVEMAP1(svwhilele_b8_s32, aarch64_sve_whilele, 4398048608265),
SVEMAP1(svwhilele_b8_s64, aarch64_sve_whilele, 4398048608265),
SVEMAP1(svwhilele_b8_s64_x2, aarch64_sve_whilele_x2, 4398046511113),
SVEMAP1(svwhilele_b8_u32, aarch64_sve_whilels, 4398048608265),
SVEMAP1(svwhilele_b8_u64, aarch64_sve_whilels, 4398048608265),
SVEMAP1(svwhilele_b8_u64_x2, aarch64_sve_whilels_x2, 4398046511113),
SVEMAP1(svwhilele_c16_s64, aarch64_sve_whilele_c16, 4398047559690),
SVEMAP1(svwhilele_c16_u64, aarch64_sve_whilels_c16, 4398047559690),
SVEMAP1(svwhilele_c32_s64, aarch64_sve_whilele_c32, 4398047559691),
SVEMAP1(svwhilele_c32_u64, aarch64_sve_whilels_c32, 4398047559691),
SVEMAP1(svwhilele_c64_s64, aarch64_sve_whilele_c64, 4398047559692),
SVEMAP1(svwhilele_c64_u64, aarch64_sve_whilels_c64, 4398047559692),
SVEMAP1(svwhilele_c8_s64, aarch64_sve_whilele_c8, 4398047559689),
SVEMAP1(svwhilele_c8_u64, aarch64_sve_whilels_c8, 4398047559689),
SVEMAP1(svwhilelt_b16_s32, aarch64_sve_whilelt, 4398048608266),
SVEMAP1(svwhilelt_b16_s64, aarch64_sve_whilelt, 4398048608266),
SVEMAP1(svwhilelt_b16_s64_x2, aarch64_sve_whilelt_x2, 4398046511114),
SVEMAP1(svwhilelt_b16_u32, aarch64_sve_whilelo, 4398048608266),
SVEMAP1(svwhilelt_b16_u64, aarch64_sve_whilelo, 4398048608266),
SVEMAP1(svwhilelt_b16_u64_x2, aarch64_sve_whilelo_x2, 4398046511114),
SVEMAP1(svwhilelt_b32_s32, aarch64_sve_whilelt, 4398048608267),
SVEMAP1(svwhilelt_b32_s64, aarch64_sve_whilelt, 4398048608267),
SVEMAP1(svwhilelt_b32_s64_x2, aarch64_sve_whilelt_x2, 4398046511115),
SVEMAP1(svwhilelt_b32_u32, aarch64_sve_whilelo, 4398048608267),
SVEMAP1(svwhilelt_b32_u64, aarch64_sve_whilelo, 4398048608267),
SVEMAP1(svwhilelt_b32_u64_x2, aarch64_sve_whilelo_x2, 4398046511115),
SVEMAP1(svwhilelt_b64_s32, aarch64_sve_whilelt, 4398048608268),
SVEMAP1(svwhilelt_b64_s64, aarch64_sve_whilelt, 4398048608268),
SVEMAP1(svwhilelt_b64_s64_x2, aarch64_sve_whilelt_x2, 4398046511116),
SVEMAP1(svwhilelt_b64_u32, aarch64_sve_whilelo, 4398048608268),
SVEMAP1(svwhilelt_b64_u64, aarch64_sve_whilelo, 4398048608268),
SVEMAP1(svwhilelt_b64_u64_x2, aarch64_sve_whilelo_x2, 4398046511116),
SVEMAP1(svwhilelt_b8_s32, aarch64_sve_whilelt, 4398048608265),
SVEMAP1(svwhilelt_b8_s64, aarch64_sve_whilelt, 4398048608265),
SVEMAP1(svwhilelt_b8_s64_x2, aarch64_sve_whilelt_x2, 4398046511113),
SVEMAP1(svwhilelt_b8_u32, aarch64_sve_whilelo, 4398048608265),
SVEMAP1(svwhilelt_b8_u64, aarch64_sve_whilelo, 4398048608265),
SVEMAP1(svwhilelt_b8_u64_x2, aarch64_sve_whilelo_x2, 4398046511113),
SVEMAP1(svwhilelt_c16_s64, aarch64_sve_whilelt_c16, 4398047559690),
SVEMAP1(svwhilelt_c16_u64, aarch64_sve_whilelo_c16, 4398047559690),
SVEMAP1(svwhilelt_c32_s64, aarch64_sve_whilelt_c32, 4398047559691),
SVEMAP1(svwhilelt_c32_u64, aarch64_sve_whilelo_c32, 4398047559691),
SVEMAP1(svwhilelt_c64_s64, aarch64_sve_whilelt_c64, 4398047559692),
SVEMAP1(svwhilelt_c64_u64, aarch64_sve_whilelo_c64, 4398047559692),
SVEMAP1(svwhilelt_c8_s64, aarch64_sve_whilelt_c8, 4398047559689),
SVEMAP1(svwhilelt_c8_u64, aarch64_sve_whilelo_c8, 4398047559689),
SVEMAP1(svwhilerw_bf16, aarch64_sve_whilerw_h, 4398050705421),
SVEMAP1(svwhilerw_f16, aarch64_sve_whilerw_h, 4398050705414),
SVEMAP1(svwhilerw_f32, aarch64_sve_whilerw_s, 4398050705415),
SVEMAP1(svwhilerw_f64, aarch64_sve_whilerw_d, 4398050705416),
SVEMAP1(svwhilerw_s16, aarch64_sve_whilerw_h, 4398050705410),
SVEMAP1(svwhilerw_s32, aarch64_sve_whilerw_s, 4398050705411),
SVEMAP1(svwhilerw_s64, aarch64_sve_whilerw_d, 4398050705412),
SVEMAP1(svwhilerw_s8, aarch64_sve_whilerw_b, 4398050705409),
SVEMAP1(svwhilerw_u16, aarch64_sve_whilerw_h, 4398050705410),
SVEMAP1(svwhilerw_u32, aarch64_sve_whilerw_s, 4398050705411),
SVEMAP1(svwhilerw_u64, aarch64_sve_whilerw_d, 4398050705412),
SVEMAP1(svwhilerw_u8, aarch64_sve_whilerw_b, 4398050705409),
SVEMAP1(svwhilewr_bf16, aarch64_sve_whilewr_h, 4398050705421),
SVEMAP1(svwhilewr_f16, aarch64_sve_whilewr_h, 4398050705414),
SVEMAP1(svwhilewr_f32, aarch64_sve_whilewr_s, 4398050705415),
SVEMAP1(svwhilewr_f64, aarch64_sve_whilewr_d, 4398050705416),
SVEMAP1(svwhilewr_s16, aarch64_sve_whilewr_h, 4398050705410),
SVEMAP1(svwhilewr_s32, aarch64_sve_whilewr_s, 4398050705411),
SVEMAP1(svwhilewr_s64, aarch64_sve_whilewr_d, 4398050705412),
SVEMAP1(svwhilewr_s8, aarch64_sve_whilewr_b, 4398050705409),
SVEMAP1(svwhilewr_u16, aarch64_sve_whilewr_h, 4398050705410),
SVEMAP1(svwhilewr_u32, aarch64_sve_whilewr_s, 4398050705411),
SVEMAP1(svwhilewr_u64, aarch64_sve_whilewr_d, 4398050705412),
SVEMAP1(svwhilewr_u8, aarch64_sve_whilewr_b, 4398050705409),
SVEMAP2(svwrffr, 1048585),
SVEMAP1(svxar_n_s16, aarch64_sve_xar, 4398046511106),
SVEMAP1(svxar_n_s32, aarch64_sve_xar, 4398046511107),
SVEMAP1(svxar_n_s64, aarch64_sve_xar, 4398046511108),
SVEMAP1(svxar_n_s8, aarch64_sve_xar, 4398046511105),
SVEMAP1(svxar_n_u16, aarch64_sve_xar, 4398046511106),
SVEMAP1(svxar_n_u32, aarch64_sve_xar, 4398046511107),
SVEMAP1(svxar_n_u64, aarch64_sve_xar, 4398046511108),
SVEMAP1(svxar_n_u8, aarch64_sve_xar, 4398046511105),
SVEMAP1(svzip1_b16, aarch64_sve_zip1_b16, 4398047559689),
SVEMAP1(svzip1_b32, aarch64_sve_zip1_b32, 4398047559689),
SVEMAP1(svzip1_b64, aarch64_sve_zip1_b64, 4398047559689),
SVEMAP1(svzip1_b8, aarch64_sve_zip1, 4398046511113),
SVEMAP1(svzip1_bf16, aarch64_sve_zip1, 4398046511117),
SVEMAP1(svzip1_f16, aarch64_sve_zip1, 4398046511110),
SVEMAP1(svzip1_f32, aarch64_sve_zip1, 4398046511111),
SVEMAP1(svzip1_f64, aarch64_sve_zip1, 4398046511112),
SVEMAP1(svzip1_s16, aarch64_sve_zip1, 4398046511106),
SVEMAP1(svzip1_s32, aarch64_sve_zip1, 4398046511107),
SVEMAP1(svzip1_s64, aarch64_sve_zip1, 4398046511108),
SVEMAP1(svzip1_s8, aarch64_sve_zip1, 4398046511105),
SVEMAP1(svzip1_u16, aarch64_sve_zip1, 4398046511106),
SVEMAP1(svzip1_u32, aarch64_sve_zip1, 4398046511107),
SVEMAP1(svzip1_u64, aarch64_sve_zip1, 4398046511108),
SVEMAP1(svzip1_u8, aarch64_sve_zip1, 4398046511105),
SVEMAP1(svzip1q_bf16, aarch64_sve_zip1q, 13),
SVEMAP1(svzip1q_f16, aarch64_sve_zip1q, 6),
SVEMAP1(svzip1q_f32, aarch64_sve_zip1q, 7),
SVEMAP1(svzip1q_f64, aarch64_sve_zip1q, 8),
SVEMAP1(svzip1q_s16, aarch64_sve_zip1q, 2),
SVEMAP1(svzip1q_s32, aarch64_sve_zip1q, 3),
SVEMAP1(svzip1q_s64, aarch64_sve_zip1q, 4),
SVEMAP1(svzip1q_s8, aarch64_sve_zip1q, 1),
SVEMAP1(svzip1q_u16, aarch64_sve_zip1q, 2),
SVEMAP1(svzip1q_u32, aarch64_sve_zip1q, 3),
SVEMAP1(svzip1q_u64, aarch64_sve_zip1q, 4),
SVEMAP1(svzip1q_u8, aarch64_sve_zip1q, 1),
SVEMAP1(svzip2_b16, aarch64_sve_zip2_b16, 4398047559689),
SVEMAP1(svzip2_b32, aarch64_sve_zip2_b32, 4398047559689),
SVEMAP1(svzip2_b64, aarch64_sve_zip2_b64, 4398047559689),
SVEMAP1(svzip2_b8, aarch64_sve_zip2, 4398046511113),
SVEMAP1(svzip2_bf16, aarch64_sve_zip2, 4398046511117),
SVEMAP1(svzip2_f16, aarch64_sve_zip2, 4398046511110),
SVEMAP1(svzip2_f32, aarch64_sve_zip2, 4398046511111),
SVEMAP1(svzip2_f64, aarch64_sve_zip2, 4398046511112),
SVEMAP1(svzip2_s16, aarch64_sve_zip2, 4398046511106),
SVEMAP1(svzip2_s32, aarch64_sve_zip2, 4398046511107),
SVEMAP1(svzip2_s64, aarch64_sve_zip2, 4398046511108),
SVEMAP1(svzip2_s8, aarch64_sve_zip2, 4398046511105),
SVEMAP1(svzip2_u16, aarch64_sve_zip2, 4398046511106),
SVEMAP1(svzip2_u32, aarch64_sve_zip2, 4398046511107),
SVEMAP1(svzip2_u64, aarch64_sve_zip2, 4398046511108),
SVEMAP1(svzip2_u8, aarch64_sve_zip2, 4398046511105),
SVEMAP1(svzip2q_bf16, aarch64_sve_zip2q, 13),
SVEMAP1(svzip2q_f16, aarch64_sve_zip2q, 6),
SVEMAP1(svzip2q_f32, aarch64_sve_zip2q, 7),
SVEMAP1(svzip2q_f64, aarch64_sve_zip2q, 8),
SVEMAP1(svzip2q_s16, aarch64_sve_zip2q, 2),
SVEMAP1(svzip2q_s32, aarch64_sve_zip2q, 3),
SVEMAP1(svzip2q_s64, aarch64_sve_zip2q, 4),
SVEMAP1(svzip2q_s8, aarch64_sve_zip2q, 1),
SVEMAP1(svzip2q_u16, aarch64_sve_zip2q, 2),
SVEMAP1(svzip2q_u32, aarch64_sve_zip2q, 3),
SVEMAP1(svzip2q_u64, aarch64_sve_zip2q, 4),
SVEMAP1(svzip2q_u8, aarch64_sve_zip2q, 1),
SVEMAP1(svzip_bf16_x2, aarch64_sve_zip_x2, 137438953485),
SVEMAP1(svzip_bf16_x4, aarch64_sve_zip_x4, 137438953485),
SVEMAP1(svzip_f16_x2, aarch64_sve_zip_x2, 137438953478),
SVEMAP1(svzip_f16_x4, aarch64_sve_zip_x4, 137438953478),
SVEMAP1(svzip_f32_x2, aarch64_sve_zip_x2, 137438953479),
SVEMAP1(svzip_f32_x4, aarch64_sve_zip_x4, 137438953479),
SVEMAP1(svzip_f64_x2, aarch64_sve_zip_x2, 137438953480),
SVEMAP1(svzip_f64_x4, aarch64_sve_zip_x4, 137438953480),
SVEMAP1(svzip_mf8_x2, aarch64_sve_zip_x2, 137438953486),
SVEMAP1(svzip_mf8_x4, aarch64_sve_zip_x4, 137438953486),
SVEMAP1(svzip_s16_x2, aarch64_sve_zip_x2, 137438953474),
SVEMAP1(svzip_s16_x4, aarch64_sve_zip_x4, 137438953474),
SVEMAP1(svzip_s32_x2, aarch64_sve_zip_x2, 137438953475),
SVEMAP1(svzip_s32_x4, aarch64_sve_zip_x4, 137438953475),
SVEMAP1(svzip_s64_x2, aarch64_sve_zip_x2, 137438953476),
SVEMAP1(svzip_s64_x4, aarch64_sve_zip_x4, 137438953476),
SVEMAP1(svzip_s8_x2, aarch64_sve_zip_x2, 137438953473),
SVEMAP1(svzip_s8_x4, aarch64_sve_zip_x4, 137438953473),
SVEMAP1(svzip_u16_x2, aarch64_sve_zip_x2, 137438953474),
SVEMAP1(svzip_u16_x4, aarch64_sve_zip_x4, 137438953474),
SVEMAP1(svzip_u32_x2, aarch64_sve_zip_x2, 137438953475),
SVEMAP1(svzip_u32_x4, aarch64_sve_zip_x4, 137438953475),
SVEMAP1(svzip_u64_x2, aarch64_sve_zip_x2, 137438953476),
SVEMAP1(svzip_u64_x4, aarch64_sve_zip_x4, 137438953476),
SVEMAP1(svzip_u8_x2, aarch64_sve_zip_x2, 137438953473),
SVEMAP1(svzip_u8_x4, aarch64_sve_zip_x4, 137438953473),
SVEMAP1(svzipq1_bf16, aarch64_sve_zipq1, 4398046511117),
SVEMAP1(svzipq1_f16, aarch64_sve_zipq1, 4398046511110),
SVEMAP1(svzipq1_f32, aarch64_sve_zipq1, 4398046511111),
SVEMAP1(svzipq1_f64, aarch64_sve_zipq1, 4398046511112),
SVEMAP1(svzipq1_mf8, aarch64_sve_zipq1, 4398046511118),
SVEMAP1(svzipq1_s16, aarch64_sve_zipq1, 4398046511106),
SVEMAP1(svzipq1_s32, aarch64_sve_zipq1, 4398046511107),
SVEMAP1(svzipq1_s64, aarch64_sve_zipq1, 4398046511108),
SVEMAP1(svzipq1_s8, aarch64_sve_zipq1, 4398046511105),
SVEMAP1(svzipq1_u16, aarch64_sve_zipq1, 4398046511106),
SVEMAP1(svzipq1_u32, aarch64_sve_zipq1, 4398046511107),
SVEMAP1(svzipq1_u64, aarch64_sve_zipq1, 4398046511108),
SVEMAP1(svzipq1_u8, aarch64_sve_zipq1, 4398046511105),
SVEMAP1(svzipq2_bf16, aarch64_sve_zipq2, 4398046511117),
SVEMAP1(svzipq2_f16, aarch64_sve_zipq2, 4398046511110),
SVEMAP1(svzipq2_f32, aarch64_sve_zipq2, 4398046511111),
SVEMAP1(svzipq2_f64, aarch64_sve_zipq2, 4398046511112),
SVEMAP1(svzipq2_mf8, aarch64_sve_zipq2, 4398046511118),
SVEMAP1(svzipq2_s16, aarch64_sve_zipq2, 4398046511106),
SVEMAP1(svzipq2_s32, aarch64_sve_zipq2, 4398046511107),
SVEMAP1(svzipq2_s64, aarch64_sve_zipq2, 4398046511108),
SVEMAP1(svzipq2_s8, aarch64_sve_zipq2, 4398046511105),
SVEMAP1(svzipq2_u16, aarch64_sve_zipq2, 4398046511106),
SVEMAP1(svzipq2_u32, aarch64_sve_zipq2, 4398046511107),
SVEMAP1(svzipq2_u64, aarch64_sve_zipq2, 4398046511108),
SVEMAP1(svzipq2_u8, aarch64_sve_zipq2, 4398046511105),
SVEMAP1(svzipq_bf16_x2, aarch64_sve_zipq_x2, 137438953485),
SVEMAP1(svzipq_bf16_x4, aarch64_sve_zipq_x4, 137438953485),
SVEMAP1(svzipq_f16_x2, aarch64_sve_zipq_x2, 137438953478),
SVEMAP1(svzipq_f16_x4, aarch64_sve_zipq_x4, 137438953478),
SVEMAP1(svzipq_f32_x2, aarch64_sve_zipq_x2, 137438953479),
SVEMAP1(svzipq_f32_x4, aarch64_sve_zipq_x4, 137438953479),
SVEMAP1(svzipq_f64_x2, aarch64_sve_zipq_x2, 137438953480),
SVEMAP1(svzipq_f64_x4, aarch64_sve_zipq_x4, 137438953480),
SVEMAP1(svzipq_mf8_x2, aarch64_sve_zipq_x2, 137438953486),
SVEMAP1(svzipq_mf8_x4, aarch64_sve_zipq_x4, 137438953486),
SVEMAP1(svzipq_s16_x2, aarch64_sve_zipq_x2, 137438953474),
SVEMAP1(svzipq_s16_x4, aarch64_sve_zipq_x4, 137438953474),
SVEMAP1(svzipq_s32_x2, aarch64_sve_zipq_x2, 137438953475),
SVEMAP1(svzipq_s32_x4, aarch64_sve_zipq_x4, 137438953475),
SVEMAP1(svzipq_s64_x2, aarch64_sve_zipq_x2, 137438953476),
SVEMAP1(svzipq_s64_x4, aarch64_sve_zipq_x4, 137438953476),
SVEMAP1(svzipq_s8_x2, aarch64_sve_zipq_x2, 137438953473),
SVEMAP1(svzipq_s8_x4, aarch64_sve_zipq_x4, 137438953473),
SVEMAP1(svzipq_u16_x2, aarch64_sve_zipq_x2, 137438953474),
SVEMAP1(svzipq_u16_x4, aarch64_sve_zipq_x4, 137438953474),
SVEMAP1(svzipq_u32_x2, aarch64_sve_zipq_x2, 137438953475),
SVEMAP1(svzipq_u32_x4, aarch64_sve_zipq_x4, 137438953475),
SVEMAP1(svzipq_u64_x2, aarch64_sve_zipq_x2, 137438953476),
SVEMAP1(svzipq_u64_x4, aarch64_sve_zipq_x4, 137438953476),
SVEMAP1(svzipq_u8_x2, aarch64_sve_zipq_x2, 137438953473),
SVEMAP1(svzipq_u8_x4, aarch64_sve_zipq_x4, 137438953473),
#endif

