/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|* List of builtins that Clang recognizes                                     *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/


#ifdef GET_BUILTIN_ENUMERATORS
    BI__bmma_m8n8k128_ld_a_b1,
    BI__bmma_m8n8k128_ld_b_b1,
    BI__bmma_m8n8k128_ld_c,
    BI__bmma_m8n8k128_mma_and_popc_b1,
    BI__bmma_m8n8k128_mma_xor_popc_b1,
    BI__bmma_m8n8k128_st_c_i32,
    BI__builtin_ptx_get_image_channel_data_typei_,
    BI__builtin_ptx_get_image_channel_orderi_,
    BI__builtin_ptx_get_image_depthi_,
    BI__builtin_ptx_get_image_heighti_,
    BI__builtin_ptx_get_image_widthi_,
    BI__builtin_ptx_read_image2Dff_,
    BI__builtin_ptx_read_image2Dfi_,
    BI__builtin_ptx_read_image2Dif_,
    BI__builtin_ptx_read_image2Dii_,
    BI__builtin_ptx_read_image3Dff_,
    BI__builtin_ptx_read_image3Dfi_,
    BI__builtin_ptx_read_image3Dif_,
    BI__builtin_ptx_read_image3Dii_,
    BI__builtin_ptx_write_image2Df_,
    BI__builtin_ptx_write_image2Di_,
    BI__builtin_ptx_write_image2Dui_,
    BI__dmma_m8n8k4_ld_a,
    BI__dmma_m8n8k4_ld_b,
    BI__dmma_m8n8k4_ld_c,
    BI__dmma_m8n8k4_mma_f64,
    BI__dmma_m8n8k4_st_c_f64,
    BI__hmma_m8n32k16_ld_a,
    BI__hmma_m8n32k16_ld_b,
    BI__hmma_m8n32k16_ld_c_f16,
    BI__hmma_m8n32k16_ld_c_f32,
    BI__hmma_m8n32k16_mma_f16f16,
    BI__hmma_m8n32k16_mma_f16f32,
    BI__hmma_m8n32k16_mma_f32f16,
    BI__hmma_m8n32k16_mma_f32f32,
    BI__hmma_m8n32k16_st_c_f16,
    BI__hmma_m8n32k16_st_c_f32,
    BI__hmma_m16n16k16_ld_a,
    BI__hmma_m16n16k16_ld_b,
    BI__hmma_m16n16k16_ld_c_f16,
    BI__hmma_m16n16k16_ld_c_f32,
    BI__hmma_m16n16k16_mma_f16f16,
    BI__hmma_m16n16k16_mma_f16f32,
    BI__hmma_m16n16k16_mma_f32f16,
    BI__hmma_m16n16k16_mma_f32f32,
    BI__hmma_m16n16k16_st_c_f16,
    BI__hmma_m16n16k16_st_c_f32,
    BI__hmma_m32n8k16_ld_a,
    BI__hmma_m32n8k16_ld_b,
    BI__hmma_m32n8k16_ld_c_f16,
    BI__hmma_m32n8k16_ld_c_f32,
    BI__hmma_m32n8k16_mma_f16f16,
    BI__hmma_m32n8k16_mma_f16f32,
    BI__hmma_m32n8k16_mma_f32f16,
    BI__hmma_m32n8k16_mma_f32f32,
    BI__hmma_m32n8k16_st_c_f16,
    BI__hmma_m32n8k16_st_c_f32,
    BI__imma_m8n8k32_ld_a_s4,
    BI__imma_m8n8k32_ld_a_u4,
    BI__imma_m8n8k32_ld_b_s4,
    BI__imma_m8n8k32_ld_b_u4,
    BI__imma_m8n8k32_ld_c,
    BI__imma_m8n8k32_mma_s4,
    BI__imma_m8n8k32_mma_u4,
    BI__imma_m8n8k32_st_c_i32,
    BI__imma_m8n32k16_ld_a_s8,
    BI__imma_m8n32k16_ld_a_u8,
    BI__imma_m8n32k16_ld_b_s8,
    BI__imma_m8n32k16_ld_b_u8,
    BI__imma_m8n32k16_ld_c,
    BI__imma_m8n32k16_mma_s8,
    BI__imma_m8n32k16_mma_u8,
    BI__imma_m8n32k16_st_c_i32,
    BI__imma_m16n16k16_ld_a_s8,
    BI__imma_m16n16k16_ld_a_u8,
    BI__imma_m16n16k16_ld_b_s8,
    BI__imma_m16n16k16_ld_b_u8,
    BI__imma_m16n16k16_ld_c,
    BI__imma_m16n16k16_mma_s8,
    BI__imma_m16n16k16_mma_u8,
    BI__imma_m16n16k16_st_c_i32,
    BI__imma_m32n8k16_ld_a_s8,
    BI__imma_m32n8k16_ld_a_u8,
    BI__imma_m32n8k16_ld_b_s8,
    BI__imma_m32n8k16_ld_b_u8,
    BI__imma_m32n8k16_ld_c,
    BI__imma_m32n8k16_mma_s8,
    BI__imma_m32n8k16_mma_u8,
    BI__imma_m32n8k16_st_c_i32,
    BI__mma_bf16_m8n32k16_ld_a,
    BI__mma_bf16_m8n32k16_ld_b,
    BI__mma_bf16_m8n32k16_mma_f32,
    BI__mma_bf16_m16n16k16_ld_a,
    BI__mma_bf16_m16n16k16_ld_b,
    BI__mma_bf16_m16n16k16_mma_f32,
    BI__mma_bf16_m32n8k16_ld_a,
    BI__mma_bf16_m32n8k16_ld_b,
    BI__mma_bf16_m32n8k16_mma_f32,
    BI__mma_m16n16k8_st_c_f32,
    BI__mma_tf32_m16n16k8_ld_a,
    BI__mma_tf32_m16n16k8_ld_b,
    BI__mma_tf32_m16n16k8_ld_c,
    BI__mma_tf32_m16n16k8_mma_f32,
    BI__nvvm_abs_bf16,
    BI__nvvm_abs_bf16x2,
    BI__nvvm_activemask,
    BI__nvvm_add_rm_d,
    BI__nvvm_add_rm_f,
    BI__nvvm_add_rm_ftz_f,
    BI__nvvm_add_rn_d,
    BI__nvvm_add_rn_f,
    BI__nvvm_add_rn_ftz_f,
    BI__nvvm_add_rp_d,
    BI__nvvm_add_rp_f,
    BI__nvvm_add_rp_ftz_f,
    BI__nvvm_add_rz_d,
    BI__nvvm_add_rz_f,
    BI__nvvm_add_rz_ftz_f,
    BI__nvvm_atom_add_gen_d,
    BI__nvvm_atom_add_gen_f,
    BI__nvvm_atom_add_gen_i,
    BI__nvvm_atom_add_gen_l,
    BI__nvvm_atom_add_gen_ll,
    BI__nvvm_atom_and_gen_i,
    BI__nvvm_atom_and_gen_l,
    BI__nvvm_atom_and_gen_ll,
    BI__nvvm_atom_cas_gen_i,
    BI__nvvm_atom_cas_gen_l,
    BI__nvvm_atom_cas_gen_ll,
    BI__nvvm_atom_cas_gen_us,
    BI__nvvm_atom_cta_add_gen_d,
    BI__nvvm_atom_cta_add_gen_f,
    BI__nvvm_atom_cta_add_gen_i,
    BI__nvvm_atom_cta_add_gen_l,
    BI__nvvm_atom_cta_add_gen_ll,
    BI__nvvm_atom_cta_and_gen_i,
    BI__nvvm_atom_cta_and_gen_l,
    BI__nvvm_atom_cta_and_gen_ll,
    BI__nvvm_atom_cta_cas_gen_i,
    BI__nvvm_atom_cta_cas_gen_l,
    BI__nvvm_atom_cta_cas_gen_ll,
    BI__nvvm_atom_cta_cas_gen_us,
    BI__nvvm_atom_cta_dec_gen_ui,
    BI__nvvm_atom_cta_inc_gen_ui,
    BI__nvvm_atom_cta_max_gen_i,
    BI__nvvm_atom_cta_max_gen_l,
    BI__nvvm_atom_cta_max_gen_ll,
    BI__nvvm_atom_cta_max_gen_ui,
    BI__nvvm_atom_cta_max_gen_ul,
    BI__nvvm_atom_cta_max_gen_ull,
    BI__nvvm_atom_cta_min_gen_i,
    BI__nvvm_atom_cta_min_gen_l,
    BI__nvvm_atom_cta_min_gen_ll,
    BI__nvvm_atom_cta_min_gen_ui,
    BI__nvvm_atom_cta_min_gen_ul,
    BI__nvvm_atom_cta_min_gen_ull,
    BI__nvvm_atom_cta_or_gen_i,
    BI__nvvm_atom_cta_or_gen_l,
    BI__nvvm_atom_cta_or_gen_ll,
    BI__nvvm_atom_cta_xchg_gen_i,
    BI__nvvm_atom_cta_xchg_gen_l,
    BI__nvvm_atom_cta_xchg_gen_ll,
    BI__nvvm_atom_cta_xor_gen_i,
    BI__nvvm_atom_cta_xor_gen_l,
    BI__nvvm_atom_cta_xor_gen_ll,
    BI__nvvm_atom_dec_gen_ui,
    BI__nvvm_atom_inc_gen_ui,
    BI__nvvm_atom_max_gen_i,
    BI__nvvm_atom_max_gen_l,
    BI__nvvm_atom_max_gen_ll,
    BI__nvvm_atom_max_gen_ui,
    BI__nvvm_atom_max_gen_ul,
    BI__nvvm_atom_max_gen_ull,
    BI__nvvm_atom_min_gen_i,
    BI__nvvm_atom_min_gen_l,
    BI__nvvm_atom_min_gen_ll,
    BI__nvvm_atom_min_gen_ui,
    BI__nvvm_atom_min_gen_ul,
    BI__nvvm_atom_min_gen_ull,
    BI__nvvm_atom_or_gen_i,
    BI__nvvm_atom_or_gen_l,
    BI__nvvm_atom_or_gen_ll,
    BI__nvvm_atom_sub_gen_i,
    BI__nvvm_atom_sub_gen_l,
    BI__nvvm_atom_sub_gen_ll,
    BI__nvvm_atom_sys_add_gen_d,
    BI__nvvm_atom_sys_add_gen_f,
    BI__nvvm_atom_sys_add_gen_i,
    BI__nvvm_atom_sys_add_gen_l,
    BI__nvvm_atom_sys_add_gen_ll,
    BI__nvvm_atom_sys_and_gen_i,
    BI__nvvm_atom_sys_and_gen_l,
    BI__nvvm_atom_sys_and_gen_ll,
    BI__nvvm_atom_sys_cas_gen_i,
    BI__nvvm_atom_sys_cas_gen_l,
    BI__nvvm_atom_sys_cas_gen_ll,
    BI__nvvm_atom_sys_cas_gen_us,
    BI__nvvm_atom_sys_dec_gen_ui,
    BI__nvvm_atom_sys_inc_gen_ui,
    BI__nvvm_atom_sys_max_gen_i,
    BI__nvvm_atom_sys_max_gen_l,
    BI__nvvm_atom_sys_max_gen_ll,
    BI__nvvm_atom_sys_max_gen_ui,
    BI__nvvm_atom_sys_max_gen_ul,
    BI__nvvm_atom_sys_max_gen_ull,
    BI__nvvm_atom_sys_min_gen_i,
    BI__nvvm_atom_sys_min_gen_l,
    BI__nvvm_atom_sys_min_gen_ll,
    BI__nvvm_atom_sys_min_gen_ui,
    BI__nvvm_atom_sys_min_gen_ul,
    BI__nvvm_atom_sys_min_gen_ull,
    BI__nvvm_atom_sys_or_gen_i,
    BI__nvvm_atom_sys_or_gen_l,
    BI__nvvm_atom_sys_or_gen_ll,
    BI__nvvm_atom_sys_xchg_gen_i,
    BI__nvvm_atom_sys_xchg_gen_l,
    BI__nvvm_atom_sys_xchg_gen_ll,
    BI__nvvm_atom_sys_xor_gen_i,
    BI__nvvm_atom_sys_xor_gen_l,
    BI__nvvm_atom_sys_xor_gen_ll,
    BI__nvvm_atom_xchg_gen_i,
    BI__nvvm_atom_xchg_gen_l,
    BI__nvvm_atom_xchg_gen_ll,
    BI__nvvm_atom_xor_gen_i,
    BI__nvvm_atom_xor_gen_l,
    BI__nvvm_atom_xor_gen_ll,
    BI__nvvm_bar0_and,
    BI__nvvm_bar0_or,
    BI__nvvm_bar0_popc,
    BI__nvvm_bar_sync,
    BI__nvvm_bar_warp_sync,
    BI__nvvm_barrier_cluster_arrive,
    BI__nvvm_barrier_cluster_arrive_relaxed,
    BI__nvvm_barrier_cluster_wait,
    BI__nvvm_barrier_sync,
    BI__nvvm_barrier_sync_cnt,
    BI__nvvm_bf16x2_to_ue8m0x2_rp,
    BI__nvvm_bf16x2_to_ue8m0x2_rp_satfinite,
    BI__nvvm_bf16x2_to_ue8m0x2_rz,
    BI__nvvm_bf16x2_to_ue8m0x2_rz_satfinite,
    BI__nvvm_ceil_d,
    BI__nvvm_ceil_f,
    BI__nvvm_ceil_ftz_f,
    BI__nvvm_compiler_error,
    BI__nvvm_compiler_warn,
    BI__nvvm_cos_approx_f,
    BI__nvvm_cos_approx_ftz_f,
    BI__nvvm_cp_async_ca_shared_global_4,
    BI__nvvm_cp_async_ca_shared_global_8,
    BI__nvvm_cp_async_ca_shared_global_16,
    BI__nvvm_cp_async_cg_shared_global_16,
    BI__nvvm_cp_async_commit_group,
    BI__nvvm_cp_async_mbarrier_arrive,
    BI__nvvm_cp_async_mbarrier_arrive_noinc,
    BI__nvvm_cp_async_mbarrier_arrive_noinc_shared,
    BI__nvvm_cp_async_mbarrier_arrive_shared,
    BI__nvvm_cp_async_wait_all,
    BI__nvvm_cp_async_wait_group,
    BI__nvvm_d2f_rm,
    BI__nvvm_d2f_rm_ftz,
    BI__nvvm_d2f_rn,
    BI__nvvm_d2f_rn_ftz,
    BI__nvvm_d2f_rp,
    BI__nvvm_d2f_rp_ftz,
    BI__nvvm_d2f_rz,
    BI__nvvm_d2f_rz_ftz,
    BI__nvvm_d2i_hi,
    BI__nvvm_d2i_lo,
    BI__nvvm_d2i_rm,
    BI__nvvm_d2i_rn,
    BI__nvvm_d2i_rp,
    BI__nvvm_d2i_rz,
    BI__nvvm_d2ll_rm,
    BI__nvvm_d2ll_rn,
    BI__nvvm_d2ll_rp,
    BI__nvvm_d2ll_rz,
    BI__nvvm_d2ui_rm,
    BI__nvvm_d2ui_rn,
    BI__nvvm_d2ui_rp,
    BI__nvvm_d2ui_rz,
    BI__nvvm_d2ull_rm,
    BI__nvvm_d2ull_rn,
    BI__nvvm_d2ull_rp,
    BI__nvvm_d2ull_rz,
    BI__nvvm_div_approx_f,
    BI__nvvm_div_approx_ftz_f,
    BI__nvvm_div_rm_d,
    BI__nvvm_div_rm_f,
    BI__nvvm_div_rm_ftz_f,
    BI__nvvm_div_rn_d,
    BI__nvvm_div_rn_f,
    BI__nvvm_div_rn_ftz_f,
    BI__nvvm_div_rp_d,
    BI__nvvm_div_rp_f,
    BI__nvvm_div_rp_ftz_f,
    BI__nvvm_div_rz_d,
    BI__nvvm_div_rz_f,
    BI__nvvm_div_rz_ftz_f,
    BI__nvvm_e2m1x2_to_f16x2_rn,
    BI__nvvm_e2m1x2_to_f16x2_rn_relu,
    BI__nvvm_e2m3x2_to_f16x2_rn,
    BI__nvvm_e2m3x2_to_f16x2_rn_relu,
    BI__nvvm_e3m2x2_to_f16x2_rn,
    BI__nvvm_e3m2x2_to_f16x2_rn_relu,
    BI__nvvm_e4m3x2_to_f16x2_rn,
    BI__nvvm_e4m3x2_to_f16x2_rn_relu,
    BI__nvvm_e5m2x2_to_f16x2_rn,
    BI__nvvm_e5m2x2_to_f16x2_rn_relu,
    BI__nvvm_ex2_approx_d,
    BI__nvvm_ex2_approx_f,
    BI__nvvm_ex2_approx_f16,
    BI__nvvm_ex2_approx_f16x2,
    BI__nvvm_ex2_approx_ftz_f,
    BI__nvvm_exit,
    BI__nvvm_f2bf16_rn,
    BI__nvvm_f2bf16_rn_relu,
    BI__nvvm_f2bf16_rz,
    BI__nvvm_f2bf16_rz_relu,
    BI__nvvm_f2h_rn,
    BI__nvvm_f2h_rn_ftz,
    BI__nvvm_f2i_rm,
    BI__nvvm_f2i_rm_ftz,
    BI__nvvm_f2i_rn,
    BI__nvvm_f2i_rn_ftz,
    BI__nvvm_f2i_rp,
    BI__nvvm_f2i_rp_ftz,
    BI__nvvm_f2i_rz,
    BI__nvvm_f2i_rz_ftz,
    BI__nvvm_f2ll_rm,
    BI__nvvm_f2ll_rm_ftz,
    BI__nvvm_f2ll_rn,
    BI__nvvm_f2ll_rn_ftz,
    BI__nvvm_f2ll_rp,
    BI__nvvm_f2ll_rp_ftz,
    BI__nvvm_f2ll_rz,
    BI__nvvm_f2ll_rz_ftz,
    BI__nvvm_f2tf32_rn,
    BI__nvvm_f2tf32_rn_relu,
    BI__nvvm_f2tf32_rn_relu_satfinite,
    BI__nvvm_f2tf32_rn_satfinite,
    BI__nvvm_f2tf32_rna,
    BI__nvvm_f2tf32_rna_satfinite,
    BI__nvvm_f2tf32_rz,
    BI__nvvm_f2tf32_rz_relu,
    BI__nvvm_f2tf32_rz_relu_satfinite,
    BI__nvvm_f2tf32_rz_satfinite,
    BI__nvvm_f2ui_rm,
    BI__nvvm_f2ui_rm_ftz,
    BI__nvvm_f2ui_rn,
    BI__nvvm_f2ui_rn_ftz,
    BI__nvvm_f2ui_rp,
    BI__nvvm_f2ui_rp_ftz,
    BI__nvvm_f2ui_rz,
    BI__nvvm_f2ui_rz_ftz,
    BI__nvvm_f2ull_rm,
    BI__nvvm_f2ull_rm_ftz,
    BI__nvvm_f2ull_rn,
    BI__nvvm_f2ull_rn_ftz,
    BI__nvvm_f2ull_rp,
    BI__nvvm_f2ull_rp_ftz,
    BI__nvvm_f2ull_rz,
    BI__nvvm_f2ull_rz_ftz,
    BI__nvvm_f16x2_to_e4m3x2_rn,
    BI__nvvm_f16x2_to_e4m3x2_rn_relu,
    BI__nvvm_f16x2_to_e5m2x2_rn,
    BI__nvvm_f16x2_to_e5m2x2_rn_relu,
    BI__nvvm_fabs_d,
    BI__nvvm_fabs_f,
    BI__nvvm_fabs_f16,
    BI__nvvm_fabs_f16x2,
    BI__nvvm_fabs_ftz_f,
    BI__nvvm_fabs_ftz_f16,
    BI__nvvm_fabs_ftz_f16x2,
    BI__nvvm_fence_sc_cluster,
    BI__nvvm_ff2bf16x2_rn,
    BI__nvvm_ff2bf16x2_rn_relu,
    BI__nvvm_ff2bf16x2_rz,
    BI__nvvm_ff2bf16x2_rz_relu,
    BI__nvvm_ff2f16x2_rn,
    BI__nvvm_ff2f16x2_rn_relu,
    BI__nvvm_ff2f16x2_rz,
    BI__nvvm_ff2f16x2_rz_relu,
    BI__nvvm_ff_to_e2m1x2_rn_relu_satfinite,
    BI__nvvm_ff_to_e2m1x2_rn_satfinite,
    BI__nvvm_ff_to_e2m3x2_rn_relu_satfinite,
    BI__nvvm_ff_to_e2m3x2_rn_satfinite,
    BI__nvvm_ff_to_e3m2x2_rn_relu_satfinite,
    BI__nvvm_ff_to_e3m2x2_rn_satfinite,
    BI__nvvm_ff_to_e4m3x2_rn,
    BI__nvvm_ff_to_e4m3x2_rn_relu,
    BI__nvvm_ff_to_e5m2x2_rn,
    BI__nvvm_ff_to_e5m2x2_rn_relu,
    BI__nvvm_ff_to_ue8m0x2_rp,
    BI__nvvm_ff_to_ue8m0x2_rp_satfinite,
    BI__nvvm_ff_to_ue8m0x2_rz,
    BI__nvvm_ff_to_ue8m0x2_rz_satfinite,
    BI__nvvm_floor_d,
    BI__nvvm_floor_f,
    BI__nvvm_floor_ftz_f,
    BI__nvvm_fma_rm_d,
    BI__nvvm_fma_rm_f,
    BI__nvvm_fma_rm_ftz_f,
    BI__nvvm_fma_rn_bf16,
    BI__nvvm_fma_rn_bf16x2,
    BI__nvvm_fma_rn_d,
    BI__nvvm_fma_rn_f,
    BI__nvvm_fma_rn_f16,
    BI__nvvm_fma_rn_f16x2,
    BI__nvvm_fma_rn_ftz_f,
    BI__nvvm_fma_rn_ftz_f16,
    BI__nvvm_fma_rn_ftz_f16x2,
    BI__nvvm_fma_rn_ftz_relu_f16,
    BI__nvvm_fma_rn_ftz_relu_f16x2,
    BI__nvvm_fma_rn_ftz_sat_f16,
    BI__nvvm_fma_rn_ftz_sat_f16x2,
    BI__nvvm_fma_rn_relu_bf16,
    BI__nvvm_fma_rn_relu_bf16x2,
    BI__nvvm_fma_rn_relu_f16,
    BI__nvvm_fma_rn_relu_f16x2,
    BI__nvvm_fma_rn_sat_f16,
    BI__nvvm_fma_rn_sat_f16x2,
    BI__nvvm_fma_rp_d,
    BI__nvvm_fma_rp_f,
    BI__nvvm_fma_rp_ftz_f,
    BI__nvvm_fma_rz_d,
    BI__nvvm_fma_rz_f,
    BI__nvvm_fma_rz_ftz_f,
    BI__nvvm_fmax_bf16,
    BI__nvvm_fmax_bf16x2,
    BI__nvvm_fmax_d,
    BI__nvvm_fmax_f,
    BI__nvvm_fmax_f16,
    BI__nvvm_fmax_f16x2,
    BI__nvvm_fmax_ftz_bf16,
    BI__nvvm_fmax_ftz_bf16x2,
    BI__nvvm_fmax_ftz_f,
    BI__nvvm_fmax_ftz_f16,
    BI__nvvm_fmax_ftz_f16x2,
    BI__nvvm_fmax_ftz_nan_bf16,
    BI__nvvm_fmax_ftz_nan_bf16x2,
    BI__nvvm_fmax_ftz_nan_f,
    BI__nvvm_fmax_ftz_nan_f16,
    BI__nvvm_fmax_ftz_nan_f16x2,
    BI__nvvm_fmax_ftz_nan_xorsign_abs_f,
    BI__nvvm_fmax_ftz_nan_xorsign_abs_f16,
    BI__nvvm_fmax_ftz_nan_xorsign_abs_f16x2,
    BI__nvvm_fmax_ftz_xorsign_abs_f,
    BI__nvvm_fmax_ftz_xorsign_abs_f16,
    BI__nvvm_fmax_ftz_xorsign_abs_f16x2,
    BI__nvvm_fmax_nan_bf16,
    BI__nvvm_fmax_nan_bf16x2,
    BI__nvvm_fmax_nan_f,
    BI__nvvm_fmax_nan_f16,
    BI__nvvm_fmax_nan_f16x2,
    BI__nvvm_fmax_nan_xorsign_abs_bf16,
    BI__nvvm_fmax_nan_xorsign_abs_bf16x2,
    BI__nvvm_fmax_nan_xorsign_abs_f,
    BI__nvvm_fmax_nan_xorsign_abs_f16,
    BI__nvvm_fmax_nan_xorsign_abs_f16x2,
    BI__nvvm_fmax_xorsign_abs_bf16,
    BI__nvvm_fmax_xorsign_abs_bf16x2,
    BI__nvvm_fmax_xorsign_abs_f,
    BI__nvvm_fmax_xorsign_abs_f16,
    BI__nvvm_fmax_xorsign_abs_f16x2,
    BI__nvvm_fmin_bf16,
    BI__nvvm_fmin_bf16x2,
    BI__nvvm_fmin_d,
    BI__nvvm_fmin_f,
    BI__nvvm_fmin_f16,
    BI__nvvm_fmin_f16x2,
    BI__nvvm_fmin_ftz_bf16,
    BI__nvvm_fmin_ftz_bf16x2,
    BI__nvvm_fmin_ftz_f,
    BI__nvvm_fmin_ftz_f16,
    BI__nvvm_fmin_ftz_f16x2,
    BI__nvvm_fmin_ftz_nan_bf16,
    BI__nvvm_fmin_ftz_nan_bf16x2,
    BI__nvvm_fmin_ftz_nan_f,
    BI__nvvm_fmin_ftz_nan_f16,
    BI__nvvm_fmin_ftz_nan_f16x2,
    BI__nvvm_fmin_ftz_nan_xorsign_abs_f,
    BI__nvvm_fmin_ftz_nan_xorsign_abs_f16,
    BI__nvvm_fmin_ftz_nan_xorsign_abs_f16x2,
    BI__nvvm_fmin_ftz_xorsign_abs_f,
    BI__nvvm_fmin_ftz_xorsign_abs_f16,
    BI__nvvm_fmin_ftz_xorsign_abs_f16x2,
    BI__nvvm_fmin_nan_bf16,
    BI__nvvm_fmin_nan_bf16x2,
    BI__nvvm_fmin_nan_f,
    BI__nvvm_fmin_nan_f16,
    BI__nvvm_fmin_nan_f16x2,
    BI__nvvm_fmin_nan_xorsign_abs_bf16,
    BI__nvvm_fmin_nan_xorsign_abs_bf16x2,
    BI__nvvm_fmin_nan_xorsign_abs_f,
    BI__nvvm_fmin_nan_xorsign_abs_f16,
    BI__nvvm_fmin_nan_xorsign_abs_f16x2,
    BI__nvvm_fmin_xorsign_abs_bf16,
    BI__nvvm_fmin_xorsign_abs_bf16x2,
    BI__nvvm_fmin_xorsign_abs_f,
    BI__nvvm_fmin_xorsign_abs_f16,
    BI__nvvm_fmin_xorsign_abs_f16x2,
    BI__nvvm_fns,
    BI__nvvm_getctarank,
    BI__nvvm_getctarank_shared_cluster,
    BI__nvvm_i2d_rm,
    BI__nvvm_i2d_rn,
    BI__nvvm_i2d_rp,
    BI__nvvm_i2d_rz,
    BI__nvvm_i2f_rm,
    BI__nvvm_i2f_rn,
    BI__nvvm_i2f_rp,
    BI__nvvm_i2f_rz,
    BI__nvvm_is_explicit_cluster,
    BI__nvvm_isspacep_const,
    BI__nvvm_isspacep_global,
    BI__nvvm_isspacep_local,
    BI__nvvm_isspacep_shared,
    BI__nvvm_isspacep_shared_cluster,
    BI__nvvm_ldg_c,
    BI__nvvm_ldg_c2,
    BI__nvvm_ldg_c4,
    BI__nvvm_ldg_d,
    BI__nvvm_ldg_d2,
    BI__nvvm_ldg_f,
    BI__nvvm_ldg_f2,
    BI__nvvm_ldg_f4,
    BI__nvvm_ldg_h,
    BI__nvvm_ldg_h2,
    BI__nvvm_ldg_i,
    BI__nvvm_ldg_i2,
    BI__nvvm_ldg_i4,
    BI__nvvm_ldg_l,
    BI__nvvm_ldg_l2,
    BI__nvvm_ldg_ll,
    BI__nvvm_ldg_ll2,
    BI__nvvm_ldg_s,
    BI__nvvm_ldg_s2,
    BI__nvvm_ldg_s4,
    BI__nvvm_ldg_sc,
    BI__nvvm_ldg_sc2,
    BI__nvvm_ldg_sc4,
    BI__nvvm_ldg_uc,
    BI__nvvm_ldg_uc2,
    BI__nvvm_ldg_uc4,
    BI__nvvm_ldg_ui,
    BI__nvvm_ldg_ui2,
    BI__nvvm_ldg_ui4,
    BI__nvvm_ldg_ul,
    BI__nvvm_ldg_ul2,
    BI__nvvm_ldg_ull,
    BI__nvvm_ldg_ull2,
    BI__nvvm_ldg_us,
    BI__nvvm_ldg_us2,
    BI__nvvm_ldg_us4,
    BI__nvvm_ldu_c,
    BI__nvvm_ldu_c2,
    BI__nvvm_ldu_c4,
    BI__nvvm_ldu_d,
    BI__nvvm_ldu_d2,
    BI__nvvm_ldu_f,
    BI__nvvm_ldu_f2,
    BI__nvvm_ldu_f4,
    BI__nvvm_ldu_h,
    BI__nvvm_ldu_h2,
    BI__nvvm_ldu_i,
    BI__nvvm_ldu_i2,
    BI__nvvm_ldu_i4,
    BI__nvvm_ldu_l,
    BI__nvvm_ldu_l2,
    BI__nvvm_ldu_ll,
    BI__nvvm_ldu_ll2,
    BI__nvvm_ldu_s,
    BI__nvvm_ldu_s2,
    BI__nvvm_ldu_s4,
    BI__nvvm_ldu_sc,
    BI__nvvm_ldu_sc2,
    BI__nvvm_ldu_sc4,
    BI__nvvm_ldu_uc,
    BI__nvvm_ldu_uc2,
    BI__nvvm_ldu_uc4,
    BI__nvvm_ldu_ui,
    BI__nvvm_ldu_ui2,
    BI__nvvm_ldu_ui4,
    BI__nvvm_ldu_ul,
    BI__nvvm_ldu_ul2,
    BI__nvvm_ldu_ull,
    BI__nvvm_ldu_ull2,
    BI__nvvm_ldu_us,
    BI__nvvm_ldu_us2,
    BI__nvvm_ldu_us4,
    BI__nvvm_lg2_approx_d,
    BI__nvvm_lg2_approx_f,
    BI__nvvm_lg2_approx_ftz_f,
    BI__nvvm_ll2d_rm,
    BI__nvvm_ll2d_rn,
    BI__nvvm_ll2d_rp,
    BI__nvvm_ll2d_rz,
    BI__nvvm_ll2f_rm,
    BI__nvvm_ll2f_rn,
    BI__nvvm_ll2f_rp,
    BI__nvvm_ll2f_rz,
    BI__nvvm_lohi_i2d,
    BI__nvvm_mapa,
    BI__nvvm_mapa_shared_cluster,
    BI__nvvm_match_all_sync_i32p,
    BI__nvvm_match_all_sync_i64p,
    BI__nvvm_match_any_sync_i32,
    BI__nvvm_match_any_sync_i64,
    BI__nvvm_mbarrier_arrive,
    BI__nvvm_mbarrier_arrive_drop,
    BI__nvvm_mbarrier_arrive_drop_noComplete,
    BI__nvvm_mbarrier_arrive_drop_noComplete_shared,
    BI__nvvm_mbarrier_arrive_drop_shared,
    BI__nvvm_mbarrier_arrive_noComplete,
    BI__nvvm_mbarrier_arrive_noComplete_shared,
    BI__nvvm_mbarrier_arrive_shared,
    BI__nvvm_mbarrier_init,
    BI__nvvm_mbarrier_init_shared,
    BI__nvvm_mbarrier_inval,
    BI__nvvm_mbarrier_inval_shared,
    BI__nvvm_mbarrier_pending_count,
    BI__nvvm_mbarrier_test_wait,
    BI__nvvm_mbarrier_test_wait_shared,
    BI__nvvm_membar_cta,
    BI__nvvm_membar_gl,
    BI__nvvm_membar_sys,
    BI__nvvm_memcpy,
    BI__nvvm_memset,
    BI__nvvm_mul24_i,
    BI__nvvm_mul24_ui,
    BI__nvvm_mul_rm_d,
    BI__nvvm_mul_rm_f,
    BI__nvvm_mul_rm_ftz_f,
    BI__nvvm_mul_rn_d,
    BI__nvvm_mul_rn_f,
    BI__nvvm_mul_rn_ftz_f,
    BI__nvvm_mul_rp_d,
    BI__nvvm_mul_rp_f,
    BI__nvvm_mul_rp_ftz_f,
    BI__nvvm_mul_rz_d,
    BI__nvvm_mul_rz_f,
    BI__nvvm_mul_rz_ftz_f,
    BI__nvvm_mulhi_i,
    BI__nvvm_mulhi_ll,
    BI__nvvm_mulhi_ui,
    BI__nvvm_mulhi_ull,
    BI__nvvm_nanosleep,
    BI__nvvm_neg_bf16,
    BI__nvvm_neg_bf16x2,
    BI__nvvm_pm_event_mask,
    BI__nvvm_prmt,
    BI__nvvm_rcp_approx_ftz_d,
    BI__nvvm_rcp_approx_ftz_f,
    BI__nvvm_rcp_rm_d,
    BI__nvvm_rcp_rm_f,
    BI__nvvm_rcp_rm_ftz_f,
    BI__nvvm_rcp_rn_d,
    BI__nvvm_rcp_rn_f,
    BI__nvvm_rcp_rn_ftz_f,
    BI__nvvm_rcp_rp_d,
    BI__nvvm_rcp_rp_f,
    BI__nvvm_rcp_rp_ftz_f,
    BI__nvvm_rcp_rz_d,
    BI__nvvm_rcp_rz_f,
    BI__nvvm_rcp_rz_ftz_f,
    BI__nvvm_read_ptx_sreg_clock,
    BI__nvvm_read_ptx_sreg_clock64,
    BI__nvvm_read_ptx_sreg_cluster_ctaid_w,
    BI__nvvm_read_ptx_sreg_cluster_ctaid_x,
    BI__nvvm_read_ptx_sreg_cluster_ctaid_y,
    BI__nvvm_read_ptx_sreg_cluster_ctaid_z,
    BI__nvvm_read_ptx_sreg_cluster_ctarank,
    BI__nvvm_read_ptx_sreg_cluster_nctaid_w,
    BI__nvvm_read_ptx_sreg_cluster_nctaid_x,
    BI__nvvm_read_ptx_sreg_cluster_nctaid_y,
    BI__nvvm_read_ptx_sreg_cluster_nctaid_z,
    BI__nvvm_read_ptx_sreg_cluster_nctarank,
    BI__nvvm_read_ptx_sreg_clusterid_w,
    BI__nvvm_read_ptx_sreg_clusterid_x,
    BI__nvvm_read_ptx_sreg_clusterid_y,
    BI__nvvm_read_ptx_sreg_clusterid_z,
    BI__nvvm_read_ptx_sreg_ctaid_w,
    BI__nvvm_read_ptx_sreg_ctaid_x,
    BI__nvvm_read_ptx_sreg_ctaid_y,
    BI__nvvm_read_ptx_sreg_ctaid_z,
    BI__nvvm_read_ptx_sreg_globaltimer,
    BI__nvvm_read_ptx_sreg_gridid,
    BI__nvvm_read_ptx_sreg_laneid,
    BI__nvvm_read_ptx_sreg_lanemask_eq,
    BI__nvvm_read_ptx_sreg_lanemask_ge,
    BI__nvvm_read_ptx_sreg_lanemask_gt,
    BI__nvvm_read_ptx_sreg_lanemask_le,
    BI__nvvm_read_ptx_sreg_lanemask_lt,
    BI__nvvm_read_ptx_sreg_nclusterid_w,
    BI__nvvm_read_ptx_sreg_nclusterid_x,
    BI__nvvm_read_ptx_sreg_nclusterid_y,
    BI__nvvm_read_ptx_sreg_nclusterid_z,
    BI__nvvm_read_ptx_sreg_nctaid_w,
    BI__nvvm_read_ptx_sreg_nctaid_x,
    BI__nvvm_read_ptx_sreg_nctaid_y,
    BI__nvvm_read_ptx_sreg_nctaid_z,
    BI__nvvm_read_ptx_sreg_nsmid,
    BI__nvvm_read_ptx_sreg_ntid_w,
    BI__nvvm_read_ptx_sreg_ntid_x,
    BI__nvvm_read_ptx_sreg_ntid_y,
    BI__nvvm_read_ptx_sreg_ntid_z,
    BI__nvvm_read_ptx_sreg_nwarpid,
    BI__nvvm_read_ptx_sreg_pm0,
    BI__nvvm_read_ptx_sreg_pm1,
    BI__nvvm_read_ptx_sreg_pm2,
    BI__nvvm_read_ptx_sreg_pm3,
    BI__nvvm_read_ptx_sreg_smid,
    BI__nvvm_read_ptx_sreg_tid_w,
    BI__nvvm_read_ptx_sreg_tid_x,
    BI__nvvm_read_ptx_sreg_tid_y,
    BI__nvvm_read_ptx_sreg_tid_z,
    BI__nvvm_read_ptx_sreg_warpid,
    BI__nvvm_read_ptx_sreg_warpsize,
    BI__nvvm_redux_sync_add,
    BI__nvvm_redux_sync_and,
    BI__nvvm_redux_sync_fmax,
    BI__nvvm_redux_sync_fmax_NaN,
    BI__nvvm_redux_sync_fmax_abs,
    BI__nvvm_redux_sync_fmax_abs_NaN,
    BI__nvvm_redux_sync_fmin,
    BI__nvvm_redux_sync_fmin_NaN,
    BI__nvvm_redux_sync_fmin_abs,
    BI__nvvm_redux_sync_fmin_abs_NaN,
    BI__nvvm_redux_sync_max,
    BI__nvvm_redux_sync_min,
    BI__nvvm_redux_sync_or,
    BI__nvvm_redux_sync_umax,
    BI__nvvm_redux_sync_umin,
    BI__nvvm_redux_sync_xor,
    BI__nvvm_reflect,
    BI__nvvm_round_d,
    BI__nvvm_round_f,
    BI__nvvm_round_ftz_f,
    BI__nvvm_rsqrt_approx_d,
    BI__nvvm_rsqrt_approx_f,
    BI__nvvm_rsqrt_approx_ftz_f,
    BI__nvvm_sad_i,
    BI__nvvm_sad_ui,
    BI__nvvm_saturate_d,
    BI__nvvm_saturate_f,
    BI__nvvm_saturate_ftz_f,
    BI__nvvm_shfl_bfly_f32,
    BI__nvvm_shfl_bfly_i32,
    BI__nvvm_shfl_down_f32,
    BI__nvvm_shfl_down_i32,
    BI__nvvm_shfl_idx_f32,
    BI__nvvm_shfl_idx_i32,
    BI__nvvm_shfl_sync_bfly_f32,
    BI__nvvm_shfl_sync_bfly_i32,
    BI__nvvm_shfl_sync_down_f32,
    BI__nvvm_shfl_sync_down_i32,
    BI__nvvm_shfl_sync_idx_f32,
    BI__nvvm_shfl_sync_idx_i32,
    BI__nvvm_shfl_sync_up_f32,
    BI__nvvm_shfl_sync_up_i32,
    BI__nvvm_shfl_up_f32,
    BI__nvvm_shfl_up_i32,
    BI__nvvm_sin_approx_f,
    BI__nvvm_sin_approx_ftz_f,
    BI__nvvm_sqrt_approx_f,
    BI__nvvm_sqrt_approx_ftz_f,
    BI__nvvm_sqrt_rm_d,
    BI__nvvm_sqrt_rm_f,
    BI__nvvm_sqrt_rm_ftz_f,
    BI__nvvm_sqrt_rn_d,
    BI__nvvm_sqrt_rn_f,
    BI__nvvm_sqrt_rn_ftz_f,
    BI__nvvm_sqrt_rp_d,
    BI__nvvm_sqrt_rp_f,
    BI__nvvm_sqrt_rp_ftz_f,
    BI__nvvm_sqrt_rz_d,
    BI__nvvm_sqrt_rz_f,
    BI__nvvm_sqrt_rz_ftz_f,
    BI__nvvm_trunc_d,
    BI__nvvm_trunc_f,
    BI__nvvm_trunc_ftz_f,
    BI__nvvm_ue8m0x2_to_bf16x2,
    BI__nvvm_ui2d_rm,
    BI__nvvm_ui2d_rn,
    BI__nvvm_ui2d_rp,
    BI__nvvm_ui2d_rz,
    BI__nvvm_ui2f_rm,
    BI__nvvm_ui2f_rn,
    BI__nvvm_ui2f_rp,
    BI__nvvm_ui2f_rz,
    BI__nvvm_ull2d_rm,
    BI__nvvm_ull2d_rn,
    BI__nvvm_ull2d_rp,
    BI__nvvm_ull2d_rz,
    BI__nvvm_ull2f_rm,
    BI__nvvm_ull2f_rn,
    BI__nvvm_ull2f_rp,
    BI__nvvm_ull2f_rz,
    BI__nvvm_vote_all,
    BI__nvvm_vote_all_sync,
    BI__nvvm_vote_any,
    BI__nvvm_vote_any_sync,
    BI__nvvm_vote_ballot,
    BI__nvvm_vote_ballot_sync,
    BI__nvvm_vote_uni,
    BI__nvvm_vote_uni_sync,
    BI__syncthreads,

#endif // GET_BUILTIN_ENUMERATORS

#ifdef GET_BUILTIN_STR_TABLE

#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Woverlength-strings"
#endif
static constexpr char BuiltinStringsStorage[] = 
  "\0"
  "__bmma_m8n8k128_ld_a_b1\0"
  "vi*iC*UiIi\0"
  "(sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87)\0"
  "__bmma_m8n8k128_ld_b_b1\0"
  "__bmma_m8n8k128_ld_c\0"
  "__bmma_m8n8k128_mma_and_popc_b1\0"
  "vi*iC*iC*iC*Ii\0"
  "(sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87)\0"
  "__bmma_m8n8k128_mma_xor_popc_b1\0"
  "__bmma_m8n8k128_st_c_i32\0"
  "__builtin_ptx_get_image_channel_data_typei_\0"
  "ii\0"
  "__builtin_ptx_get_image_channel_orderi_\0"
  "__builtin_ptx_get_image_depthi_\0"
  "__builtin_ptx_get_image_heighti_\0"
  "__builtin_ptx_get_image_widthi_\0"
  "__builtin_ptx_read_image2Dff_\0"
  "V4fiiff\0"
  "__builtin_ptx_read_image2Dfi_\0"
  "V4fiiii\0"
  "__builtin_ptx_read_image2Dif_\0"
  "V4iiiff\0"
  "__builtin_ptx_read_image2Dii_\0"
  "V4iiiii\0"
  "__builtin_ptx_read_image3Dff_\0"
  "V4fiiffff\0"
  "__builtin_ptx_read_image3Dfi_\0"
  "V4fiiiiii\0"
  "__builtin_ptx_read_image3Dif_\0"
  "V4iiiffff\0"
  "__builtin_ptx_read_image3Dii_\0"
  "V4iiiiiii\0"
  "__builtin_ptx_write_image2Df_\0"
  "viiiffff\0"
  "__builtin_ptx_write_image2Di_\0"
  "viiiiiii\0"
  "__builtin_ptx_write_image2Dui_\0"
  "viiiUiUiUiUi\0"
  "__dmma_m8n8k4_ld_a\0"
  "vd*dC*UiIi\0"
  "(sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87)\0"
  "__dmma_m8n8k4_ld_b\0"
  "__dmma_m8n8k4_ld_c\0"
  "__dmma_m8n8k4_mma_f64\0"
  "vd*dC*dC*dC*IiIi\0"
  "__dmma_m8n8k4_st_c_f64\0"
  "__hmma_m8n32k16_ld_a\0"
  "(sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87)\0"
  "__hmma_m8n32k16_ld_b\0"
  "__hmma_m8n32k16_ld_c_f16\0"
  "__hmma_m8n32k16_ld_c_f32\0"
  "vf*fC*UiIi\0"
  "__hmma_m8n32k16_mma_f16f16\0"
  "vi*iC*iC*iC*IiIi\0"
  "__hmma_m8n32k16_mma_f16f32\0"
  "vi*iC*iC*fC*IiIi\0"
  "__hmma_m8n32k16_mma_f32f16\0"
  "vf*iC*iC*iC*IiIi\0"
  "__hmma_m8n32k16_mma_f32f32\0"
  "vf*iC*iC*fC*IiIi\0"
  "__hmma_m8n32k16_st_c_f16\0"
  "__hmma_m8n32k16_st_c_f32\0"
  "__hmma_m16n16k16_ld_a\0"
  "(sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87)\0"
  "__hmma_m16n16k16_ld_b\0"
  "__hmma_m16n16k16_ld_c_f16\0"
  "__hmma_m16n16k16_ld_c_f32\0"
  "__hmma_m16n16k16_mma_f16f16\0"
  "__hmma_m16n16k16_mma_f16f32\0"
  "__hmma_m16n16k16_mma_f32f16\0"
  "__hmma_m16n16k16_mma_f32f32\0"
  "__hmma_m16n16k16_st_c_f16\0"
  "__hmma_m16n16k16_st_c_f32\0"
  "__hmma_m32n8k16_ld_a\0"
  "__hmma_m32n8k16_ld_b\0"
  "__hmma_m32n8k16_ld_c_f16\0"
  "__hmma_m32n8k16_ld_c_f32\0"
  "__hmma_m32n8k16_mma_f16f16\0"
  "__hmma_m32n8k16_mma_f16f32\0"
  "__hmma_m32n8k16_mma_f32f16\0"
  "__hmma_m32n8k16_mma_f32f32\0"
  "__hmma_m32n8k16_st_c_f16\0"
  "__hmma_m32n8k16_st_c_f32\0"
  "__imma_m8n8k32_ld_a_s4\0"
  "__imma_m8n8k32_ld_a_u4\0"
  "__imma_m8n8k32_ld_b_s4\0"
  "__imma_m8n8k32_ld_b_u4\0"
  "__imma_m8n8k32_ld_c\0"
  "__imma_m8n8k32_mma_s4\0"
  "__imma_m8n8k32_mma_u4\0"
  "__imma_m8n8k32_st_c_i32\0"
  "__imma_m8n32k16_ld_a_s8\0"
  "(sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87)\0"
  "__imma_m8n32k16_ld_a_u8\0"
  "__imma_m8n32k16_ld_b_s8\0"
  "__imma_m8n32k16_ld_b_u8\0"
  "__imma_m8n32k16_ld_c\0"
  "__imma_m8n32k16_mma_s8\0"
  "__imma_m8n32k16_mma_u8\0"
  "__imma_m8n32k16_st_c_i32\0"
  "__imma_m16n16k16_ld_a_s8\0"
  "__imma_m16n16k16_ld_a_u8\0"
  "__imma_m16n16k16_ld_b_s8\0"
  "__imma_m16n16k16_ld_b_u8\0"
  "__imma_m16n16k16_ld_c\0"
  "__imma_m16n16k16_mma_s8\0"
  "__imma_m16n16k16_mma_u8\0"
  "__imma_m16n16k16_st_c_i32\0"
  "__imma_m32n8k16_ld_a_s8\0"
  "__imma_m32n8k16_ld_a_u8\0"
  "__imma_m32n8k16_ld_b_s8\0"
  "__imma_m32n8k16_ld_b_u8\0"
  "__imma_m32n8k16_ld_c\0"
  "__imma_m32n8k16_mma_s8\0"
  "__imma_m32n8k16_mma_u8\0"
  "__imma_m32n8k16_st_c_i32\0"
  "__mma_bf16_m8n32k16_ld_a\0"
  "__mma_bf16_m8n32k16_ld_b\0"
  "__mma_bf16_m8n32k16_mma_f32\0"
  "__mma_bf16_m16n16k16_ld_a\0"
  "__mma_bf16_m16n16k16_ld_b\0"
  "__mma_bf16_m16n16k16_mma_f32\0"
  "__mma_bf16_m32n8k16_ld_a\0"
  "__mma_bf16_m32n8k16_ld_b\0"
  "__mma_bf16_m32n8k16_mma_f32\0"
  "__mma_m16n16k8_st_c_f32\0"
  "__mma_tf32_m16n16k8_ld_a\0"
  "__mma_tf32_m16n16k8_ld_b\0"
  "__mma_tf32_m16n16k8_ld_c\0"
  "__mma_tf32_m16n16k8_mma_f32\0"
  "__nvvm_abs_bf16\0"
  "yy\0"
  "__nvvm_abs_bf16x2\0"
  "V2yV2y\0"
  "__nvvm_activemask\0"
  "Ui\0"
  "n\0"
  "ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87\0"
  "__nvvm_add_rm_d\0"
  "ddd\0"
  "__nvvm_add_rm_f\0"
  "fff\0"
  "__nvvm_add_rm_ftz_f\0"
  "__nvvm_add_rn_d\0"
  "__nvvm_add_rn_f\0"
  "__nvvm_add_rn_ftz_f\0"
  "__nvvm_add_rp_d\0"
  "__nvvm_add_rp_f\0"
  "__nvvm_add_rp_ftz_f\0"
  "__nvvm_add_rz_d\0"
  "__nvvm_add_rz_f\0"
  "__nvvm_add_rz_ftz_f\0"
  "__nvvm_atom_add_gen_d\0"
  "ddD*d\0"
  "sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a\0"
  "__nvvm_atom_add_gen_f\0"
  "ffD*f\0"
  "__nvvm_atom_add_gen_i\0"
  "iiD*i\0"
  "__nvvm_atom_add_gen_l\0"
  "LiLiD*Li\0"
  "__nvvm_atom_add_gen_ll\0"
  "LLiLLiD*LLi\0"
  "__nvvm_atom_and_gen_i\0"
  "__nvvm_atom_and_gen_l\0"
  "__nvvm_atom_and_gen_ll\0"
  "__nvvm_atom_cas_gen_i\0"
  "iiD*ii\0"
  "__nvvm_atom_cas_gen_l\0"
  "LiLiD*LiLi\0"
  "__nvvm_atom_cas_gen_ll\0"
  "LLiLLiD*LLiLLi\0"
  "__nvvm_atom_cas_gen_us\0"
  "UsUsD*UsUs\0"
  "sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a\0"
  "__nvvm_atom_cta_add_gen_d\0"
  "__nvvm_atom_cta_add_gen_f\0"
  "__nvvm_atom_cta_add_gen_i\0"
  "__nvvm_atom_cta_add_gen_l\0"
  "__nvvm_atom_cta_add_gen_ll\0"
  "__nvvm_atom_cta_and_gen_i\0"
  "__nvvm_atom_cta_and_gen_l\0"
  "__nvvm_atom_cta_and_gen_ll\0"
  "__nvvm_atom_cta_cas_gen_i\0"
  "__nvvm_atom_cta_cas_gen_l\0"
  "__nvvm_atom_cta_cas_gen_ll\0"
  "__nvvm_atom_cta_cas_gen_us\0"
  "__nvvm_atom_cta_dec_gen_ui\0"
  "UiUiD*Ui\0"
  "__nvvm_atom_cta_inc_gen_ui\0"
  "__nvvm_atom_cta_max_gen_i\0"
  "__nvvm_atom_cta_max_gen_l\0"
  "__nvvm_atom_cta_max_gen_ll\0"
  "__nvvm_atom_cta_max_gen_ui\0"
  "__nvvm_atom_cta_max_gen_ul\0"
  "ULiULiD*ULi\0"
  "__nvvm_atom_cta_max_gen_ull\0"
  "ULLiULLiD*ULLi\0"
  "__nvvm_atom_cta_min_gen_i\0"
  "__nvvm_atom_cta_min_gen_l\0"
  "__nvvm_atom_cta_min_gen_ll\0"
  "__nvvm_atom_cta_min_gen_ui\0"
  "__nvvm_atom_cta_min_gen_ul\0"
  "__nvvm_atom_cta_min_gen_ull\0"
  "__nvvm_atom_cta_or_gen_i\0"
  "__nvvm_atom_cta_or_gen_l\0"
  "__nvvm_atom_cta_or_gen_ll\0"
  "__nvvm_atom_cta_xchg_gen_i\0"
  "__nvvm_atom_cta_xchg_gen_l\0"
  "__nvvm_atom_cta_xchg_gen_ll\0"
  "__nvvm_atom_cta_xor_gen_i\0"
  "__nvvm_atom_cta_xor_gen_l\0"
  "__nvvm_atom_cta_xor_gen_ll\0"
  "__nvvm_atom_dec_gen_ui\0"
  "__nvvm_atom_inc_gen_ui\0"
  "__nvvm_atom_max_gen_i\0"
  "__nvvm_atom_max_gen_l\0"
  "__nvvm_atom_max_gen_ll\0"
  "__nvvm_atom_max_gen_ui\0"
  "__nvvm_atom_max_gen_ul\0"
  "__nvvm_atom_max_gen_ull\0"
  "__nvvm_atom_min_gen_i\0"
  "__nvvm_atom_min_gen_l\0"
  "__nvvm_atom_min_gen_ll\0"
  "__nvvm_atom_min_gen_ui\0"
  "__nvvm_atom_min_gen_ul\0"
  "__nvvm_atom_min_gen_ull\0"
  "__nvvm_atom_or_gen_i\0"
  "__nvvm_atom_or_gen_l\0"
  "__nvvm_atom_or_gen_ll\0"
  "__nvvm_atom_sub_gen_i\0"
  "__nvvm_atom_sub_gen_l\0"
  "__nvvm_atom_sub_gen_ll\0"
  "__nvvm_atom_sys_add_gen_d\0"
  "__nvvm_atom_sys_add_gen_f\0"
  "__nvvm_atom_sys_add_gen_i\0"
  "__nvvm_atom_sys_add_gen_l\0"
  "__nvvm_atom_sys_add_gen_ll\0"
  "__nvvm_atom_sys_and_gen_i\0"
  "__nvvm_atom_sys_and_gen_l\0"
  "__nvvm_atom_sys_and_gen_ll\0"
  "__nvvm_atom_sys_cas_gen_i\0"
  "__nvvm_atom_sys_cas_gen_l\0"
  "__nvvm_atom_sys_cas_gen_ll\0"
  "__nvvm_atom_sys_cas_gen_us\0"
  "__nvvm_atom_sys_dec_gen_ui\0"
  "__nvvm_atom_sys_inc_gen_ui\0"
  "__nvvm_atom_sys_max_gen_i\0"
  "__nvvm_atom_sys_max_gen_l\0"
  "__nvvm_atom_sys_max_gen_ll\0"
  "__nvvm_atom_sys_max_gen_ui\0"
  "__nvvm_atom_sys_max_gen_ul\0"
  "__nvvm_atom_sys_max_gen_ull\0"
  "__nvvm_atom_sys_min_gen_i\0"
  "__nvvm_atom_sys_min_gen_l\0"
  "__nvvm_atom_sys_min_gen_ll\0"
  "__nvvm_atom_sys_min_gen_ui\0"
  "__nvvm_atom_sys_min_gen_ul\0"
  "__nvvm_atom_sys_min_gen_ull\0"
  "__nvvm_atom_sys_or_gen_i\0"
  "__nvvm_atom_sys_or_gen_l\0"
  "__nvvm_atom_sys_or_gen_ll\0"
  "__nvvm_atom_sys_xchg_gen_i\0"
  "__nvvm_atom_sys_xchg_gen_l\0"
  "__nvvm_atom_sys_xchg_gen_ll\0"
  "__nvvm_atom_sys_xor_gen_i\0"
  "__nvvm_atom_sys_xor_gen_l\0"
  "__nvvm_atom_sys_xor_gen_ll\0"
  "__nvvm_atom_xchg_gen_i\0"
  "__nvvm_atom_xchg_gen_l\0"
  "__nvvm_atom_xchg_gen_ll\0"
  "__nvvm_atom_xor_gen_i\0"
  "__nvvm_atom_xor_gen_l\0"
  "__nvvm_atom_xor_gen_ll\0"
  "__nvvm_bar0_and\0"
  "__nvvm_bar0_or\0"
  "__nvvm_bar0_popc\0"
  "__nvvm_bar_sync\0"
  "vi\0"
  "__nvvm_bar_warp_sync\0"
  "vUi\0"
  "ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87\0"
  "__nvvm_barrier_cluster_arrive\0"
  "v\0"
  "(sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87)\0"
  "__nvvm_barrier_cluster_arrive_relaxed\0"
  "(sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87)\0"
  "__nvvm_barrier_cluster_wait\0"
  "__nvvm_barrier_sync\0"
  "__nvvm_barrier_sync_cnt\0"
  "vUiUi\0"
  "__nvvm_bf16x2_to_ue8m0x2_rp\0"
  "sV2y\0"
  "(sm_100a|sm_101a|sm_120a),(ptx86|ptx87)\0"
  "__nvvm_bf16x2_to_ue8m0x2_rp_satfinite\0"
  "__nvvm_bf16x2_to_ue8m0x2_rz\0"
  "__nvvm_bf16x2_to_ue8m0x2_rz_satfinite\0"
  "__nvvm_ceil_d\0"
  "dd\0"
  "__nvvm_ceil_f\0"
  "ff\0"
  "__nvvm_ceil_ftz_f\0"
  "__nvvm_compiler_error\0"
  "vcC*4\0"
  "__nvvm_compiler_warn\0"
  "__nvvm_cos_approx_f\0"
  "__nvvm_cos_approx_ftz_f\0"
  "__nvvm_cp_async_ca_shared_global_4\0"
  "vv*3vC*1.\0"
  "__nvvm_cp_async_ca_shared_global_8\0"
  "__nvvm_cp_async_ca_shared_global_16\0"
  "__nvvm_cp_async_cg_shared_global_16\0"
  "__nvvm_cp_async_commit_group\0"
  "__nvvm_cp_async_mbarrier_arrive\0"
  "vWi*\0"
  "__nvvm_cp_async_mbarrier_arrive_noinc\0"
  "__nvvm_cp_async_mbarrier_arrive_noinc_shared\0"
  "vWi*3\0"
  "__nvvm_cp_async_mbarrier_arrive_shared\0"
  "__nvvm_cp_async_wait_all\0"
  "__nvvm_cp_async_wait_group\0"
  "vIi\0"
  "__nvvm_d2f_rm\0"
  "fd\0"
  "__nvvm_d2f_rm_ftz\0"
  "__nvvm_d2f_rn\0"
  "__nvvm_d2f_rn_ftz\0"
  "__nvvm_d2f_rp\0"
  "__nvvm_d2f_rp_ftz\0"
  "__nvvm_d2f_rz\0"
  "__nvvm_d2f_rz_ftz\0"
  "__nvvm_d2i_hi\0"
  "id\0"
  "__nvvm_d2i_lo\0"
  "__nvvm_d2i_rm\0"
  "__nvvm_d2i_rn\0"
  "__nvvm_d2i_rp\0"
  "__nvvm_d2i_rz\0"
  "__nvvm_d2ll_rm\0"
  "LLid\0"
  "__nvvm_d2ll_rn\0"
  "__nvvm_d2ll_rp\0"
  "__nvvm_d2ll_rz\0"
  "__nvvm_d2ui_rm\0"
  "Uid\0"
  "__nvvm_d2ui_rn\0"
  "__nvvm_d2ui_rp\0"
  "__nvvm_d2ui_rz\0"
  "__nvvm_d2ull_rm\0"
  "ULLid\0"
  "__nvvm_d2ull_rn\0"
  "__nvvm_d2ull_rp\0"
  "__nvvm_d2ull_rz\0"
  "__nvvm_div_approx_f\0"
  "__nvvm_div_approx_ftz_f\0"
  "__nvvm_div_rm_d\0"
  "__nvvm_div_rm_f\0"
  "__nvvm_div_rm_ftz_f\0"
  "__nvvm_div_rn_d\0"
  "__nvvm_div_rn_f\0"
  "__nvvm_div_rn_ftz_f\0"
  "__nvvm_div_rp_d\0"
  "__nvvm_div_rp_f\0"
  "__nvvm_div_rp_ftz_f\0"
  "__nvvm_div_rz_d\0"
  "__nvvm_div_rz_f\0"
  "__nvvm_div_rz_ftz_f\0"
  "__nvvm_e2m1x2_to_f16x2_rn\0"
  "V2hs\0"
  "__nvvm_e2m1x2_to_f16x2_rn_relu\0"
  "__nvvm_e2m3x2_to_f16x2_rn\0"
  "__nvvm_e2m3x2_to_f16x2_rn_relu\0"
  "__nvvm_e3m2x2_to_f16x2_rn\0"
  "__nvvm_e3m2x2_to_f16x2_rn_relu\0"
  "__nvvm_e4m3x2_to_f16x2_rn\0"
  "(sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87)\0"
  "__nvvm_e4m3x2_to_f16x2_rn_relu\0"
  "__nvvm_e5m2x2_to_f16x2_rn\0"
  "__nvvm_e5m2x2_to_f16x2_rn_relu\0"
  "__nvvm_ex2_approx_d\0"
  "__nvvm_ex2_approx_f\0"
  "__nvvm_ex2_approx_f16\0"
  "hh\0"
  "(sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87)\0"
  "__nvvm_ex2_approx_f16x2\0"
  "V2hV2h\0"
  "__nvvm_ex2_approx_ftz_f\0"
  "__nvvm_exit\0"
  "r\0"
  "__nvvm_f2bf16_rn\0"
  "yf\0"
  "__nvvm_f2bf16_rn_relu\0"
  "__nvvm_f2bf16_rz\0"
  "__nvvm_f2bf16_rz_relu\0"
  "__nvvm_f2h_rn\0"
  "Usf\0"
  "__nvvm_f2h_rn_ftz\0"
  "__nvvm_f2i_rm\0"
  "if\0"
  "__nvvm_f2i_rm_ftz\0"
  "__nvvm_f2i_rn\0"
  "__nvvm_f2i_rn_ftz\0"
  "__nvvm_f2i_rp\0"
  "__nvvm_f2i_rp_ftz\0"
  "__nvvm_f2i_rz\0"
  "__nvvm_f2i_rz_ftz\0"
  "__nvvm_f2ll_rm\0"
  "LLif\0"
  "__nvvm_f2ll_rm_ftz\0"
  "__nvvm_f2ll_rn\0"
  "__nvvm_f2ll_rn_ftz\0"
  "__nvvm_f2ll_rp\0"
  "__nvvm_f2ll_rp_ftz\0"
  "__nvvm_f2ll_rz\0"
  "__nvvm_f2ll_rz_ftz\0"
  "__nvvm_f2tf32_rn\0"
  "Zif\0"
  "__nvvm_f2tf32_rn_relu\0"
  "__nvvm_f2tf32_rn_relu_satfinite\0"
  "(sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx86|ptx87)\0"
  "__nvvm_f2tf32_rn_satfinite\0"
  "__nvvm_f2tf32_rna\0"
  "__nvvm_f2tf32_rna_satfinite\0"
  "__nvvm_f2tf32_rz\0"
  "__nvvm_f2tf32_rz_relu\0"
  "__nvvm_f2tf32_rz_relu_satfinite\0"
  "__nvvm_f2tf32_rz_satfinite\0"
  "__nvvm_f2ui_rm\0"
  "Uif\0"
  "__nvvm_f2ui_rm_ftz\0"
  "__nvvm_f2ui_rn\0"
  "__nvvm_f2ui_rn_ftz\0"
  "__nvvm_f2ui_rp\0"
  "__nvvm_f2ui_rp_ftz\0"
  "__nvvm_f2ui_rz\0"
  "__nvvm_f2ui_rz_ftz\0"
  "__nvvm_f2ull_rm\0"
  "ULLif\0"
  "__nvvm_f2ull_rm_ftz\0"
  "__nvvm_f2ull_rn\0"
  "__nvvm_f2ull_rn_ftz\0"
  "__nvvm_f2ull_rp\0"
  "__nvvm_f2ull_rp_ftz\0"
  "__nvvm_f2ull_rz\0"
  "__nvvm_f2ull_rz_ftz\0"
  "__nvvm_f16x2_to_e4m3x2_rn\0"
  "sV2h\0"
  "__nvvm_f16x2_to_e4m3x2_rn_relu\0"
  "__nvvm_f16x2_to_e5m2x2_rn\0"
  "__nvvm_f16x2_to_e5m2x2_rn_relu\0"
  "__nvvm_fabs_d\0"
  "__nvvm_fabs_f\0"
  "__nvvm_fabs_f16\0"
  "(sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87)\0"
  "__nvvm_fabs_f16x2\0"
  "__nvvm_fabs_ftz_f\0"
  "__nvvm_fabs_ftz_f16\0"
  "__nvvm_fabs_ftz_f16x2\0"
  "__nvvm_fence_sc_cluster\0"
  "__nvvm_ff2bf16x2_rn\0"
  "V2yff\0"
  "__nvvm_ff2bf16x2_rn_relu\0"
  "__nvvm_ff2bf16x2_rz\0"
  "__nvvm_ff2bf16x2_rz_relu\0"
  "__nvvm_ff2f16x2_rn\0"
  "V2hff\0"
  "__nvvm_ff2f16x2_rn_relu\0"
  "__nvvm_ff2f16x2_rz\0"
  "__nvvm_ff2f16x2_rz_relu\0"
  "__nvvm_ff_to_e2m1x2_rn_relu_satfinite\0"
  "sff\0"
  "__nvvm_ff_to_e2m1x2_rn_satfinite\0"
  "__nvvm_ff_to_e2m3x2_rn_relu_satfinite\0"
  "__nvvm_ff_to_e2m3x2_rn_satfinite\0"
  "__nvvm_ff_to_e3m2x2_rn_relu_satfinite\0"
  "__nvvm_ff_to_e3m2x2_rn_satfinite\0"
  "__nvvm_ff_to_e4m3x2_rn\0"
  "__nvvm_ff_to_e4m3x2_rn_relu\0"
  "__nvvm_ff_to_e5m2x2_rn\0"
  "__nvvm_ff_to_e5m2x2_rn_relu\0"
  "__nvvm_ff_to_ue8m0x2_rp\0"
  "__nvvm_ff_to_ue8m0x2_rp_satfinite\0"
  "__nvvm_ff_to_ue8m0x2_rz\0"
  "__nvvm_ff_to_ue8m0x2_rz_satfinite\0"
  "__nvvm_floor_d\0"
  "__nvvm_floor_f\0"
  "__nvvm_floor_ftz_f\0"
  "__nvvm_fma_rm_d\0"
  "dddd\0"
  "__nvvm_fma_rm_f\0"
  "ffff\0"
  "__nvvm_fma_rm_ftz_f\0"
  "__nvvm_fma_rn_bf16\0"
  "yyyy\0"
  "__nvvm_fma_rn_bf16x2\0"
  "V2yV2yV2yV2y\0"
  "__nvvm_fma_rn_d\0"
  "__nvvm_fma_rn_f\0"
  "__nvvm_fma_rn_f16\0"
  "hhhh\0"
  "(sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx42|ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87)\0"
  "__nvvm_fma_rn_f16x2\0"
  "V2hV2hV2hV2h\0"
  "__nvvm_fma_rn_ftz_f\0"
  "__nvvm_fma_rn_ftz_f16\0"
  "__nvvm_fma_rn_ftz_f16x2\0"
  "__nvvm_fma_rn_ftz_relu_f16\0"
  "__nvvm_fma_rn_ftz_relu_f16x2\0"
  "__nvvm_fma_rn_ftz_sat_f16\0"
  "__nvvm_fma_rn_ftz_sat_f16x2\0"
  "__nvvm_fma_rn_relu_bf16\0"
  "__nvvm_fma_rn_relu_bf16x2\0"
  "__nvvm_fma_rn_relu_f16\0"
  "__nvvm_fma_rn_relu_f16x2\0"
  "__nvvm_fma_rn_sat_f16\0"
  "__nvvm_fma_rn_sat_f16x2\0"
  "__nvvm_fma_rp_d\0"
  "__nvvm_fma_rp_f\0"
  "__nvvm_fma_rp_ftz_f\0"
  "__nvvm_fma_rz_d\0"
  "__nvvm_fma_rz_f\0"
  "__nvvm_fma_rz_ftz_f\0"
  "__nvvm_fmax_bf16\0"
  "yyy\0"
  "__nvvm_fmax_bf16x2\0"
  "V2yV2yV2y\0"
  "__nvvm_fmax_d\0"
  "__nvvm_fmax_f\0"
  "__nvvm_fmax_f16\0"
  "hhh\0"
  "__nvvm_fmax_f16x2\0"
  "V2hV2hV2h\0"
  "__nvvm_fmax_ftz_bf16\0"
  "__nvvm_fmax_ftz_bf16x2\0"
  "__nvvm_fmax_ftz_f\0"
  "__nvvm_fmax_ftz_f16\0"
  "__nvvm_fmax_ftz_f16x2\0"
  "__nvvm_fmax_ftz_nan_bf16\0"
  "__nvvm_fmax_ftz_nan_bf16x2\0"
  "__nvvm_fmax_ftz_nan_f\0"
  "__nvvm_fmax_ftz_nan_f16\0"
  "__nvvm_fmax_ftz_nan_f16x2\0"
  "__nvvm_fmax_ftz_nan_xorsign_abs_f\0"
  "(sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87)\0"
  "__nvvm_fmax_ftz_nan_xorsign_abs_f16\0"
  "__nvvm_fmax_ftz_nan_xorsign_abs_f16x2\0"
  "__nvvm_fmax_ftz_xorsign_abs_f\0"
  "__nvvm_fmax_ftz_xorsign_abs_f16\0"
  "__nvvm_fmax_ftz_xorsign_abs_f16x2\0"
  "__nvvm_fmax_nan_bf16\0"
  "__nvvm_fmax_nan_bf16x2\0"
  "__nvvm_fmax_nan_f\0"
  "__nvvm_fmax_nan_f16\0"
  "__nvvm_fmax_nan_f16x2\0"
  "__nvvm_fmax_nan_xorsign_abs_bf16\0"
  "__nvvm_fmax_nan_xorsign_abs_bf16x2\0"
  "__nvvm_fmax_nan_xorsign_abs_f\0"
  "__nvvm_fmax_nan_xorsign_abs_f16\0"
  "__nvvm_fmax_nan_xorsign_abs_f16x2\0"
  "__nvvm_fmax_xorsign_abs_bf16\0"
  "__nvvm_fmax_xorsign_abs_bf16x2\0"
  "__nvvm_fmax_xorsign_abs_f\0"
  "__nvvm_fmax_xorsign_abs_f16\0"
  "__nvvm_fmax_xorsign_abs_f16x2\0"
  "__nvvm_fmin_bf16\0"
  "__nvvm_fmin_bf16x2\0"
  "__nvvm_fmin_d\0"
  "__nvvm_fmin_f\0"
  "__nvvm_fmin_f16\0"
  "__nvvm_fmin_f16x2\0"
  "__nvvm_fmin_ftz_bf16\0"
  "__nvvm_fmin_ftz_bf16x2\0"
  "__nvvm_fmin_ftz_f\0"
  "__nvvm_fmin_ftz_f16\0"
  "__nvvm_fmin_ftz_f16x2\0"
  "__nvvm_fmin_ftz_nan_bf16\0"
  "__nvvm_fmin_ftz_nan_bf16x2\0"
  "__nvvm_fmin_ftz_nan_f\0"
  "__nvvm_fmin_ftz_nan_f16\0"
  "__nvvm_fmin_ftz_nan_f16x2\0"
  "__nvvm_fmin_ftz_nan_xorsign_abs_f\0"
  "__nvvm_fmin_ftz_nan_xorsign_abs_f16\0"
  "__nvvm_fmin_ftz_nan_xorsign_abs_f16x2\0"
  "__nvvm_fmin_ftz_xorsign_abs_f\0"
  "__nvvm_fmin_ftz_xorsign_abs_f16\0"
  "__nvvm_fmin_ftz_xorsign_abs_f16x2\0"
  "__nvvm_fmin_nan_bf16\0"
  "__nvvm_fmin_nan_bf16x2\0"
  "__nvvm_fmin_nan_f\0"
  "__nvvm_fmin_nan_f16\0"
  "__nvvm_fmin_nan_f16x2\0"
  "__nvvm_fmin_nan_xorsign_abs_bf16\0"
  "__nvvm_fmin_nan_xorsign_abs_bf16x2\0"
  "__nvvm_fmin_nan_xorsign_abs_f\0"
  "__nvvm_fmin_nan_xorsign_abs_f16\0"
  "__nvvm_fmin_nan_xorsign_abs_f16x2\0"
  "__nvvm_fmin_xorsign_abs_bf16\0"
  "__nvvm_fmin_xorsign_abs_bf16x2\0"
  "__nvvm_fmin_xorsign_abs_f\0"
  "__nvvm_fmin_xorsign_abs_f16\0"
  "__nvvm_fmin_xorsign_abs_f16x2\0"
  "__nvvm_fns\0"
  "UiUiUii\0"
  "__nvvm_getctarank\0"
  "iv*\0"
  "__nvvm_getctarank_shared_cluster\0"
  "iv*3\0"
  "__nvvm_i2d_rm\0"
  "di\0"
  "__nvvm_i2d_rn\0"
  "__nvvm_i2d_rp\0"
  "__nvvm_i2d_rz\0"
  "__nvvm_i2f_rm\0"
  "fi\0"
  "__nvvm_i2f_rn\0"
  "__nvvm_i2f_rp\0"
  "__nvvm_i2f_rz\0"
  "__nvvm_is_explicit_cluster\0"
  "b\0"
  "nc\0"
  "__nvvm_isspacep_const\0"
  "bvC*\0"
  "__nvvm_isspacep_global\0"
  "__nvvm_isspacep_local\0"
  "__nvvm_isspacep_shared\0"
  "__nvvm_isspacep_shared_cluster\0"
  "__nvvm_ldg_c\0"
  "ccC*\0"
  "__nvvm_ldg_c2\0"
  "E2cE2cC*\0"
  "__nvvm_ldg_c4\0"
  "E4cE4cC*\0"
  "__nvvm_ldg_d\0"
  "ddC*\0"
  "__nvvm_ldg_d2\0"
  "E2dE2dC*\0"
  "__nvvm_ldg_f\0"
  "ffC*\0"
  "__nvvm_ldg_f2\0"
  "E2fE2fC*\0"
  "__nvvm_ldg_f4\0"
  "E4fE4fC*\0"
  "__nvvm_ldg_h\0"
  "hhC*\0"
  "__nvvm_ldg_h2\0"
  "E2hE2hC*\0"
  "__nvvm_ldg_i\0"
  "iiC*\0"
  "__nvvm_ldg_i2\0"
  "E2iE2iC*\0"
  "__nvvm_ldg_i4\0"
  "E4iE4iC*\0"
  "__nvvm_ldg_l\0"
  "LiLiC*\0"
  "__nvvm_ldg_l2\0"
  "E2LiE2LiC*\0"
  "__nvvm_ldg_ll\0"
  "LLiLLiC*\0"
  "__nvvm_ldg_ll2\0"
  "E2LLiE2LLiC*\0"
  "__nvvm_ldg_s\0"
  "ssC*\0"
  "__nvvm_ldg_s2\0"
  "E2sE2sC*\0"
  "__nvvm_ldg_s4\0"
  "E4sE4sC*\0"
  "__nvvm_ldg_sc\0"
  "ScScC*\0"
  "__nvvm_ldg_sc2\0"
  "E2ScE2ScC*\0"
  "__nvvm_ldg_sc4\0"
  "E4ScE4ScC*\0"
  "__nvvm_ldg_uc\0"
  "UcUcC*\0"
  "__nvvm_ldg_uc2\0"
  "E2UcE2UcC*\0"
  "__nvvm_ldg_uc4\0"
  "E4UcE4UcC*\0"
  "__nvvm_ldg_ui\0"
  "UiUiC*\0"
  "__nvvm_ldg_ui2\0"
  "E2UiE2UiC*\0"
  "__nvvm_ldg_ui4\0"
  "E4UiE4UiC*\0"
  "__nvvm_ldg_ul\0"
  "ULiULiC*\0"
  "__nvvm_ldg_ul2\0"
  "E2ULiE2ULiC*\0"
  "__nvvm_ldg_ull\0"
  "ULLiULLiC*\0"
  "__nvvm_ldg_ull2\0"
  "E2ULLiE2ULLiC*\0"
  "__nvvm_ldg_us\0"
  "UsUsC*\0"
  "__nvvm_ldg_us2\0"
  "E2UsE2UsC*\0"
  "__nvvm_ldg_us4\0"
  "E4UsE4UsC*\0"
  "__nvvm_ldu_c\0"
  "__nvvm_ldu_c2\0"
  "__nvvm_ldu_c4\0"
  "__nvvm_ldu_d\0"
  "__nvvm_ldu_d2\0"
  "__nvvm_ldu_f\0"
  "__nvvm_ldu_f2\0"
  "__nvvm_ldu_f4\0"
  "__nvvm_ldu_h\0"
  "__nvvm_ldu_h2\0"
  "__nvvm_ldu_i\0"
  "__nvvm_ldu_i2\0"
  "__nvvm_ldu_i4\0"
  "__nvvm_ldu_l\0"
  "__nvvm_ldu_l2\0"
  "__nvvm_ldu_ll\0"
  "__nvvm_ldu_ll2\0"
  "__nvvm_ldu_s\0"
  "__nvvm_ldu_s2\0"
  "__nvvm_ldu_s4\0"
  "__nvvm_ldu_sc\0"
  "__nvvm_ldu_sc2\0"
  "__nvvm_ldu_sc4\0"
  "__nvvm_ldu_uc\0"
  "__nvvm_ldu_uc2\0"
  "__nvvm_ldu_uc4\0"
  "__nvvm_ldu_ui\0"
  "__nvvm_ldu_ui2\0"
  "__nvvm_ldu_ui4\0"
  "__nvvm_ldu_ul\0"
  "__nvvm_ldu_ul2\0"
  "__nvvm_ldu_ull\0"
  "__nvvm_ldu_ull2\0"
  "__nvvm_ldu_us\0"
  "__nvvm_ldu_us2\0"
  "__nvvm_ldu_us4\0"
  "__nvvm_lg2_approx_d\0"
  "__nvvm_lg2_approx_f\0"
  "__nvvm_lg2_approx_ftz_f\0"
  "__nvvm_ll2d_rm\0"
  "dLLi\0"
  "__nvvm_ll2d_rn\0"
  "__nvvm_ll2d_rp\0"
  "__nvvm_ll2d_rz\0"
  "__nvvm_ll2f_rm\0"
  "fLLi\0"
  "__nvvm_ll2f_rn\0"
  "__nvvm_ll2f_rp\0"
  "__nvvm_ll2f_rz\0"
  "__nvvm_lohi_i2d\0"
  "dii\0"
  "__nvvm_mapa\0"
  "v*v*i\0"
  "__nvvm_mapa_shared_cluster\0"
  "v*3v*3i\0"
  "__nvvm_match_all_sync_i32p\0"
  "UiUiUii*\0"
  "__nvvm_match_all_sync_i64p\0"
  "UiUiWii*\0"
  "__nvvm_match_any_sync_i32\0"
  "UiUiUi\0"
  "__nvvm_match_any_sync_i64\0"
  "UiUiWi\0"
  "__nvvm_mbarrier_arrive\0"
  "WiWi*\0"
  "__nvvm_mbarrier_arrive_drop\0"
  "__nvvm_mbarrier_arrive_drop_noComplete\0"
  "WiWi*i\0"
  "__nvvm_mbarrier_arrive_drop_noComplete_shared\0"
  "WiWi*3i\0"
  "__nvvm_mbarrier_arrive_drop_shared\0"
  "WiWi*3\0"
  "__nvvm_mbarrier_arrive_noComplete\0"
  "__nvvm_mbarrier_arrive_noComplete_shared\0"
  "__nvvm_mbarrier_arrive_shared\0"
  "__nvvm_mbarrier_init\0"
  "vWi*i\0"
  "__nvvm_mbarrier_init_shared\0"
  "vWi*3i\0"
  "__nvvm_mbarrier_inval\0"
  "__nvvm_mbarrier_inval_shared\0"
  "__nvvm_mbarrier_pending_count\0"
  "iWi\0"
  "__nvvm_mbarrier_test_wait\0"
  "bWi*Wi\0"
  "__nvvm_mbarrier_test_wait_shared\0"
  "bWi*3Wi\0"
  "__nvvm_membar_cta\0"
  "__nvvm_membar_gl\0"
  "__nvvm_membar_sys\0"
  "__nvvm_memcpy\0"
  "vUc*Uc*zi\0"
  "__nvvm_memset\0"
  "vUc*Uczi\0"
  "__nvvm_mul24_i\0"
  "iii\0"
  "__nvvm_mul24_ui\0"
  "__nvvm_mul_rm_d\0"
  "__nvvm_mul_rm_f\0"
  "__nvvm_mul_rm_ftz_f\0"
  "__nvvm_mul_rn_d\0"
  "__nvvm_mul_rn_f\0"
  "__nvvm_mul_rn_ftz_f\0"
  "__nvvm_mul_rp_d\0"
  "__nvvm_mul_rp_f\0"
  "__nvvm_mul_rp_ftz_f\0"
  "__nvvm_mul_rz_d\0"
  "__nvvm_mul_rz_f\0"
  "__nvvm_mul_rz_ftz_f\0"
  "__nvvm_mulhi_i\0"
  "__nvvm_mulhi_ll\0"
  "LLiLLiLLi\0"
  "__nvvm_mulhi_ui\0"
  "__nvvm_mulhi_ull\0"
  "ULLiULLiULLi\0"
  "__nvvm_nanosleep\0"
  "(sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87)\0"
  "__nvvm_neg_bf16\0"
  "__nvvm_neg_bf16x2\0"
  "__nvvm_pm_event_mask\0"
  "vIUs\0"
  "__nvvm_prmt\0"
  "UiUiUiUi\0"
  "__nvvm_rcp_approx_ftz_d\0"
  "__nvvm_rcp_approx_ftz_f\0"
  "__nvvm_rcp_rm_d\0"
  "__nvvm_rcp_rm_f\0"
  "__nvvm_rcp_rm_ftz_f\0"
  "__nvvm_rcp_rn_d\0"
  "__nvvm_rcp_rn_f\0"
  "__nvvm_rcp_rn_ftz_f\0"
  "__nvvm_rcp_rp_d\0"
  "__nvvm_rcp_rp_f\0"
  "__nvvm_rcp_rp_ftz_f\0"
  "__nvvm_rcp_rz_d\0"
  "__nvvm_rcp_rz_f\0"
  "__nvvm_rcp_rz_ftz_f\0"
  "__nvvm_read_ptx_sreg_clock\0"
  "i\0"
  "__nvvm_read_ptx_sreg_clock64\0"
  "LLi\0"
  "__nvvm_read_ptx_sreg_cluster_ctaid_w\0"
  "__nvvm_read_ptx_sreg_cluster_ctaid_x\0"
  "__nvvm_read_ptx_sreg_cluster_ctaid_y\0"
  "__nvvm_read_ptx_sreg_cluster_ctaid_z\0"
  "__nvvm_read_ptx_sreg_cluster_ctarank\0"
  "__nvvm_read_ptx_sreg_cluster_nctaid_w\0"
  "__nvvm_read_ptx_sreg_cluster_nctaid_x\0"
  "__nvvm_read_ptx_sreg_cluster_nctaid_y\0"
  "__nvvm_read_ptx_sreg_cluster_nctaid_z\0"
  "__nvvm_read_ptx_sreg_cluster_nctarank\0"
  "__nvvm_read_ptx_sreg_clusterid_w\0"
  "__nvvm_read_ptx_sreg_clusterid_x\0"
  "__nvvm_read_ptx_sreg_clusterid_y\0"
  "__nvvm_read_ptx_sreg_clusterid_z\0"
  "__nvvm_read_ptx_sreg_ctaid_w\0"
  "__nvvm_read_ptx_sreg_ctaid_x\0"
  "__nvvm_read_ptx_sreg_ctaid_y\0"
  "__nvvm_read_ptx_sreg_ctaid_z\0"
  "__nvvm_read_ptx_sreg_globaltimer\0"
  "__nvvm_read_ptx_sreg_gridid\0"
  "__nvvm_read_ptx_sreg_laneid\0"
  "__nvvm_read_ptx_sreg_lanemask_eq\0"
  "__nvvm_read_ptx_sreg_lanemask_ge\0"
  "__nvvm_read_ptx_sreg_lanemask_gt\0"
  "__nvvm_read_ptx_sreg_lanemask_le\0"
  "__nvvm_read_ptx_sreg_lanemask_lt\0"
  "__nvvm_read_ptx_sreg_nclusterid_w\0"
  "__nvvm_read_ptx_sreg_nclusterid_x\0"
  "__nvvm_read_ptx_sreg_nclusterid_y\0"
  "__nvvm_read_ptx_sreg_nclusterid_z\0"
  "__nvvm_read_ptx_sreg_nctaid_w\0"
  "__nvvm_read_ptx_sreg_nctaid_x\0"
  "__nvvm_read_ptx_sreg_nctaid_y\0"
  "__nvvm_read_ptx_sreg_nctaid_z\0"
  "__nvvm_read_ptx_sreg_nsmid\0"
  "__nvvm_read_ptx_sreg_ntid_w\0"
  "__nvvm_read_ptx_sreg_ntid_x\0"
  "__nvvm_read_ptx_sreg_ntid_y\0"
  "__nvvm_read_ptx_sreg_ntid_z\0"
  "__nvvm_read_ptx_sreg_nwarpid\0"
  "__nvvm_read_ptx_sreg_pm0\0"
  "__nvvm_read_ptx_sreg_pm1\0"
  "__nvvm_read_ptx_sreg_pm2\0"
  "__nvvm_read_ptx_sreg_pm3\0"
  "__nvvm_read_ptx_sreg_smid\0"
  "__nvvm_read_ptx_sreg_tid_w\0"
  "__nvvm_read_ptx_sreg_tid_x\0"
  "__nvvm_read_ptx_sreg_tid_y\0"
  "__nvvm_read_ptx_sreg_tid_z\0"
  "__nvvm_read_ptx_sreg_warpid\0"
  "__nvvm_read_ptx_sreg_warpsize\0"
  "__nvvm_redux_sync_add\0"
  "__nvvm_redux_sync_and\0"
  "__nvvm_redux_sync_fmax\0"
  "ffi\0"
  "(sm_100a),(ptx86|ptx87)\0"
  "__nvvm_redux_sync_fmax_NaN\0"
  "__nvvm_redux_sync_fmax_abs\0"
  "__nvvm_redux_sync_fmax_abs_NaN\0"
  "__nvvm_redux_sync_fmin\0"
  "__nvvm_redux_sync_fmin_NaN\0"
  "__nvvm_redux_sync_fmin_abs\0"
  "__nvvm_redux_sync_fmin_abs_NaN\0"
  "__nvvm_redux_sync_max\0"
  "__nvvm_redux_sync_min\0"
  "__nvvm_redux_sync_or\0"
  "__nvvm_redux_sync_umax\0"
  "UiUii\0"
  "__nvvm_redux_sync_umin\0"
  "__nvvm_redux_sync_xor\0"
  "__nvvm_reflect\0"
  "UicC*\0"
  "__nvvm_round_d\0"
  "__nvvm_round_f\0"
  "__nvvm_round_ftz_f\0"
  "__nvvm_rsqrt_approx_d\0"
  "__nvvm_rsqrt_approx_f\0"
  "__nvvm_rsqrt_approx_ftz_f\0"
  "__nvvm_sad_i\0"
  "iiii\0"
  "__nvvm_sad_ui\0"
  "__nvvm_saturate_d\0"
  "__nvvm_saturate_f\0"
  "__nvvm_saturate_ftz_f\0"
  "__nvvm_shfl_bfly_f32\0"
  "ffii\0"
  "__nvvm_shfl_bfly_i32\0"
  "__nvvm_shfl_down_f32\0"
  "__nvvm_shfl_down_i32\0"
  "__nvvm_shfl_idx_f32\0"
  "__nvvm_shfl_idx_i32\0"
  "__nvvm_shfl_sync_bfly_f32\0"
  "fUifii\0"
  "__nvvm_shfl_sync_bfly_i32\0"
  "iUiiii\0"
  "__nvvm_shfl_sync_down_f32\0"
  "__nvvm_shfl_sync_down_i32\0"
  "__nvvm_shfl_sync_idx_f32\0"
  "__nvvm_shfl_sync_idx_i32\0"
  "__nvvm_shfl_sync_up_f32\0"
  "__nvvm_shfl_sync_up_i32\0"
  "__nvvm_shfl_up_f32\0"
  "__nvvm_shfl_up_i32\0"
  "__nvvm_sin_approx_f\0"
  "__nvvm_sin_approx_ftz_f\0"
  "__nvvm_sqrt_approx_f\0"
  "__nvvm_sqrt_approx_ftz_f\0"
  "__nvvm_sqrt_rm_d\0"
  "__nvvm_sqrt_rm_f\0"
  "__nvvm_sqrt_rm_ftz_f\0"
  "__nvvm_sqrt_rn_d\0"
  "__nvvm_sqrt_rn_f\0"
  "__nvvm_sqrt_rn_ftz_f\0"
  "__nvvm_sqrt_rp_d\0"
  "__nvvm_sqrt_rp_f\0"
  "__nvvm_sqrt_rp_ftz_f\0"
  "__nvvm_sqrt_rz_d\0"
  "__nvvm_sqrt_rz_f\0"
  "__nvvm_sqrt_rz_ftz_f\0"
  "__nvvm_trunc_d\0"
  "__nvvm_trunc_f\0"
  "__nvvm_trunc_ftz_f\0"
  "__nvvm_ue8m0x2_to_bf16x2\0"
  "V2ys\0"
  "__nvvm_ui2d_rm\0"
  "dUi\0"
  "__nvvm_ui2d_rn\0"
  "__nvvm_ui2d_rp\0"
  "__nvvm_ui2d_rz\0"
  "__nvvm_ui2f_rm\0"
  "fUi\0"
  "__nvvm_ui2f_rn\0"
  "__nvvm_ui2f_rp\0"
  "__nvvm_ui2f_rz\0"
  "__nvvm_ull2d_rm\0"
  "dULLi\0"
  "__nvvm_ull2d_rn\0"
  "__nvvm_ull2d_rp\0"
  "__nvvm_ull2d_rz\0"
  "__nvvm_ull2f_rm\0"
  "fULLi\0"
  "__nvvm_ull2f_rn\0"
  "__nvvm_ull2f_rp\0"
  "__nvvm_ull2f_rz\0"
  "__nvvm_vote_all\0"
  "bb\0"
  "__nvvm_vote_all_sync\0"
  "bUib\0"
  "__nvvm_vote_any\0"
  "__nvvm_vote_any_sync\0"
  "__nvvm_vote_ballot\0"
  "Uib\0"
  "__nvvm_vote_ballot_sync\0"
  "UiUib\0"
  "__nvvm_vote_uni\0"
  "__nvvm_vote_uni_sync\0"
  "__syncthreads\0"
  ;
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif

static constexpr llvm::StringTable
BuiltinStrings = BuiltinStringsStorage;

#endif // GET_BUILTIN_STR_TABLE

#ifdef GET_BUILTIN_INFOS
    Builtin::Info{Builtin::Info::StrOffsets{1 /* __bmma_m8n8k128_ld_a_b1 */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{248 /* __bmma_m8n8k128_ld_b_b1 */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{272 /* __bmma_m8n8k128_ld_c */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{293 /* __bmma_m8n8k128_mma_and_popc_b1 */, 325 /* vi*iC*iC*iC*Ii */, 0 /*  */, 340 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{522 /* __bmma_m8n8k128_mma_xor_popc_b1 */, 325 /* vi*iC*iC*iC*Ii */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{554 /* __bmma_m8n8k128_st_c_i32 */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{579 /* __builtin_ptx_get_image_channel_data_typei_ */, 623 /* ii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{626 /* __builtin_ptx_get_image_channel_orderi_ */, 623 /* ii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{666 /* __builtin_ptx_get_image_depthi_ */, 623 /* ii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{698 /* __builtin_ptx_get_image_heighti_ */, 623 /* ii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{731 /* __builtin_ptx_get_image_widthi_ */, 623 /* ii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{763 /* __builtin_ptx_read_image2Dff_ */, 793 /* V4fiiff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{801 /* __builtin_ptx_read_image2Dfi_ */, 831 /* V4fiiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{839 /* __builtin_ptx_read_image2Dif_ */, 869 /* V4iiiff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{877 /* __builtin_ptx_read_image2Dii_ */, 907 /* V4iiiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{915 /* __builtin_ptx_read_image3Dff_ */, 945 /* V4fiiffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{955 /* __builtin_ptx_read_image3Dfi_ */, 985 /* V4fiiiiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{995 /* __builtin_ptx_read_image3Dif_ */, 1025 /* V4iiiffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1035 /* __builtin_ptx_read_image3Dii_ */, 1065 /* V4iiiiiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1075 /* __builtin_ptx_write_image2Df_ */, 1105 /* viiiffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1114 /* __builtin_ptx_write_image2Di_ */, 1144 /* viiiiiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1153 /* __builtin_ptx_write_image2Dui_ */, 1184 /* viiiUiUiUiUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1197 /* __dmma_m8n8k4_ld_a */, 1216 /* vd*dC*UiIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1415 /* __dmma_m8n8k4_ld_b */, 1216 /* vd*dC*UiIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1434 /* __dmma_m8n8k4_ld_c */, 1216 /* vd*dC*UiIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1453 /* __dmma_m8n8k4_mma_f64 */, 1475 /* vd*dC*dC*dC*IiIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1492 /* __dmma_m8n8k4_st_c_f64 */, 1216 /* vd*dC*UiIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1515 /* __hmma_m8n32k16_ld_a */, 25 /* vi*iC*UiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1772 /* __hmma_m8n32k16_ld_b */, 25 /* vi*iC*UiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1793 /* __hmma_m8n32k16_ld_c_f16 */, 25 /* vi*iC*UiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1818 /* __hmma_m8n32k16_ld_c_f32 */, 1843 /* vf*fC*UiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1854 /* __hmma_m8n32k16_mma_f16f16 */, 1881 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1898 /* __hmma_m8n32k16_mma_f16f32 */, 1925 /* vi*iC*iC*fC*IiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1942 /* __hmma_m8n32k16_mma_f32f16 */, 1969 /* vf*iC*iC*iC*IiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1986 /* __hmma_m8n32k16_mma_f32f32 */, 2013 /* vf*iC*iC*fC*IiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2030 /* __hmma_m8n32k16_st_c_f16 */, 25 /* vi*iC*UiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2055 /* __hmma_m8n32k16_st_c_f32 */, 1843 /* vf*fC*UiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2080 /* __hmma_m16n16k16_ld_a */, 25 /* vi*iC*UiIi */, 0 /*  */, 2102 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2344 /* __hmma_m16n16k16_ld_b */, 25 /* vi*iC*UiIi */, 0 /*  */, 2102 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2366 /* __hmma_m16n16k16_ld_c_f16 */, 25 /* vi*iC*UiIi */, 0 /*  */, 2102 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2392 /* __hmma_m16n16k16_ld_c_f32 */, 1843 /* vf*fC*UiIi */, 0 /*  */, 2102 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2418 /* __hmma_m16n16k16_mma_f16f16 */, 1881 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 2102 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2446 /* __hmma_m16n16k16_mma_f16f32 */, 1925 /* vi*iC*iC*fC*IiIi */, 0 /*  */, 2102 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2474 /* __hmma_m16n16k16_mma_f32f16 */, 1969 /* vf*iC*iC*iC*IiIi */, 0 /*  */, 2102 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2502 /* __hmma_m16n16k16_mma_f32f32 */, 2013 /* vf*iC*iC*fC*IiIi */, 0 /*  */, 2102 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2530 /* __hmma_m16n16k16_st_c_f16 */, 25 /* vi*iC*UiIi */, 0 /*  */, 2102 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2556 /* __hmma_m16n16k16_st_c_f32 */, 1843 /* vf*fC*UiIi */, 0 /*  */, 2102 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2582 /* __hmma_m32n8k16_ld_a */, 25 /* vi*iC*UiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2603 /* __hmma_m32n8k16_ld_b */, 25 /* vi*iC*UiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2624 /* __hmma_m32n8k16_ld_c_f16 */, 25 /* vi*iC*UiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2649 /* __hmma_m32n8k16_ld_c_f32 */, 1843 /* vf*fC*UiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2674 /* __hmma_m32n8k16_mma_f16f16 */, 1881 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2701 /* __hmma_m32n8k16_mma_f16f32 */, 1925 /* vi*iC*iC*fC*IiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2728 /* __hmma_m32n8k16_mma_f32f16 */, 1969 /* vf*iC*iC*iC*IiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2755 /* __hmma_m32n8k16_mma_f32f32 */, 2013 /* vf*iC*iC*fC*IiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2782 /* __hmma_m32n8k16_st_c_f16 */, 25 /* vi*iC*UiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2807 /* __hmma_m32n8k16_st_c_f32 */, 1843 /* vf*fC*UiIi */, 0 /*  */, 1536 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2832 /* __imma_m8n8k32_ld_a_s4 */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2855 /* __imma_m8n8k32_ld_a_u4 */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2878 /* __imma_m8n8k32_ld_b_s4 */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2901 /* __imma_m8n8k32_ld_b_u4 */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2924 /* __imma_m8n8k32_ld_c */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2944 /* __imma_m8n8k32_mma_s4 */, 1881 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2966 /* __imma_m8n8k32_mma_u4 */, 1881 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2988 /* __imma_m8n8k32_st_c_i32 */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3012 /* __imma_m8n32k16_ld_a_s8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3254 /* __imma_m8n32k16_ld_a_u8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3278 /* __imma_m8n32k16_ld_b_s8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3302 /* __imma_m8n32k16_ld_b_u8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3326 /* __imma_m8n32k16_ld_c */, 25 /* vi*iC*UiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3347 /* __imma_m8n32k16_mma_s8 */, 1881 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3370 /* __imma_m8n32k16_mma_u8 */, 1881 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3393 /* __imma_m8n32k16_st_c_i32 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3418 /* __imma_m16n16k16_ld_a_s8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3443 /* __imma_m16n16k16_ld_a_u8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3468 /* __imma_m16n16k16_ld_b_s8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3493 /* __imma_m16n16k16_ld_b_u8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3518 /* __imma_m16n16k16_ld_c */, 25 /* vi*iC*UiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3540 /* __imma_m16n16k16_mma_s8 */, 1881 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3564 /* __imma_m16n16k16_mma_u8 */, 1881 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3588 /* __imma_m16n16k16_st_c_i32 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3614 /* __imma_m32n8k16_ld_a_s8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3638 /* __imma_m32n8k16_ld_a_u8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3662 /* __imma_m32n8k16_ld_b_s8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3686 /* __imma_m32n8k16_ld_b_u8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3710 /* __imma_m32n8k16_ld_c */, 25 /* vi*iC*UiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3731 /* __imma_m32n8k16_mma_s8 */, 1881 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3754 /* __imma_m32n8k16_mma_u8 */, 1881 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3777 /* __imma_m32n8k16_st_c_i32 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3036 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3802 /* __mma_bf16_m8n32k16_ld_a */, 25 /* vi*iC*UiIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3827 /* __mma_bf16_m8n32k16_ld_b */, 25 /* vi*iC*UiIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3852 /* __mma_bf16_m8n32k16_mma_f32 */, 2013 /* vf*iC*iC*fC*IiIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3880 /* __mma_bf16_m16n16k16_ld_a */, 25 /* vi*iC*UiIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3906 /* __mma_bf16_m16n16k16_ld_b */, 25 /* vi*iC*UiIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3932 /* __mma_bf16_m16n16k16_mma_f32 */, 2013 /* vf*iC*iC*fC*IiIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3961 /* __mma_bf16_m32n8k16_ld_a */, 25 /* vi*iC*UiIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3986 /* __mma_bf16_m32n8k16_ld_b */, 25 /* vi*iC*UiIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4011 /* __mma_bf16_m32n8k16_mma_f32 */, 2013 /* vf*iC*iC*fC*IiIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4039 /* __mma_m16n16k8_st_c_f32 */, 1843 /* vf*fC*UiIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4063 /* __mma_tf32_m16n16k8_ld_a */, 25 /* vi*iC*UiIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4088 /* __mma_tf32_m16n16k8_ld_b */, 25 /* vi*iC*UiIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4113 /* __mma_tf32_m16n16k8_ld_c */, 1843 /* vf*fC*UiIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4138 /* __mma_tf32_m16n16k8_mma_f32 */, 2013 /* vf*iC*iC*fC*IiIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4166 /* __nvvm_abs_bf16 */, 4182 /* yy */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4185 /* __nvvm_abs_bf16x2 */, 4203 /* V2yV2y */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4210 /* __nvvm_activemask */, 4228 /* Ui */, 4231 /* n */, 4233 /* ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4359 /* __nvvm_add_rm_d */, 4375 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4379 /* __nvvm_add_rm_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4399 /* __nvvm_add_rm_ftz_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4419 /* __nvvm_add_rn_d */, 4375 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4435 /* __nvvm_add_rn_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4451 /* __nvvm_add_rn_ftz_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4471 /* __nvvm_add_rp_d */, 4375 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4487 /* __nvvm_add_rp_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4503 /* __nvvm_add_rp_ftz_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4523 /* __nvvm_add_rz_d */, 4375 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4539 /* __nvvm_add_rz_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4555 /* __nvvm_add_rz_ftz_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4575 /* __nvvm_atom_add_gen_d */, 4597 /* ddD*d */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4721 /* __nvvm_atom_add_gen_f */, 4743 /* ffD*f */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4749 /* __nvvm_atom_add_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4777 /* __nvvm_atom_add_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4808 /* __nvvm_atom_add_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4843 /* __nvvm_atom_and_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4865 /* __nvvm_atom_and_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4887 /* __nvvm_atom_and_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4910 /* __nvvm_atom_cas_gen_i */, 4932 /* iiD*ii */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4939 /* __nvvm_atom_cas_gen_l */, 4961 /* LiLiD*LiLi */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4972 /* __nvvm_atom_cas_gen_ll */, 4995 /* LLiLLiD*LLiLLi */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5010 /* __nvvm_atom_cas_gen_us */, 5033 /* UsUsD*UsUs */, 4231 /* n */, 5044 /* sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5144 /* __nvvm_atom_cta_add_gen_d */, 4597 /* ddD*d */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5170 /* __nvvm_atom_cta_add_gen_f */, 4743 /* ffD*f */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5196 /* __nvvm_atom_cta_add_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5222 /* __nvvm_atom_cta_add_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5248 /* __nvvm_atom_cta_add_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5275 /* __nvvm_atom_cta_and_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5301 /* __nvvm_atom_cta_and_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5327 /* __nvvm_atom_cta_and_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5354 /* __nvvm_atom_cta_cas_gen_i */, 4932 /* iiD*ii */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5380 /* __nvvm_atom_cta_cas_gen_l */, 4961 /* LiLiD*LiLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5406 /* __nvvm_atom_cta_cas_gen_ll */, 4995 /* LLiLLiD*LLiLLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5433 /* __nvvm_atom_cta_cas_gen_us */, 5033 /* UsUsD*UsUs */, 4231 /* n */, 5044 /* sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5460 /* __nvvm_atom_cta_dec_gen_ui */, 5487 /* UiUiD*Ui */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5496 /* __nvvm_atom_cta_inc_gen_ui */, 5487 /* UiUiD*Ui */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5523 /* __nvvm_atom_cta_max_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5549 /* __nvvm_atom_cta_max_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5575 /* __nvvm_atom_cta_max_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5602 /* __nvvm_atom_cta_max_gen_ui */, 5487 /* UiUiD*Ui */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5629 /* __nvvm_atom_cta_max_gen_ul */, 5656 /* ULiULiD*ULi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5668 /* __nvvm_atom_cta_max_gen_ull */, 5696 /* ULLiULLiD*ULLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5711 /* __nvvm_atom_cta_min_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5737 /* __nvvm_atom_cta_min_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5763 /* __nvvm_atom_cta_min_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5790 /* __nvvm_atom_cta_min_gen_ui */, 5487 /* UiUiD*Ui */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5817 /* __nvvm_atom_cta_min_gen_ul */, 5656 /* ULiULiD*ULi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5844 /* __nvvm_atom_cta_min_gen_ull */, 5696 /* ULLiULLiD*ULLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5872 /* __nvvm_atom_cta_or_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5897 /* __nvvm_atom_cta_or_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5922 /* __nvvm_atom_cta_or_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5948 /* __nvvm_atom_cta_xchg_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5975 /* __nvvm_atom_cta_xchg_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6002 /* __nvvm_atom_cta_xchg_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6030 /* __nvvm_atom_cta_xor_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6056 /* __nvvm_atom_cta_xor_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6082 /* __nvvm_atom_cta_xor_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6109 /* __nvvm_atom_dec_gen_ui */, 5487 /* UiUiD*Ui */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6132 /* __nvvm_atom_inc_gen_ui */, 5487 /* UiUiD*Ui */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6155 /* __nvvm_atom_max_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6177 /* __nvvm_atom_max_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6199 /* __nvvm_atom_max_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6222 /* __nvvm_atom_max_gen_ui */, 5487 /* UiUiD*Ui */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6245 /* __nvvm_atom_max_gen_ul */, 5656 /* ULiULiD*ULi */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6268 /* __nvvm_atom_max_gen_ull */, 5696 /* ULLiULLiD*ULLi */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6292 /* __nvvm_atom_min_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6314 /* __nvvm_atom_min_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6336 /* __nvvm_atom_min_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6359 /* __nvvm_atom_min_gen_ui */, 5487 /* UiUiD*Ui */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6382 /* __nvvm_atom_min_gen_ul */, 5656 /* ULiULiD*ULi */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6405 /* __nvvm_atom_min_gen_ull */, 5696 /* ULLiULLiD*ULLi */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6429 /* __nvvm_atom_or_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6450 /* __nvvm_atom_or_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6471 /* __nvvm_atom_or_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6493 /* __nvvm_atom_sub_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6515 /* __nvvm_atom_sub_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6537 /* __nvvm_atom_sub_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6560 /* __nvvm_atom_sys_add_gen_d */, 4597 /* ddD*d */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6586 /* __nvvm_atom_sys_add_gen_f */, 4743 /* ffD*f */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6612 /* __nvvm_atom_sys_add_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6638 /* __nvvm_atom_sys_add_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6664 /* __nvvm_atom_sys_add_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6691 /* __nvvm_atom_sys_and_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6717 /* __nvvm_atom_sys_and_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6743 /* __nvvm_atom_sys_and_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6770 /* __nvvm_atom_sys_cas_gen_i */, 4932 /* iiD*ii */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6796 /* __nvvm_atom_sys_cas_gen_l */, 4961 /* LiLiD*LiLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6822 /* __nvvm_atom_sys_cas_gen_ll */, 4995 /* LLiLLiD*LLiLLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6849 /* __nvvm_atom_sys_cas_gen_us */, 5033 /* UsUsD*UsUs */, 4231 /* n */, 5044 /* sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6876 /* __nvvm_atom_sys_dec_gen_ui */, 5487 /* UiUiD*Ui */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6903 /* __nvvm_atom_sys_inc_gen_ui */, 5487 /* UiUiD*Ui */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6930 /* __nvvm_atom_sys_max_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6956 /* __nvvm_atom_sys_max_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6982 /* __nvvm_atom_sys_max_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7009 /* __nvvm_atom_sys_max_gen_ui */, 5487 /* UiUiD*Ui */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7036 /* __nvvm_atom_sys_max_gen_ul */, 5656 /* ULiULiD*ULi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7063 /* __nvvm_atom_sys_max_gen_ull */, 5696 /* ULLiULLiD*ULLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7091 /* __nvvm_atom_sys_min_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7117 /* __nvvm_atom_sys_min_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7143 /* __nvvm_atom_sys_min_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7170 /* __nvvm_atom_sys_min_gen_ui */, 5487 /* UiUiD*Ui */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7197 /* __nvvm_atom_sys_min_gen_ul */, 5656 /* ULiULiD*ULi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7224 /* __nvvm_atom_sys_min_gen_ull */, 5696 /* ULLiULLiD*ULLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7252 /* __nvvm_atom_sys_or_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7277 /* __nvvm_atom_sys_or_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7302 /* __nvvm_atom_sys_or_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7328 /* __nvvm_atom_sys_xchg_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7355 /* __nvvm_atom_sys_xchg_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7382 /* __nvvm_atom_sys_xchg_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7410 /* __nvvm_atom_sys_xor_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7436 /* __nvvm_atom_sys_xor_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7462 /* __nvvm_atom_sys_xor_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 4603 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7489 /* __nvvm_atom_xchg_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7512 /* __nvvm_atom_xchg_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7535 /* __nvvm_atom_xchg_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7559 /* __nvvm_atom_xor_gen_i */, 4771 /* iiD*i */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7581 /* __nvvm_atom_xor_gen_l */, 4799 /* LiLiD*Li */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7603 /* __nvvm_atom_xor_gen_ll */, 4831 /* LLiLLiD*LLi */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7626 /* __nvvm_bar0_and */, 623 /* ii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7642 /* __nvvm_bar0_or */, 623 /* ii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7657 /* __nvvm_bar0_popc */, 623 /* ii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7674 /* __nvvm_bar_sync */, 7690 /* vi */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7693 /* __nvvm_bar_warp_sync */, 7714 /* vUi */, 4231 /* n */, 7718 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7856 /* __nvvm_barrier_cluster_arrive */, 7886 /* v */, 4231 /* n */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8004 /* __nvvm_barrier_cluster_arrive_relaxed */, 7886 /* v */, 4231 /* n */, 8042 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8152 /* __nvvm_barrier_cluster_wait */, 7886 /* v */, 4231 /* n */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8180 /* __nvvm_barrier_sync */, 7714 /* vUi */, 4231 /* n */, 7718 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8200 /* __nvvm_barrier_sync_cnt */, 8224 /* vUiUi */, 4231 /* n */, 7718 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8230 /* __nvvm_bf16x2_to_ue8m0x2_rp */, 8258 /* sV2y */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8303 /* __nvvm_bf16x2_to_ue8m0x2_rp_satfinite */, 8258 /* sV2y */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8341 /* __nvvm_bf16x2_to_ue8m0x2_rz */, 8258 /* sV2y */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8369 /* __nvvm_bf16x2_to_ue8m0x2_rz_satfinite */, 8258 /* sV2y */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8407 /* __nvvm_ceil_d */, 8421 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8424 /* __nvvm_ceil_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8441 /* __nvvm_ceil_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8459 /* __nvvm_compiler_error */, 8481 /* vcC*4 */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8487 /* __nvvm_compiler_warn */, 8481 /* vcC*4 */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8508 /* __nvvm_cos_approx_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8528 /* __nvvm_cos_approx_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8552 /* __nvvm_cp_async_ca_shared_global_4 */, 8587 /* vv*3vC*1. */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8597 /* __nvvm_cp_async_ca_shared_global_8 */, 8587 /* vv*3vC*1. */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8632 /* __nvvm_cp_async_ca_shared_global_16 */, 8587 /* vv*3vC*1. */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8668 /* __nvvm_cp_async_cg_shared_global_16 */, 8587 /* vv*3vC*1. */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8704 /* __nvvm_cp_async_commit_group */, 7886 /* v */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8733 /* __nvvm_cp_async_mbarrier_arrive */, 8765 /* vWi* */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8770 /* __nvvm_cp_async_mbarrier_arrive_noinc */, 8765 /* vWi* */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8808 /* __nvvm_cp_async_mbarrier_arrive_noinc_shared */, 8853 /* vWi*3 */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8859 /* __nvvm_cp_async_mbarrier_arrive_shared */, 8853 /* vWi*3 */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8898 /* __nvvm_cp_async_wait_all */, 7886 /* v */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8923 /* __nvvm_cp_async_wait_group */, 8950 /* vIi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8954 /* __nvvm_d2f_rm */, 8968 /* fd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8971 /* __nvvm_d2f_rm_ftz */, 8968 /* fd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8989 /* __nvvm_d2f_rn */, 8968 /* fd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9003 /* __nvvm_d2f_rn_ftz */, 8968 /* fd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9021 /* __nvvm_d2f_rp */, 8968 /* fd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9035 /* __nvvm_d2f_rp_ftz */, 8968 /* fd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9053 /* __nvvm_d2f_rz */, 8968 /* fd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9067 /* __nvvm_d2f_rz_ftz */, 8968 /* fd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9085 /* __nvvm_d2i_hi */, 9099 /* id */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9102 /* __nvvm_d2i_lo */, 9099 /* id */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9116 /* __nvvm_d2i_rm */, 9099 /* id */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9130 /* __nvvm_d2i_rn */, 9099 /* id */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9144 /* __nvvm_d2i_rp */, 9099 /* id */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9158 /* __nvvm_d2i_rz */, 9099 /* id */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9172 /* __nvvm_d2ll_rm */, 9187 /* LLid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9192 /* __nvvm_d2ll_rn */, 9187 /* LLid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9207 /* __nvvm_d2ll_rp */, 9187 /* LLid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9222 /* __nvvm_d2ll_rz */, 9187 /* LLid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9237 /* __nvvm_d2ui_rm */, 9252 /* Uid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9256 /* __nvvm_d2ui_rn */, 9252 /* Uid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9271 /* __nvvm_d2ui_rp */, 9252 /* Uid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9286 /* __nvvm_d2ui_rz */, 9252 /* Uid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9301 /* __nvvm_d2ull_rm */, 9317 /* ULLid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9323 /* __nvvm_d2ull_rn */, 9317 /* ULLid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9339 /* __nvvm_d2ull_rp */, 9317 /* ULLid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9355 /* __nvvm_d2ull_rz */, 9317 /* ULLid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9371 /* __nvvm_div_approx_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9391 /* __nvvm_div_approx_ftz_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9415 /* __nvvm_div_rm_d */, 4375 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9431 /* __nvvm_div_rm_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9447 /* __nvvm_div_rm_ftz_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9467 /* __nvvm_div_rn_d */, 4375 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9483 /* __nvvm_div_rn_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9499 /* __nvvm_div_rn_ftz_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9519 /* __nvvm_div_rp_d */, 4375 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9535 /* __nvvm_div_rp_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9551 /* __nvvm_div_rp_ftz_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9571 /* __nvvm_div_rz_d */, 4375 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9587 /* __nvvm_div_rz_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9603 /* __nvvm_div_rz_ftz_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9623 /* __nvvm_e2m1x2_to_f16x2_rn */, 9649 /* V2hs */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9654 /* __nvvm_e2m1x2_to_f16x2_rn_relu */, 9649 /* V2hs */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9685 /* __nvvm_e2m3x2_to_f16x2_rn */, 9649 /* V2hs */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9711 /* __nvvm_e2m3x2_to_f16x2_rn_relu */, 9649 /* V2hs */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9742 /* __nvvm_e3m2x2_to_f16x2_rn */, 9649 /* V2hs */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9768 /* __nvvm_e3m2x2_to_f16x2_rn_relu */, 9649 /* V2hs */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9799 /* __nvvm_e4m3x2_to_f16x2_rn */, 9649 /* V2hs */, 0 /*  */, 9825 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9935 /* __nvvm_e4m3x2_to_f16x2_rn_relu */, 9649 /* V2hs */, 0 /*  */, 9825 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9966 /* __nvvm_e5m2x2_to_f16x2_rn */, 9649 /* V2hs */, 0 /*  */, 9825 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9992 /* __nvvm_e5m2x2_to_f16x2_rn_relu */, 9649 /* V2hs */, 0 /*  */, 9825 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10023 /* __nvvm_ex2_approx_d */, 8421 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10043 /* __nvvm_ex2_approx_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10063 /* __nvvm_ex2_approx_f16 */, 10085 /* hh */, 0 /*  */, 10088 /* (sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10282 /* __nvvm_ex2_approx_f16x2 */, 10306 /* V2hV2h */, 0 /*  */, 10088 /* (sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10313 /* __nvvm_ex2_approx_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10337 /* __nvvm_exit */, 7886 /* v */, 10349 /* r */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10351 /* __nvvm_f2bf16_rn */, 10368 /* yf */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10371 /* __nvvm_f2bf16_rn_relu */, 10368 /* yf */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10393 /* __nvvm_f2bf16_rz */, 10368 /* yf */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10410 /* __nvvm_f2bf16_rz_relu */, 10368 /* yf */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10432 /* __nvvm_f2h_rn */, 10446 /* Usf */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10450 /* __nvvm_f2h_rn_ftz */, 10446 /* Usf */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10468 /* __nvvm_f2i_rm */, 10482 /* if */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10485 /* __nvvm_f2i_rm_ftz */, 10482 /* if */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10503 /* __nvvm_f2i_rn */, 10482 /* if */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10517 /* __nvvm_f2i_rn_ftz */, 10482 /* if */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10535 /* __nvvm_f2i_rp */, 10482 /* if */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10549 /* __nvvm_f2i_rp_ftz */, 10482 /* if */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10567 /* __nvvm_f2i_rz */, 10482 /* if */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10581 /* __nvvm_f2i_rz_ftz */, 10482 /* if */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10599 /* __nvvm_f2ll_rm */, 10614 /* LLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10619 /* __nvvm_f2ll_rm_ftz */, 10614 /* LLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10638 /* __nvvm_f2ll_rn */, 10614 /* LLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10653 /* __nvvm_f2ll_rn_ftz */, 10614 /* LLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10672 /* __nvvm_f2ll_rp */, 10614 /* LLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10687 /* __nvvm_f2ll_rp_ftz */, 10614 /* LLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10706 /* __nvvm_f2ll_rz */, 10614 /* LLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10721 /* __nvvm_f2ll_rz_ftz */, 10614 /* LLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10740 /* __nvvm_f2tf32_rn */, 10757 /* Zif */, 0 /*  */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10761 /* __nvvm_f2tf32_rn_relu */, 10757 /* Zif */, 0 /*  */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10783 /* __nvvm_f2tf32_rn_relu_satfinite */, 10757 /* Zif */, 0 /*  */, 10815 /* (sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10876 /* __nvvm_f2tf32_rn_satfinite */, 10757 /* Zif */, 0 /*  */, 10815 /* (sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10903 /* __nvvm_f2tf32_rna */, 10757 /* Zif */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10921 /* __nvvm_f2tf32_rna_satfinite */, 10757 /* Zif */, 0 /*  */, 9825 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10949 /* __nvvm_f2tf32_rz */, 10757 /* Zif */, 0 /*  */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10966 /* __nvvm_f2tf32_rz_relu */, 10757 /* Zif */, 0 /*  */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10988 /* __nvvm_f2tf32_rz_relu_satfinite */, 10757 /* Zif */, 0 /*  */, 10815 /* (sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11020 /* __nvvm_f2tf32_rz_satfinite */, 10757 /* Zif */, 0 /*  */, 10815 /* (sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11047 /* __nvvm_f2ui_rm */, 11062 /* Uif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11066 /* __nvvm_f2ui_rm_ftz */, 11062 /* Uif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11085 /* __nvvm_f2ui_rn */, 11062 /* Uif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11100 /* __nvvm_f2ui_rn_ftz */, 11062 /* Uif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11119 /* __nvvm_f2ui_rp */, 11062 /* Uif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11134 /* __nvvm_f2ui_rp_ftz */, 11062 /* Uif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11153 /* __nvvm_f2ui_rz */, 11062 /* Uif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11168 /* __nvvm_f2ui_rz_ftz */, 11062 /* Uif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11187 /* __nvvm_f2ull_rm */, 11203 /* ULLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11209 /* __nvvm_f2ull_rm_ftz */, 11203 /* ULLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11229 /* __nvvm_f2ull_rn */, 11203 /* ULLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11245 /* __nvvm_f2ull_rn_ftz */, 11203 /* ULLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11265 /* __nvvm_f2ull_rp */, 11203 /* ULLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11281 /* __nvvm_f2ull_rp_ftz */, 11203 /* ULLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11301 /* __nvvm_f2ull_rz */, 11203 /* ULLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11317 /* __nvvm_f2ull_rz_ftz */, 11203 /* ULLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11337 /* __nvvm_f16x2_to_e4m3x2_rn */, 11363 /* sV2h */, 0 /*  */, 9825 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11368 /* __nvvm_f16x2_to_e4m3x2_rn_relu */, 11363 /* sV2h */, 0 /*  */, 9825 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11399 /* __nvvm_f16x2_to_e5m2x2_rn */, 11363 /* sV2h */, 0 /*  */, 9825 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11425 /* __nvvm_f16x2_to_e5m2x2_rn_relu */, 11363 /* sV2h */, 0 /*  */, 9825 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11456 /* __nvvm_fabs_d */, 8421 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11470 /* __nvvm_fabs_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11484 /* __nvvm_fabs_f16 */, 10085 /* hh */, 0 /*  */, 11500 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11736 /* __nvvm_fabs_f16x2 */, 10306 /* V2hV2h */, 0 /*  */, 11500 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11754 /* __nvvm_fabs_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11772 /* __nvvm_fabs_ftz_f16 */, 10085 /* hh */, 0 /*  */, 11500 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11792 /* __nvvm_fabs_ftz_f16x2 */, 10306 /* V2hV2h */, 0 /*  */, 11500 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11814 /* __nvvm_fence_sc_cluster */, 7886 /* v */, 4231 /* n */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11838 /* __nvvm_ff2bf16x2_rn */, 11858 /* V2yff */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11864 /* __nvvm_ff2bf16x2_rn_relu */, 11858 /* V2yff */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11889 /* __nvvm_ff2bf16x2_rz */, 11858 /* V2yff */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11909 /* __nvvm_ff2bf16x2_rz_relu */, 11858 /* V2yff */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11934 /* __nvvm_ff2f16x2_rn */, 11953 /* V2hff */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11959 /* __nvvm_ff2f16x2_rn_relu */, 11953 /* V2hff */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11983 /* __nvvm_ff2f16x2_rz */, 11953 /* V2hff */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12002 /* __nvvm_ff2f16x2_rz_relu */, 11953 /* V2hff */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12026 /* __nvvm_ff_to_e2m1x2_rn_relu_satfinite */, 12064 /* sff */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12068 /* __nvvm_ff_to_e2m1x2_rn_satfinite */, 12064 /* sff */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12101 /* __nvvm_ff_to_e2m3x2_rn_relu_satfinite */, 12064 /* sff */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12139 /* __nvvm_ff_to_e2m3x2_rn_satfinite */, 12064 /* sff */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12172 /* __nvvm_ff_to_e3m2x2_rn_relu_satfinite */, 12064 /* sff */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12210 /* __nvvm_ff_to_e3m2x2_rn_satfinite */, 12064 /* sff */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12243 /* __nvvm_ff_to_e4m3x2_rn */, 12064 /* sff */, 0 /*  */, 9825 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12266 /* __nvvm_ff_to_e4m3x2_rn_relu */, 12064 /* sff */, 0 /*  */, 9825 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12294 /* __nvvm_ff_to_e5m2x2_rn */, 12064 /* sff */, 0 /*  */, 9825 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12317 /* __nvvm_ff_to_e5m2x2_rn_relu */, 12064 /* sff */, 0 /*  */, 9825 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12345 /* __nvvm_ff_to_ue8m0x2_rp */, 12064 /* sff */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12369 /* __nvvm_ff_to_ue8m0x2_rp_satfinite */, 12064 /* sff */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12403 /* __nvvm_ff_to_ue8m0x2_rz */, 12064 /* sff */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12427 /* __nvvm_ff_to_ue8m0x2_rz_satfinite */, 12064 /* sff */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12461 /* __nvvm_floor_d */, 8421 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12476 /* __nvvm_floor_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12491 /* __nvvm_floor_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12510 /* __nvvm_fma_rm_d */, 12526 /* dddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12531 /* __nvvm_fma_rm_f */, 12547 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12552 /* __nvvm_fma_rm_ftz_f */, 12547 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12572 /* __nvvm_fma_rn_bf16 */, 12591 /* yyyy */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12596 /* __nvvm_fma_rn_bf16x2 */, 12617 /* V2yV2yV2yV2y */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12630 /* __nvvm_fma_rn_d */, 12526 /* dddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12646 /* __nvvm_fma_rn_f */, 12547 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12662 /* __nvvm_fma_rn_f16 */, 12680 /* hhhh */, 0 /*  */, 12685 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx42|ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12957 /* __nvvm_fma_rn_f16x2 */, 12977 /* V2hV2hV2hV2h */, 0 /*  */, 12685 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx42|ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12990 /* __nvvm_fma_rn_ftz_f */, 12547 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13010 /* __nvvm_fma_rn_ftz_f16 */, 12680 /* hhhh */, 0 /*  */, 12685 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx42|ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13032 /* __nvvm_fma_rn_ftz_f16x2 */, 12977 /* V2hV2hV2hV2h */, 0 /*  */, 12685 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx42|ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13056 /* __nvvm_fma_rn_ftz_relu_f16 */, 12680 /* hhhh */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13083 /* __nvvm_fma_rn_ftz_relu_f16x2 */, 12977 /* V2hV2hV2hV2h */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13112 /* __nvvm_fma_rn_ftz_sat_f16 */, 12680 /* hhhh */, 0 /*  */, 12685 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx42|ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13138 /* __nvvm_fma_rn_ftz_sat_f16x2 */, 12977 /* V2hV2hV2hV2h */, 0 /*  */, 12685 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx42|ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13166 /* __nvvm_fma_rn_relu_bf16 */, 12591 /* yyyy */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13190 /* __nvvm_fma_rn_relu_bf16x2 */, 12617 /* V2yV2yV2yV2y */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13216 /* __nvvm_fma_rn_relu_f16 */, 12680 /* hhhh */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13239 /* __nvvm_fma_rn_relu_f16x2 */, 12977 /* V2hV2hV2hV2h */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13264 /* __nvvm_fma_rn_sat_f16 */, 12680 /* hhhh */, 0 /*  */, 12685 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx42|ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13286 /* __nvvm_fma_rn_sat_f16x2 */, 12977 /* V2hV2hV2hV2h */, 0 /*  */, 12685 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx42|ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13310 /* __nvvm_fma_rp_d */, 12526 /* dddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13326 /* __nvvm_fma_rp_f */, 12547 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13342 /* __nvvm_fma_rp_ftz_f */, 12547 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13362 /* __nvvm_fma_rz_d */, 12526 /* dddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13378 /* __nvvm_fma_rz_f */, 12547 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13394 /* __nvvm_fma_rz_ftz_f */, 12547 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13414 /* __nvvm_fmax_bf16 */, 13431 /* yyy */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13435 /* __nvvm_fmax_bf16x2 */, 13454 /* V2yV2yV2y */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13464 /* __nvvm_fmax_d */, 4375 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13478 /* __nvvm_fmax_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13492 /* __nvvm_fmax_f16 */, 13508 /* hhh */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13512 /* __nvvm_fmax_f16x2 */, 13530 /* V2hV2hV2h */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13540 /* __nvvm_fmax_ftz_bf16 */, 13431 /* yyy */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13561 /* __nvvm_fmax_ftz_bf16x2 */, 13454 /* V2yV2yV2y */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13584 /* __nvvm_fmax_ftz_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13602 /* __nvvm_fmax_ftz_f16 */, 13508 /* hhh */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13622 /* __nvvm_fmax_ftz_f16x2 */, 13530 /* V2hV2hV2h */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13644 /* __nvvm_fmax_ftz_nan_bf16 */, 13431 /* yyy */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13669 /* __nvvm_fmax_ftz_nan_bf16x2 */, 13454 /* V2yV2yV2y */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13696 /* __nvvm_fmax_ftz_nan_f */, 4395 /* fff */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13718 /* __nvvm_fmax_ftz_nan_f16 */, 13508 /* hhh */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13742 /* __nvvm_fmax_ftz_nan_f16x2 */, 13530 /* V2hV2hV2h */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13768 /* __nvvm_fmax_ftz_nan_xorsign_abs_f */, 4395 /* fff */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13972 /* __nvvm_fmax_ftz_nan_xorsign_abs_f16 */, 13508 /* hhh */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14008 /* __nvvm_fmax_ftz_nan_xorsign_abs_f16x2 */, 13530 /* V2hV2hV2h */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14046 /* __nvvm_fmax_ftz_xorsign_abs_f */, 4395 /* fff */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14076 /* __nvvm_fmax_ftz_xorsign_abs_f16 */, 13508 /* hhh */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14108 /* __nvvm_fmax_ftz_xorsign_abs_f16x2 */, 13530 /* V2hV2hV2h */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14142 /* __nvvm_fmax_nan_bf16 */, 13431 /* yyy */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14163 /* __nvvm_fmax_nan_bf16x2 */, 13454 /* V2yV2yV2y */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14186 /* __nvvm_fmax_nan_f */, 4395 /* fff */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14204 /* __nvvm_fmax_nan_f16 */, 13508 /* hhh */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14224 /* __nvvm_fmax_nan_f16x2 */, 13530 /* V2hV2hV2h */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14246 /* __nvvm_fmax_nan_xorsign_abs_bf16 */, 13431 /* yyy */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14279 /* __nvvm_fmax_nan_xorsign_abs_bf16x2 */, 13454 /* V2yV2yV2y */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14314 /* __nvvm_fmax_nan_xorsign_abs_f */, 4395 /* fff */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14344 /* __nvvm_fmax_nan_xorsign_abs_f16 */, 13508 /* hhh */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14376 /* __nvvm_fmax_nan_xorsign_abs_f16x2 */, 13530 /* V2hV2hV2h */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14410 /* __nvvm_fmax_xorsign_abs_bf16 */, 13431 /* yyy */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14439 /* __nvvm_fmax_xorsign_abs_bf16x2 */, 13454 /* V2yV2yV2y */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14470 /* __nvvm_fmax_xorsign_abs_f */, 4395 /* fff */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14496 /* __nvvm_fmax_xorsign_abs_f16 */, 13508 /* hhh */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14524 /* __nvvm_fmax_xorsign_abs_f16x2 */, 13530 /* V2hV2hV2h */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14554 /* __nvvm_fmin_bf16 */, 13431 /* yyy */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14571 /* __nvvm_fmin_bf16x2 */, 13454 /* V2yV2yV2y */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14590 /* __nvvm_fmin_d */, 4375 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14604 /* __nvvm_fmin_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14618 /* __nvvm_fmin_f16 */, 13508 /* hhh */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14634 /* __nvvm_fmin_f16x2 */, 13530 /* V2hV2hV2h */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14652 /* __nvvm_fmin_ftz_bf16 */, 13431 /* yyy */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14673 /* __nvvm_fmin_ftz_bf16x2 */, 13454 /* V2yV2yV2y */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14696 /* __nvvm_fmin_ftz_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14714 /* __nvvm_fmin_ftz_f16 */, 13508 /* hhh */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14734 /* __nvvm_fmin_ftz_f16x2 */, 13530 /* V2hV2hV2h */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14756 /* __nvvm_fmin_ftz_nan_bf16 */, 13431 /* yyy */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14781 /* __nvvm_fmin_ftz_nan_bf16x2 */, 13454 /* V2yV2yV2y */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14808 /* __nvvm_fmin_ftz_nan_f */, 4395 /* fff */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14830 /* __nvvm_fmin_ftz_nan_f16 */, 13508 /* hhh */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14854 /* __nvvm_fmin_ftz_nan_f16x2 */, 13530 /* V2hV2hV2h */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14880 /* __nvvm_fmin_ftz_nan_xorsign_abs_f */, 4395 /* fff */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14914 /* __nvvm_fmin_ftz_nan_xorsign_abs_f16 */, 13508 /* hhh */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14950 /* __nvvm_fmin_ftz_nan_xorsign_abs_f16x2 */, 13530 /* V2hV2hV2h */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14988 /* __nvvm_fmin_ftz_xorsign_abs_f */, 4395 /* fff */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15018 /* __nvvm_fmin_ftz_xorsign_abs_f16 */, 13508 /* hhh */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15050 /* __nvvm_fmin_ftz_xorsign_abs_f16x2 */, 13530 /* V2hV2hV2h */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15084 /* __nvvm_fmin_nan_bf16 */, 13431 /* yyy */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15105 /* __nvvm_fmin_nan_bf16x2 */, 13454 /* V2yV2yV2y */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15128 /* __nvvm_fmin_nan_f */, 4395 /* fff */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15146 /* __nvvm_fmin_nan_f16 */, 13508 /* hhh */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15166 /* __nvvm_fmin_nan_f16x2 */, 13530 /* V2hV2hV2h */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15188 /* __nvvm_fmin_nan_xorsign_abs_bf16 */, 13431 /* yyy */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15221 /* __nvvm_fmin_nan_xorsign_abs_bf16x2 */, 13454 /* V2yV2yV2y */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15256 /* __nvvm_fmin_nan_xorsign_abs_f */, 4395 /* fff */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15286 /* __nvvm_fmin_nan_xorsign_abs_f16 */, 13508 /* hhh */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15318 /* __nvvm_fmin_nan_xorsign_abs_f16x2 */, 13530 /* V2hV2hV2h */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15352 /* __nvvm_fmin_xorsign_abs_bf16 */, 13431 /* yyy */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15381 /* __nvvm_fmin_xorsign_abs_bf16x2 */, 13454 /* V2yV2yV2y */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15412 /* __nvvm_fmin_xorsign_abs_f */, 4395 /* fff */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15438 /* __nvvm_fmin_xorsign_abs_f16 */, 13508 /* hhh */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15466 /* __nvvm_fmin_xorsign_abs_f16x2 */, 13530 /* V2hV2hV2h */, 0 /*  */, 13802 /* (sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15496 /* __nvvm_fns */, 15507 /* UiUiUii */, 4231 /* n */, 7718 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15515 /* __nvvm_getctarank */, 15533 /* iv* */, 0 /*  */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15537 /* __nvvm_getctarank_shared_cluster */, 15570 /* iv*3 */, 0 /*  */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15575 /* __nvvm_i2d_rm */, 15589 /* di */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15592 /* __nvvm_i2d_rn */, 15589 /* di */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15606 /* __nvvm_i2d_rp */, 15589 /* di */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15620 /* __nvvm_i2d_rz */, 15589 /* di */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15634 /* __nvvm_i2f_rm */, 15648 /* fi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15651 /* __nvvm_i2f_rn */, 15648 /* fi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15665 /* __nvvm_i2f_rp */, 15648 /* fi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15679 /* __nvvm_i2f_rz */, 15648 /* fi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15693 /* __nvvm_is_explicit_cluster */, 15720 /* b */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15725 /* __nvvm_isspacep_const */, 15747 /* bvC* */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15752 /* __nvvm_isspacep_global */, 15747 /* bvC* */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15775 /* __nvvm_isspacep_local */, 15747 /* bvC* */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15797 /* __nvvm_isspacep_shared */, 15747 /* bvC* */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15820 /* __nvvm_isspacep_shared_cluster */, 15747 /* bvC* */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15851 /* __nvvm_ldg_c */, 15864 /* ccC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15869 /* __nvvm_ldg_c2 */, 15883 /* E2cE2cC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15892 /* __nvvm_ldg_c4 */, 15906 /* E4cE4cC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15915 /* __nvvm_ldg_d */, 15928 /* ddC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15933 /* __nvvm_ldg_d2 */, 15947 /* E2dE2dC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15956 /* __nvvm_ldg_f */, 15969 /* ffC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15974 /* __nvvm_ldg_f2 */, 15988 /* E2fE2fC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15997 /* __nvvm_ldg_f4 */, 16011 /* E4fE4fC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16020 /* __nvvm_ldg_h */, 16033 /* hhC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16038 /* __nvvm_ldg_h2 */, 16052 /* E2hE2hC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16061 /* __nvvm_ldg_i */, 16074 /* iiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16079 /* __nvvm_ldg_i2 */, 16093 /* E2iE2iC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16102 /* __nvvm_ldg_i4 */, 16116 /* E4iE4iC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16125 /* __nvvm_ldg_l */, 16138 /* LiLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16145 /* __nvvm_ldg_l2 */, 16159 /* E2LiE2LiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16170 /* __nvvm_ldg_ll */, 16184 /* LLiLLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16193 /* __nvvm_ldg_ll2 */, 16208 /* E2LLiE2LLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16221 /* __nvvm_ldg_s */, 16234 /* ssC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16239 /* __nvvm_ldg_s2 */, 16253 /* E2sE2sC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16262 /* __nvvm_ldg_s4 */, 16276 /* E4sE4sC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16285 /* __nvvm_ldg_sc */, 16299 /* ScScC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16306 /* __nvvm_ldg_sc2 */, 16321 /* E2ScE2ScC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16332 /* __nvvm_ldg_sc4 */, 16347 /* E4ScE4ScC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16358 /* __nvvm_ldg_uc */, 16372 /* UcUcC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16379 /* __nvvm_ldg_uc2 */, 16394 /* E2UcE2UcC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16405 /* __nvvm_ldg_uc4 */, 16420 /* E4UcE4UcC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16431 /* __nvvm_ldg_ui */, 16445 /* UiUiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16452 /* __nvvm_ldg_ui2 */, 16467 /* E2UiE2UiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16478 /* __nvvm_ldg_ui4 */, 16493 /* E4UiE4UiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16504 /* __nvvm_ldg_ul */, 16518 /* ULiULiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16527 /* __nvvm_ldg_ul2 */, 16542 /* E2ULiE2ULiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16555 /* __nvvm_ldg_ull */, 16570 /* ULLiULLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16581 /* __nvvm_ldg_ull2 */, 16597 /* E2ULLiE2ULLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16612 /* __nvvm_ldg_us */, 16626 /* UsUsC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16633 /* __nvvm_ldg_us2 */, 16648 /* E2UsE2UsC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16659 /* __nvvm_ldg_us4 */, 16674 /* E4UsE4UsC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16685 /* __nvvm_ldu_c */, 15864 /* ccC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16698 /* __nvvm_ldu_c2 */, 15883 /* E2cE2cC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16712 /* __nvvm_ldu_c4 */, 15906 /* E4cE4cC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16726 /* __nvvm_ldu_d */, 15928 /* ddC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16739 /* __nvvm_ldu_d2 */, 15947 /* E2dE2dC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16753 /* __nvvm_ldu_f */, 15969 /* ffC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16766 /* __nvvm_ldu_f2 */, 15988 /* E2fE2fC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16780 /* __nvvm_ldu_f4 */, 16011 /* E4fE4fC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16794 /* __nvvm_ldu_h */, 16033 /* hhC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16807 /* __nvvm_ldu_h2 */, 16052 /* E2hE2hC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16821 /* __nvvm_ldu_i */, 16074 /* iiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16834 /* __nvvm_ldu_i2 */, 16093 /* E2iE2iC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16848 /* __nvvm_ldu_i4 */, 16116 /* E4iE4iC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16862 /* __nvvm_ldu_l */, 16138 /* LiLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16875 /* __nvvm_ldu_l2 */, 16159 /* E2LiE2LiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16889 /* __nvvm_ldu_ll */, 16184 /* LLiLLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16903 /* __nvvm_ldu_ll2 */, 16208 /* E2LLiE2LLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16918 /* __nvvm_ldu_s */, 16234 /* ssC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16931 /* __nvvm_ldu_s2 */, 16253 /* E2sE2sC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16945 /* __nvvm_ldu_s4 */, 16276 /* E4sE4sC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16959 /* __nvvm_ldu_sc */, 16299 /* ScScC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16973 /* __nvvm_ldu_sc2 */, 16321 /* E2ScE2ScC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16988 /* __nvvm_ldu_sc4 */, 16347 /* E4ScE4ScC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17003 /* __nvvm_ldu_uc */, 16372 /* UcUcC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17017 /* __nvvm_ldu_uc2 */, 16394 /* E2UcE2UcC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17032 /* __nvvm_ldu_uc4 */, 16420 /* E4UcE4UcC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17047 /* __nvvm_ldu_ui */, 16445 /* UiUiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17061 /* __nvvm_ldu_ui2 */, 16467 /* E2UiE2UiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17076 /* __nvvm_ldu_ui4 */, 16493 /* E4UiE4UiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17091 /* __nvvm_ldu_ul */, 16518 /* ULiULiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17105 /* __nvvm_ldu_ul2 */, 16542 /* E2ULiE2ULiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17120 /* __nvvm_ldu_ull */, 16570 /* ULLiULLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17135 /* __nvvm_ldu_ull2 */, 16597 /* E2ULLiE2ULLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17151 /* __nvvm_ldu_us */, 16626 /* UsUsC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17165 /* __nvvm_ldu_us2 */, 16648 /* E2UsE2UsC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17180 /* __nvvm_ldu_us4 */, 16674 /* E4UsE4UsC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17195 /* __nvvm_lg2_approx_d */, 8421 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17215 /* __nvvm_lg2_approx_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17235 /* __nvvm_lg2_approx_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17259 /* __nvvm_ll2d_rm */, 17274 /* dLLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17279 /* __nvvm_ll2d_rn */, 17274 /* dLLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17294 /* __nvvm_ll2d_rp */, 17274 /* dLLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17309 /* __nvvm_ll2d_rz */, 17274 /* dLLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17324 /* __nvvm_ll2f_rm */, 17339 /* fLLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17344 /* __nvvm_ll2f_rn */, 17339 /* fLLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17359 /* __nvvm_ll2f_rp */, 17339 /* fLLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17374 /* __nvvm_ll2f_rz */, 17339 /* fLLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17389 /* __nvvm_lohi_i2d */, 17405 /* dii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17409 /* __nvvm_mapa */, 17421 /* v*v*i */, 0 /*  */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17427 /* __nvvm_mapa_shared_cluster */, 17454 /* v*3v*3i */, 0 /*  */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17462 /* __nvvm_match_all_sync_i32p */, 17489 /* UiUiUii* */, 0 /*  */, 2102 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17498 /* __nvvm_match_all_sync_i64p */, 17525 /* UiUiWii* */, 0 /*  */, 2102 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17534 /* __nvvm_match_any_sync_i32 */, 17560 /* UiUiUi */, 0 /*  */, 2102 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17567 /* __nvvm_match_any_sync_i64 */, 17593 /* UiUiWi */, 0 /*  */, 2102 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17600 /* __nvvm_mbarrier_arrive */, 17623 /* WiWi* */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17629 /* __nvvm_mbarrier_arrive_drop */, 17623 /* WiWi* */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17657 /* __nvvm_mbarrier_arrive_drop_noComplete */, 17696 /* WiWi*i */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17703 /* __nvvm_mbarrier_arrive_drop_noComplete_shared */, 17749 /* WiWi*3i */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17757 /* __nvvm_mbarrier_arrive_drop_shared */, 17792 /* WiWi*3 */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17799 /* __nvvm_mbarrier_arrive_noComplete */, 17696 /* WiWi*i */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17833 /* __nvvm_mbarrier_arrive_noComplete_shared */, 17749 /* WiWi*3i */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17874 /* __nvvm_mbarrier_arrive_shared */, 17792 /* WiWi*3 */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17904 /* __nvvm_mbarrier_init */, 17925 /* vWi*i */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17931 /* __nvvm_mbarrier_init_shared */, 17959 /* vWi*3i */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17966 /* __nvvm_mbarrier_inval */, 8765 /* vWi* */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17988 /* __nvvm_mbarrier_inval_shared */, 8853 /* vWi*3 */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18017 /* __nvvm_mbarrier_pending_count */, 18047 /* iWi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18051 /* __nvvm_mbarrier_test_wait */, 18077 /* bWi*Wi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18084 /* __nvvm_mbarrier_test_wait_shared */, 18117 /* bWi*3Wi */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18125 /* __nvvm_membar_cta */, 7886 /* v */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18143 /* __nvvm_membar_gl */, 7886 /* v */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18160 /* __nvvm_membar_sys */, 7886 /* v */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18178 /* __nvvm_memcpy */, 18192 /* vUc*Uc*zi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18202 /* __nvvm_memset */, 18216 /* vUc*Uczi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18225 /* __nvvm_mul24_i */, 18240 /* iii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18244 /* __nvvm_mul24_ui */, 17560 /* UiUiUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18260 /* __nvvm_mul_rm_d */, 4375 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18276 /* __nvvm_mul_rm_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18292 /* __nvvm_mul_rm_ftz_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18312 /* __nvvm_mul_rn_d */, 4375 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18328 /* __nvvm_mul_rn_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18344 /* __nvvm_mul_rn_ftz_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18364 /* __nvvm_mul_rp_d */, 4375 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18380 /* __nvvm_mul_rp_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18396 /* __nvvm_mul_rp_ftz_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18416 /* __nvvm_mul_rz_d */, 4375 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18432 /* __nvvm_mul_rz_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18448 /* __nvvm_mul_rz_ftz_f */, 4395 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18468 /* __nvvm_mulhi_i */, 18240 /* iii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18483 /* __nvvm_mulhi_ll */, 18499 /* LLiLLiLLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18509 /* __nvvm_mulhi_ui */, 17560 /* UiUiUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18525 /* __nvvm_mulhi_ull */, 18542 /* ULLiULLiULLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18555 /* __nvvm_nanosleep */, 7714 /* vUi */, 4231 /* n */, 18572 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18796 /* __nvvm_neg_bf16 */, 4182 /* yy */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18812 /* __nvvm_neg_bf16x2 */, 4203 /* V2yV2y */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18830 /* __nvvm_pm_event_mask */, 18851 /* vIUs */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18856 /* __nvvm_prmt */, 18868 /* UiUiUiUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18877 /* __nvvm_rcp_approx_ftz_d */, 8421 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18901 /* __nvvm_rcp_approx_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18925 /* __nvvm_rcp_rm_d */, 8421 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18941 /* __nvvm_rcp_rm_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18957 /* __nvvm_rcp_rm_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18977 /* __nvvm_rcp_rn_d */, 8421 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18993 /* __nvvm_rcp_rn_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19009 /* __nvvm_rcp_rn_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19029 /* __nvvm_rcp_rp_d */, 8421 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19045 /* __nvvm_rcp_rp_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19061 /* __nvvm_rcp_rp_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19081 /* __nvvm_rcp_rz_d */, 8421 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19097 /* __nvvm_rcp_rz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19113 /* __nvvm_rcp_rz_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19133 /* __nvvm_read_ptx_sreg_clock */, 19160 /* i */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19162 /* __nvvm_read_ptx_sreg_clock64 */, 19191 /* LLi */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19195 /* __nvvm_read_ptx_sreg_cluster_ctaid_w */, 19160 /* i */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19232 /* __nvvm_read_ptx_sreg_cluster_ctaid_x */, 19160 /* i */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19269 /* __nvvm_read_ptx_sreg_cluster_ctaid_y */, 19160 /* i */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19306 /* __nvvm_read_ptx_sreg_cluster_ctaid_z */, 19160 /* i */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19343 /* __nvvm_read_ptx_sreg_cluster_ctarank */, 19160 /* i */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19380 /* __nvvm_read_ptx_sreg_cluster_nctaid_w */, 19160 /* i */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19418 /* __nvvm_read_ptx_sreg_cluster_nctaid_x */, 19160 /* i */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19456 /* __nvvm_read_ptx_sreg_cluster_nctaid_y */, 19160 /* i */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19494 /* __nvvm_read_ptx_sreg_cluster_nctaid_z */, 19160 /* i */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19532 /* __nvvm_read_ptx_sreg_cluster_nctarank */, 19160 /* i */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19570 /* __nvvm_read_ptx_sreg_clusterid_w */, 19160 /* i */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19603 /* __nvvm_read_ptx_sreg_clusterid_x */, 19160 /* i */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19636 /* __nvvm_read_ptx_sreg_clusterid_y */, 19160 /* i */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19669 /* __nvvm_read_ptx_sreg_clusterid_z */, 19160 /* i */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19702 /* __nvvm_read_ptx_sreg_ctaid_w */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19731 /* __nvvm_read_ptx_sreg_ctaid_x */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19760 /* __nvvm_read_ptx_sreg_ctaid_y */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19789 /* __nvvm_read_ptx_sreg_ctaid_z */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19818 /* __nvvm_read_ptx_sreg_globaltimer */, 19191 /* LLi */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19851 /* __nvvm_read_ptx_sreg_gridid */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19879 /* __nvvm_read_ptx_sreg_laneid */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19907 /* __nvvm_read_ptx_sreg_lanemask_eq */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19940 /* __nvvm_read_ptx_sreg_lanemask_ge */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19973 /* __nvvm_read_ptx_sreg_lanemask_gt */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20006 /* __nvvm_read_ptx_sreg_lanemask_le */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20039 /* __nvvm_read_ptx_sreg_lanemask_lt */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20072 /* __nvvm_read_ptx_sreg_nclusterid_w */, 19160 /* i */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20106 /* __nvvm_read_ptx_sreg_nclusterid_x */, 19160 /* i */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20140 /* __nvvm_read_ptx_sreg_nclusterid_y */, 19160 /* i */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20174 /* __nvvm_read_ptx_sreg_nclusterid_z */, 19160 /* i */, 15722 /* nc */, 7888 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20208 /* __nvvm_read_ptx_sreg_nctaid_w */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20238 /* __nvvm_read_ptx_sreg_nctaid_x */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20268 /* __nvvm_read_ptx_sreg_nctaid_y */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20298 /* __nvvm_read_ptx_sreg_nctaid_z */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20328 /* __nvvm_read_ptx_sreg_nsmid */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20355 /* __nvvm_read_ptx_sreg_ntid_w */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20383 /* __nvvm_read_ptx_sreg_ntid_x */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20411 /* __nvvm_read_ptx_sreg_ntid_y */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20439 /* __nvvm_read_ptx_sreg_ntid_z */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20467 /* __nvvm_read_ptx_sreg_nwarpid */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20496 /* __nvvm_read_ptx_sreg_pm0 */, 19160 /* i */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20521 /* __nvvm_read_ptx_sreg_pm1 */, 19160 /* i */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20546 /* __nvvm_read_ptx_sreg_pm2 */, 19160 /* i */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20571 /* __nvvm_read_ptx_sreg_pm3 */, 19160 /* i */, 4231 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20596 /* __nvvm_read_ptx_sreg_smid */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20622 /* __nvvm_read_ptx_sreg_tid_w */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20649 /* __nvvm_read_ptx_sreg_tid_x */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20676 /* __nvvm_read_ptx_sreg_tid_y */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20703 /* __nvvm_read_ptx_sreg_tid_z */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20730 /* __nvvm_read_ptx_sreg_warpid */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20758 /* __nvvm_read_ptx_sreg_warpsize */, 19160 /* i */, 15722 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20788 /* __nvvm_redux_sync_add */, 18240 /* iii */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20810 /* __nvvm_redux_sync_and */, 18240 /* iii */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20832 /* __nvvm_redux_sync_fmax */, 20855 /* ffi */, 0 /*  */, 20859 /* (sm_100a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20883 /* __nvvm_redux_sync_fmax_NaN */, 20855 /* ffi */, 0 /*  */, 20859 /* (sm_100a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20910 /* __nvvm_redux_sync_fmax_abs */, 20855 /* ffi */, 0 /*  */, 20859 /* (sm_100a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20937 /* __nvvm_redux_sync_fmax_abs_NaN */, 20855 /* ffi */, 0 /*  */, 20859 /* (sm_100a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20968 /* __nvvm_redux_sync_fmin */, 20855 /* ffi */, 0 /*  */, 20859 /* (sm_100a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20991 /* __nvvm_redux_sync_fmin_NaN */, 20855 /* ffi */, 0 /*  */, 20859 /* (sm_100a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21018 /* __nvvm_redux_sync_fmin_abs */, 20855 /* ffi */, 0 /*  */, 20859 /* (sm_100a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21045 /* __nvvm_redux_sync_fmin_abs_NaN */, 20855 /* ffi */, 0 /*  */, 20859 /* (sm_100a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21076 /* __nvvm_redux_sync_max */, 18240 /* iii */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21098 /* __nvvm_redux_sync_min */, 18240 /* iii */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21120 /* __nvvm_redux_sync_or */, 18240 /* iii */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21141 /* __nvvm_redux_sync_umax */, 21164 /* UiUii */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21170 /* __nvvm_redux_sync_umin */, 21164 /* UiUii */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21193 /* __nvvm_redux_sync_xor */, 18240 /* iii */, 0 /*  */, 1227 /* (sm_80|sm_86|sm_87|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_120|sm_120a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21215 /* __nvvm_reflect */, 21230 /* UicC* */, 10349 /* r */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21236 /* __nvvm_round_d */, 8421 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21251 /* __nvvm_round_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21266 /* __nvvm_round_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21285 /* __nvvm_rsqrt_approx_d */, 8421 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21307 /* __nvvm_rsqrt_approx_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21329 /* __nvvm_rsqrt_approx_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21355 /* __nvvm_sad_i */, 21368 /* iiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21373 /* __nvvm_sad_ui */, 18868 /* UiUiUiUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21387 /* __nvvm_saturate_d */, 8421 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21405 /* __nvvm_saturate_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21423 /* __nvvm_saturate_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21445 /* __nvvm_shfl_bfly_f32 */, 21466 /* ffii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21471 /* __nvvm_shfl_bfly_i32 */, 21368 /* iiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21492 /* __nvvm_shfl_down_f32 */, 21466 /* ffii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21513 /* __nvvm_shfl_down_i32 */, 21368 /* iiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21534 /* __nvvm_shfl_idx_f32 */, 21466 /* ffii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21554 /* __nvvm_shfl_idx_i32 */, 21368 /* iiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21574 /* __nvvm_shfl_sync_bfly_f32 */, 21600 /* fUifii */, 0 /*  */, 7718 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21607 /* __nvvm_shfl_sync_bfly_i32 */, 21633 /* iUiiii */, 0 /*  */, 7718 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21640 /* __nvvm_shfl_sync_down_f32 */, 21600 /* fUifii */, 0 /*  */, 7718 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21666 /* __nvvm_shfl_sync_down_i32 */, 21633 /* iUiiii */, 0 /*  */, 7718 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21692 /* __nvvm_shfl_sync_idx_f32 */, 21600 /* fUifii */, 0 /*  */, 7718 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21717 /* __nvvm_shfl_sync_idx_i32 */, 21633 /* iUiiii */, 0 /*  */, 7718 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21742 /* __nvvm_shfl_sync_up_f32 */, 21600 /* fUifii */, 0 /*  */, 7718 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21766 /* __nvvm_shfl_sync_up_i32 */, 21633 /* iUiiii */, 0 /*  */, 7718 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21790 /* __nvvm_shfl_up_f32 */, 21466 /* ffii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21809 /* __nvvm_shfl_up_i32 */, 21368 /* iiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21828 /* __nvvm_sin_approx_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21848 /* __nvvm_sin_approx_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21872 /* __nvvm_sqrt_approx_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21893 /* __nvvm_sqrt_approx_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21918 /* __nvvm_sqrt_rm_d */, 8421 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21935 /* __nvvm_sqrt_rm_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21952 /* __nvvm_sqrt_rm_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21973 /* __nvvm_sqrt_rn_d */, 8421 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21990 /* __nvvm_sqrt_rn_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22007 /* __nvvm_sqrt_rn_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22028 /* __nvvm_sqrt_rp_d */, 8421 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22045 /* __nvvm_sqrt_rp_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22062 /* __nvvm_sqrt_rp_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22083 /* __nvvm_sqrt_rz_d */, 8421 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22100 /* __nvvm_sqrt_rz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22117 /* __nvvm_sqrt_rz_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22138 /* __nvvm_trunc_d */, 8421 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22153 /* __nvvm_trunc_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22168 /* __nvvm_trunc_ftz_f */, 8438 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22187 /* __nvvm_ue8m0x2_to_bf16x2 */, 22212 /* V2ys */, 0 /*  */, 8263 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22217 /* __nvvm_ui2d_rm */, 22232 /* dUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22236 /* __nvvm_ui2d_rn */, 22232 /* dUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22251 /* __nvvm_ui2d_rp */, 22232 /* dUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22266 /* __nvvm_ui2d_rz */, 22232 /* dUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22281 /* __nvvm_ui2f_rm */, 22296 /* fUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22300 /* __nvvm_ui2f_rn */, 22296 /* fUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22315 /* __nvvm_ui2f_rp */, 22296 /* fUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22330 /* __nvvm_ui2f_rz */, 22296 /* fUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22345 /* __nvvm_ull2d_rm */, 22361 /* dULLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22367 /* __nvvm_ull2d_rn */, 22361 /* dULLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22383 /* __nvvm_ull2d_rp */, 22361 /* dULLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22399 /* __nvvm_ull2d_rz */, 22361 /* dULLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22415 /* __nvvm_ull2f_rm */, 22431 /* fULLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22437 /* __nvvm_ull2f_rn */, 22431 /* fULLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22453 /* __nvvm_ull2f_rp */, 22431 /* fULLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22469 /* __nvvm_ull2f_rz */, 22431 /* fULLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22485 /* __nvvm_vote_all */, 22501 /* bb */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22504 /* __nvvm_vote_all_sync */, 22525 /* bUib */, 0 /*  */, 7718 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22530 /* __nvvm_vote_any */, 22501 /* bb */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22546 /* __nvvm_vote_any_sync */, 22525 /* bUib */, 0 /*  */, 7718 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22567 /* __nvvm_vote_ballot */, 22586 /* Uib */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22590 /* __nvvm_vote_ballot_sync */, 22614 /* UiUib */, 0 /*  */, 7718 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22620 /* __nvvm_vote_uni */, 22501 /* bb */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22636 /* __nvvm_vote_uni_sync */, 22525 /* bUib */, 0 /*  */, 7718 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22657 /* __syncthreads */, 7886 /* v */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},

#endif // GET_BUILTIN_INFOS

#ifdef GET_BUILTIN_PREFIXED_INFOS

#endif // GET_BUILTIN_PREFIXED_INFOS

#ifdef ATOMIC_BUILTIN

#endif // ATOMIC_BUILTIN
#undef ATOMIC_BUILTIN
