#ifdef GET_MVE_BUILTIN_ENUMERATORS
  BI__builtin_arm_mve_asrl,
  BI__builtin_arm_mve_lsll,
  BI__builtin_arm_mve_sqrshr,
  BI__builtin_arm_mve_sqrshrl,
  BI__builtin_arm_mve_sqrshrl_sat48,
  BI__builtin_arm_mve_sqshl,
  BI__builtin_arm_mve_sqshll,
  BI__builtin_arm_mve_srshr,
  BI__builtin_arm_mve_srshrl,
  BI__builtin_arm_mve_uqrshl,
  BI__builtin_arm_mve_uqrshll,
  BI__builtin_arm_mve_uqrshll_sat48,
  BI__builtin_arm_mve_uqshl,
  BI__builtin_arm_mve_uqshll,
  BI__builtin_arm_mve_urshr,
  BI__builtin_arm_mve_urshrl,
  BI__builtin_arm_mve_vabavq_p_s16,
  BI__builtin_arm_mve_vabavq_p_s32,
  BI__builtin_arm_mve_vabavq_p_s8,
  BI__builtin_arm_mve_vabavq_p_u16,
  BI__builtin_arm_mve_vabavq_p_u32,
  BI__builtin_arm_mve_vabavq_p_u8,
  BI__builtin_arm_mve_vabavq_s16,
  BI__builtin_arm_mve_vabavq_s32,
  BI__builtin_arm_mve_vabavq_s8,
  BI__builtin_arm_mve_vabavq_u16,
  BI__builtin_arm_mve_vabavq_u32,
  BI__builtin_arm_mve_vabavq_u8,
  BI__builtin_arm_mve_vabdq_f16,
  BI__builtin_arm_mve_vabdq_f32,
  BI__builtin_arm_mve_vabdq_m_f16,
  BI__builtin_arm_mve_vabdq_m_f32,
  BI__builtin_arm_mve_vabdq_m_s16,
  BI__builtin_arm_mve_vabdq_m_s32,
  BI__builtin_arm_mve_vabdq_m_s8,
  BI__builtin_arm_mve_vabdq_m_u16,
  BI__builtin_arm_mve_vabdq_m_u32,
  BI__builtin_arm_mve_vabdq_m_u8,
  BI__builtin_arm_mve_vabdq_s16,
  BI__builtin_arm_mve_vabdq_s32,
  BI__builtin_arm_mve_vabdq_s8,
  BI__builtin_arm_mve_vabdq_u16,
  BI__builtin_arm_mve_vabdq_u32,
  BI__builtin_arm_mve_vabdq_u8,
  BI__builtin_arm_mve_vabdq_x_f16,
  BI__builtin_arm_mve_vabdq_x_f32,
  BI__builtin_arm_mve_vabdq_x_s16,
  BI__builtin_arm_mve_vabdq_x_s32,
  BI__builtin_arm_mve_vabdq_x_s8,
  BI__builtin_arm_mve_vabdq_x_u16,
  BI__builtin_arm_mve_vabdq_x_u32,
  BI__builtin_arm_mve_vabdq_x_u8,
  BI__builtin_arm_mve_vabsq_f16,
  BI__builtin_arm_mve_vabsq_f32,
  BI__builtin_arm_mve_vabsq_m_f16,
  BI__builtin_arm_mve_vabsq_m_f32,
  BI__builtin_arm_mve_vabsq_m_s16,
  BI__builtin_arm_mve_vabsq_m_s32,
  BI__builtin_arm_mve_vabsq_m_s8,
  BI__builtin_arm_mve_vabsq_s16,
  BI__builtin_arm_mve_vabsq_s32,
  BI__builtin_arm_mve_vabsq_s8,
  BI__builtin_arm_mve_vabsq_x_f16,
  BI__builtin_arm_mve_vabsq_x_f32,
  BI__builtin_arm_mve_vabsq_x_s16,
  BI__builtin_arm_mve_vabsq_x_s32,
  BI__builtin_arm_mve_vabsq_x_s8,
  BI__builtin_arm_mve_vadciq_m_s32,
  BI__builtin_arm_mve_vadciq_m_u32,
  BI__builtin_arm_mve_vadciq_s32,
  BI__builtin_arm_mve_vadciq_u32,
  BI__builtin_arm_mve_vadcq_m_s32,
  BI__builtin_arm_mve_vadcq_m_u32,
  BI__builtin_arm_mve_vadcq_s32,
  BI__builtin_arm_mve_vadcq_u32,
  BI__builtin_arm_mve_vaddlvaq_p_s32,
  BI__builtin_arm_mve_vaddlvaq_p_u32,
  BI__builtin_arm_mve_vaddlvaq_s32,
  BI__builtin_arm_mve_vaddlvaq_u32,
  BI__builtin_arm_mve_vaddlvq_p_s32,
  BI__builtin_arm_mve_vaddlvq_p_u32,
  BI__builtin_arm_mve_vaddlvq_s32,
  BI__builtin_arm_mve_vaddlvq_u32,
  BI__builtin_arm_mve_vaddq_f16,
  BI__builtin_arm_mve_vaddq_f32,
  BI__builtin_arm_mve_vaddq_m_f16,
  BI__builtin_arm_mve_vaddq_m_f32,
  BI__builtin_arm_mve_vaddq_m_n_f16,
  BI__builtin_arm_mve_vaddq_m_n_f32,
  BI__builtin_arm_mve_vaddq_m_n_s16,
  BI__builtin_arm_mve_vaddq_m_n_s32,
  BI__builtin_arm_mve_vaddq_m_n_s8,
  BI__builtin_arm_mve_vaddq_m_n_u16,
  BI__builtin_arm_mve_vaddq_m_n_u32,
  BI__builtin_arm_mve_vaddq_m_n_u8,
  BI__builtin_arm_mve_vaddq_m_s16,
  BI__builtin_arm_mve_vaddq_m_s32,
  BI__builtin_arm_mve_vaddq_m_s8,
  BI__builtin_arm_mve_vaddq_m_u16,
  BI__builtin_arm_mve_vaddq_m_u32,
  BI__builtin_arm_mve_vaddq_m_u8,
  BI__builtin_arm_mve_vaddq_n_f16,
  BI__builtin_arm_mve_vaddq_n_f32,
  BI__builtin_arm_mve_vaddq_n_s16,
  BI__builtin_arm_mve_vaddq_n_s32,
  BI__builtin_arm_mve_vaddq_n_s8,
  BI__builtin_arm_mve_vaddq_n_u16,
  BI__builtin_arm_mve_vaddq_n_u32,
  BI__builtin_arm_mve_vaddq_n_u8,
  BI__builtin_arm_mve_vaddq_s16,
  BI__builtin_arm_mve_vaddq_s32,
  BI__builtin_arm_mve_vaddq_s8,
  BI__builtin_arm_mve_vaddq_u16,
  BI__builtin_arm_mve_vaddq_u32,
  BI__builtin_arm_mve_vaddq_u8,
  BI__builtin_arm_mve_vaddq_x_f16,
  BI__builtin_arm_mve_vaddq_x_f32,
  BI__builtin_arm_mve_vaddq_x_n_f16,
  BI__builtin_arm_mve_vaddq_x_n_f32,
  BI__builtin_arm_mve_vaddq_x_n_s16,
  BI__builtin_arm_mve_vaddq_x_n_s32,
  BI__builtin_arm_mve_vaddq_x_n_s8,
  BI__builtin_arm_mve_vaddq_x_n_u16,
  BI__builtin_arm_mve_vaddq_x_n_u32,
  BI__builtin_arm_mve_vaddq_x_n_u8,
  BI__builtin_arm_mve_vaddq_x_s16,
  BI__builtin_arm_mve_vaddq_x_s32,
  BI__builtin_arm_mve_vaddq_x_s8,
  BI__builtin_arm_mve_vaddq_x_u16,
  BI__builtin_arm_mve_vaddq_x_u32,
  BI__builtin_arm_mve_vaddq_x_u8,
  BI__builtin_arm_mve_vaddvaq_p_s16,
  BI__builtin_arm_mve_vaddvaq_p_s32,
  BI__builtin_arm_mve_vaddvaq_p_s8,
  BI__builtin_arm_mve_vaddvaq_p_u16,
  BI__builtin_arm_mve_vaddvaq_p_u32,
  BI__builtin_arm_mve_vaddvaq_p_u8,
  BI__builtin_arm_mve_vaddvaq_s16,
  BI__builtin_arm_mve_vaddvaq_s32,
  BI__builtin_arm_mve_vaddvaq_s8,
  BI__builtin_arm_mve_vaddvaq_u16,
  BI__builtin_arm_mve_vaddvaq_u32,
  BI__builtin_arm_mve_vaddvaq_u8,
  BI__builtin_arm_mve_vaddvq_p_s16,
  BI__builtin_arm_mve_vaddvq_p_s32,
  BI__builtin_arm_mve_vaddvq_p_s8,
  BI__builtin_arm_mve_vaddvq_p_u16,
  BI__builtin_arm_mve_vaddvq_p_u32,
  BI__builtin_arm_mve_vaddvq_p_u8,
  BI__builtin_arm_mve_vaddvq_s16,
  BI__builtin_arm_mve_vaddvq_s32,
  BI__builtin_arm_mve_vaddvq_s8,
  BI__builtin_arm_mve_vaddvq_u16,
  BI__builtin_arm_mve_vaddvq_u32,
  BI__builtin_arm_mve_vaddvq_u8,
  BI__builtin_arm_mve_vandq_f16,
  BI__builtin_arm_mve_vandq_f32,
  BI__builtin_arm_mve_vandq_m_f16,
  BI__builtin_arm_mve_vandq_m_f32,
  BI__builtin_arm_mve_vandq_m_s16,
  BI__builtin_arm_mve_vandq_m_s32,
  BI__builtin_arm_mve_vandq_m_s8,
  BI__builtin_arm_mve_vandq_m_u16,
  BI__builtin_arm_mve_vandq_m_u32,
  BI__builtin_arm_mve_vandq_m_u8,
  BI__builtin_arm_mve_vandq_s16,
  BI__builtin_arm_mve_vandq_s32,
  BI__builtin_arm_mve_vandq_s8,
  BI__builtin_arm_mve_vandq_u16,
  BI__builtin_arm_mve_vandq_u32,
  BI__builtin_arm_mve_vandq_u8,
  BI__builtin_arm_mve_vandq_x_f16,
  BI__builtin_arm_mve_vandq_x_f32,
  BI__builtin_arm_mve_vandq_x_s16,
  BI__builtin_arm_mve_vandq_x_s32,
  BI__builtin_arm_mve_vandq_x_s8,
  BI__builtin_arm_mve_vandq_x_u16,
  BI__builtin_arm_mve_vandq_x_u32,
  BI__builtin_arm_mve_vandq_x_u8,
  BI__builtin_arm_mve_vbicq_f16,
  BI__builtin_arm_mve_vbicq_f32,
  BI__builtin_arm_mve_vbicq_m_f16,
  BI__builtin_arm_mve_vbicq_m_f32,
  BI__builtin_arm_mve_vbicq_m_n_s16,
  BI__builtin_arm_mve_vbicq_m_n_s32,
  BI__builtin_arm_mve_vbicq_m_n_u16,
  BI__builtin_arm_mve_vbicq_m_n_u32,
  BI__builtin_arm_mve_vbicq_m_s16,
  BI__builtin_arm_mve_vbicq_m_s32,
  BI__builtin_arm_mve_vbicq_m_s8,
  BI__builtin_arm_mve_vbicq_m_u16,
  BI__builtin_arm_mve_vbicq_m_u32,
  BI__builtin_arm_mve_vbicq_m_u8,
  BI__builtin_arm_mve_vbicq_n_s16,
  BI__builtin_arm_mve_vbicq_n_s32,
  BI__builtin_arm_mve_vbicq_n_u16,
  BI__builtin_arm_mve_vbicq_n_u32,
  BI__builtin_arm_mve_vbicq_s16,
  BI__builtin_arm_mve_vbicq_s32,
  BI__builtin_arm_mve_vbicq_s8,
  BI__builtin_arm_mve_vbicq_u16,
  BI__builtin_arm_mve_vbicq_u32,
  BI__builtin_arm_mve_vbicq_u8,
  BI__builtin_arm_mve_vbicq_x_f16,
  BI__builtin_arm_mve_vbicq_x_f32,
  BI__builtin_arm_mve_vbicq_x_s16,
  BI__builtin_arm_mve_vbicq_x_s32,
  BI__builtin_arm_mve_vbicq_x_s8,
  BI__builtin_arm_mve_vbicq_x_u16,
  BI__builtin_arm_mve_vbicq_x_u32,
  BI__builtin_arm_mve_vbicq_x_u8,
  BI__builtin_arm_mve_vbrsrq_m_n_f16,
  BI__builtin_arm_mve_vbrsrq_m_n_f32,
  BI__builtin_arm_mve_vbrsrq_m_n_s16,
  BI__builtin_arm_mve_vbrsrq_m_n_s32,
  BI__builtin_arm_mve_vbrsrq_m_n_s8,
  BI__builtin_arm_mve_vbrsrq_m_n_u16,
  BI__builtin_arm_mve_vbrsrq_m_n_u32,
  BI__builtin_arm_mve_vbrsrq_m_n_u8,
  BI__builtin_arm_mve_vbrsrq_n_f16,
  BI__builtin_arm_mve_vbrsrq_n_f32,
  BI__builtin_arm_mve_vbrsrq_n_s16,
  BI__builtin_arm_mve_vbrsrq_n_s32,
  BI__builtin_arm_mve_vbrsrq_n_s8,
  BI__builtin_arm_mve_vbrsrq_n_u16,
  BI__builtin_arm_mve_vbrsrq_n_u32,
  BI__builtin_arm_mve_vbrsrq_n_u8,
  BI__builtin_arm_mve_vbrsrq_x_n_f16,
  BI__builtin_arm_mve_vbrsrq_x_n_f32,
  BI__builtin_arm_mve_vbrsrq_x_n_s16,
  BI__builtin_arm_mve_vbrsrq_x_n_s32,
  BI__builtin_arm_mve_vbrsrq_x_n_s8,
  BI__builtin_arm_mve_vbrsrq_x_n_u16,
  BI__builtin_arm_mve_vbrsrq_x_n_u32,
  BI__builtin_arm_mve_vbrsrq_x_n_u8,
  BI__builtin_arm_mve_vcaddq_rot270_f16,
  BI__builtin_arm_mve_vcaddq_rot270_f32,
  BI__builtin_arm_mve_vcaddq_rot270_m_f16,
  BI__builtin_arm_mve_vcaddq_rot270_m_f32,
  BI__builtin_arm_mve_vcaddq_rot270_m_s16,
  BI__builtin_arm_mve_vcaddq_rot270_m_s32,
  BI__builtin_arm_mve_vcaddq_rot270_m_s8,
  BI__builtin_arm_mve_vcaddq_rot270_m_u16,
  BI__builtin_arm_mve_vcaddq_rot270_m_u32,
  BI__builtin_arm_mve_vcaddq_rot270_m_u8,
  BI__builtin_arm_mve_vcaddq_rot270_s16,
  BI__builtin_arm_mve_vcaddq_rot270_s32,
  BI__builtin_arm_mve_vcaddq_rot270_s8,
  BI__builtin_arm_mve_vcaddq_rot270_u16,
  BI__builtin_arm_mve_vcaddq_rot270_u32,
  BI__builtin_arm_mve_vcaddq_rot270_u8,
  BI__builtin_arm_mve_vcaddq_rot270_x_f16,
  BI__builtin_arm_mve_vcaddq_rot270_x_f32,
  BI__builtin_arm_mve_vcaddq_rot270_x_s16,
  BI__builtin_arm_mve_vcaddq_rot270_x_s32,
  BI__builtin_arm_mve_vcaddq_rot270_x_s8,
  BI__builtin_arm_mve_vcaddq_rot270_x_u16,
  BI__builtin_arm_mve_vcaddq_rot270_x_u32,
  BI__builtin_arm_mve_vcaddq_rot270_x_u8,
  BI__builtin_arm_mve_vcaddq_rot90_f16,
  BI__builtin_arm_mve_vcaddq_rot90_f32,
  BI__builtin_arm_mve_vcaddq_rot90_m_f16,
  BI__builtin_arm_mve_vcaddq_rot90_m_f32,
  BI__builtin_arm_mve_vcaddq_rot90_m_s16,
  BI__builtin_arm_mve_vcaddq_rot90_m_s32,
  BI__builtin_arm_mve_vcaddq_rot90_m_s8,
  BI__builtin_arm_mve_vcaddq_rot90_m_u16,
  BI__builtin_arm_mve_vcaddq_rot90_m_u32,
  BI__builtin_arm_mve_vcaddq_rot90_m_u8,
  BI__builtin_arm_mve_vcaddq_rot90_s16,
  BI__builtin_arm_mve_vcaddq_rot90_s32,
  BI__builtin_arm_mve_vcaddq_rot90_s8,
  BI__builtin_arm_mve_vcaddq_rot90_u16,
  BI__builtin_arm_mve_vcaddq_rot90_u32,
  BI__builtin_arm_mve_vcaddq_rot90_u8,
  BI__builtin_arm_mve_vcaddq_rot90_x_f16,
  BI__builtin_arm_mve_vcaddq_rot90_x_f32,
  BI__builtin_arm_mve_vcaddq_rot90_x_s16,
  BI__builtin_arm_mve_vcaddq_rot90_x_s32,
  BI__builtin_arm_mve_vcaddq_rot90_x_s8,
  BI__builtin_arm_mve_vcaddq_rot90_x_u16,
  BI__builtin_arm_mve_vcaddq_rot90_x_u32,
  BI__builtin_arm_mve_vcaddq_rot90_x_u8,
  BI__builtin_arm_mve_vclsq_m_s16,
  BI__builtin_arm_mve_vclsq_m_s32,
  BI__builtin_arm_mve_vclsq_m_s8,
  BI__builtin_arm_mve_vclsq_s16,
  BI__builtin_arm_mve_vclsq_s32,
  BI__builtin_arm_mve_vclsq_s8,
  BI__builtin_arm_mve_vclsq_x_s16,
  BI__builtin_arm_mve_vclsq_x_s32,
  BI__builtin_arm_mve_vclsq_x_s8,
  BI__builtin_arm_mve_vclzq_m_s16,
  BI__builtin_arm_mve_vclzq_m_s32,
  BI__builtin_arm_mve_vclzq_m_s8,
  BI__builtin_arm_mve_vclzq_m_u16,
  BI__builtin_arm_mve_vclzq_m_u32,
  BI__builtin_arm_mve_vclzq_m_u8,
  BI__builtin_arm_mve_vclzq_s16,
  BI__builtin_arm_mve_vclzq_s32,
  BI__builtin_arm_mve_vclzq_s8,
  BI__builtin_arm_mve_vclzq_u16,
  BI__builtin_arm_mve_vclzq_u32,
  BI__builtin_arm_mve_vclzq_u8,
  BI__builtin_arm_mve_vclzq_x_s16,
  BI__builtin_arm_mve_vclzq_x_s32,
  BI__builtin_arm_mve_vclzq_x_s8,
  BI__builtin_arm_mve_vclzq_x_u16,
  BI__builtin_arm_mve_vclzq_x_u32,
  BI__builtin_arm_mve_vclzq_x_u8,
  BI__builtin_arm_mve_vcmlaq_f16,
  BI__builtin_arm_mve_vcmlaq_f32,
  BI__builtin_arm_mve_vcmlaq_m_f16,
  BI__builtin_arm_mve_vcmlaq_m_f32,
  BI__builtin_arm_mve_vcmlaq_rot180_f16,
  BI__builtin_arm_mve_vcmlaq_rot180_f32,
  BI__builtin_arm_mve_vcmlaq_rot180_m_f16,
  BI__builtin_arm_mve_vcmlaq_rot180_m_f32,
  BI__builtin_arm_mve_vcmlaq_rot270_f16,
  BI__builtin_arm_mve_vcmlaq_rot270_f32,
  BI__builtin_arm_mve_vcmlaq_rot270_m_f16,
  BI__builtin_arm_mve_vcmlaq_rot270_m_f32,
  BI__builtin_arm_mve_vcmlaq_rot90_f16,
  BI__builtin_arm_mve_vcmlaq_rot90_f32,
  BI__builtin_arm_mve_vcmlaq_rot90_m_f16,
  BI__builtin_arm_mve_vcmlaq_rot90_m_f32,
  BI__builtin_arm_mve_vcmpcsq_m_n_u16,
  BI__builtin_arm_mve_vcmpcsq_m_n_u32,
  BI__builtin_arm_mve_vcmpcsq_m_n_u8,
  BI__builtin_arm_mve_vcmpcsq_m_u16,
  BI__builtin_arm_mve_vcmpcsq_m_u32,
  BI__builtin_arm_mve_vcmpcsq_m_u8,
  BI__builtin_arm_mve_vcmpcsq_n_u16,
  BI__builtin_arm_mve_vcmpcsq_n_u32,
  BI__builtin_arm_mve_vcmpcsq_n_u8,
  BI__builtin_arm_mve_vcmpcsq_u16,
  BI__builtin_arm_mve_vcmpcsq_u32,
  BI__builtin_arm_mve_vcmpcsq_u8,
  BI__builtin_arm_mve_vcmpeqq_f16,
  BI__builtin_arm_mve_vcmpeqq_f32,
  BI__builtin_arm_mve_vcmpeqq_m_f16,
  BI__builtin_arm_mve_vcmpeqq_m_f32,
  BI__builtin_arm_mve_vcmpeqq_m_n_f16,
  BI__builtin_arm_mve_vcmpeqq_m_n_f32,
  BI__builtin_arm_mve_vcmpeqq_m_n_s16,
  BI__builtin_arm_mve_vcmpeqq_m_n_s32,
  BI__builtin_arm_mve_vcmpeqq_m_n_s8,
  BI__builtin_arm_mve_vcmpeqq_m_n_u16,
  BI__builtin_arm_mve_vcmpeqq_m_n_u32,
  BI__builtin_arm_mve_vcmpeqq_m_n_u8,
  BI__builtin_arm_mve_vcmpeqq_m_s16,
  BI__builtin_arm_mve_vcmpeqq_m_s32,
  BI__builtin_arm_mve_vcmpeqq_m_s8,
  BI__builtin_arm_mve_vcmpeqq_m_u16,
  BI__builtin_arm_mve_vcmpeqq_m_u32,
  BI__builtin_arm_mve_vcmpeqq_m_u8,
  BI__builtin_arm_mve_vcmpeqq_n_f16,
  BI__builtin_arm_mve_vcmpeqq_n_f32,
  BI__builtin_arm_mve_vcmpeqq_n_s16,
  BI__builtin_arm_mve_vcmpeqq_n_s32,
  BI__builtin_arm_mve_vcmpeqq_n_s8,
  BI__builtin_arm_mve_vcmpeqq_n_u16,
  BI__builtin_arm_mve_vcmpeqq_n_u32,
  BI__builtin_arm_mve_vcmpeqq_n_u8,
  BI__builtin_arm_mve_vcmpeqq_s16,
  BI__builtin_arm_mve_vcmpeqq_s32,
  BI__builtin_arm_mve_vcmpeqq_s8,
  BI__builtin_arm_mve_vcmpeqq_u16,
  BI__builtin_arm_mve_vcmpeqq_u32,
  BI__builtin_arm_mve_vcmpeqq_u8,
  BI__builtin_arm_mve_vcmpgeq_f16,
  BI__builtin_arm_mve_vcmpgeq_f32,
  BI__builtin_arm_mve_vcmpgeq_m_f16,
  BI__builtin_arm_mve_vcmpgeq_m_f32,
  BI__builtin_arm_mve_vcmpgeq_m_n_f16,
  BI__builtin_arm_mve_vcmpgeq_m_n_f32,
  BI__builtin_arm_mve_vcmpgeq_m_n_s16,
  BI__builtin_arm_mve_vcmpgeq_m_n_s32,
  BI__builtin_arm_mve_vcmpgeq_m_n_s8,
  BI__builtin_arm_mve_vcmpgeq_m_s16,
  BI__builtin_arm_mve_vcmpgeq_m_s32,
  BI__builtin_arm_mve_vcmpgeq_m_s8,
  BI__builtin_arm_mve_vcmpgeq_n_f16,
  BI__builtin_arm_mve_vcmpgeq_n_f32,
  BI__builtin_arm_mve_vcmpgeq_n_s16,
  BI__builtin_arm_mve_vcmpgeq_n_s32,
  BI__builtin_arm_mve_vcmpgeq_n_s8,
  BI__builtin_arm_mve_vcmpgeq_s16,
  BI__builtin_arm_mve_vcmpgeq_s32,
  BI__builtin_arm_mve_vcmpgeq_s8,
  BI__builtin_arm_mve_vcmpgtq_f16,
  BI__builtin_arm_mve_vcmpgtq_f32,
  BI__builtin_arm_mve_vcmpgtq_m_f16,
  BI__builtin_arm_mve_vcmpgtq_m_f32,
  BI__builtin_arm_mve_vcmpgtq_m_n_f16,
  BI__builtin_arm_mve_vcmpgtq_m_n_f32,
  BI__builtin_arm_mve_vcmpgtq_m_n_s16,
  BI__builtin_arm_mve_vcmpgtq_m_n_s32,
  BI__builtin_arm_mve_vcmpgtq_m_n_s8,
  BI__builtin_arm_mve_vcmpgtq_m_s16,
  BI__builtin_arm_mve_vcmpgtq_m_s32,
  BI__builtin_arm_mve_vcmpgtq_m_s8,
  BI__builtin_arm_mve_vcmpgtq_n_f16,
  BI__builtin_arm_mve_vcmpgtq_n_f32,
  BI__builtin_arm_mve_vcmpgtq_n_s16,
  BI__builtin_arm_mve_vcmpgtq_n_s32,
  BI__builtin_arm_mve_vcmpgtq_n_s8,
  BI__builtin_arm_mve_vcmpgtq_s16,
  BI__builtin_arm_mve_vcmpgtq_s32,
  BI__builtin_arm_mve_vcmpgtq_s8,
  BI__builtin_arm_mve_vcmphiq_m_n_u16,
  BI__builtin_arm_mve_vcmphiq_m_n_u32,
  BI__builtin_arm_mve_vcmphiq_m_n_u8,
  BI__builtin_arm_mve_vcmphiq_m_u16,
  BI__builtin_arm_mve_vcmphiq_m_u32,
  BI__builtin_arm_mve_vcmphiq_m_u8,
  BI__builtin_arm_mve_vcmphiq_n_u16,
  BI__builtin_arm_mve_vcmphiq_n_u32,
  BI__builtin_arm_mve_vcmphiq_n_u8,
  BI__builtin_arm_mve_vcmphiq_u16,
  BI__builtin_arm_mve_vcmphiq_u32,
  BI__builtin_arm_mve_vcmphiq_u8,
  BI__builtin_arm_mve_vcmpleq_f16,
  BI__builtin_arm_mve_vcmpleq_f32,
  BI__builtin_arm_mve_vcmpleq_m_f16,
  BI__builtin_arm_mve_vcmpleq_m_f32,
  BI__builtin_arm_mve_vcmpleq_m_n_f16,
  BI__builtin_arm_mve_vcmpleq_m_n_f32,
  BI__builtin_arm_mve_vcmpleq_m_n_s16,
  BI__builtin_arm_mve_vcmpleq_m_n_s32,
  BI__builtin_arm_mve_vcmpleq_m_n_s8,
  BI__builtin_arm_mve_vcmpleq_m_s16,
  BI__builtin_arm_mve_vcmpleq_m_s32,
  BI__builtin_arm_mve_vcmpleq_m_s8,
  BI__builtin_arm_mve_vcmpleq_n_f16,
  BI__builtin_arm_mve_vcmpleq_n_f32,
  BI__builtin_arm_mve_vcmpleq_n_s16,
  BI__builtin_arm_mve_vcmpleq_n_s32,
  BI__builtin_arm_mve_vcmpleq_n_s8,
  BI__builtin_arm_mve_vcmpleq_s16,
  BI__builtin_arm_mve_vcmpleq_s32,
  BI__builtin_arm_mve_vcmpleq_s8,
  BI__builtin_arm_mve_vcmpltq_f16,
  BI__builtin_arm_mve_vcmpltq_f32,
  BI__builtin_arm_mve_vcmpltq_m_f16,
  BI__builtin_arm_mve_vcmpltq_m_f32,
  BI__builtin_arm_mve_vcmpltq_m_n_f16,
  BI__builtin_arm_mve_vcmpltq_m_n_f32,
  BI__builtin_arm_mve_vcmpltq_m_n_s16,
  BI__builtin_arm_mve_vcmpltq_m_n_s32,
  BI__builtin_arm_mve_vcmpltq_m_n_s8,
  BI__builtin_arm_mve_vcmpltq_m_s16,
  BI__builtin_arm_mve_vcmpltq_m_s32,
  BI__builtin_arm_mve_vcmpltq_m_s8,
  BI__builtin_arm_mve_vcmpltq_n_f16,
  BI__builtin_arm_mve_vcmpltq_n_f32,
  BI__builtin_arm_mve_vcmpltq_n_s16,
  BI__builtin_arm_mve_vcmpltq_n_s32,
  BI__builtin_arm_mve_vcmpltq_n_s8,
  BI__builtin_arm_mve_vcmpltq_s16,
  BI__builtin_arm_mve_vcmpltq_s32,
  BI__builtin_arm_mve_vcmpltq_s8,
  BI__builtin_arm_mve_vcmpneq_f16,
  BI__builtin_arm_mve_vcmpneq_f32,
  BI__builtin_arm_mve_vcmpneq_m_f16,
  BI__builtin_arm_mve_vcmpneq_m_f32,
  BI__builtin_arm_mve_vcmpneq_m_n_f16,
  BI__builtin_arm_mve_vcmpneq_m_n_f32,
  BI__builtin_arm_mve_vcmpneq_m_n_s16,
  BI__builtin_arm_mve_vcmpneq_m_n_s32,
  BI__builtin_arm_mve_vcmpneq_m_n_s8,
  BI__builtin_arm_mve_vcmpneq_m_n_u16,
  BI__builtin_arm_mve_vcmpneq_m_n_u32,
  BI__builtin_arm_mve_vcmpneq_m_n_u8,
  BI__builtin_arm_mve_vcmpneq_m_s16,
  BI__builtin_arm_mve_vcmpneq_m_s32,
  BI__builtin_arm_mve_vcmpneq_m_s8,
  BI__builtin_arm_mve_vcmpneq_m_u16,
  BI__builtin_arm_mve_vcmpneq_m_u32,
  BI__builtin_arm_mve_vcmpneq_m_u8,
  BI__builtin_arm_mve_vcmpneq_n_f16,
  BI__builtin_arm_mve_vcmpneq_n_f32,
  BI__builtin_arm_mve_vcmpneq_n_s16,
  BI__builtin_arm_mve_vcmpneq_n_s32,
  BI__builtin_arm_mve_vcmpneq_n_s8,
  BI__builtin_arm_mve_vcmpneq_n_u16,
  BI__builtin_arm_mve_vcmpneq_n_u32,
  BI__builtin_arm_mve_vcmpneq_n_u8,
  BI__builtin_arm_mve_vcmpneq_s16,
  BI__builtin_arm_mve_vcmpneq_s32,
  BI__builtin_arm_mve_vcmpneq_s8,
  BI__builtin_arm_mve_vcmpneq_u16,
  BI__builtin_arm_mve_vcmpneq_u32,
  BI__builtin_arm_mve_vcmpneq_u8,
  BI__builtin_arm_mve_vcmulq_f16,
  BI__builtin_arm_mve_vcmulq_f32,
  BI__builtin_arm_mve_vcmulq_m_f16,
  BI__builtin_arm_mve_vcmulq_m_f32,
  BI__builtin_arm_mve_vcmulq_rot180_f16,
  BI__builtin_arm_mve_vcmulq_rot180_f32,
  BI__builtin_arm_mve_vcmulq_rot180_m_f16,
  BI__builtin_arm_mve_vcmulq_rot180_m_f32,
  BI__builtin_arm_mve_vcmulq_rot180_x_f16,
  BI__builtin_arm_mve_vcmulq_rot180_x_f32,
  BI__builtin_arm_mve_vcmulq_rot270_f16,
  BI__builtin_arm_mve_vcmulq_rot270_f32,
  BI__builtin_arm_mve_vcmulq_rot270_m_f16,
  BI__builtin_arm_mve_vcmulq_rot270_m_f32,
  BI__builtin_arm_mve_vcmulq_rot270_x_f16,
  BI__builtin_arm_mve_vcmulq_rot270_x_f32,
  BI__builtin_arm_mve_vcmulq_rot90_f16,
  BI__builtin_arm_mve_vcmulq_rot90_f32,
  BI__builtin_arm_mve_vcmulq_rot90_m_f16,
  BI__builtin_arm_mve_vcmulq_rot90_m_f32,
  BI__builtin_arm_mve_vcmulq_rot90_x_f16,
  BI__builtin_arm_mve_vcmulq_rot90_x_f32,
  BI__builtin_arm_mve_vcmulq_x_f16,
  BI__builtin_arm_mve_vcmulq_x_f32,
  BI__builtin_arm_mve_vcreateq_f16,
  BI__builtin_arm_mve_vcreateq_f32,
  BI__builtin_arm_mve_vcreateq_s16,
  BI__builtin_arm_mve_vcreateq_s32,
  BI__builtin_arm_mve_vcreateq_s64,
  BI__builtin_arm_mve_vcreateq_s8,
  BI__builtin_arm_mve_vcreateq_u16,
  BI__builtin_arm_mve_vcreateq_u32,
  BI__builtin_arm_mve_vcreateq_u64,
  BI__builtin_arm_mve_vcreateq_u8,
  BI__builtin_arm_mve_vctp16q,
  BI__builtin_arm_mve_vctp16q_m,
  BI__builtin_arm_mve_vctp32q,
  BI__builtin_arm_mve_vctp32q_m,
  BI__builtin_arm_mve_vctp64q,
  BI__builtin_arm_mve_vctp64q_m,
  BI__builtin_arm_mve_vctp8q,
  BI__builtin_arm_mve_vctp8q_m,
  BI__builtin_arm_mve_vcvtaq_m_s16_f16,
  BI__builtin_arm_mve_vcvtaq_m_s32_f32,
  BI__builtin_arm_mve_vcvtaq_m_u16_f16,
  BI__builtin_arm_mve_vcvtaq_m_u32_f32,
  BI__builtin_arm_mve_vcvtaq_s16_f16,
  BI__builtin_arm_mve_vcvtaq_s32_f32,
  BI__builtin_arm_mve_vcvtaq_u16_f16,
  BI__builtin_arm_mve_vcvtaq_u32_f32,
  BI__builtin_arm_mve_vcvtaq_x_s16_f16,
  BI__builtin_arm_mve_vcvtaq_x_s32_f32,
  BI__builtin_arm_mve_vcvtaq_x_u16_f16,
  BI__builtin_arm_mve_vcvtaq_x_u32_f32,
  BI__builtin_arm_mve_vcvtbq_f16_f32,
  BI__builtin_arm_mve_vcvtbq_f32_f16,
  BI__builtin_arm_mve_vcvtbq_m_f16_f32,
  BI__builtin_arm_mve_vcvtbq_m_f32_f16,
  BI__builtin_arm_mve_vcvtbq_x_f32_f16,
  BI__builtin_arm_mve_vcvtmq_m_s16_f16,
  BI__builtin_arm_mve_vcvtmq_m_s32_f32,
  BI__builtin_arm_mve_vcvtmq_m_u16_f16,
  BI__builtin_arm_mve_vcvtmq_m_u32_f32,
  BI__builtin_arm_mve_vcvtmq_s16_f16,
  BI__builtin_arm_mve_vcvtmq_s32_f32,
  BI__builtin_arm_mve_vcvtmq_u16_f16,
  BI__builtin_arm_mve_vcvtmq_u32_f32,
  BI__builtin_arm_mve_vcvtmq_x_s16_f16,
  BI__builtin_arm_mve_vcvtmq_x_s32_f32,
  BI__builtin_arm_mve_vcvtmq_x_u16_f16,
  BI__builtin_arm_mve_vcvtmq_x_u32_f32,
  BI__builtin_arm_mve_vcvtnq_m_s16_f16,
  BI__builtin_arm_mve_vcvtnq_m_s32_f32,
  BI__builtin_arm_mve_vcvtnq_m_u16_f16,
  BI__builtin_arm_mve_vcvtnq_m_u32_f32,
  BI__builtin_arm_mve_vcvtnq_s16_f16,
  BI__builtin_arm_mve_vcvtnq_s32_f32,
  BI__builtin_arm_mve_vcvtnq_u16_f16,
  BI__builtin_arm_mve_vcvtnq_u32_f32,
  BI__builtin_arm_mve_vcvtnq_x_s16_f16,
  BI__builtin_arm_mve_vcvtnq_x_s32_f32,
  BI__builtin_arm_mve_vcvtnq_x_u16_f16,
  BI__builtin_arm_mve_vcvtnq_x_u32_f32,
  BI__builtin_arm_mve_vcvtpq_m_s16_f16,
  BI__builtin_arm_mve_vcvtpq_m_s32_f32,
  BI__builtin_arm_mve_vcvtpq_m_u16_f16,
  BI__builtin_arm_mve_vcvtpq_m_u32_f32,
  BI__builtin_arm_mve_vcvtpq_s16_f16,
  BI__builtin_arm_mve_vcvtpq_s32_f32,
  BI__builtin_arm_mve_vcvtpq_u16_f16,
  BI__builtin_arm_mve_vcvtpq_u32_f32,
  BI__builtin_arm_mve_vcvtpq_x_s16_f16,
  BI__builtin_arm_mve_vcvtpq_x_s32_f32,
  BI__builtin_arm_mve_vcvtpq_x_u16_f16,
  BI__builtin_arm_mve_vcvtpq_x_u32_f32,
  BI__builtin_arm_mve_vcvtq_f16_s16,
  BI__builtin_arm_mve_vcvtq_f16_u16,
  BI__builtin_arm_mve_vcvtq_f32_s32,
  BI__builtin_arm_mve_vcvtq_f32_u32,
  BI__builtin_arm_mve_vcvtq_m_f16_s16,
  BI__builtin_arm_mve_vcvtq_m_f16_u16,
  BI__builtin_arm_mve_vcvtq_m_f32_s32,
  BI__builtin_arm_mve_vcvtq_m_f32_u32,
  BI__builtin_arm_mve_vcvtq_m_n_f16_s16,
  BI__builtin_arm_mve_vcvtq_m_n_f16_u16,
  BI__builtin_arm_mve_vcvtq_m_n_f32_s32,
  BI__builtin_arm_mve_vcvtq_m_n_f32_u32,
  BI__builtin_arm_mve_vcvtq_m_n_s16_f16,
  BI__builtin_arm_mve_vcvtq_m_n_s32_f32,
  BI__builtin_arm_mve_vcvtq_m_n_u16_f16,
  BI__builtin_arm_mve_vcvtq_m_n_u32_f32,
  BI__builtin_arm_mve_vcvtq_m_s16_f16,
  BI__builtin_arm_mve_vcvtq_m_s32_f32,
  BI__builtin_arm_mve_vcvtq_m_u16_f16,
  BI__builtin_arm_mve_vcvtq_m_u32_f32,
  BI__builtin_arm_mve_vcvtq_n_f16_s16,
  BI__builtin_arm_mve_vcvtq_n_f16_u16,
  BI__builtin_arm_mve_vcvtq_n_f32_s32,
  BI__builtin_arm_mve_vcvtq_n_f32_u32,
  BI__builtin_arm_mve_vcvtq_n_s16_f16,
  BI__builtin_arm_mve_vcvtq_n_s32_f32,
  BI__builtin_arm_mve_vcvtq_n_u16_f16,
  BI__builtin_arm_mve_vcvtq_n_u32_f32,
  BI__builtin_arm_mve_vcvtq_s16_f16,
  BI__builtin_arm_mve_vcvtq_s32_f32,
  BI__builtin_arm_mve_vcvtq_u16_f16,
  BI__builtin_arm_mve_vcvtq_u32_f32,
  BI__builtin_arm_mve_vcvtq_x_f16_s16,
  BI__builtin_arm_mve_vcvtq_x_f16_u16,
  BI__builtin_arm_mve_vcvtq_x_f32_s32,
  BI__builtin_arm_mve_vcvtq_x_f32_u32,
  BI__builtin_arm_mve_vcvtq_x_n_f16_s16,
  BI__builtin_arm_mve_vcvtq_x_n_f16_u16,
  BI__builtin_arm_mve_vcvtq_x_n_f32_s32,
  BI__builtin_arm_mve_vcvtq_x_n_f32_u32,
  BI__builtin_arm_mve_vcvtq_x_n_s16_f16,
  BI__builtin_arm_mve_vcvtq_x_n_s32_f32,
  BI__builtin_arm_mve_vcvtq_x_n_u16_f16,
  BI__builtin_arm_mve_vcvtq_x_n_u32_f32,
  BI__builtin_arm_mve_vcvtq_x_s16_f16,
  BI__builtin_arm_mve_vcvtq_x_s32_f32,
  BI__builtin_arm_mve_vcvtq_x_u16_f16,
  BI__builtin_arm_mve_vcvtq_x_u32_f32,
  BI__builtin_arm_mve_vcvttq_f16_f32,
  BI__builtin_arm_mve_vcvttq_f32_f16,
  BI__builtin_arm_mve_vcvttq_m_f16_f32,
  BI__builtin_arm_mve_vcvttq_m_f32_f16,
  BI__builtin_arm_mve_vcvttq_x_f32_f16,
  BI__builtin_arm_mve_vddupq_m_n_u16,
  BI__builtin_arm_mve_vddupq_m_n_u32,
  BI__builtin_arm_mve_vddupq_m_n_u8,
  BI__builtin_arm_mve_vddupq_m_wb_u16,
  BI__builtin_arm_mve_vddupq_m_wb_u32,
  BI__builtin_arm_mve_vddupq_m_wb_u8,
  BI__builtin_arm_mve_vddupq_n_u16,
  BI__builtin_arm_mve_vddupq_n_u32,
  BI__builtin_arm_mve_vddupq_n_u8,
  BI__builtin_arm_mve_vddupq_wb_u16,
  BI__builtin_arm_mve_vddupq_wb_u32,
  BI__builtin_arm_mve_vddupq_wb_u8,
  BI__builtin_arm_mve_vddupq_x_n_u16,
  BI__builtin_arm_mve_vddupq_x_n_u32,
  BI__builtin_arm_mve_vddupq_x_n_u8,
  BI__builtin_arm_mve_vddupq_x_wb_u16,
  BI__builtin_arm_mve_vddupq_x_wb_u32,
  BI__builtin_arm_mve_vddupq_x_wb_u8,
  BI__builtin_arm_mve_vdupq_m_n_f16,
  BI__builtin_arm_mve_vdupq_m_n_f32,
  BI__builtin_arm_mve_vdupq_m_n_s16,
  BI__builtin_arm_mve_vdupq_m_n_s32,
  BI__builtin_arm_mve_vdupq_m_n_s8,
  BI__builtin_arm_mve_vdupq_m_n_u16,
  BI__builtin_arm_mve_vdupq_m_n_u32,
  BI__builtin_arm_mve_vdupq_m_n_u8,
  BI__builtin_arm_mve_vdupq_n_f16,
  BI__builtin_arm_mve_vdupq_n_f32,
  BI__builtin_arm_mve_vdupq_n_s16,
  BI__builtin_arm_mve_vdupq_n_s32,
  BI__builtin_arm_mve_vdupq_n_s8,
  BI__builtin_arm_mve_vdupq_n_u16,
  BI__builtin_arm_mve_vdupq_n_u32,
  BI__builtin_arm_mve_vdupq_n_u8,
  BI__builtin_arm_mve_vdupq_x_n_f16,
  BI__builtin_arm_mve_vdupq_x_n_f32,
  BI__builtin_arm_mve_vdupq_x_n_s16,
  BI__builtin_arm_mve_vdupq_x_n_s32,
  BI__builtin_arm_mve_vdupq_x_n_s8,
  BI__builtin_arm_mve_vdupq_x_n_u16,
  BI__builtin_arm_mve_vdupq_x_n_u32,
  BI__builtin_arm_mve_vdupq_x_n_u8,
  BI__builtin_arm_mve_vdwdupq_m_n_u16,
  BI__builtin_arm_mve_vdwdupq_m_n_u32,
  BI__builtin_arm_mve_vdwdupq_m_n_u8,
  BI__builtin_arm_mve_vdwdupq_m_wb_u16,
  BI__builtin_arm_mve_vdwdupq_m_wb_u32,
  BI__builtin_arm_mve_vdwdupq_m_wb_u8,
  BI__builtin_arm_mve_vdwdupq_n_u16,
  BI__builtin_arm_mve_vdwdupq_n_u32,
  BI__builtin_arm_mve_vdwdupq_n_u8,
  BI__builtin_arm_mve_vdwdupq_wb_u16,
  BI__builtin_arm_mve_vdwdupq_wb_u32,
  BI__builtin_arm_mve_vdwdupq_wb_u8,
  BI__builtin_arm_mve_vdwdupq_x_n_u16,
  BI__builtin_arm_mve_vdwdupq_x_n_u32,
  BI__builtin_arm_mve_vdwdupq_x_n_u8,
  BI__builtin_arm_mve_vdwdupq_x_wb_u16,
  BI__builtin_arm_mve_vdwdupq_x_wb_u32,
  BI__builtin_arm_mve_vdwdupq_x_wb_u8,
  BI__builtin_arm_mve_veorq_f16,
  BI__builtin_arm_mve_veorq_f32,
  BI__builtin_arm_mve_veorq_m_f16,
  BI__builtin_arm_mve_veorq_m_f32,
  BI__builtin_arm_mve_veorq_m_s16,
  BI__builtin_arm_mve_veorq_m_s32,
  BI__builtin_arm_mve_veorq_m_s8,
  BI__builtin_arm_mve_veorq_m_u16,
  BI__builtin_arm_mve_veorq_m_u32,
  BI__builtin_arm_mve_veorq_m_u8,
  BI__builtin_arm_mve_veorq_s16,
  BI__builtin_arm_mve_veorq_s32,
  BI__builtin_arm_mve_veorq_s8,
  BI__builtin_arm_mve_veorq_u16,
  BI__builtin_arm_mve_veorq_u32,
  BI__builtin_arm_mve_veorq_u8,
  BI__builtin_arm_mve_veorq_x_f16,
  BI__builtin_arm_mve_veorq_x_f32,
  BI__builtin_arm_mve_veorq_x_s16,
  BI__builtin_arm_mve_veorq_x_s32,
  BI__builtin_arm_mve_veorq_x_s8,
  BI__builtin_arm_mve_veorq_x_u16,
  BI__builtin_arm_mve_veorq_x_u32,
  BI__builtin_arm_mve_veorq_x_u8,
  BI__builtin_arm_mve_vfmaq_f16,
  BI__builtin_arm_mve_vfmaq_f32,
  BI__builtin_arm_mve_vfmaq_m_f16,
  BI__builtin_arm_mve_vfmaq_m_f32,
  BI__builtin_arm_mve_vfmaq_m_n_f16,
  BI__builtin_arm_mve_vfmaq_m_n_f32,
  BI__builtin_arm_mve_vfmaq_n_f16,
  BI__builtin_arm_mve_vfmaq_n_f32,
  BI__builtin_arm_mve_vfmasq_m_n_f16,
  BI__builtin_arm_mve_vfmasq_m_n_f32,
  BI__builtin_arm_mve_vfmasq_n_f16,
  BI__builtin_arm_mve_vfmasq_n_f32,
  BI__builtin_arm_mve_vfmsq_f16,
  BI__builtin_arm_mve_vfmsq_f32,
  BI__builtin_arm_mve_vfmsq_m_f16,
  BI__builtin_arm_mve_vfmsq_m_f32,
  BI__builtin_arm_mve_vgetq_lane_f16,
  BI__builtin_arm_mve_vgetq_lane_f32,
  BI__builtin_arm_mve_vgetq_lane_s16,
  BI__builtin_arm_mve_vgetq_lane_s32,
  BI__builtin_arm_mve_vgetq_lane_s64,
  BI__builtin_arm_mve_vgetq_lane_s8,
  BI__builtin_arm_mve_vgetq_lane_u16,
  BI__builtin_arm_mve_vgetq_lane_u32,
  BI__builtin_arm_mve_vgetq_lane_u64,
  BI__builtin_arm_mve_vgetq_lane_u8,
  BI__builtin_arm_mve_vhaddq_m_n_s16,
  BI__builtin_arm_mve_vhaddq_m_n_s32,
  BI__builtin_arm_mve_vhaddq_m_n_s8,
  BI__builtin_arm_mve_vhaddq_m_n_u16,
  BI__builtin_arm_mve_vhaddq_m_n_u32,
  BI__builtin_arm_mve_vhaddq_m_n_u8,
  BI__builtin_arm_mve_vhaddq_m_s16,
  BI__builtin_arm_mve_vhaddq_m_s32,
  BI__builtin_arm_mve_vhaddq_m_s8,
  BI__builtin_arm_mve_vhaddq_m_u16,
  BI__builtin_arm_mve_vhaddq_m_u32,
  BI__builtin_arm_mve_vhaddq_m_u8,
  BI__builtin_arm_mve_vhaddq_n_s16,
  BI__builtin_arm_mve_vhaddq_n_s32,
  BI__builtin_arm_mve_vhaddq_n_s8,
  BI__builtin_arm_mve_vhaddq_n_u16,
  BI__builtin_arm_mve_vhaddq_n_u32,
  BI__builtin_arm_mve_vhaddq_n_u8,
  BI__builtin_arm_mve_vhaddq_s16,
  BI__builtin_arm_mve_vhaddq_s32,
  BI__builtin_arm_mve_vhaddq_s8,
  BI__builtin_arm_mve_vhaddq_u16,
  BI__builtin_arm_mve_vhaddq_u32,
  BI__builtin_arm_mve_vhaddq_u8,
  BI__builtin_arm_mve_vhaddq_x_n_s16,
  BI__builtin_arm_mve_vhaddq_x_n_s32,
  BI__builtin_arm_mve_vhaddq_x_n_s8,
  BI__builtin_arm_mve_vhaddq_x_n_u16,
  BI__builtin_arm_mve_vhaddq_x_n_u32,
  BI__builtin_arm_mve_vhaddq_x_n_u8,
  BI__builtin_arm_mve_vhaddq_x_s16,
  BI__builtin_arm_mve_vhaddq_x_s32,
  BI__builtin_arm_mve_vhaddq_x_s8,
  BI__builtin_arm_mve_vhaddq_x_u16,
  BI__builtin_arm_mve_vhaddq_x_u32,
  BI__builtin_arm_mve_vhaddq_x_u8,
  BI__builtin_arm_mve_vhcaddq_rot270_m_s16,
  BI__builtin_arm_mve_vhcaddq_rot270_m_s32,
  BI__builtin_arm_mve_vhcaddq_rot270_m_s8,
  BI__builtin_arm_mve_vhcaddq_rot270_s16,
  BI__builtin_arm_mve_vhcaddq_rot270_s32,
  BI__builtin_arm_mve_vhcaddq_rot270_s8,
  BI__builtin_arm_mve_vhcaddq_rot270_x_s16,
  BI__builtin_arm_mve_vhcaddq_rot270_x_s32,
  BI__builtin_arm_mve_vhcaddq_rot270_x_s8,
  BI__builtin_arm_mve_vhcaddq_rot90_m_s16,
  BI__builtin_arm_mve_vhcaddq_rot90_m_s32,
  BI__builtin_arm_mve_vhcaddq_rot90_m_s8,
  BI__builtin_arm_mve_vhcaddq_rot90_s16,
  BI__builtin_arm_mve_vhcaddq_rot90_s32,
  BI__builtin_arm_mve_vhcaddq_rot90_s8,
  BI__builtin_arm_mve_vhcaddq_rot90_x_s16,
  BI__builtin_arm_mve_vhcaddq_rot90_x_s32,
  BI__builtin_arm_mve_vhcaddq_rot90_x_s8,
  BI__builtin_arm_mve_vhsubq_m_n_s16,
  BI__builtin_arm_mve_vhsubq_m_n_s32,
  BI__builtin_arm_mve_vhsubq_m_n_s8,
  BI__builtin_arm_mve_vhsubq_m_n_u16,
  BI__builtin_arm_mve_vhsubq_m_n_u32,
  BI__builtin_arm_mve_vhsubq_m_n_u8,
  BI__builtin_arm_mve_vhsubq_m_s16,
  BI__builtin_arm_mve_vhsubq_m_s32,
  BI__builtin_arm_mve_vhsubq_m_s8,
  BI__builtin_arm_mve_vhsubq_m_u16,
  BI__builtin_arm_mve_vhsubq_m_u32,
  BI__builtin_arm_mve_vhsubq_m_u8,
  BI__builtin_arm_mve_vhsubq_n_s16,
  BI__builtin_arm_mve_vhsubq_n_s32,
  BI__builtin_arm_mve_vhsubq_n_s8,
  BI__builtin_arm_mve_vhsubq_n_u16,
  BI__builtin_arm_mve_vhsubq_n_u32,
  BI__builtin_arm_mve_vhsubq_n_u8,
  BI__builtin_arm_mve_vhsubq_s16,
  BI__builtin_arm_mve_vhsubq_s32,
  BI__builtin_arm_mve_vhsubq_s8,
  BI__builtin_arm_mve_vhsubq_u16,
  BI__builtin_arm_mve_vhsubq_u32,
  BI__builtin_arm_mve_vhsubq_u8,
  BI__builtin_arm_mve_vhsubq_x_n_s16,
  BI__builtin_arm_mve_vhsubq_x_n_s32,
  BI__builtin_arm_mve_vhsubq_x_n_s8,
  BI__builtin_arm_mve_vhsubq_x_n_u16,
  BI__builtin_arm_mve_vhsubq_x_n_u32,
  BI__builtin_arm_mve_vhsubq_x_n_u8,
  BI__builtin_arm_mve_vhsubq_x_s16,
  BI__builtin_arm_mve_vhsubq_x_s32,
  BI__builtin_arm_mve_vhsubq_x_s8,
  BI__builtin_arm_mve_vhsubq_x_u16,
  BI__builtin_arm_mve_vhsubq_x_u32,
  BI__builtin_arm_mve_vhsubq_x_u8,
  BI__builtin_arm_mve_vidupq_m_n_u16,
  BI__builtin_arm_mve_vidupq_m_n_u32,
  BI__builtin_arm_mve_vidupq_m_n_u8,
  BI__builtin_arm_mve_vidupq_m_wb_u16,
  BI__builtin_arm_mve_vidupq_m_wb_u32,
  BI__builtin_arm_mve_vidupq_m_wb_u8,
  BI__builtin_arm_mve_vidupq_n_u16,
  BI__builtin_arm_mve_vidupq_n_u32,
  BI__builtin_arm_mve_vidupq_n_u8,
  BI__builtin_arm_mve_vidupq_wb_u16,
  BI__builtin_arm_mve_vidupq_wb_u32,
  BI__builtin_arm_mve_vidupq_wb_u8,
  BI__builtin_arm_mve_vidupq_x_n_u16,
  BI__builtin_arm_mve_vidupq_x_n_u32,
  BI__builtin_arm_mve_vidupq_x_n_u8,
  BI__builtin_arm_mve_vidupq_x_wb_u16,
  BI__builtin_arm_mve_vidupq_x_wb_u32,
  BI__builtin_arm_mve_vidupq_x_wb_u8,
  BI__builtin_arm_mve_viwdupq_m_n_u16,
  BI__builtin_arm_mve_viwdupq_m_n_u32,
  BI__builtin_arm_mve_viwdupq_m_n_u8,
  BI__builtin_arm_mve_viwdupq_m_wb_u16,
  BI__builtin_arm_mve_viwdupq_m_wb_u32,
  BI__builtin_arm_mve_viwdupq_m_wb_u8,
  BI__builtin_arm_mve_viwdupq_n_u16,
  BI__builtin_arm_mve_viwdupq_n_u32,
  BI__builtin_arm_mve_viwdupq_n_u8,
  BI__builtin_arm_mve_viwdupq_wb_u16,
  BI__builtin_arm_mve_viwdupq_wb_u32,
  BI__builtin_arm_mve_viwdupq_wb_u8,
  BI__builtin_arm_mve_viwdupq_x_n_u16,
  BI__builtin_arm_mve_viwdupq_x_n_u32,
  BI__builtin_arm_mve_viwdupq_x_n_u8,
  BI__builtin_arm_mve_viwdupq_x_wb_u16,
  BI__builtin_arm_mve_viwdupq_x_wb_u32,
  BI__builtin_arm_mve_viwdupq_x_wb_u8,
  BI__builtin_arm_mve_vld1q_f16,
  BI__builtin_arm_mve_vld1q_f32,
  BI__builtin_arm_mve_vld1q_s16,
  BI__builtin_arm_mve_vld1q_s32,
  BI__builtin_arm_mve_vld1q_s8,
  BI__builtin_arm_mve_vld1q_u16,
  BI__builtin_arm_mve_vld1q_u32,
  BI__builtin_arm_mve_vld1q_u8,
  BI__builtin_arm_mve_vld1q_z_f16,
  BI__builtin_arm_mve_vld1q_z_f32,
  BI__builtin_arm_mve_vld1q_z_s16,
  BI__builtin_arm_mve_vld1q_z_s32,
  BI__builtin_arm_mve_vld1q_z_s8,
  BI__builtin_arm_mve_vld1q_z_u16,
  BI__builtin_arm_mve_vld1q_z_u32,
  BI__builtin_arm_mve_vld1q_z_u8,
  BI__builtin_arm_mve_vld2q_f16,
  BI__builtin_arm_mve_vld2q_f32,
  BI__builtin_arm_mve_vld2q_s16,
  BI__builtin_arm_mve_vld2q_s32,
  BI__builtin_arm_mve_vld2q_s8,
  BI__builtin_arm_mve_vld2q_u16,
  BI__builtin_arm_mve_vld2q_u32,
  BI__builtin_arm_mve_vld2q_u8,
  BI__builtin_arm_mve_vld4q_f16,
  BI__builtin_arm_mve_vld4q_f32,
  BI__builtin_arm_mve_vld4q_s16,
  BI__builtin_arm_mve_vld4q_s32,
  BI__builtin_arm_mve_vld4q_s8,
  BI__builtin_arm_mve_vld4q_u16,
  BI__builtin_arm_mve_vld4q_u32,
  BI__builtin_arm_mve_vld4q_u8,
  BI__builtin_arm_mve_vldrbq_gather_offset_s16,
  BI__builtin_arm_mve_vldrbq_gather_offset_s32,
  BI__builtin_arm_mve_vldrbq_gather_offset_s8,
  BI__builtin_arm_mve_vldrbq_gather_offset_u16,
  BI__builtin_arm_mve_vldrbq_gather_offset_u32,
  BI__builtin_arm_mve_vldrbq_gather_offset_u8,
  BI__builtin_arm_mve_vldrbq_gather_offset_z_s16,
  BI__builtin_arm_mve_vldrbq_gather_offset_z_s32,
  BI__builtin_arm_mve_vldrbq_gather_offset_z_s8,
  BI__builtin_arm_mve_vldrbq_gather_offset_z_u16,
  BI__builtin_arm_mve_vldrbq_gather_offset_z_u32,
  BI__builtin_arm_mve_vldrbq_gather_offset_z_u8,
  BI__builtin_arm_mve_vldrbq_s16,
  BI__builtin_arm_mve_vldrbq_s32,
  BI__builtin_arm_mve_vldrbq_s8,
  BI__builtin_arm_mve_vldrbq_u16,
  BI__builtin_arm_mve_vldrbq_u32,
  BI__builtin_arm_mve_vldrbq_u8,
  BI__builtin_arm_mve_vldrbq_z_s16,
  BI__builtin_arm_mve_vldrbq_z_s32,
  BI__builtin_arm_mve_vldrbq_z_s8,
  BI__builtin_arm_mve_vldrbq_z_u16,
  BI__builtin_arm_mve_vldrbq_z_u32,
  BI__builtin_arm_mve_vldrbq_z_u8,
  BI__builtin_arm_mve_vldrdq_gather_base_s64,
  BI__builtin_arm_mve_vldrdq_gather_base_u64,
  BI__builtin_arm_mve_vldrdq_gather_base_wb_s64,
  BI__builtin_arm_mve_vldrdq_gather_base_wb_u64,
  BI__builtin_arm_mve_vldrdq_gather_base_wb_z_s64,
  BI__builtin_arm_mve_vldrdq_gather_base_wb_z_u64,
  BI__builtin_arm_mve_vldrdq_gather_base_z_s64,
  BI__builtin_arm_mve_vldrdq_gather_base_z_u64,
  BI__builtin_arm_mve_vldrdq_gather_offset_s64,
  BI__builtin_arm_mve_vldrdq_gather_offset_u64,
  BI__builtin_arm_mve_vldrdq_gather_offset_z_s64,
  BI__builtin_arm_mve_vldrdq_gather_offset_z_u64,
  BI__builtin_arm_mve_vldrdq_gather_shifted_offset_s64,
  BI__builtin_arm_mve_vldrdq_gather_shifted_offset_u64,
  BI__builtin_arm_mve_vldrdq_gather_shifted_offset_z_s64,
  BI__builtin_arm_mve_vldrdq_gather_shifted_offset_z_u64,
  BI__builtin_arm_mve_vldrhq_f16,
  BI__builtin_arm_mve_vldrhq_gather_offset_f16,
  BI__builtin_arm_mve_vldrhq_gather_offset_s16,
  BI__builtin_arm_mve_vldrhq_gather_offset_s32,
  BI__builtin_arm_mve_vldrhq_gather_offset_u16,
  BI__builtin_arm_mve_vldrhq_gather_offset_u32,
  BI__builtin_arm_mve_vldrhq_gather_offset_z_f16,
  BI__builtin_arm_mve_vldrhq_gather_offset_z_s16,
  BI__builtin_arm_mve_vldrhq_gather_offset_z_s32,
  BI__builtin_arm_mve_vldrhq_gather_offset_z_u16,
  BI__builtin_arm_mve_vldrhq_gather_offset_z_u32,
  BI__builtin_arm_mve_vldrhq_gather_shifted_offset_f16,
  BI__builtin_arm_mve_vldrhq_gather_shifted_offset_s16,
  BI__builtin_arm_mve_vldrhq_gather_shifted_offset_s32,
  BI__builtin_arm_mve_vldrhq_gather_shifted_offset_u16,
  BI__builtin_arm_mve_vldrhq_gather_shifted_offset_u32,
  BI__builtin_arm_mve_vldrhq_gather_shifted_offset_z_f16,
  BI__builtin_arm_mve_vldrhq_gather_shifted_offset_z_s16,
  BI__builtin_arm_mve_vldrhq_gather_shifted_offset_z_s32,
  BI__builtin_arm_mve_vldrhq_gather_shifted_offset_z_u16,
  BI__builtin_arm_mve_vldrhq_gather_shifted_offset_z_u32,
  BI__builtin_arm_mve_vldrhq_s16,
  BI__builtin_arm_mve_vldrhq_s32,
  BI__builtin_arm_mve_vldrhq_u16,
  BI__builtin_arm_mve_vldrhq_u32,
  BI__builtin_arm_mve_vldrhq_z_f16,
  BI__builtin_arm_mve_vldrhq_z_s16,
  BI__builtin_arm_mve_vldrhq_z_s32,
  BI__builtin_arm_mve_vldrhq_z_u16,
  BI__builtin_arm_mve_vldrhq_z_u32,
  BI__builtin_arm_mve_vldrwq_f32,
  BI__builtin_arm_mve_vldrwq_gather_base_f32,
  BI__builtin_arm_mve_vldrwq_gather_base_s32,
  BI__builtin_arm_mve_vldrwq_gather_base_u32,
  BI__builtin_arm_mve_vldrwq_gather_base_wb_f32,
  BI__builtin_arm_mve_vldrwq_gather_base_wb_s32,
  BI__builtin_arm_mve_vldrwq_gather_base_wb_u32,
  BI__builtin_arm_mve_vldrwq_gather_base_wb_z_f32,
  BI__builtin_arm_mve_vldrwq_gather_base_wb_z_s32,
  BI__builtin_arm_mve_vldrwq_gather_base_wb_z_u32,
  BI__builtin_arm_mve_vldrwq_gather_base_z_f32,
  BI__builtin_arm_mve_vldrwq_gather_base_z_s32,
  BI__builtin_arm_mve_vldrwq_gather_base_z_u32,
  BI__builtin_arm_mve_vldrwq_gather_offset_f32,
  BI__builtin_arm_mve_vldrwq_gather_offset_s32,
  BI__builtin_arm_mve_vldrwq_gather_offset_u32,
  BI__builtin_arm_mve_vldrwq_gather_offset_z_f32,
  BI__builtin_arm_mve_vldrwq_gather_offset_z_s32,
  BI__builtin_arm_mve_vldrwq_gather_offset_z_u32,
  BI__builtin_arm_mve_vldrwq_gather_shifted_offset_f32,
  BI__builtin_arm_mve_vldrwq_gather_shifted_offset_s32,
  BI__builtin_arm_mve_vldrwq_gather_shifted_offset_u32,
  BI__builtin_arm_mve_vldrwq_gather_shifted_offset_z_f32,
  BI__builtin_arm_mve_vldrwq_gather_shifted_offset_z_s32,
  BI__builtin_arm_mve_vldrwq_gather_shifted_offset_z_u32,
  BI__builtin_arm_mve_vldrwq_s32,
  BI__builtin_arm_mve_vldrwq_u32,
  BI__builtin_arm_mve_vldrwq_z_f32,
  BI__builtin_arm_mve_vldrwq_z_s32,
  BI__builtin_arm_mve_vldrwq_z_u32,
  BI__builtin_arm_mve_vmaxaq_m_s16,
  BI__builtin_arm_mve_vmaxaq_m_s32,
  BI__builtin_arm_mve_vmaxaq_m_s8,
  BI__builtin_arm_mve_vmaxaq_s16,
  BI__builtin_arm_mve_vmaxaq_s32,
  BI__builtin_arm_mve_vmaxaq_s8,
  BI__builtin_arm_mve_vmaxavq_p_s16,
  BI__builtin_arm_mve_vmaxavq_p_s32,
  BI__builtin_arm_mve_vmaxavq_p_s8,
  BI__builtin_arm_mve_vmaxavq_s16,
  BI__builtin_arm_mve_vmaxavq_s32,
  BI__builtin_arm_mve_vmaxavq_s8,
  BI__builtin_arm_mve_vmaxnmaq_f16,
  BI__builtin_arm_mve_vmaxnmaq_f32,
  BI__builtin_arm_mve_vmaxnmaq_m_f16,
  BI__builtin_arm_mve_vmaxnmaq_m_f32,
  BI__builtin_arm_mve_vmaxnmavq_f16,
  BI__builtin_arm_mve_vmaxnmavq_f32,
  BI__builtin_arm_mve_vmaxnmavq_p_f16,
  BI__builtin_arm_mve_vmaxnmavq_p_f32,
  BI__builtin_arm_mve_vmaxnmq_f16,
  BI__builtin_arm_mve_vmaxnmq_f32,
  BI__builtin_arm_mve_vmaxnmq_m_f16,
  BI__builtin_arm_mve_vmaxnmq_m_f32,
  BI__builtin_arm_mve_vmaxnmq_x_f16,
  BI__builtin_arm_mve_vmaxnmq_x_f32,
  BI__builtin_arm_mve_vmaxnmvq_f16,
  BI__builtin_arm_mve_vmaxnmvq_f32,
  BI__builtin_arm_mve_vmaxnmvq_p_f16,
  BI__builtin_arm_mve_vmaxnmvq_p_f32,
  BI__builtin_arm_mve_vmaxq_m_s16,
  BI__builtin_arm_mve_vmaxq_m_s32,
  BI__builtin_arm_mve_vmaxq_m_s8,
  BI__builtin_arm_mve_vmaxq_m_u16,
  BI__builtin_arm_mve_vmaxq_m_u32,
  BI__builtin_arm_mve_vmaxq_m_u8,
  BI__builtin_arm_mve_vmaxq_s16,
  BI__builtin_arm_mve_vmaxq_s32,
  BI__builtin_arm_mve_vmaxq_s8,
  BI__builtin_arm_mve_vmaxq_u16,
  BI__builtin_arm_mve_vmaxq_u32,
  BI__builtin_arm_mve_vmaxq_u8,
  BI__builtin_arm_mve_vmaxq_x_s16,
  BI__builtin_arm_mve_vmaxq_x_s32,
  BI__builtin_arm_mve_vmaxq_x_s8,
  BI__builtin_arm_mve_vmaxq_x_u16,
  BI__builtin_arm_mve_vmaxq_x_u32,
  BI__builtin_arm_mve_vmaxq_x_u8,
  BI__builtin_arm_mve_vmaxvq_p_s16,
  BI__builtin_arm_mve_vmaxvq_p_s32,
  BI__builtin_arm_mve_vmaxvq_p_s8,
  BI__builtin_arm_mve_vmaxvq_p_u16,
  BI__builtin_arm_mve_vmaxvq_p_u32,
  BI__builtin_arm_mve_vmaxvq_p_u8,
  BI__builtin_arm_mve_vmaxvq_s16,
  BI__builtin_arm_mve_vmaxvq_s32,
  BI__builtin_arm_mve_vmaxvq_s8,
  BI__builtin_arm_mve_vmaxvq_u16,
  BI__builtin_arm_mve_vmaxvq_u32,
  BI__builtin_arm_mve_vmaxvq_u8,
  BI__builtin_arm_mve_vminaq_m_s16,
  BI__builtin_arm_mve_vminaq_m_s32,
  BI__builtin_arm_mve_vminaq_m_s8,
  BI__builtin_arm_mve_vminaq_s16,
  BI__builtin_arm_mve_vminaq_s32,
  BI__builtin_arm_mve_vminaq_s8,
  BI__builtin_arm_mve_vminavq_p_s16,
  BI__builtin_arm_mve_vminavq_p_s32,
  BI__builtin_arm_mve_vminavq_p_s8,
  BI__builtin_arm_mve_vminavq_s16,
  BI__builtin_arm_mve_vminavq_s32,
  BI__builtin_arm_mve_vminavq_s8,
  BI__builtin_arm_mve_vminnmaq_f16,
  BI__builtin_arm_mve_vminnmaq_f32,
  BI__builtin_arm_mve_vminnmaq_m_f16,
  BI__builtin_arm_mve_vminnmaq_m_f32,
  BI__builtin_arm_mve_vminnmavq_f16,
  BI__builtin_arm_mve_vminnmavq_f32,
  BI__builtin_arm_mve_vminnmavq_p_f16,
  BI__builtin_arm_mve_vminnmavq_p_f32,
  BI__builtin_arm_mve_vminnmq_f16,
  BI__builtin_arm_mve_vminnmq_f32,
  BI__builtin_arm_mve_vminnmq_m_f16,
  BI__builtin_arm_mve_vminnmq_m_f32,
  BI__builtin_arm_mve_vminnmq_x_f16,
  BI__builtin_arm_mve_vminnmq_x_f32,
  BI__builtin_arm_mve_vminnmvq_f16,
  BI__builtin_arm_mve_vminnmvq_f32,
  BI__builtin_arm_mve_vminnmvq_p_f16,
  BI__builtin_arm_mve_vminnmvq_p_f32,
  BI__builtin_arm_mve_vminq_m_s16,
  BI__builtin_arm_mve_vminq_m_s32,
  BI__builtin_arm_mve_vminq_m_s8,
  BI__builtin_arm_mve_vminq_m_u16,
  BI__builtin_arm_mve_vminq_m_u32,
  BI__builtin_arm_mve_vminq_m_u8,
  BI__builtin_arm_mve_vminq_s16,
  BI__builtin_arm_mve_vminq_s32,
  BI__builtin_arm_mve_vminq_s8,
  BI__builtin_arm_mve_vminq_u16,
  BI__builtin_arm_mve_vminq_u32,
  BI__builtin_arm_mve_vminq_u8,
  BI__builtin_arm_mve_vminq_x_s16,
  BI__builtin_arm_mve_vminq_x_s32,
  BI__builtin_arm_mve_vminq_x_s8,
  BI__builtin_arm_mve_vminq_x_u16,
  BI__builtin_arm_mve_vminq_x_u32,
  BI__builtin_arm_mve_vminq_x_u8,
  BI__builtin_arm_mve_vminvq_p_s16,
  BI__builtin_arm_mve_vminvq_p_s32,
  BI__builtin_arm_mve_vminvq_p_s8,
  BI__builtin_arm_mve_vminvq_p_u16,
  BI__builtin_arm_mve_vminvq_p_u32,
  BI__builtin_arm_mve_vminvq_p_u8,
  BI__builtin_arm_mve_vminvq_s16,
  BI__builtin_arm_mve_vminvq_s32,
  BI__builtin_arm_mve_vminvq_s8,
  BI__builtin_arm_mve_vminvq_u16,
  BI__builtin_arm_mve_vminvq_u32,
  BI__builtin_arm_mve_vminvq_u8,
  BI__builtin_arm_mve_vmladavaq_p_s16,
  BI__builtin_arm_mve_vmladavaq_p_s32,
  BI__builtin_arm_mve_vmladavaq_p_s8,
  BI__builtin_arm_mve_vmladavaq_p_u16,
  BI__builtin_arm_mve_vmladavaq_p_u32,
  BI__builtin_arm_mve_vmladavaq_p_u8,
  BI__builtin_arm_mve_vmladavaq_s16,
  BI__builtin_arm_mve_vmladavaq_s32,
  BI__builtin_arm_mve_vmladavaq_s8,
  BI__builtin_arm_mve_vmladavaq_u16,
  BI__builtin_arm_mve_vmladavaq_u32,
  BI__builtin_arm_mve_vmladavaq_u8,
  BI__builtin_arm_mve_vmladavaxq_p_s16,
  BI__builtin_arm_mve_vmladavaxq_p_s32,
  BI__builtin_arm_mve_vmladavaxq_p_s8,
  BI__builtin_arm_mve_vmladavaxq_s16,
  BI__builtin_arm_mve_vmladavaxq_s32,
  BI__builtin_arm_mve_vmladavaxq_s8,
  BI__builtin_arm_mve_vmladavq_p_s16,
  BI__builtin_arm_mve_vmladavq_p_s32,
  BI__builtin_arm_mve_vmladavq_p_s8,
  BI__builtin_arm_mve_vmladavq_p_u16,
  BI__builtin_arm_mve_vmladavq_p_u32,
  BI__builtin_arm_mve_vmladavq_p_u8,
  BI__builtin_arm_mve_vmladavq_s16,
  BI__builtin_arm_mve_vmladavq_s32,
  BI__builtin_arm_mve_vmladavq_s8,
  BI__builtin_arm_mve_vmladavq_u16,
  BI__builtin_arm_mve_vmladavq_u32,
  BI__builtin_arm_mve_vmladavq_u8,
  BI__builtin_arm_mve_vmladavxq_p_s16,
  BI__builtin_arm_mve_vmladavxq_p_s32,
  BI__builtin_arm_mve_vmladavxq_p_s8,
  BI__builtin_arm_mve_vmladavxq_s16,
  BI__builtin_arm_mve_vmladavxq_s32,
  BI__builtin_arm_mve_vmladavxq_s8,
  BI__builtin_arm_mve_vmlaldavaq_p_s16,
  BI__builtin_arm_mve_vmlaldavaq_p_s32,
  BI__builtin_arm_mve_vmlaldavaq_p_u16,
  BI__builtin_arm_mve_vmlaldavaq_p_u32,
  BI__builtin_arm_mve_vmlaldavaq_s16,
  BI__builtin_arm_mve_vmlaldavaq_s32,
  BI__builtin_arm_mve_vmlaldavaq_u16,
  BI__builtin_arm_mve_vmlaldavaq_u32,
  BI__builtin_arm_mve_vmlaldavaxq_p_s16,
  BI__builtin_arm_mve_vmlaldavaxq_p_s32,
  BI__builtin_arm_mve_vmlaldavaxq_s16,
  BI__builtin_arm_mve_vmlaldavaxq_s32,
  BI__builtin_arm_mve_vmlaldavq_p_s16,
  BI__builtin_arm_mve_vmlaldavq_p_s32,
  BI__builtin_arm_mve_vmlaldavq_p_u16,
  BI__builtin_arm_mve_vmlaldavq_p_u32,
  BI__builtin_arm_mve_vmlaldavq_s16,
  BI__builtin_arm_mve_vmlaldavq_s32,
  BI__builtin_arm_mve_vmlaldavq_u16,
  BI__builtin_arm_mve_vmlaldavq_u32,
  BI__builtin_arm_mve_vmlaldavxq_p_s16,
  BI__builtin_arm_mve_vmlaldavxq_p_s32,
  BI__builtin_arm_mve_vmlaldavxq_s16,
  BI__builtin_arm_mve_vmlaldavxq_s32,
  BI__builtin_arm_mve_vmlaq_m_n_s16,
  BI__builtin_arm_mve_vmlaq_m_n_s32,
  BI__builtin_arm_mve_vmlaq_m_n_s8,
  BI__builtin_arm_mve_vmlaq_m_n_u16,
  BI__builtin_arm_mve_vmlaq_m_n_u32,
  BI__builtin_arm_mve_vmlaq_m_n_u8,
  BI__builtin_arm_mve_vmlaq_n_s16,
  BI__builtin_arm_mve_vmlaq_n_s32,
  BI__builtin_arm_mve_vmlaq_n_s8,
  BI__builtin_arm_mve_vmlaq_n_u16,
  BI__builtin_arm_mve_vmlaq_n_u32,
  BI__builtin_arm_mve_vmlaq_n_u8,
  BI__builtin_arm_mve_vmlasq_m_n_s16,
  BI__builtin_arm_mve_vmlasq_m_n_s32,
  BI__builtin_arm_mve_vmlasq_m_n_s8,
  BI__builtin_arm_mve_vmlasq_m_n_u16,
  BI__builtin_arm_mve_vmlasq_m_n_u32,
  BI__builtin_arm_mve_vmlasq_m_n_u8,
  BI__builtin_arm_mve_vmlasq_n_s16,
  BI__builtin_arm_mve_vmlasq_n_s32,
  BI__builtin_arm_mve_vmlasq_n_s8,
  BI__builtin_arm_mve_vmlasq_n_u16,
  BI__builtin_arm_mve_vmlasq_n_u32,
  BI__builtin_arm_mve_vmlasq_n_u8,
  BI__builtin_arm_mve_vmlsdavaq_p_s16,
  BI__builtin_arm_mve_vmlsdavaq_p_s32,
  BI__builtin_arm_mve_vmlsdavaq_p_s8,
  BI__builtin_arm_mve_vmlsdavaq_s16,
  BI__builtin_arm_mve_vmlsdavaq_s32,
  BI__builtin_arm_mve_vmlsdavaq_s8,
  BI__builtin_arm_mve_vmlsdavaxq_p_s16,
  BI__builtin_arm_mve_vmlsdavaxq_p_s32,
  BI__builtin_arm_mve_vmlsdavaxq_p_s8,
  BI__builtin_arm_mve_vmlsdavaxq_s16,
  BI__builtin_arm_mve_vmlsdavaxq_s32,
  BI__builtin_arm_mve_vmlsdavaxq_s8,
  BI__builtin_arm_mve_vmlsdavq_p_s16,
  BI__builtin_arm_mve_vmlsdavq_p_s32,
  BI__builtin_arm_mve_vmlsdavq_p_s8,
  BI__builtin_arm_mve_vmlsdavq_s16,
  BI__builtin_arm_mve_vmlsdavq_s32,
  BI__builtin_arm_mve_vmlsdavq_s8,
  BI__builtin_arm_mve_vmlsdavxq_p_s16,
  BI__builtin_arm_mve_vmlsdavxq_p_s32,
  BI__builtin_arm_mve_vmlsdavxq_p_s8,
  BI__builtin_arm_mve_vmlsdavxq_s16,
  BI__builtin_arm_mve_vmlsdavxq_s32,
  BI__builtin_arm_mve_vmlsdavxq_s8,
  BI__builtin_arm_mve_vmlsldavaq_p_s16,
  BI__builtin_arm_mve_vmlsldavaq_p_s32,
  BI__builtin_arm_mve_vmlsldavaq_s16,
  BI__builtin_arm_mve_vmlsldavaq_s32,
  BI__builtin_arm_mve_vmlsldavaxq_p_s16,
  BI__builtin_arm_mve_vmlsldavaxq_p_s32,
  BI__builtin_arm_mve_vmlsldavaxq_s16,
  BI__builtin_arm_mve_vmlsldavaxq_s32,
  BI__builtin_arm_mve_vmlsldavq_p_s16,
  BI__builtin_arm_mve_vmlsldavq_p_s32,
  BI__builtin_arm_mve_vmlsldavq_s16,
  BI__builtin_arm_mve_vmlsldavq_s32,
  BI__builtin_arm_mve_vmlsldavxq_p_s16,
  BI__builtin_arm_mve_vmlsldavxq_p_s32,
  BI__builtin_arm_mve_vmlsldavxq_s16,
  BI__builtin_arm_mve_vmlsldavxq_s32,
  BI__builtin_arm_mve_vmovlbq_m_s16,
  BI__builtin_arm_mve_vmovlbq_m_s8,
  BI__builtin_arm_mve_vmovlbq_m_u16,
  BI__builtin_arm_mve_vmovlbq_m_u8,
  BI__builtin_arm_mve_vmovlbq_s16,
  BI__builtin_arm_mve_vmovlbq_s8,
  BI__builtin_arm_mve_vmovlbq_u16,
  BI__builtin_arm_mve_vmovlbq_u8,
  BI__builtin_arm_mve_vmovlbq_x_s16,
  BI__builtin_arm_mve_vmovlbq_x_s8,
  BI__builtin_arm_mve_vmovlbq_x_u16,
  BI__builtin_arm_mve_vmovlbq_x_u8,
  BI__builtin_arm_mve_vmovltq_m_s16,
  BI__builtin_arm_mve_vmovltq_m_s8,
  BI__builtin_arm_mve_vmovltq_m_u16,
  BI__builtin_arm_mve_vmovltq_m_u8,
  BI__builtin_arm_mve_vmovltq_s16,
  BI__builtin_arm_mve_vmovltq_s8,
  BI__builtin_arm_mve_vmovltq_u16,
  BI__builtin_arm_mve_vmovltq_u8,
  BI__builtin_arm_mve_vmovltq_x_s16,
  BI__builtin_arm_mve_vmovltq_x_s8,
  BI__builtin_arm_mve_vmovltq_x_u16,
  BI__builtin_arm_mve_vmovltq_x_u8,
  BI__builtin_arm_mve_vmovnbq_m_s16,
  BI__builtin_arm_mve_vmovnbq_m_s32,
  BI__builtin_arm_mve_vmovnbq_m_u16,
  BI__builtin_arm_mve_vmovnbq_m_u32,
  BI__builtin_arm_mve_vmovnbq_s16,
  BI__builtin_arm_mve_vmovnbq_s32,
  BI__builtin_arm_mve_vmovnbq_u16,
  BI__builtin_arm_mve_vmovnbq_u32,
  BI__builtin_arm_mve_vmovntq_m_s16,
  BI__builtin_arm_mve_vmovntq_m_s32,
  BI__builtin_arm_mve_vmovntq_m_u16,
  BI__builtin_arm_mve_vmovntq_m_u32,
  BI__builtin_arm_mve_vmovntq_s16,
  BI__builtin_arm_mve_vmovntq_s32,
  BI__builtin_arm_mve_vmovntq_u16,
  BI__builtin_arm_mve_vmovntq_u32,
  BI__builtin_arm_mve_vmulhq_m_s16,
  BI__builtin_arm_mve_vmulhq_m_s32,
  BI__builtin_arm_mve_vmulhq_m_s8,
  BI__builtin_arm_mve_vmulhq_m_u16,
  BI__builtin_arm_mve_vmulhq_m_u32,
  BI__builtin_arm_mve_vmulhq_m_u8,
  BI__builtin_arm_mve_vmulhq_s16,
  BI__builtin_arm_mve_vmulhq_s32,
  BI__builtin_arm_mve_vmulhq_s8,
  BI__builtin_arm_mve_vmulhq_u16,
  BI__builtin_arm_mve_vmulhq_u32,
  BI__builtin_arm_mve_vmulhq_u8,
  BI__builtin_arm_mve_vmulhq_x_s16,
  BI__builtin_arm_mve_vmulhq_x_s32,
  BI__builtin_arm_mve_vmulhq_x_s8,
  BI__builtin_arm_mve_vmulhq_x_u16,
  BI__builtin_arm_mve_vmulhq_x_u32,
  BI__builtin_arm_mve_vmulhq_x_u8,
  BI__builtin_arm_mve_vmullbq_int_m_s16,
  BI__builtin_arm_mve_vmullbq_int_m_s32,
  BI__builtin_arm_mve_vmullbq_int_m_s8,
  BI__builtin_arm_mve_vmullbq_int_m_u16,
  BI__builtin_arm_mve_vmullbq_int_m_u32,
  BI__builtin_arm_mve_vmullbq_int_m_u8,
  BI__builtin_arm_mve_vmullbq_int_s16,
  BI__builtin_arm_mve_vmullbq_int_s32,
  BI__builtin_arm_mve_vmullbq_int_s8,
  BI__builtin_arm_mve_vmullbq_int_u16,
  BI__builtin_arm_mve_vmullbq_int_u32,
  BI__builtin_arm_mve_vmullbq_int_u8,
  BI__builtin_arm_mve_vmullbq_int_x_s16,
  BI__builtin_arm_mve_vmullbq_int_x_s32,
  BI__builtin_arm_mve_vmullbq_int_x_s8,
  BI__builtin_arm_mve_vmullbq_int_x_u16,
  BI__builtin_arm_mve_vmullbq_int_x_u32,
  BI__builtin_arm_mve_vmullbq_int_x_u8,
  BI__builtin_arm_mve_vmullbq_poly_m_p16,
  BI__builtin_arm_mve_vmullbq_poly_m_p8,
  BI__builtin_arm_mve_vmullbq_poly_p16,
  BI__builtin_arm_mve_vmullbq_poly_p8,
  BI__builtin_arm_mve_vmullbq_poly_x_p16,
  BI__builtin_arm_mve_vmullbq_poly_x_p8,
  BI__builtin_arm_mve_vmulltq_int_m_s16,
  BI__builtin_arm_mve_vmulltq_int_m_s32,
  BI__builtin_arm_mve_vmulltq_int_m_s8,
  BI__builtin_arm_mve_vmulltq_int_m_u16,
  BI__builtin_arm_mve_vmulltq_int_m_u32,
  BI__builtin_arm_mve_vmulltq_int_m_u8,
  BI__builtin_arm_mve_vmulltq_int_s16,
  BI__builtin_arm_mve_vmulltq_int_s32,
  BI__builtin_arm_mve_vmulltq_int_s8,
  BI__builtin_arm_mve_vmulltq_int_u16,
  BI__builtin_arm_mve_vmulltq_int_u32,
  BI__builtin_arm_mve_vmulltq_int_u8,
  BI__builtin_arm_mve_vmulltq_int_x_s16,
  BI__builtin_arm_mve_vmulltq_int_x_s32,
  BI__builtin_arm_mve_vmulltq_int_x_s8,
  BI__builtin_arm_mve_vmulltq_int_x_u16,
  BI__builtin_arm_mve_vmulltq_int_x_u32,
  BI__builtin_arm_mve_vmulltq_int_x_u8,
  BI__builtin_arm_mve_vmulltq_poly_m_p16,
  BI__builtin_arm_mve_vmulltq_poly_m_p8,
  BI__builtin_arm_mve_vmulltq_poly_p16,
  BI__builtin_arm_mve_vmulltq_poly_p8,
  BI__builtin_arm_mve_vmulltq_poly_x_p16,
  BI__builtin_arm_mve_vmulltq_poly_x_p8,
  BI__builtin_arm_mve_vmulq_f16,
  BI__builtin_arm_mve_vmulq_f32,
  BI__builtin_arm_mve_vmulq_m_f16,
  BI__builtin_arm_mve_vmulq_m_f32,
  BI__builtin_arm_mve_vmulq_m_n_f16,
  BI__builtin_arm_mve_vmulq_m_n_f32,
  BI__builtin_arm_mve_vmulq_m_n_s16,
  BI__builtin_arm_mve_vmulq_m_n_s32,
  BI__builtin_arm_mve_vmulq_m_n_s8,
  BI__builtin_arm_mve_vmulq_m_n_u16,
  BI__builtin_arm_mve_vmulq_m_n_u32,
  BI__builtin_arm_mve_vmulq_m_n_u8,
  BI__builtin_arm_mve_vmulq_m_s16,
  BI__builtin_arm_mve_vmulq_m_s32,
  BI__builtin_arm_mve_vmulq_m_s8,
  BI__builtin_arm_mve_vmulq_m_u16,
  BI__builtin_arm_mve_vmulq_m_u32,
  BI__builtin_arm_mve_vmulq_m_u8,
  BI__builtin_arm_mve_vmulq_n_f16,
  BI__builtin_arm_mve_vmulq_n_f32,
  BI__builtin_arm_mve_vmulq_n_s16,
  BI__builtin_arm_mve_vmulq_n_s32,
  BI__builtin_arm_mve_vmulq_n_s8,
  BI__builtin_arm_mve_vmulq_n_u16,
  BI__builtin_arm_mve_vmulq_n_u32,
  BI__builtin_arm_mve_vmulq_n_u8,
  BI__builtin_arm_mve_vmulq_s16,
  BI__builtin_arm_mve_vmulq_s32,
  BI__builtin_arm_mve_vmulq_s8,
  BI__builtin_arm_mve_vmulq_u16,
  BI__builtin_arm_mve_vmulq_u32,
  BI__builtin_arm_mve_vmulq_u8,
  BI__builtin_arm_mve_vmulq_x_f16,
  BI__builtin_arm_mve_vmulq_x_f32,
  BI__builtin_arm_mve_vmulq_x_n_f16,
  BI__builtin_arm_mve_vmulq_x_n_f32,
  BI__builtin_arm_mve_vmulq_x_n_s16,
  BI__builtin_arm_mve_vmulq_x_n_s32,
  BI__builtin_arm_mve_vmulq_x_n_s8,
  BI__builtin_arm_mve_vmulq_x_n_u16,
  BI__builtin_arm_mve_vmulq_x_n_u32,
  BI__builtin_arm_mve_vmulq_x_n_u8,
  BI__builtin_arm_mve_vmulq_x_s16,
  BI__builtin_arm_mve_vmulq_x_s32,
  BI__builtin_arm_mve_vmulq_x_s8,
  BI__builtin_arm_mve_vmulq_x_u16,
  BI__builtin_arm_mve_vmulq_x_u32,
  BI__builtin_arm_mve_vmulq_x_u8,
  BI__builtin_arm_mve_vmvnq_m_n_s16,
  BI__builtin_arm_mve_vmvnq_m_n_s32,
  BI__builtin_arm_mve_vmvnq_m_n_u16,
  BI__builtin_arm_mve_vmvnq_m_n_u32,
  BI__builtin_arm_mve_vmvnq_m_s16,
  BI__builtin_arm_mve_vmvnq_m_s32,
  BI__builtin_arm_mve_vmvnq_m_s8,
  BI__builtin_arm_mve_vmvnq_m_u16,
  BI__builtin_arm_mve_vmvnq_m_u32,
  BI__builtin_arm_mve_vmvnq_m_u8,
  BI__builtin_arm_mve_vmvnq_n_s16,
  BI__builtin_arm_mve_vmvnq_n_s32,
  BI__builtin_arm_mve_vmvnq_n_u16,
  BI__builtin_arm_mve_vmvnq_n_u32,
  BI__builtin_arm_mve_vmvnq_s16,
  BI__builtin_arm_mve_vmvnq_s32,
  BI__builtin_arm_mve_vmvnq_s8,
  BI__builtin_arm_mve_vmvnq_u16,
  BI__builtin_arm_mve_vmvnq_u32,
  BI__builtin_arm_mve_vmvnq_u8,
  BI__builtin_arm_mve_vmvnq_x_n_s16,
  BI__builtin_arm_mve_vmvnq_x_n_s32,
  BI__builtin_arm_mve_vmvnq_x_n_u16,
  BI__builtin_arm_mve_vmvnq_x_n_u32,
  BI__builtin_arm_mve_vmvnq_x_s16,
  BI__builtin_arm_mve_vmvnq_x_s32,
  BI__builtin_arm_mve_vmvnq_x_s8,
  BI__builtin_arm_mve_vmvnq_x_u16,
  BI__builtin_arm_mve_vmvnq_x_u32,
  BI__builtin_arm_mve_vmvnq_x_u8,
  BI__builtin_arm_mve_vnegq_f16,
  BI__builtin_arm_mve_vnegq_f32,
  BI__builtin_arm_mve_vnegq_m_f16,
  BI__builtin_arm_mve_vnegq_m_f32,
  BI__builtin_arm_mve_vnegq_m_s16,
  BI__builtin_arm_mve_vnegq_m_s32,
  BI__builtin_arm_mve_vnegq_m_s8,
  BI__builtin_arm_mve_vnegq_s16,
  BI__builtin_arm_mve_vnegq_s32,
  BI__builtin_arm_mve_vnegq_s8,
  BI__builtin_arm_mve_vnegq_x_f16,
  BI__builtin_arm_mve_vnegq_x_f32,
  BI__builtin_arm_mve_vnegq_x_s16,
  BI__builtin_arm_mve_vnegq_x_s32,
  BI__builtin_arm_mve_vnegq_x_s8,
  BI__builtin_arm_mve_vornq_f16,
  BI__builtin_arm_mve_vornq_f32,
  BI__builtin_arm_mve_vornq_m_f16,
  BI__builtin_arm_mve_vornq_m_f32,
  BI__builtin_arm_mve_vornq_m_s16,
  BI__builtin_arm_mve_vornq_m_s32,
  BI__builtin_arm_mve_vornq_m_s8,
  BI__builtin_arm_mve_vornq_m_u16,
  BI__builtin_arm_mve_vornq_m_u32,
  BI__builtin_arm_mve_vornq_m_u8,
  BI__builtin_arm_mve_vornq_s16,
  BI__builtin_arm_mve_vornq_s32,
  BI__builtin_arm_mve_vornq_s8,
  BI__builtin_arm_mve_vornq_u16,
  BI__builtin_arm_mve_vornq_u32,
  BI__builtin_arm_mve_vornq_u8,
  BI__builtin_arm_mve_vornq_x_f16,
  BI__builtin_arm_mve_vornq_x_f32,
  BI__builtin_arm_mve_vornq_x_s16,
  BI__builtin_arm_mve_vornq_x_s32,
  BI__builtin_arm_mve_vornq_x_s8,
  BI__builtin_arm_mve_vornq_x_u16,
  BI__builtin_arm_mve_vornq_x_u32,
  BI__builtin_arm_mve_vornq_x_u8,
  BI__builtin_arm_mve_vorrq_f16,
  BI__builtin_arm_mve_vorrq_f32,
  BI__builtin_arm_mve_vorrq_m_f16,
  BI__builtin_arm_mve_vorrq_m_f32,
  BI__builtin_arm_mve_vorrq_m_n_s16,
  BI__builtin_arm_mve_vorrq_m_n_s32,
  BI__builtin_arm_mve_vorrq_m_n_u16,
  BI__builtin_arm_mve_vorrq_m_n_u32,
  BI__builtin_arm_mve_vorrq_m_s16,
  BI__builtin_arm_mve_vorrq_m_s32,
  BI__builtin_arm_mve_vorrq_m_s8,
  BI__builtin_arm_mve_vorrq_m_u16,
  BI__builtin_arm_mve_vorrq_m_u32,
  BI__builtin_arm_mve_vorrq_m_u8,
  BI__builtin_arm_mve_vorrq_n_s16,
  BI__builtin_arm_mve_vorrq_n_s32,
  BI__builtin_arm_mve_vorrq_n_u16,
  BI__builtin_arm_mve_vorrq_n_u32,
  BI__builtin_arm_mve_vorrq_s16,
  BI__builtin_arm_mve_vorrq_s32,
  BI__builtin_arm_mve_vorrq_s8,
  BI__builtin_arm_mve_vorrq_u16,
  BI__builtin_arm_mve_vorrq_u32,
  BI__builtin_arm_mve_vorrq_u8,
  BI__builtin_arm_mve_vorrq_x_f16,
  BI__builtin_arm_mve_vorrq_x_f32,
  BI__builtin_arm_mve_vorrq_x_s16,
  BI__builtin_arm_mve_vorrq_x_s32,
  BI__builtin_arm_mve_vorrq_x_s8,
  BI__builtin_arm_mve_vorrq_x_u16,
  BI__builtin_arm_mve_vorrq_x_u32,
  BI__builtin_arm_mve_vorrq_x_u8,
  BI__builtin_arm_mve_vpnot,
  BI__builtin_arm_mve_vpselq_f16,
  BI__builtin_arm_mve_vpselq_f32,
  BI__builtin_arm_mve_vpselq_s16,
  BI__builtin_arm_mve_vpselq_s32,
  BI__builtin_arm_mve_vpselq_s64,
  BI__builtin_arm_mve_vpselq_s8,
  BI__builtin_arm_mve_vpselq_u16,
  BI__builtin_arm_mve_vpselq_u32,
  BI__builtin_arm_mve_vpselq_u64,
  BI__builtin_arm_mve_vpselq_u8,
  BI__builtin_arm_mve_vqabsq_m_s16,
  BI__builtin_arm_mve_vqabsq_m_s32,
  BI__builtin_arm_mve_vqabsq_m_s8,
  BI__builtin_arm_mve_vqabsq_s16,
  BI__builtin_arm_mve_vqabsq_s32,
  BI__builtin_arm_mve_vqabsq_s8,
  BI__builtin_arm_mve_vqaddq_m_n_s16,
  BI__builtin_arm_mve_vqaddq_m_n_s32,
  BI__builtin_arm_mve_vqaddq_m_n_s8,
  BI__builtin_arm_mve_vqaddq_m_n_u16,
  BI__builtin_arm_mve_vqaddq_m_n_u32,
  BI__builtin_arm_mve_vqaddq_m_n_u8,
  BI__builtin_arm_mve_vqaddq_m_s16,
  BI__builtin_arm_mve_vqaddq_m_s32,
  BI__builtin_arm_mve_vqaddq_m_s8,
  BI__builtin_arm_mve_vqaddq_m_u16,
  BI__builtin_arm_mve_vqaddq_m_u32,
  BI__builtin_arm_mve_vqaddq_m_u8,
  BI__builtin_arm_mve_vqaddq_n_s16,
  BI__builtin_arm_mve_vqaddq_n_s32,
  BI__builtin_arm_mve_vqaddq_n_s8,
  BI__builtin_arm_mve_vqaddq_n_u16,
  BI__builtin_arm_mve_vqaddq_n_u32,
  BI__builtin_arm_mve_vqaddq_n_u8,
  BI__builtin_arm_mve_vqaddq_s16,
  BI__builtin_arm_mve_vqaddq_s32,
  BI__builtin_arm_mve_vqaddq_s8,
  BI__builtin_arm_mve_vqaddq_u16,
  BI__builtin_arm_mve_vqaddq_u32,
  BI__builtin_arm_mve_vqaddq_u8,
  BI__builtin_arm_mve_vqdmladhq_m_s16,
  BI__builtin_arm_mve_vqdmladhq_m_s32,
  BI__builtin_arm_mve_vqdmladhq_m_s8,
  BI__builtin_arm_mve_vqdmladhq_s16,
  BI__builtin_arm_mve_vqdmladhq_s32,
  BI__builtin_arm_mve_vqdmladhq_s8,
  BI__builtin_arm_mve_vqdmladhxq_m_s16,
  BI__builtin_arm_mve_vqdmladhxq_m_s32,
  BI__builtin_arm_mve_vqdmladhxq_m_s8,
  BI__builtin_arm_mve_vqdmladhxq_s16,
  BI__builtin_arm_mve_vqdmladhxq_s32,
  BI__builtin_arm_mve_vqdmladhxq_s8,
  BI__builtin_arm_mve_vqdmlahq_m_n_s16,
  BI__builtin_arm_mve_vqdmlahq_m_n_s32,
  BI__builtin_arm_mve_vqdmlahq_m_n_s8,
  BI__builtin_arm_mve_vqdmlahq_n_s16,
  BI__builtin_arm_mve_vqdmlahq_n_s32,
  BI__builtin_arm_mve_vqdmlahq_n_s8,
  BI__builtin_arm_mve_vqdmlashq_m_n_s16,
  BI__builtin_arm_mve_vqdmlashq_m_n_s32,
  BI__builtin_arm_mve_vqdmlashq_m_n_s8,
  BI__builtin_arm_mve_vqdmlashq_n_s16,
  BI__builtin_arm_mve_vqdmlashq_n_s32,
  BI__builtin_arm_mve_vqdmlashq_n_s8,
  BI__builtin_arm_mve_vqdmlsdhq_m_s16,
  BI__builtin_arm_mve_vqdmlsdhq_m_s32,
  BI__builtin_arm_mve_vqdmlsdhq_m_s8,
  BI__builtin_arm_mve_vqdmlsdhq_s16,
  BI__builtin_arm_mve_vqdmlsdhq_s32,
  BI__builtin_arm_mve_vqdmlsdhq_s8,
  BI__builtin_arm_mve_vqdmlsdhxq_m_s16,
  BI__builtin_arm_mve_vqdmlsdhxq_m_s32,
  BI__builtin_arm_mve_vqdmlsdhxq_m_s8,
  BI__builtin_arm_mve_vqdmlsdhxq_s16,
  BI__builtin_arm_mve_vqdmlsdhxq_s32,
  BI__builtin_arm_mve_vqdmlsdhxq_s8,
  BI__builtin_arm_mve_vqdmulhq_m_n_s16,
  BI__builtin_arm_mve_vqdmulhq_m_n_s32,
  BI__builtin_arm_mve_vqdmulhq_m_n_s8,
  BI__builtin_arm_mve_vqdmulhq_m_s16,
  BI__builtin_arm_mve_vqdmulhq_m_s32,
  BI__builtin_arm_mve_vqdmulhq_m_s8,
  BI__builtin_arm_mve_vqdmulhq_n_s16,
  BI__builtin_arm_mve_vqdmulhq_n_s32,
  BI__builtin_arm_mve_vqdmulhq_n_s8,
  BI__builtin_arm_mve_vqdmulhq_s16,
  BI__builtin_arm_mve_vqdmulhq_s32,
  BI__builtin_arm_mve_vqdmulhq_s8,
  BI__builtin_arm_mve_vqdmullbq_m_n_s16,
  BI__builtin_arm_mve_vqdmullbq_m_n_s32,
  BI__builtin_arm_mve_vqdmullbq_m_s16,
  BI__builtin_arm_mve_vqdmullbq_m_s32,
  BI__builtin_arm_mve_vqdmullbq_n_s16,
  BI__builtin_arm_mve_vqdmullbq_n_s32,
  BI__builtin_arm_mve_vqdmullbq_s16,
  BI__builtin_arm_mve_vqdmullbq_s32,
  BI__builtin_arm_mve_vqdmulltq_m_n_s16,
  BI__builtin_arm_mve_vqdmulltq_m_n_s32,
  BI__builtin_arm_mve_vqdmulltq_m_s16,
  BI__builtin_arm_mve_vqdmulltq_m_s32,
  BI__builtin_arm_mve_vqdmulltq_n_s16,
  BI__builtin_arm_mve_vqdmulltq_n_s32,
  BI__builtin_arm_mve_vqdmulltq_s16,
  BI__builtin_arm_mve_vqdmulltq_s32,
  BI__builtin_arm_mve_vqmovnbq_m_s16,
  BI__builtin_arm_mve_vqmovnbq_m_s32,
  BI__builtin_arm_mve_vqmovnbq_m_u16,
  BI__builtin_arm_mve_vqmovnbq_m_u32,
  BI__builtin_arm_mve_vqmovnbq_s16,
  BI__builtin_arm_mve_vqmovnbq_s32,
  BI__builtin_arm_mve_vqmovnbq_u16,
  BI__builtin_arm_mve_vqmovnbq_u32,
  BI__builtin_arm_mve_vqmovntq_m_s16,
  BI__builtin_arm_mve_vqmovntq_m_s32,
  BI__builtin_arm_mve_vqmovntq_m_u16,
  BI__builtin_arm_mve_vqmovntq_m_u32,
  BI__builtin_arm_mve_vqmovntq_s16,
  BI__builtin_arm_mve_vqmovntq_s32,
  BI__builtin_arm_mve_vqmovntq_u16,
  BI__builtin_arm_mve_vqmovntq_u32,
  BI__builtin_arm_mve_vqmovunbq_m_s16,
  BI__builtin_arm_mve_vqmovunbq_m_s32,
  BI__builtin_arm_mve_vqmovunbq_s16,
  BI__builtin_arm_mve_vqmovunbq_s32,
  BI__builtin_arm_mve_vqmovuntq_m_s16,
  BI__builtin_arm_mve_vqmovuntq_m_s32,
  BI__builtin_arm_mve_vqmovuntq_s16,
  BI__builtin_arm_mve_vqmovuntq_s32,
  BI__builtin_arm_mve_vqnegq_m_s16,
  BI__builtin_arm_mve_vqnegq_m_s32,
  BI__builtin_arm_mve_vqnegq_m_s8,
  BI__builtin_arm_mve_vqnegq_s16,
  BI__builtin_arm_mve_vqnegq_s32,
  BI__builtin_arm_mve_vqnegq_s8,
  BI__builtin_arm_mve_vqrdmladhq_m_s16,
  BI__builtin_arm_mve_vqrdmladhq_m_s32,
  BI__builtin_arm_mve_vqrdmladhq_m_s8,
  BI__builtin_arm_mve_vqrdmladhq_s16,
  BI__builtin_arm_mve_vqrdmladhq_s32,
  BI__builtin_arm_mve_vqrdmladhq_s8,
  BI__builtin_arm_mve_vqrdmladhxq_m_s16,
  BI__builtin_arm_mve_vqrdmladhxq_m_s32,
  BI__builtin_arm_mve_vqrdmladhxq_m_s8,
  BI__builtin_arm_mve_vqrdmladhxq_s16,
  BI__builtin_arm_mve_vqrdmladhxq_s32,
  BI__builtin_arm_mve_vqrdmladhxq_s8,
  BI__builtin_arm_mve_vqrdmlahq_m_n_s16,
  BI__builtin_arm_mve_vqrdmlahq_m_n_s32,
  BI__builtin_arm_mve_vqrdmlahq_m_n_s8,
  BI__builtin_arm_mve_vqrdmlahq_n_s16,
  BI__builtin_arm_mve_vqrdmlahq_n_s32,
  BI__builtin_arm_mve_vqrdmlahq_n_s8,
  BI__builtin_arm_mve_vqrdmlashq_m_n_s16,
  BI__builtin_arm_mve_vqrdmlashq_m_n_s32,
  BI__builtin_arm_mve_vqrdmlashq_m_n_s8,
  BI__builtin_arm_mve_vqrdmlashq_n_s16,
  BI__builtin_arm_mve_vqrdmlashq_n_s32,
  BI__builtin_arm_mve_vqrdmlashq_n_s8,
  BI__builtin_arm_mve_vqrdmlsdhq_m_s16,
  BI__builtin_arm_mve_vqrdmlsdhq_m_s32,
  BI__builtin_arm_mve_vqrdmlsdhq_m_s8,
  BI__builtin_arm_mve_vqrdmlsdhq_s16,
  BI__builtin_arm_mve_vqrdmlsdhq_s32,
  BI__builtin_arm_mve_vqrdmlsdhq_s8,
  BI__builtin_arm_mve_vqrdmlsdhxq_m_s16,
  BI__builtin_arm_mve_vqrdmlsdhxq_m_s32,
  BI__builtin_arm_mve_vqrdmlsdhxq_m_s8,
  BI__builtin_arm_mve_vqrdmlsdhxq_s16,
  BI__builtin_arm_mve_vqrdmlsdhxq_s32,
  BI__builtin_arm_mve_vqrdmlsdhxq_s8,
  BI__builtin_arm_mve_vqrdmulhq_m_n_s16,
  BI__builtin_arm_mve_vqrdmulhq_m_n_s32,
  BI__builtin_arm_mve_vqrdmulhq_m_n_s8,
  BI__builtin_arm_mve_vqrdmulhq_m_s16,
  BI__builtin_arm_mve_vqrdmulhq_m_s32,
  BI__builtin_arm_mve_vqrdmulhq_m_s8,
  BI__builtin_arm_mve_vqrdmulhq_n_s16,
  BI__builtin_arm_mve_vqrdmulhq_n_s32,
  BI__builtin_arm_mve_vqrdmulhq_n_s8,
  BI__builtin_arm_mve_vqrdmulhq_s16,
  BI__builtin_arm_mve_vqrdmulhq_s32,
  BI__builtin_arm_mve_vqrdmulhq_s8,
  BI__builtin_arm_mve_vqrshlq_m_n_s16,
  BI__builtin_arm_mve_vqrshlq_m_n_s32,
  BI__builtin_arm_mve_vqrshlq_m_n_s8,
  BI__builtin_arm_mve_vqrshlq_m_n_u16,
  BI__builtin_arm_mve_vqrshlq_m_n_u32,
  BI__builtin_arm_mve_vqrshlq_m_n_u8,
  BI__builtin_arm_mve_vqrshlq_m_s16,
  BI__builtin_arm_mve_vqrshlq_m_s32,
  BI__builtin_arm_mve_vqrshlq_m_s8,
  BI__builtin_arm_mve_vqrshlq_m_u16,
  BI__builtin_arm_mve_vqrshlq_m_u32,
  BI__builtin_arm_mve_vqrshlq_m_u8,
  BI__builtin_arm_mve_vqrshlq_n_s16,
  BI__builtin_arm_mve_vqrshlq_n_s32,
  BI__builtin_arm_mve_vqrshlq_n_s8,
  BI__builtin_arm_mve_vqrshlq_n_u16,
  BI__builtin_arm_mve_vqrshlq_n_u32,
  BI__builtin_arm_mve_vqrshlq_n_u8,
  BI__builtin_arm_mve_vqrshlq_s16,
  BI__builtin_arm_mve_vqrshlq_s32,
  BI__builtin_arm_mve_vqrshlq_s8,
  BI__builtin_arm_mve_vqrshlq_u16,
  BI__builtin_arm_mve_vqrshlq_u32,
  BI__builtin_arm_mve_vqrshlq_u8,
  BI__builtin_arm_mve_vqrshrnbq_m_n_s16,
  BI__builtin_arm_mve_vqrshrnbq_m_n_s32,
  BI__builtin_arm_mve_vqrshrnbq_m_n_u16,
  BI__builtin_arm_mve_vqrshrnbq_m_n_u32,
  BI__builtin_arm_mve_vqrshrnbq_n_s16,
  BI__builtin_arm_mve_vqrshrnbq_n_s32,
  BI__builtin_arm_mve_vqrshrnbq_n_u16,
  BI__builtin_arm_mve_vqrshrnbq_n_u32,
  BI__builtin_arm_mve_vqrshrntq_m_n_s16,
  BI__builtin_arm_mve_vqrshrntq_m_n_s32,
  BI__builtin_arm_mve_vqrshrntq_m_n_u16,
  BI__builtin_arm_mve_vqrshrntq_m_n_u32,
  BI__builtin_arm_mve_vqrshrntq_n_s16,
  BI__builtin_arm_mve_vqrshrntq_n_s32,
  BI__builtin_arm_mve_vqrshrntq_n_u16,
  BI__builtin_arm_mve_vqrshrntq_n_u32,
  BI__builtin_arm_mve_vqrshrunbq_m_n_s16,
  BI__builtin_arm_mve_vqrshrunbq_m_n_s32,
  BI__builtin_arm_mve_vqrshrunbq_n_s16,
  BI__builtin_arm_mve_vqrshrunbq_n_s32,
  BI__builtin_arm_mve_vqrshruntq_m_n_s16,
  BI__builtin_arm_mve_vqrshruntq_m_n_s32,
  BI__builtin_arm_mve_vqrshruntq_n_s16,
  BI__builtin_arm_mve_vqrshruntq_n_s32,
  BI__builtin_arm_mve_vqshlq_m_n_s16,
  BI__builtin_arm_mve_vqshlq_m_n_s32,
  BI__builtin_arm_mve_vqshlq_m_n_s8,
  BI__builtin_arm_mve_vqshlq_m_n_u16,
  BI__builtin_arm_mve_vqshlq_m_n_u32,
  BI__builtin_arm_mve_vqshlq_m_n_u8,
  BI__builtin_arm_mve_vqshlq_m_r_s16,
  BI__builtin_arm_mve_vqshlq_m_r_s32,
  BI__builtin_arm_mve_vqshlq_m_r_s8,
  BI__builtin_arm_mve_vqshlq_m_r_u16,
  BI__builtin_arm_mve_vqshlq_m_r_u32,
  BI__builtin_arm_mve_vqshlq_m_r_u8,
  BI__builtin_arm_mve_vqshlq_m_s16,
  BI__builtin_arm_mve_vqshlq_m_s32,
  BI__builtin_arm_mve_vqshlq_m_s8,
  BI__builtin_arm_mve_vqshlq_m_u16,
  BI__builtin_arm_mve_vqshlq_m_u32,
  BI__builtin_arm_mve_vqshlq_m_u8,
  BI__builtin_arm_mve_vqshlq_n_s16,
  BI__builtin_arm_mve_vqshlq_n_s32,
  BI__builtin_arm_mve_vqshlq_n_s8,
  BI__builtin_arm_mve_vqshlq_n_u16,
  BI__builtin_arm_mve_vqshlq_n_u32,
  BI__builtin_arm_mve_vqshlq_n_u8,
  BI__builtin_arm_mve_vqshlq_r_s16,
  BI__builtin_arm_mve_vqshlq_r_s32,
  BI__builtin_arm_mve_vqshlq_r_s8,
  BI__builtin_arm_mve_vqshlq_r_u16,
  BI__builtin_arm_mve_vqshlq_r_u32,
  BI__builtin_arm_mve_vqshlq_r_u8,
  BI__builtin_arm_mve_vqshlq_s16,
  BI__builtin_arm_mve_vqshlq_s32,
  BI__builtin_arm_mve_vqshlq_s8,
  BI__builtin_arm_mve_vqshlq_u16,
  BI__builtin_arm_mve_vqshlq_u32,
  BI__builtin_arm_mve_vqshlq_u8,
  BI__builtin_arm_mve_vqshluq_m_n_s16,
  BI__builtin_arm_mve_vqshluq_m_n_s32,
  BI__builtin_arm_mve_vqshluq_m_n_s8,
  BI__builtin_arm_mve_vqshluq_n_s16,
  BI__builtin_arm_mve_vqshluq_n_s32,
  BI__builtin_arm_mve_vqshluq_n_s8,
  BI__builtin_arm_mve_vqshrnbq_m_n_s16,
  BI__builtin_arm_mve_vqshrnbq_m_n_s32,
  BI__builtin_arm_mve_vqshrnbq_m_n_u16,
  BI__builtin_arm_mve_vqshrnbq_m_n_u32,
  BI__builtin_arm_mve_vqshrnbq_n_s16,
  BI__builtin_arm_mve_vqshrnbq_n_s32,
  BI__builtin_arm_mve_vqshrnbq_n_u16,
  BI__builtin_arm_mve_vqshrnbq_n_u32,
  BI__builtin_arm_mve_vqshrntq_m_n_s16,
  BI__builtin_arm_mve_vqshrntq_m_n_s32,
  BI__builtin_arm_mve_vqshrntq_m_n_u16,
  BI__builtin_arm_mve_vqshrntq_m_n_u32,
  BI__builtin_arm_mve_vqshrntq_n_s16,
  BI__builtin_arm_mve_vqshrntq_n_s32,
  BI__builtin_arm_mve_vqshrntq_n_u16,
  BI__builtin_arm_mve_vqshrntq_n_u32,
  BI__builtin_arm_mve_vqshrunbq_m_n_s16,
  BI__builtin_arm_mve_vqshrunbq_m_n_s32,
  BI__builtin_arm_mve_vqshrunbq_n_s16,
  BI__builtin_arm_mve_vqshrunbq_n_s32,
  BI__builtin_arm_mve_vqshruntq_m_n_s16,
  BI__builtin_arm_mve_vqshruntq_m_n_s32,
  BI__builtin_arm_mve_vqshruntq_n_s16,
  BI__builtin_arm_mve_vqshruntq_n_s32,
  BI__builtin_arm_mve_vqsubq_m_n_s16,
  BI__builtin_arm_mve_vqsubq_m_n_s32,
  BI__builtin_arm_mve_vqsubq_m_n_s8,
  BI__builtin_arm_mve_vqsubq_m_n_u16,
  BI__builtin_arm_mve_vqsubq_m_n_u32,
  BI__builtin_arm_mve_vqsubq_m_n_u8,
  BI__builtin_arm_mve_vqsubq_m_s16,
  BI__builtin_arm_mve_vqsubq_m_s32,
  BI__builtin_arm_mve_vqsubq_m_s8,
  BI__builtin_arm_mve_vqsubq_m_u16,
  BI__builtin_arm_mve_vqsubq_m_u32,
  BI__builtin_arm_mve_vqsubq_m_u8,
  BI__builtin_arm_mve_vqsubq_n_s16,
  BI__builtin_arm_mve_vqsubq_n_s32,
  BI__builtin_arm_mve_vqsubq_n_s8,
  BI__builtin_arm_mve_vqsubq_n_u16,
  BI__builtin_arm_mve_vqsubq_n_u32,
  BI__builtin_arm_mve_vqsubq_n_u8,
  BI__builtin_arm_mve_vqsubq_s16,
  BI__builtin_arm_mve_vqsubq_s32,
  BI__builtin_arm_mve_vqsubq_s8,
  BI__builtin_arm_mve_vqsubq_u16,
  BI__builtin_arm_mve_vqsubq_u32,
  BI__builtin_arm_mve_vqsubq_u8,
  BI__builtin_arm_mve_vreinterpretq_f16_f32,
  BI__builtin_arm_mve_vreinterpretq_f16_s16,
  BI__builtin_arm_mve_vreinterpretq_f16_s32,
  BI__builtin_arm_mve_vreinterpretq_f16_s64,
  BI__builtin_arm_mve_vreinterpretq_f16_s8,
  BI__builtin_arm_mve_vreinterpretq_f16_u16,
  BI__builtin_arm_mve_vreinterpretq_f16_u32,
  BI__builtin_arm_mve_vreinterpretq_f16_u64,
  BI__builtin_arm_mve_vreinterpretq_f16_u8,
  BI__builtin_arm_mve_vreinterpretq_f32_f16,
  BI__builtin_arm_mve_vreinterpretq_f32_s16,
  BI__builtin_arm_mve_vreinterpretq_f32_s32,
  BI__builtin_arm_mve_vreinterpretq_f32_s64,
  BI__builtin_arm_mve_vreinterpretq_f32_s8,
  BI__builtin_arm_mve_vreinterpretq_f32_u16,
  BI__builtin_arm_mve_vreinterpretq_f32_u32,
  BI__builtin_arm_mve_vreinterpretq_f32_u64,
  BI__builtin_arm_mve_vreinterpretq_f32_u8,
  BI__builtin_arm_mve_vreinterpretq_s16_f16,
  BI__builtin_arm_mve_vreinterpretq_s16_f32,
  BI__builtin_arm_mve_vreinterpretq_s16_s32,
  BI__builtin_arm_mve_vreinterpretq_s16_s64,
  BI__builtin_arm_mve_vreinterpretq_s16_s8,
  BI__builtin_arm_mve_vreinterpretq_s16_u16,
  BI__builtin_arm_mve_vreinterpretq_s16_u32,
  BI__builtin_arm_mve_vreinterpretq_s16_u64,
  BI__builtin_arm_mve_vreinterpretq_s16_u8,
  BI__builtin_arm_mve_vreinterpretq_s32_f16,
  BI__builtin_arm_mve_vreinterpretq_s32_f32,
  BI__builtin_arm_mve_vreinterpretq_s32_s16,
  BI__builtin_arm_mve_vreinterpretq_s32_s64,
  BI__builtin_arm_mve_vreinterpretq_s32_s8,
  BI__builtin_arm_mve_vreinterpretq_s32_u16,
  BI__builtin_arm_mve_vreinterpretq_s32_u32,
  BI__builtin_arm_mve_vreinterpretq_s32_u64,
  BI__builtin_arm_mve_vreinterpretq_s32_u8,
  BI__builtin_arm_mve_vreinterpretq_s64_f16,
  BI__builtin_arm_mve_vreinterpretq_s64_f32,
  BI__builtin_arm_mve_vreinterpretq_s64_s16,
  BI__builtin_arm_mve_vreinterpretq_s64_s32,
  BI__builtin_arm_mve_vreinterpretq_s64_s8,
  BI__builtin_arm_mve_vreinterpretq_s64_u16,
  BI__builtin_arm_mve_vreinterpretq_s64_u32,
  BI__builtin_arm_mve_vreinterpretq_s64_u64,
  BI__builtin_arm_mve_vreinterpretq_s64_u8,
  BI__builtin_arm_mve_vreinterpretq_s8_f16,
  BI__builtin_arm_mve_vreinterpretq_s8_f32,
  BI__builtin_arm_mve_vreinterpretq_s8_s16,
  BI__builtin_arm_mve_vreinterpretq_s8_s32,
  BI__builtin_arm_mve_vreinterpretq_s8_s64,
  BI__builtin_arm_mve_vreinterpretq_s8_u16,
  BI__builtin_arm_mve_vreinterpretq_s8_u32,
  BI__builtin_arm_mve_vreinterpretq_s8_u64,
  BI__builtin_arm_mve_vreinterpretq_s8_u8,
  BI__builtin_arm_mve_vreinterpretq_u16_f16,
  BI__builtin_arm_mve_vreinterpretq_u16_f32,
  BI__builtin_arm_mve_vreinterpretq_u16_s16,
  BI__builtin_arm_mve_vreinterpretq_u16_s32,
  BI__builtin_arm_mve_vreinterpretq_u16_s64,
  BI__builtin_arm_mve_vreinterpretq_u16_s8,
  BI__builtin_arm_mve_vreinterpretq_u16_u32,
  BI__builtin_arm_mve_vreinterpretq_u16_u64,
  BI__builtin_arm_mve_vreinterpretq_u16_u8,
  BI__builtin_arm_mve_vreinterpretq_u32_f16,
  BI__builtin_arm_mve_vreinterpretq_u32_f32,
  BI__builtin_arm_mve_vreinterpretq_u32_s16,
  BI__builtin_arm_mve_vreinterpretq_u32_s32,
  BI__builtin_arm_mve_vreinterpretq_u32_s64,
  BI__builtin_arm_mve_vreinterpretq_u32_s8,
  BI__builtin_arm_mve_vreinterpretq_u32_u16,
  BI__builtin_arm_mve_vreinterpretq_u32_u64,
  BI__builtin_arm_mve_vreinterpretq_u32_u8,
  BI__builtin_arm_mve_vreinterpretq_u64_f16,
  BI__builtin_arm_mve_vreinterpretq_u64_f32,
  BI__builtin_arm_mve_vreinterpretq_u64_s16,
  BI__builtin_arm_mve_vreinterpretq_u64_s32,
  BI__builtin_arm_mve_vreinterpretq_u64_s64,
  BI__builtin_arm_mve_vreinterpretq_u64_s8,
  BI__builtin_arm_mve_vreinterpretq_u64_u16,
  BI__builtin_arm_mve_vreinterpretq_u64_u32,
  BI__builtin_arm_mve_vreinterpretq_u64_u8,
  BI__builtin_arm_mve_vreinterpretq_u8_f16,
  BI__builtin_arm_mve_vreinterpretq_u8_f32,
  BI__builtin_arm_mve_vreinterpretq_u8_s16,
  BI__builtin_arm_mve_vreinterpretq_u8_s32,
  BI__builtin_arm_mve_vreinterpretq_u8_s64,
  BI__builtin_arm_mve_vreinterpretq_u8_s8,
  BI__builtin_arm_mve_vreinterpretq_u8_u16,
  BI__builtin_arm_mve_vreinterpretq_u8_u32,
  BI__builtin_arm_mve_vreinterpretq_u8_u64,
  BI__builtin_arm_mve_vrev16q_m_s8,
  BI__builtin_arm_mve_vrev16q_m_u8,
  BI__builtin_arm_mve_vrev16q_s8,
  BI__builtin_arm_mve_vrev16q_u8,
  BI__builtin_arm_mve_vrev16q_x_s8,
  BI__builtin_arm_mve_vrev16q_x_u8,
  BI__builtin_arm_mve_vrev32q_f16,
  BI__builtin_arm_mve_vrev32q_m_f16,
  BI__builtin_arm_mve_vrev32q_m_s16,
  BI__builtin_arm_mve_vrev32q_m_s8,
  BI__builtin_arm_mve_vrev32q_m_u16,
  BI__builtin_arm_mve_vrev32q_m_u8,
  BI__builtin_arm_mve_vrev32q_s16,
  BI__builtin_arm_mve_vrev32q_s8,
  BI__builtin_arm_mve_vrev32q_u16,
  BI__builtin_arm_mve_vrev32q_u8,
  BI__builtin_arm_mve_vrev32q_x_f16,
  BI__builtin_arm_mve_vrev32q_x_s16,
  BI__builtin_arm_mve_vrev32q_x_s8,
  BI__builtin_arm_mve_vrev32q_x_u16,
  BI__builtin_arm_mve_vrev32q_x_u8,
  BI__builtin_arm_mve_vrev64q_f16,
  BI__builtin_arm_mve_vrev64q_f32,
  BI__builtin_arm_mve_vrev64q_m_f16,
  BI__builtin_arm_mve_vrev64q_m_f32,
  BI__builtin_arm_mve_vrev64q_m_s16,
  BI__builtin_arm_mve_vrev64q_m_s32,
  BI__builtin_arm_mve_vrev64q_m_s8,
  BI__builtin_arm_mve_vrev64q_m_u16,
  BI__builtin_arm_mve_vrev64q_m_u32,
  BI__builtin_arm_mve_vrev64q_m_u8,
  BI__builtin_arm_mve_vrev64q_s16,
  BI__builtin_arm_mve_vrev64q_s32,
  BI__builtin_arm_mve_vrev64q_s8,
  BI__builtin_arm_mve_vrev64q_u16,
  BI__builtin_arm_mve_vrev64q_u32,
  BI__builtin_arm_mve_vrev64q_u8,
  BI__builtin_arm_mve_vrev64q_x_f16,
  BI__builtin_arm_mve_vrev64q_x_f32,
  BI__builtin_arm_mve_vrev64q_x_s16,
  BI__builtin_arm_mve_vrev64q_x_s32,
  BI__builtin_arm_mve_vrev64q_x_s8,
  BI__builtin_arm_mve_vrev64q_x_u16,
  BI__builtin_arm_mve_vrev64q_x_u32,
  BI__builtin_arm_mve_vrev64q_x_u8,
  BI__builtin_arm_mve_vrhaddq_m_s16,
  BI__builtin_arm_mve_vrhaddq_m_s32,
  BI__builtin_arm_mve_vrhaddq_m_s8,
  BI__builtin_arm_mve_vrhaddq_m_u16,
  BI__builtin_arm_mve_vrhaddq_m_u32,
  BI__builtin_arm_mve_vrhaddq_m_u8,
  BI__builtin_arm_mve_vrhaddq_s16,
  BI__builtin_arm_mve_vrhaddq_s32,
  BI__builtin_arm_mve_vrhaddq_s8,
  BI__builtin_arm_mve_vrhaddq_u16,
  BI__builtin_arm_mve_vrhaddq_u32,
  BI__builtin_arm_mve_vrhaddq_u8,
  BI__builtin_arm_mve_vrhaddq_x_s16,
  BI__builtin_arm_mve_vrhaddq_x_s32,
  BI__builtin_arm_mve_vrhaddq_x_s8,
  BI__builtin_arm_mve_vrhaddq_x_u16,
  BI__builtin_arm_mve_vrhaddq_x_u32,
  BI__builtin_arm_mve_vrhaddq_x_u8,
  BI__builtin_arm_mve_vrmlaldavhaq_p_s32,
  BI__builtin_arm_mve_vrmlaldavhaq_p_u32,
  BI__builtin_arm_mve_vrmlaldavhaq_s32,
  BI__builtin_arm_mve_vrmlaldavhaq_u32,
  BI__builtin_arm_mve_vrmlaldavhaxq_p_s32,
  BI__builtin_arm_mve_vrmlaldavhaxq_s32,
  BI__builtin_arm_mve_vrmlaldavhq_p_s32,
  BI__builtin_arm_mve_vrmlaldavhq_p_u32,
  BI__builtin_arm_mve_vrmlaldavhq_s32,
  BI__builtin_arm_mve_vrmlaldavhq_u32,
  BI__builtin_arm_mve_vrmlaldavhxq_p_s32,
  BI__builtin_arm_mve_vrmlaldavhxq_s32,
  BI__builtin_arm_mve_vrmlsldavhaq_p_s32,
  BI__builtin_arm_mve_vrmlsldavhaq_s32,
  BI__builtin_arm_mve_vrmlsldavhaxq_p_s32,
  BI__builtin_arm_mve_vrmlsldavhaxq_s32,
  BI__builtin_arm_mve_vrmlsldavhq_p_s32,
  BI__builtin_arm_mve_vrmlsldavhq_s32,
  BI__builtin_arm_mve_vrmlsldavhxq_p_s32,
  BI__builtin_arm_mve_vrmlsldavhxq_s32,
  BI__builtin_arm_mve_vrmulhq_m_s16,
  BI__builtin_arm_mve_vrmulhq_m_s32,
  BI__builtin_arm_mve_vrmulhq_m_s8,
  BI__builtin_arm_mve_vrmulhq_m_u16,
  BI__builtin_arm_mve_vrmulhq_m_u32,
  BI__builtin_arm_mve_vrmulhq_m_u8,
  BI__builtin_arm_mve_vrmulhq_s16,
  BI__builtin_arm_mve_vrmulhq_s32,
  BI__builtin_arm_mve_vrmulhq_s8,
  BI__builtin_arm_mve_vrmulhq_u16,
  BI__builtin_arm_mve_vrmulhq_u32,
  BI__builtin_arm_mve_vrmulhq_u8,
  BI__builtin_arm_mve_vrmulhq_x_s16,
  BI__builtin_arm_mve_vrmulhq_x_s32,
  BI__builtin_arm_mve_vrmulhq_x_s8,
  BI__builtin_arm_mve_vrmulhq_x_u16,
  BI__builtin_arm_mve_vrmulhq_x_u32,
  BI__builtin_arm_mve_vrmulhq_x_u8,
  BI__builtin_arm_mve_vrndaq_f16,
  BI__builtin_arm_mve_vrndaq_f32,
  BI__builtin_arm_mve_vrndaq_m_f16,
  BI__builtin_arm_mve_vrndaq_m_f32,
  BI__builtin_arm_mve_vrndaq_x_f16,
  BI__builtin_arm_mve_vrndaq_x_f32,
  BI__builtin_arm_mve_vrndmq_f16,
  BI__builtin_arm_mve_vrndmq_f32,
  BI__builtin_arm_mve_vrndmq_m_f16,
  BI__builtin_arm_mve_vrndmq_m_f32,
  BI__builtin_arm_mve_vrndmq_x_f16,
  BI__builtin_arm_mve_vrndmq_x_f32,
  BI__builtin_arm_mve_vrndnq_f16,
  BI__builtin_arm_mve_vrndnq_f32,
  BI__builtin_arm_mve_vrndnq_m_f16,
  BI__builtin_arm_mve_vrndnq_m_f32,
  BI__builtin_arm_mve_vrndnq_x_f16,
  BI__builtin_arm_mve_vrndnq_x_f32,
  BI__builtin_arm_mve_vrndpq_f16,
  BI__builtin_arm_mve_vrndpq_f32,
  BI__builtin_arm_mve_vrndpq_m_f16,
  BI__builtin_arm_mve_vrndpq_m_f32,
  BI__builtin_arm_mve_vrndpq_x_f16,
  BI__builtin_arm_mve_vrndpq_x_f32,
  BI__builtin_arm_mve_vrndq_f16,
  BI__builtin_arm_mve_vrndq_f32,
  BI__builtin_arm_mve_vrndq_m_f16,
  BI__builtin_arm_mve_vrndq_m_f32,
  BI__builtin_arm_mve_vrndq_x_f16,
  BI__builtin_arm_mve_vrndq_x_f32,
  BI__builtin_arm_mve_vrndxq_f16,
  BI__builtin_arm_mve_vrndxq_f32,
  BI__builtin_arm_mve_vrndxq_m_f16,
  BI__builtin_arm_mve_vrndxq_m_f32,
  BI__builtin_arm_mve_vrndxq_x_f16,
  BI__builtin_arm_mve_vrndxq_x_f32,
  BI__builtin_arm_mve_vrshlq_m_n_s16,
  BI__builtin_arm_mve_vrshlq_m_n_s32,
  BI__builtin_arm_mve_vrshlq_m_n_s8,
  BI__builtin_arm_mve_vrshlq_m_n_u16,
  BI__builtin_arm_mve_vrshlq_m_n_u32,
  BI__builtin_arm_mve_vrshlq_m_n_u8,
  BI__builtin_arm_mve_vrshlq_m_s16,
  BI__builtin_arm_mve_vrshlq_m_s32,
  BI__builtin_arm_mve_vrshlq_m_s8,
  BI__builtin_arm_mve_vrshlq_m_u16,
  BI__builtin_arm_mve_vrshlq_m_u32,
  BI__builtin_arm_mve_vrshlq_m_u8,
  BI__builtin_arm_mve_vrshlq_n_s16,
  BI__builtin_arm_mve_vrshlq_n_s32,
  BI__builtin_arm_mve_vrshlq_n_s8,
  BI__builtin_arm_mve_vrshlq_n_u16,
  BI__builtin_arm_mve_vrshlq_n_u32,
  BI__builtin_arm_mve_vrshlq_n_u8,
  BI__builtin_arm_mve_vrshlq_s16,
  BI__builtin_arm_mve_vrshlq_s32,
  BI__builtin_arm_mve_vrshlq_s8,
  BI__builtin_arm_mve_vrshlq_u16,
  BI__builtin_arm_mve_vrshlq_u32,
  BI__builtin_arm_mve_vrshlq_u8,
  BI__builtin_arm_mve_vrshlq_x_s16,
  BI__builtin_arm_mve_vrshlq_x_s32,
  BI__builtin_arm_mve_vrshlq_x_s8,
  BI__builtin_arm_mve_vrshlq_x_u16,
  BI__builtin_arm_mve_vrshlq_x_u32,
  BI__builtin_arm_mve_vrshlq_x_u8,
  BI__builtin_arm_mve_vrshrnbq_m_n_s16,
  BI__builtin_arm_mve_vrshrnbq_m_n_s32,
  BI__builtin_arm_mve_vrshrnbq_m_n_u16,
  BI__builtin_arm_mve_vrshrnbq_m_n_u32,
  BI__builtin_arm_mve_vrshrnbq_n_s16,
  BI__builtin_arm_mve_vrshrnbq_n_s32,
  BI__builtin_arm_mve_vrshrnbq_n_u16,
  BI__builtin_arm_mve_vrshrnbq_n_u32,
  BI__builtin_arm_mve_vrshrntq_m_n_s16,
  BI__builtin_arm_mve_vrshrntq_m_n_s32,
  BI__builtin_arm_mve_vrshrntq_m_n_u16,
  BI__builtin_arm_mve_vrshrntq_m_n_u32,
  BI__builtin_arm_mve_vrshrntq_n_s16,
  BI__builtin_arm_mve_vrshrntq_n_s32,
  BI__builtin_arm_mve_vrshrntq_n_u16,
  BI__builtin_arm_mve_vrshrntq_n_u32,
  BI__builtin_arm_mve_vrshrq_m_n_s16,
  BI__builtin_arm_mve_vrshrq_m_n_s32,
  BI__builtin_arm_mve_vrshrq_m_n_s8,
  BI__builtin_arm_mve_vrshrq_m_n_u16,
  BI__builtin_arm_mve_vrshrq_m_n_u32,
  BI__builtin_arm_mve_vrshrq_m_n_u8,
  BI__builtin_arm_mve_vrshrq_n_s16,
  BI__builtin_arm_mve_vrshrq_n_s32,
  BI__builtin_arm_mve_vrshrq_n_s8,
  BI__builtin_arm_mve_vrshrq_n_u16,
  BI__builtin_arm_mve_vrshrq_n_u32,
  BI__builtin_arm_mve_vrshrq_n_u8,
  BI__builtin_arm_mve_vrshrq_x_n_s16,
  BI__builtin_arm_mve_vrshrq_x_n_s32,
  BI__builtin_arm_mve_vrshrq_x_n_s8,
  BI__builtin_arm_mve_vrshrq_x_n_u16,
  BI__builtin_arm_mve_vrshrq_x_n_u32,
  BI__builtin_arm_mve_vrshrq_x_n_u8,
  BI__builtin_arm_mve_vsbciq_m_s32,
  BI__builtin_arm_mve_vsbciq_m_u32,
  BI__builtin_arm_mve_vsbciq_s32,
  BI__builtin_arm_mve_vsbciq_u32,
  BI__builtin_arm_mve_vsbcq_m_s32,
  BI__builtin_arm_mve_vsbcq_m_u32,
  BI__builtin_arm_mve_vsbcq_s32,
  BI__builtin_arm_mve_vsbcq_u32,
  BI__builtin_arm_mve_vsetq_lane_f16,
  BI__builtin_arm_mve_vsetq_lane_f32,
  BI__builtin_arm_mve_vsetq_lane_s16,
  BI__builtin_arm_mve_vsetq_lane_s32,
  BI__builtin_arm_mve_vsetq_lane_s64,
  BI__builtin_arm_mve_vsetq_lane_s8,
  BI__builtin_arm_mve_vsetq_lane_u16,
  BI__builtin_arm_mve_vsetq_lane_u32,
  BI__builtin_arm_mve_vsetq_lane_u64,
  BI__builtin_arm_mve_vsetq_lane_u8,
  BI__builtin_arm_mve_vshlcq_m_s16,
  BI__builtin_arm_mve_vshlcq_m_s32,
  BI__builtin_arm_mve_vshlcq_m_s8,
  BI__builtin_arm_mve_vshlcq_m_u16,
  BI__builtin_arm_mve_vshlcq_m_u32,
  BI__builtin_arm_mve_vshlcq_m_u8,
  BI__builtin_arm_mve_vshlcq_s16,
  BI__builtin_arm_mve_vshlcq_s32,
  BI__builtin_arm_mve_vshlcq_s8,
  BI__builtin_arm_mve_vshlcq_u16,
  BI__builtin_arm_mve_vshlcq_u32,
  BI__builtin_arm_mve_vshlcq_u8,
  BI__builtin_arm_mve_vshllbq_m_n_s16,
  BI__builtin_arm_mve_vshllbq_m_n_s8,
  BI__builtin_arm_mve_vshllbq_m_n_u16,
  BI__builtin_arm_mve_vshllbq_m_n_u8,
  BI__builtin_arm_mve_vshllbq_n_s16,
  BI__builtin_arm_mve_vshllbq_n_s8,
  BI__builtin_arm_mve_vshllbq_n_u16,
  BI__builtin_arm_mve_vshllbq_n_u8,
  BI__builtin_arm_mve_vshllbq_x_n_s16,
  BI__builtin_arm_mve_vshllbq_x_n_s8,
  BI__builtin_arm_mve_vshllbq_x_n_u16,
  BI__builtin_arm_mve_vshllbq_x_n_u8,
  BI__builtin_arm_mve_vshlltq_m_n_s16,
  BI__builtin_arm_mve_vshlltq_m_n_s8,
  BI__builtin_arm_mve_vshlltq_m_n_u16,
  BI__builtin_arm_mve_vshlltq_m_n_u8,
  BI__builtin_arm_mve_vshlltq_n_s16,
  BI__builtin_arm_mve_vshlltq_n_s8,
  BI__builtin_arm_mve_vshlltq_n_u16,
  BI__builtin_arm_mve_vshlltq_n_u8,
  BI__builtin_arm_mve_vshlltq_x_n_s16,
  BI__builtin_arm_mve_vshlltq_x_n_s8,
  BI__builtin_arm_mve_vshlltq_x_n_u16,
  BI__builtin_arm_mve_vshlltq_x_n_u8,
  BI__builtin_arm_mve_vshlq_m_n_s16,
  BI__builtin_arm_mve_vshlq_m_n_s32,
  BI__builtin_arm_mve_vshlq_m_n_s8,
  BI__builtin_arm_mve_vshlq_m_n_u16,
  BI__builtin_arm_mve_vshlq_m_n_u32,
  BI__builtin_arm_mve_vshlq_m_n_u8,
  BI__builtin_arm_mve_vshlq_m_r_s16,
  BI__builtin_arm_mve_vshlq_m_r_s32,
  BI__builtin_arm_mve_vshlq_m_r_s8,
  BI__builtin_arm_mve_vshlq_m_r_u16,
  BI__builtin_arm_mve_vshlq_m_r_u32,
  BI__builtin_arm_mve_vshlq_m_r_u8,
  BI__builtin_arm_mve_vshlq_m_s16,
  BI__builtin_arm_mve_vshlq_m_s32,
  BI__builtin_arm_mve_vshlq_m_s8,
  BI__builtin_arm_mve_vshlq_m_u16,
  BI__builtin_arm_mve_vshlq_m_u32,
  BI__builtin_arm_mve_vshlq_m_u8,
  BI__builtin_arm_mve_vshlq_n_s16,
  BI__builtin_arm_mve_vshlq_n_s32,
  BI__builtin_arm_mve_vshlq_n_s8,
  BI__builtin_arm_mve_vshlq_n_u16,
  BI__builtin_arm_mve_vshlq_n_u32,
  BI__builtin_arm_mve_vshlq_n_u8,
  BI__builtin_arm_mve_vshlq_r_s16,
  BI__builtin_arm_mve_vshlq_r_s32,
  BI__builtin_arm_mve_vshlq_r_s8,
  BI__builtin_arm_mve_vshlq_r_u16,
  BI__builtin_arm_mve_vshlq_r_u32,
  BI__builtin_arm_mve_vshlq_r_u8,
  BI__builtin_arm_mve_vshlq_s16,
  BI__builtin_arm_mve_vshlq_s32,
  BI__builtin_arm_mve_vshlq_s8,
  BI__builtin_arm_mve_vshlq_u16,
  BI__builtin_arm_mve_vshlq_u32,
  BI__builtin_arm_mve_vshlq_u8,
  BI__builtin_arm_mve_vshlq_x_n_s16,
  BI__builtin_arm_mve_vshlq_x_n_s32,
  BI__builtin_arm_mve_vshlq_x_n_s8,
  BI__builtin_arm_mve_vshlq_x_n_u16,
  BI__builtin_arm_mve_vshlq_x_n_u32,
  BI__builtin_arm_mve_vshlq_x_n_u8,
  BI__builtin_arm_mve_vshlq_x_s16,
  BI__builtin_arm_mve_vshlq_x_s32,
  BI__builtin_arm_mve_vshlq_x_s8,
  BI__builtin_arm_mve_vshlq_x_u16,
  BI__builtin_arm_mve_vshlq_x_u32,
  BI__builtin_arm_mve_vshlq_x_u8,
  BI__builtin_arm_mve_vshrnbq_m_n_s16,
  BI__builtin_arm_mve_vshrnbq_m_n_s32,
  BI__builtin_arm_mve_vshrnbq_m_n_u16,
  BI__builtin_arm_mve_vshrnbq_m_n_u32,
  BI__builtin_arm_mve_vshrnbq_n_s16,
  BI__builtin_arm_mve_vshrnbq_n_s32,
  BI__builtin_arm_mve_vshrnbq_n_u16,
  BI__builtin_arm_mve_vshrnbq_n_u32,
  BI__builtin_arm_mve_vshrntq_m_n_s16,
  BI__builtin_arm_mve_vshrntq_m_n_s32,
  BI__builtin_arm_mve_vshrntq_m_n_u16,
  BI__builtin_arm_mve_vshrntq_m_n_u32,
  BI__builtin_arm_mve_vshrntq_n_s16,
  BI__builtin_arm_mve_vshrntq_n_s32,
  BI__builtin_arm_mve_vshrntq_n_u16,
  BI__builtin_arm_mve_vshrntq_n_u32,
  BI__builtin_arm_mve_vshrq_m_n_s16,
  BI__builtin_arm_mve_vshrq_m_n_s32,
  BI__builtin_arm_mve_vshrq_m_n_s8,
  BI__builtin_arm_mve_vshrq_m_n_u16,
  BI__builtin_arm_mve_vshrq_m_n_u32,
  BI__builtin_arm_mve_vshrq_m_n_u8,
  BI__builtin_arm_mve_vshrq_n_s16,
  BI__builtin_arm_mve_vshrq_n_s32,
  BI__builtin_arm_mve_vshrq_n_s8,
  BI__builtin_arm_mve_vshrq_n_u16,
  BI__builtin_arm_mve_vshrq_n_u32,
  BI__builtin_arm_mve_vshrq_n_u8,
  BI__builtin_arm_mve_vshrq_x_n_s16,
  BI__builtin_arm_mve_vshrq_x_n_s32,
  BI__builtin_arm_mve_vshrq_x_n_s8,
  BI__builtin_arm_mve_vshrq_x_n_u16,
  BI__builtin_arm_mve_vshrq_x_n_u32,
  BI__builtin_arm_mve_vshrq_x_n_u8,
  BI__builtin_arm_mve_vsliq_m_n_s16,
  BI__builtin_arm_mve_vsliq_m_n_s32,
  BI__builtin_arm_mve_vsliq_m_n_s8,
  BI__builtin_arm_mve_vsliq_m_n_u16,
  BI__builtin_arm_mve_vsliq_m_n_u32,
  BI__builtin_arm_mve_vsliq_m_n_u8,
  BI__builtin_arm_mve_vsliq_n_s16,
  BI__builtin_arm_mve_vsliq_n_s32,
  BI__builtin_arm_mve_vsliq_n_s8,
  BI__builtin_arm_mve_vsliq_n_u16,
  BI__builtin_arm_mve_vsliq_n_u32,
  BI__builtin_arm_mve_vsliq_n_u8,
  BI__builtin_arm_mve_vsriq_m_n_s16,
  BI__builtin_arm_mve_vsriq_m_n_s32,
  BI__builtin_arm_mve_vsriq_m_n_s8,
  BI__builtin_arm_mve_vsriq_m_n_u16,
  BI__builtin_arm_mve_vsriq_m_n_u32,
  BI__builtin_arm_mve_vsriq_m_n_u8,
  BI__builtin_arm_mve_vsriq_n_s16,
  BI__builtin_arm_mve_vsriq_n_s32,
  BI__builtin_arm_mve_vsriq_n_s8,
  BI__builtin_arm_mve_vsriq_n_u16,
  BI__builtin_arm_mve_vsriq_n_u32,
  BI__builtin_arm_mve_vsriq_n_u8,
  BI__builtin_arm_mve_vst1q_f16,
  BI__builtin_arm_mve_vst1q_f32,
  BI__builtin_arm_mve_vst1q_p_f16,
  BI__builtin_arm_mve_vst1q_p_f32,
  BI__builtin_arm_mve_vst1q_p_s16,
  BI__builtin_arm_mve_vst1q_p_s32,
  BI__builtin_arm_mve_vst1q_p_s8,
  BI__builtin_arm_mve_vst1q_p_u16,
  BI__builtin_arm_mve_vst1q_p_u32,
  BI__builtin_arm_mve_vst1q_p_u8,
  BI__builtin_arm_mve_vst1q_s16,
  BI__builtin_arm_mve_vst1q_s32,
  BI__builtin_arm_mve_vst1q_s8,
  BI__builtin_arm_mve_vst1q_u16,
  BI__builtin_arm_mve_vst1q_u32,
  BI__builtin_arm_mve_vst1q_u8,
  BI__builtin_arm_mve_vst2q_f16,
  BI__builtin_arm_mve_vst2q_f32,
  BI__builtin_arm_mve_vst2q_s16,
  BI__builtin_arm_mve_vst2q_s32,
  BI__builtin_arm_mve_vst2q_s8,
  BI__builtin_arm_mve_vst2q_u16,
  BI__builtin_arm_mve_vst2q_u32,
  BI__builtin_arm_mve_vst2q_u8,
  BI__builtin_arm_mve_vst4q_f16,
  BI__builtin_arm_mve_vst4q_f32,
  BI__builtin_arm_mve_vst4q_s16,
  BI__builtin_arm_mve_vst4q_s32,
  BI__builtin_arm_mve_vst4q_s8,
  BI__builtin_arm_mve_vst4q_u16,
  BI__builtin_arm_mve_vst4q_u32,
  BI__builtin_arm_mve_vst4q_u8,
  BI__builtin_arm_mve_vstrbq_p_s16,
  BI__builtin_arm_mve_vstrbq_p_s32,
  BI__builtin_arm_mve_vstrbq_p_s8,
  BI__builtin_arm_mve_vstrbq_p_u16,
  BI__builtin_arm_mve_vstrbq_p_u32,
  BI__builtin_arm_mve_vstrbq_p_u8,
  BI__builtin_arm_mve_vstrbq_s16,
  BI__builtin_arm_mve_vstrbq_s32,
  BI__builtin_arm_mve_vstrbq_s8,
  BI__builtin_arm_mve_vstrbq_scatter_offset_p_s16,
  BI__builtin_arm_mve_vstrbq_scatter_offset_p_s32,
  BI__builtin_arm_mve_vstrbq_scatter_offset_p_s8,
  BI__builtin_arm_mve_vstrbq_scatter_offset_p_u16,
  BI__builtin_arm_mve_vstrbq_scatter_offset_p_u32,
  BI__builtin_arm_mve_vstrbq_scatter_offset_p_u8,
  BI__builtin_arm_mve_vstrbq_scatter_offset_s16,
  BI__builtin_arm_mve_vstrbq_scatter_offset_s32,
  BI__builtin_arm_mve_vstrbq_scatter_offset_s8,
  BI__builtin_arm_mve_vstrbq_scatter_offset_u16,
  BI__builtin_arm_mve_vstrbq_scatter_offset_u32,
  BI__builtin_arm_mve_vstrbq_scatter_offset_u8,
  BI__builtin_arm_mve_vstrbq_u16,
  BI__builtin_arm_mve_vstrbq_u32,
  BI__builtin_arm_mve_vstrbq_u8,
  BI__builtin_arm_mve_vstrdq_scatter_base_p_s64,
  BI__builtin_arm_mve_vstrdq_scatter_base_p_u64,
  BI__builtin_arm_mve_vstrdq_scatter_base_s64,
  BI__builtin_arm_mve_vstrdq_scatter_base_u64,
  BI__builtin_arm_mve_vstrdq_scatter_base_wb_p_s64,
  BI__builtin_arm_mve_vstrdq_scatter_base_wb_p_u64,
  BI__builtin_arm_mve_vstrdq_scatter_base_wb_s64,
  BI__builtin_arm_mve_vstrdq_scatter_base_wb_u64,
  BI__builtin_arm_mve_vstrdq_scatter_offset_p_s64,
  BI__builtin_arm_mve_vstrdq_scatter_offset_p_u64,
  BI__builtin_arm_mve_vstrdq_scatter_offset_s64,
  BI__builtin_arm_mve_vstrdq_scatter_offset_u64,
  BI__builtin_arm_mve_vstrdq_scatter_shifted_offset_p_s64,
  BI__builtin_arm_mve_vstrdq_scatter_shifted_offset_p_u64,
  BI__builtin_arm_mve_vstrdq_scatter_shifted_offset_s64,
  BI__builtin_arm_mve_vstrdq_scatter_shifted_offset_u64,
  BI__builtin_arm_mve_vstrhq_f16,
  BI__builtin_arm_mve_vstrhq_p_f16,
  BI__builtin_arm_mve_vstrhq_p_s16,
  BI__builtin_arm_mve_vstrhq_p_s32,
  BI__builtin_arm_mve_vstrhq_p_u16,
  BI__builtin_arm_mve_vstrhq_p_u32,
  BI__builtin_arm_mve_vstrhq_s16,
  BI__builtin_arm_mve_vstrhq_s32,
  BI__builtin_arm_mve_vstrhq_scatter_offset_f16,
  BI__builtin_arm_mve_vstrhq_scatter_offset_p_f16,
  BI__builtin_arm_mve_vstrhq_scatter_offset_p_s16,
  BI__builtin_arm_mve_vstrhq_scatter_offset_p_s32,
  BI__builtin_arm_mve_vstrhq_scatter_offset_p_u16,
  BI__builtin_arm_mve_vstrhq_scatter_offset_p_u32,
  BI__builtin_arm_mve_vstrhq_scatter_offset_s16,
  BI__builtin_arm_mve_vstrhq_scatter_offset_s32,
  BI__builtin_arm_mve_vstrhq_scatter_offset_u16,
  BI__builtin_arm_mve_vstrhq_scatter_offset_u32,
  BI__builtin_arm_mve_vstrhq_scatter_shifted_offset_f16,
  BI__builtin_arm_mve_vstrhq_scatter_shifted_offset_p_f16,
  BI__builtin_arm_mve_vstrhq_scatter_shifted_offset_p_s16,
  BI__builtin_arm_mve_vstrhq_scatter_shifted_offset_p_s32,
  BI__builtin_arm_mve_vstrhq_scatter_shifted_offset_p_u16,
  BI__builtin_arm_mve_vstrhq_scatter_shifted_offset_p_u32,
  BI__builtin_arm_mve_vstrhq_scatter_shifted_offset_s16,
  BI__builtin_arm_mve_vstrhq_scatter_shifted_offset_s32,
  BI__builtin_arm_mve_vstrhq_scatter_shifted_offset_u16,
  BI__builtin_arm_mve_vstrhq_scatter_shifted_offset_u32,
  BI__builtin_arm_mve_vstrhq_u16,
  BI__builtin_arm_mve_vstrhq_u32,
  BI__builtin_arm_mve_vstrwq_f32,
  BI__builtin_arm_mve_vstrwq_p_f32,
  BI__builtin_arm_mve_vstrwq_p_s32,
  BI__builtin_arm_mve_vstrwq_p_u32,
  BI__builtin_arm_mve_vstrwq_s32,
  BI__builtin_arm_mve_vstrwq_scatter_base_f32,
  BI__builtin_arm_mve_vstrwq_scatter_base_p_f32,
  BI__builtin_arm_mve_vstrwq_scatter_base_p_s32,
  BI__builtin_arm_mve_vstrwq_scatter_base_p_u32,
  BI__builtin_arm_mve_vstrwq_scatter_base_s32,
  BI__builtin_arm_mve_vstrwq_scatter_base_u32,
  BI__builtin_arm_mve_vstrwq_scatter_base_wb_f32,
  BI__builtin_arm_mve_vstrwq_scatter_base_wb_p_f32,
  BI__builtin_arm_mve_vstrwq_scatter_base_wb_p_s32,
  BI__builtin_arm_mve_vstrwq_scatter_base_wb_p_u32,
  BI__builtin_arm_mve_vstrwq_scatter_base_wb_s32,
  BI__builtin_arm_mve_vstrwq_scatter_base_wb_u32,
  BI__builtin_arm_mve_vstrwq_scatter_offset_f32,
  BI__builtin_arm_mve_vstrwq_scatter_offset_p_f32,
  BI__builtin_arm_mve_vstrwq_scatter_offset_p_s32,
  BI__builtin_arm_mve_vstrwq_scatter_offset_p_u32,
  BI__builtin_arm_mve_vstrwq_scatter_offset_s32,
  BI__builtin_arm_mve_vstrwq_scatter_offset_u32,
  BI__builtin_arm_mve_vstrwq_scatter_shifted_offset_f32,
  BI__builtin_arm_mve_vstrwq_scatter_shifted_offset_p_f32,
  BI__builtin_arm_mve_vstrwq_scatter_shifted_offset_p_s32,
  BI__builtin_arm_mve_vstrwq_scatter_shifted_offset_p_u32,
  BI__builtin_arm_mve_vstrwq_scatter_shifted_offset_s32,
  BI__builtin_arm_mve_vstrwq_scatter_shifted_offset_u32,
  BI__builtin_arm_mve_vstrwq_u32,
  BI__builtin_arm_mve_vsubq_f16,
  BI__builtin_arm_mve_vsubq_f32,
  BI__builtin_arm_mve_vsubq_m_f16,
  BI__builtin_arm_mve_vsubq_m_f32,
  BI__builtin_arm_mve_vsubq_m_n_f16,
  BI__builtin_arm_mve_vsubq_m_n_f32,
  BI__builtin_arm_mve_vsubq_m_n_s16,
  BI__builtin_arm_mve_vsubq_m_n_s32,
  BI__builtin_arm_mve_vsubq_m_n_s8,
  BI__builtin_arm_mve_vsubq_m_n_u16,
  BI__builtin_arm_mve_vsubq_m_n_u32,
  BI__builtin_arm_mve_vsubq_m_n_u8,
  BI__builtin_arm_mve_vsubq_m_s16,
  BI__builtin_arm_mve_vsubq_m_s32,
  BI__builtin_arm_mve_vsubq_m_s8,
  BI__builtin_arm_mve_vsubq_m_u16,
  BI__builtin_arm_mve_vsubq_m_u32,
  BI__builtin_arm_mve_vsubq_m_u8,
  BI__builtin_arm_mve_vsubq_n_f16,
  BI__builtin_arm_mve_vsubq_n_f32,
  BI__builtin_arm_mve_vsubq_n_s16,
  BI__builtin_arm_mve_vsubq_n_s32,
  BI__builtin_arm_mve_vsubq_n_s8,
  BI__builtin_arm_mve_vsubq_n_u16,
  BI__builtin_arm_mve_vsubq_n_u32,
  BI__builtin_arm_mve_vsubq_n_u8,
  BI__builtin_arm_mve_vsubq_s16,
  BI__builtin_arm_mve_vsubq_s32,
  BI__builtin_arm_mve_vsubq_s8,
  BI__builtin_arm_mve_vsubq_u16,
  BI__builtin_arm_mve_vsubq_u32,
  BI__builtin_arm_mve_vsubq_u8,
  BI__builtin_arm_mve_vsubq_x_f16,
  BI__builtin_arm_mve_vsubq_x_f32,
  BI__builtin_arm_mve_vsubq_x_n_f16,
  BI__builtin_arm_mve_vsubq_x_n_f32,
  BI__builtin_arm_mve_vsubq_x_n_s16,
  BI__builtin_arm_mve_vsubq_x_n_s32,
  BI__builtin_arm_mve_vsubq_x_n_s8,
  BI__builtin_arm_mve_vsubq_x_n_u16,
  BI__builtin_arm_mve_vsubq_x_n_u32,
  BI__builtin_arm_mve_vsubq_x_n_u8,
  BI__builtin_arm_mve_vsubq_x_s16,
  BI__builtin_arm_mve_vsubq_x_s32,
  BI__builtin_arm_mve_vsubq_x_s8,
  BI__builtin_arm_mve_vsubq_x_u16,
  BI__builtin_arm_mve_vsubq_x_u32,
  BI__builtin_arm_mve_vsubq_x_u8,
  BI__builtin_arm_mve_vuninitializedq_f16,
  BI__builtin_arm_mve_vuninitializedq_f32,
  BI__builtin_arm_mve_vuninitializedq_polymorphic_f16,
  BI__builtin_arm_mve_vuninitializedq_polymorphic_f32,
  BI__builtin_arm_mve_vuninitializedq_polymorphic_s16,
  BI__builtin_arm_mve_vuninitializedq_polymorphic_s32,
  BI__builtin_arm_mve_vuninitializedq_polymorphic_s64,
  BI__builtin_arm_mve_vuninitializedq_polymorphic_s8,
  BI__builtin_arm_mve_vuninitializedq_polymorphic_u16,
  BI__builtin_arm_mve_vuninitializedq_polymorphic_u32,
  BI__builtin_arm_mve_vuninitializedq_polymorphic_u64,
  BI__builtin_arm_mve_vuninitializedq_polymorphic_u8,
  BI__builtin_arm_mve_vuninitializedq_s16,
  BI__builtin_arm_mve_vuninitializedq_s32,
  BI__builtin_arm_mve_vuninitializedq_s64,
  BI__builtin_arm_mve_vuninitializedq_s8,
  BI__builtin_arm_mve_vuninitializedq_u16,
  BI__builtin_arm_mve_vuninitializedq_u32,
  BI__builtin_arm_mve_vuninitializedq_u64,
  BI__builtin_arm_mve_vuninitializedq_u8,
  BI__builtin_arm_mve_vabavq,
  BI__builtin_arm_mve_vabavq_p,
  BI__builtin_arm_mve_vabdq,
  BI__builtin_arm_mve_vabdq_m,
  BI__builtin_arm_mve_vabdq_x,
  BI__builtin_arm_mve_vabsq,
  BI__builtin_arm_mve_vabsq_m,
  BI__builtin_arm_mve_vabsq_x,
  BI__builtin_arm_mve_vadciq,
  BI__builtin_arm_mve_vadciq_m,
  BI__builtin_arm_mve_vadcq,
  BI__builtin_arm_mve_vadcq_m,
  BI__builtin_arm_mve_vaddlvaq,
  BI__builtin_arm_mve_vaddlvaq_p,
  BI__builtin_arm_mve_vaddlvq,
  BI__builtin_arm_mve_vaddlvq_p,
  BI__builtin_arm_mve_vaddq,
  BI__builtin_arm_mve_vaddq_m,
  BI__builtin_arm_mve_vaddq_x,
  BI__builtin_arm_mve_vaddvaq,
  BI__builtin_arm_mve_vaddvaq_p,
  BI__builtin_arm_mve_vaddvq,
  BI__builtin_arm_mve_vaddvq_p,
  BI__builtin_arm_mve_vandq,
  BI__builtin_arm_mve_vandq_m,
  BI__builtin_arm_mve_vandq_x,
  BI__builtin_arm_mve_vbicq,
  BI__builtin_arm_mve_vbicq_m,
  BI__builtin_arm_mve_vbicq_m_n,
  BI__builtin_arm_mve_vbicq_x,
  BI__builtin_arm_mve_vbrsrq,
  BI__builtin_arm_mve_vbrsrq_m,
  BI__builtin_arm_mve_vbrsrq_x,
  BI__builtin_arm_mve_vcaddq_rot270,
  BI__builtin_arm_mve_vcaddq_rot270_m,
  BI__builtin_arm_mve_vcaddq_rot270_x,
  BI__builtin_arm_mve_vcaddq_rot90,
  BI__builtin_arm_mve_vcaddq_rot90_m,
  BI__builtin_arm_mve_vcaddq_rot90_x,
  BI__builtin_arm_mve_vclsq,
  BI__builtin_arm_mve_vclsq_m,
  BI__builtin_arm_mve_vclsq_x,
  BI__builtin_arm_mve_vclzq,
  BI__builtin_arm_mve_vclzq_m,
  BI__builtin_arm_mve_vclzq_x,
  BI__builtin_arm_mve_vcmlaq,
  BI__builtin_arm_mve_vcmlaq_m,
  BI__builtin_arm_mve_vcmlaq_rot180,
  BI__builtin_arm_mve_vcmlaq_rot180_m,
  BI__builtin_arm_mve_vcmlaq_rot270,
  BI__builtin_arm_mve_vcmlaq_rot270_m,
  BI__builtin_arm_mve_vcmlaq_rot90,
  BI__builtin_arm_mve_vcmlaq_rot90_m,
  BI__builtin_arm_mve_vcmpcsq,
  BI__builtin_arm_mve_vcmpcsq_m,
  BI__builtin_arm_mve_vcmpeqq,
  BI__builtin_arm_mve_vcmpeqq_m,
  BI__builtin_arm_mve_vcmpgeq,
  BI__builtin_arm_mve_vcmpgeq_m,
  BI__builtin_arm_mve_vcmpgtq,
  BI__builtin_arm_mve_vcmpgtq_m,
  BI__builtin_arm_mve_vcmphiq,
  BI__builtin_arm_mve_vcmphiq_m,
  BI__builtin_arm_mve_vcmpleq,
  BI__builtin_arm_mve_vcmpleq_m,
  BI__builtin_arm_mve_vcmpltq,
  BI__builtin_arm_mve_vcmpltq_m,
  BI__builtin_arm_mve_vcmpneq,
  BI__builtin_arm_mve_vcmpneq_m,
  BI__builtin_arm_mve_vcmulq,
  BI__builtin_arm_mve_vcmulq_m,
  BI__builtin_arm_mve_vcmulq_rot180,
  BI__builtin_arm_mve_vcmulq_rot180_m,
  BI__builtin_arm_mve_vcmulq_rot180_x,
  BI__builtin_arm_mve_vcmulq_rot270,
  BI__builtin_arm_mve_vcmulq_rot270_m,
  BI__builtin_arm_mve_vcmulq_rot270_x,
  BI__builtin_arm_mve_vcmulq_rot90,
  BI__builtin_arm_mve_vcmulq_rot90_m,
  BI__builtin_arm_mve_vcmulq_rot90_x,
  BI__builtin_arm_mve_vcmulq_x,
  BI__builtin_arm_mve_vcvtaq_m,
  BI__builtin_arm_mve_vcvtmq_m,
  BI__builtin_arm_mve_vcvtnq_m,
  BI__builtin_arm_mve_vcvtpq_m,
  BI__builtin_arm_mve_vcvtq,
  BI__builtin_arm_mve_vcvtq_m,
  BI__builtin_arm_mve_vcvtq_m_n,
  BI__builtin_arm_mve_vcvtq_n,
  BI__builtin_arm_mve_vcvtq_x,
  BI__builtin_arm_mve_vcvtq_x_n,
  BI__builtin_arm_mve_vddupq_m,
  BI__builtin_arm_mve_vddupq_u16,
  BI__builtin_arm_mve_vddupq_u32,
  BI__builtin_arm_mve_vddupq_u8,
  BI__builtin_arm_mve_vddupq_x_u16,
  BI__builtin_arm_mve_vddupq_x_u32,
  BI__builtin_arm_mve_vddupq_x_u8,
  BI__builtin_arm_mve_vdupq_m,
  BI__builtin_arm_mve_vdwdupq_m,
  BI__builtin_arm_mve_vdwdupq_u16,
  BI__builtin_arm_mve_vdwdupq_u32,
  BI__builtin_arm_mve_vdwdupq_u8,
  BI__builtin_arm_mve_vdwdupq_x_u16,
  BI__builtin_arm_mve_vdwdupq_x_u32,
  BI__builtin_arm_mve_vdwdupq_x_u8,
  BI__builtin_arm_mve_veorq,
  BI__builtin_arm_mve_veorq_m,
  BI__builtin_arm_mve_veorq_x,
  BI__builtin_arm_mve_vfmaq,
  BI__builtin_arm_mve_vfmaq_m,
  BI__builtin_arm_mve_vfmasq,
  BI__builtin_arm_mve_vfmasq_m,
  BI__builtin_arm_mve_vfmsq,
  BI__builtin_arm_mve_vfmsq_m,
  BI__builtin_arm_mve_vgetq_lane,
  BI__builtin_arm_mve_vhaddq,
  BI__builtin_arm_mve_vhaddq_m,
  BI__builtin_arm_mve_vhaddq_x,
  BI__builtin_arm_mve_vhcaddq_rot270,
  BI__builtin_arm_mve_vhcaddq_rot270_m,
  BI__builtin_arm_mve_vhcaddq_rot270_x,
  BI__builtin_arm_mve_vhcaddq_rot90,
  BI__builtin_arm_mve_vhcaddq_rot90_m,
  BI__builtin_arm_mve_vhcaddq_rot90_x,
  BI__builtin_arm_mve_vhsubq,
  BI__builtin_arm_mve_vhsubq_m,
  BI__builtin_arm_mve_vhsubq_x,
  BI__builtin_arm_mve_vidupq_m,
  BI__builtin_arm_mve_vidupq_u16,
  BI__builtin_arm_mve_vidupq_u32,
  BI__builtin_arm_mve_vidupq_u8,
  BI__builtin_arm_mve_vidupq_x_u16,
  BI__builtin_arm_mve_vidupq_x_u32,
  BI__builtin_arm_mve_vidupq_x_u8,
  BI__builtin_arm_mve_viwdupq_m,
  BI__builtin_arm_mve_viwdupq_u16,
  BI__builtin_arm_mve_viwdupq_u32,
  BI__builtin_arm_mve_viwdupq_u8,
  BI__builtin_arm_mve_viwdupq_x_u16,
  BI__builtin_arm_mve_viwdupq_x_u32,
  BI__builtin_arm_mve_viwdupq_x_u8,
  BI__builtin_arm_mve_vld1q,
  BI__builtin_arm_mve_vld1q_z,
  BI__builtin_arm_mve_vld2q,
  BI__builtin_arm_mve_vld4q,
  BI__builtin_arm_mve_vldrbq_gather_offset,
  BI__builtin_arm_mve_vldrbq_gather_offset_z,
  BI__builtin_arm_mve_vldrdq_gather_offset,
  BI__builtin_arm_mve_vldrdq_gather_offset_z,
  BI__builtin_arm_mve_vldrdq_gather_shifted_offset,
  BI__builtin_arm_mve_vldrdq_gather_shifted_offset_z,
  BI__builtin_arm_mve_vldrhq_gather_offset,
  BI__builtin_arm_mve_vldrhq_gather_offset_z,
  BI__builtin_arm_mve_vldrhq_gather_shifted_offset,
  BI__builtin_arm_mve_vldrhq_gather_shifted_offset_z,
  BI__builtin_arm_mve_vldrwq_gather_offset,
  BI__builtin_arm_mve_vldrwq_gather_offset_z,
  BI__builtin_arm_mve_vldrwq_gather_shifted_offset,
  BI__builtin_arm_mve_vldrwq_gather_shifted_offset_z,
  BI__builtin_arm_mve_vmaxaq,
  BI__builtin_arm_mve_vmaxaq_m,
  BI__builtin_arm_mve_vmaxavq,
  BI__builtin_arm_mve_vmaxavq_p,
  BI__builtin_arm_mve_vmaxnmaq,
  BI__builtin_arm_mve_vmaxnmaq_m,
  BI__builtin_arm_mve_vmaxnmavq,
  BI__builtin_arm_mve_vmaxnmavq_p,
  BI__builtin_arm_mve_vmaxnmq,
  BI__builtin_arm_mve_vmaxnmq_m,
  BI__builtin_arm_mve_vmaxnmq_x,
  BI__builtin_arm_mve_vmaxnmvq,
  BI__builtin_arm_mve_vmaxnmvq_p,
  BI__builtin_arm_mve_vmaxq,
  BI__builtin_arm_mve_vmaxq_m,
  BI__builtin_arm_mve_vmaxq_x,
  BI__builtin_arm_mve_vmaxvq,
  BI__builtin_arm_mve_vmaxvq_p,
  BI__builtin_arm_mve_vminaq,
  BI__builtin_arm_mve_vminaq_m,
  BI__builtin_arm_mve_vminavq,
  BI__builtin_arm_mve_vminavq_p,
  BI__builtin_arm_mve_vminnmaq,
  BI__builtin_arm_mve_vminnmaq_m,
  BI__builtin_arm_mve_vminnmavq,
  BI__builtin_arm_mve_vminnmavq_p,
  BI__builtin_arm_mve_vminnmq,
  BI__builtin_arm_mve_vminnmq_m,
  BI__builtin_arm_mve_vminnmq_x,
  BI__builtin_arm_mve_vminnmvq,
  BI__builtin_arm_mve_vminnmvq_p,
  BI__builtin_arm_mve_vminq,
  BI__builtin_arm_mve_vminq_m,
  BI__builtin_arm_mve_vminq_x,
  BI__builtin_arm_mve_vminvq,
  BI__builtin_arm_mve_vminvq_p,
  BI__builtin_arm_mve_vmladavaq,
  BI__builtin_arm_mve_vmladavaq_p,
  BI__builtin_arm_mve_vmladavaxq,
  BI__builtin_arm_mve_vmladavaxq_p,
  BI__builtin_arm_mve_vmladavq,
  BI__builtin_arm_mve_vmladavq_p,
  BI__builtin_arm_mve_vmladavxq,
  BI__builtin_arm_mve_vmladavxq_p,
  BI__builtin_arm_mve_vmlaldavaq,
  BI__builtin_arm_mve_vmlaldavaq_p,
  BI__builtin_arm_mve_vmlaldavaxq,
  BI__builtin_arm_mve_vmlaldavaxq_p,
  BI__builtin_arm_mve_vmlaldavq,
  BI__builtin_arm_mve_vmlaldavq_p,
  BI__builtin_arm_mve_vmlaldavxq,
  BI__builtin_arm_mve_vmlaldavxq_p,
  BI__builtin_arm_mve_vmlaq,
  BI__builtin_arm_mve_vmlaq_m,
  BI__builtin_arm_mve_vmlasq,
  BI__builtin_arm_mve_vmlasq_m,
  BI__builtin_arm_mve_vmlsdavaq,
  BI__builtin_arm_mve_vmlsdavaq_p,
  BI__builtin_arm_mve_vmlsdavaxq,
  BI__builtin_arm_mve_vmlsdavaxq_p,
  BI__builtin_arm_mve_vmlsdavq,
  BI__builtin_arm_mve_vmlsdavq_p,
  BI__builtin_arm_mve_vmlsdavxq,
  BI__builtin_arm_mve_vmlsdavxq_p,
  BI__builtin_arm_mve_vmlsldavaq,
  BI__builtin_arm_mve_vmlsldavaq_p,
  BI__builtin_arm_mve_vmlsldavaxq,
  BI__builtin_arm_mve_vmlsldavaxq_p,
  BI__builtin_arm_mve_vmlsldavq,
  BI__builtin_arm_mve_vmlsldavq_p,
  BI__builtin_arm_mve_vmlsldavxq,
  BI__builtin_arm_mve_vmlsldavxq_p,
  BI__builtin_arm_mve_vmovlbq,
  BI__builtin_arm_mve_vmovlbq_m,
  BI__builtin_arm_mve_vmovlbq_x,
  BI__builtin_arm_mve_vmovltq,
  BI__builtin_arm_mve_vmovltq_m,
  BI__builtin_arm_mve_vmovltq_x,
  BI__builtin_arm_mve_vmovnbq,
  BI__builtin_arm_mve_vmovnbq_m,
  BI__builtin_arm_mve_vmovntq,
  BI__builtin_arm_mve_vmovntq_m,
  BI__builtin_arm_mve_vmulhq,
  BI__builtin_arm_mve_vmulhq_m,
  BI__builtin_arm_mve_vmulhq_x,
  BI__builtin_arm_mve_vmullbq_int,
  BI__builtin_arm_mve_vmullbq_int_m,
  BI__builtin_arm_mve_vmullbq_int_x,
  BI__builtin_arm_mve_vmullbq_poly,
  BI__builtin_arm_mve_vmullbq_poly_m,
  BI__builtin_arm_mve_vmullbq_poly_x,
  BI__builtin_arm_mve_vmulltq_int,
  BI__builtin_arm_mve_vmulltq_int_m,
  BI__builtin_arm_mve_vmulltq_int_x,
  BI__builtin_arm_mve_vmulltq_poly,
  BI__builtin_arm_mve_vmulltq_poly_m,
  BI__builtin_arm_mve_vmulltq_poly_x,
  BI__builtin_arm_mve_vmulq,
  BI__builtin_arm_mve_vmulq_m,
  BI__builtin_arm_mve_vmulq_x,
  BI__builtin_arm_mve_vmvnq,
  BI__builtin_arm_mve_vmvnq_m,
  BI__builtin_arm_mve_vmvnq_x,
  BI__builtin_arm_mve_vnegq,
  BI__builtin_arm_mve_vnegq_m,
  BI__builtin_arm_mve_vnegq_x,
  BI__builtin_arm_mve_vornq,
  BI__builtin_arm_mve_vornq_m,
  BI__builtin_arm_mve_vornq_x,
  BI__builtin_arm_mve_vorrq,
  BI__builtin_arm_mve_vorrq_m,
  BI__builtin_arm_mve_vorrq_m_n,
  BI__builtin_arm_mve_vorrq_x,
  BI__builtin_arm_mve_vpselq,
  BI__builtin_arm_mve_vqabsq,
  BI__builtin_arm_mve_vqabsq_m,
  BI__builtin_arm_mve_vqaddq,
  BI__builtin_arm_mve_vqaddq_m,
  BI__builtin_arm_mve_vqdmladhq,
  BI__builtin_arm_mve_vqdmladhq_m,
  BI__builtin_arm_mve_vqdmladhxq,
  BI__builtin_arm_mve_vqdmladhxq_m,
  BI__builtin_arm_mve_vqdmlahq,
  BI__builtin_arm_mve_vqdmlahq_m,
  BI__builtin_arm_mve_vqdmlashq,
  BI__builtin_arm_mve_vqdmlashq_m,
  BI__builtin_arm_mve_vqdmlsdhq,
  BI__builtin_arm_mve_vqdmlsdhq_m,
  BI__builtin_arm_mve_vqdmlsdhxq,
  BI__builtin_arm_mve_vqdmlsdhxq_m,
  BI__builtin_arm_mve_vqdmulhq,
  BI__builtin_arm_mve_vqdmulhq_m,
  BI__builtin_arm_mve_vqdmullbq,
  BI__builtin_arm_mve_vqdmullbq_m,
  BI__builtin_arm_mve_vqdmulltq,
  BI__builtin_arm_mve_vqdmulltq_m,
  BI__builtin_arm_mve_vqmovnbq,
  BI__builtin_arm_mve_vqmovnbq_m,
  BI__builtin_arm_mve_vqmovntq,
  BI__builtin_arm_mve_vqmovntq_m,
  BI__builtin_arm_mve_vqmovunbq,
  BI__builtin_arm_mve_vqmovunbq_m,
  BI__builtin_arm_mve_vqmovuntq,
  BI__builtin_arm_mve_vqmovuntq_m,
  BI__builtin_arm_mve_vqnegq,
  BI__builtin_arm_mve_vqnegq_m,
  BI__builtin_arm_mve_vqrdmladhq,
  BI__builtin_arm_mve_vqrdmladhq_m,
  BI__builtin_arm_mve_vqrdmladhxq,
  BI__builtin_arm_mve_vqrdmladhxq_m,
  BI__builtin_arm_mve_vqrdmlahq,
  BI__builtin_arm_mve_vqrdmlahq_m,
  BI__builtin_arm_mve_vqrdmlashq,
  BI__builtin_arm_mve_vqrdmlashq_m,
  BI__builtin_arm_mve_vqrdmlsdhq,
  BI__builtin_arm_mve_vqrdmlsdhq_m,
  BI__builtin_arm_mve_vqrdmlsdhxq,
  BI__builtin_arm_mve_vqrdmlsdhxq_m,
  BI__builtin_arm_mve_vqrdmulhq,
  BI__builtin_arm_mve_vqrdmulhq_m,
  BI__builtin_arm_mve_vqrshlq,
  BI__builtin_arm_mve_vqrshlq_m,
  BI__builtin_arm_mve_vqrshlq_m_n,
  BI__builtin_arm_mve_vqrshrnbq,
  BI__builtin_arm_mve_vqrshrnbq_m,
  BI__builtin_arm_mve_vqrshrntq,
  BI__builtin_arm_mve_vqrshrntq_m,
  BI__builtin_arm_mve_vqrshrunbq,
  BI__builtin_arm_mve_vqrshrunbq_m,
  BI__builtin_arm_mve_vqrshruntq,
  BI__builtin_arm_mve_vqrshruntq_m,
  BI__builtin_arm_mve_vqshlq,
  BI__builtin_arm_mve_vqshlq_m,
  BI__builtin_arm_mve_vqshlq_m_n,
  BI__builtin_arm_mve_vqshlq_m_r,
  BI__builtin_arm_mve_vqshlq_n,
  BI__builtin_arm_mve_vqshlq_r,
  BI__builtin_arm_mve_vqshluq,
  BI__builtin_arm_mve_vqshluq_m,
  BI__builtin_arm_mve_vqshrnbq,
  BI__builtin_arm_mve_vqshrnbq_m,
  BI__builtin_arm_mve_vqshrntq,
  BI__builtin_arm_mve_vqshrntq_m,
  BI__builtin_arm_mve_vqshrunbq,
  BI__builtin_arm_mve_vqshrunbq_m,
  BI__builtin_arm_mve_vqshruntq,
  BI__builtin_arm_mve_vqshruntq_m,
  BI__builtin_arm_mve_vqsubq,
  BI__builtin_arm_mve_vqsubq_m,
  BI__builtin_arm_mve_vreinterpretq_f16,
  BI__builtin_arm_mve_vreinterpretq_f32,
  BI__builtin_arm_mve_vreinterpretq_s16,
  BI__builtin_arm_mve_vreinterpretq_s32,
  BI__builtin_arm_mve_vreinterpretq_s64,
  BI__builtin_arm_mve_vreinterpretq_s8,
  BI__builtin_arm_mve_vreinterpretq_u16,
  BI__builtin_arm_mve_vreinterpretq_u32,
  BI__builtin_arm_mve_vreinterpretq_u64,
  BI__builtin_arm_mve_vreinterpretq_u8,
  BI__builtin_arm_mve_vrev16q,
  BI__builtin_arm_mve_vrev16q_m,
  BI__builtin_arm_mve_vrev16q_x,
  BI__builtin_arm_mve_vrev32q,
  BI__builtin_arm_mve_vrev32q_m,
  BI__builtin_arm_mve_vrev32q_x,
  BI__builtin_arm_mve_vrev64q,
  BI__builtin_arm_mve_vrev64q_m,
  BI__builtin_arm_mve_vrev64q_x,
  BI__builtin_arm_mve_vrhaddq,
  BI__builtin_arm_mve_vrhaddq_m,
  BI__builtin_arm_mve_vrhaddq_x,
  BI__builtin_arm_mve_vrmlaldavhaq,
  BI__builtin_arm_mve_vrmlaldavhaq_p,
  BI__builtin_arm_mve_vrmlaldavhaxq,
  BI__builtin_arm_mve_vrmlaldavhaxq_p,
  BI__builtin_arm_mve_vrmlaldavhq,
  BI__builtin_arm_mve_vrmlaldavhq_p,
  BI__builtin_arm_mve_vrmlaldavhxq,
  BI__builtin_arm_mve_vrmlaldavhxq_p,
  BI__builtin_arm_mve_vrmlsldavhaq,
  BI__builtin_arm_mve_vrmlsldavhaq_p,
  BI__builtin_arm_mve_vrmlsldavhaxq,
  BI__builtin_arm_mve_vrmlsldavhaxq_p,
  BI__builtin_arm_mve_vrmlsldavhq,
  BI__builtin_arm_mve_vrmlsldavhq_p,
  BI__builtin_arm_mve_vrmlsldavhxq,
  BI__builtin_arm_mve_vrmlsldavhxq_p,
  BI__builtin_arm_mve_vrmulhq,
  BI__builtin_arm_mve_vrmulhq_m,
  BI__builtin_arm_mve_vrmulhq_x,
  BI__builtin_arm_mve_vrndaq,
  BI__builtin_arm_mve_vrndaq_m,
  BI__builtin_arm_mve_vrndaq_x,
  BI__builtin_arm_mve_vrndmq,
  BI__builtin_arm_mve_vrndmq_m,
  BI__builtin_arm_mve_vrndmq_x,
  BI__builtin_arm_mve_vrndnq,
  BI__builtin_arm_mve_vrndnq_m,
  BI__builtin_arm_mve_vrndnq_x,
  BI__builtin_arm_mve_vrndpq,
  BI__builtin_arm_mve_vrndpq_m,
  BI__builtin_arm_mve_vrndpq_x,
  BI__builtin_arm_mve_vrndq,
  BI__builtin_arm_mve_vrndq_m,
  BI__builtin_arm_mve_vrndq_x,
  BI__builtin_arm_mve_vrndxq,
  BI__builtin_arm_mve_vrndxq_m,
  BI__builtin_arm_mve_vrndxq_x,
  BI__builtin_arm_mve_vrshlq,
  BI__builtin_arm_mve_vrshlq_m,
  BI__builtin_arm_mve_vrshlq_m_n,
  BI__builtin_arm_mve_vrshlq_x,
  BI__builtin_arm_mve_vrshrnbq,
  BI__builtin_arm_mve_vrshrnbq_m,
  BI__builtin_arm_mve_vrshrntq,
  BI__builtin_arm_mve_vrshrntq_m,
  BI__builtin_arm_mve_vrshrq,
  BI__builtin_arm_mve_vrshrq_m,
  BI__builtin_arm_mve_vrshrq_x,
  BI__builtin_arm_mve_vsbciq,
  BI__builtin_arm_mve_vsbciq_m,
  BI__builtin_arm_mve_vsbcq,
  BI__builtin_arm_mve_vsbcq_m,
  BI__builtin_arm_mve_vsetq_lane,
  BI__builtin_arm_mve_vshlcq,
  BI__builtin_arm_mve_vshlcq_m,
  BI__builtin_arm_mve_vshllbq,
  BI__builtin_arm_mve_vshllbq_m,
  BI__builtin_arm_mve_vshllbq_x,
  BI__builtin_arm_mve_vshlltq,
  BI__builtin_arm_mve_vshlltq_m,
  BI__builtin_arm_mve_vshlltq_x,
  BI__builtin_arm_mve_vshlq,
  BI__builtin_arm_mve_vshlq_m,
  BI__builtin_arm_mve_vshlq_m_n,
  BI__builtin_arm_mve_vshlq_m_r,
  BI__builtin_arm_mve_vshlq_n,
  BI__builtin_arm_mve_vshlq_r,
  BI__builtin_arm_mve_vshlq_x,
  BI__builtin_arm_mve_vshlq_x_n,
  BI__builtin_arm_mve_vshrnbq,
  BI__builtin_arm_mve_vshrnbq_m,
  BI__builtin_arm_mve_vshrntq,
  BI__builtin_arm_mve_vshrntq_m,
  BI__builtin_arm_mve_vshrq,
  BI__builtin_arm_mve_vshrq_m,
  BI__builtin_arm_mve_vshrq_x,
  BI__builtin_arm_mve_vsliq,
  BI__builtin_arm_mve_vsliq_m,
  BI__builtin_arm_mve_vsriq,
  BI__builtin_arm_mve_vsriq_m,
  BI__builtin_arm_mve_vst1q,
  BI__builtin_arm_mve_vst1q_p,
  BI__builtin_arm_mve_vst2q,
  BI__builtin_arm_mve_vst4q,
  BI__builtin_arm_mve_vstrbq,
  BI__builtin_arm_mve_vstrbq_p,
  BI__builtin_arm_mve_vstrbq_scatter_offset,
  BI__builtin_arm_mve_vstrbq_scatter_offset_p,
  BI__builtin_arm_mve_vstrdq_scatter_base,
  BI__builtin_arm_mve_vstrdq_scatter_base_p,
  BI__builtin_arm_mve_vstrdq_scatter_base_wb,
  BI__builtin_arm_mve_vstrdq_scatter_base_wb_p,
  BI__builtin_arm_mve_vstrdq_scatter_offset,
  BI__builtin_arm_mve_vstrdq_scatter_offset_p,
  BI__builtin_arm_mve_vstrdq_scatter_shifted_offset,
  BI__builtin_arm_mve_vstrdq_scatter_shifted_offset_p,
  BI__builtin_arm_mve_vstrhq,
  BI__builtin_arm_mve_vstrhq_p,
  BI__builtin_arm_mve_vstrhq_scatter_offset,
  BI__builtin_arm_mve_vstrhq_scatter_offset_p,
  BI__builtin_arm_mve_vstrhq_scatter_shifted_offset,
  BI__builtin_arm_mve_vstrhq_scatter_shifted_offset_p,
  BI__builtin_arm_mve_vstrwq,
  BI__builtin_arm_mve_vstrwq_p,
  BI__builtin_arm_mve_vstrwq_scatter_base,
  BI__builtin_arm_mve_vstrwq_scatter_base_p,
  BI__builtin_arm_mve_vstrwq_scatter_base_wb,
  BI__builtin_arm_mve_vstrwq_scatter_base_wb_p,
  BI__builtin_arm_mve_vstrwq_scatter_offset,
  BI__builtin_arm_mve_vstrwq_scatter_offset_p,
  BI__builtin_arm_mve_vstrwq_scatter_shifted_offset,
  BI__builtin_arm_mve_vstrwq_scatter_shifted_offset_p,
  BI__builtin_arm_mve_vsubq,
  BI__builtin_arm_mve_vsubq_m,
  BI__builtin_arm_mve_vsubq_x,
  BI__builtin_arm_mve_vuninitializedq,
#endif // GET_MVE_BUILTIN_ENUMERATORS

#ifdef GET_MVE_BUILTIN_STR_TABLE

#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Woverlength-strings"
#endif
static constexpr char BuiltinStringsStorage[] = 
  "\0"
  "n\0"
  "nt\0"
  "ntu\0"
  "vi.\0"
  "asrl\0"
  "lsll\0"
  "sqrshr\0"
  "sqrshrl\0"
  "sqrshrl_sat48\0"
  "sqshl\0"
  "sqshll\0"
  "srshr\0"
  "srshrl\0"
  "uqrshl\0"
  "uqrshll\0"
  "uqrshll_sat48\0"
  "uqshl\0"
  "uqshll\0"
  "urshr\0"
  "urshrl\0"
  "vabavq_p_s16\0"
  "vabavq_p_s32\0"
  "vabavq_p_s8\0"
  "vabavq_p_u16\0"
  "vabavq_p_u32\0"
  "vabavq_p_u8\0"
  "vabavq_s16\0"
  "vabavq_s32\0"
  "vabavq_s8\0"
  "vabavq_u16\0"
  "vabavq_u32\0"
  "vabavq_u8\0"
  "vabdq_f16\0"
  "vabdq_f32\0"
  "vabdq_m_f16\0"
  "vabdq_m_f32\0"
  "vabdq_m_s16\0"
  "vabdq_m_s32\0"
  "vabdq_m_s8\0"
  "vabdq_m_u16\0"
  "vabdq_m_u32\0"
  "vabdq_m_u8\0"
  "vabdq_s16\0"
  "vabdq_s32\0"
  "vabdq_s8\0"
  "vabdq_u16\0"
  "vabdq_u32\0"
  "vabdq_u8\0"
  "vabdq_x_f16\0"
  "vabdq_x_f32\0"
  "vabdq_x_s16\0"
  "vabdq_x_s32\0"
  "vabdq_x_s8\0"
  "vabdq_x_u16\0"
  "vabdq_x_u32\0"
  "vabdq_x_u8\0"
  "vabsq_f16\0"
  "vabsq_f32\0"
  "vabsq_m_f16\0"
  "vabsq_m_f32\0"
  "vabsq_m_s16\0"
  "vabsq_m_s32\0"
  "vabsq_m_s8\0"
  "vabsq_s16\0"
  "vabsq_s32\0"
  "vabsq_s8\0"
  "vabsq_x_f16\0"
  "vabsq_x_f32\0"
  "vabsq_x_s16\0"
  "vabsq_x_s32\0"
  "vabsq_x_s8\0"
  "vadciq_m_s32\0"
  "vadciq_m_u32\0"
  "vadciq_s32\0"
  "vadciq_u32\0"
  "vadcq_m_s32\0"
  "vadcq_m_u32\0"
  "vadcq_s32\0"
  "vadcq_u32\0"
  "vaddlvaq_p_s32\0"
  "vaddlvaq_p_u32\0"
  "vaddlvaq_s32\0"
  "vaddlvaq_u32\0"
  "vaddlvq_p_s32\0"
  "vaddlvq_p_u32\0"
  "vaddlvq_s32\0"
  "vaddlvq_u32\0"
  "vaddq_f16\0"
  "vaddq_f32\0"
  "vaddq_m_f16\0"
  "vaddq_m_f32\0"
  "vaddq_m_n_f16\0"
  "vaddq_m_n_f32\0"
  "vaddq_m_n_s16\0"
  "vaddq_m_n_s32\0"
  "vaddq_m_n_s8\0"
  "vaddq_m_n_u16\0"
  "vaddq_m_n_u32\0"
  "vaddq_m_n_u8\0"
  "vaddq_m_s16\0"
  "vaddq_m_s32\0"
  "vaddq_m_s8\0"
  "vaddq_m_u16\0"
  "vaddq_m_u32\0"
  "vaddq_m_u8\0"
  "vaddq_n_f16\0"
  "vaddq_n_f32\0"
  "vaddq_n_s16\0"
  "vaddq_n_s32\0"
  "vaddq_n_s8\0"
  "vaddq_n_u16\0"
  "vaddq_n_u32\0"
  "vaddq_n_u8\0"
  "vaddq_s16\0"
  "vaddq_s32\0"
  "vaddq_s8\0"
  "vaddq_u16\0"
  "vaddq_u32\0"
  "vaddq_u8\0"
  "vaddq_x_f16\0"
  "vaddq_x_f32\0"
  "vaddq_x_n_f16\0"
  "vaddq_x_n_f32\0"
  "vaddq_x_n_s16\0"
  "vaddq_x_n_s32\0"
  "vaddq_x_n_s8\0"
  "vaddq_x_n_u16\0"
  "vaddq_x_n_u32\0"
  "vaddq_x_n_u8\0"
  "vaddq_x_s16\0"
  "vaddq_x_s32\0"
  "vaddq_x_s8\0"
  "vaddq_x_u16\0"
  "vaddq_x_u32\0"
  "vaddq_x_u8\0"
  "vaddvaq_p_s16\0"
  "vaddvaq_p_s32\0"
  "vaddvaq_p_s8\0"
  "vaddvaq_p_u16\0"
  "vaddvaq_p_u32\0"
  "vaddvaq_p_u8\0"
  "vaddvaq_s16\0"
  "vaddvaq_s32\0"
  "vaddvaq_s8\0"
  "vaddvaq_u16\0"
  "vaddvaq_u32\0"
  "vaddvaq_u8\0"
  "vaddvq_p_s16\0"
  "vaddvq_p_s32\0"
  "vaddvq_p_s8\0"
  "vaddvq_p_u16\0"
  "vaddvq_p_u32\0"
  "vaddvq_p_u8\0"
  "vaddvq_s16\0"
  "vaddvq_s32\0"
  "vaddvq_s8\0"
  "vaddvq_u16\0"
  "vaddvq_u32\0"
  "vaddvq_u8\0"
  "vandq_f16\0"
  "vandq_f32\0"
  "vandq_m_f16\0"
  "vandq_m_f32\0"
  "vandq_m_s16\0"
  "vandq_m_s32\0"
  "vandq_m_s8\0"
  "vandq_m_u16\0"
  "vandq_m_u32\0"
  "vandq_m_u8\0"
  "vandq_s16\0"
  "vandq_s32\0"
  "vandq_s8\0"
  "vandq_u16\0"
  "vandq_u32\0"
  "vandq_u8\0"
  "vandq_x_f16\0"
  "vandq_x_f32\0"
  "vandq_x_s16\0"
  "vandq_x_s32\0"
  "vandq_x_s8\0"
  "vandq_x_u16\0"
  "vandq_x_u32\0"
  "vandq_x_u8\0"
  "vbicq_f16\0"
  "vbicq_f32\0"
  "vbicq_m_f16\0"
  "vbicq_m_f32\0"
  "vbicq_m_n_s16\0"
  "vbicq_m_n_s32\0"
  "vbicq_m_n_u16\0"
  "vbicq_m_n_u32\0"
  "vbicq_m_s16\0"
  "vbicq_m_s32\0"
  "vbicq_m_s8\0"
  "vbicq_m_u16\0"
  "vbicq_m_u32\0"
  "vbicq_m_u8\0"
  "vbicq_n_s16\0"
  "vbicq_n_s32\0"
  "vbicq_n_u16\0"
  "vbicq_n_u32\0"
  "vbicq_s16\0"
  "vbicq_s32\0"
  "vbicq_s8\0"
  "vbicq_u16\0"
  "vbicq_u32\0"
  "vbicq_u8\0"
  "vbicq_x_f16\0"
  "vbicq_x_f32\0"
  "vbicq_x_s16\0"
  "vbicq_x_s32\0"
  "vbicq_x_s8\0"
  "vbicq_x_u16\0"
  "vbicq_x_u32\0"
  "vbicq_x_u8\0"
  "vbrsrq_m_n_f16\0"
  "vbrsrq_m_n_f32\0"
  "vbrsrq_m_n_s16\0"
  "vbrsrq_m_n_s32\0"
  "vbrsrq_m_n_s8\0"
  "vbrsrq_m_n_u16\0"
  "vbrsrq_m_n_u32\0"
  "vbrsrq_m_n_u8\0"
  "vbrsrq_n_f16\0"
  "vbrsrq_n_f32\0"
  "vbrsrq_n_s16\0"
  "vbrsrq_n_s32\0"
  "vbrsrq_n_s8\0"
  "vbrsrq_n_u16\0"
  "vbrsrq_n_u32\0"
  "vbrsrq_n_u8\0"
  "vbrsrq_x_n_f16\0"
  "vbrsrq_x_n_f32\0"
  "vbrsrq_x_n_s16\0"
  "vbrsrq_x_n_s32\0"
  "vbrsrq_x_n_s8\0"
  "vbrsrq_x_n_u16\0"
  "vbrsrq_x_n_u32\0"
  "vbrsrq_x_n_u8\0"
  "vcaddq_rot270_f16\0"
  "vcaddq_rot270_f32\0"
  "vcaddq_rot270_m_f16\0"
  "vcaddq_rot270_m_f32\0"
  "vcaddq_rot270_m_s16\0"
  "vcaddq_rot270_m_s32\0"
  "vcaddq_rot270_m_s8\0"
  "vcaddq_rot270_m_u16\0"
  "vcaddq_rot270_m_u32\0"
  "vcaddq_rot270_m_u8\0"
  "vcaddq_rot270_s16\0"
  "vcaddq_rot270_s32\0"
  "vcaddq_rot270_s8\0"
  "vcaddq_rot270_u16\0"
  "vcaddq_rot270_u32\0"
  "vcaddq_rot270_u8\0"
  "vcaddq_rot270_x_f16\0"
  "vcaddq_rot270_x_f32\0"
  "vcaddq_rot270_x_s16\0"
  "vcaddq_rot270_x_s32\0"
  "vcaddq_rot270_x_s8\0"
  "vcaddq_rot270_x_u16\0"
  "vcaddq_rot270_x_u32\0"
  "vcaddq_rot270_x_u8\0"
  "vcaddq_rot90_f16\0"
  "vcaddq_rot90_f32\0"
  "vcaddq_rot90_m_f16\0"
  "vcaddq_rot90_m_f32\0"
  "vcaddq_rot90_m_s16\0"
  "vcaddq_rot90_m_s32\0"
  "vcaddq_rot90_m_s8\0"
  "vcaddq_rot90_m_u16\0"
  "vcaddq_rot90_m_u32\0"
  "vcaddq_rot90_m_u8\0"
  "vcaddq_rot90_s16\0"
  "vcaddq_rot90_s32\0"
  "vcaddq_rot90_s8\0"
  "vcaddq_rot90_u16\0"
  "vcaddq_rot90_u32\0"
  "vcaddq_rot90_u8\0"
  "vcaddq_rot90_x_f16\0"
  "vcaddq_rot90_x_f32\0"
  "vcaddq_rot90_x_s16\0"
  "vcaddq_rot90_x_s32\0"
  "vcaddq_rot90_x_s8\0"
  "vcaddq_rot90_x_u16\0"
  "vcaddq_rot90_x_u32\0"
  "vcaddq_rot90_x_u8\0"
  "vclsq_m_s16\0"
  "vclsq_m_s32\0"
  "vclsq_m_s8\0"
  "vclsq_s16\0"
  "vclsq_s32\0"
  "vclsq_s8\0"
  "vclsq_x_s16\0"
  "vclsq_x_s32\0"
  "vclsq_x_s8\0"
  "vclzq_m_s16\0"
  "vclzq_m_s32\0"
  "vclzq_m_s8\0"
  "vclzq_m_u16\0"
  "vclzq_m_u32\0"
  "vclzq_m_u8\0"
  "vclzq_s16\0"
  "vclzq_s32\0"
  "vclzq_s8\0"
  "vclzq_u16\0"
  "vclzq_u32\0"
  "vclzq_u8\0"
  "vclzq_x_s16\0"
  "vclzq_x_s32\0"
  "vclzq_x_s8\0"
  "vclzq_x_u16\0"
  "vclzq_x_u32\0"
  "vclzq_x_u8\0"
  "vcmlaq_f16\0"
  "vcmlaq_f32\0"
  "vcmlaq_m_f16\0"
  "vcmlaq_m_f32\0"
  "vcmlaq_rot180_f16\0"
  "vcmlaq_rot180_f32\0"
  "vcmlaq_rot180_m_f16\0"
  "vcmlaq_rot180_m_f32\0"
  "vcmlaq_rot270_f16\0"
  "vcmlaq_rot270_f32\0"
  "vcmlaq_rot270_m_f16\0"
  "vcmlaq_rot270_m_f32\0"
  "vcmlaq_rot90_f16\0"
  "vcmlaq_rot90_f32\0"
  "vcmlaq_rot90_m_f16\0"
  "vcmlaq_rot90_m_f32\0"
  "vcmpcsq_m_n_u16\0"
  "vcmpcsq_m_n_u32\0"
  "vcmpcsq_m_n_u8\0"
  "vcmpcsq_m_u16\0"
  "vcmpcsq_m_u32\0"
  "vcmpcsq_m_u8\0"
  "vcmpcsq_n_u16\0"
  "vcmpcsq_n_u32\0"
  "vcmpcsq_n_u8\0"
  "vcmpcsq_u16\0"
  "vcmpcsq_u32\0"
  "vcmpcsq_u8\0"
  "vcmpeqq_f16\0"
  "vcmpeqq_f32\0"
  "vcmpeqq_m_f16\0"
  "vcmpeqq_m_f32\0"
  "vcmpeqq_m_n_f16\0"
  "vcmpeqq_m_n_f32\0"
  "vcmpeqq_m_n_s16\0"
  "vcmpeqq_m_n_s32\0"
  "vcmpeqq_m_n_s8\0"
  "vcmpeqq_m_n_u16\0"
  "vcmpeqq_m_n_u32\0"
  "vcmpeqq_m_n_u8\0"
  "vcmpeqq_m_s16\0"
  "vcmpeqq_m_s32\0"
  "vcmpeqq_m_s8\0"
  "vcmpeqq_m_u16\0"
  "vcmpeqq_m_u32\0"
  "vcmpeqq_m_u8\0"
  "vcmpeqq_n_f16\0"
  "vcmpeqq_n_f32\0"
  "vcmpeqq_n_s16\0"
  "vcmpeqq_n_s32\0"
  "vcmpeqq_n_s8\0"
  "vcmpeqq_n_u16\0"
  "vcmpeqq_n_u32\0"
  "vcmpeqq_n_u8\0"
  "vcmpeqq_s16\0"
  "vcmpeqq_s32\0"
  "vcmpeqq_s8\0"
  "vcmpeqq_u16\0"
  "vcmpeqq_u32\0"
  "vcmpeqq_u8\0"
  "vcmpgeq_f16\0"
  "vcmpgeq_f32\0"
  "vcmpgeq_m_f16\0"
  "vcmpgeq_m_f32\0"
  "vcmpgeq_m_n_f16\0"
  "vcmpgeq_m_n_f32\0"
  "vcmpgeq_m_n_s16\0"
  "vcmpgeq_m_n_s32\0"
  "vcmpgeq_m_n_s8\0"
  "vcmpgeq_m_s16\0"
  "vcmpgeq_m_s32\0"
  "vcmpgeq_m_s8\0"
  "vcmpgeq_n_f16\0"
  "vcmpgeq_n_f32\0"
  "vcmpgeq_n_s16\0"
  "vcmpgeq_n_s32\0"
  "vcmpgeq_n_s8\0"
  "vcmpgeq_s16\0"
  "vcmpgeq_s32\0"
  "vcmpgeq_s8\0"
  "vcmpgtq_f16\0"
  "vcmpgtq_f32\0"
  "vcmpgtq_m_f16\0"
  "vcmpgtq_m_f32\0"
  "vcmpgtq_m_n_f16\0"
  "vcmpgtq_m_n_f32\0"
  "vcmpgtq_m_n_s16\0"
  "vcmpgtq_m_n_s32\0"
  "vcmpgtq_m_n_s8\0"
  "vcmpgtq_m_s16\0"
  "vcmpgtq_m_s32\0"
  "vcmpgtq_m_s8\0"
  "vcmpgtq_n_f16\0"
  "vcmpgtq_n_f32\0"
  "vcmpgtq_n_s16\0"
  "vcmpgtq_n_s32\0"
  "vcmpgtq_n_s8\0"
  "vcmpgtq_s16\0"
  "vcmpgtq_s32\0"
  "vcmpgtq_s8\0"
  "vcmphiq_m_n_u16\0"
  "vcmphiq_m_n_u32\0"
  "vcmphiq_m_n_u8\0"
  "vcmphiq_m_u16\0"
  "vcmphiq_m_u32\0"
  "vcmphiq_m_u8\0"
  "vcmphiq_n_u16\0"
  "vcmphiq_n_u32\0"
  "vcmphiq_n_u8\0"
  "vcmphiq_u16\0"
  "vcmphiq_u32\0"
  "vcmphiq_u8\0"
  "vcmpleq_f16\0"
  "vcmpleq_f32\0"
  "vcmpleq_m_f16\0"
  "vcmpleq_m_f32\0"
  "vcmpleq_m_n_f16\0"
  "vcmpleq_m_n_f32\0"
  "vcmpleq_m_n_s16\0"
  "vcmpleq_m_n_s32\0"
  "vcmpleq_m_n_s8\0"
  "vcmpleq_m_s16\0"
  "vcmpleq_m_s32\0"
  "vcmpleq_m_s8\0"
  "vcmpleq_n_f16\0"
  "vcmpleq_n_f32\0"
  "vcmpleq_n_s16\0"
  "vcmpleq_n_s32\0"
  "vcmpleq_n_s8\0"
  "vcmpleq_s16\0"
  "vcmpleq_s32\0"
  "vcmpleq_s8\0"
  "vcmpltq_f16\0"
  "vcmpltq_f32\0"
  "vcmpltq_m_f16\0"
  "vcmpltq_m_f32\0"
  "vcmpltq_m_n_f16\0"
  "vcmpltq_m_n_f32\0"
  "vcmpltq_m_n_s16\0"
  "vcmpltq_m_n_s32\0"
  "vcmpltq_m_n_s8\0"
  "vcmpltq_m_s16\0"
  "vcmpltq_m_s32\0"
  "vcmpltq_m_s8\0"
  "vcmpltq_n_f16\0"
  "vcmpltq_n_f32\0"
  "vcmpltq_n_s16\0"
  "vcmpltq_n_s32\0"
  "vcmpltq_n_s8\0"
  "vcmpltq_s16\0"
  "vcmpltq_s32\0"
  "vcmpltq_s8\0"
  "vcmpneq_f16\0"
  "vcmpneq_f32\0"
  "vcmpneq_m_f16\0"
  "vcmpneq_m_f32\0"
  "vcmpneq_m_n_f16\0"
  "vcmpneq_m_n_f32\0"
  "vcmpneq_m_n_s16\0"
  "vcmpneq_m_n_s32\0"
  "vcmpneq_m_n_s8\0"
  "vcmpneq_m_n_u16\0"
  "vcmpneq_m_n_u32\0"
  "vcmpneq_m_n_u8\0"
  "vcmpneq_m_s16\0"
  "vcmpneq_m_s32\0"
  "vcmpneq_m_s8\0"
  "vcmpneq_m_u16\0"
  "vcmpneq_m_u32\0"
  "vcmpneq_m_u8\0"
  "vcmpneq_n_f16\0"
  "vcmpneq_n_f32\0"
  "vcmpneq_n_s16\0"
  "vcmpneq_n_s32\0"
  "vcmpneq_n_s8\0"
  "vcmpneq_n_u16\0"
  "vcmpneq_n_u32\0"
  "vcmpneq_n_u8\0"
  "vcmpneq_s16\0"
  "vcmpneq_s32\0"
  "vcmpneq_s8\0"
  "vcmpneq_u16\0"
  "vcmpneq_u32\0"
  "vcmpneq_u8\0"
  "vcmulq_f16\0"
  "vcmulq_f32\0"
  "vcmulq_m_f16\0"
  "vcmulq_m_f32\0"
  "vcmulq_rot180_f16\0"
  "vcmulq_rot180_f32\0"
  "vcmulq_rot180_m_f16\0"
  "vcmulq_rot180_m_f32\0"
  "vcmulq_rot180_x_f16\0"
  "vcmulq_rot180_x_f32\0"
  "vcmulq_rot270_f16\0"
  "vcmulq_rot270_f32\0"
  "vcmulq_rot270_m_f16\0"
  "vcmulq_rot270_m_f32\0"
  "vcmulq_rot270_x_f16\0"
  "vcmulq_rot270_x_f32\0"
  "vcmulq_rot90_f16\0"
  "vcmulq_rot90_f32\0"
  "vcmulq_rot90_m_f16\0"
  "vcmulq_rot90_m_f32\0"
  "vcmulq_rot90_x_f16\0"
  "vcmulq_rot90_x_f32\0"
  "vcmulq_x_f16\0"
  "vcmulq_x_f32\0"
  "vcreateq_f16\0"
  "vcreateq_f32\0"
  "vcreateq_s16\0"
  "vcreateq_s32\0"
  "vcreateq_s64\0"
  "vcreateq_s8\0"
  "vcreateq_u16\0"
  "vcreateq_u32\0"
  "vcreateq_u64\0"
  "vcreateq_u8\0"
  "vctp16q\0"
  "vctp16q_m\0"
  "vctp32q\0"
  "vctp32q_m\0"
  "vctp64q\0"
  "vctp64q_m\0"
  "vctp8q\0"
  "vctp8q_m\0"
  "vcvtaq_m_s16_f16\0"
  "vcvtaq_m_s32_f32\0"
  "vcvtaq_m_u16_f16\0"
  "vcvtaq_m_u32_f32\0"
  "vcvtaq_s16_f16\0"
  "vcvtaq_s32_f32\0"
  "vcvtaq_u16_f16\0"
  "vcvtaq_u32_f32\0"
  "vcvtaq_x_s16_f16\0"
  "vcvtaq_x_s32_f32\0"
  "vcvtaq_x_u16_f16\0"
  "vcvtaq_x_u32_f32\0"
  "vcvtbq_f16_f32\0"
  "vcvtbq_f32_f16\0"
  "vcvtbq_m_f16_f32\0"
  "vcvtbq_m_f32_f16\0"
  "vcvtbq_x_f32_f16\0"
  "vcvtmq_m_s16_f16\0"
  "vcvtmq_m_s32_f32\0"
  "vcvtmq_m_u16_f16\0"
  "vcvtmq_m_u32_f32\0"
  "vcvtmq_s16_f16\0"
  "vcvtmq_s32_f32\0"
  "vcvtmq_u16_f16\0"
  "vcvtmq_u32_f32\0"
  "vcvtmq_x_s16_f16\0"
  "vcvtmq_x_s32_f32\0"
  "vcvtmq_x_u16_f16\0"
  "vcvtmq_x_u32_f32\0"
  "vcvtnq_m_s16_f16\0"
  "vcvtnq_m_s32_f32\0"
  "vcvtnq_m_u16_f16\0"
  "vcvtnq_m_u32_f32\0"
  "vcvtnq_s16_f16\0"
  "vcvtnq_s32_f32\0"
  "vcvtnq_u16_f16\0"
  "vcvtnq_u32_f32\0"
  "vcvtnq_x_s16_f16\0"
  "vcvtnq_x_s32_f32\0"
  "vcvtnq_x_u16_f16\0"
  "vcvtnq_x_u32_f32\0"
  "vcvtpq_m_s16_f16\0"
  "vcvtpq_m_s32_f32\0"
  "vcvtpq_m_u16_f16\0"
  "vcvtpq_m_u32_f32\0"
  "vcvtpq_s16_f16\0"
  "vcvtpq_s32_f32\0"
  "vcvtpq_u16_f16\0"
  "vcvtpq_u32_f32\0"
  "vcvtpq_x_s16_f16\0"
  "vcvtpq_x_s32_f32\0"
  "vcvtpq_x_u16_f16\0"
  "vcvtpq_x_u32_f32\0"
  "vcvtq_f16_s16\0"
  "vcvtq_f16_u16\0"
  "vcvtq_f32_s32\0"
  "vcvtq_f32_u32\0"
  "vcvtq_m_f16_s16\0"
  "vcvtq_m_f16_u16\0"
  "vcvtq_m_f32_s32\0"
  "vcvtq_m_f32_u32\0"
  "vcvtq_m_n_f16_s16\0"
  "vcvtq_m_n_f16_u16\0"
  "vcvtq_m_n_f32_s32\0"
  "vcvtq_m_n_f32_u32\0"
  "vcvtq_m_n_s16_f16\0"
  "vcvtq_m_n_s32_f32\0"
  "vcvtq_m_n_u16_f16\0"
  "vcvtq_m_n_u32_f32\0"
  "vcvtq_m_s16_f16\0"
  "vcvtq_m_s32_f32\0"
  "vcvtq_m_u16_f16\0"
  "vcvtq_m_u32_f32\0"
  "vcvtq_n_f16_s16\0"
  "vcvtq_n_f16_u16\0"
  "vcvtq_n_f32_s32\0"
  "vcvtq_n_f32_u32\0"
  "vcvtq_n_s16_f16\0"
  "vcvtq_n_s32_f32\0"
  "vcvtq_n_u16_f16\0"
  "vcvtq_n_u32_f32\0"
  "vcvtq_s16_f16\0"
  "vcvtq_s32_f32\0"
  "vcvtq_u16_f16\0"
  "vcvtq_u32_f32\0"
  "vcvtq_x_f16_s16\0"
  "vcvtq_x_f16_u16\0"
  "vcvtq_x_f32_s32\0"
  "vcvtq_x_f32_u32\0"
  "vcvtq_x_n_f16_s16\0"
  "vcvtq_x_n_f16_u16\0"
  "vcvtq_x_n_f32_s32\0"
  "vcvtq_x_n_f32_u32\0"
  "vcvtq_x_n_s16_f16\0"
  "vcvtq_x_n_s32_f32\0"
  "vcvtq_x_n_u16_f16\0"
  "vcvtq_x_n_u32_f32\0"
  "vcvtq_x_s16_f16\0"
  "vcvtq_x_s32_f32\0"
  "vcvtq_x_u16_f16\0"
  "vcvtq_x_u32_f32\0"
  "vcvttq_f16_f32\0"
  "vcvttq_f32_f16\0"
  "vcvttq_m_f16_f32\0"
  "vcvttq_m_f32_f16\0"
  "vcvttq_x_f32_f16\0"
  "vddupq_m_n_u16\0"
  "vddupq_m_n_u32\0"
  "vddupq_m_n_u8\0"
  "vddupq_m_wb_u16\0"
  "vddupq_m_wb_u32\0"
  "vddupq_m_wb_u8\0"
  "vddupq_n_u16\0"
  "vddupq_n_u32\0"
  "vddupq_n_u8\0"
  "vddupq_wb_u16\0"
  "vddupq_wb_u32\0"
  "vddupq_wb_u8\0"
  "vddupq_x_n_u16\0"
  "vddupq_x_n_u32\0"
  "vddupq_x_n_u8\0"
  "vddupq_x_wb_u16\0"
  "vddupq_x_wb_u32\0"
  "vddupq_x_wb_u8\0"
  "vdupq_m_n_f16\0"
  "vdupq_m_n_f32\0"
  "vdupq_m_n_s16\0"
  "vdupq_m_n_s32\0"
  "vdupq_m_n_s8\0"
  "vdupq_m_n_u16\0"
  "vdupq_m_n_u32\0"
  "vdupq_m_n_u8\0"
  "vdupq_n_f16\0"
  "vdupq_n_f32\0"
  "vdupq_n_s16\0"
  "vdupq_n_s32\0"
  "vdupq_n_s8\0"
  "vdupq_n_u16\0"
  "vdupq_n_u32\0"
  "vdupq_n_u8\0"
  "vdupq_x_n_f16\0"
  "vdupq_x_n_f32\0"
  "vdupq_x_n_s16\0"
  "vdupq_x_n_s32\0"
  "vdupq_x_n_s8\0"
  "vdupq_x_n_u16\0"
  "vdupq_x_n_u32\0"
  "vdupq_x_n_u8\0"
  "vdwdupq_m_n_u16\0"
  "vdwdupq_m_n_u32\0"
  "vdwdupq_m_n_u8\0"
  "vdwdupq_m_wb_u16\0"
  "vdwdupq_m_wb_u32\0"
  "vdwdupq_m_wb_u8\0"
  "vdwdupq_n_u16\0"
  "vdwdupq_n_u32\0"
  "vdwdupq_n_u8\0"
  "vdwdupq_wb_u16\0"
  "vdwdupq_wb_u32\0"
  "vdwdupq_wb_u8\0"
  "vdwdupq_x_n_u16\0"
  "vdwdupq_x_n_u32\0"
  "vdwdupq_x_n_u8\0"
  "vdwdupq_x_wb_u16\0"
  "vdwdupq_x_wb_u32\0"
  "vdwdupq_x_wb_u8\0"
  "veorq_f16\0"
  "veorq_f32\0"
  "veorq_m_f16\0"
  "veorq_m_f32\0"
  "veorq_m_s16\0"
  "veorq_m_s32\0"
  "veorq_m_s8\0"
  "veorq_m_u16\0"
  "veorq_m_u32\0"
  "veorq_m_u8\0"
  "veorq_s16\0"
  "veorq_s32\0"
  "veorq_s8\0"
  "veorq_u16\0"
  "veorq_u32\0"
  "veorq_u8\0"
  "veorq_x_f16\0"
  "veorq_x_f32\0"
  "veorq_x_s16\0"
  "veorq_x_s32\0"
  "veorq_x_s8\0"
  "veorq_x_u16\0"
  "veorq_x_u32\0"
  "veorq_x_u8\0"
  "vfmaq_f16\0"
  "vfmaq_f32\0"
  "vfmaq_m_f16\0"
  "vfmaq_m_f32\0"
  "vfmaq_m_n_f16\0"
  "vfmaq_m_n_f32\0"
  "vfmaq_n_f16\0"
  "vfmaq_n_f32\0"
  "vfmasq_m_n_f16\0"
  "vfmasq_m_n_f32\0"
  "vfmasq_n_f16\0"
  "vfmasq_n_f32\0"
  "vfmsq_f16\0"
  "vfmsq_f32\0"
  "vfmsq_m_f16\0"
  "vfmsq_m_f32\0"
  "vgetq_lane_f16\0"
  "vgetq_lane_f32\0"
  "vgetq_lane_s16\0"
  "vgetq_lane_s32\0"
  "vgetq_lane_s64\0"
  "vgetq_lane_s8\0"
  "vgetq_lane_u16\0"
  "vgetq_lane_u32\0"
  "vgetq_lane_u64\0"
  "vgetq_lane_u8\0"
  "vhaddq_m_n_s16\0"
  "vhaddq_m_n_s32\0"
  "vhaddq_m_n_s8\0"
  "vhaddq_m_n_u16\0"
  "vhaddq_m_n_u32\0"
  "vhaddq_m_n_u8\0"
  "vhaddq_m_s16\0"
  "vhaddq_m_s32\0"
  "vhaddq_m_s8\0"
  "vhaddq_m_u16\0"
  "vhaddq_m_u32\0"
  "vhaddq_m_u8\0"
  "vhaddq_n_s16\0"
  "vhaddq_n_s32\0"
  "vhaddq_n_s8\0"
  "vhaddq_n_u16\0"
  "vhaddq_n_u32\0"
  "vhaddq_n_u8\0"
  "vhaddq_s16\0"
  "vhaddq_s32\0"
  "vhaddq_s8\0"
  "vhaddq_u16\0"
  "vhaddq_u32\0"
  "vhaddq_u8\0"
  "vhaddq_x_n_s16\0"
  "vhaddq_x_n_s32\0"
  "vhaddq_x_n_s8\0"
  "vhaddq_x_n_u16\0"
  "vhaddq_x_n_u32\0"
  "vhaddq_x_n_u8\0"
  "vhaddq_x_s16\0"
  "vhaddq_x_s32\0"
  "vhaddq_x_s8\0"
  "vhaddq_x_u16\0"
  "vhaddq_x_u32\0"
  "vhaddq_x_u8\0"
  "vhcaddq_rot270_m_s16\0"
  "vhcaddq_rot270_m_s32\0"
  "vhcaddq_rot270_m_s8\0"
  "vhcaddq_rot270_s16\0"
  "vhcaddq_rot270_s32\0"
  "vhcaddq_rot270_s8\0"
  "vhcaddq_rot270_x_s16\0"
  "vhcaddq_rot270_x_s32\0"
  "vhcaddq_rot270_x_s8\0"
  "vhcaddq_rot90_m_s16\0"
  "vhcaddq_rot90_m_s32\0"
  "vhcaddq_rot90_m_s8\0"
  "vhcaddq_rot90_s16\0"
  "vhcaddq_rot90_s32\0"
  "vhcaddq_rot90_s8\0"
  "vhcaddq_rot90_x_s16\0"
  "vhcaddq_rot90_x_s32\0"
  "vhcaddq_rot90_x_s8\0"
  "vhsubq_m_n_s16\0"
  "vhsubq_m_n_s32\0"
  "vhsubq_m_n_s8\0"
  "vhsubq_m_n_u16\0"
  "vhsubq_m_n_u32\0"
  "vhsubq_m_n_u8\0"
  "vhsubq_m_s16\0"
  "vhsubq_m_s32\0"
  "vhsubq_m_s8\0"
  "vhsubq_m_u16\0"
  "vhsubq_m_u32\0"
  "vhsubq_m_u8\0"
  "vhsubq_n_s16\0"
  "vhsubq_n_s32\0"
  "vhsubq_n_s8\0"
  "vhsubq_n_u16\0"
  "vhsubq_n_u32\0"
  "vhsubq_n_u8\0"
  "vhsubq_s16\0"
  "vhsubq_s32\0"
  "vhsubq_s8\0"
  "vhsubq_u16\0"
  "vhsubq_u32\0"
  "vhsubq_u8\0"
  "vhsubq_x_n_s16\0"
  "vhsubq_x_n_s32\0"
  "vhsubq_x_n_s8\0"
  "vhsubq_x_n_u16\0"
  "vhsubq_x_n_u32\0"
  "vhsubq_x_n_u8\0"
  "vhsubq_x_s16\0"
  "vhsubq_x_s32\0"
  "vhsubq_x_s8\0"
  "vhsubq_x_u16\0"
  "vhsubq_x_u32\0"
  "vhsubq_x_u8\0"
  "vidupq_m_n_u16\0"
  "vidupq_m_n_u32\0"
  "vidupq_m_n_u8\0"
  "vidupq_m_wb_u16\0"
  "vidupq_m_wb_u32\0"
  "vidupq_m_wb_u8\0"
  "vidupq_n_u16\0"
  "vidupq_n_u32\0"
  "vidupq_n_u8\0"
  "vidupq_wb_u16\0"
  "vidupq_wb_u32\0"
  "vidupq_wb_u8\0"
  "vidupq_x_n_u16\0"
  "vidupq_x_n_u32\0"
  "vidupq_x_n_u8\0"
  "vidupq_x_wb_u16\0"
  "vidupq_x_wb_u32\0"
  "vidupq_x_wb_u8\0"
  "viwdupq_m_n_u16\0"
  "viwdupq_m_n_u32\0"
  "viwdupq_m_n_u8\0"
  "viwdupq_m_wb_u16\0"
  "viwdupq_m_wb_u32\0"
  "viwdupq_m_wb_u8\0"
  "viwdupq_n_u16\0"
  "viwdupq_n_u32\0"
  "viwdupq_n_u8\0"
  "viwdupq_wb_u16\0"
  "viwdupq_wb_u32\0"
  "viwdupq_wb_u8\0"
  "viwdupq_x_n_u16\0"
  "viwdupq_x_n_u32\0"
  "viwdupq_x_n_u8\0"
  "viwdupq_x_wb_u16\0"
  "viwdupq_x_wb_u32\0"
  "viwdupq_x_wb_u8\0"
  "vld1q_f16\0"
  "vld1q_f32\0"
  "vld1q_s16\0"
  "vld1q_s32\0"
  "vld1q_s8\0"
  "vld1q_u16\0"
  "vld1q_u32\0"
  "vld1q_u8\0"
  "vld1q_z_f16\0"
  "vld1q_z_f32\0"
  "vld1q_z_s16\0"
  "vld1q_z_s32\0"
  "vld1q_z_s8\0"
  "vld1q_z_u16\0"
  "vld1q_z_u32\0"
  "vld1q_z_u8\0"
  "vld2q_f16\0"
  "vld2q_f32\0"
  "vld2q_s16\0"
  "vld2q_s32\0"
  "vld2q_s8\0"
  "vld2q_u16\0"
  "vld2q_u32\0"
  "vld2q_u8\0"
  "vld4q_f16\0"
  "vld4q_f32\0"
  "vld4q_s16\0"
  "vld4q_s32\0"
  "vld4q_s8\0"
  "vld4q_u16\0"
  "vld4q_u32\0"
  "vld4q_u8\0"
  "vldrbq_gather_offset_s16\0"
  "vldrbq_gather_offset_s32\0"
  "vldrbq_gather_offset_s8\0"
  "vldrbq_gather_offset_u16\0"
  "vldrbq_gather_offset_u32\0"
  "vldrbq_gather_offset_u8\0"
  "vldrbq_gather_offset_z_s16\0"
  "vldrbq_gather_offset_z_s32\0"
  "vldrbq_gather_offset_z_s8\0"
  "vldrbq_gather_offset_z_u16\0"
  "vldrbq_gather_offset_z_u32\0"
  "vldrbq_gather_offset_z_u8\0"
  "vldrbq_s16\0"
  "vldrbq_s32\0"
  "vldrbq_s8\0"
  "vldrbq_u16\0"
  "vldrbq_u32\0"
  "vldrbq_u8\0"
  "vldrbq_z_s16\0"
  "vldrbq_z_s32\0"
  "vldrbq_z_s8\0"
  "vldrbq_z_u16\0"
  "vldrbq_z_u32\0"
  "vldrbq_z_u8\0"
  "vldrdq_gather_base_s64\0"
  "vldrdq_gather_base_u64\0"
  "vldrdq_gather_base_wb_s64\0"
  "vldrdq_gather_base_wb_u64\0"
  "vldrdq_gather_base_wb_z_s64\0"
  "vldrdq_gather_base_wb_z_u64\0"
  "vldrdq_gather_base_z_s64\0"
  "vldrdq_gather_base_z_u64\0"
  "vldrdq_gather_offset_s64\0"
  "vldrdq_gather_offset_u64\0"
  "vldrdq_gather_offset_z_s64\0"
  "vldrdq_gather_offset_z_u64\0"
  "vldrdq_gather_shifted_offset_s64\0"
  "vldrdq_gather_shifted_offset_u64\0"
  "vldrdq_gather_shifted_offset_z_s64\0"
  "vldrdq_gather_shifted_offset_z_u64\0"
  "vldrhq_f16\0"
  "vldrhq_gather_offset_f16\0"
  "vldrhq_gather_offset_s16\0"
  "vldrhq_gather_offset_s32\0"
  "vldrhq_gather_offset_u16\0"
  "vldrhq_gather_offset_u32\0"
  "vldrhq_gather_offset_z_f16\0"
  "vldrhq_gather_offset_z_s16\0"
  "vldrhq_gather_offset_z_s32\0"
  "vldrhq_gather_offset_z_u16\0"
  "vldrhq_gather_offset_z_u32\0"
  "vldrhq_gather_shifted_offset_f16\0"
  "vldrhq_gather_shifted_offset_s16\0"
  "vldrhq_gather_shifted_offset_s32\0"
  "vldrhq_gather_shifted_offset_u16\0"
  "vldrhq_gather_shifted_offset_u32\0"
  "vldrhq_gather_shifted_offset_z_f16\0"
  "vldrhq_gather_shifted_offset_z_s16\0"
  "vldrhq_gather_shifted_offset_z_s32\0"
  "vldrhq_gather_shifted_offset_z_u16\0"
  "vldrhq_gather_shifted_offset_z_u32\0"
  "vldrhq_s16\0"
  "vldrhq_s32\0"
  "vldrhq_u16\0"
  "vldrhq_u32\0"
  "vldrhq_z_f16\0"
  "vldrhq_z_s16\0"
  "vldrhq_z_s32\0"
  "vldrhq_z_u16\0"
  "vldrhq_z_u32\0"
  "vldrwq_f32\0"
  "vldrwq_gather_base_f32\0"
  "vldrwq_gather_base_s32\0"
  "vldrwq_gather_base_u32\0"
  "vldrwq_gather_base_wb_f32\0"
  "vldrwq_gather_base_wb_s32\0"
  "vldrwq_gather_base_wb_u32\0"
  "vldrwq_gather_base_wb_z_f32\0"
  "vldrwq_gather_base_wb_z_s32\0"
  "vldrwq_gather_base_wb_z_u32\0"
  "vldrwq_gather_base_z_f32\0"
  "vldrwq_gather_base_z_s32\0"
  "vldrwq_gather_base_z_u32\0"
  "vldrwq_gather_offset_f32\0"
  "vldrwq_gather_offset_s32\0"
  "vldrwq_gather_offset_u32\0"
  "vldrwq_gather_offset_z_f32\0"
  "vldrwq_gather_offset_z_s32\0"
  "vldrwq_gather_offset_z_u32\0"
  "vldrwq_gather_shifted_offset_f32\0"
  "vldrwq_gather_shifted_offset_s32\0"
  "vldrwq_gather_shifted_offset_u32\0"
  "vldrwq_gather_shifted_offset_z_f32\0"
  "vldrwq_gather_shifted_offset_z_s32\0"
  "vldrwq_gather_shifted_offset_z_u32\0"
  "vldrwq_s32\0"
  "vldrwq_u32\0"
  "vldrwq_z_f32\0"
  "vldrwq_z_s32\0"
  "vldrwq_z_u32\0"
  "vmaxaq_m_s16\0"
  "vmaxaq_m_s32\0"
  "vmaxaq_m_s8\0"
  "vmaxaq_s16\0"
  "vmaxaq_s32\0"
  "vmaxaq_s8\0"
  "vmaxavq_p_s16\0"
  "vmaxavq_p_s32\0"
  "vmaxavq_p_s8\0"
  "vmaxavq_s16\0"
  "vmaxavq_s32\0"
  "vmaxavq_s8\0"
  "vmaxnmaq_f16\0"
  "vmaxnmaq_f32\0"
  "vmaxnmaq_m_f16\0"
  "vmaxnmaq_m_f32\0"
  "vmaxnmavq_f16\0"
  "vmaxnmavq_f32\0"
  "vmaxnmavq_p_f16\0"
  "vmaxnmavq_p_f32\0"
  "vmaxnmq_f16\0"
  "vmaxnmq_f32\0"
  "vmaxnmq_m_f16\0"
  "vmaxnmq_m_f32\0"
  "vmaxnmq_x_f16\0"
  "vmaxnmq_x_f32\0"
  "vmaxnmvq_f16\0"
  "vmaxnmvq_f32\0"
  "vmaxnmvq_p_f16\0"
  "vmaxnmvq_p_f32\0"
  "vmaxq_m_s16\0"
  "vmaxq_m_s32\0"
  "vmaxq_m_s8\0"
  "vmaxq_m_u16\0"
  "vmaxq_m_u32\0"
  "vmaxq_m_u8\0"
  "vmaxq_s16\0"
  "vmaxq_s32\0"
  "vmaxq_s8\0"
  "vmaxq_u16\0"
  "vmaxq_u32\0"
  "vmaxq_u8\0"
  "vmaxq_x_s16\0"
  "vmaxq_x_s32\0"
  "vmaxq_x_s8\0"
  "vmaxq_x_u16\0"
  "vmaxq_x_u32\0"
  "vmaxq_x_u8\0"
  "vmaxvq_p_s16\0"
  "vmaxvq_p_s32\0"
  "vmaxvq_p_s8\0"
  "vmaxvq_p_u16\0"
  "vmaxvq_p_u32\0"
  "vmaxvq_p_u8\0"
  "vmaxvq_s16\0"
  "vmaxvq_s32\0"
  "vmaxvq_s8\0"
  "vmaxvq_u16\0"
  "vmaxvq_u32\0"
  "vmaxvq_u8\0"
  "vminaq_m_s16\0"
  "vminaq_m_s32\0"
  "vminaq_m_s8\0"
  "vminaq_s16\0"
  "vminaq_s32\0"
  "vminaq_s8\0"
  "vminavq_p_s16\0"
  "vminavq_p_s32\0"
  "vminavq_p_s8\0"
  "vminavq_s16\0"
  "vminavq_s32\0"
  "vminavq_s8\0"
  "vminnmaq_f16\0"
  "vminnmaq_f32\0"
  "vminnmaq_m_f16\0"
  "vminnmaq_m_f32\0"
  "vminnmavq_f16\0"
  "vminnmavq_f32\0"
  "vminnmavq_p_f16\0"
  "vminnmavq_p_f32\0"
  "vminnmq_f16\0"
  "vminnmq_f32\0"
  "vminnmq_m_f16\0"
  "vminnmq_m_f32\0"
  "vminnmq_x_f16\0"
  "vminnmq_x_f32\0"
  "vminnmvq_f16\0"
  "vminnmvq_f32\0"
  "vminnmvq_p_f16\0"
  "vminnmvq_p_f32\0"
  "vminq_m_s16\0"
  "vminq_m_s32\0"
  "vminq_m_s8\0"
  "vminq_m_u16\0"
  "vminq_m_u32\0"
  "vminq_m_u8\0"
  "vminq_s16\0"
  "vminq_s32\0"
  "vminq_s8\0"
  "vminq_u16\0"
  "vminq_u32\0"
  "vminq_u8\0"
  "vminq_x_s16\0"
  "vminq_x_s32\0"
  "vminq_x_s8\0"
  "vminq_x_u16\0"
  "vminq_x_u32\0"
  "vminq_x_u8\0"
  "vminvq_p_s16\0"
  "vminvq_p_s32\0"
  "vminvq_p_s8\0"
  "vminvq_p_u16\0"
  "vminvq_p_u32\0"
  "vminvq_p_u8\0"
  "vminvq_s16\0"
  "vminvq_s32\0"
  "vminvq_s8\0"
  "vminvq_u16\0"
  "vminvq_u32\0"
  "vminvq_u8\0"
  "vmladavaq_p_s16\0"
  "vmladavaq_p_s32\0"
  "vmladavaq_p_s8\0"
  "vmladavaq_p_u16\0"
  "vmladavaq_p_u32\0"
  "vmladavaq_p_u8\0"
  "vmladavaq_s16\0"
  "vmladavaq_s32\0"
  "vmladavaq_s8\0"
  "vmladavaq_u16\0"
  "vmladavaq_u32\0"
  "vmladavaq_u8\0"
  "vmladavaxq_p_s16\0"
  "vmladavaxq_p_s32\0"
  "vmladavaxq_p_s8\0"
  "vmladavaxq_s16\0"
  "vmladavaxq_s32\0"
  "vmladavaxq_s8\0"
  "vmladavq_p_s16\0"
  "vmladavq_p_s32\0"
  "vmladavq_p_s8\0"
  "vmladavq_p_u16\0"
  "vmladavq_p_u32\0"
  "vmladavq_p_u8\0"
  "vmladavq_s16\0"
  "vmladavq_s32\0"
  "vmladavq_s8\0"
  "vmladavq_u16\0"
  "vmladavq_u32\0"
  "vmladavq_u8\0"
  "vmladavxq_p_s16\0"
  "vmladavxq_p_s32\0"
  "vmladavxq_p_s8\0"
  "vmladavxq_s16\0"
  "vmladavxq_s32\0"
  "vmladavxq_s8\0"
  "vmlaldavaq_p_s16\0"
  "vmlaldavaq_p_s32\0"
  "vmlaldavaq_p_u16\0"
  "vmlaldavaq_p_u32\0"
  "vmlaldavaq_s16\0"
  "vmlaldavaq_s32\0"
  "vmlaldavaq_u16\0"
  "vmlaldavaq_u32\0"
  "vmlaldavaxq_p_s16\0"
  "vmlaldavaxq_p_s32\0"
  "vmlaldavaxq_s16\0"
  "vmlaldavaxq_s32\0"
  "vmlaldavq_p_s16\0"
  "vmlaldavq_p_s32\0"
  "vmlaldavq_p_u16\0"
  "vmlaldavq_p_u32\0"
  "vmlaldavq_s16\0"
  "vmlaldavq_s32\0"
  "vmlaldavq_u16\0"
  "vmlaldavq_u32\0"
  "vmlaldavxq_p_s16\0"
  "vmlaldavxq_p_s32\0"
  "vmlaldavxq_s16\0"
  "vmlaldavxq_s32\0"
  "vmlaq_m_n_s16\0"
  "vmlaq_m_n_s32\0"
  "vmlaq_m_n_s8\0"
  "vmlaq_m_n_u16\0"
  "vmlaq_m_n_u32\0"
  "vmlaq_m_n_u8\0"
  "vmlaq_n_s16\0"
  "vmlaq_n_s32\0"
  "vmlaq_n_s8\0"
  "vmlaq_n_u16\0"
  "vmlaq_n_u32\0"
  "vmlaq_n_u8\0"
  "vmlasq_m_n_s16\0"
  "vmlasq_m_n_s32\0"
  "vmlasq_m_n_s8\0"
  "vmlasq_m_n_u16\0"
  "vmlasq_m_n_u32\0"
  "vmlasq_m_n_u8\0"
  "vmlasq_n_s16\0"
  "vmlasq_n_s32\0"
  "vmlasq_n_s8\0"
  "vmlasq_n_u16\0"
  "vmlasq_n_u32\0"
  "vmlasq_n_u8\0"
  "vmlsdavaq_p_s16\0"
  "vmlsdavaq_p_s32\0"
  "vmlsdavaq_p_s8\0"
  "vmlsdavaq_s16\0"
  "vmlsdavaq_s32\0"
  "vmlsdavaq_s8\0"
  "vmlsdavaxq_p_s16\0"
  "vmlsdavaxq_p_s32\0"
  "vmlsdavaxq_p_s8\0"
  "vmlsdavaxq_s16\0"
  "vmlsdavaxq_s32\0"
  "vmlsdavaxq_s8\0"
  "vmlsdavq_p_s16\0"
  "vmlsdavq_p_s32\0"
  "vmlsdavq_p_s8\0"
  "vmlsdavq_s16\0"
  "vmlsdavq_s32\0"
  "vmlsdavq_s8\0"
  "vmlsdavxq_p_s16\0"
  "vmlsdavxq_p_s32\0"
  "vmlsdavxq_p_s8\0"
  "vmlsdavxq_s16\0"
  "vmlsdavxq_s32\0"
  "vmlsdavxq_s8\0"
  "vmlsldavaq_p_s16\0"
  "vmlsldavaq_p_s32\0"
  "vmlsldavaq_s16\0"
  "vmlsldavaq_s32\0"
  "vmlsldavaxq_p_s16\0"
  "vmlsldavaxq_p_s32\0"
  "vmlsldavaxq_s16\0"
  "vmlsldavaxq_s32\0"
  "vmlsldavq_p_s16\0"
  "vmlsldavq_p_s32\0"
  "vmlsldavq_s16\0"
  "vmlsldavq_s32\0"
  "vmlsldavxq_p_s16\0"
  "vmlsldavxq_p_s32\0"
  "vmlsldavxq_s16\0"
  "vmlsldavxq_s32\0"
  "vmovlbq_m_s16\0"
  "vmovlbq_m_s8\0"
  "vmovlbq_m_u16\0"
  "vmovlbq_m_u8\0"
  "vmovlbq_s16\0"
  "vmovlbq_s8\0"
  "vmovlbq_u16\0"
  "vmovlbq_u8\0"
  "vmovlbq_x_s16\0"
  "vmovlbq_x_s8\0"
  "vmovlbq_x_u16\0"
  "vmovlbq_x_u8\0"
  "vmovltq_m_s16\0"
  "vmovltq_m_s8\0"
  "vmovltq_m_u16\0"
  "vmovltq_m_u8\0"
  "vmovltq_s16\0"
  "vmovltq_s8\0"
  "vmovltq_u16\0"
  "vmovltq_u8\0"
  "vmovltq_x_s16\0"
  "vmovltq_x_s8\0"
  "vmovltq_x_u16\0"
  "vmovltq_x_u8\0"
  "vmovnbq_m_s16\0"
  "vmovnbq_m_s32\0"
  "vmovnbq_m_u16\0"
  "vmovnbq_m_u32\0"
  "vmovnbq_s16\0"
  "vmovnbq_s32\0"
  "vmovnbq_u16\0"
  "vmovnbq_u32\0"
  "vmovntq_m_s16\0"
  "vmovntq_m_s32\0"
  "vmovntq_m_u16\0"
  "vmovntq_m_u32\0"
  "vmovntq_s16\0"
  "vmovntq_s32\0"
  "vmovntq_u16\0"
  "vmovntq_u32\0"
  "vmulhq_m_s16\0"
  "vmulhq_m_s32\0"
  "vmulhq_m_s8\0"
  "vmulhq_m_u16\0"
  "vmulhq_m_u32\0"
  "vmulhq_m_u8\0"
  "vmulhq_s16\0"
  "vmulhq_s32\0"
  "vmulhq_s8\0"
  "vmulhq_u16\0"
  "vmulhq_u32\0"
  "vmulhq_u8\0"
  "vmulhq_x_s16\0"
  "vmulhq_x_s32\0"
  "vmulhq_x_s8\0"
  "vmulhq_x_u16\0"
  "vmulhq_x_u32\0"
  "vmulhq_x_u8\0"
  "vmullbq_int_m_s16\0"
  "vmullbq_int_m_s32\0"
  "vmullbq_int_m_s8\0"
  "vmullbq_int_m_u16\0"
  "vmullbq_int_m_u32\0"
  "vmullbq_int_m_u8\0"
  "vmullbq_int_s16\0"
  "vmullbq_int_s32\0"
  "vmullbq_int_s8\0"
  "vmullbq_int_u16\0"
  "vmullbq_int_u32\0"
  "vmullbq_int_u8\0"
  "vmullbq_int_x_s16\0"
  "vmullbq_int_x_s32\0"
  "vmullbq_int_x_s8\0"
  "vmullbq_int_x_u16\0"
  "vmullbq_int_x_u32\0"
  "vmullbq_int_x_u8\0"
  "vmullbq_poly_m_p16\0"
  "vmullbq_poly_m_p8\0"
  "vmullbq_poly_p16\0"
  "vmullbq_poly_p8\0"
  "vmullbq_poly_x_p16\0"
  "vmullbq_poly_x_p8\0"
  "vmulltq_int_m_s16\0"
  "vmulltq_int_m_s32\0"
  "vmulltq_int_m_s8\0"
  "vmulltq_int_m_u16\0"
  "vmulltq_int_m_u32\0"
  "vmulltq_int_m_u8\0"
  "vmulltq_int_s16\0"
  "vmulltq_int_s32\0"
  "vmulltq_int_s8\0"
  "vmulltq_int_u16\0"
  "vmulltq_int_u32\0"
  "vmulltq_int_u8\0"
  "vmulltq_int_x_s16\0"
  "vmulltq_int_x_s32\0"
  "vmulltq_int_x_s8\0"
  "vmulltq_int_x_u16\0"
  "vmulltq_int_x_u32\0"
  "vmulltq_int_x_u8\0"
  "vmulltq_poly_m_p16\0"
  "vmulltq_poly_m_p8\0"
  "vmulltq_poly_p16\0"
  "vmulltq_poly_p8\0"
  "vmulltq_poly_x_p16\0"
  "vmulltq_poly_x_p8\0"
  "vmulq_f16\0"
  "vmulq_f32\0"
  "vmulq_m_f16\0"
  "vmulq_m_f32\0"
  "vmulq_m_n_f16\0"
  "vmulq_m_n_f32\0"
  "vmulq_m_n_s16\0"
  "vmulq_m_n_s32\0"
  "vmulq_m_n_s8\0"
  "vmulq_m_n_u16\0"
  "vmulq_m_n_u32\0"
  "vmulq_m_n_u8\0"
  "vmulq_m_s16\0"
  "vmulq_m_s32\0"
  "vmulq_m_s8\0"
  "vmulq_m_u16\0"
  "vmulq_m_u32\0"
  "vmulq_m_u8\0"
  "vmulq_n_f16\0"
  "vmulq_n_f32\0"
  "vmulq_n_s16\0"
  "vmulq_n_s32\0"
  "vmulq_n_s8\0"
  "vmulq_n_u16\0"
  "vmulq_n_u32\0"
  "vmulq_n_u8\0"
  "vmulq_s16\0"
  "vmulq_s32\0"
  "vmulq_s8\0"
  "vmulq_u16\0"
  "vmulq_u32\0"
  "vmulq_u8\0"
  "vmulq_x_f16\0"
  "vmulq_x_f32\0"
  "vmulq_x_n_f16\0"
  "vmulq_x_n_f32\0"
  "vmulq_x_n_s16\0"
  "vmulq_x_n_s32\0"
  "vmulq_x_n_s8\0"
  "vmulq_x_n_u16\0"
  "vmulq_x_n_u32\0"
  "vmulq_x_n_u8\0"
  "vmulq_x_s16\0"
  "vmulq_x_s32\0"
  "vmulq_x_s8\0"
  "vmulq_x_u16\0"
  "vmulq_x_u32\0"
  "vmulq_x_u8\0"
  "vmvnq_m_n_s16\0"
  "vmvnq_m_n_s32\0"
  "vmvnq_m_n_u16\0"
  "vmvnq_m_n_u32\0"
  "vmvnq_m_s16\0"
  "vmvnq_m_s32\0"
  "vmvnq_m_s8\0"
  "vmvnq_m_u16\0"
  "vmvnq_m_u32\0"
  "vmvnq_m_u8\0"
  "vmvnq_n_s16\0"
  "vmvnq_n_s32\0"
  "vmvnq_n_u16\0"
  "vmvnq_n_u32\0"
  "vmvnq_s16\0"
  "vmvnq_s32\0"
  "vmvnq_s8\0"
  "vmvnq_u16\0"
  "vmvnq_u32\0"
  "vmvnq_u8\0"
  "vmvnq_x_n_s16\0"
  "vmvnq_x_n_s32\0"
  "vmvnq_x_n_u16\0"
  "vmvnq_x_n_u32\0"
  "vmvnq_x_s16\0"
  "vmvnq_x_s32\0"
  "vmvnq_x_s8\0"
  "vmvnq_x_u16\0"
  "vmvnq_x_u32\0"
  "vmvnq_x_u8\0"
  "vnegq_f16\0"
  "vnegq_f32\0"
  "vnegq_m_f16\0"
  "vnegq_m_f32\0"
  "vnegq_m_s16\0"
  "vnegq_m_s32\0"
  "vnegq_m_s8\0"
  "vnegq_s16\0"
  "vnegq_s32\0"
  "vnegq_s8\0"
  "vnegq_x_f16\0"
  "vnegq_x_f32\0"
  "vnegq_x_s16\0"
  "vnegq_x_s32\0"
  "vnegq_x_s8\0"
  "vornq_f16\0"
  "vornq_f32\0"
  "vornq_m_f16\0"
  "vornq_m_f32\0"
  "vornq_m_s16\0"
  "vornq_m_s32\0"
  "vornq_m_s8\0"
  "vornq_m_u16\0"
  "vornq_m_u32\0"
  "vornq_m_u8\0"
  "vornq_s16\0"
  "vornq_s32\0"
  "vornq_s8\0"
  "vornq_u16\0"
  "vornq_u32\0"
  "vornq_u8\0"
  "vornq_x_f16\0"
  "vornq_x_f32\0"
  "vornq_x_s16\0"
  "vornq_x_s32\0"
  "vornq_x_s8\0"
  "vornq_x_u16\0"
  "vornq_x_u32\0"
  "vornq_x_u8\0"
  "vorrq_f16\0"
  "vorrq_f32\0"
  "vorrq_m_f16\0"
  "vorrq_m_f32\0"
  "vorrq_m_n_s16\0"
  "vorrq_m_n_s32\0"
  "vorrq_m_n_u16\0"
  "vorrq_m_n_u32\0"
  "vorrq_m_s16\0"
  "vorrq_m_s32\0"
  "vorrq_m_s8\0"
  "vorrq_m_u16\0"
  "vorrq_m_u32\0"
  "vorrq_m_u8\0"
  "vorrq_n_s16\0"
  "vorrq_n_s32\0"
  "vorrq_n_u16\0"
  "vorrq_n_u32\0"
  "vorrq_s16\0"
  "vorrq_s32\0"
  "vorrq_s8\0"
  "vorrq_u16\0"
  "vorrq_u32\0"
  "vorrq_u8\0"
  "vorrq_x_f16\0"
  "vorrq_x_f32\0"
  "vorrq_x_s16\0"
  "vorrq_x_s32\0"
  "vorrq_x_s8\0"
  "vorrq_x_u16\0"
  "vorrq_x_u32\0"
  "vorrq_x_u8\0"
  "vpnot\0"
  "vpselq_f16\0"
  "vpselq_f32\0"
  "vpselq_s16\0"
  "vpselq_s32\0"
  "vpselq_s64\0"
  "vpselq_s8\0"
  "vpselq_u16\0"
  "vpselq_u32\0"
  "vpselq_u64\0"
  "vpselq_u8\0"
  "vqabsq_m_s16\0"
  "vqabsq_m_s32\0"
  "vqabsq_m_s8\0"
  "vqabsq_s16\0"
  "vqabsq_s32\0"
  "vqabsq_s8\0"
  "vqaddq_m_n_s16\0"
  "vqaddq_m_n_s32\0"
  "vqaddq_m_n_s8\0"
  "vqaddq_m_n_u16\0"
  "vqaddq_m_n_u32\0"
  "vqaddq_m_n_u8\0"
  "vqaddq_m_s16\0"
  "vqaddq_m_s32\0"
  "vqaddq_m_s8\0"
  "vqaddq_m_u16\0"
  "vqaddq_m_u32\0"
  "vqaddq_m_u8\0"
  "vqaddq_n_s16\0"
  "vqaddq_n_s32\0"
  "vqaddq_n_s8\0"
  "vqaddq_n_u16\0"
  "vqaddq_n_u32\0"
  "vqaddq_n_u8\0"
  "vqaddq_s16\0"
  "vqaddq_s32\0"
  "vqaddq_s8\0"
  "vqaddq_u16\0"
  "vqaddq_u32\0"
  "vqaddq_u8\0"
  "vqdmladhq_m_s16\0"
  "vqdmladhq_m_s32\0"
  "vqdmladhq_m_s8\0"
  "vqdmladhq_s16\0"
  "vqdmladhq_s32\0"
  "vqdmladhq_s8\0"
  "vqdmladhxq_m_s16\0"
  "vqdmladhxq_m_s32\0"
  "vqdmladhxq_m_s8\0"
  "vqdmladhxq_s16\0"
  "vqdmladhxq_s32\0"
  "vqdmladhxq_s8\0"
  "vqdmlahq_m_n_s16\0"
  "vqdmlahq_m_n_s32\0"
  "vqdmlahq_m_n_s8\0"
  "vqdmlahq_n_s16\0"
  "vqdmlahq_n_s32\0"
  "vqdmlahq_n_s8\0"
  "vqdmlashq_m_n_s16\0"
  "vqdmlashq_m_n_s32\0"
  "vqdmlashq_m_n_s8\0"
  "vqdmlashq_n_s16\0"
  "vqdmlashq_n_s32\0"
  "vqdmlashq_n_s8\0"
  "vqdmlsdhq_m_s16\0"
  "vqdmlsdhq_m_s32\0"
  "vqdmlsdhq_m_s8\0"
  "vqdmlsdhq_s16\0"
  "vqdmlsdhq_s32\0"
  "vqdmlsdhq_s8\0"
  "vqdmlsdhxq_m_s16\0"
  "vqdmlsdhxq_m_s32\0"
  "vqdmlsdhxq_m_s8\0"
  "vqdmlsdhxq_s16\0"
  "vqdmlsdhxq_s32\0"
  "vqdmlsdhxq_s8\0"
  "vqdmulhq_m_n_s16\0"
  "vqdmulhq_m_n_s32\0"
  "vqdmulhq_m_n_s8\0"
  "vqdmulhq_m_s16\0"
  "vqdmulhq_m_s32\0"
  "vqdmulhq_m_s8\0"
  "vqdmulhq_n_s16\0"
  "vqdmulhq_n_s32\0"
  "vqdmulhq_n_s8\0"
  "vqdmulhq_s16\0"
  "vqdmulhq_s32\0"
  "vqdmulhq_s8\0"
  "vqdmullbq_m_n_s16\0"
  "vqdmullbq_m_n_s32\0"
  "vqdmullbq_m_s16\0"
  "vqdmullbq_m_s32\0"
  "vqdmullbq_n_s16\0"
  "vqdmullbq_n_s32\0"
  "vqdmullbq_s16\0"
  "vqdmullbq_s32\0"
  "vqdmulltq_m_n_s16\0"
  "vqdmulltq_m_n_s32\0"
  "vqdmulltq_m_s16\0"
  "vqdmulltq_m_s32\0"
  "vqdmulltq_n_s16\0"
  "vqdmulltq_n_s32\0"
  "vqdmulltq_s16\0"
  "vqdmulltq_s32\0"
  "vqmovnbq_m_s16\0"
  "vqmovnbq_m_s32\0"
  "vqmovnbq_m_u16\0"
  "vqmovnbq_m_u32\0"
  "vqmovnbq_s16\0"
  "vqmovnbq_s32\0"
  "vqmovnbq_u16\0"
  "vqmovnbq_u32\0"
  "vqmovntq_m_s16\0"
  "vqmovntq_m_s32\0"
  "vqmovntq_m_u16\0"
  "vqmovntq_m_u32\0"
  "vqmovntq_s16\0"
  "vqmovntq_s32\0"
  "vqmovntq_u16\0"
  "vqmovntq_u32\0"
  "vqmovunbq_m_s16\0"
  "vqmovunbq_m_s32\0"
  "vqmovunbq_s16\0"
  "vqmovunbq_s32\0"
  "vqmovuntq_m_s16\0"
  "vqmovuntq_m_s32\0"
  "vqmovuntq_s16\0"
  "vqmovuntq_s32\0"
  "vqnegq_m_s16\0"
  "vqnegq_m_s32\0"
  "vqnegq_m_s8\0"
  "vqnegq_s16\0"
  "vqnegq_s32\0"
  "vqnegq_s8\0"
  "vqrdmladhq_m_s16\0"
  "vqrdmladhq_m_s32\0"
  "vqrdmladhq_m_s8\0"
  "vqrdmladhq_s16\0"
  "vqrdmladhq_s32\0"
  "vqrdmladhq_s8\0"
  "vqrdmladhxq_m_s16\0"
  "vqrdmladhxq_m_s32\0"
  "vqrdmladhxq_m_s8\0"
  "vqrdmladhxq_s16\0"
  "vqrdmladhxq_s32\0"
  "vqrdmladhxq_s8\0"
  "vqrdmlahq_m_n_s16\0"
  "vqrdmlahq_m_n_s32\0"
  "vqrdmlahq_m_n_s8\0"
  "vqrdmlahq_n_s16\0"
  "vqrdmlahq_n_s32\0"
  "vqrdmlahq_n_s8\0"
  "vqrdmlashq_m_n_s16\0"
  "vqrdmlashq_m_n_s32\0"
  "vqrdmlashq_m_n_s8\0"
  "vqrdmlashq_n_s16\0"
  "vqrdmlashq_n_s32\0"
  "vqrdmlashq_n_s8\0"
  "vqrdmlsdhq_m_s16\0"
  "vqrdmlsdhq_m_s32\0"
  "vqrdmlsdhq_m_s8\0"
  "vqrdmlsdhq_s16\0"
  "vqrdmlsdhq_s32\0"
  "vqrdmlsdhq_s8\0"
  "vqrdmlsdhxq_m_s16\0"
  "vqrdmlsdhxq_m_s32\0"
  "vqrdmlsdhxq_m_s8\0"
  "vqrdmlsdhxq_s16\0"
  "vqrdmlsdhxq_s32\0"
  "vqrdmlsdhxq_s8\0"
  "vqrdmulhq_m_n_s16\0"
  "vqrdmulhq_m_n_s32\0"
  "vqrdmulhq_m_n_s8\0"
  "vqrdmulhq_m_s16\0"
  "vqrdmulhq_m_s32\0"
  "vqrdmulhq_m_s8\0"
  "vqrdmulhq_n_s16\0"
  "vqrdmulhq_n_s32\0"
  "vqrdmulhq_n_s8\0"
  "vqrdmulhq_s16\0"
  "vqrdmulhq_s32\0"
  "vqrdmulhq_s8\0"
  "vqrshlq_m_n_s16\0"
  "vqrshlq_m_n_s32\0"
  "vqrshlq_m_n_s8\0"
  "vqrshlq_m_n_u16\0"
  "vqrshlq_m_n_u32\0"
  "vqrshlq_m_n_u8\0"
  "vqrshlq_m_s16\0"
  "vqrshlq_m_s32\0"
  "vqrshlq_m_s8\0"
  "vqrshlq_m_u16\0"
  "vqrshlq_m_u32\0"
  "vqrshlq_m_u8\0"
  "vqrshlq_n_s16\0"
  "vqrshlq_n_s32\0"
  "vqrshlq_n_s8\0"
  "vqrshlq_n_u16\0"
  "vqrshlq_n_u32\0"
  "vqrshlq_n_u8\0"
  "vqrshlq_s16\0"
  "vqrshlq_s32\0"
  "vqrshlq_s8\0"
  "vqrshlq_u16\0"
  "vqrshlq_u32\0"
  "vqrshlq_u8\0"
  "vqrshrnbq_m_n_s16\0"
  "vqrshrnbq_m_n_s32\0"
  "vqrshrnbq_m_n_u16\0"
  "vqrshrnbq_m_n_u32\0"
  "vqrshrnbq_n_s16\0"
  "vqrshrnbq_n_s32\0"
  "vqrshrnbq_n_u16\0"
  "vqrshrnbq_n_u32\0"
  "vqrshrntq_m_n_s16\0"
  "vqrshrntq_m_n_s32\0"
  "vqrshrntq_m_n_u16\0"
  "vqrshrntq_m_n_u32\0"
  "vqrshrntq_n_s16\0"
  "vqrshrntq_n_s32\0"
  "vqrshrntq_n_u16\0"
  "vqrshrntq_n_u32\0"
  "vqrshrunbq_m_n_s16\0"
  "vqrshrunbq_m_n_s32\0"
  "vqrshrunbq_n_s16\0"
  "vqrshrunbq_n_s32\0"
  "vqrshruntq_m_n_s16\0"
  "vqrshruntq_m_n_s32\0"
  "vqrshruntq_n_s16\0"
  "vqrshruntq_n_s32\0"
  "vqshlq_m_n_s16\0"
  "vqshlq_m_n_s32\0"
  "vqshlq_m_n_s8\0"
  "vqshlq_m_n_u16\0"
  "vqshlq_m_n_u32\0"
  "vqshlq_m_n_u8\0"
  "vqshlq_m_r_s16\0"
  "vqshlq_m_r_s32\0"
  "vqshlq_m_r_s8\0"
  "vqshlq_m_r_u16\0"
  "vqshlq_m_r_u32\0"
  "vqshlq_m_r_u8\0"
  "vqshlq_m_s16\0"
  "vqshlq_m_s32\0"
  "vqshlq_m_s8\0"
  "vqshlq_m_u16\0"
  "vqshlq_m_u32\0"
  "vqshlq_m_u8\0"
  "vqshlq_n_s16\0"
  "vqshlq_n_s32\0"
  "vqshlq_n_s8\0"
  "vqshlq_n_u16\0"
  "vqshlq_n_u32\0"
  "vqshlq_n_u8\0"
  "vqshlq_r_s16\0"
  "vqshlq_r_s32\0"
  "vqshlq_r_s8\0"
  "vqshlq_r_u16\0"
  "vqshlq_r_u32\0"
  "vqshlq_r_u8\0"
  "vqshlq_s16\0"
  "vqshlq_s32\0"
  "vqshlq_s8\0"
  "vqshlq_u16\0"
  "vqshlq_u32\0"
  "vqshlq_u8\0"
  "vqshluq_m_n_s16\0"
  "vqshluq_m_n_s32\0"
  "vqshluq_m_n_s8\0"
  "vqshluq_n_s16\0"
  "vqshluq_n_s32\0"
  "vqshluq_n_s8\0"
  "vqshrnbq_m_n_s16\0"
  "vqshrnbq_m_n_s32\0"
  "vqshrnbq_m_n_u16\0"
  "vqshrnbq_m_n_u32\0"
  "vqshrnbq_n_s16\0"
  "vqshrnbq_n_s32\0"
  "vqshrnbq_n_u16\0"
  "vqshrnbq_n_u32\0"
  "vqshrntq_m_n_s16\0"
  "vqshrntq_m_n_s32\0"
  "vqshrntq_m_n_u16\0"
  "vqshrntq_m_n_u32\0"
  "vqshrntq_n_s16\0"
  "vqshrntq_n_s32\0"
  "vqshrntq_n_u16\0"
  "vqshrntq_n_u32\0"
  "vqshrunbq_m_n_s16\0"
  "vqshrunbq_m_n_s32\0"
  "vqshrunbq_n_s16\0"
  "vqshrunbq_n_s32\0"
  "vqshruntq_m_n_s16\0"
  "vqshruntq_m_n_s32\0"
  "vqshruntq_n_s16\0"
  "vqshruntq_n_s32\0"
  "vqsubq_m_n_s16\0"
  "vqsubq_m_n_s32\0"
  "vqsubq_m_n_s8\0"
  "vqsubq_m_n_u16\0"
  "vqsubq_m_n_u32\0"
  "vqsubq_m_n_u8\0"
  "vqsubq_m_s16\0"
  "vqsubq_m_s32\0"
  "vqsubq_m_s8\0"
  "vqsubq_m_u16\0"
  "vqsubq_m_u32\0"
  "vqsubq_m_u8\0"
  "vqsubq_n_s16\0"
  "vqsubq_n_s32\0"
  "vqsubq_n_s8\0"
  "vqsubq_n_u16\0"
  "vqsubq_n_u32\0"
  "vqsubq_n_u8\0"
  "vqsubq_s16\0"
  "vqsubq_s32\0"
  "vqsubq_s8\0"
  "vqsubq_u16\0"
  "vqsubq_u32\0"
  "vqsubq_u8\0"
  "vreinterpretq_f16_f32\0"
  "vreinterpretq_f16_s16\0"
  "vreinterpretq_f16_s32\0"
  "vreinterpretq_f16_s64\0"
  "vreinterpretq_f16_s8\0"
  "vreinterpretq_f16_u16\0"
  "vreinterpretq_f16_u32\0"
  "vreinterpretq_f16_u64\0"
  "vreinterpretq_f16_u8\0"
  "vreinterpretq_f32_f16\0"
  "vreinterpretq_f32_s16\0"
  "vreinterpretq_f32_s32\0"
  "vreinterpretq_f32_s64\0"
  "vreinterpretq_f32_s8\0"
  "vreinterpretq_f32_u16\0"
  "vreinterpretq_f32_u32\0"
  "vreinterpretq_f32_u64\0"
  "vreinterpretq_f32_u8\0"
  "vreinterpretq_s16_f16\0"
  "vreinterpretq_s16_f32\0"
  "vreinterpretq_s16_s32\0"
  "vreinterpretq_s16_s64\0"
  "vreinterpretq_s16_s8\0"
  "vreinterpretq_s16_u16\0"
  "vreinterpretq_s16_u32\0"
  "vreinterpretq_s16_u64\0"
  "vreinterpretq_s16_u8\0"
  "vreinterpretq_s32_f16\0"
  "vreinterpretq_s32_f32\0"
  "vreinterpretq_s32_s16\0"
  "vreinterpretq_s32_s64\0"
  "vreinterpretq_s32_s8\0"
  "vreinterpretq_s32_u16\0"
  "vreinterpretq_s32_u32\0"
  "vreinterpretq_s32_u64\0"
  "vreinterpretq_s32_u8\0"
  "vreinterpretq_s64_f16\0"
  "vreinterpretq_s64_f32\0"
  "vreinterpretq_s64_s16\0"
  "vreinterpretq_s64_s32\0"
  "vreinterpretq_s64_s8\0"
  "vreinterpretq_s64_u16\0"
  "vreinterpretq_s64_u32\0"
  "vreinterpretq_s64_u64\0"
  "vreinterpretq_s64_u8\0"
  "vreinterpretq_s8_f16\0"
  "vreinterpretq_s8_f32\0"
  "vreinterpretq_s8_s16\0"
  "vreinterpretq_s8_s32\0"
  "vreinterpretq_s8_s64\0"
  "vreinterpretq_s8_u16\0"
  "vreinterpretq_s8_u32\0"
  "vreinterpretq_s8_u64\0"
  "vreinterpretq_s8_u8\0"
  "vreinterpretq_u16_f16\0"
  "vreinterpretq_u16_f32\0"
  "vreinterpretq_u16_s16\0"
  "vreinterpretq_u16_s32\0"
  "vreinterpretq_u16_s64\0"
  "vreinterpretq_u16_s8\0"
  "vreinterpretq_u16_u32\0"
  "vreinterpretq_u16_u64\0"
  "vreinterpretq_u16_u8\0"
  "vreinterpretq_u32_f16\0"
  "vreinterpretq_u32_f32\0"
  "vreinterpretq_u32_s16\0"
  "vreinterpretq_u32_s32\0"
  "vreinterpretq_u32_s64\0"
  "vreinterpretq_u32_s8\0"
  "vreinterpretq_u32_u16\0"
  "vreinterpretq_u32_u64\0"
  "vreinterpretq_u32_u8\0"
  "vreinterpretq_u64_f16\0"
  "vreinterpretq_u64_f32\0"
  "vreinterpretq_u64_s16\0"
  "vreinterpretq_u64_s32\0"
  "vreinterpretq_u64_s64\0"
  "vreinterpretq_u64_s8\0"
  "vreinterpretq_u64_u16\0"
  "vreinterpretq_u64_u32\0"
  "vreinterpretq_u64_u8\0"
  "vreinterpretq_u8_f16\0"
  "vreinterpretq_u8_f32\0"
  "vreinterpretq_u8_s16\0"
  "vreinterpretq_u8_s32\0"
  "vreinterpretq_u8_s64\0"
  "vreinterpretq_u8_s8\0"
  "vreinterpretq_u8_u16\0"
  "vreinterpretq_u8_u32\0"
  "vreinterpretq_u8_u64\0"
  "vrev16q_m_s8\0"
  "vrev16q_m_u8\0"
  "vrev16q_s8\0"
  "vrev16q_u8\0"
  "vrev16q_x_s8\0"
  "vrev16q_x_u8\0"
  "vrev32q_f16\0"
  "vrev32q_m_f16\0"
  "vrev32q_m_s16\0"
  "vrev32q_m_s8\0"
  "vrev32q_m_u16\0"
  "vrev32q_m_u8\0"
  "vrev32q_s16\0"
  "vrev32q_s8\0"
  "vrev32q_u16\0"
  "vrev32q_u8\0"
  "vrev32q_x_f16\0"
  "vrev32q_x_s16\0"
  "vrev32q_x_s8\0"
  "vrev32q_x_u16\0"
  "vrev32q_x_u8\0"
  "vrev64q_f16\0"
  "vrev64q_f32\0"
  "vrev64q_m_f16\0"
  "vrev64q_m_f32\0"
  "vrev64q_m_s16\0"
  "vrev64q_m_s32\0"
  "vrev64q_m_s8\0"
  "vrev64q_m_u16\0"
  "vrev64q_m_u32\0"
  "vrev64q_m_u8\0"
  "vrev64q_s16\0"
  "vrev64q_s32\0"
  "vrev64q_s8\0"
  "vrev64q_u16\0"
  "vrev64q_u32\0"
  "vrev64q_u8\0"
  "vrev64q_x_f16\0"
  "vrev64q_x_f32\0"
  "vrev64q_x_s16\0"
  "vrev64q_x_s32\0"
  "vrev64q_x_s8\0"
  "vrev64q_x_u16\0"
  "vrev64q_x_u32\0"
  "vrev64q_x_u8\0"
  "vrhaddq_m_s16\0"
  "vrhaddq_m_s32\0"
  "vrhaddq_m_s8\0"
  "vrhaddq_m_u16\0"
  "vrhaddq_m_u32\0"
  "vrhaddq_m_u8\0"
  "vrhaddq_s16\0"
  "vrhaddq_s32\0"
  "vrhaddq_s8\0"
  "vrhaddq_u16\0"
  "vrhaddq_u32\0"
  "vrhaddq_u8\0"
  "vrhaddq_x_s16\0"
  "vrhaddq_x_s32\0"
  "vrhaddq_x_s8\0"
  "vrhaddq_x_u16\0"
  "vrhaddq_x_u32\0"
  "vrhaddq_x_u8\0"
  "vrmlaldavhaq_p_s32\0"
  "vrmlaldavhaq_p_u32\0"
  "vrmlaldavhaq_s32\0"
  "vrmlaldavhaq_u32\0"
  "vrmlaldavhaxq_p_s32\0"
  "vrmlaldavhaxq_s32\0"
  "vrmlaldavhq_p_s32\0"
  "vrmlaldavhq_p_u32\0"
  "vrmlaldavhq_s32\0"
  "vrmlaldavhq_u32\0"
  "vrmlaldavhxq_p_s32\0"
  "vrmlaldavhxq_s32\0"
  "vrmlsldavhaq_p_s32\0"
  "vrmlsldavhaq_s32\0"
  "vrmlsldavhaxq_p_s32\0"
  "vrmlsldavhaxq_s32\0"
  "vrmlsldavhq_p_s32\0"
  "vrmlsldavhq_s32\0"
  "vrmlsldavhxq_p_s32\0"
  "vrmlsldavhxq_s32\0"
  "vrmulhq_m_s16\0"
  "vrmulhq_m_s32\0"
  "vrmulhq_m_s8\0"
  "vrmulhq_m_u16\0"
  "vrmulhq_m_u32\0"
  "vrmulhq_m_u8\0"
  "vrmulhq_s16\0"
  "vrmulhq_s32\0"
  "vrmulhq_s8\0"
  "vrmulhq_u16\0"
  "vrmulhq_u32\0"
  "vrmulhq_u8\0"
  "vrmulhq_x_s16\0"
  "vrmulhq_x_s32\0"
  "vrmulhq_x_s8\0"
  "vrmulhq_x_u16\0"
  "vrmulhq_x_u32\0"
  "vrmulhq_x_u8\0"
  "vrndaq_f16\0"
  "vrndaq_f32\0"
  "vrndaq_m_f16\0"
  "vrndaq_m_f32\0"
  "vrndaq_x_f16\0"
  "vrndaq_x_f32\0"
  "vrndmq_f16\0"
  "vrndmq_f32\0"
  "vrndmq_m_f16\0"
  "vrndmq_m_f32\0"
  "vrndmq_x_f16\0"
  "vrndmq_x_f32\0"
  "vrndnq_f16\0"
  "vrndnq_f32\0"
  "vrndnq_m_f16\0"
  "vrndnq_m_f32\0"
  "vrndnq_x_f16\0"
  "vrndnq_x_f32\0"
  "vrndpq_f16\0"
  "vrndpq_f32\0"
  "vrndpq_m_f16\0"
  "vrndpq_m_f32\0"
  "vrndpq_x_f16\0"
  "vrndpq_x_f32\0"
  "vrndq_f16\0"
  "vrndq_f32\0"
  "vrndq_m_f16\0"
  "vrndq_m_f32\0"
  "vrndq_x_f16\0"
  "vrndq_x_f32\0"
  "vrndxq_f16\0"
  "vrndxq_f32\0"
  "vrndxq_m_f16\0"
  "vrndxq_m_f32\0"
  "vrndxq_x_f16\0"
  "vrndxq_x_f32\0"
  "vrshlq_m_n_s16\0"
  "vrshlq_m_n_s32\0"
  "vrshlq_m_n_s8\0"
  "vrshlq_m_n_u16\0"
  "vrshlq_m_n_u32\0"
  "vrshlq_m_n_u8\0"
  "vrshlq_m_s16\0"
  "vrshlq_m_s32\0"
  "vrshlq_m_s8\0"
  "vrshlq_m_u16\0"
  "vrshlq_m_u32\0"
  "vrshlq_m_u8\0"
  "vrshlq_n_s16\0"
  "vrshlq_n_s32\0"
  "vrshlq_n_s8\0"
  "vrshlq_n_u16\0"
  "vrshlq_n_u32\0"
  "vrshlq_n_u8\0"
  "vrshlq_s16\0"
  "vrshlq_s32\0"
  "vrshlq_s8\0"
  "vrshlq_u16\0"
  "vrshlq_u32\0"
  "vrshlq_u8\0"
  "vrshlq_x_s16\0"
  "vrshlq_x_s32\0"
  "vrshlq_x_s8\0"
  "vrshlq_x_u16\0"
  "vrshlq_x_u32\0"
  "vrshlq_x_u8\0"
  "vrshrnbq_m_n_s16\0"
  "vrshrnbq_m_n_s32\0"
  "vrshrnbq_m_n_u16\0"
  "vrshrnbq_m_n_u32\0"
  "vrshrnbq_n_s16\0"
  "vrshrnbq_n_s32\0"
  "vrshrnbq_n_u16\0"
  "vrshrnbq_n_u32\0"
  "vrshrntq_m_n_s16\0"
  "vrshrntq_m_n_s32\0"
  "vrshrntq_m_n_u16\0"
  "vrshrntq_m_n_u32\0"
  "vrshrntq_n_s16\0"
  "vrshrntq_n_s32\0"
  "vrshrntq_n_u16\0"
  "vrshrntq_n_u32\0"
  "vrshrq_m_n_s16\0"
  "vrshrq_m_n_s32\0"
  "vrshrq_m_n_s8\0"
  "vrshrq_m_n_u16\0"
  "vrshrq_m_n_u32\0"
  "vrshrq_m_n_u8\0"
  "vrshrq_n_s16\0"
  "vrshrq_n_s32\0"
  "vrshrq_n_s8\0"
  "vrshrq_n_u16\0"
  "vrshrq_n_u32\0"
  "vrshrq_n_u8\0"
  "vrshrq_x_n_s16\0"
  "vrshrq_x_n_s32\0"
  "vrshrq_x_n_s8\0"
  "vrshrq_x_n_u16\0"
  "vrshrq_x_n_u32\0"
  "vrshrq_x_n_u8\0"
  "vsbciq_m_s32\0"
  "vsbciq_m_u32\0"
  "vsbciq_s32\0"
  "vsbciq_u32\0"
  "vsbcq_m_s32\0"
  "vsbcq_m_u32\0"
  "vsbcq_s32\0"
  "vsbcq_u32\0"
  "vsetq_lane_f16\0"
  "vsetq_lane_f32\0"
  "vsetq_lane_s16\0"
  "vsetq_lane_s32\0"
  "vsetq_lane_s64\0"
  "vsetq_lane_s8\0"
  "vsetq_lane_u16\0"
  "vsetq_lane_u32\0"
  "vsetq_lane_u64\0"
  "vsetq_lane_u8\0"
  "vshlcq_m_s16\0"
  "vshlcq_m_s32\0"
  "vshlcq_m_s8\0"
  "vshlcq_m_u16\0"
  "vshlcq_m_u32\0"
  "vshlcq_m_u8\0"
  "vshlcq_s16\0"
  "vshlcq_s32\0"
  "vshlcq_s8\0"
  "vshlcq_u16\0"
  "vshlcq_u32\0"
  "vshlcq_u8\0"
  "vshllbq_m_n_s16\0"
  "vshllbq_m_n_s8\0"
  "vshllbq_m_n_u16\0"
  "vshllbq_m_n_u8\0"
  "vshllbq_n_s16\0"
  "vshllbq_n_s8\0"
  "vshllbq_n_u16\0"
  "vshllbq_n_u8\0"
  "vshllbq_x_n_s16\0"
  "vshllbq_x_n_s8\0"
  "vshllbq_x_n_u16\0"
  "vshllbq_x_n_u8\0"
  "vshlltq_m_n_s16\0"
  "vshlltq_m_n_s8\0"
  "vshlltq_m_n_u16\0"
  "vshlltq_m_n_u8\0"
  "vshlltq_n_s16\0"
  "vshlltq_n_s8\0"
  "vshlltq_n_u16\0"
  "vshlltq_n_u8\0"
  "vshlltq_x_n_s16\0"
  "vshlltq_x_n_s8\0"
  "vshlltq_x_n_u16\0"
  "vshlltq_x_n_u8\0"
  "vshlq_m_n_s16\0"
  "vshlq_m_n_s32\0"
  "vshlq_m_n_s8\0"
  "vshlq_m_n_u16\0"
  "vshlq_m_n_u32\0"
  "vshlq_m_n_u8\0"
  "vshlq_m_r_s16\0"
  "vshlq_m_r_s32\0"
  "vshlq_m_r_s8\0"
  "vshlq_m_r_u16\0"
  "vshlq_m_r_u32\0"
  "vshlq_m_r_u8\0"
  "vshlq_m_s16\0"
  "vshlq_m_s32\0"
  "vshlq_m_s8\0"
  "vshlq_m_u16\0"
  "vshlq_m_u32\0"
  "vshlq_m_u8\0"
  "vshlq_n_s16\0"
  "vshlq_n_s32\0"
  "vshlq_n_s8\0"
  "vshlq_n_u16\0"
  "vshlq_n_u32\0"
  "vshlq_n_u8\0"
  "vshlq_r_s16\0"
  "vshlq_r_s32\0"
  "vshlq_r_s8\0"
  "vshlq_r_u16\0"
  "vshlq_r_u32\0"
  "vshlq_r_u8\0"
  "vshlq_s16\0"
  "vshlq_s32\0"
  "vshlq_s8\0"
  "vshlq_u16\0"
  "vshlq_u32\0"
  "vshlq_u8\0"
  "vshlq_x_n_s16\0"
  "vshlq_x_n_s32\0"
  "vshlq_x_n_s8\0"
  "vshlq_x_n_u16\0"
  "vshlq_x_n_u32\0"
  "vshlq_x_n_u8\0"
  "vshlq_x_s16\0"
  "vshlq_x_s32\0"
  "vshlq_x_s8\0"
  "vshlq_x_u16\0"
  "vshlq_x_u32\0"
  "vshlq_x_u8\0"
  "vshrnbq_m_n_s16\0"
  "vshrnbq_m_n_s32\0"
  "vshrnbq_m_n_u16\0"
  "vshrnbq_m_n_u32\0"
  "vshrnbq_n_s16\0"
  "vshrnbq_n_s32\0"
  "vshrnbq_n_u16\0"
  "vshrnbq_n_u32\0"
  "vshrntq_m_n_s16\0"
  "vshrntq_m_n_s32\0"
  "vshrntq_m_n_u16\0"
  "vshrntq_m_n_u32\0"
  "vshrntq_n_s16\0"
  "vshrntq_n_s32\0"
  "vshrntq_n_u16\0"
  "vshrntq_n_u32\0"
  "vshrq_m_n_s16\0"
  "vshrq_m_n_s32\0"
  "vshrq_m_n_s8\0"
  "vshrq_m_n_u16\0"
  "vshrq_m_n_u32\0"
  "vshrq_m_n_u8\0"
  "vshrq_n_s16\0"
  "vshrq_n_s32\0"
  "vshrq_n_s8\0"
  "vshrq_n_u16\0"
  "vshrq_n_u32\0"
  "vshrq_n_u8\0"
  "vshrq_x_n_s16\0"
  "vshrq_x_n_s32\0"
  "vshrq_x_n_s8\0"
  "vshrq_x_n_u16\0"
  "vshrq_x_n_u32\0"
  "vshrq_x_n_u8\0"
  "vsliq_m_n_s16\0"
  "vsliq_m_n_s32\0"
  "vsliq_m_n_s8\0"
  "vsliq_m_n_u16\0"
  "vsliq_m_n_u32\0"
  "vsliq_m_n_u8\0"
  "vsliq_n_s16\0"
  "vsliq_n_s32\0"
  "vsliq_n_s8\0"
  "vsliq_n_u16\0"
  "vsliq_n_u32\0"
  "vsliq_n_u8\0"
  "vsriq_m_n_s16\0"
  "vsriq_m_n_s32\0"
  "vsriq_m_n_s8\0"
  "vsriq_m_n_u16\0"
  "vsriq_m_n_u32\0"
  "vsriq_m_n_u8\0"
  "vsriq_n_s16\0"
  "vsriq_n_s32\0"
  "vsriq_n_s8\0"
  "vsriq_n_u16\0"
  "vsriq_n_u32\0"
  "vsriq_n_u8\0"
  "vst1q_f16\0"
  "vst1q_f32\0"
  "vst1q_p_f16\0"
  "vst1q_p_f32\0"
  "vst1q_p_s16\0"
  "vst1q_p_s32\0"
  "vst1q_p_s8\0"
  "vst1q_p_u16\0"
  "vst1q_p_u32\0"
  "vst1q_p_u8\0"
  "vst1q_s16\0"
  "vst1q_s32\0"
  "vst1q_s8\0"
  "vst1q_u16\0"
  "vst1q_u32\0"
  "vst1q_u8\0"
  "vst2q_f16\0"
  "vst2q_f32\0"
  "vst2q_s16\0"
  "vst2q_s32\0"
  "vst2q_s8\0"
  "vst2q_u16\0"
  "vst2q_u32\0"
  "vst2q_u8\0"
  "vst4q_f16\0"
  "vst4q_f32\0"
  "vst4q_s16\0"
  "vst4q_s32\0"
  "vst4q_s8\0"
  "vst4q_u16\0"
  "vst4q_u32\0"
  "vst4q_u8\0"
  "vstrbq_p_s16\0"
  "vstrbq_p_s32\0"
  "vstrbq_p_s8\0"
  "vstrbq_p_u16\0"
  "vstrbq_p_u32\0"
  "vstrbq_p_u8\0"
  "vstrbq_s16\0"
  "vstrbq_s32\0"
  "vstrbq_s8\0"
  "vstrbq_scatter_offset_p_s16\0"
  "vstrbq_scatter_offset_p_s32\0"
  "vstrbq_scatter_offset_p_s8\0"
  "vstrbq_scatter_offset_p_u16\0"
  "vstrbq_scatter_offset_p_u32\0"
  "vstrbq_scatter_offset_p_u8\0"
  "vstrbq_scatter_offset_s16\0"
  "vstrbq_scatter_offset_s32\0"
  "vstrbq_scatter_offset_s8\0"
  "vstrbq_scatter_offset_u16\0"
  "vstrbq_scatter_offset_u32\0"
  "vstrbq_scatter_offset_u8\0"
  "vstrbq_u16\0"
  "vstrbq_u32\0"
  "vstrbq_u8\0"
  "vstrdq_scatter_base_p_s64\0"
  "vstrdq_scatter_base_p_u64\0"
  "vstrdq_scatter_base_s64\0"
  "vstrdq_scatter_base_u64\0"
  "vstrdq_scatter_base_wb_p_s64\0"
  "vstrdq_scatter_base_wb_p_u64\0"
  "vstrdq_scatter_base_wb_s64\0"
  "vstrdq_scatter_base_wb_u64\0"
  "vstrdq_scatter_offset_p_s64\0"
  "vstrdq_scatter_offset_p_u64\0"
  "vstrdq_scatter_offset_s64\0"
  "vstrdq_scatter_offset_u64\0"
  "vstrdq_scatter_shifted_offset_p_s64\0"
  "vstrdq_scatter_shifted_offset_p_u64\0"
  "vstrdq_scatter_shifted_offset_s64\0"
  "vstrdq_scatter_shifted_offset_u64\0"
  "vstrhq_f16\0"
  "vstrhq_p_f16\0"
  "vstrhq_p_s16\0"
  "vstrhq_p_s32\0"
  "vstrhq_p_u16\0"
  "vstrhq_p_u32\0"
  "vstrhq_s16\0"
  "vstrhq_s32\0"
  "vstrhq_scatter_offset_f16\0"
  "vstrhq_scatter_offset_p_f16\0"
  "vstrhq_scatter_offset_p_s16\0"
  "vstrhq_scatter_offset_p_s32\0"
  "vstrhq_scatter_offset_p_u16\0"
  "vstrhq_scatter_offset_p_u32\0"
  "vstrhq_scatter_offset_s16\0"
  "vstrhq_scatter_offset_s32\0"
  "vstrhq_scatter_offset_u16\0"
  "vstrhq_scatter_offset_u32\0"
  "vstrhq_scatter_shifted_offset_f16\0"
  "vstrhq_scatter_shifted_offset_p_f16\0"
  "vstrhq_scatter_shifted_offset_p_s16\0"
  "vstrhq_scatter_shifted_offset_p_s32\0"
  "vstrhq_scatter_shifted_offset_p_u16\0"
  "vstrhq_scatter_shifted_offset_p_u32\0"
  "vstrhq_scatter_shifted_offset_s16\0"
  "vstrhq_scatter_shifted_offset_s32\0"
  "vstrhq_scatter_shifted_offset_u16\0"
  "vstrhq_scatter_shifted_offset_u32\0"
  "vstrhq_u16\0"
  "vstrhq_u32\0"
  "vstrwq_f32\0"
  "vstrwq_p_f32\0"
  "vstrwq_p_s32\0"
  "vstrwq_p_u32\0"
  "vstrwq_s32\0"
  "vstrwq_scatter_base_f32\0"
  "vstrwq_scatter_base_p_f32\0"
  "vstrwq_scatter_base_p_s32\0"
  "vstrwq_scatter_base_p_u32\0"
  "vstrwq_scatter_base_s32\0"
  "vstrwq_scatter_base_u32\0"
  "vstrwq_scatter_base_wb_f32\0"
  "vstrwq_scatter_base_wb_p_f32\0"
  "vstrwq_scatter_base_wb_p_s32\0"
  "vstrwq_scatter_base_wb_p_u32\0"
  "vstrwq_scatter_base_wb_s32\0"
  "vstrwq_scatter_base_wb_u32\0"
  "vstrwq_scatter_offset_f32\0"
  "vstrwq_scatter_offset_p_f32\0"
  "vstrwq_scatter_offset_p_s32\0"
  "vstrwq_scatter_offset_p_u32\0"
  "vstrwq_scatter_offset_s32\0"
  "vstrwq_scatter_offset_u32\0"
  "vstrwq_scatter_shifted_offset_f32\0"
  "vstrwq_scatter_shifted_offset_p_f32\0"
  "vstrwq_scatter_shifted_offset_p_s32\0"
  "vstrwq_scatter_shifted_offset_p_u32\0"
  "vstrwq_scatter_shifted_offset_s32\0"
  "vstrwq_scatter_shifted_offset_u32\0"
  "vstrwq_u32\0"
  "vsubq_f16\0"
  "vsubq_f32\0"
  "vsubq_m_f16\0"
  "vsubq_m_f32\0"
  "vsubq_m_n_f16\0"
  "vsubq_m_n_f32\0"
  "vsubq_m_n_s16\0"
  "vsubq_m_n_s32\0"
  "vsubq_m_n_s8\0"
  "vsubq_m_n_u16\0"
  "vsubq_m_n_u32\0"
  "vsubq_m_n_u8\0"
  "vsubq_m_s16\0"
  "vsubq_m_s32\0"
  "vsubq_m_s8\0"
  "vsubq_m_u16\0"
  "vsubq_m_u32\0"
  "vsubq_m_u8\0"
  "vsubq_n_f16\0"
  "vsubq_n_f32\0"
  "vsubq_n_s16\0"
  "vsubq_n_s32\0"
  "vsubq_n_s8\0"
  "vsubq_n_u16\0"
  "vsubq_n_u32\0"
  "vsubq_n_u8\0"
  "vsubq_s16\0"
  "vsubq_s32\0"
  "vsubq_s8\0"
  "vsubq_u16\0"
  "vsubq_u32\0"
  "vsubq_u8\0"
  "vsubq_x_f16\0"
  "vsubq_x_f32\0"
  "vsubq_x_n_f16\0"
  "vsubq_x_n_f32\0"
  "vsubq_x_n_s16\0"
  "vsubq_x_n_s32\0"
  "vsubq_x_n_s8\0"
  "vsubq_x_n_u16\0"
  "vsubq_x_n_u32\0"
  "vsubq_x_n_u8\0"
  "vsubq_x_s16\0"
  "vsubq_x_s32\0"
  "vsubq_x_s8\0"
  "vsubq_x_u16\0"
  "vsubq_x_u32\0"
  "vsubq_x_u8\0"
  "vuninitializedq_f16\0"
  "vuninitializedq_f32\0"
  "vuninitializedq_polymorphic_f16\0"
  "vuninitializedq_polymorphic_f32\0"
  "vuninitializedq_polymorphic_s16\0"
  "vuninitializedq_polymorphic_s32\0"
  "vuninitializedq_polymorphic_s64\0"
  "vuninitializedq_polymorphic_s8\0"
  "vuninitializedq_polymorphic_u16\0"
  "vuninitializedq_polymorphic_u32\0"
  "vuninitializedq_polymorphic_u64\0"
  "vuninitializedq_polymorphic_u8\0"
  "vuninitializedq_s16\0"
  "vuninitializedq_s32\0"
  "vuninitializedq_s64\0"
  "vuninitializedq_s8\0"
  "vuninitializedq_u16\0"
  "vuninitializedq_u32\0"
  "vuninitializedq_u64\0"
  "vuninitializedq_u8\0"
  "vabavq_p\0"
  "vabavq\0"
  "vabdq\0"
  "vabdq_m\0"
  "vabdq_x\0"
  "vabsq\0"
  "vabsq_m\0"
  "vabsq_x\0"
  "vadciq_m\0"
  "vadciq\0"
  "vadcq_m\0"
  "vadcq\0"
  "vaddlvaq_p\0"
  "vaddlvaq\0"
  "vaddlvq_p\0"
  "vaddlvq\0"
  "vaddq\0"
  "vaddq_m\0"
  "vaddq_x\0"
  "vaddvaq_p\0"
  "vaddvaq\0"
  "vaddvq_p\0"
  "vaddvq\0"
  "vandq\0"
  "vandq_m\0"
  "vandq_x\0"
  "vbicq\0"
  "vbicq_m\0"
  "vbicq_m_n\0"
  "vbicq_x\0"
  "vbrsrq_m\0"
  "vbrsrq\0"
  "vbrsrq_x\0"
  "vcaddq_rot270\0"
  "vcaddq_rot270_m\0"
  "vcaddq_rot270_x\0"
  "vcaddq_rot90\0"
  "vcaddq_rot90_m\0"
  "vcaddq_rot90_x\0"
  "vclsq_m\0"
  "vclsq\0"
  "vclsq_x\0"
  "vclzq_m\0"
  "vclzq\0"
  "vclzq_x\0"
  "vcmlaq\0"
  "vcmlaq_m\0"
  "vcmlaq_rot180\0"
  "vcmlaq_rot180_m\0"
  "vcmlaq_rot270\0"
  "vcmlaq_rot270_m\0"
  "vcmlaq_rot90\0"
  "vcmlaq_rot90_m\0"
  "vcmpcsq_m\0"
  "vcmpcsq\0"
  "vcmpeqq\0"
  "vcmpeqq_m\0"
  "vcmpgeq\0"
  "vcmpgeq_m\0"
  "vcmpgtq\0"
  "vcmpgtq_m\0"
  "vcmphiq_m\0"
  "vcmphiq\0"
  "vcmpleq\0"
  "vcmpleq_m\0"
  "vcmpltq\0"
  "vcmpltq_m\0"
  "vcmpneq\0"
  "vcmpneq_m\0"
  "vcmulq\0"
  "vcmulq_m\0"
  "vcmulq_rot180\0"
  "vcmulq_rot180_m\0"
  "vcmulq_rot180_x\0"
  "vcmulq_rot270\0"
  "vcmulq_rot270_m\0"
  "vcmulq_rot270_x\0"
  "vcmulq_rot90\0"
  "vcmulq_rot90_m\0"
  "vcmulq_rot90_x\0"
  "vcmulq_x\0"
  "vcvtaq_m\0"
  "vcvtmq_m\0"
  "vcvtnq_m\0"
  "vcvtpq_m\0"
  "vcvtq\0"
  "vcvtq_m\0"
  "vcvtq_m_n\0"
  "vcvtq_n\0"
  "vcvtq_x\0"
  "vcvtq_x_n\0"
  "vddupq_m\0"
  "vddupq_u16\0"
  "vddupq_u32\0"
  "vddupq_u8\0"
  "vddupq_x_u16\0"
  "vddupq_x_u32\0"
  "vddupq_x_u8\0"
  "vdupq_m\0"
  "vdwdupq_m\0"
  "vdwdupq_u16\0"
  "vdwdupq_u32\0"
  "vdwdupq_u8\0"
  "vdwdupq_x_u16\0"
  "vdwdupq_x_u32\0"
  "vdwdupq_x_u8\0"
  "veorq\0"
  "veorq_m\0"
  "veorq_x\0"
  "vfmaq\0"
  "vfmaq_m\0"
  "vfmasq_m\0"
  "vfmasq\0"
  "vfmsq\0"
  "vfmsq_m\0"
  "vgetq_lane\0"
  "vhaddq_m\0"
  "vhaddq\0"
  "vhaddq_x\0"
  "vhcaddq_rot270_m\0"
  "vhcaddq_rot270\0"
  "vhcaddq_rot270_x\0"
  "vhcaddq_rot90_m\0"
  "vhcaddq_rot90\0"
  "vhcaddq_rot90_x\0"
  "vhsubq_m\0"
  "vhsubq\0"
  "vhsubq_x\0"
  "vidupq_m\0"
  "vidupq_u16\0"
  "vidupq_u32\0"
  "vidupq_u8\0"
  "vidupq_x_u16\0"
  "vidupq_x_u32\0"
  "vidupq_x_u8\0"
  "viwdupq_m\0"
  "viwdupq_u16\0"
  "viwdupq_u32\0"
  "viwdupq_u8\0"
  "viwdupq_x_u16\0"
  "viwdupq_x_u32\0"
  "viwdupq_x_u8\0"
  "vld1q\0"
  "vld1q_z\0"
  "vld2q\0"
  "vld4q\0"
  "vldrbq_gather_offset\0"
  "vldrbq_gather_offset_z\0"
  "vldrdq_gather_offset\0"
  "vldrdq_gather_offset_z\0"
  "vldrdq_gather_shifted_offset\0"
  "vldrdq_gather_shifted_offset_z\0"
  "vldrhq_gather_offset\0"
  "vldrhq_gather_offset_z\0"
  "vldrhq_gather_shifted_offset\0"
  "vldrhq_gather_shifted_offset_z\0"
  "vldrwq_gather_offset\0"
  "vldrwq_gather_offset_z\0"
  "vldrwq_gather_shifted_offset\0"
  "vldrwq_gather_shifted_offset_z\0"
  "vmaxaq_m\0"
  "vmaxaq\0"
  "vmaxavq_p\0"
  "vmaxavq\0"
  "vmaxnmaq\0"
  "vmaxnmaq_m\0"
  "vmaxnmavq\0"
  "vmaxnmavq_p\0"
  "vmaxnmq\0"
  "vmaxnmq_m\0"
  "vmaxnmq_x\0"
  "vmaxnmvq\0"
  "vmaxnmvq_p\0"
  "vmaxq_m\0"
  "vmaxq\0"
  "vmaxq_x\0"
  "vmaxvq_p\0"
  "vmaxvq\0"
  "vminaq_m\0"
  "vminaq\0"
  "vminavq_p\0"
  "vminavq\0"
  "vminnmaq\0"
  "vminnmaq_m\0"
  "vminnmavq\0"
  "vminnmavq_p\0"
  "vminnmq\0"
  "vminnmq_m\0"
  "vminnmq_x\0"
  "vminnmvq\0"
  "vminnmvq_p\0"
  "vminq_m\0"
  "vminq\0"
  "vminq_x\0"
  "vminvq_p\0"
  "vminvq\0"
  "vmladavaq_p\0"
  "vmladavaq\0"
  "vmladavaxq_p\0"
  "vmladavaxq\0"
  "vmladavq_p\0"
  "vmladavq\0"
  "vmladavxq_p\0"
  "vmladavxq\0"
  "vmlaldavaq_p\0"
  "vmlaldavaq\0"
  "vmlaldavaxq_p\0"
  "vmlaldavaxq\0"
  "vmlaldavq_p\0"
  "vmlaldavq\0"
  "vmlaldavxq_p\0"
  "vmlaldavxq\0"
  "vmlaq_m\0"
  "vmlaq\0"
  "vmlasq_m\0"
  "vmlasq\0"
  "vmlsdavaq_p\0"
  "vmlsdavaq\0"
  "vmlsdavaxq_p\0"
  "vmlsdavaxq\0"
  "vmlsdavq_p\0"
  "vmlsdavq\0"
  "vmlsdavxq_p\0"
  "vmlsdavxq\0"
  "vmlsldavaq_p\0"
  "vmlsldavaq\0"
  "vmlsldavaxq_p\0"
  "vmlsldavaxq\0"
  "vmlsldavq_p\0"
  "vmlsldavq\0"
  "vmlsldavxq_p\0"
  "vmlsldavxq\0"
  "vmovlbq_m\0"
  "vmovlbq\0"
  "vmovlbq_x\0"
  "vmovltq_m\0"
  "vmovltq\0"
  "vmovltq_x\0"
  "vmovnbq_m\0"
  "vmovnbq\0"
  "vmovntq_m\0"
  "vmovntq\0"
  "vmulhq_m\0"
  "vmulhq\0"
  "vmulhq_x\0"
  "vmullbq_int_m\0"
  "vmullbq_int\0"
  "vmullbq_int_x\0"
  "vmullbq_poly_m\0"
  "vmullbq_poly\0"
  "vmullbq_poly_x\0"
  "vmulltq_int_m\0"
  "vmulltq_int\0"
  "vmulltq_int_x\0"
  "vmulltq_poly_m\0"
  "vmulltq_poly\0"
  "vmulltq_poly_x\0"
  "vmulq\0"
  "vmulq_m\0"
  "vmulq_x\0"
  "vmvnq_m\0"
  "vmvnq\0"
  "vmvnq_x\0"
  "vnegq\0"
  "vnegq_m\0"
  "vnegq_x\0"
  "vornq\0"
  "vornq_m\0"
  "vornq_x\0"
  "vorrq\0"
  "vorrq_m\0"
  "vorrq_m_n\0"
  "vorrq_x\0"
  "vpselq\0"
  "vqabsq_m\0"
  "vqabsq\0"
  "vqaddq_m\0"
  "vqaddq\0"
  "vqdmladhq_m\0"
  "vqdmladhq\0"
  "vqdmladhxq_m\0"
  "vqdmladhxq\0"
  "vqdmlahq_m\0"
  "vqdmlahq\0"
  "vqdmlashq_m\0"
  "vqdmlashq\0"
  "vqdmlsdhq_m\0"
  "vqdmlsdhq\0"
  "vqdmlsdhxq_m\0"
  "vqdmlsdhxq\0"
  "vqdmulhq_m\0"
  "vqdmulhq\0"
  "vqdmullbq_m\0"
  "vqdmullbq\0"
  "vqdmulltq_m\0"
  "vqdmulltq\0"
  "vqmovnbq_m\0"
  "vqmovnbq\0"
  "vqmovntq_m\0"
  "vqmovntq\0"
  "vqmovunbq_m\0"
  "vqmovunbq\0"
  "vqmovuntq_m\0"
  "vqmovuntq\0"
  "vqnegq_m\0"
  "vqnegq\0"
  "vqrdmladhq_m\0"
  "vqrdmladhq\0"
  "vqrdmladhxq_m\0"
  "vqrdmladhxq\0"
  "vqrdmlahq_m\0"
  "vqrdmlahq\0"
  "vqrdmlashq_m\0"
  "vqrdmlashq\0"
  "vqrdmlsdhq_m\0"
  "vqrdmlsdhq\0"
  "vqrdmlsdhxq_m\0"
  "vqrdmlsdhxq\0"
  "vqrdmulhq_m\0"
  "vqrdmulhq\0"
  "vqrshlq_m_n\0"
  "vqrshlq_m\0"
  "vqrshlq\0"
  "vqrshrnbq_m\0"
  "vqrshrnbq\0"
  "vqrshrntq_m\0"
  "vqrshrntq\0"
  "vqrshrunbq_m\0"
  "vqrshrunbq\0"
  "vqrshruntq_m\0"
  "vqrshruntq\0"
  "vqshlq_m_n\0"
  "vqshlq_m_r\0"
  "vqshlq_m\0"
  "vqshlq_n\0"
  "vqshlq_r\0"
  "vqshlq\0"
  "vqshluq_m\0"
  "vqshluq\0"
  "vqshrnbq_m\0"
  "vqshrnbq\0"
  "vqshrntq_m\0"
  "vqshrntq\0"
  "vqshrunbq_m\0"
  "vqshrunbq\0"
  "vqshruntq_m\0"
  "vqshruntq\0"
  "vqsubq_m\0"
  "vqsubq\0"
  "vreinterpretq_f16\0"
  "vreinterpretq_f32\0"
  "vreinterpretq_s16\0"
  "vreinterpretq_s32\0"
  "vreinterpretq_s64\0"
  "vreinterpretq_s8\0"
  "vreinterpretq_u16\0"
  "vreinterpretq_u32\0"
  "vreinterpretq_u64\0"
  "vreinterpretq_u8\0"
  "vrev16q_m\0"
  "vrev16q\0"
  "vrev16q_x\0"
  "vrev32q\0"
  "vrev32q_m\0"
  "vrev32q_x\0"
  "vrev64q\0"
  "vrev64q_m\0"
  "vrev64q_x\0"
  "vrhaddq_m\0"
  "vrhaddq\0"
  "vrhaddq_x\0"
  "vrmlaldavhaq_p\0"
  "vrmlaldavhaq\0"
  "vrmlaldavhaxq_p\0"
  "vrmlaldavhaxq\0"
  "vrmlaldavhq_p\0"
  "vrmlaldavhq\0"
  "vrmlaldavhxq_p\0"
  "vrmlaldavhxq\0"
  "vrmlsldavhaq_p\0"
  "vrmlsldavhaq\0"
  "vrmlsldavhaxq_p\0"
  "vrmlsldavhaxq\0"
  "vrmlsldavhq_p\0"
  "vrmlsldavhq\0"
  "vrmlsldavhxq_p\0"
  "vrmlsldavhxq\0"
  "vrmulhq_m\0"
  "vrmulhq\0"
  "vrmulhq_x\0"
  "vrndaq\0"
  "vrndaq_m\0"
  "vrndaq_x\0"
  "vrndmq\0"
  "vrndmq_m\0"
  "vrndmq_x\0"
  "vrndnq\0"
  "vrndnq_m\0"
  "vrndnq_x\0"
  "vrndpq\0"
  "vrndpq_m\0"
  "vrndpq_x\0"
  "vrndq\0"
  "vrndq_m\0"
  "vrndq_x\0"
  "vrndxq\0"
  "vrndxq_m\0"
  "vrndxq_x\0"
  "vrshlq_m_n\0"
  "vrshlq_m\0"
  "vrshlq\0"
  "vrshlq_x\0"
  "vrshrnbq_m\0"
  "vrshrnbq\0"
  "vrshrntq_m\0"
  "vrshrntq\0"
  "vrshrq_m\0"
  "vrshrq\0"
  "vrshrq_x\0"
  "vsbciq_m\0"
  "vsbciq\0"
  "vsbcq_m\0"
  "vsbcq\0"
  "vsetq_lane\0"
  "vshlcq_m\0"
  "vshlcq\0"
  "vshllbq_m\0"
  "vshllbq\0"
  "vshllbq_x\0"
  "vshlltq_m\0"
  "vshlltq\0"
  "vshlltq_x\0"
  "vshlq_m_n\0"
  "vshlq_m_r\0"
  "vshlq_m\0"
  "vshlq_n\0"
  "vshlq_r\0"
  "vshlq\0"
  "vshlq_x_n\0"
  "vshlq_x\0"
  "vshrnbq_m\0"
  "vshrnbq\0"
  "vshrntq_m\0"
  "vshrntq\0"
  "vshrq_m\0"
  "vshrq\0"
  "vshrq_x\0"
  "vsliq_m\0"
  "vsliq\0"
  "vsriq_m\0"
  "vsriq\0"
  "vst1q\0"
  "vst1q_p\0"
  "vst2q\0"
  "vst4q\0"
  "vstrbq_p\0"
  "vstrbq\0"
  "vstrbq_scatter_offset_p\0"
  "vstrbq_scatter_offset\0"
  "vstrdq_scatter_base_p\0"
  "vstrdq_scatter_base\0"
  "vstrdq_scatter_base_wb_p\0"
  "vstrdq_scatter_base_wb\0"
  "vstrdq_scatter_offset_p\0"
  "vstrdq_scatter_offset\0"
  "vstrdq_scatter_shifted_offset_p\0"
  "vstrdq_scatter_shifted_offset\0"
  "vstrhq\0"
  "vstrhq_p\0"
  "vstrhq_scatter_offset\0"
  "vstrhq_scatter_offset_p\0"
  "vstrhq_scatter_shifted_offset\0"
  "vstrhq_scatter_shifted_offset_p\0"
  "vstrwq\0"
  "vstrwq_p\0"
  "vstrwq_scatter_base\0"
  "vstrwq_scatter_base_p\0"
  "vstrwq_scatter_base_wb\0"
  "vstrwq_scatter_base_wb_p\0"
  "vstrwq_scatter_offset\0"
  "vstrwq_scatter_offset_p\0"
  "vstrwq_scatter_shifted_offset\0"
  "vstrwq_scatter_shifted_offset_p\0"
  "vsubq\0"
  "vsubq_m\0"
  "vsubq_x\0"
  "vuninitializedq\0"
  ;
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif

static constexpr llvm::StringTable
BuiltinStrings = BuiltinStringsStorage;
#endif // GET_MVE_BUILTIN_STR_TABLE

#ifdef GET_MVE_BUILTIN_INFOS
    Builtin::Info{Builtin::Info::StrOffsets{14 /* asrl */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19 /* lsll */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24 /* sqrshr */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31 /* sqrshrl */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{39 /* sqrshrl_sat48 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{53 /* sqshl */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{59 /* sqshll */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{66 /* srshr */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{72 /* srshrl */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{79 /* uqrshl */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{86 /* uqrshll */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{94 /* uqrshll_sat48 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{108 /* uqshl */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{114 /* uqshll */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{121 /* urshr */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{127 /* urshrl */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{134 /* vabavq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{147 /* vabavq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{160 /* vabavq_p_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{172 /* vabavq_p_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{185 /* vabavq_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{198 /* vabavq_p_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{210 /* vabavq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{221 /* vabavq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{232 /* vabavq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{242 /* vabavq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{253 /* vabavq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{264 /* vabavq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{274 /* vabdq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{284 /* vabdq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{294 /* vabdq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{306 /* vabdq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{318 /* vabdq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{330 /* vabdq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{342 /* vabdq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{353 /* vabdq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{365 /* vabdq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{377 /* vabdq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{388 /* vabdq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{398 /* vabdq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{408 /* vabdq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{417 /* vabdq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{427 /* vabdq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{437 /* vabdq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{446 /* vabdq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{458 /* vabdq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{470 /* vabdq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{482 /* vabdq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{494 /* vabdq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{505 /* vabdq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{517 /* vabdq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{529 /* vabdq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{540 /* vabsq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{550 /* vabsq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{560 /* vabsq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{572 /* vabsq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{584 /* vabsq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{596 /* vabsq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{608 /* vabsq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{619 /* vabsq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{629 /* vabsq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{639 /* vabsq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{648 /* vabsq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{660 /* vabsq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{672 /* vabsq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{684 /* vabsq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{696 /* vabsq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{707 /* vadciq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{720 /* vadciq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{733 /* vadciq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{744 /* vadciq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{755 /* vadcq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{767 /* vadcq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{779 /* vadcq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{789 /* vadcq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{799 /* vaddlvaq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{814 /* vaddlvaq_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{829 /* vaddlvaq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{842 /* vaddlvaq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{855 /* vaddlvq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{869 /* vaddlvq_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{883 /* vaddlvq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{895 /* vaddlvq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{907 /* vaddq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{917 /* vaddq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{927 /* vaddq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{939 /* vaddq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{951 /* vaddq_m_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{965 /* vaddq_m_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{979 /* vaddq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{993 /* vaddq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1007 /* vaddq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1020 /* vaddq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1034 /* vaddq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1048 /* vaddq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1061 /* vaddq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1073 /* vaddq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1085 /* vaddq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1096 /* vaddq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1108 /* vaddq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1120 /* vaddq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1131 /* vaddq_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1143 /* vaddq_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1155 /* vaddq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1167 /* vaddq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1179 /* vaddq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1190 /* vaddq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1202 /* vaddq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1214 /* vaddq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1225 /* vaddq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1235 /* vaddq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1245 /* vaddq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1254 /* vaddq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1264 /* vaddq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1274 /* vaddq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1283 /* vaddq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1295 /* vaddq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1307 /* vaddq_x_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1321 /* vaddq_x_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1335 /* vaddq_x_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1349 /* vaddq_x_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1363 /* vaddq_x_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1376 /* vaddq_x_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1390 /* vaddq_x_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1404 /* vaddq_x_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1417 /* vaddq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1429 /* vaddq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1441 /* vaddq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1452 /* vaddq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1464 /* vaddq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1476 /* vaddq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1487 /* vaddvaq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1501 /* vaddvaq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1515 /* vaddvaq_p_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1528 /* vaddvaq_p_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1542 /* vaddvaq_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1556 /* vaddvaq_p_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1569 /* vaddvaq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1581 /* vaddvaq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1593 /* vaddvaq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1604 /* vaddvaq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1616 /* vaddvaq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1628 /* vaddvaq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1639 /* vaddvq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1652 /* vaddvq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1665 /* vaddvq_p_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1677 /* vaddvq_p_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1690 /* vaddvq_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1703 /* vaddvq_p_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1715 /* vaddvq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1726 /* vaddvq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1737 /* vaddvq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1747 /* vaddvq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1758 /* vaddvq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1769 /* vaddvq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1779 /* vandq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1789 /* vandq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1799 /* vandq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1811 /* vandq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1823 /* vandq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1835 /* vandq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1847 /* vandq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1858 /* vandq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1870 /* vandq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1882 /* vandq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1893 /* vandq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1903 /* vandq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1913 /* vandq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1922 /* vandq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1932 /* vandq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1942 /* vandq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1951 /* vandq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1963 /* vandq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1975 /* vandq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1987 /* vandq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{1999 /* vandq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2010 /* vandq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2022 /* vandq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2034 /* vandq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2045 /* vbicq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2055 /* vbicq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2065 /* vbicq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2077 /* vbicq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2089 /* vbicq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2103 /* vbicq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2117 /* vbicq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2131 /* vbicq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2145 /* vbicq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2157 /* vbicq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2169 /* vbicq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2180 /* vbicq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2192 /* vbicq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2204 /* vbicq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2215 /* vbicq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2227 /* vbicq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2239 /* vbicq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2251 /* vbicq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2263 /* vbicq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2273 /* vbicq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2283 /* vbicq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2292 /* vbicq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2302 /* vbicq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2312 /* vbicq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2321 /* vbicq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2333 /* vbicq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2345 /* vbicq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2357 /* vbicq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2369 /* vbicq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2380 /* vbicq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2392 /* vbicq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2404 /* vbicq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2415 /* vbrsrq_m_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2430 /* vbrsrq_m_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2445 /* vbrsrq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2460 /* vbrsrq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2475 /* vbrsrq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2489 /* vbrsrq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2504 /* vbrsrq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2519 /* vbrsrq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2533 /* vbrsrq_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2546 /* vbrsrq_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2559 /* vbrsrq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2572 /* vbrsrq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2585 /* vbrsrq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2597 /* vbrsrq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2610 /* vbrsrq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2623 /* vbrsrq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2635 /* vbrsrq_x_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2650 /* vbrsrq_x_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2665 /* vbrsrq_x_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2680 /* vbrsrq_x_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2695 /* vbrsrq_x_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2709 /* vbrsrq_x_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2724 /* vbrsrq_x_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2739 /* vbrsrq_x_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2753 /* vcaddq_rot270_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2771 /* vcaddq_rot270_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2789 /* vcaddq_rot270_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2809 /* vcaddq_rot270_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2829 /* vcaddq_rot270_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2849 /* vcaddq_rot270_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2869 /* vcaddq_rot270_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2888 /* vcaddq_rot270_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2908 /* vcaddq_rot270_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2928 /* vcaddq_rot270_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2947 /* vcaddq_rot270_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2965 /* vcaddq_rot270_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{2983 /* vcaddq_rot270_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3000 /* vcaddq_rot270_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3018 /* vcaddq_rot270_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3036 /* vcaddq_rot270_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3053 /* vcaddq_rot270_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3073 /* vcaddq_rot270_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3093 /* vcaddq_rot270_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3113 /* vcaddq_rot270_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3133 /* vcaddq_rot270_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3152 /* vcaddq_rot270_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3172 /* vcaddq_rot270_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3192 /* vcaddq_rot270_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3211 /* vcaddq_rot90_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3228 /* vcaddq_rot90_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3245 /* vcaddq_rot90_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3264 /* vcaddq_rot90_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3283 /* vcaddq_rot90_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3302 /* vcaddq_rot90_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3321 /* vcaddq_rot90_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3339 /* vcaddq_rot90_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3358 /* vcaddq_rot90_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3377 /* vcaddq_rot90_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3395 /* vcaddq_rot90_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3412 /* vcaddq_rot90_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3429 /* vcaddq_rot90_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3445 /* vcaddq_rot90_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3462 /* vcaddq_rot90_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3479 /* vcaddq_rot90_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3495 /* vcaddq_rot90_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3514 /* vcaddq_rot90_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3533 /* vcaddq_rot90_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3552 /* vcaddq_rot90_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3571 /* vcaddq_rot90_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3589 /* vcaddq_rot90_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3608 /* vcaddq_rot90_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3627 /* vcaddq_rot90_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3645 /* vclsq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3657 /* vclsq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3669 /* vclsq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3680 /* vclsq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3690 /* vclsq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3700 /* vclsq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3709 /* vclsq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3721 /* vclsq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3733 /* vclsq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3744 /* vclzq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3756 /* vclzq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3768 /* vclzq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3779 /* vclzq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3791 /* vclzq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3803 /* vclzq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3814 /* vclzq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3824 /* vclzq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3834 /* vclzq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3843 /* vclzq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3853 /* vclzq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3863 /* vclzq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3872 /* vclzq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3884 /* vclzq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3896 /* vclzq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3907 /* vclzq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3919 /* vclzq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3931 /* vclzq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3942 /* vcmlaq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3953 /* vcmlaq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3964 /* vcmlaq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3977 /* vcmlaq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{3990 /* vcmlaq_rot180_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4008 /* vcmlaq_rot180_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4026 /* vcmlaq_rot180_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4046 /* vcmlaq_rot180_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4066 /* vcmlaq_rot270_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4084 /* vcmlaq_rot270_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4102 /* vcmlaq_rot270_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4122 /* vcmlaq_rot270_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4142 /* vcmlaq_rot90_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4159 /* vcmlaq_rot90_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4176 /* vcmlaq_rot90_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4195 /* vcmlaq_rot90_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4214 /* vcmpcsq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4230 /* vcmpcsq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4246 /* vcmpcsq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4261 /* vcmpcsq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4275 /* vcmpcsq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4289 /* vcmpcsq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4302 /* vcmpcsq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4316 /* vcmpcsq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4330 /* vcmpcsq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4343 /* vcmpcsq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4355 /* vcmpcsq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4367 /* vcmpcsq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4378 /* vcmpeqq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4390 /* vcmpeqq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4402 /* vcmpeqq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4416 /* vcmpeqq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4430 /* vcmpeqq_m_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4446 /* vcmpeqq_m_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4462 /* vcmpeqq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4478 /* vcmpeqq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4494 /* vcmpeqq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4509 /* vcmpeqq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4525 /* vcmpeqq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4541 /* vcmpeqq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4556 /* vcmpeqq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4570 /* vcmpeqq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4584 /* vcmpeqq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4597 /* vcmpeqq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4611 /* vcmpeqq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4625 /* vcmpeqq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4638 /* vcmpeqq_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4652 /* vcmpeqq_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4666 /* vcmpeqq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4680 /* vcmpeqq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4694 /* vcmpeqq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4707 /* vcmpeqq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4721 /* vcmpeqq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4735 /* vcmpeqq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4748 /* vcmpeqq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4760 /* vcmpeqq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4772 /* vcmpeqq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4783 /* vcmpeqq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4795 /* vcmpeqq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4807 /* vcmpeqq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4818 /* vcmpgeq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4830 /* vcmpgeq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4842 /* vcmpgeq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4856 /* vcmpgeq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4870 /* vcmpgeq_m_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4886 /* vcmpgeq_m_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4902 /* vcmpgeq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4918 /* vcmpgeq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4934 /* vcmpgeq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4949 /* vcmpgeq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4963 /* vcmpgeq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4977 /* vcmpgeq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{4990 /* vcmpgeq_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5004 /* vcmpgeq_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5018 /* vcmpgeq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5032 /* vcmpgeq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5046 /* vcmpgeq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5059 /* vcmpgeq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5071 /* vcmpgeq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5083 /* vcmpgeq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5094 /* vcmpgtq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5106 /* vcmpgtq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5118 /* vcmpgtq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5132 /* vcmpgtq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5146 /* vcmpgtq_m_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5162 /* vcmpgtq_m_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5178 /* vcmpgtq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5194 /* vcmpgtq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5210 /* vcmpgtq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5225 /* vcmpgtq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5239 /* vcmpgtq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5253 /* vcmpgtq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5266 /* vcmpgtq_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5280 /* vcmpgtq_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5294 /* vcmpgtq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5308 /* vcmpgtq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5322 /* vcmpgtq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5335 /* vcmpgtq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5347 /* vcmpgtq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5359 /* vcmpgtq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5370 /* vcmphiq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5386 /* vcmphiq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5402 /* vcmphiq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5417 /* vcmphiq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5431 /* vcmphiq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5445 /* vcmphiq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5458 /* vcmphiq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5472 /* vcmphiq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5486 /* vcmphiq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5499 /* vcmphiq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5511 /* vcmphiq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5523 /* vcmphiq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5534 /* vcmpleq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5546 /* vcmpleq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5558 /* vcmpleq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5572 /* vcmpleq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5586 /* vcmpleq_m_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5602 /* vcmpleq_m_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5618 /* vcmpleq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5634 /* vcmpleq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5650 /* vcmpleq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5665 /* vcmpleq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5679 /* vcmpleq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5693 /* vcmpleq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5706 /* vcmpleq_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5720 /* vcmpleq_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5734 /* vcmpleq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5748 /* vcmpleq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5762 /* vcmpleq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5775 /* vcmpleq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5787 /* vcmpleq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5799 /* vcmpleq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5810 /* vcmpltq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5822 /* vcmpltq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5834 /* vcmpltq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5848 /* vcmpltq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5862 /* vcmpltq_m_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5878 /* vcmpltq_m_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5894 /* vcmpltq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5910 /* vcmpltq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5926 /* vcmpltq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5941 /* vcmpltq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5955 /* vcmpltq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5969 /* vcmpltq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5982 /* vcmpltq_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{5996 /* vcmpltq_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6010 /* vcmpltq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6024 /* vcmpltq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6038 /* vcmpltq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6051 /* vcmpltq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6063 /* vcmpltq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6075 /* vcmpltq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6086 /* vcmpneq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6098 /* vcmpneq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6110 /* vcmpneq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6124 /* vcmpneq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6138 /* vcmpneq_m_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6154 /* vcmpneq_m_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6170 /* vcmpneq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6186 /* vcmpneq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6202 /* vcmpneq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6217 /* vcmpneq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6233 /* vcmpneq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6249 /* vcmpneq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6264 /* vcmpneq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6278 /* vcmpneq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6292 /* vcmpneq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6305 /* vcmpneq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6319 /* vcmpneq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6333 /* vcmpneq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6346 /* vcmpneq_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6360 /* vcmpneq_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6374 /* vcmpneq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6388 /* vcmpneq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6402 /* vcmpneq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6415 /* vcmpneq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6429 /* vcmpneq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6443 /* vcmpneq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6456 /* vcmpneq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6468 /* vcmpneq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6480 /* vcmpneq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6491 /* vcmpneq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6503 /* vcmpneq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6515 /* vcmpneq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6526 /* vcmulq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6537 /* vcmulq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6548 /* vcmulq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6561 /* vcmulq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6574 /* vcmulq_rot180_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6592 /* vcmulq_rot180_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6610 /* vcmulq_rot180_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6630 /* vcmulq_rot180_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6650 /* vcmulq_rot180_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6670 /* vcmulq_rot180_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6690 /* vcmulq_rot270_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6708 /* vcmulq_rot270_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6726 /* vcmulq_rot270_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6746 /* vcmulq_rot270_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6766 /* vcmulq_rot270_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6786 /* vcmulq_rot270_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6806 /* vcmulq_rot90_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6823 /* vcmulq_rot90_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6840 /* vcmulq_rot90_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6859 /* vcmulq_rot90_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6878 /* vcmulq_rot90_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6897 /* vcmulq_rot90_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6916 /* vcmulq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6929 /* vcmulq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6942 /* vcreateq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6955 /* vcreateq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6968 /* vcreateq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6981 /* vcreateq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{6994 /* vcreateq_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7007 /* vcreateq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7019 /* vcreateq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7032 /* vcreateq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7045 /* vcreateq_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7058 /* vcreateq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7070 /* vctp16q */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7078 /* vctp16q_m */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7088 /* vctp32q */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7096 /* vctp32q_m */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7106 /* vctp64q */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7114 /* vctp64q_m */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7124 /* vctp8q */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7131 /* vctp8q_m */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7140 /* vcvtaq_m_s16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7157 /* vcvtaq_m_s32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7174 /* vcvtaq_m_u16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7191 /* vcvtaq_m_u32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7208 /* vcvtaq_s16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7223 /* vcvtaq_s32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7238 /* vcvtaq_u16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7253 /* vcvtaq_u32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7268 /* vcvtaq_x_s16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7285 /* vcvtaq_x_s32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7302 /* vcvtaq_x_u16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7319 /* vcvtaq_x_u32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7336 /* vcvtbq_f16_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7351 /* vcvtbq_f32_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7366 /* vcvtbq_m_f16_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7383 /* vcvtbq_m_f32_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7400 /* vcvtbq_x_f32_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7417 /* vcvtmq_m_s16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7434 /* vcvtmq_m_s32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7451 /* vcvtmq_m_u16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7468 /* vcvtmq_m_u32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7485 /* vcvtmq_s16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7500 /* vcvtmq_s32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7515 /* vcvtmq_u16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7530 /* vcvtmq_u32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7545 /* vcvtmq_x_s16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7562 /* vcvtmq_x_s32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7579 /* vcvtmq_x_u16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7596 /* vcvtmq_x_u32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7613 /* vcvtnq_m_s16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7630 /* vcvtnq_m_s32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7647 /* vcvtnq_m_u16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7664 /* vcvtnq_m_u32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7681 /* vcvtnq_s16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7696 /* vcvtnq_s32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7711 /* vcvtnq_u16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7726 /* vcvtnq_u32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7741 /* vcvtnq_x_s16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7758 /* vcvtnq_x_s32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7775 /* vcvtnq_x_u16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7792 /* vcvtnq_x_u32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7809 /* vcvtpq_m_s16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7826 /* vcvtpq_m_s32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7843 /* vcvtpq_m_u16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7860 /* vcvtpq_m_u32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7877 /* vcvtpq_s16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7892 /* vcvtpq_s32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7907 /* vcvtpq_u16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7922 /* vcvtpq_u32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7937 /* vcvtpq_x_s16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7954 /* vcvtpq_x_s32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7971 /* vcvtpq_x_u16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{7988 /* vcvtpq_x_u32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8005 /* vcvtq_f16_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8019 /* vcvtq_f16_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8033 /* vcvtq_f32_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8047 /* vcvtq_f32_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8061 /* vcvtq_m_f16_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8077 /* vcvtq_m_f16_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8093 /* vcvtq_m_f32_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8109 /* vcvtq_m_f32_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8125 /* vcvtq_m_n_f16_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8143 /* vcvtq_m_n_f16_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8161 /* vcvtq_m_n_f32_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8179 /* vcvtq_m_n_f32_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8197 /* vcvtq_m_n_s16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8215 /* vcvtq_m_n_s32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8233 /* vcvtq_m_n_u16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8251 /* vcvtq_m_n_u32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8269 /* vcvtq_m_s16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8285 /* vcvtq_m_s32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8301 /* vcvtq_m_u16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8317 /* vcvtq_m_u32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8333 /* vcvtq_n_f16_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8349 /* vcvtq_n_f16_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8365 /* vcvtq_n_f32_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8381 /* vcvtq_n_f32_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8397 /* vcvtq_n_s16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8413 /* vcvtq_n_s32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8429 /* vcvtq_n_u16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8445 /* vcvtq_n_u32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8461 /* vcvtq_s16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8475 /* vcvtq_s32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8489 /* vcvtq_u16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8503 /* vcvtq_u32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8517 /* vcvtq_x_f16_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8533 /* vcvtq_x_f16_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8549 /* vcvtq_x_f32_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8565 /* vcvtq_x_f32_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8581 /* vcvtq_x_n_f16_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8599 /* vcvtq_x_n_f16_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8617 /* vcvtq_x_n_f32_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8635 /* vcvtq_x_n_f32_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8653 /* vcvtq_x_n_s16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8671 /* vcvtq_x_n_s32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8689 /* vcvtq_x_n_u16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8707 /* vcvtq_x_n_u32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8725 /* vcvtq_x_s16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8741 /* vcvtq_x_s32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8757 /* vcvtq_x_u16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8773 /* vcvtq_x_u32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8789 /* vcvttq_f16_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8804 /* vcvttq_f32_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8819 /* vcvttq_m_f16_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8836 /* vcvttq_m_f32_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8853 /* vcvttq_x_f32_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8870 /* vddupq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8885 /* vddupq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8900 /* vddupq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8914 /* vddupq_m_wb_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8930 /* vddupq_m_wb_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8946 /* vddupq_m_wb_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8961 /* vddupq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8974 /* vddupq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8987 /* vddupq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{8999 /* vddupq_wb_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9013 /* vddupq_wb_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9027 /* vddupq_wb_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9040 /* vddupq_x_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9055 /* vddupq_x_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9070 /* vddupq_x_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9084 /* vddupq_x_wb_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9100 /* vddupq_x_wb_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9116 /* vddupq_x_wb_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9131 /* vdupq_m_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9145 /* vdupq_m_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9159 /* vdupq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9173 /* vdupq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9187 /* vdupq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9200 /* vdupq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9214 /* vdupq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9228 /* vdupq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9241 /* vdupq_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9253 /* vdupq_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9265 /* vdupq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9277 /* vdupq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9289 /* vdupq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9300 /* vdupq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9312 /* vdupq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9324 /* vdupq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9335 /* vdupq_x_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9349 /* vdupq_x_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9363 /* vdupq_x_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9377 /* vdupq_x_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9391 /* vdupq_x_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9404 /* vdupq_x_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9418 /* vdupq_x_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9432 /* vdupq_x_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9445 /* vdwdupq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9461 /* vdwdupq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9477 /* vdwdupq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9492 /* vdwdupq_m_wb_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9509 /* vdwdupq_m_wb_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9526 /* vdwdupq_m_wb_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9542 /* vdwdupq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9556 /* vdwdupq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9570 /* vdwdupq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9583 /* vdwdupq_wb_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9598 /* vdwdupq_wb_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9613 /* vdwdupq_wb_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9627 /* vdwdupq_x_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9643 /* vdwdupq_x_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9659 /* vdwdupq_x_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9674 /* vdwdupq_x_wb_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9691 /* vdwdupq_x_wb_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9708 /* vdwdupq_x_wb_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9724 /* veorq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9734 /* veorq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9744 /* veorq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9756 /* veorq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9768 /* veorq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9780 /* veorq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9792 /* veorq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9803 /* veorq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9815 /* veorq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9827 /* veorq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9838 /* veorq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9848 /* veorq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9858 /* veorq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9867 /* veorq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9877 /* veorq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9887 /* veorq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9896 /* veorq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9908 /* veorq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9920 /* veorq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9932 /* veorq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9944 /* veorq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9955 /* veorq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9967 /* veorq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9979 /* veorq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{9990 /* vfmaq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10000 /* vfmaq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10010 /* vfmaq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10022 /* vfmaq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10034 /* vfmaq_m_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10048 /* vfmaq_m_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10062 /* vfmaq_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10074 /* vfmaq_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10086 /* vfmasq_m_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10101 /* vfmasq_m_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10116 /* vfmasq_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10129 /* vfmasq_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10142 /* vfmsq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10152 /* vfmsq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10162 /* vfmsq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10174 /* vfmsq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10186 /* vgetq_lane_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10201 /* vgetq_lane_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10216 /* vgetq_lane_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10231 /* vgetq_lane_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10246 /* vgetq_lane_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10261 /* vgetq_lane_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10275 /* vgetq_lane_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10290 /* vgetq_lane_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10305 /* vgetq_lane_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10320 /* vgetq_lane_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10334 /* vhaddq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10349 /* vhaddq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10364 /* vhaddq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10378 /* vhaddq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10393 /* vhaddq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10408 /* vhaddq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10422 /* vhaddq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10435 /* vhaddq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10448 /* vhaddq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10460 /* vhaddq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10473 /* vhaddq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10486 /* vhaddq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10498 /* vhaddq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10511 /* vhaddq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10524 /* vhaddq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10536 /* vhaddq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10549 /* vhaddq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10562 /* vhaddq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10574 /* vhaddq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10585 /* vhaddq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10596 /* vhaddq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10606 /* vhaddq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10617 /* vhaddq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10628 /* vhaddq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10638 /* vhaddq_x_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10653 /* vhaddq_x_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10668 /* vhaddq_x_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10682 /* vhaddq_x_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10697 /* vhaddq_x_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10712 /* vhaddq_x_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10726 /* vhaddq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10739 /* vhaddq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10752 /* vhaddq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10764 /* vhaddq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10777 /* vhaddq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10790 /* vhaddq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10802 /* vhcaddq_rot270_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10823 /* vhcaddq_rot270_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10844 /* vhcaddq_rot270_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10864 /* vhcaddq_rot270_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10883 /* vhcaddq_rot270_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10902 /* vhcaddq_rot270_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10920 /* vhcaddq_rot270_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10941 /* vhcaddq_rot270_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10962 /* vhcaddq_rot270_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{10982 /* vhcaddq_rot90_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11002 /* vhcaddq_rot90_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11022 /* vhcaddq_rot90_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11041 /* vhcaddq_rot90_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11059 /* vhcaddq_rot90_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11077 /* vhcaddq_rot90_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11094 /* vhcaddq_rot90_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11114 /* vhcaddq_rot90_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11134 /* vhcaddq_rot90_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11153 /* vhsubq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11168 /* vhsubq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11183 /* vhsubq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11197 /* vhsubq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11212 /* vhsubq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11227 /* vhsubq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11241 /* vhsubq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11254 /* vhsubq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11267 /* vhsubq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11279 /* vhsubq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11292 /* vhsubq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11305 /* vhsubq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11317 /* vhsubq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11330 /* vhsubq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11343 /* vhsubq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11355 /* vhsubq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11368 /* vhsubq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11381 /* vhsubq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11393 /* vhsubq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11404 /* vhsubq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11415 /* vhsubq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11425 /* vhsubq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11436 /* vhsubq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11447 /* vhsubq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11457 /* vhsubq_x_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11472 /* vhsubq_x_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11487 /* vhsubq_x_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11501 /* vhsubq_x_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11516 /* vhsubq_x_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11531 /* vhsubq_x_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11545 /* vhsubq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11558 /* vhsubq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11571 /* vhsubq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11583 /* vhsubq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11596 /* vhsubq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11609 /* vhsubq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11621 /* vidupq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11636 /* vidupq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11651 /* vidupq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11665 /* vidupq_m_wb_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11681 /* vidupq_m_wb_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11697 /* vidupq_m_wb_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11712 /* vidupq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11725 /* vidupq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11738 /* vidupq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11750 /* vidupq_wb_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11764 /* vidupq_wb_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11778 /* vidupq_wb_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11791 /* vidupq_x_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11806 /* vidupq_x_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11821 /* vidupq_x_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11835 /* vidupq_x_wb_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11851 /* vidupq_x_wb_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11867 /* vidupq_x_wb_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11882 /* viwdupq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11898 /* viwdupq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11914 /* viwdupq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11929 /* viwdupq_m_wb_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11946 /* viwdupq_m_wb_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11963 /* viwdupq_m_wb_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11979 /* viwdupq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{11993 /* viwdupq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12007 /* viwdupq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12020 /* viwdupq_wb_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12035 /* viwdupq_wb_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12050 /* viwdupq_wb_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12064 /* viwdupq_x_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12080 /* viwdupq_x_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12096 /* viwdupq_x_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12111 /* viwdupq_x_wb_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12128 /* viwdupq_x_wb_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12145 /* viwdupq_x_wb_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12161 /* vld1q_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12171 /* vld1q_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12181 /* vld1q_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12191 /* vld1q_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12201 /* vld1q_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12210 /* vld1q_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12220 /* vld1q_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12230 /* vld1q_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12239 /* vld1q_z_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12251 /* vld1q_z_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12263 /* vld1q_z_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12275 /* vld1q_z_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12287 /* vld1q_z_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12298 /* vld1q_z_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12310 /* vld1q_z_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12322 /* vld1q_z_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12333 /* vld2q_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12343 /* vld2q_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12353 /* vld2q_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12363 /* vld2q_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12373 /* vld2q_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12382 /* vld2q_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12392 /* vld2q_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12402 /* vld2q_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12411 /* vld4q_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12421 /* vld4q_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12431 /* vld4q_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12441 /* vld4q_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12451 /* vld4q_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12460 /* vld4q_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12470 /* vld4q_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12480 /* vld4q_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12489 /* vldrbq_gather_offset_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12514 /* vldrbq_gather_offset_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12539 /* vldrbq_gather_offset_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12563 /* vldrbq_gather_offset_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12588 /* vldrbq_gather_offset_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12613 /* vldrbq_gather_offset_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12637 /* vldrbq_gather_offset_z_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12664 /* vldrbq_gather_offset_z_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12691 /* vldrbq_gather_offset_z_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12717 /* vldrbq_gather_offset_z_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12744 /* vldrbq_gather_offset_z_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12771 /* vldrbq_gather_offset_z_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12797 /* vldrbq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12808 /* vldrbq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12819 /* vldrbq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12829 /* vldrbq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12840 /* vldrbq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12851 /* vldrbq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12861 /* vldrbq_z_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12874 /* vldrbq_z_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12887 /* vldrbq_z_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12899 /* vldrbq_z_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12912 /* vldrbq_z_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12925 /* vldrbq_z_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12937 /* vldrdq_gather_base_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12960 /* vldrdq_gather_base_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{12983 /* vldrdq_gather_base_wb_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13009 /* vldrdq_gather_base_wb_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13035 /* vldrdq_gather_base_wb_z_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13063 /* vldrdq_gather_base_wb_z_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13091 /* vldrdq_gather_base_z_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13116 /* vldrdq_gather_base_z_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13141 /* vldrdq_gather_offset_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13166 /* vldrdq_gather_offset_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13191 /* vldrdq_gather_offset_z_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13218 /* vldrdq_gather_offset_z_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13245 /* vldrdq_gather_shifted_offset_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13278 /* vldrdq_gather_shifted_offset_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13311 /* vldrdq_gather_shifted_offset_z_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13346 /* vldrdq_gather_shifted_offset_z_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13381 /* vldrhq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13392 /* vldrhq_gather_offset_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13417 /* vldrhq_gather_offset_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13442 /* vldrhq_gather_offset_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13467 /* vldrhq_gather_offset_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13492 /* vldrhq_gather_offset_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13517 /* vldrhq_gather_offset_z_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13544 /* vldrhq_gather_offset_z_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13571 /* vldrhq_gather_offset_z_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13598 /* vldrhq_gather_offset_z_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13625 /* vldrhq_gather_offset_z_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13652 /* vldrhq_gather_shifted_offset_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13685 /* vldrhq_gather_shifted_offset_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13718 /* vldrhq_gather_shifted_offset_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13751 /* vldrhq_gather_shifted_offset_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13784 /* vldrhq_gather_shifted_offset_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13817 /* vldrhq_gather_shifted_offset_z_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13852 /* vldrhq_gather_shifted_offset_z_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13887 /* vldrhq_gather_shifted_offset_z_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13922 /* vldrhq_gather_shifted_offset_z_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13957 /* vldrhq_gather_shifted_offset_z_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{13992 /* vldrhq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14003 /* vldrhq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14014 /* vldrhq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14025 /* vldrhq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14036 /* vldrhq_z_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14049 /* vldrhq_z_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14062 /* vldrhq_z_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14075 /* vldrhq_z_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14088 /* vldrhq_z_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14101 /* vldrwq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14112 /* vldrwq_gather_base_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14135 /* vldrwq_gather_base_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14158 /* vldrwq_gather_base_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14181 /* vldrwq_gather_base_wb_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14207 /* vldrwq_gather_base_wb_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14233 /* vldrwq_gather_base_wb_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14259 /* vldrwq_gather_base_wb_z_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14287 /* vldrwq_gather_base_wb_z_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14315 /* vldrwq_gather_base_wb_z_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14343 /* vldrwq_gather_base_z_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14368 /* vldrwq_gather_base_z_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14393 /* vldrwq_gather_base_z_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14418 /* vldrwq_gather_offset_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14443 /* vldrwq_gather_offset_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14468 /* vldrwq_gather_offset_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14493 /* vldrwq_gather_offset_z_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14520 /* vldrwq_gather_offset_z_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14547 /* vldrwq_gather_offset_z_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14574 /* vldrwq_gather_shifted_offset_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14607 /* vldrwq_gather_shifted_offset_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14640 /* vldrwq_gather_shifted_offset_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14673 /* vldrwq_gather_shifted_offset_z_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14708 /* vldrwq_gather_shifted_offset_z_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14743 /* vldrwq_gather_shifted_offset_z_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14778 /* vldrwq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14789 /* vldrwq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14800 /* vldrwq_z_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14813 /* vldrwq_z_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14826 /* vldrwq_z_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14839 /* vmaxaq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14852 /* vmaxaq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14865 /* vmaxaq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14877 /* vmaxaq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14888 /* vmaxaq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14899 /* vmaxaq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14909 /* vmaxavq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14923 /* vmaxavq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14937 /* vmaxavq_p_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14950 /* vmaxavq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14962 /* vmaxavq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14974 /* vmaxavq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14985 /* vmaxnmaq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{14998 /* vmaxnmaq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15011 /* vmaxnmaq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15026 /* vmaxnmaq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15041 /* vmaxnmavq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15055 /* vmaxnmavq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15069 /* vmaxnmavq_p_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15085 /* vmaxnmavq_p_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15101 /* vmaxnmq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15113 /* vmaxnmq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15125 /* vmaxnmq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15139 /* vmaxnmq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15153 /* vmaxnmq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15167 /* vmaxnmq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15181 /* vmaxnmvq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15194 /* vmaxnmvq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15207 /* vmaxnmvq_p_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15222 /* vmaxnmvq_p_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15237 /* vmaxq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15249 /* vmaxq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15261 /* vmaxq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15272 /* vmaxq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15284 /* vmaxq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15296 /* vmaxq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15307 /* vmaxq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15317 /* vmaxq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15327 /* vmaxq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15336 /* vmaxq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15346 /* vmaxq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15356 /* vmaxq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15365 /* vmaxq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15377 /* vmaxq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15389 /* vmaxq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15400 /* vmaxq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15412 /* vmaxq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15424 /* vmaxq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15435 /* vmaxvq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15448 /* vmaxvq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15461 /* vmaxvq_p_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15473 /* vmaxvq_p_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15486 /* vmaxvq_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15499 /* vmaxvq_p_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15511 /* vmaxvq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15522 /* vmaxvq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15533 /* vmaxvq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15543 /* vmaxvq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15554 /* vmaxvq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15565 /* vmaxvq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15575 /* vminaq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15588 /* vminaq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15601 /* vminaq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15613 /* vminaq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15624 /* vminaq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15635 /* vminaq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15645 /* vminavq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15659 /* vminavq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15673 /* vminavq_p_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15686 /* vminavq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15698 /* vminavq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15710 /* vminavq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15721 /* vminnmaq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15734 /* vminnmaq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15747 /* vminnmaq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15762 /* vminnmaq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15777 /* vminnmavq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15791 /* vminnmavq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15805 /* vminnmavq_p_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15821 /* vminnmavq_p_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15837 /* vminnmq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15849 /* vminnmq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15861 /* vminnmq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15875 /* vminnmq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15889 /* vminnmq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15903 /* vminnmq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15917 /* vminnmvq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15930 /* vminnmvq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15943 /* vminnmvq_p_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15958 /* vminnmvq_p_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15973 /* vminq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15985 /* vminq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{15997 /* vminq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16008 /* vminq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16020 /* vminq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16032 /* vminq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16043 /* vminq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16053 /* vminq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16063 /* vminq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16072 /* vminq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16082 /* vminq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16092 /* vminq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16101 /* vminq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16113 /* vminq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16125 /* vminq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16136 /* vminq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16148 /* vminq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16160 /* vminq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16171 /* vminvq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16184 /* vminvq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16197 /* vminvq_p_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16209 /* vminvq_p_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16222 /* vminvq_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16235 /* vminvq_p_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16247 /* vminvq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16258 /* vminvq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16269 /* vminvq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16279 /* vminvq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16290 /* vminvq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16301 /* vminvq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16311 /* vmladavaq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16327 /* vmladavaq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16343 /* vmladavaq_p_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16358 /* vmladavaq_p_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16374 /* vmladavaq_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16390 /* vmladavaq_p_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16405 /* vmladavaq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16419 /* vmladavaq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16433 /* vmladavaq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16446 /* vmladavaq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16460 /* vmladavaq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16474 /* vmladavaq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16487 /* vmladavaxq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16504 /* vmladavaxq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16521 /* vmladavaxq_p_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16537 /* vmladavaxq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16552 /* vmladavaxq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16567 /* vmladavaxq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16581 /* vmladavq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16596 /* vmladavq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16611 /* vmladavq_p_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16625 /* vmladavq_p_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16640 /* vmladavq_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16655 /* vmladavq_p_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16669 /* vmladavq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16682 /* vmladavq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16695 /* vmladavq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16707 /* vmladavq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16720 /* vmladavq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16733 /* vmladavq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16745 /* vmladavxq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16761 /* vmladavxq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16777 /* vmladavxq_p_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16792 /* vmladavxq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16806 /* vmladavxq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16820 /* vmladavxq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16833 /* vmlaldavaq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16850 /* vmlaldavaq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16867 /* vmlaldavaq_p_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16884 /* vmlaldavaq_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16901 /* vmlaldavaq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16916 /* vmlaldavaq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16931 /* vmlaldavaq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16946 /* vmlaldavaq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16961 /* vmlaldavaxq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16979 /* vmlaldavaxq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{16997 /* vmlaldavaxq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17013 /* vmlaldavaxq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17029 /* vmlaldavq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17045 /* vmlaldavq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17061 /* vmlaldavq_p_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17077 /* vmlaldavq_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17093 /* vmlaldavq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17107 /* vmlaldavq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17121 /* vmlaldavq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17135 /* vmlaldavq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17149 /* vmlaldavxq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17166 /* vmlaldavxq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17183 /* vmlaldavxq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17198 /* vmlaldavxq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17213 /* vmlaq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17227 /* vmlaq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17241 /* vmlaq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17254 /* vmlaq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17268 /* vmlaq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17282 /* vmlaq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17295 /* vmlaq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17307 /* vmlaq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17319 /* vmlaq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17330 /* vmlaq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17342 /* vmlaq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17354 /* vmlaq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17365 /* vmlasq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17380 /* vmlasq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17395 /* vmlasq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17409 /* vmlasq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17424 /* vmlasq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17439 /* vmlasq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17453 /* vmlasq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17466 /* vmlasq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17479 /* vmlasq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17491 /* vmlasq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17504 /* vmlasq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17517 /* vmlasq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17529 /* vmlsdavaq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17545 /* vmlsdavaq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17561 /* vmlsdavaq_p_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17576 /* vmlsdavaq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17590 /* vmlsdavaq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17604 /* vmlsdavaq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17617 /* vmlsdavaxq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17634 /* vmlsdavaxq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17651 /* vmlsdavaxq_p_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17667 /* vmlsdavaxq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17682 /* vmlsdavaxq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17697 /* vmlsdavaxq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17711 /* vmlsdavq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17726 /* vmlsdavq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17741 /* vmlsdavq_p_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17755 /* vmlsdavq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17768 /* vmlsdavq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17781 /* vmlsdavq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17793 /* vmlsdavxq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17809 /* vmlsdavxq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17825 /* vmlsdavxq_p_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17840 /* vmlsdavxq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17854 /* vmlsdavxq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17868 /* vmlsdavxq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17881 /* vmlsldavaq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17898 /* vmlsldavaq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17915 /* vmlsldavaq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17930 /* vmlsldavaq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17945 /* vmlsldavaxq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17963 /* vmlsldavaxq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17981 /* vmlsldavaxq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{17997 /* vmlsldavaxq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18013 /* vmlsldavq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18029 /* vmlsldavq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18045 /* vmlsldavq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18059 /* vmlsldavq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18073 /* vmlsldavxq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18090 /* vmlsldavxq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18107 /* vmlsldavxq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18122 /* vmlsldavxq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18137 /* vmovlbq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18151 /* vmovlbq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18164 /* vmovlbq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18178 /* vmovlbq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18191 /* vmovlbq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18203 /* vmovlbq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18214 /* vmovlbq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18226 /* vmovlbq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18237 /* vmovlbq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18251 /* vmovlbq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18264 /* vmovlbq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18278 /* vmovlbq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18291 /* vmovltq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18305 /* vmovltq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18318 /* vmovltq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18332 /* vmovltq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18345 /* vmovltq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18357 /* vmovltq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18368 /* vmovltq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18380 /* vmovltq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18391 /* vmovltq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18405 /* vmovltq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18418 /* vmovltq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18432 /* vmovltq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18445 /* vmovnbq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18459 /* vmovnbq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18473 /* vmovnbq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18487 /* vmovnbq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18501 /* vmovnbq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18513 /* vmovnbq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18525 /* vmovnbq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18537 /* vmovnbq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18549 /* vmovntq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18563 /* vmovntq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18577 /* vmovntq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18591 /* vmovntq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18605 /* vmovntq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18617 /* vmovntq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18629 /* vmovntq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18641 /* vmovntq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18653 /* vmulhq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18666 /* vmulhq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18679 /* vmulhq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18691 /* vmulhq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18704 /* vmulhq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18717 /* vmulhq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18729 /* vmulhq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18740 /* vmulhq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18751 /* vmulhq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18761 /* vmulhq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18772 /* vmulhq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18783 /* vmulhq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18793 /* vmulhq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18806 /* vmulhq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18819 /* vmulhq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18831 /* vmulhq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18844 /* vmulhq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18857 /* vmulhq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18869 /* vmullbq_int_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18887 /* vmullbq_int_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18905 /* vmullbq_int_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18922 /* vmullbq_int_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18940 /* vmullbq_int_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18958 /* vmullbq_int_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18975 /* vmullbq_int_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{18991 /* vmullbq_int_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19007 /* vmullbq_int_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19022 /* vmullbq_int_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19038 /* vmullbq_int_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19054 /* vmullbq_int_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19069 /* vmullbq_int_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19087 /* vmullbq_int_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19105 /* vmullbq_int_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19122 /* vmullbq_int_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19140 /* vmullbq_int_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19158 /* vmullbq_int_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19175 /* vmullbq_poly_m_p16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19194 /* vmullbq_poly_m_p8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19212 /* vmullbq_poly_p16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19229 /* vmullbq_poly_p8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19245 /* vmullbq_poly_x_p16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19264 /* vmullbq_poly_x_p8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19282 /* vmulltq_int_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19300 /* vmulltq_int_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19318 /* vmulltq_int_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19335 /* vmulltq_int_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19353 /* vmulltq_int_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19371 /* vmulltq_int_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19388 /* vmulltq_int_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19404 /* vmulltq_int_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19420 /* vmulltq_int_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19435 /* vmulltq_int_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19451 /* vmulltq_int_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19467 /* vmulltq_int_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19482 /* vmulltq_int_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19500 /* vmulltq_int_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19518 /* vmulltq_int_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19535 /* vmulltq_int_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19553 /* vmulltq_int_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19571 /* vmulltq_int_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19588 /* vmulltq_poly_m_p16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19607 /* vmulltq_poly_m_p8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19625 /* vmulltq_poly_p16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19642 /* vmulltq_poly_p8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19658 /* vmulltq_poly_x_p16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19677 /* vmulltq_poly_x_p8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19695 /* vmulq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19705 /* vmulq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19715 /* vmulq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19727 /* vmulq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19739 /* vmulq_m_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19753 /* vmulq_m_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19767 /* vmulq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19781 /* vmulq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19795 /* vmulq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19808 /* vmulq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19822 /* vmulq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19836 /* vmulq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19849 /* vmulq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19861 /* vmulq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19873 /* vmulq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19884 /* vmulq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19896 /* vmulq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19908 /* vmulq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19919 /* vmulq_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19931 /* vmulq_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19943 /* vmulq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19955 /* vmulq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19967 /* vmulq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19978 /* vmulq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{19990 /* vmulq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20002 /* vmulq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20013 /* vmulq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20023 /* vmulq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20033 /* vmulq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20042 /* vmulq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20052 /* vmulq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20062 /* vmulq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20071 /* vmulq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20083 /* vmulq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20095 /* vmulq_x_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20109 /* vmulq_x_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20123 /* vmulq_x_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20137 /* vmulq_x_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20151 /* vmulq_x_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20164 /* vmulq_x_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20178 /* vmulq_x_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20192 /* vmulq_x_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20205 /* vmulq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20217 /* vmulq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20229 /* vmulq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20240 /* vmulq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20252 /* vmulq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20264 /* vmulq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20275 /* vmvnq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20289 /* vmvnq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20303 /* vmvnq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20317 /* vmvnq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20331 /* vmvnq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20343 /* vmvnq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20355 /* vmvnq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20366 /* vmvnq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20378 /* vmvnq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20390 /* vmvnq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20401 /* vmvnq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20413 /* vmvnq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20425 /* vmvnq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20437 /* vmvnq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20449 /* vmvnq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20459 /* vmvnq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20469 /* vmvnq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20478 /* vmvnq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20488 /* vmvnq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20498 /* vmvnq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20507 /* vmvnq_x_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20521 /* vmvnq_x_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20535 /* vmvnq_x_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20549 /* vmvnq_x_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20563 /* vmvnq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20575 /* vmvnq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20587 /* vmvnq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20598 /* vmvnq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20610 /* vmvnq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20622 /* vmvnq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20633 /* vnegq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20643 /* vnegq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20653 /* vnegq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20665 /* vnegq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20677 /* vnegq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20689 /* vnegq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20701 /* vnegq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20712 /* vnegq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20722 /* vnegq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20732 /* vnegq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20741 /* vnegq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20753 /* vnegq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20765 /* vnegq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20777 /* vnegq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20789 /* vnegq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20800 /* vornq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20810 /* vornq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20820 /* vornq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20832 /* vornq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20844 /* vornq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20856 /* vornq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20868 /* vornq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20879 /* vornq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20891 /* vornq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20903 /* vornq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20914 /* vornq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20924 /* vornq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20934 /* vornq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20943 /* vornq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20953 /* vornq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20963 /* vornq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20972 /* vornq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20984 /* vornq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{20996 /* vornq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21008 /* vornq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21020 /* vornq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21031 /* vornq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21043 /* vornq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21055 /* vornq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21066 /* vorrq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21076 /* vorrq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21086 /* vorrq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21098 /* vorrq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21110 /* vorrq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21124 /* vorrq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21138 /* vorrq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21152 /* vorrq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21166 /* vorrq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21178 /* vorrq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21190 /* vorrq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21201 /* vorrq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21213 /* vorrq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21225 /* vorrq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21236 /* vorrq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21248 /* vorrq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21260 /* vorrq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21272 /* vorrq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21284 /* vorrq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21294 /* vorrq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21304 /* vorrq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21313 /* vorrq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21323 /* vorrq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21333 /* vorrq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21342 /* vorrq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21354 /* vorrq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21366 /* vorrq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21378 /* vorrq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21390 /* vorrq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21401 /* vorrq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21413 /* vorrq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21425 /* vorrq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21436 /* vpnot */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21442 /* vpselq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21453 /* vpselq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21464 /* vpselq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21475 /* vpselq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21486 /* vpselq_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21497 /* vpselq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21507 /* vpselq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21518 /* vpselq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21529 /* vpselq_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21540 /* vpselq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21550 /* vqabsq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21563 /* vqabsq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21576 /* vqabsq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21588 /* vqabsq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21599 /* vqabsq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21610 /* vqabsq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21620 /* vqaddq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21635 /* vqaddq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21650 /* vqaddq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21664 /* vqaddq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21679 /* vqaddq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21694 /* vqaddq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21708 /* vqaddq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21721 /* vqaddq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21734 /* vqaddq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21746 /* vqaddq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21759 /* vqaddq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21772 /* vqaddq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21784 /* vqaddq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21797 /* vqaddq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21810 /* vqaddq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21822 /* vqaddq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21835 /* vqaddq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21848 /* vqaddq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21860 /* vqaddq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21871 /* vqaddq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21882 /* vqaddq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21892 /* vqaddq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21903 /* vqaddq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21914 /* vqaddq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21924 /* vqdmladhq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21940 /* vqdmladhq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21956 /* vqdmladhq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21971 /* vqdmladhq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21985 /* vqdmladhq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{21999 /* vqdmladhq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22012 /* vqdmladhxq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22029 /* vqdmladhxq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22046 /* vqdmladhxq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22062 /* vqdmladhxq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22077 /* vqdmladhxq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22092 /* vqdmladhxq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22106 /* vqdmlahq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22123 /* vqdmlahq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22140 /* vqdmlahq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22156 /* vqdmlahq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22171 /* vqdmlahq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22186 /* vqdmlahq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22200 /* vqdmlashq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22218 /* vqdmlashq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22236 /* vqdmlashq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22253 /* vqdmlashq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22269 /* vqdmlashq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22285 /* vqdmlashq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22300 /* vqdmlsdhq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22316 /* vqdmlsdhq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22332 /* vqdmlsdhq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22347 /* vqdmlsdhq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22361 /* vqdmlsdhq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22375 /* vqdmlsdhq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22388 /* vqdmlsdhxq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22405 /* vqdmlsdhxq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22422 /* vqdmlsdhxq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22438 /* vqdmlsdhxq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22453 /* vqdmlsdhxq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22468 /* vqdmlsdhxq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22482 /* vqdmulhq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22499 /* vqdmulhq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22516 /* vqdmulhq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22532 /* vqdmulhq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22547 /* vqdmulhq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22562 /* vqdmulhq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22576 /* vqdmulhq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22591 /* vqdmulhq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22606 /* vqdmulhq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22620 /* vqdmulhq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22633 /* vqdmulhq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22646 /* vqdmulhq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22658 /* vqdmullbq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22676 /* vqdmullbq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22694 /* vqdmullbq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22710 /* vqdmullbq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22726 /* vqdmullbq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22742 /* vqdmullbq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22758 /* vqdmullbq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22772 /* vqdmullbq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22786 /* vqdmulltq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22804 /* vqdmulltq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22822 /* vqdmulltq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22838 /* vqdmulltq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22854 /* vqdmulltq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22870 /* vqdmulltq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22886 /* vqdmulltq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22900 /* vqdmulltq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22914 /* vqmovnbq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22929 /* vqmovnbq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22944 /* vqmovnbq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22959 /* vqmovnbq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22974 /* vqmovnbq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{22987 /* vqmovnbq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23000 /* vqmovnbq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23013 /* vqmovnbq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23026 /* vqmovntq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23041 /* vqmovntq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23056 /* vqmovntq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23071 /* vqmovntq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23086 /* vqmovntq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23099 /* vqmovntq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23112 /* vqmovntq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23125 /* vqmovntq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23138 /* vqmovunbq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23154 /* vqmovunbq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23170 /* vqmovunbq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23184 /* vqmovunbq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23198 /* vqmovuntq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23214 /* vqmovuntq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23230 /* vqmovuntq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23244 /* vqmovuntq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23258 /* vqnegq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23271 /* vqnegq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23284 /* vqnegq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23296 /* vqnegq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23307 /* vqnegq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23318 /* vqnegq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23328 /* vqrdmladhq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23345 /* vqrdmladhq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23362 /* vqrdmladhq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23378 /* vqrdmladhq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23393 /* vqrdmladhq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23408 /* vqrdmladhq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23422 /* vqrdmladhxq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23440 /* vqrdmladhxq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23458 /* vqrdmladhxq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23475 /* vqrdmladhxq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23491 /* vqrdmladhxq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23507 /* vqrdmladhxq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23522 /* vqrdmlahq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23540 /* vqrdmlahq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23558 /* vqrdmlahq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23575 /* vqrdmlahq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23591 /* vqrdmlahq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23607 /* vqrdmlahq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23622 /* vqrdmlashq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23641 /* vqrdmlashq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23660 /* vqrdmlashq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23678 /* vqrdmlashq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23695 /* vqrdmlashq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23712 /* vqrdmlashq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23728 /* vqrdmlsdhq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23745 /* vqrdmlsdhq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23762 /* vqrdmlsdhq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23778 /* vqrdmlsdhq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23793 /* vqrdmlsdhq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23808 /* vqrdmlsdhq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23822 /* vqrdmlsdhxq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23840 /* vqrdmlsdhxq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23858 /* vqrdmlsdhxq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23875 /* vqrdmlsdhxq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23891 /* vqrdmlsdhxq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23907 /* vqrdmlsdhxq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23922 /* vqrdmulhq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23940 /* vqrdmulhq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23958 /* vqrdmulhq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23975 /* vqrdmulhq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{23991 /* vqrdmulhq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24007 /* vqrdmulhq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24022 /* vqrdmulhq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24038 /* vqrdmulhq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24054 /* vqrdmulhq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24069 /* vqrdmulhq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24083 /* vqrdmulhq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24097 /* vqrdmulhq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24110 /* vqrshlq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24126 /* vqrshlq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24142 /* vqrshlq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24157 /* vqrshlq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24173 /* vqrshlq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24189 /* vqrshlq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24204 /* vqrshlq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24218 /* vqrshlq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24232 /* vqrshlq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24245 /* vqrshlq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24259 /* vqrshlq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24273 /* vqrshlq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24286 /* vqrshlq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24300 /* vqrshlq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24314 /* vqrshlq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24327 /* vqrshlq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24341 /* vqrshlq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24355 /* vqrshlq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24368 /* vqrshlq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24380 /* vqrshlq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24392 /* vqrshlq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24403 /* vqrshlq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24415 /* vqrshlq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24427 /* vqrshlq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24438 /* vqrshrnbq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24456 /* vqrshrnbq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24474 /* vqrshrnbq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24492 /* vqrshrnbq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24510 /* vqrshrnbq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24526 /* vqrshrnbq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24542 /* vqrshrnbq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24558 /* vqrshrnbq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24574 /* vqrshrntq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24592 /* vqrshrntq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24610 /* vqrshrntq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24628 /* vqrshrntq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24646 /* vqrshrntq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24662 /* vqrshrntq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24678 /* vqrshrntq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24694 /* vqrshrntq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24710 /* vqrshrunbq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24729 /* vqrshrunbq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24748 /* vqrshrunbq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24765 /* vqrshrunbq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24782 /* vqrshruntq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24801 /* vqrshruntq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24820 /* vqrshruntq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24837 /* vqrshruntq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24854 /* vqshlq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24869 /* vqshlq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24884 /* vqshlq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24898 /* vqshlq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24913 /* vqshlq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24928 /* vqshlq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24942 /* vqshlq_m_r_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24957 /* vqshlq_m_r_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24972 /* vqshlq_m_r_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{24986 /* vqshlq_m_r_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25001 /* vqshlq_m_r_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25016 /* vqshlq_m_r_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25030 /* vqshlq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25043 /* vqshlq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25056 /* vqshlq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25068 /* vqshlq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25081 /* vqshlq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25094 /* vqshlq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25106 /* vqshlq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25119 /* vqshlq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25132 /* vqshlq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25144 /* vqshlq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25157 /* vqshlq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25170 /* vqshlq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25182 /* vqshlq_r_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25195 /* vqshlq_r_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25208 /* vqshlq_r_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25220 /* vqshlq_r_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25233 /* vqshlq_r_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25246 /* vqshlq_r_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25258 /* vqshlq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25269 /* vqshlq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25280 /* vqshlq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25290 /* vqshlq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25301 /* vqshlq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25312 /* vqshlq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25322 /* vqshluq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25338 /* vqshluq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25354 /* vqshluq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25369 /* vqshluq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25383 /* vqshluq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25397 /* vqshluq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25410 /* vqshrnbq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25427 /* vqshrnbq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25444 /* vqshrnbq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25461 /* vqshrnbq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25478 /* vqshrnbq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25493 /* vqshrnbq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25508 /* vqshrnbq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25523 /* vqshrnbq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25538 /* vqshrntq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25555 /* vqshrntq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25572 /* vqshrntq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25589 /* vqshrntq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25606 /* vqshrntq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25621 /* vqshrntq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25636 /* vqshrntq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25651 /* vqshrntq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25666 /* vqshrunbq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25684 /* vqshrunbq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25702 /* vqshrunbq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25718 /* vqshrunbq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25734 /* vqshruntq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25752 /* vqshruntq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25770 /* vqshruntq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25786 /* vqshruntq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25802 /* vqsubq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25817 /* vqsubq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25832 /* vqsubq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25846 /* vqsubq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25861 /* vqsubq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25876 /* vqsubq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25890 /* vqsubq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25903 /* vqsubq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25916 /* vqsubq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25928 /* vqsubq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25941 /* vqsubq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25954 /* vqsubq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25966 /* vqsubq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25979 /* vqsubq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{25992 /* vqsubq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26004 /* vqsubq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26017 /* vqsubq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26030 /* vqsubq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26042 /* vqsubq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26053 /* vqsubq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26064 /* vqsubq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26074 /* vqsubq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26085 /* vqsubq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26096 /* vqsubq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26106 /* vreinterpretq_f16_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26128 /* vreinterpretq_f16_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26150 /* vreinterpretq_f16_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26172 /* vreinterpretq_f16_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26194 /* vreinterpretq_f16_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26215 /* vreinterpretq_f16_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26237 /* vreinterpretq_f16_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26259 /* vreinterpretq_f16_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26281 /* vreinterpretq_f16_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26302 /* vreinterpretq_f32_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26324 /* vreinterpretq_f32_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26346 /* vreinterpretq_f32_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26368 /* vreinterpretq_f32_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26390 /* vreinterpretq_f32_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26411 /* vreinterpretq_f32_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26433 /* vreinterpretq_f32_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26455 /* vreinterpretq_f32_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26477 /* vreinterpretq_f32_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26498 /* vreinterpretq_s16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26520 /* vreinterpretq_s16_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26542 /* vreinterpretq_s16_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26564 /* vreinterpretq_s16_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26586 /* vreinterpretq_s16_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26607 /* vreinterpretq_s16_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26629 /* vreinterpretq_s16_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26651 /* vreinterpretq_s16_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26673 /* vreinterpretq_s16_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26694 /* vreinterpretq_s32_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26716 /* vreinterpretq_s32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26738 /* vreinterpretq_s32_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26760 /* vreinterpretq_s32_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26782 /* vreinterpretq_s32_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26803 /* vreinterpretq_s32_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26825 /* vreinterpretq_s32_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26847 /* vreinterpretq_s32_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26869 /* vreinterpretq_s32_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26890 /* vreinterpretq_s64_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26912 /* vreinterpretq_s64_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26934 /* vreinterpretq_s64_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26956 /* vreinterpretq_s64_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26978 /* vreinterpretq_s64_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{26999 /* vreinterpretq_s64_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27021 /* vreinterpretq_s64_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27043 /* vreinterpretq_s64_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27065 /* vreinterpretq_s64_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27086 /* vreinterpretq_s8_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27107 /* vreinterpretq_s8_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27128 /* vreinterpretq_s8_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27149 /* vreinterpretq_s8_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27170 /* vreinterpretq_s8_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27191 /* vreinterpretq_s8_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27212 /* vreinterpretq_s8_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27233 /* vreinterpretq_s8_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27254 /* vreinterpretq_s8_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27274 /* vreinterpretq_u16_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27296 /* vreinterpretq_u16_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27318 /* vreinterpretq_u16_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27340 /* vreinterpretq_u16_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27362 /* vreinterpretq_u16_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27384 /* vreinterpretq_u16_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27405 /* vreinterpretq_u16_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27427 /* vreinterpretq_u16_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27449 /* vreinterpretq_u16_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27470 /* vreinterpretq_u32_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27492 /* vreinterpretq_u32_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27514 /* vreinterpretq_u32_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27536 /* vreinterpretq_u32_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27558 /* vreinterpretq_u32_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27580 /* vreinterpretq_u32_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27601 /* vreinterpretq_u32_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27623 /* vreinterpretq_u32_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27645 /* vreinterpretq_u32_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27666 /* vreinterpretq_u64_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27688 /* vreinterpretq_u64_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27710 /* vreinterpretq_u64_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27732 /* vreinterpretq_u64_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27754 /* vreinterpretq_u64_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27776 /* vreinterpretq_u64_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27797 /* vreinterpretq_u64_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27819 /* vreinterpretq_u64_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27841 /* vreinterpretq_u64_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27862 /* vreinterpretq_u8_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27883 /* vreinterpretq_u8_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27904 /* vreinterpretq_u8_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27925 /* vreinterpretq_u8_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27946 /* vreinterpretq_u8_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27967 /* vreinterpretq_u8_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{27987 /* vreinterpretq_u8_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28008 /* vreinterpretq_u8_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28029 /* vreinterpretq_u8_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28050 /* vrev16q_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28063 /* vrev16q_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28076 /* vrev16q_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28087 /* vrev16q_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28098 /* vrev16q_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28111 /* vrev16q_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28124 /* vrev32q_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28136 /* vrev32q_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28150 /* vrev32q_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28164 /* vrev32q_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28177 /* vrev32q_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28191 /* vrev32q_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28204 /* vrev32q_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28216 /* vrev32q_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28227 /* vrev32q_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28239 /* vrev32q_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28250 /* vrev32q_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28264 /* vrev32q_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28278 /* vrev32q_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28291 /* vrev32q_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28305 /* vrev32q_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28318 /* vrev64q_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28330 /* vrev64q_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28342 /* vrev64q_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28356 /* vrev64q_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28370 /* vrev64q_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28384 /* vrev64q_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28398 /* vrev64q_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28411 /* vrev64q_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28425 /* vrev64q_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28439 /* vrev64q_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28452 /* vrev64q_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28464 /* vrev64q_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28476 /* vrev64q_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28487 /* vrev64q_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28499 /* vrev64q_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28511 /* vrev64q_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28522 /* vrev64q_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28536 /* vrev64q_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28550 /* vrev64q_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28564 /* vrev64q_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28578 /* vrev64q_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28591 /* vrev64q_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28605 /* vrev64q_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28619 /* vrev64q_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28632 /* vrhaddq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28646 /* vrhaddq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28660 /* vrhaddq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28673 /* vrhaddq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28687 /* vrhaddq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28701 /* vrhaddq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28714 /* vrhaddq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28726 /* vrhaddq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28738 /* vrhaddq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28749 /* vrhaddq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28761 /* vrhaddq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28773 /* vrhaddq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28784 /* vrhaddq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28798 /* vrhaddq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28812 /* vrhaddq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28825 /* vrhaddq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28839 /* vrhaddq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28853 /* vrhaddq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28866 /* vrmlaldavhaq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28885 /* vrmlaldavhaq_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28904 /* vrmlaldavhaq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28921 /* vrmlaldavhaq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28938 /* vrmlaldavhaxq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28958 /* vrmlaldavhaxq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28976 /* vrmlaldavhq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{28994 /* vrmlaldavhq_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29012 /* vrmlaldavhq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29028 /* vrmlaldavhq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29044 /* vrmlaldavhxq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29063 /* vrmlaldavhxq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29080 /* vrmlsldavhaq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29099 /* vrmlsldavhaq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29116 /* vrmlsldavhaxq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29136 /* vrmlsldavhaxq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29154 /* vrmlsldavhq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29172 /* vrmlsldavhq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29188 /* vrmlsldavhxq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29207 /* vrmlsldavhxq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29224 /* vrmulhq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29238 /* vrmulhq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29252 /* vrmulhq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29265 /* vrmulhq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29279 /* vrmulhq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29293 /* vrmulhq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29306 /* vrmulhq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29318 /* vrmulhq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29330 /* vrmulhq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29341 /* vrmulhq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29353 /* vrmulhq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29365 /* vrmulhq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29376 /* vrmulhq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29390 /* vrmulhq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29404 /* vrmulhq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29417 /* vrmulhq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29431 /* vrmulhq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29445 /* vrmulhq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29458 /* vrndaq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29469 /* vrndaq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29480 /* vrndaq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29493 /* vrndaq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29506 /* vrndaq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29519 /* vrndaq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29532 /* vrndmq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29543 /* vrndmq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29554 /* vrndmq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29567 /* vrndmq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29580 /* vrndmq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29593 /* vrndmq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29606 /* vrndnq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29617 /* vrndnq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29628 /* vrndnq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29641 /* vrndnq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29654 /* vrndnq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29667 /* vrndnq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29680 /* vrndpq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29691 /* vrndpq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29702 /* vrndpq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29715 /* vrndpq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29728 /* vrndpq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29741 /* vrndpq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29754 /* vrndq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29764 /* vrndq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29774 /* vrndq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29786 /* vrndq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29798 /* vrndq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29810 /* vrndq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29822 /* vrndxq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29833 /* vrndxq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29844 /* vrndxq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29857 /* vrndxq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29870 /* vrndxq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29883 /* vrndxq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29896 /* vrshlq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29911 /* vrshlq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29926 /* vrshlq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29940 /* vrshlq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29955 /* vrshlq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29970 /* vrshlq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29984 /* vrshlq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{29997 /* vrshlq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30010 /* vrshlq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30022 /* vrshlq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30035 /* vrshlq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30048 /* vrshlq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30060 /* vrshlq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30073 /* vrshlq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30086 /* vrshlq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30098 /* vrshlq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30111 /* vrshlq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30124 /* vrshlq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30136 /* vrshlq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30147 /* vrshlq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30158 /* vrshlq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30168 /* vrshlq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30179 /* vrshlq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30190 /* vrshlq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30200 /* vrshlq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30213 /* vrshlq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30226 /* vrshlq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30238 /* vrshlq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30251 /* vrshlq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30264 /* vrshlq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30276 /* vrshrnbq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30293 /* vrshrnbq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30310 /* vrshrnbq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30327 /* vrshrnbq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30344 /* vrshrnbq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30359 /* vrshrnbq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30374 /* vrshrnbq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30389 /* vrshrnbq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30404 /* vrshrntq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30421 /* vrshrntq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30438 /* vrshrntq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30455 /* vrshrntq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30472 /* vrshrntq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30487 /* vrshrntq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30502 /* vrshrntq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30517 /* vrshrntq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30532 /* vrshrq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30547 /* vrshrq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30562 /* vrshrq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30576 /* vrshrq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30591 /* vrshrq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30606 /* vrshrq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30620 /* vrshrq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30633 /* vrshrq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30646 /* vrshrq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30658 /* vrshrq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30671 /* vrshrq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30684 /* vrshrq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30696 /* vrshrq_x_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30711 /* vrshrq_x_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30726 /* vrshrq_x_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30740 /* vrshrq_x_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30755 /* vrshrq_x_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30770 /* vrshrq_x_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30784 /* vsbciq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30797 /* vsbciq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30810 /* vsbciq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30821 /* vsbciq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30832 /* vsbcq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30844 /* vsbcq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30856 /* vsbcq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30866 /* vsbcq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30876 /* vsetq_lane_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30891 /* vsetq_lane_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30906 /* vsetq_lane_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30921 /* vsetq_lane_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30936 /* vsetq_lane_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30951 /* vsetq_lane_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30965 /* vsetq_lane_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30980 /* vsetq_lane_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{30995 /* vsetq_lane_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31010 /* vsetq_lane_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31024 /* vshlcq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31037 /* vshlcq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31050 /* vshlcq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31062 /* vshlcq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31075 /* vshlcq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31088 /* vshlcq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31100 /* vshlcq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31111 /* vshlcq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31122 /* vshlcq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31132 /* vshlcq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31143 /* vshlcq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31154 /* vshlcq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31164 /* vshllbq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31180 /* vshllbq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31195 /* vshllbq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31211 /* vshllbq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31226 /* vshllbq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31240 /* vshllbq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31253 /* vshllbq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31267 /* vshllbq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31280 /* vshllbq_x_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31296 /* vshllbq_x_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31311 /* vshllbq_x_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31327 /* vshllbq_x_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31342 /* vshlltq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31358 /* vshlltq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31373 /* vshlltq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31389 /* vshlltq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31404 /* vshlltq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31418 /* vshlltq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31431 /* vshlltq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31445 /* vshlltq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31458 /* vshlltq_x_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31474 /* vshlltq_x_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31489 /* vshlltq_x_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31505 /* vshlltq_x_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31520 /* vshlq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31534 /* vshlq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31548 /* vshlq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31561 /* vshlq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31575 /* vshlq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31589 /* vshlq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31602 /* vshlq_m_r_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31616 /* vshlq_m_r_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31630 /* vshlq_m_r_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31643 /* vshlq_m_r_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31657 /* vshlq_m_r_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31671 /* vshlq_m_r_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31684 /* vshlq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31696 /* vshlq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31708 /* vshlq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31719 /* vshlq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31731 /* vshlq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31743 /* vshlq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31754 /* vshlq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31766 /* vshlq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31778 /* vshlq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31789 /* vshlq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31801 /* vshlq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31813 /* vshlq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31824 /* vshlq_r_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31836 /* vshlq_r_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31848 /* vshlq_r_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31859 /* vshlq_r_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31871 /* vshlq_r_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31883 /* vshlq_r_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31894 /* vshlq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31904 /* vshlq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31914 /* vshlq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31923 /* vshlq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31933 /* vshlq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31943 /* vshlq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31952 /* vshlq_x_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31966 /* vshlq_x_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31980 /* vshlq_x_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{31993 /* vshlq_x_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32007 /* vshlq_x_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32021 /* vshlq_x_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32034 /* vshlq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32046 /* vshlq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32058 /* vshlq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32069 /* vshlq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32081 /* vshlq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32093 /* vshlq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32104 /* vshrnbq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32120 /* vshrnbq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32136 /* vshrnbq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32152 /* vshrnbq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32168 /* vshrnbq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32182 /* vshrnbq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32196 /* vshrnbq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32210 /* vshrnbq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32224 /* vshrntq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32240 /* vshrntq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32256 /* vshrntq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32272 /* vshrntq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32288 /* vshrntq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32302 /* vshrntq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32316 /* vshrntq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32330 /* vshrntq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32344 /* vshrq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32358 /* vshrq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32372 /* vshrq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32385 /* vshrq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32399 /* vshrq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32413 /* vshrq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32426 /* vshrq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32438 /* vshrq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32450 /* vshrq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32461 /* vshrq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32473 /* vshrq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32485 /* vshrq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32496 /* vshrq_x_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32510 /* vshrq_x_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32524 /* vshrq_x_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32537 /* vshrq_x_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32551 /* vshrq_x_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32565 /* vshrq_x_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32578 /* vsliq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32592 /* vsliq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32606 /* vsliq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32619 /* vsliq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32633 /* vsliq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32647 /* vsliq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32660 /* vsliq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32672 /* vsliq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32684 /* vsliq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32695 /* vsliq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32707 /* vsliq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32719 /* vsliq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32730 /* vsriq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32744 /* vsriq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32758 /* vsriq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32771 /* vsriq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32785 /* vsriq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32799 /* vsriq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32812 /* vsriq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32824 /* vsriq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32836 /* vsriq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32847 /* vsriq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32859 /* vsriq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32871 /* vsriq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32882 /* vst1q_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32892 /* vst1q_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32902 /* vst1q_p_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32914 /* vst1q_p_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32926 /* vst1q_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32938 /* vst1q_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32950 /* vst1q_p_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32961 /* vst1q_p_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32973 /* vst1q_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32985 /* vst1q_p_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{32996 /* vst1q_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33006 /* vst1q_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33016 /* vst1q_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33025 /* vst1q_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33035 /* vst1q_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33045 /* vst1q_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33054 /* vst2q_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33064 /* vst2q_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33074 /* vst2q_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33084 /* vst2q_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33094 /* vst2q_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33103 /* vst2q_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33113 /* vst2q_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33123 /* vst2q_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33132 /* vst4q_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33142 /* vst4q_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33152 /* vst4q_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33162 /* vst4q_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33172 /* vst4q_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33181 /* vst4q_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33191 /* vst4q_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33201 /* vst4q_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33210 /* vstrbq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33223 /* vstrbq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33236 /* vstrbq_p_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33248 /* vstrbq_p_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33261 /* vstrbq_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33274 /* vstrbq_p_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33286 /* vstrbq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33297 /* vstrbq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33308 /* vstrbq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33318 /* vstrbq_scatter_offset_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33346 /* vstrbq_scatter_offset_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33374 /* vstrbq_scatter_offset_p_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33401 /* vstrbq_scatter_offset_p_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33429 /* vstrbq_scatter_offset_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33457 /* vstrbq_scatter_offset_p_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33484 /* vstrbq_scatter_offset_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33510 /* vstrbq_scatter_offset_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33536 /* vstrbq_scatter_offset_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33561 /* vstrbq_scatter_offset_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33587 /* vstrbq_scatter_offset_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33613 /* vstrbq_scatter_offset_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33638 /* vstrbq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33649 /* vstrbq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33660 /* vstrbq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33670 /* vstrdq_scatter_base_p_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33696 /* vstrdq_scatter_base_p_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33722 /* vstrdq_scatter_base_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33746 /* vstrdq_scatter_base_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33770 /* vstrdq_scatter_base_wb_p_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33799 /* vstrdq_scatter_base_wb_p_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33828 /* vstrdq_scatter_base_wb_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33855 /* vstrdq_scatter_base_wb_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33882 /* vstrdq_scatter_offset_p_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33910 /* vstrdq_scatter_offset_p_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33938 /* vstrdq_scatter_offset_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33964 /* vstrdq_scatter_offset_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{33990 /* vstrdq_scatter_shifted_offset_p_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34026 /* vstrdq_scatter_shifted_offset_p_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34062 /* vstrdq_scatter_shifted_offset_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34096 /* vstrdq_scatter_shifted_offset_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34130 /* vstrhq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34141 /* vstrhq_p_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34154 /* vstrhq_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34167 /* vstrhq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34180 /* vstrhq_p_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34193 /* vstrhq_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34206 /* vstrhq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34217 /* vstrhq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34228 /* vstrhq_scatter_offset_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34254 /* vstrhq_scatter_offset_p_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34282 /* vstrhq_scatter_offset_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34310 /* vstrhq_scatter_offset_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34338 /* vstrhq_scatter_offset_p_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34366 /* vstrhq_scatter_offset_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34394 /* vstrhq_scatter_offset_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34420 /* vstrhq_scatter_offset_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34446 /* vstrhq_scatter_offset_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34472 /* vstrhq_scatter_offset_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34498 /* vstrhq_scatter_shifted_offset_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34532 /* vstrhq_scatter_shifted_offset_p_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34568 /* vstrhq_scatter_shifted_offset_p_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34604 /* vstrhq_scatter_shifted_offset_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34640 /* vstrhq_scatter_shifted_offset_p_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34676 /* vstrhq_scatter_shifted_offset_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34712 /* vstrhq_scatter_shifted_offset_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34746 /* vstrhq_scatter_shifted_offset_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34780 /* vstrhq_scatter_shifted_offset_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34814 /* vstrhq_scatter_shifted_offset_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34848 /* vstrhq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34859 /* vstrhq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34870 /* vstrwq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34881 /* vstrwq_p_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34894 /* vstrwq_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34907 /* vstrwq_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34920 /* vstrwq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34931 /* vstrwq_scatter_base_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34955 /* vstrwq_scatter_base_p_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{34981 /* vstrwq_scatter_base_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35007 /* vstrwq_scatter_base_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35033 /* vstrwq_scatter_base_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35057 /* vstrwq_scatter_base_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35081 /* vstrwq_scatter_base_wb_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35108 /* vstrwq_scatter_base_wb_p_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35137 /* vstrwq_scatter_base_wb_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35166 /* vstrwq_scatter_base_wb_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35195 /* vstrwq_scatter_base_wb_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35222 /* vstrwq_scatter_base_wb_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35249 /* vstrwq_scatter_offset_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35275 /* vstrwq_scatter_offset_p_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35303 /* vstrwq_scatter_offset_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35331 /* vstrwq_scatter_offset_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35359 /* vstrwq_scatter_offset_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35385 /* vstrwq_scatter_offset_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35411 /* vstrwq_scatter_shifted_offset_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35445 /* vstrwq_scatter_shifted_offset_p_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35481 /* vstrwq_scatter_shifted_offset_p_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35517 /* vstrwq_scatter_shifted_offset_p_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35553 /* vstrwq_scatter_shifted_offset_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35587 /* vstrwq_scatter_shifted_offset_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35621 /* vstrwq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35632 /* vsubq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35642 /* vsubq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35652 /* vsubq_m_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35664 /* vsubq_m_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35676 /* vsubq_m_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35690 /* vsubq_m_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35704 /* vsubq_m_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35718 /* vsubq_m_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35732 /* vsubq_m_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35745 /* vsubq_m_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35759 /* vsubq_m_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35773 /* vsubq_m_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35786 /* vsubq_m_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35798 /* vsubq_m_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35810 /* vsubq_m_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35821 /* vsubq_m_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35833 /* vsubq_m_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35845 /* vsubq_m_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35856 /* vsubq_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35868 /* vsubq_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35880 /* vsubq_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35892 /* vsubq_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35904 /* vsubq_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35915 /* vsubq_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35927 /* vsubq_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35939 /* vsubq_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35950 /* vsubq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35960 /* vsubq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35970 /* vsubq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35979 /* vsubq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35989 /* vsubq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{35999 /* vsubq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36008 /* vsubq_x_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36020 /* vsubq_x_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36032 /* vsubq_x_n_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36046 /* vsubq_x_n_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36060 /* vsubq_x_n_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36074 /* vsubq_x_n_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36088 /* vsubq_x_n_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36101 /* vsubq_x_n_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36115 /* vsubq_x_n_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36129 /* vsubq_x_n_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36142 /* vsubq_x_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36154 /* vsubq_x_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36166 /* vsubq_x_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36177 /* vsubq_x_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36189 /* vsubq_x_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36201 /* vsubq_x_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36212 /* vuninitializedq_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36232 /* vuninitializedq_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36252 /* vuninitializedq_polymorphic_f16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36284 /* vuninitializedq_polymorphic_f32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36316 /* vuninitializedq_polymorphic_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36348 /* vuninitializedq_polymorphic_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36380 /* vuninitializedq_polymorphic_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36412 /* vuninitializedq_polymorphic_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36443 /* vuninitializedq_polymorphic_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36475 /* vuninitializedq_polymorphic_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36507 /* vuninitializedq_polymorphic_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36539 /* vuninitializedq_polymorphic_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36570 /* vuninitializedq_s16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36590 /* vuninitializedq_s32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36610 /* vuninitializedq_s64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36630 /* vuninitializedq_s8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36649 /* vuninitializedq_u16 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36669 /* vuninitializedq_u32 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36689 /* vuninitializedq_u64 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36709 /* vuninitializedq_u8 */, 0, 1 /* n */}},
    Builtin::Info{Builtin::Info::StrOffsets{36737 /* vabavq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36728 /* vabavq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36744 /* vabdq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36750 /* vabdq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36758 /* vabdq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36766 /* vabsq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36772 /* vabsq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36780 /* vabsq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36797 /* vadciq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36788 /* vadciq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36812 /* vadcq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36804 /* vadcq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36829 /* vaddlvaq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36818 /* vaddlvaq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36848 /* vaddlvq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36838 /* vaddlvq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36856 /* vaddq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36862 /* vaddq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36870 /* vaddq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36888 /* vaddvaq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36878 /* vaddvaq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36905 /* vaddvq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36896 /* vaddvq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36912 /* vandq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36918 /* vandq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36926 /* vandq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36934 /* vbicq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36940 /* vbicq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36948 /* vbicq_m_n */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36958 /* vbicq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36975 /* vbrsrq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36966 /* vbrsrq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36982 /* vbrsrq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{36991 /* vcaddq_rot270 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37005 /* vcaddq_rot270_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37021 /* vcaddq_rot270_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37037 /* vcaddq_rot90 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37050 /* vcaddq_rot90_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37065 /* vcaddq_rot90_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37088 /* vclsq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37080 /* vclsq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37094 /* vclsq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37110 /* vclzq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37102 /* vclzq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37116 /* vclzq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37124 /* vcmlaq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37131 /* vcmlaq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37140 /* vcmlaq_rot180 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37154 /* vcmlaq_rot180_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37170 /* vcmlaq_rot270 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37184 /* vcmlaq_rot270_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37200 /* vcmlaq_rot90 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37213 /* vcmlaq_rot90_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37238 /* vcmpcsq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37228 /* vcmpcsq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37246 /* vcmpeqq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37254 /* vcmpeqq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37264 /* vcmpgeq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37272 /* vcmpgeq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37282 /* vcmpgtq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37290 /* vcmpgtq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37310 /* vcmphiq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37300 /* vcmphiq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37318 /* vcmpleq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37326 /* vcmpleq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37336 /* vcmpltq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37344 /* vcmpltq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37354 /* vcmpneq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37362 /* vcmpneq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37372 /* vcmulq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37379 /* vcmulq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37388 /* vcmulq_rot180 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37402 /* vcmulq_rot180_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37418 /* vcmulq_rot180_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37434 /* vcmulq_rot270 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37448 /* vcmulq_rot270_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37464 /* vcmulq_rot270_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37480 /* vcmulq_rot90 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37493 /* vcmulq_rot90_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37508 /* vcmulq_rot90_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37523 /* vcmulq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37532 /* vcvtaq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37541 /* vcvtmq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37550 /* vcvtnq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37559 /* vcvtpq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37568 /* vcvtq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37574 /* vcvtq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37582 /* vcvtq_m_n */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37592 /* vcvtq_n */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37600 /* vcvtq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37608 /* vcvtq_x_n */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37618 /* vddupq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37627 /* vddupq_u16 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37638 /* vddupq_u32 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37649 /* vddupq_u8 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37659 /* vddupq_x_u16 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37672 /* vddupq_x_u32 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37685 /* vddupq_x_u8 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37697 /* vdupq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37705 /* vdwdupq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37715 /* vdwdupq_u16 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37727 /* vdwdupq_u32 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37739 /* vdwdupq_u8 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37750 /* vdwdupq_x_u16 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37764 /* vdwdupq_x_u32 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37778 /* vdwdupq_x_u8 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37791 /* veorq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37797 /* veorq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37805 /* veorq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37813 /* vfmaq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37819 /* vfmaq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37836 /* vfmasq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37827 /* vfmasq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37843 /* vfmsq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37849 /* vfmsq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37857 /* vgetq_lane */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37877 /* vhaddq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37868 /* vhaddq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37884 /* vhaddq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37910 /* vhcaddq_rot270 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37893 /* vhcaddq_rot270_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37925 /* vhcaddq_rot270_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37958 /* vhcaddq_rot90 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37942 /* vhcaddq_rot90_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37972 /* vhcaddq_rot90_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37997 /* vhsubq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{37988 /* vhsubq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38004 /* vhsubq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38013 /* vidupq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38022 /* vidupq_u16 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38033 /* vidupq_u32 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38044 /* vidupq_u8 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38054 /* vidupq_x_u16 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38067 /* vidupq_x_u32 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38080 /* vidupq_x_u8 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38092 /* viwdupq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38102 /* viwdupq_u16 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38114 /* viwdupq_u32 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38126 /* viwdupq_u8 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38137 /* viwdupq_x_u16 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38151 /* viwdupq_x_u32 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38165 /* viwdupq_x_u8 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38178 /* vld1q */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38184 /* vld1q_z */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38192 /* vld2q */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38198 /* vld4q */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38204 /* vldrbq_gather_offset */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38225 /* vldrbq_gather_offset_z */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38248 /* vldrdq_gather_offset */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38269 /* vldrdq_gather_offset_z */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38292 /* vldrdq_gather_shifted_offset */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38321 /* vldrdq_gather_shifted_offset_z */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38352 /* vldrhq_gather_offset */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38373 /* vldrhq_gather_offset_z */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38396 /* vldrhq_gather_shifted_offset */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38425 /* vldrhq_gather_shifted_offset_z */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38456 /* vldrwq_gather_offset */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38477 /* vldrwq_gather_offset_z */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38500 /* vldrwq_gather_shifted_offset */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38529 /* vldrwq_gather_shifted_offset_z */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38569 /* vmaxaq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38560 /* vmaxaq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38586 /* vmaxavq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38576 /* vmaxavq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38594 /* vmaxnmaq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38603 /* vmaxnmaq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38614 /* vmaxnmavq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38624 /* vmaxnmavq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38636 /* vmaxnmq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38644 /* vmaxnmq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38654 /* vmaxnmq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38664 /* vmaxnmvq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38673 /* vmaxnmvq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38692 /* vmaxq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38684 /* vmaxq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38698 /* vmaxq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38715 /* vmaxvq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38706 /* vmaxvq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38731 /* vminaq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38722 /* vminaq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38748 /* vminavq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38738 /* vminavq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38756 /* vminnmaq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38765 /* vminnmaq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38776 /* vminnmavq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38786 /* vminnmavq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38798 /* vminnmq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38806 /* vminnmq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38816 /* vminnmq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38826 /* vminnmvq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38835 /* vminnmvq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38854 /* vminq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38846 /* vminq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38860 /* vminq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38877 /* vminvq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38868 /* vminvq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38896 /* vmladavaq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38884 /* vmladavaq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38919 /* vmladavaxq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38906 /* vmladavaxq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38941 /* vmladavq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38930 /* vmladavq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38962 /* vmladavxq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38950 /* vmladavxq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38985 /* vmlaldavaq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38972 /* vmlaldavaq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39010 /* vmlaldavaxq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{38996 /* vmlaldavaxq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39034 /* vmlaldavq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39022 /* vmlaldavq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39057 /* vmlaldavxq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39044 /* vmlaldavxq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39076 /* vmlaq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39068 /* vmlaq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39091 /* vmlasq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39082 /* vmlasq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39110 /* vmlsdavaq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39098 /* vmlsdavaq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39133 /* vmlsdavaxq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39120 /* vmlsdavaxq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39155 /* vmlsdavq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39144 /* vmlsdavq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39176 /* vmlsdavxq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39164 /* vmlsdavxq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39199 /* vmlsldavaq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39186 /* vmlsldavaq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39224 /* vmlsldavaxq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39210 /* vmlsldavaxq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39248 /* vmlsldavq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39236 /* vmlsldavq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39271 /* vmlsldavxq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39258 /* vmlsldavxq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39292 /* vmovlbq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39282 /* vmovlbq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39300 /* vmovlbq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39320 /* vmovltq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39310 /* vmovltq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39328 /* vmovltq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39348 /* vmovnbq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39338 /* vmovnbq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39366 /* vmovntq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39356 /* vmovntq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39383 /* vmulhq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39374 /* vmulhq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39390 /* vmulhq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39413 /* vmullbq_int */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39399 /* vmullbq_int_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39425 /* vmullbq_int_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39454 /* vmullbq_poly */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39439 /* vmullbq_poly_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39467 /* vmullbq_poly_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39496 /* vmulltq_int */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39482 /* vmulltq_int_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39508 /* vmulltq_int_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39537 /* vmulltq_poly */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39522 /* vmulltq_poly_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39550 /* vmulltq_poly_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39565 /* vmulq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39571 /* vmulq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39579 /* vmulq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39595 /* vmvnq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39587 /* vmvnq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39601 /* vmvnq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39609 /* vnegq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39615 /* vnegq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39623 /* vnegq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39631 /* vornq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39637 /* vornq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39645 /* vornq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39653 /* vorrq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39659 /* vorrq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39667 /* vorrq_m_n */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39677 /* vorrq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39685 /* vpselq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39701 /* vqabsq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39692 /* vqabsq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39717 /* vqaddq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39708 /* vqaddq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39736 /* vqdmladhq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39724 /* vqdmladhq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39759 /* vqdmladhxq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39746 /* vqdmladhxq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39781 /* vqdmlahq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39770 /* vqdmlahq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39802 /* vqdmlashq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39790 /* vqdmlashq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39824 /* vqdmlsdhq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39812 /* vqdmlsdhq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39847 /* vqdmlsdhxq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39834 /* vqdmlsdhxq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39869 /* vqdmulhq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39858 /* vqdmulhq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39890 /* vqdmullbq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39878 /* vqdmullbq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39912 /* vqdmulltq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39900 /* vqdmulltq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39933 /* vqmovnbq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39922 /* vqmovnbq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39953 /* vqmovntq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39942 /* vqmovntq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39974 /* vqmovunbq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39962 /* vqmovunbq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39996 /* vqmovuntq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{39984 /* vqmovuntq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40015 /* vqnegq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40006 /* vqnegq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40035 /* vqrdmladhq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40022 /* vqrdmladhq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40060 /* vqrdmladhxq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40046 /* vqrdmladhxq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40084 /* vqrdmlahq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40072 /* vqrdmlahq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40107 /* vqrdmlashq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40094 /* vqrdmlashq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40131 /* vqrdmlsdhq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40118 /* vqrdmlsdhq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40156 /* vqrdmlsdhxq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40142 /* vqrdmlsdhxq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40180 /* vqrdmulhq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40168 /* vqrdmulhq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40212 /* vqrshlq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40202 /* vqrshlq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40190 /* vqrshlq_m_n */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40232 /* vqrshrnbq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40220 /* vqrshrnbq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40254 /* vqrshrntq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40242 /* vqrshrntq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40277 /* vqrshrunbq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40264 /* vqrshrunbq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40301 /* vqrshruntq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40288 /* vqrshruntq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40361 /* vqshlq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40334 /* vqshlq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40312 /* vqshlq_m_n */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40323 /* vqshlq_m_r */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40343 /* vqshlq_n */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40352 /* vqshlq_r */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40378 /* vqshluq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40368 /* vqshluq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40397 /* vqshrnbq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40386 /* vqshrnbq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40417 /* vqshrntq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40406 /* vqshrntq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40438 /* vqshrunbq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40426 /* vqshrunbq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40460 /* vqshruntq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40448 /* vqshruntq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40479 /* vqsubq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40470 /* vqsubq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40486 /* vreinterpretq_f16 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40504 /* vreinterpretq_f32 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40522 /* vreinterpretq_s16 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40540 /* vreinterpretq_s32 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40558 /* vreinterpretq_s64 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40576 /* vreinterpretq_s8 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40593 /* vreinterpretq_u16 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40611 /* vreinterpretq_u32 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40629 /* vreinterpretq_u64 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40647 /* vreinterpretq_u8 */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40674 /* vrev16q */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40664 /* vrev16q_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40682 /* vrev16q_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40692 /* vrev32q */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40700 /* vrev32q_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40710 /* vrev32q_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40720 /* vrev64q */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40728 /* vrev64q_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40738 /* vrev64q_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40758 /* vrhaddq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40748 /* vrhaddq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40766 /* vrhaddq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40791 /* vrmlaldavhaq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40776 /* vrmlaldavhaq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40820 /* vrmlaldavhaxq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40804 /* vrmlaldavhaxq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40848 /* vrmlaldavhq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40834 /* vrmlaldavhq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40875 /* vrmlaldavhxq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40860 /* vrmlaldavhxq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40903 /* vrmlsldavhaq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40888 /* vrmlsldavhaq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40932 /* vrmlsldavhaxq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40916 /* vrmlsldavhaxq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40960 /* vrmlsldavhq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40946 /* vrmlsldavhq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40987 /* vrmlsldavhxq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{40972 /* vrmlsldavhxq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41010 /* vrmulhq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41000 /* vrmulhq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41018 /* vrmulhq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41028 /* vrndaq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41035 /* vrndaq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41044 /* vrndaq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41053 /* vrndmq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41060 /* vrndmq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41069 /* vrndmq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41078 /* vrndnq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41085 /* vrndnq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41094 /* vrndnq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41103 /* vrndpq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41110 /* vrndpq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41119 /* vrndpq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41128 /* vrndq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41134 /* vrndq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41142 /* vrndq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41150 /* vrndxq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41157 /* vrndxq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41166 /* vrndxq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41195 /* vrshlq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41186 /* vrshlq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41175 /* vrshlq_m_n */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41202 /* vrshlq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41222 /* vrshrnbq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41211 /* vrshrnbq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41242 /* vrshrntq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41231 /* vrshrntq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41260 /* vrshrq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41251 /* vrshrq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41267 /* vrshrq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41285 /* vsbciq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41276 /* vsbciq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41300 /* vsbcq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41292 /* vsbcq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41306 /* vsetq_lane */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41326 /* vshlcq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41317 /* vshlcq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41343 /* vshllbq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41333 /* vshllbq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41351 /* vshllbq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41371 /* vshlltq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41361 /* vshlltq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41379 /* vshlltq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41433 /* vshlq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41409 /* vshlq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41389 /* vshlq_m_n */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41399 /* vshlq_m_r */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41417 /* vshlq_n */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41425 /* vshlq_r */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41449 /* vshlq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41439 /* vshlq_x_n */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41467 /* vshrnbq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41457 /* vshrnbq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41485 /* vshrntq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41475 /* vshrntq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41501 /* vshrq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41493 /* vshrq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41507 /* vshrq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41523 /* vsliq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41515 /* vsliq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41537 /* vsriq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41529 /* vsriq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41543 /* vst1q */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41549 /* vst1q_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41557 /* vst2q */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41563 /* vst4q */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41578 /* vstrbq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41569 /* vstrbq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41609 /* vstrbq_scatter_offset */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41585 /* vstrbq_scatter_offset_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41653 /* vstrdq_scatter_base */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41631 /* vstrdq_scatter_base_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41698 /* vstrdq_scatter_base_wb */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41673 /* vstrdq_scatter_base_wb_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41745 /* vstrdq_scatter_offset */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41721 /* vstrdq_scatter_offset_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41799 /* vstrdq_scatter_shifted_offset */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41767 /* vstrdq_scatter_shifted_offset_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41829 /* vstrhq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41836 /* vstrhq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41845 /* vstrhq_scatter_offset */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41867 /* vstrhq_scatter_offset_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41891 /* vstrhq_scatter_shifted_offset */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41921 /* vstrhq_scatter_shifted_offset_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41953 /* vstrwq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41960 /* vstrwq_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41969 /* vstrwq_scatter_base */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{41989 /* vstrwq_scatter_base_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{42011 /* vstrwq_scatter_base_wb */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{42034 /* vstrwq_scatter_base_wb_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{42059 /* vstrwq_scatter_offset */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{42081 /* vstrwq_scatter_offset_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{42105 /* vstrwq_scatter_shifted_offset */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{42135 /* vstrwq_scatter_shifted_offset_p */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{42167 /* vsubq */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{42173 /* vsubq_m */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{42181 /* vsubq_x */, 10 /* vi. */, 3 /* nt */}},
    Builtin::Info{Builtin::Info::StrOffsets{42189 /* vuninitializedq */, 10 /* vi. */, 6 /* ntu */}},
#endif // GET_MVE_BUILTIN_INFOS

